#ifndef POINT3D_H
#define POINT3D_H
#include <wx/wx.h>

#define EPSILON 0.0001

class Point3D
{
    public:
        Point3D( double x = 0, double y = 0, double z = 0 )
        {
            this->x = x;
            this->y = y;
            this->z = z;
        }

        Point3D operator-() { return Point3D( -x, -y, -z ); }
        Point3D operator+( Point3D& p ) { return Point3D( x + p.x, y + p.y, z + p.z ); }
        Point3D operator-( Point3D& p ) { return Point3D( x - p.x, y - p.y, z - p.z ); }
        void operator+=( Point3D& p ) { x += p.x; y += p.y; z += p.z; }
        void  operator/=( double d );
        Point3D operator*( double s ) { return Point3D( x * s, y * s, z * s ); }
        Point3D operator/( double s );
        Point3D operator%( Point3D& v )
        {
            return Point3D(y * v.z - z * v.y, z * v.x - x * v.z, x * v.y - y * v.x);
        }
        double  operator*( Point3D& p ) { return (x * p.x + y * p.y + z * p.z); }
        double  length();
        void    normalize()
        {
            double l = length();
            *this = (*this) * ( (l > EPSILON) ? (1/l) : 1 );
        }

        wxString toString() const
        {
            return wxString::Format( _T("( %f, %f, %f )"), x, y, z );
        }

        double x;
        double y;
        double z;
};

typedef Point3D Vector3D;

WX_DEFINE_ARRAY( Point3D*, Point3DArray );

#endif // POINT3D_H
