#include "TableEditor3DDefs.h"

int TE3D_CursorMovingMode::stringToId( const wxString& align )
{
    if ( align.IsSameAs( _T("RELATIVE"), false ) ) {
        return CMM_RELATIVE;
    } else if ( align.IsSameAs( _T("ABSOLUTE"), false ) ) {
        return CMM_ABSOLUTE;
    } else {
        return CMM_RELATIVE;
    }
}

wxString TE3D_CursorMovingMode::idToConfigString( int align )
{
    switch ( align )
    {
        case CMM_RELATIVE:
            return _T("RELATIVE");

        case CMM_ABSOLUTE:
            return _T("ABSOLUTE");

        default:
            return _T("RELATIVE");
    }
}

wxString TE3D_CursorMovingMode::idToLabelString( int align )
{
    switch ( align )
    {
        case CMM_RELATIVE:
            return _T("Relative");

        case CMM_ABSOLUTE:
            return _T("Absolute");

        default:
            return _T("Relative");
    }
}


int TE3D_Align::stringToId( const wxString& align )
{
    if ( align.IsSameAs( _T("TOP_LEFT"), false ) || align.IsSameAs( _T("Top-Left") )  ){
        return TOP_LEFT;
    } else if ( align.IsSameAs( _T("TOP_RIGHT"), false ) || align.IsSameAs( _T("Top-Right") ) ) {
        return TOP_RIGHT;
    } else if ( align.IsSameAs( _T("BOTTOM_LEFT"), false ) || align.IsSameAs( _T("Bottom-Left") ) ) {
        return BOTTOM_LEFT;
    } else if ( align.IsSameAs( _T("BOTTOM_RIGHT"), false ) || align.IsSameAs( _T("Bottom-Right") ) ) {
        return BOTTOM_RIGHT;
    } else {
        return TOP_LEFT;
    }
}

wxString TE3D_Align::idToConfigString( int align )
{
    switch ( align )
    {
        case TOP_LEFT:
            return _T("TOP_LEFT");

        case TOP_RIGHT:
            return _T("TOP_RIGHT");

        case BOTTOM_LEFT:
            return _T("BOTTOM_LEFT");

        case BOTTOM_RIGHT:
            return _T("BOTTOM_RIGHT");

        default:
            return _T("TOP_LEFT");
    }
}

wxString TE3D_Align::idToLabelString( int align )
{
    switch ( align )
    {
        case TOP_LEFT:
            return _T("Top-Left");

        case TOP_RIGHT:
            return _T("Top-Right");

        case BOTTOM_LEFT:
            return _T("Bottom-Left");

        case BOTTOM_RIGHT:
            return _T("Bottom-Right");

        default:
            return _T("Top-Left");
    }
}

wxArrayString TE3D_Align::getLabels()
{
    wxArrayString labels;

    labels.Add( _T("Top-Left") );
    labels.Add( _T("Top-Right") );
    labels.Add( _T("Bottom-Left") );
    labels.Add( _T("Bottom-Right") );

    return labels;
}

int TE3D_Align::getWxAlign( int align )
{
    switch ( align )
    {
        case TOP_LEFT:
            return ( wxALIGN_TOP | wxALIGN_LEFT );

        case TOP_RIGHT:
            return ( wxALIGN_TOP | wxALIGN_RIGHT );

        case BOTTOM_LEFT:
            return ( wxALIGN_BOTTOM | wxALIGN_LEFT );

        case BOTTOM_RIGHT:
            return ( wxALIGN_BOTTOM | wxALIGN_RIGHT );

        default:
            return ( wxALIGN_TOP | wxALIGN_LEFT );
    }
}

const size_t TE3D_Font::MIN_SIZE = 8;
const size_t TE3D_Font::MAX_SIZE = 20;
const size_t TE3D_Font::DEFAULT_SIZE = 14;

IntArray TE3D_Font::getValidSizes()
{
    IntArray sizes;

    for ( size_t i = MIN_SIZE; i <= MAX_SIZE; ++i ) {
        sizes.Add(i);
    }

    return sizes;
}

wxArrayString TE3D_Font::getValidSizesStrings()
{
    wxArrayString validSizesStrings;

    for ( size_t i = MIN_SIZE; i <= MAX_SIZE; ++i ) {
        validSizesStrings.Add( wxString::Format( _T("%d"), i ) );
    }

    return validSizesStrings;
}

wxString TE3D_Font::getDefaultSizeStr()
{
    wxString defaultSizeStr = wxString::Format( _T("%d"), TE3D_Font::DEFAULT_SIZE );
    return defaultSizeStr;
}

