#ifndef AIMPARSER_H
#define AIMPARSER_H


#include "common/types/TxDataList.h"
#include "common/types/Int2IntHash.h"
#include "vemsprotocol/ProtocolBase.h"


struct AIMData
{
    uint8_t channel;
    uint8_t high;
    uint8_t low;
    uint8_t sum;
    static const uint8_t AIM_CONST;
    void reset();
};

class AIMParser  : public ProtocolBase
{
    public:
        enum State { START = 2, END = 5, CHECKSUM_ERROR=6 };
        enum Sync {SYNC_0 = 4, SYNC_OK = 5,  SYNC_2 = 1, SYNC_3 = 2, SYNC_4 =4};
        AIMData data;

        AIMParser( wxEvtHandler* eventHandler = NULL );
        virtual ~AIMParser();

          //ProtocolBase virtual functions
        int     rx( uint8_t* c, int size );
        int     tx( uint8_t* c, int& size );
        int     type() const { return ECUInfo::AIM; }
        bool    isDisabled() { return m_bIsDisabled; }
        void    request( VemsCommand* command );

        bool hasError() { return false; }

        int feed( uint8_t c );
        int syncRecord( uint8_t* c );

        Int2IntHash& getChannelDataHash(){return channelDataHash; }
        //Int2IntHash& getChannelWeightHash(){ return channelWeightHash; }
        void sendChannel(uint8_t channel);

        void disable() { m_bNeedDisabled = true; }
        void enable()   { m_bIsDisabled = false; }
        void reset() { m_bIsDisabled = false; m_bNeedDisabled = false; }

    protected:
    private:
        wxEvtHandler*   m_pEventHandler;
        TxDataList      m_txQueue;
        uint8_t         m_prevByte;
        uint8_t         m_sum;
        uint8_t         m_state;
        bool            m_bNeedDisabled;
        bool            m_bIsDisabled;
        bool            m_bIsIdle;
        //int             m_iClearCount;
        int             m_iEmptyCount;

        Int2IntHash channelDataHash;
        //Int2IntHash channelWeightHash;

        void setSum(TxData* tx);
        TxData*         m_pCurrentCommand;

};


#endif // AIMPARSER_H
