#ifndef COMMMODELGUIMAIN_H
#define COMMMODELGUIMAIN_H


#include <wx/wx.h>
#include <wx/progdlg.h>
#include <wx/tglbtn.h>
#include <wx/slider.h>

#include "Observable.h"
#include "ObserverBase.h"

class VemsDeviceControllerEvent;
class CommunicationEvent;
class InputEvent;
class OffsetReaderEvent;

class VemsDeviceController;
class SIPRFileLogger;
class STDataModelBase;
class FirmwareConfigItemArray;
class SIPRPacket;
class DeviceControllerProgressBarManager;
class SDCardEvent;
class ConfigDownloadController;
class ConfigSaver2;
class FirmwareConfigManager;
class DummyUserDialogBase;
class MTConfigData;

class GaugeFrame;

class MyFrame: public wxFrame, public Observable, public ObserverBase
{
	public:
		MyFrame(wxFrame *frame, const wxString& title);
		~MyFrame();

    public:
		void notify( int i, void* ptr );

	private:
        VemsDeviceController*   m_controller;
        STDataModelBase*        m_model;
        SIPRFileLogger*         m_siprFileLogger;

        DeviceControllerProgressBarManager* m_progressBarManager;

        ConfigDownloadController*   m_configDownloadController;
        ConfigSaver2*               m_configSaver;

        FirmwareConfigManager*      m_pFirmwareConfigManager;
        DummyUserDialogBase*        m_dummyUserDialogBase;

        GaugeFrame*             m_gaugeFrame;

        wxSlider*               m_filePosSlider;

        bool                    m_v3Online;

        wxButton*               m_startStopButton;

    private:
        void            startDevice();

        // wxProgressDialog*       m_sdcardDlg;
        // wxToggleButton*         m_readOffsetButton;

        void            onSliderScroll( wxScrollEvent& event );

        /// MenuItemEvent handlers
		void            onECUConfig( wxCommandEvent & event );
        // void            onTestMenu( wxCommandEvent& event );

        void            onFileOpen( wxCommandEvent& event );

        void            onV3Button( wxCommandEvent& event );

		/// ToolBarButton handlers
//        void onDeviceButton( wxCommandEvent& event );
//        void onBaudrateComboBox( wxCommandEvent& event );
//        void onReadOffset( wxCommandEvent& event );

        /// CommunicationEvent handlers
        void onCannotOpenPort( CommunicationEvent& event );
        void onCommunicationError( CommunicationEvent& event );

        /// VemsDeviceControllerEvent handlers
        void onDeviceFound( VemsDeviceControllerEvent& event );
        void onDeviceNotFound( VemsDeviceControllerEvent& event );
        void onDeviceConnected( VemsDeviceControllerEvent& event );
        void onDeviceDisconnected( VemsDeviceControllerEvent& event );
        void onDeviceControllerStarted( VemsDeviceControllerEvent& event );
        void onDeviceControllerStopped( VemsDeviceControllerEvent& event );
        void onDeviceBaudRateChanged( VemsDeviceControllerEvent& event );
        //void onDeviceResumed( VemsDeviceControllerEvent& event );


		/// InputEvent handler
		void onInputEvent( InputEvent& event );
		void onConfigDownloadEvent( InputEvent& event );

		void reloadGauges( MTConfigData * mtconfigdata = NULL, VemsDeviceController* controller = NULL );
		//void onOffsetREvent( OffsetReaderEvent& evt );

		DECLARE_EVENT_TABLE();
};
#endif // COMMMODELGUIMAIN_H
