#include "ConfigFile.h"

#include <wx/wfstream.h>
#include <wx/fileconf.h>
#include <wx/filename.h>
#include "logger.h"

ConfigFile::ConfigFile(const wxString& fileName)
{
    m_fileName = fileName;
    load();
}

ConfigFile::~ConfigFile()
{
	delete m_wxFileConfig;
	delete m_wxFileInputStream;
}

bool ConfigFile::load()
{
    checkAndCreateFile();
    m_wxFileInputStream = new wxFileInputStream(m_fileName);
    m_wxFileConfig = new wxFileConfig(  *m_wxFileInputStream,  wxConvUTF8);
    return true;
}


bool ConfigFile::save()
{
    wxOutputStream * fos = (wxOutputStream *) new wxFileOutputStream(m_fileName);
    m_wxFileConfig->Save(*fos, wxConvUTF8);
    return true;
}

bool ConfigFile::flush()
{
    return m_wxFileConfig->Flush();
}

void ConfigFile::setPath(const wxString& strPath)
{
    return m_wxFileConfig->SetPath(strPath);
}

const wxString& ConfigFile::getPath()
{
    return m_wxFileConfig->GetPath();
}

bool ConfigFile::read(const wxString& key, wxString* str)
{
    return m_wxFileConfig->Read(key, str);
}

bool ConfigFile::read(const wxString& key, wxString* str, const wxString& defVal)
{
    return m_wxFileConfig->Read(key, str, defVal);
}

wxString ConfigFile::read(const wxString& key, const wxString& defVal)
{
    return m_wxFileConfig->Read(key, defVal);
}

bool ConfigFile::read(const wxString& key, long* data)
{
    return m_wxFileConfig->Read(key, data);
}

bool ConfigFile::read(const wxString& key, long* data, long defVal)
{
    return m_wxFileConfig->Read(key, data, defVal);
}

long ConfigFile::read(const wxString& key, long defVal)
{
    return m_wxFileConfig->Read(key, defVal);
}

bool ConfigFile::read(const wxString& key, double* data)
{
    return m_wxFileConfig->Read(key, data);
}

bool ConfigFile::read(const wxString& key, double* data, double defVal)
{
    return m_wxFileConfig->Read(key, data, defVal);
}

double ConfigFile::read(const wxString& key, double defVal)
{
    double value;
    m_wxFileConfig->Read(key, &value, defVal);
    return value;
}

bool ConfigFile::read(const wxString& key, bool* data)
{
    return m_wxFileConfig->Read(key, data);
}

bool ConfigFile::read(const wxString& key, bool* data, bool defVal)
{
    return m_wxFileConfig->Read(key, data, defVal);
}

bool ConfigFile::read(const wxString& key, bool defVal)
{
    return m_wxFileConfig->Read(key, defVal);
}

bool ConfigFile::saveValue(const wxString& path, const wxString& key, const wxString& value)
{
    wxString oldPath = m_wxFileConfig->GetPath();
    m_wxFileConfig->SetPath( path );
    bool res = m_wxFileConfig->Write(key, value );
    flush();
    save();
    m_wxFileConfig->SetPath( oldPath );
    return res;
}

wxString ConfigFile::getValue(const wxString& path, const wxString& key, const wxString& defValue)
{

    //Logger::logf(LOG_MSG,_T("ConfigHelper::getValue config=%p"),config);
    wxString oldPath = m_wxFileConfig->GetPath();
    m_wxFileConfig->SetPath( path );
    wxString value = m_wxFileConfig->Read(key, defValue );
    m_wxFileConfig->SetPath( oldPath );
    return value;
}

bool ConfigFile::write(const wxString& key, const wxString& value)
{
    return m_wxFileConfig->Write(key, value);
}

bool ConfigFile::write(const wxString& key, int value)
{
    return m_wxFileConfig->Write(key, value);
}

bool ConfigFile::write(const wxString& key, double value)
{
    return m_wxFileConfig->Write(key, value);
}

bool ConfigFile::write(const wxString& key, bool value)
{
    return m_wxFileConfig->Write(key, value);
}

uint32_t ConfigFile::getNumberOfEntries(bool bRecursive)
{
    return (uint32_t) m_wxFileConfig->GetNumberOfEntries(bRecursive);
}

bool ConfigFile::getFirstEntry(wxString& str, long& index)
{
    return m_wxFileConfig->GetFirstEntry(str, index);
}

bool ConfigFile::getNextEntry(wxString& str, long& index)
{
    return m_wxFileConfig->GetNextEntry(str, index);
}

ConfigFile::EntryType ConfigFile::getEntryType(const wxString& name)
{
    //return ConfigFile::Type_Unknown; /// why didn't implement this?

    switch(m_wxFileConfig->GetEntryType(name)){
        case wxConfigBase::Type_Unknown: return ConfigFile::Type_Unknown;
        case wxConfigBase::Type_Boolean: return ConfigFile::Type_Boolean;
        case wxConfigBase::Type_String: return ConfigFile::Type_String;
        case wxConfigBase::Type_Float: return ConfigFile::Type_Float;
        case wxConfigBase::Type_Integer: return ConfigFile::Type_Integer;
        default: return ConfigFile::Type_Unknown;
    }

}

bool ConfigFile::hasEntry(const wxString& name)
{
    return m_wxFileConfig->HasEntry(name);
}

void ConfigFile::checkAndCreateFile()
{
    bool isExist = wxFileName::FileExists(m_fileName);
    if(! isExist){
        wxFile tmpFile;
        tmpFile.Create(m_fileName);
        tmpFile.Close();
   }
}
