#ifndef FILEDIALOG_H
#define FILEDIALOG_H

#include <wx/listctrl.h>
#include <wx/wx.h>
#include "common/typedef.h"

enum FileDialogColoums{

    FD_NAME = 0x01,
    FD_SIZE = 0x02,
    FD_DATE = 0x04,
    FD_DEFAULT = FD_NAME | FD_SIZE | FD_DATE

};

class FileDialog: public wxDialog
{
    public:
        FileDialog(wxWindow *parent, long cols = FD_DEFAULT, wxWindowID id = wxID_ANY, const wxString &title = _T("Open file"), wxPoint pos = wxDefaultPosition, wxSize size = wxSize(240,320), long style = wxDEFAULT_DIALOG_STYLE);        virtual ~FileDialog();

        void SetDir(const wxString &path){
            m_currentDir = path;
        }

        wxString GetPath(){
            return m_selectedFile;
        }

        void GetAllPath(wxArrayString *names, bool isFullPath = true);

        virtual int ShowModal();

        void SetResponseTime(uint16_t millisec) { m_responseTime = millisec; };

    protected:

        enum NameType{
            DIR,
            FILE
        };

        void InitList();
        void FillList();

        void AddLine(const wxString &name, NameType type);
        void SetActualPath(const wxString &path);

        void OnItemActivated(wxListEvent &evt);
        void OnItemSelected(wxListEvent &evt);

        void OnListCtrlLeftDown(wxMouseEvent &evt);

        bool OpenFolder(const wxString &name);
        void OnTimer( wxTimerEvent& event );

    private:

        wxTimer     m_timer;

        wxListCtrl  *m_listCtrl;
        wxTextCtrl  *m_currPath;

        wxBoxSizer  *m_mainsizer;

        wxString    m_currentDir;
        wxString    m_selectedFile;

        long        m_cols;

        bool        m_isClick;

        wxString    m_currItem;
        uint16_t    m_responseTime;

    DECLARE_EVENT_TABLE();

};



#endif // FILEDIALOG_H
