#include "FirmwareInfo.h"
#include "FirmwareInfoHelper.h"

#include <wx/filename.h>
#include <wx/dir.h>

#include "logger.h"
#include "DirHelper.h"
#include "Math.h"
#include "vemsprotocol/ECUInfo.h"

IMPLEMENT_DYNAMIC_CLASS(FirmwareInfo, wxObject)

FirmwareInfo::FirmwareInfo()
        :m_compilationTime (wxEmptyString)
        ,m_firmwareVersion (wxEmptyString)
        ,m_serialNumber (wxEmptyString)
{
    //ctor
    m_fwOptionsH[PS2KEYBOARD] = _T("PS2KEYBOARD");
    m_fwOptionsH[N2O_RETARD] = _T("N2O_RETARD");
    m_fwOptionsH[WARNING_BEEP] = _T("WARNING_BEEP");
    m_fwOptionsH[EXPENSIVE_MISC1OUT] = _T("EXPENSIVE_MISC1OUT");
    m_fwOptionsH[EXPENSIVE_MISC2OUT] = _T("EXPENSIVE_MISC2OUT");
    m_fwOptionsH[INJECTOR_DEADTIME_SIMPLE] = _T("INJECTOR_DEADTIME_SIMPLE");
    m_fwOptionsH[INJECTOR_TRIM] = _T("INJECTOR_TRIM");
    m_fwOptionsH[KNOCK_ALTERNATIVE] = _T("KNOCK_ALTERNATIVE");

    m_fwOptionsH[TOYOTA_1JZ_VVTI] = _T("TOYOTA_1JZ_VVTI");
    m_fwOptionsH[A340E] = _T("A340E");
    m_fwOptionsH[ALTERNATIVE_AE_DE_FILM] = _T("ALTERNATIVE_AE_DE_FILM");

	m_fwOptions = -1;
	LOGP( wxString::Format( _T("m_fwOptions: %d"), m_fwOptions ) );
	m_device = ECUInfo::DEVICE_UNKNOWN;
}

FirmwareInfo::~FirmwareInfo()
{
    //dtor
}

void FirmwareInfo::getProhibit(int& prohibit)
{

    FirmwareVersion fwv;
    extractFirmwareVersion( m_firmwareVersion, fwv );
    uint8_t build = fwv.build & 127;    // 7 bit
    uint8_t minor = fwv.minor & 7;      // 3 bit;
    uint8_t major = fwv.major & 7;      // 3 bit;

    bool isPs2 = m_detectedParams[ m_fwOptionsH[PS2KEYBOARD] ];
    uint16_t ps2 = 0;
    if(isPs2){
        ps2 = 1 << 15;
    }
    // bit 13, 14 is leaved as it was
    prohibit =  ps2 | (major << 10) | (minor << 7) | build;
}

FirmwareVersion FirmwareInfo::getFirmwareVersion() const
{
    FirmwareVersion fwv;
    extractFirmwareVersion( m_firmwareVersion, fwv );
    return fwv;
}

void FirmwareInfo::setCompilationTime(const wxString& str)
{
    /// removes /r/0
    const wxCharBuffer buffer = str.To8BitData();
    m_compilationTime = wxString::FromAscii(buffer);
    m_compilationTime.Trim(true);
     m_compilationTime.Trim(false);
/*
    m_compilationTime = str;
    Logger::logf( LOG_MSG, _T("FirmwareInfo::setCompilationTime() ") + getCompilationTime() );
    m_compilationTime.Trim();
*/
//    Logger::logf( LOG_MSG, _T("FirmwareInfo::setCompilationTime():") + getCompilationTime() +_T("|"));

}

void FirmwareInfo::setFirmwareVersion(const wxString& str)
{
     /// removes /r/0
     const wxCharBuffer buffer = str.To8BitData();
    m_firmwareVersion = wxString::FromAscii(buffer);
    m_firmwareVersion.Trim(true);
    m_firmwareVersion.Trim(false);
//    Logger::logf( LOG_MSG, _T("FirmwareInfo::setFirmwareVersion():") + getFirmwareVersionAsString()+_T("|"));
}

void FirmwareInfo::setSerialNumber(const wxString& str)
{
    m_serialNumber = str;
    m_serialNumber.Trim(true);
    m_serialNumber.Trim(false);
//    Logger::logf( LOG_MSG, _T("FirmwareInfo::setSerialNumber():") + getSerialNumber()+_T("|"));
    if( m_serialNumber.StartsWith(_T("v")) )
    {
        m_device = ECUInfo::DEVICE_V3;
    }else if( m_serialNumber.StartsWith(_T("r")) )
    {
        m_device = ECUInfo::DEVICE_ROUND;
    }else{
        m_serialNumber = _T("UNKNOWN");
        m_device = ECUInfo::DEVICE_V3;
        //m_device = ECUInfo::DEVICE_UNKNOWN;
    }
}

void FirmwareInfo::setSignature(const wxString& str)
{
    m_signature = str;
    m_signature.Trim(true);
    m_signature.Trim(false);
/*
    if( str.Find(_T("Round")) != wxNOT_FOUND ){
        m_device = ECUInfo::DEVICE_ROUND;
    }else{
        m_device = ECUInfo::DEVICE_V3;
    }
*/
//    Logger::logf( LOG_MSG, _T("FirmwareInfo::setSignature():") + getSignature()+_T("|"));
}

bool FirmwareInfo::isEmpty() const
{
    switch ( m_device )
    {
        case ECUInfo::DEVICE_V3:
            return ( m_compilationTime.IsEmpty() &&
                     m_firmwareVersion.IsEmpty() &&
                     m_serialNumber.IsEmpty() &&
                     m_signature.IsEmpty() );
        case ECUInfo::DEVICE_ROUND:
            return ( m_firmwareVersion.IsEmpty() && m_serialNumber.IsEmpty() && m_signature.IsEmpty() );
        default:
            return true;
    }
}

bool FirmwareInfo::is12x12() const
{
    return ( ( m_signature.Find( _T("12x12") ) != wxNOT_FOUND ) ||
             ( m_signature.Find( _T("12X12") ) != wxNOT_FOUND ) );
}

bool FirmwareInfo::is16x14() const
{
    return ( ( m_signature.Find( _T("16x14") ) != wxNOT_FOUND ) ||
             ( m_signature.Find( _T("16X14") ) != wxNOT_FOUND ) );
}

bool FirmwareInfo::isHighBoost() const
{

    return ( ( m_signature.Find( _T("kpa=2,2") ) != wxNOT_FOUND ) ||
                ( m_signature.Find( _T("kpa=4,2") ) != wxNOT_FOUND ) );
}

bool FirmwareInfo::isNormalBoost() const
{
    return ( ( m_signature.Find( _T("kpa=2,1") ) != wxNOT_FOUND ) ||
            ( m_signature.Find( _T("kpa=4,1") ) != wxNOT_FOUND ) );
}

void FirmwareInfo::extractFirmwareVersion( const wxString& s, FirmwareVersion& v )
{
    FirmwareInfoHelper::fillVersion( s, v );
}

void FirmwareInfo::forceParams(const BoolHash& detectedParams)
{
    BoolHash::const_iterator IT = detectedParams.begin();
    for ( ; IT !=  detectedParams.end(); ++IT ){
        wxString configName = IT->first;
        bool value = IT->second;
        m_detectedParams[configName] = value;
    }
}

void FirmwareInfo::setFirmwareOptions(const wxString& str)
{
    m_fwOptionsStr = str;
    size_t origLen = str.Len();
    m_detectedParams.clear();
    if( (origLen >= 26) && (origLen < 30) ){
        wxChar upper = str.GetChar(24) & 255;
        wxChar lower = str.GetChar(25) & 255;
        m_fwOptions = 256 * upper + lower;
        fillDetectedParams();
    }

    setHighAndLowBoost();

    LOGP( wxString::Format( _T("m_fwOptions: %d"), m_fwOptions ) );
}


void FirmwareInfo::setFirmwareCompileOptions(const wxString& str)
{
    if(str == wxEmptyString) return;
    wxString s = str;
    s.Trim(false);
    s.Trim(true);
    Logger::logf( LOG_MSG, _T("FirmwareInfo::setFirmwareCompileOptions(): ")+s);

    m_fwOptions = Math::hex2int(s);
    m_detectedParams.clear();
    fillDetectedParams();
    setHighAndLowBoost();
    LOGP( wxString::Format( _T("m_fwOptions: %d"), m_fwOptions ) );
}

void FirmwareInfo::fillDetectedParams()
{
    Int2StringHash::const_iterator IT = m_fwOptionsH.begin();
    for ( ; IT !=  m_fwOptionsH.end(); ++IT ){
        int intKey = IT->first;
        wxString configName = IT->second;

        int value = (m_fwOptions & intKey);
        if(value == intKey){
            m_detectedParams[configName] = 1;
        }else{
            m_detectedParams[configName] = 0;
        }
    }
}

void FirmwareInfo::setHighAndLowBoost()
{
    if( is12x12() ){
         m_detectedParams[_T("12X12")] = true;
    }else{
        m_detectedParams[_T("12X12")] = false;
    }

    if(isHighBoost() ){
        m_detectedParams[_T("HIGHBOOST")] = true;
    }else{
        m_detectedParams[_T("HIGHBOOST")] = false;
    }
}

 bool FirmwareInfo::isValid() const
 {
    switch ( m_device )
    {
        case ECUInfo::DEVICE_V3:
            if( m_firmwareVersion.StartsWith(_T("1")) ) return true;
            return false;
        case ECUInfo::DEVICE_ROUND:
            return true;
        default:
            return false;
    }
}

wxString FirmwareInfo::getHeaderFile() const
{
    wxString fwVersion = getFirmwareVersionAsString();

	wxString headerFile;
    if(getDevice()==ECUInfo::DEVICE_ROUND)
		headerFile = DirHelper::GetPath( dPROJECTS ) + _T("round_") + fwVersion + DirHelper::Separ() + _T("global.h");
	else
		headerFile = DirHelper::GetPath( dPROJECTS ) + _T("firmware_") + fwVersion + DirHelper::Separ() + _T("global.h");

    Logger::logf( LOG_MSG, _T("FirmwareInfo::getHeaderFile() header: ") + headerFile);

    if( !wxFileName::FileExists(headerFile) ){
        return wxEmptyString;
    }
    return headerFile;
}

wxString FirmwareInfo::geDefaultConfigFile() const
{
    FirmwareVersion fwvECU  = getFirmwareVersion();

	wxString path = DirHelper::GetPath( dDEFAULTCONFIGS );

	if ( !wxDir::Exists(path) ) { return wxEmptyString; }

    wxDir dir( path );

    if ( !dir.IsOpened() ) { return wxEmptyString; }

    FirmwareVersion fwv;
    wxString nearest;
    wxString filename;

    wxString filespec;
    int device = getDevice();
    switch(device){
        case ECUInfo::DEVICE_ARM:
            filespec = _T("arm_*.vemscfg");
            break;
        case ECUInfo::DEVICE_V3:
            filespec = _T("config_*.vemscfg");
            break;
        case ECUInfo::DEVICE_ROUND:
            filespec = _T("round_*.vemscfg");
            break;
        default:
            filespec = _T("config_*.vemscfg");
            break;
    }

    for ( bool b = dir.GetFirst(&filename, filespec ); b; b = dir.GetNext(&filename) )
    {
        wxString version = filename.BeforeLast( _T('.') ).AfterLast( _T('_') );
        FirmwareInfo::extractFirmwareVersion( version, fwv );



        if ( fwv == fwvECU ) { return path + filename; }
        else if ( ( fwv < fwvECU ) && ( fwv.minor == fwvECU.minor ) )
        {
            FirmwareVersion f;
            wxString v = nearest.BeforeLast( _T('.') ).AfterLast( _T('_') );
            FirmwareInfo::extractFirmwareVersion( v, f );
            if( fwv > f ) { nearest = filename; }
        }
    }

    if ( !nearest.IsEmpty() ) nearest = path + nearest;

    return nearest;

    //wxString fwVersion = getFirmwareVersionAsString();
    //return DirHelper::GetPath( dDEFAULTCONFIGS ) + _T("config_") + fwVersion + _T(".vemscfg");
}

void FirmwareInfo::setDevice(int device)
{
    m_device = device;
    //LOGP( wxString::Format( _T("m_device: %d, device: %d"), m_device, device ) )
}

int FirmwareInfo::getDevice() const
{
    //LOGP( wxString::Format( _T("m_device: %d"), m_device ) )
    return m_device;
}

wxString FirmwareInfo::getCompileOptions() const
{
	if(m_fwOptions == -1) return  wxEmptyString;

	return wxString::Format(_T("%08X"), m_fwOptions);
}

long FirmwareInfo::getSerialNumberAsLong() const
{
    long serialNumber = -1;
    wxString tmp = m_serialNumber.Lower().AfterFirst( _T('_') );
    if(tmp.IsEmpty() ) return serialNumber;

    for(size_t i = 0; i< tmp.Len(); ++i ){
        bool res = tmp.Mid( i ).ToLong( &serialNumber );
        if(res) return serialNumber;
    }

    //LOGP( wxString::Format( _T("serialNumber: ")+ m_serialNumber ) );
    //LOGP( wxString::Format( _T("serialNumber: %d"), serialNumber ) );
    return serialNumber;
};


wxString FirmwareInfo::getIniFile() const
{
    FirmwareVersion fwvECU  = getFirmwareVersion();

	wxString path = DirHelper::GetPath( dCONFIG );

	if ( !wxDir::Exists(path) ) { return wxEmptyString; }

    wxDir dir( path );

    if ( !dir.IsOpened() ) { return wxEmptyString; }

    FirmwareVersion fwv;
    wxString nearest;
    wxString filename;

    wxString devicePrefix = getIniFileDevicePrefix( getDevice() );

    wxString platformPrefix;
	#if defined(__WINCE_STANDARDSDK__)
		platformPrefix = _T("cockpit-");
	#else
		platformPrefix = _T("vemsTune-");
	#endif
    wxString filespec = platformPrefix + devicePrefix+ _T("*.ini");

    for ( bool b = dir.GetFirst(&filename, filespec ); b; b = dir.GetNext(&filename) )
    {
        wxString version = filename.BeforeLast( _T('.') ).AfterLast( _T('-') );
        FirmwareInfo::extractFirmwareVersion( version, fwv );

        if ( fwv == fwvECU ) { return filename; }
        else if ( ( fwv < fwvECU ) && ( fwv.minor == fwvECU.minor ) )
        {
            FirmwareVersion f;
            wxString v = nearest.BeforeLast( _T('.') ).AfterLast( _T('-') );
            FirmwareInfo::extractFirmwareVersion( v, f );
            if( fwv > f ) { nearest = filename; }
        }
    }

    return nearest;
}


wxArrayString FirmwareInfo::getAvailableIniFiles( int device )
{
    wxArrayString iniFiles;

    wxString configDirName = DirHelper::GetPath( dCONFIG );

    wxString platformPrefix = getIniFilePlatformPrefix();
    wxString devicePrefix = getIniFileDevicePrefix( device );
    wxString filespec = platformPrefix + devicePrefix + _T("*.ini");

    wxDir::GetAllFiles( configDirName, &iniFiles, filespec );

    return iniFiles;
}

bool FirmwareInfo::isVersionAndCompileOptionsIdentical( const FirmwareInfo& firmware ) const
{
    bool verEq = ( getFirmwareVersion() == firmware.getFirmwareVersion() );
    bool opEq = ( getCompileOptions() == firmware.getCompileOptions() );
    bool opEmpty = ( getCompileOptions().IsEmpty() && firmware.getCompileOptions().IsEmpty() );

    /*
        verEq
        opEq


    */

    bool isIdentical = false;

    if ( m_device == ECUInfo::DEVICE_V3 ) {
        isIdentical = verEq && opEq && !getCompileOptions().IsEmpty();
    } else {
        isIdentical = verEq;
    }

    LOGP( wxString::Format( _T("m_device: %d"), m_device ) );
    LOGP( wxString::Format( _T("isVersionEqual: %d"), opEq ) );
    LOGP( _T("this: ") + getFirmwareVersionAsString() + _T("param: ") + firmware.getFirmwareVersionAsString() );
    LOGP( wxString::Format( _T("isOptionsEqual: %d"), opEq ) );
    LOGP( wxString::Format( _T("this: ") + getCompileOptions() + _T(", param: ") + firmware.getCompileOptions() ) );
    LOGP( wxString::Format( _T("isIdentical: %d"), isIdentical ) );

    return isIdentical;
}


wxString FirmwareInfo::getIniFileDevicePrefix( int device )
{
    switch( device )
    {
        case ECUInfo::DEVICE_ARM:
            return _T("Arm");
        case ECUInfo::DEVICE_V3:
            return _T("v3");
        case ECUInfo::DEVICE_ROUND:
            return _T("round");
        default:
            return _T("v3");
    }
}


wxString FirmwareInfo::getIniFilePlatformPrefix()
{
	#if defined(__WINCE_STANDARDSDK__)
		return _T("cockpit-");
	#else
		return _T("vemsTune-");
	#endif
}
