#include <math.h>
#include "MGraphics.h"

// #include <iostream>
// #include "gaugeframe.h"
// #include "vemsgauge.h"

#include <wx/dcclient.h>
#include <wx/brush.h>

#include <wx/image.h>

#include "../logger.h"


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
TMGraphics::TMGraphics()
  {
    BMP = NULL;
    SL = NULL;

    ROW = new BYTE[3*100];

    GT = NULL;

  }//TMGraphics::TMGraphics()

//-----------------------------------------------------------------------
TMGraphics::~TMGraphics()
  {
    delete BMP;
    delete SL;
    delete ROW;
    delete GT;
  }//TMGraphics::~TMGraphics()

//-----------------------------------------------------------------------
bool TMGraphics::CheckRawAccess(wxBitmap Bitmap)
  {
    bool ret_val;
    try
      {
        PixelData data(Bitmap);
        ret_val = true;
      }
    catch(...)
      {
        ret_val = false;
      }

    return ret_val;
  }//TMGraphics::CheckRawAccess() 1.

//-----------------------------------------------------------------------
bool TMGraphics::CheckRawAccess()
  {
    RAW_ACCESS = CheckRawAccess(*BMP);
    return RAW_ACCESS;
  }//TMGraphics::CheckRawAccess() 2.

//-----------------------------------------------------------------------
// Unsafety method, the DC wxWidget can to be buggy the BMP bitmap
// Not allowed to join this DC another DC, with Blit or another like function!
// Use this only drawing!
//
void TMGraphics::GetMemoryDC(wxMemoryDC *DC)
  {
    DC->SelectObject(*BMP);
  }//TMGraphics::GetMemoryDC()

//-----------------------------------------------------------------------
bool TMGraphics::CopyToBitmap(wxBitmap **bmp_T)
  {
    if((*bmp_T)->GetDepth() != 24)
      {
        // Don't warranted the good working (DC bug!)
        Logger::log(LOG_MSG,_T("Target BitDepth is not supported!"));
        return false;
      }
    else
      {
        return CopyBitmap(bmp_T, BMP);
      }
  }//TMGraphics::CopyToBitmap()

//-----------------------------------------------------------------------
bool TMGraphics::CopyFromBitmap(wxBitmap *bmp_S)
  {
    return CopyBitmap(&BMP, bmp_S);
  }//TMGraphics::CopyFromBitmap()

//-----------------------------------------------------------------------
bool TMGraphics::CopyFrom(TMGraphics *From)
  {
    bool ret_val;

    ret_val = CopyBitmap(&BMP, From->BMP);
    if(ret_val)
      ret_val = BuildSL();

    return ret_val;
  }//TMGraphics::CopyFrom()

//-----------------------------------------------------------------------
bool TMGraphics::IsValid()
  {
    return (BMP != NULL);
  }//TMGraphics::IsValid()

//-----------------------------------------------------------------------
bool TMGraphics::Reset(int w, int h)
  {
    bool  ret_val;

    delete BMP;
    W = w;
    H = h;
    BMP = new wxBitmap(W, H, 24);

    if(BMP != NULL)
      {
        ret_val = BuildSL();
      }
    else
      ret_val = false;

    return ret_val;
  }//TMGraphics::Reset()

//-----------------------------------------------------------------------
bool TMGraphics::BuildSL()
  {
    bool  ret_val;
    int   Y;

    // Build SL ScanLines
    if(BMP != NULL)
      {
        delete SL;

        H = BMP->GetHeight();
        SL = new BYTE*[H];
        //Logger::logf(LOG_MSG, _T("TMGraphics::BuildSL() Height = %i"), H);
        PixelData Data(*BMP);

        if ( !Data )
          {
            ret_val = false;
          }
        else
          {
            ret_val = true;
            PixelData::Iterator P(Data);

            for(Y = 0; Y < H; Y++)
              {
                P.MoveTo(Data, 0, Y);

                //(void *)&(p_B.Blue())
                SL[Y] = (BYTE *)&(P.Blue());
//                Logger::logf(LOG_MSG, _T("TMGraphics::BuildSL() SL[%i] = %0X"),Y, SL[Y]);
              }
          }
      }
    else{
        Logger::logf(LOG_MSG, _T("TMGraphics::BuildSL() FAILED BMP = NULL"));
      ret_val = false;
    }
    return ret_val;
  }//TMGraphics::BuildSL()

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
bool TMGraphics::CopyBitmap(wxBitmap **BMP_A, wxBitmap *BMP_B)
  {
    int W, H;
    int Y;
    int LW;

    if(BMP_B == NULL)
      return false;

    if(!BMP_B->IsOk())
      return false;

    W = BMP_B->GetHeight();
    H = BMP_B->GetWidth();

    if(BMP_B->GetDepth() == 32)
      LW = 4 * W;
    else if(BMP_B->GetDepth() == 24)
      LW = 3 * W;
    else if(BMP_B->GetDepth() == 16)
      LW = 2 * W;
    else if(BMP_B->GetDepth() == 8)
      LW = W;
    else if(BMP_B->GetDepth() == 1)
      LW = (W / 8);
    else
      {
        Logger::log(LOG_MSG,_T("BitDepth is unbeknown!"));
        return false;
      }

    delete *BMP_A;

   *BMP_A = new wxBitmap(W, H, BMP_B->GetDepth());

    if(*BMP_A == NULL)
      return false;

    if(!(*BMP_A)->IsOk())
      return false;


    PixelData data_A(**BMP_A);
    PixelData data_B(*BMP_B);

    if ( !data_A || !data_B)
      {
        // ... raw access to bitmap data unavailable, do something else ...
        return false;
      }

    PixelData::Iterator p_A(data_A);
    PixelData::Iterator p_B(data_B);

    try
      {
        for(Y = 0; Y < H; Y++)
          {
            p_A.MoveTo(data_A, 0, Y);
            p_B.MoveTo(data_B, 0, Y);
            memcpy((void *)&(p_A.Blue()), (void *)&(p_B.Blue()), LW);
          }

        return true;
      }
    catch(...)
      {
        return false;
      }
  }//TMGraphics::CopyBitmap()

//-----------------------------------------------------------------------
void TMGraphics::CreateGradient(TRGB rgb_A, TRGB rgb_B, int n)
  {
    int     i;
    BYTE    AR, AG, AB, BR, BG, BB;
    double  dR, dG, dB;

    if(GT != NULL)
      delete GT;

    CtG = n;
    GT = new TRGB[n];

    rgb_A.GetRGB(&AR, &AG, &AB);
    rgb_B.GetRGB(&BR, &BG, &BB);

    dR = (double)(BR - AR) / n;
    dG = (double)(BG - AG) / n;
    dB = (double)(BB - AB) / n;

    for(i=0; i<n; i++)
      {
        GT[i].R = (BYTE)(AR + i * dR);
        GT[i].G = (BYTE)(AG + i * dG);
        GT[i].B = (BYTE)(AB + i * dB);
      }

  }//TMGraphics::CreateGradient()

//-----------------------------------------------------------------------
void TMGraphics::FillRow(TRGB RGB)
  {
    int   X;
    BYTE  B, G, R;
    BYTE  *pB;

    RGB.GetRGB(&R, &G, &B);

    pB = ROW;
    for(X = 0; X < 100; X++)
      {
        *pB++ = B;
        *pB++ = G;
        *pB++ = R;
      }
  }//TMGraphics::FillRow()

//-----------------------------------------------------------------------
void TMGraphics::FillSegment(int Y, int X1, int X2)
  {
    int   T;
    int   Ct;
    int   X;
    BYTE  *pB;

    if(X2 < X1)
      {
        T = X2;
        X2 = X1;
        X1 = T;
      }

    if((0 <= Y) && (Y < H) &&
       (0 <= X1) && (X2 < W))
      {
        Ct = X2 - X1 + 1;
        X = X1;
        while(100 < Ct)
          {
            pB = SL[Y] + 3 * X;
            memcpy(pB, ROW, 300);
            Ct -= 100;
            X += 100;
          }
        pB = SL[Y] + 3 * X;
        memcpy(pB, ROW, Ct * 3);
      }

  }//TMGraphics::FillSegment()

//-----------------------------------------------------------------------
void TMGraphics::FillGradient(int Y, int X0, int X1, int X2)
  {
    int   T;
    int   Ct;
    int   XG1, XG2;

    if(GT == NULL)
      return;

    if(X2 < X1)
      {
        T = X2;
        X2 = X1;
        X1 = T;
      }

    Ct = X2 - X1 + 1;

    XG1 = X0;
    XG2 = X0 + Ct;

    if((0 <= Y) && (Y < H) &&
       (0 <= X1) && (X2 < W) &&
       (0 <= XG1) && (XG2 < CtG))
      {
        memcpy(SL[Y] + X1 * 3, (BYTE*)&GT[XG1], Ct * 3);
      }
  }//TMGraphics::FillGradient()
