#include "Math.h"

/// just for debug
#include "logger.h"

Math::Math()
{
    //ctor
}

Math::~Math()
{
    //dtor
}

double Math::GetPointOnLine(const wxRealPoint& A, const wxRealPoint& B, double Cy )  /// point A,B,C on same line
{
    double Cx = (Cy - (( B.y*A.x - B.x*A.y ) / ( A.x - B.x ))) / ( ( B.y - A.y ) / ( B.x - A.x ) );
    return Cx;
}

wxRealPoint Math::CalcCircPoint(double alpha, double radius, wxRealPoint center)
{
    wxRealPoint res;

    res.x = center.x + radius*cos(alpha*RAD_FACT);
    res.y = center.y - radius*sin(alpha*RAD_FACT);

    return res;

}

unsigned int Math::hex2int(const wxString & hexStr)
{
    size_t size = hexStr.size();
    if(size >8) size = 8;
    //fprintf(stdout,  "Math::hex2int string size: %d\n", size);
    size_t i;
    unsigned int result = 0;
    for(i=0; i<size; i++){
        char chr = hexStr.GetChar(i);
        uint8_t tmp = Math::hexDigitValue(chr);
        if(tmp == DIGITERROR) return 0;
        int raise = size -1 - i;
        int mul = powInt(16, raise);
        //fprintf(stdout,  "Math::hex2int mul= %d, tmp= %d\n", mul, tmp);
        result += mul * tmp;

    }
    return result;
}

wxString Math::char2hex(const uint8_t* buf, int cnt)
{
        wxString str;
        int i;
        for(i=0; i<cnt; i++){
                wxString tmpS;
                uint8_t tmp = buf[i];
                tmpS.Printf(_T(" %02X"),tmp);
                str.Append(tmpS);
        }

        return str;

}

int Math::powInt(int base, int raise)
{
    int result = 1;
    for(int i=0; i< raise; i++)
    {
        result *=base;
    }
    return result;
}


// input : '0'..'9' .. 'A' .. 'F'
// output: 0..15
// or 0xFF if error
uint8_t Math::hexDigitValue(uint8_t c)
{
    if(c<='9')
    {
        if(c>='0')
        {
            return c-'0';
        }
        else
            return DIGITERROR;
    }
    else
    {        // c>'9'
        if(c<='F' && c>= 'A')
        {
            return c-('A'-10);
        }
        else if(c<='f' && c>= 'a')
        {
            return c-('a'-10);
        }
        else
            return DIGITERROR;
    }
}

void Math::getBytes( int i, uint8_t* c, uint8_t length)
{
    switch(length){
        case 1:
            c[0] = i & 0xFF;
            break;
        case 2:
            c[0] = (i >> 8) & 0xFF;
            c[1] = i & 0xFF;
            break;
        case 3:
            c[0] = (i >> 16) & 0xFF;
            c[1] = (i >>  8) & 0xFF;
            c[2] = i & 0xFF;
            break;
        case 4:
        default:
            c[0] = (i >> 24) & 0xFF;
            c[1] = (i >> 16) & 0xFF;
            c[2] = (i >>  8) & 0xFF;
            c[3] = i & 0xFF;
    }

}

uint32_t Math::getNumber(uint8_t * ptr, uint8_t length)
{
	uint32_t value = 0;
    switch(length){
        case 1:
            value = ptr[0];
            break;
        case 2:
            value = ptr[0] << 8;
            value += ptr[1];
            break;
        case 3:
            value = ptr[0] << 16;
            value += ptr[1] << 8;
            value += ptr[2];
            break;
        case 4:
        default:
            value = ptr[0] << 24;
            value += ptr[1] << 16;
            value += ptr[2] << 8;
            value += ptr[3];
    }
	return value;
}

double Math::round( double d )
{

    double c = ceil( d );

    if( ( c - d ) > 0.5 )
    {
        return floor( d );
    }
    else
    {
        return ceil( d );
    }
}

/*
    Polynomial: x^16 + x^12 + x^5 + 1 (0x1021)<br>
    Initial value: 0x0

    SD Card uses it! crc_xmodem_update
*/
uint16_t Math::crc16(uint16_t crc, uint8_t data)
{
    int i;

    crc = crc ^ ((uint16_t)data << 8);
    for (i=0; i<8; i++)
    {
        if (crc & 0x8000)
            crc = (crc << 1) ^ 0x1021;
        else
            crc <<= 1;
    }

    return crc;
}

/*
 Optimized CRC-16 calculation.

    Polynomial: x^16 + x^15 + x^2 + 1 (0xa001)<br>
    Initial value: 0xffff

*/
uint16_t Math::crc16_sipr(uint16_t crc, uint8_t data)
{
    int i;

	crc ^= data;
	for (i = 0; i < 8; ++i)
	{
	    if (crc & 1)
		crc = (crc >> 1) ^ 0xA001;
	    else
		crc = (crc >> 1);
	}
    return crc;
}




/*
    p0 contains x0, y0
    p1 contains x1, y1
    px contains x   y = result
*/
bool Math::linearInterpolation2D(double* p0, double* p1, double* px)
{
    double x0 = p0[0];
    double y0 = p0[1];

    double x1 = p1[0];
    double y1 = p1[1];

    double x = px[0];

    if( (x1 > x0) && (x < x0 || x > x1) ||
        (x1 < x0) && (x > x0 || x < x1) ){
/*        Logger::logf(LOG_MSG, _T("-Math::linearInterpolation2D out of range") );
        Logger::logf(LOG_MSG, _T("-Math::linearInterpolation2D:? p0:(%f,%f"), x0, y0);
        Logger::logf(LOG_MSG, _T("-Math::linearInterpolation2D:? p1:(%f,%f"), x1, y1);
        Logger::logf(LOG_MSG, _T("-Math::linearInterpolation2D:? px:(%f,%f"), x,  px[1]);
*/
        return false;
    }

    if(x0 == x1 && y0 == y1){
        px[1] = y0;
/*        Logger::logf(LOG_MSG, _T("Math::linearInterpolation2D:? p0:(%f,%f"), x0, y0);
        Logger::logf(LOG_MSG, _T("Math::linearInterpolation2D:? p1:(%f,%f"), x1, y1);
        Logger::logf(LOG_MSG, _T("Math::linearInterpolation2D:? px:(%f,%f"), x,  px[1]);
*/
        return true;
    }

    if(x0 == x1){
/*
        Logger::logf(LOG_MSG, _T("-Math::linearInterpolation2D x0 == x1") );
        Logger::logf(LOG_MSG, _T("-Math::linearInterpolation2D:? p0:(%f,%f"), x0, y0);
        Logger::logf(LOG_MSG, _T("-Math::linearInterpolation2D:? p1:(%f,%f"), x1, y1);
*/
        return false;
    }

    px[1] = (y1-y0)/(x1-x0)*(x-x0)+y0;
/*
    Logger::logf(LOG_MSG, _T("Math::linearInterpolation2D: p0:(%f,%f"), x0, y0);
    Logger::logf(LOG_MSG, _T("Math::linearInterpolation2D: p1:(%f,%f"), x1, y1);
    Logger::logf(LOG_MSG, _T("Math::linearInterpolation2D: px:(%f,%f"), x,  px[1]);
*/
    return true;
}

/**
p0(x0,y0,z0) | p1(x1,y1,z1)
p2(x2,y2,z2) | p3(x3,y3,z3)

assume: x0 = x2, x1 = x3
assume: y0 = y1, y2 = y3

**/
bool Math::linearInterpolation3D(double* p0, double* p1, double* p2, double* p3, double* px)
{

    double p0Y[2];
    double p1Y[2];
    double pxY[2];

    /// y0
    ///interpolate: p0(x0,z0) | p1(x1,z1)
    p0Y[0]= p0[0]; /// x0
    p0Y[1]= p0[2]; /// z0
    p1Y[0]= p1[0]; /// x1
    p1Y[1]= p1[2]; /// z1
    pxY[0]= px[0]; /// px X
    bool res = linearInterpolation2D(p0Y, p1Y, pxY);
    if(!res) return false;

    double zP0P1 = pxY[1];

    /// y1
    ///interpolate: p2(x2,z2) | p3(x3,z3)
    p0Y[0]= p2[0]; /// x2
    p0Y[1]= p2[2]; /// z2
    p1Y[0]= p3[0]; /// x3
    p1Y[1]= p3[2]; /// z3
    pxY[0]= px[0]; /// xY1
    res = linearInterpolation2D(p0Y, p1Y, pxY);
    if(!res) return false;
    double zP2P3 = pxY[1];

    /// px
    ///pxY0
    ///---
    ///pxY1
    p0Y[0]= p0[1]; /// y0 or y1
    p0Y[1]= zP0P1; /// zxY0
    p1Y[0]= p2[1]; /// y2 or y3
    p1Y[1]= zP2P3; /// zxY2
    pxY[0]= px[1]; /// px Y
    res = linearInterpolation2D(p0Y, p1Y, pxY);
    if(!res) return false;
    px[2] = pxY[1];
    return true;
}

wxString printArray( double* d, size_t s );

bool Math::resample2D(const DoubleArray& oldX, const DoubleArray& oldY,
                const DoubleArray& newX, const DoubleArray& newY, double* data)
{

    size_t size = oldX.GetCount() * oldY.GetCount();
    double* oldData = new double[size];
    memcpy(oldData, data, size * sizeof( double ) );

    //Logger::logf( LOG_MSG, printArray( oldData, size ) );

    double p0[3],p1[3],p2[3],p3[3],px[3];
    int idxY0 =0, idxY1 =0, idxX0 = 0, idxX1 =0, idx =0;
    bool res = false;

    Logger::logf( LOG_MSG, _T("Math::resample2D()\n")
                  _T("newY.GetCount() = %d\n")
                  _T("newX.GetCount() = %d\n"),
                  newY.GetCount(), newX.GetCount() );

    for(size_t y =0; y < newY.GetCount(); y++){
        ///if ( y != 0 ) { continue; } TEST

        px[1] = newY.Item(y); /// px.y
        p0[1] = p1[1] = getLeftNeighbour(px[1], oldY, idxY0);
        p2[1] = p3[1] = getRightNeighbour(px[1], oldY, idxY1);

        size_t xSize = newX.GetCount();

        for(size_t x =0; x < newX.GetCount(); x++){
                px[0] = newX.Item(x); /// px.x
                p0[0] = p2[0] = getLeftNeighbour(px[0], oldX, idxX0);
                p1[0] = p3[0] = getRightNeighbour(px[0], oldX, idxX1);

                p0[2] = oldData[idxX0 + xSize * idxY0]; /// p0.z
                p1[2] = oldData[idxX1 + xSize * idxY0]; /// p1.z
                p2[2] = oldData[idxX0 + xSize * idxY1]; /// p2.z
                p3[2] = oldData[idxX1 + xSize * idxY1]; /// p3.z

                /*
                Logger::logf( LOG_MSG, _T(" p0: %d, p1: %d, p2: %d, p3: %d"),
                              idxX0 + xSize * idxY0,
                              idxX1 + xSize * idxY0,
                              idxX0 + xSize * idxY1,
                              idxX1 + xSize * idxY1 );
                */
                //Logger::logf( LOG_MSG, _T(" x0: %d, x1: %d, y0: %d, y1: %d"),
                //              idxX0, idxX1, idxY0, idxY1 );

            //if(x == 0)
            /*
                Logger::logf( LOG_MSG, _T("p0 :") + printArray( p0, 3 ) );
                Logger::logf( LOG_MSG, _T("p1 :") + printArray( p1, 3 ) );
                Logger::logf( LOG_MSG, _T("p2 :") + printArray( p2, 3 ) );
                Logger::logf( LOG_MSG, _T("p3 :") + printArray( p3, 3 ) );
            */
                res = Math::linearInterpolation3D(p0,p1,p2,p3,px);

                //Logger::logf( LOG_MSG, _T("px :") + printArray( px, 3 ) );
                //Logger::logf( LOG_MSG, _T("") );


                if(!res){
                    Logger::logf( LOG_MSG, _T("Math::resample2D() - !res - idx : %d"), idx );
                    delete[] oldData;
                    return false;
                }
                data[idx] = px[2];
                idx++;
        }

        Logger::logf( LOG_MSG, _T("Math::resample2D() - idx : %d"), idx );
    }

    delete[] oldData;

    return true;
}

double Math::getLeftNeighbour(double x, const DoubleArray& oldA, int& oldIdx)
{
    if(oldA.GetCount() < 1) return 0;

    for(size_t i = 0; i < oldA.GetCount()-1; i++){
        if ( oldA.Item( i ) == x ){
            oldIdx = i;
            return x;
        }

        if(oldA.Item(i) <= x && x < oldA.Item(i+1)){
            //Logger::logf( LOG_MSG, _T("Math::getLeftNeighbour() - idx=%i, value(i): %f,x=%f,value(x+1):%f"),i, oldA.Item(i), x, oldA.Item(i+1));
            oldIdx = i;
            return oldA.Item(i);
        }
    }

    if ( oldA.Item( oldA.GetCount()-1 ) == x )
    {
        oldIdx = oldA.GetCount()-1;
        return x;
    }

    oldIdx = 0;
    return 0;
}

double Math::getRightNeighbour(double x, const DoubleArray& oldA, int& oldIdx)
{
    if(oldA.GetCount() < 1) return 0;

    for(size_t i = 0; i < oldA.GetCount()-1; i++){

        if ( oldA.Item(i) == x ){
            oldIdx = i;
            return x;
        }

        if(oldA.Item(i) < x && x <= oldA.Item(i+1)){
            oldIdx = i+1;
            //Logger::logf( LOG_MSG, _T("Math::getRightNeighbour() - idx=%i, value(i): %f,x=%f,value(x+1):%f"),i, oldA.Item(i), x, oldA.Item(i+1));
            return oldA.Item(i+1);
        }
    }
    oldIdx = 0;
    return 0;
}

wxString printArray( double* d, size_t s )
{
    wxString str;

    for ( size_t i = 0; i < s; ++i )
    {
        str.Append( wxString::Format( _T("[%2i] = %12f "), i, d[i] ) );
    }

    return str;
}


double Math::linearInterpolation2D( double x0, double y0, double x1, double y1, double x )
{
    if ( x0 > x1 )
    {
        double tx = x0;
        double ty = y0;

        x0 = x1;
        y0 = y1;
        x1 = tx;
        y1 = ty;
    }

    if ( x < x0 ) { return y0; }

    if ( x > x1 ) { return y1; }

    if( x0 == x1 ) { return y0; }

    double y = (y1-y0)/(x1-x0)*(x-x0)+y0;

    return y;
}

double Math::linearInterpolation2D( double* p0, double* p1, double x )
{
    return linearInterpolation2D( p0[0], p0[1], p1[0], p1[1], x );
}


/**
p0(x0,y0,z0) | p1(x1,y1,z1)
p2(x2,y2,z2) | p3(x3,y3,z3)

assume: x0 = x2, x1 = x3
assume: y0 = y1, y2 = y3

px[2]

**/
double Math::linearInterpolation3D( double* p0, double* p1, double* p2, double* p3, double x, double y )
{
    /// interpolate: p0(x0,z0) | p1(x1,z1)
    double zP0P1 = linearInterpolation2D( p0[0], p0[2], p1[0], p1[2], x );

    /// interpolate: p2(x2,z2) | p3(x3,z3)
    double zP2P3 = linearInterpolation2D( p2[0], p2[2], p3[0], p3[2], x );

//    Logger::logf( LOG_MSG, _T("-----Math::linearInterpolation3D() - p0[1] = %f; zP0P1 = %f; p[1] = %f; zP2P3 = %f; y = %f"),
//                    p0[1], zP0P1, p2[1], zP2P3, y );

    /// px - pxY0, pxY1
    return linearInterpolation2D( p0[1], zP0P1, p2[1], zP2P3, y );
}

void Math::NormalizeRect(wxRect& rect)
{
    if( rect.width < 0 ) { rect.width *= -1; rect.x -= rect.width;  }

    if( rect.height < 0 ) { rect.height *= -1; rect.y -= rect.height;  }
}
