#ifndef MATH_H
#define MATH_H

#include <wx/string.h>
#include <wx/gdicmn.h>
#include "common/typedef.h"
#include "common/types/DoubleArray.h"

#define DIGITERROR 0xFF

#define RAD_FACT 0.01745329252

class Math
{
    public:
        Math();
        virtual ~Math();

        static unsigned int hex2int(const wxString & hexStr);
        static uint8_t hexDigitValue(uint8_t c);
        static int powInt(int base, int raise);
        static wxString char2hex(const uint8_t* buf, int cnt);
        static void getBytes( int i, uint8_t* c, uint8_t length = 4);
        static uint32_t getNumber(uint8_t * ptr, uint8_t length = 4);
        static double round( double d );

        static wxRealPoint CalcCircPoint(double alpha, double radius, wxRealPoint center);
        static wxRealPoint CalcCircPoint(double alpha, double radius, wxPoint center){
            return CalcCircPoint(alpha, radius, wxRealPoint(center.x,center.y));
        }
        static uint16_t crc16(uint16_t crc, uint8_t data);
        static uint16_t crc16_sipr(uint16_t crc, uint8_t data);
        static bool linearInterpolation2D(double* p0, double* p1, double* px);
        static bool linearInterpolation3D(double* p0, double* p1, double* p2, double* p3, double* px);
        static bool resample2D(const DoubleArray& oldX, const DoubleArray& oldY,
                const DoubleArray& nexX, const DoubleArray& newY, double* data);
        static double getLeftNeighbour(double x, const DoubleArray& oldA, int& oldIdx);
        static double getRightNeighbour(double x, const DoubleArray& oldA, int& oldIdx);
        static double linearInterpolation2D( double* p0, double* p1, double x );
        static double linearInterpolation2D( double x0, double y0, double x1, double y1, double x );
        static double linearInterpolation3D( double* p0, double* p1, double* p2, double* p3, double x, double y );


        static double GetPointOnLine(const wxRealPoint& A, const wxRealPoint& B, double Cy );

        static void NormalizeRect(wxRect& rect);

    protected:

    private:

};



#endif // MATH_H
