#include "RecentVemsFiles.h"
#include <wx/wx.h>

#include "wxSqliteHelper.h"
#include "DirHelper.h"
#include "logger.h"

wxArrayString RecentVemsFiles::m_lfileA;
wxArrayString RecentVemsFiles::m_cfileA;
unsigned int RecentVemsFiles::m_maxFiles = 10;
wxSQLite3Database RecentVemsFiles::m_db;
wxString RecentVemsFiles::m_dbdir = wxEmptyString;

RecentVemsFiles::RecentVemsFiles()
{
    //ctor
}

void RecentVemsFiles::setDbDir( wxString dbdir )
{
    m_dbdir = dbdir;
    if ( m_dbdir.EndsWith( DirHelper::Separ() ) )
        m_dbdir.RemoveLast();

    checkDb();
    refreshList( RECENT_ALL );
}

void RecentVemsFiles::updateFile( const wxString& filename, bool logFile )
{
    openDb();

    try {

        wxString strIsLog = ( logFile ) ? _T("log") : _T("cfg");
        time_t now = wxDateTime::GetTimeNow();

        wxString query = _T("REPLACE INTO recentfiles ( fname, ftype, flastopen ) VALUES ('") + filename + _T("', '") + strIsLog + _T("', ");
        query += wxString::Format( _T("%d)"), now );

        m_db.ExecuteUpdate( query );

        // remove "too" old entries
        query = _T("SELECT flastopen FROM recentfiles WHERE ftype ='") + strIsLog + _T("' ORDER BY flastopen DESC LIMIT 9,1");
        wxSQLite3ResultSet res = m_db.ExecuteQuery( query );
        if ( res.NextRow() )
        {
            unsigned int fopentime = (unsigned int) res.GetInt( _T("flastopen") );
            query = _T("DELETE FROM recentfiles WHERE ftype = '") + strIsLog + wxString::Format( _T("' AND flastopen < %d"), fopentime );
            m_db.ExecuteUpdate( query );
        }

        ( logFile ) ? refreshList( RECENT_LOGFILE ) : refreshList ( RECENT_CONFIGFILE );

    }
    catch ( wxSQLite3Exception e )
    {
        Logger::logf(LOG_MSG,_T("RecentVemsFiles::updateFile() - could not update database") );
    }
}

void RecentVemsFiles::deleteFile( const wxString& filename, bool logFile )
{
    openDb();

    try {

        wxString query = _T("DELETE from recentfiles WHERE fname = '") + filename + _T("'");

        m_db.ExecuteUpdate( query );

        ( logFile ) ? refreshList( RECENT_LOGFILE ) : refreshList ( RECENT_CONFIGFILE );

    }
    catch ( wxSQLite3Exception e )
    {
        Logger::logf(LOG_MSG,_T("RecentVemsFiles::deleteFile() - could not update database") );
    }
}

wxArrayString& RecentVemsFiles::getRecentFiles( bool logFile, size_t maxFiles )
{
    if ( maxFiles != m_maxFiles )
    {
        m_maxFiles = maxFiles;
        refreshList( RECENT_ALL ); // refresh both log and cfg list
    }

    if ( logFile )
        return m_lfileA;
    else
        return m_cfileA;
}

/// private methods

void RecentVemsFiles::checkDb()
{
    wxString dbName = m_dbdir + DirHelper::Separ() + _T("recentfiles.db");

    ColumnDescArray cols;

    ColumnDesc fname( _T("fname"), _T("VARCHAR(255)"), wxNullVariant, false, false, true );
    ColumnDesc ftype( _T("ftype"), _T("VARCHAR(3)") );
    ColumnDesc flastopen( _T("flastopen"), _T("INTEGER") );

    cols.Add( fname ); cols.Add( ftype ); cols.Add( flastopen );

    ColumnDescArrayHash table;
    table[_T("recentfiles")] = cols;

    wxSqliteHelper::DatabaseCheck( dbName, table );
}

void RecentVemsFiles::openDb()
{
    if ( m_db.IsOpen() )
        return;

    wxString dbName = m_dbdir + DirHelper::Separ() + _T("recentfiles.db");
    m_db.Open( dbName );
}

void RecentVemsFiles::refreshList( uint8_t fileTypes )
{
    openDb();

    wxString query;

    // select recent logfiles

    if ( fileTypes & RECENT_LOGFILE )
    {
        m_lfileA.Clear();
        try {
            query = wxString::Format( _T("SELECT * FROM recentfiles WHERE ftype='log' ORDER BY flastopen DESC LIMIT 0, %d"), m_maxFiles );
            wxSQLite3ResultSet res = m_db.ExecuteQuery( query );

            while ( res.NextRow() )
            {
                wxString s = res.GetString( _T("fname") );
                m_lfileA.Add( s );
            }
        }
        catch ( wxSQLite3Exception e )
        {
            Logger::logf(LOG_MSG,_T("RecentVemsFiles::refreshList() - could not refresh list of recent log files"));
        }
    }

    // select recent cfg files
    if ( fileTypes & RECENT_CONFIGFILE )
    {
        m_cfileA.Clear();
        try {
            query = wxString::Format( _T("SELECT * FROM recentfiles WHERE ftype='cfg' ORDER BY flastopen DESC LIMIT 0, %d"), m_maxFiles );
            wxSQLite3ResultSet res = m_db.ExecuteQuery( query );

            while ( res.NextRow() )
            {
                wxString s = res.GetString( _T("fname") );
                m_cfileA.Add( s );
            }
        }
        catch ( wxSQLite3Exception e )
        {
            Logger::logf(LOG_MSG,_T("RecentVemsFiles::refreshList() - could not refresh list of recent config files"));
        }
    }
}
