#ifndef SIMPLEFWDLG_H
#define SIMPLEFWDLG_H

#include <wx/filename.h>
#include <wx/gbsizer.h>
#include <wx/statline.h>

#include "firmwareconfigitem.h"
#include "FirmwareInfo.h"
#include "common/types/wxString2wxVariantHash.h"
#include "common/typedef.h"

class Project;

class FwPath;
WX_DEFINE_ARRAY(FwPath*, FwPathArray);
WX_DECLARE_STRING_HASH_MAP( FwPathArray , FwPathHash  );

enum RestrictState{
   RS_CELSIUS = 1,
   RS_FAHRENHEIT = 2,
   RS_NONE = 3
};

WX_DECLARE_STRING_HASH_MAP( FirmwareVersion , FirmwareVersionHash  );

class FwDesc
{
    public:
        static void Init();
        static void cleanup();

        static wxString2wxVariantHash FwDescriptorHash;

};

class FwPath
{
    public:

        bool ParseDir(const wxString &tmp, FwPathHash * hash = NULL);


        bool operator==( const FwPath& pd ) const;
        bool operator!=( const FwPath& pd ) const;


        const FirmwareVersion& GetVer() const { return Ver;};
        const wxString& GetTitle() const { return Title;};
        const wxString& GetPath() const { return Path;};
        const wxString& GetPath16() const { return Path16;};
        char GetTable() const { return Table;};
        const wxString& GetFilename() const { return Filename;};
        const wxString& GetDeviceStr() const { return DeviceStr;};

        uint8_t GetCurrType() const { return CurrType; };

        void SetCurrType(uint8_t type) { CurrType = type; };

        int Title_Compare_function_Desc(FwPath **A,FwPath **B);

    private:

        FirmwareVersion Ver;
        wxString        Title;
        wxString        Path;
        wxString        Path16;
        char            Table; /// -1 = no; 0 = 12x12; 1 = 16x14
        wxString        Filename;
        wxString        DeviceStr;

        uint8_t         CurrType;  ///  0: vems.hex in the fw root dir. 1: vems.hex in the fw my_make.16x14 dir

};

class PatchDesc
{
    public:
        PatchDesc(){
            //isPTC = false;
            isCelsius = false;
            m_valid = false;
        };

        PatchDesc(const wxString &tempstr, const wxString &coec, const wxString &aref, const wxString &rref){

            RRef = rref;
            ARef = aref;
            Coec = coec;

            isCelsius = false;

            if(tempstr == _T("C°"))
                isCelsius = true;

            m_valid = true;
        };

        bool ParseFile( wxFileName fn, const wxString &dir );
        bool ParseFileNameStr( const wxString &filename );

        bool operator==( const PatchDesc& pd ) const;
        bool operator!=( const PatchDesc& pd ) const;

        const wxString&  GetCoec() const { return Coec; }
        const wxString&  GetRRef() const { return RRef; }
        const wxString&  GetARef() const { return ARef; }
        wxString GetTempStr() const {
            if(isCelsius)
                return _T("C°");
            else
                return _T("F°");
        }
        bool  GetTemp() { return isCelsius; }

        bool IsValid() const { return m_valid; }

        const wxString& GetPath() const { return Path; }

        wxString ToString() const {
            return wxString::Format(_T("isCelsius=%i"),isCelsius) + _T(", RRef=") +
                            RRef + _T(", ARef=") + ARef + _T(", Coec=") + Coec + _T(", Path=") + Path;
        };

    private:

        //bool isPTC;
        bool isCelsius;

        wxString RRef;
        wxString ARef;
        wxString Coec;

        wxString Ext;

        wxString Path;

        bool    m_valid;

};

WX_DEFINE_ARRAY(PatchDesc*, PatchDescArray);



class PatchUiDesc: public wxEvtHandler
{
    public:

        PatchUiDesc();

        void Init(wxWindow * parent, wxGridBagSizer * sizer, int row);

        enum {
            idCoec = 2000,
            idRRef,
            idARef,
            idTemp
        };

        void EnableCB(bool enable);
        bool isEnabled();
        void InitTemp();
        bool SetValues( const PatchDesc &pd );

        void SetRestrictStateC( RestrictState state );

        PatchDesc * GetPatch();


        PatchDescArray * PatchArr;

        wxComboBox * CoecCB;
        wxComboBox * RRefCB;
        wxComboBox * ARefCB;
        wxComboBox * TempCB;

    protected:

        bool m_isActive;

        void CheckComboCnt(wxComboBox * cb);

        void FillCombos( int id );

        bool AddPatchDetail(const wxString &det, wxComboBox *cb);
        void OnComboBox( wxCommandEvent &evt );


};


WX_DECLARE_HASH_MAP( int, PatchUiDesc*, wxIntegerHash, wxIntegerEqual, Int2PatchUiDescHash);

class PatchManagger: public wxEvtHandler
{
    public:
        PatchManagger();

        void Init(wxWindow * parent, wxGridBagSizer * sizer, PatchDescArray * patcharr, Project * prj = NULL);

        PatchDesc * GetPatch();

        wxString PatchExt;

        void SetRestrictState( RestrictState state );


    protected:

        int AddLine(wxWindow * parent, wxGridBagSizer * sizer, PatchDescArray * patcharr, const PatchDesc &pd );

        PatchDesc& GetLastUsedPatch();

    private:

        void OnRadioButton( wxCommandEvent &evt );

        PatchDescArray * PatchArr;

        Int2PatchUiDescHash     m_patchLines;


};

class FirmwareInfo;

class SimpleFwDlg: public wxDialog
{
    public:
        SimpleFwDlg(wxWindow * parent,int type);
        SimpleFwDlg(wxWindow * parent,const wxString &title);
        virtual ~SimpleFwDlg();

        void Init(Project * prj = NULL);
        void Init( const FirmwareInfo *fi);

        void getFirmwareConfigItemArray(FirmwareConfigItemArray &arr);

        FwPath * GetFwPath(wxString devstr = wxEmptyString, wxString fwverstr = wxEmptyString);

    protected:

        void FillDlg(Project * prj = NULL);
        void FillVerCombo(const wxString &devstr, bool isFillTable = true);
        void FillTableCombo(FwPath * fw);


        void ReadFwDir();
        void ReadPatchDir(const wxString &dir, const wxString &prefix, const wxString &ext, PatchDescArray &arr);

        void CreatePatchSection(Project * prj, wxBoxSizer * sizer);

        void InitFwPanel(Project * prj);

    private:
        void saveCfg(const wxString& key, const wxString& value);
        void OnComboBox(wxCommandEvent &evt);

        void OnEnable( wxCommandEvent &evt );

        void OnButton(wxCommandEvent& evt);

        void OnFwWebToolClose(wxCloseEvent& evt);

        enum {
            idDevList = 1000,
            idVerList,
            idTableList,
            idOpenWebToolBtn
        };

        wxWindow        *   m_parent;

        wxComboBox      *   m_devCombo;
        wxComboBox      *   m_verCombo;
        wxComboBox      *   m_tableCombo;

        FwPathHash          m_fwList;


        PatchDescArray      m_airList;
        PatchDescArray      m_cltList;

        wxGridBagSizer  *   m_patchSizer;

        PatchManagger   *   m_airPatchMgr;
        PatchManagger   *   m_cltPatchMgr;

        FirmwareVersionHash m_fwVerLimitations;

        wxBoxSizer      *   m_mainSizer;

        wxStaticLine    *   m_firstStatLine;

        DECLARE_EVENT_TABLE();

};

#endif // SIMPLEFWDLG_H
