#ifndef SYMBOLMGR_H
#define SYMBOLMGR_H

#include "Symbol.h"
#include "dag/Graph.h"
#include "SymbolMgrBase.h"

class StringHashSet;

/**     ----- |     RT                       |
        COST   RT1  GPS AIM  SIPR1   SIPR2
SCALAR  1       8  32  128
ARRAY   2      XX  XX  XX
BIT     4      XX  XX  XX
---------------------------------------
EXP     XXX    16  16  16
*/

WX_DECLARE_HASH_MAP( int, SymbolTable*, wxIntegerHash, wxIntegerEqual, SymbolTables);
WX_DECLARE_HASH_MAP( int, SymbolArray*, wxIntegerHash, wxIntegerEqual, Int2SymbolArrayHash);

class SymbolMgr: public SymbolMgrBase
{
    public:
        SymbolMgr();
        virtual ~SymbolMgr();

        virtual bool isExistSymbol(const wxString& symbolName);

        const SymbolTable& getBitTable(){
            return m_bitTable;
        }

        const SymbolTable& getArrayTable(){
            return m_arrayTable;
        }

        const SymbolTable* getScalarTable(int type);

        const SymbolTable& getExpTable(){
            return m_expTable;
        }

        void        addSymbol(Symbol* symbol);
        void        addOrReplaceSymbol( Symbol* symbol );
        Symbol*     getSymbol(const wxString& symbolName);
        int         getSymbolPage(const wxString& symbolName);
        wxString    getSymbolType(const wxString& symbolName);
        Graph& getDAG() { return m_dag; }

        void    clear();

        void addGroup( const wxString& name );
        void setGroupOffset( const wxString& name, int offset );
        SymbolGroup* getGroup( const wxString& groupName );
        SymbolTable& getSymbolTable() { return m_allSymbolTable; }
        const Int2SymbolArrayHash& getTheSymbolOrder();
        static int compareByPageAndOffset(Symbol **A,Symbol **B);

        bool isConst(const wxString& symbolName);
        SymbolTable& getUsedStaticAndBit();
        void setUsedStatic(StringHashSet& symbolNameSet);
        SymbolTable& getConfigTableSymbols();
        SymbolArray& getConfigTableSymbolsInOrder();
        void addConfigTableSymbolsAsScalar(SymbolTable& symbolTable);

        SymbolTable&    getCfgSymbolTable() { return m_cfgSymbolTable; }

    protected:
    private:
        SymbolTable     m_bitTable;
        SymbolTable     m_arrayTable;
        SymbolTable     m_expTable;

        SymbolTable     m_allSymbolTable;   //all symbols
        SymbolTables    m_scalarTables;      //scalar symbolTables
        SymbolGroupHash         m_symbolGroups;

        Graph           m_dag;       //m_expTable dependency graph
        Int2SymbolArrayHash m_page2SymbolArrayHash;
        SymbolTable     m_usedStatic; // ST scalar and bit symbols;
        SymbolTable     m_configTableSymbolH;   /// calculated!!
        SymbolArray     m_configTableSymbolsInOrder;

        SymbolTable     m_cfgSymbolTable;

        friend class MTConfigData;

    private:
        //bool    isConfigArraySymbol( const wxString& name ) const;
        void    addConfigArraySymbolInOrder( const wxString& name, Symbol* symbol );

};

#endif // SYMBOLMGR_H
