#ifndef USERLOG_H
#define USERLOG_H
#include <wx/wx.h>

class UserLogWindow;
class VemsMessage;
class ObserverBase;


enum UserLogType
{
    ULOG_INFO,
    ULOG_WARNING,
    ULOG_ERROR,
    ULOG_STOP,
    ULOG_OK,
    ULOG_UP,
    ULOG_DOWN

};

enum UserLogLineSize
{
    ULOGLS_SMALL,
    ULOGLS_MEDIUM,
    ULOGLS_LARGE,
    ULOGLS_XLARGE,
    ULOGLS_XXLARGE,
    ULOGLS_PREDEFINED
};

enum ShowPolicy
{
    SPC_NORMAL,
    SPC_NO_DUPLICATE_LINES

};

class MessageContainer
{
    public:
        MessageContainer( VemsMessage * vmsg, UserLogLineSize usize, ShowPolicy spol){
            msg = vmsg;
            size = usize;
            pol = spol;
        }

        VemsMessage *   msg;
        UserLogLineSize size;
        ShowPolicy      pol;
};

WX_DEFINE_ARRAY( MessageContainer*,  MessageContainerArray );

class UserLog
{
    public:
        UserLog();
        virtual ~UserLog();

        static void Init( UserLogWindow * lw, UserLogLineSize defsize = ULOGLS_MEDIUM);
        static void uLog(UserLogType type, const wxString& str, UserLogLineSize size = ULOGLS_PREDEFINED, ShowPolicy pol = SPC_NORMAL);
        static void rLog(const wxString& rawstr);
        static void vLog(const wxString &vmsgname, UserLogLineSize size = ULOGLS_PREDEFINED, ShowPolicy pol = SPC_NORMAL);
        static void vLog(const VemsMessage& vmsg, UserLogLineSize size = ULOGLS_PREDEFINED, ShowPolicy pol = SPC_NORMAL);

        static void ClearLogWindow();

        static ObserverBase* GetObserver();

    protected:
    private:

        static UserLogWindow * m_lw;

        static UserLogLineSize m_ulDefSize;

        static MessageContainerArray m_messageQueue;

};

#endif // USERLOG_H
