#include "VemsNumPad.h"

#include "logger.h"

#define STR_POINT   _T(".")
#define STR_BACK    _T("<-")
#define STR_CLEAR   _T("Clr")
#define STR_MINUS   _T("+/-")

VemsNumPad::VemsNumPad(wxWindow *parent, long numStyle, wxWindowID id, const wxString &title, wxPoint pos, wxSize size, long style)
        : m_titleText (NULL)
        , m_valueCaption (NULL)
        , m_value (NULL)
{
    //ctor

    Create(parent, id, title, pos, size, style);

    m_style = numStyle;

    m_mainsizer = new wxBoxSizer(wxVERTICAL);

    CreateControls();

    CreateNumButtons();


    wxSizer *buttonsSizer = CreateButtonSizer( wxCANCEL | wxOK );
    m_mainsizer->Add(buttonsSizer ,0, wxLEFT | wxRIGHT | wxALIGN_CENTER_HORIZONTAL,5 );


	SetSizer(m_mainsizer);
	m_mainsizer->Fit(this);
	m_mainsizer->SetSizeHints(this);

}

VemsNumPad::~VemsNumPad()
{
    //dtor
}

void VemsNumPad::CreateControls()
{

    if( m_style & VNP_TITLE ) {
        m_titleText = new wxStaticText(this, wxID_ANY, _T("NumPad"));
        m_mainsizer->Add(m_titleText, 0, wxLEFT | wxRIGHT | wxALIGN_CENTER_HORIZONTAL, 5);
    }

    wxBoxSizer *captionSizer = new wxBoxSizer(wxHORIZONTAL);

    if( m_style & VNP_CAPTION ) {
        m_valueCaption = new wxStaticText(this, wxID_ANY, _T("Value:") );
        //wxDefaultPosition, wxSize(150,16), wxST_NO_AUTORESIZE);
        //m_valueCaption->Wrap(20);

        captionSizer->Add(m_valueCaption, 0, wxLEFT | wxRIGHT | wxALIGN_CENTRE_VERTICAL , 5);
    }

    m_value = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(40,18) );
    captionSizer->Add(m_value , 1, wxLEFT | wxRIGHT | wxALIGN_CENTRE_VERTICAL , 5);
    captionSizer->SetItemMinSize(m_value , 50, 18);
    m_mainsizer->Add(captionSizer, 0, wxEXPAND);


}

void VemsNumPad::CreateNumButtons()
{
    wxArrayString tmp;
    tmp.Add(_T("7"));
    tmp.Add(_T("8"));
    tmp.Add(_T("9"));

    tmp.Add(_T("4"));
    tmp.Add(_T("5"));
    tmp.Add(_T("6"));

    tmp.Add(_T("1"));
    tmp.Add(_T("2"));
    tmp.Add(_T("3"));

    tmp.Add(_T("0"));


    wxArrayString extButts;
    extButts.Add(STR_POINT);
    extButts.Add(STR_BACK);
    extButts.Add(STR_CLEAR);
    extButts.Add(STR_MINUS);

    wxFont font;
    wxButton *butt;


    /// create normal(0-9) buttons

    for(uint8_t y = 0; y < 3; y++) {
        wxBoxSizer *y_buttonSizer = new wxBoxSizer(wxHORIZONTAL);
        for(uint8_t x = 0; x < 3; x++) {
            int id = wxNewId();
            butt = new wxButton(this, id, tmp[y*3 + x], wxDefaultPosition , wxSize(70,60) );
            m_buttonHash[id] = butt;
            font = butt->GetFont();
            font.SetPointSize(30);
            butt->SetFont(font);
            Connect(id,wxEVT_COMMAND_BUTTON_CLICKED,wxCommandEventHandler(VemsNumPad::OnButton));
            y_buttonSizer->Add(butt, 1, 0 , 3);
        }
        m_mainsizer->Add(y_buttonSizer,1, 0 | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    }

    wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);

    int id = wxNewId();
    butt = new wxButton(this, id, tmp[9], wxDefaultPosition , wxSize(70,60) );
    m_buttonHash[id] = butt;
    buttonSizer->Add(butt, 0, 0 | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 3);
    Connect(id,wxEVT_COMMAND_BUTTON_CLICKED,wxCommandEventHandler(VemsNumPad::OnButton));
    font = butt->GetFont();
    font.SetPointSize(30);
    butt->SetFont(font);


    /// create extended buttons

    wxFlexGridSizer *extButtonSizer = new wxFlexGridSizer(2,2,0,0);
    for(uint8_t a = 0; a < extButts.GetCount(); a++) {
        int id = wxNewId();
        m_buttonHash[id] = new wxButton(this, id, extButts[a], wxDefaultPosition, wxSize(70,30));
        extButtonSizer->Add(m_buttonHash[id], 1, 0 | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 3);
        Connect(id,wxEVT_COMMAND_BUTTON_CLICKED,wxCommandEventHandler(VemsNumPad::OnButton));
    }
    buttonSizer->Add(extButtonSizer,1,wxEXPAND|wxALL,0);
    m_mainsizer->Add(buttonSizer,1, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL);



}

void VemsNumPad::OnButton(wxCommandEvent &evt)
{

    wxString str = m_buttonHash[evt.GetId()]->GetLabel();
//    Logger::log(LOG_MSG, str );

    if(str == STR_BACK)
        m_value->SetValue( m_value->GetValue().RemoveLast() );

    else if(str == STR_CLEAR)
        m_value->SetValue(_T(""));

    else if(str == STR_POINT) {
        if( m_value->GetValue().Find(STR_POINT) == wxNOT_FOUND )
            m_value->SetValue(m_value->GetValue()+str);
    }

    else if(str == STR_MINUS) {
        if(m_value->GetValue().StartsWith(_T("-")))
            m_value->SetValue( m_value->GetValue().Mid(1) );
        else
            m_value->SetValue( _T("-") + m_value->GetValue());
    }

    else
        m_value->SetValue(m_value->GetValue()+str);

}

wxString VemsNumPad::GetValueAsString()
{
    return m_value->GetValue();
}

double VemsNumPad::GetValueAsDouble()
{
    double tmp = 0;
    m_value->GetValue().ToDouble(&tmp);
    return tmp;
}

void VemsNumPad::SetValue(double value, const wxString &format)
{
    m_value->SetValue(wxString::Format(format, value));

    m_value->SetSelection(-1,-1);
}

void VemsNumPad::SetValue(const wxString &value)
{
    m_value->SetValue(value);

    m_value->SetSelection(-1,-1);
}

/** @brief SetValueCaption
  *
  * @todo: document this function
  */
void VemsNumPad::SetValueCaption(const wxString &value)
{
    if( m_style & VNP_CAPTION ) {
        m_valueCaption->SetLabel(value);

    }
}

/** @brief SetTitle
  *
  * @todo: document this function
  */
void VemsNumPad::SetTitle(const wxString &value)
{
    if( m_style & VNP_TITLE )
        m_titleText->SetLabel(value);
}

