#ifndef VEMSTUNECONTROLLER_H
#define VEMSTUNECONTROLLER_H


#include <wx/event.h>


#include "rtdatamanager/RTDataInputBase.h"
#include "stdatamanager/STDataViewerBase.h"
#include "FirmwareInfo.h"
#include "vemsprotocol/DeviceInfo.h"
#include "common/types/IntToIntSetHash.h"
#include "common/types/IntArray.h"
#include "loghistory/FilePlayCommander.h"
#include "VemsTuneControllerTypes.h"

class MTConfigData;
class RTDataMgrFacade;
class RTDataAnalyzer;
class STDataMgrFacade;
class VemsLogReader;
class SIPRFileLogger;
class GPSCommunicationController;
class VemsCommand;
class UserDialogBase;
class ThreadLogPlayer;
class Observable;
class LogHistoryMgr;
class LogHistoryBase;
class QuickAnalyze;
class InputEvent;
class VemsDeviceController;
class VemsDeviceControllerEvent;
class CommunicationEvent;


class VemsTuneController : public wxEvtHandler
{
    public:
        VemsTuneController( wxEvtHandler* pEventHandler = NULL );
        virtual ~VemsTuneController();

        void init();
        void startThreads();

        void selectInputFile(const wxString& fileName);
        void selectInputECU();

        FilePlayCommander*  getFilePlayCommander();

        VemsDeviceController*   getECUController();
        DeviceInfo              getDeviceInfo() const;


        void setECUOnline( bool state,  bool isBlocking = false );
        bool isECUOnline();
        void setGPSOnline( bool state, bool reloadCfg = false );
        bool isGPSOnline();
        int getGPSState();

//        void setFilePlay(bool b);
//        bool getFilePlay();
        bool isStaticInputConfigFile() const { return m_stInputType == ST_INPUT_FILE; }

        void setECULog(bool b, bool saveSDCard = false );
        bool isECULog();
        int getDevice();
        int getSelectedInput();

        wxString    getVemsLogFile() const { return m_vemsLogFile; }
        int         getVemsLogFileVersion() const { return m_vemsLogFileVersion; }

        bool saveMSQ(wxString& fileName);
        VemsLogReader* getVemsLogReader() { return m_vemsLogReader; };
        DoubleHash* getRTViewerModel() { return &m_rtViewerModel; }
        wxString getFirmwareInfoStr();

        FirmwareInfo* getFirmwareInfo();
        bool is12x12() { return m_fwInfo.is12x12(); }

        void        request( VemsCommand* command );
        void        setLogSaveSDCard( bool enable );
        wxString    fillLogHeader( bool isSDSave = false );
        void        registerSTConst();
        void        selectInputVemsCfg( const wxString& filename );



        void setUserDialog(UserDialogBase* u){
            m_userDialog = u;
        }

        UserDialogBase* getUserDialog(){
            return m_userDialog;
        }

        wxString getRealtimeSourceAsString();
        wxString getStaticSourceAsString();

        LogHistoryBase* getLogHistory();
        Observable*     getLogHistoryObs();
        LogHistoryMgr*  getLogHistoryMgr();
        RTDataMgrBase*  getRTDataMgr();
        STDataMgrBase*  getSTDataMgr();
        VTCOfflineTypes getOfflineState() const { return m_offlineState; }

        VemsDeviceController* getDeviceContoller() { return m_deviceController; }

        bool isOnline() const;
        void setOffline();
        void setOnline();

        bool isDeviceSameAsPrev() const;
        bool isConfigSameAsPrev() const;
        void writeConfigDifferences();
        void enableRealtime();

    private:
        enum StaticInputType { ST_INPUT_FILE, ST_INPUT_SIPR };
        RTDataMgrFacade*    m_rtDataMgr;

        STDataMgrFacade*    m_stController;

        QuickAnalyze*       m_loganalyzer;

        DoubleHash                  m_rtViewerModel, m_rtViewerModel2;
        STDataViewerBaseA           m_stScalarViewerA;
        MTConfigData*               m_mtConfig;
        VemsLogReader*              m_vemsLogReader;
        FirmwareInfo                m_fwInfo;
        FirmwareInfo                m_fwInfoPrev;
        GPSCommunicationController* m_pCommunicationControllerGPS;
        VemsDeviceController*       m_deviceController;
        //ThreadLogPlayer*            m_pVemslogCtrl;
        SIPRFileLogger*             m_siprFileLogger;
        wxString                    m_vemsLogFile;
        int                         m_vemsLogFileVersion;
        wxEvtHandler*               m_pEventHandler;
        int                         m_selectedInput;
        int                         m_stInputType;
        wxString                    m_stInputFileName;
        wxString                    m_vemslogFilename;
        UserDialogBase*             m_userDialog;
        LogHistoryMgr*              m_logHistory;
        RTDataAnalyzer*             m_rtanalyzer;
        FilePlayCommander*          m_fpc;
        VTCOfflineTypes             m_offlineState;

        IntToIntSetHash             m_stModelsDiff;

    private:
        void initCommunicationModule();
        void initRTModule();
        void initSTModule();
        void initVemsLogModule();
        void initGPSModule();
        void sendInputSourceEvent(RTDataInputType type);
        void checkSignature();
        void logAllPages();
        void sendDetectedPagesToLogHistory();

        void OnSIPRInput( InputEvent& event );

        void OnCannotOpenPort( VemsDeviceControllerEvent& event );
        void OnDeviceFound( VemsDeviceControllerEvent& event );
        void OnDeviceNotFound( VemsDeviceControllerEvent& event );
        void OnDeviceConnected( VemsDeviceControllerEvent& event );
        void OnDeviceDisconnected( VemsDeviceControllerEvent& event );
        void OnDeviceReconnected( VemsDeviceControllerEvent& event );
        void OnDeviceControllerStopped( VemsDeviceControllerEvent& event );
        void OnDeviceResumed( VemsDeviceControllerEvent& event );
        void OnDeviceProtected( VemsDeviceControllerEvent& event );

        void OnCommunicationError( CommunicationEvent& event );

        DECLARE_EVENT_TABLE()
};


#endif // VEMSTUNECONTROLLER_H
