#ifndef WXAGG_H
#define WXAGG_H

#include <wx/bitmap.h>
#include <wx/dc.h>
#include "../common/typedef.h"
#include "../common/types/RealPointArray.h"
#include "../wxRealRect.h"

#include "agg_pixfmt_rgb.h"
#include "agg_pixfmt_gray.h"
#include "agg_rendering_buffer.h"
#include "agg_renderer_base.h"
#include "agg_renderer_scanline.h"
#include "agg_rasterizer_scanline_aa.h"
#include "agg_scanline_u.h"
#include "agg_trans_affine.h"
#include "agg_span_gradient.h"
#include "agg_span_interpolator_linear.h"
#include "agg_span_interpolator_trans.h"
#include "agg_span_allocator.h"
#include "agg_math_stroke.h"
#include "agg_font_freetype.h"
#include "agg_conv_contour.h"

typedef agg::pixfmt_rgb24                                       pixfmt;
typedef agg::renderer_base< pixfmt >                            renderer_base_type;
typedef agg::renderer_scanline_aa_solid< renderer_base_type >   renderer_scanline_type;
typedef agg::rasterizer_scanline_aa<>                           rasterizer_type;
typedef agg::scanline_u8                                        scanline_type;
typedef agg::pod_auto_array< agg::rgba8, 256 >                  color_array_type;
typedef agg::gradient_x                                         gradient_func_type;
typedef agg::span_interpolator_linear<>                         interpolator_type;
typedef agg::span_allocator<agg::rgba8>                         span_allocator_type;

typedef agg::span_gradient<agg::rgba8,
                           interpolator_type,
                           gradient_func_type,
                           color_array_type>                    span_gradient_type;

typedef agg::renderer_scanline_aa<renderer_base_type,
                                  span_allocator_type,
                                  span_gradient_type>           renderer_gradient_type;
//alpha channel
typedef agg::renderer_base< agg::pixfmt_gray8 >                 ren_base_alpha;
typedef agg::renderer_scanline_aa_solid< ren_base_alpha >       renderer_alpha;

//WX_DEFINE_ARRAY( wxRealPoint*, RealPointArray );

class BitmapCanvasWidgetBase;

class wxAgg
{
    public:
        //wxAgg( wxDC& dc, uint16_t width, uint16_t height );
        wxAgg( uint32_t width, uint32_t height );
        wxAgg( const wxSize& size );
        wxAgg( wxWindow* win );
        wxAgg( BitmapCanvasWidgetBase* bcWidget );
        virtual ~wxAgg();

        void Create( uint32_t width, uint32_t height );

        //test void draw();
        void setPen( const wxColour& c );
        void setBrush( const wxColour& c );
        void clear( const wxColor& c );
        void setClippingRect( wxRect rect ) { m_clippingRect = rect; }
        void clearClipping() { m_clippingRect = wxDefaultRect; }

        void drawLine( double x1, double y1, double x2, double y2, double width = 1.0, agg::line_cap_e cap = agg::butt_cap );

        void drawDashedLine( double x1, double y1, double x2, double y2, double width = 1.0, int patternN = 0, double* pattern = NULL,agg::line_cap_e cap = agg::butt_cap );
        void drawDashedLine( double x1, double y1, double x2, double y2, wxPen& pen );

        void drawLine( wxRealPoint c1, wxRealPoint c2, double width = 1.0, agg::line_cap_e cap = agg::butt_cap) { drawLine( c1.x, c1.y, c2.x, c2.y, width, cap); }
        void drawLine( RealPointArray coords, double width = 1.0, agg::line_cap_e cap = agg::butt_cap);

        void drawEllipse( double x, double y, double r1, double r2, double width = 1.0 );
        void drawFilledEllipse( double x, double y, double r1, double r2, double startangle, double deltaangle );

        void drawCircle( double x, double y, double r, double width = 1.0 );
        void drawCircle( wxRealPoint c, double r, double width = 1.0 ) { drawCircle( c.x, c.y, r, width ); };
        void drawFilledCircle( double x, double y, double r );
        void drawFilledCircle( wxRealPoint c, double r ) { drawFilledCircle( c.x, c.y, r ); };
        void drawGradientFilledCircle( const wxColour& bc, const wxColour& ec, double deg, double cx, double cy, double r );

        void drawArc( double centerx, double centery, double r, double startAngle, double deltaAngle, double w = 1.0 );
        void drawArc( wxRealPoint center, double radius, double startAngle, double deltaAngle, double w = 1.0 ){ drawArc(center.x,center.y,radius,startAngle,deltaAngle,w); }
        void drawArc( wxPoint center, double radius, double startAngle, double deltaAngle, double w = 1.0 ){ drawArc(center.x,center.y,radius,startAngle,deltaAngle,w); }
        void drawFilledArc( double x, double y, double r, double a1, double a2 );
        void drawFilledCircleSection( double x, double y, double r, double startAngle, double deltaAngle );

        void drawRectangle( double x, double y, double w, double h, double s );
        void drawSolidRectangle( double x, double y, double w, double h );
        void drawFilledRectangle( double x, double y, double w, double h, double s );
        //void drawRectangle( point, size );
        void drawRotatedRectangle( double x, double y, double w, double h, double s, double d );

        void drawRoundedRectangle( double x, double y, double w, double h, double r, double s=1.0 );
        void drawFilledRoundedRectangle( double x, double y, double w, double h, double r, double s=1.0 );
        void drawFilledRoundedRectangle( const wxRealRect& rect, double r, double s=1.0 ) {
            drawFilledRoundedRectangle(rect.x,rect.y,rect.w,rect.h,r,s);
        };

        void drawPolygon( const RealPointArray& points );
        void drawSolidPolygon( const RealPointArray& points, double s );

        //void drawText( const wxString& text, double x, double y );
        //void getTextExtent( const wxString& text );
        void drawText( const wxString& text, double x, double y, double w, double h, double s );
        static double getTextWidth( const wxString& text, double h );


        void render( wxDC& dc );
        wxBitmap* renderToBitmap();
        wxImage* renderToImage();
        wxImage* renderCopyToImage(const wxRect& renderRect = wxDefaultRect);
        static wxColor gradient( const wxColor& bc, const wxColor& ec, double k );

        bool loadTTFont( const wxString& fontfilename, double height, double weight, double width = 0 );
        void setFontHeight( double h );
        double getFontHeight();
        wxRealRect drawTTFText( double x, double y, const wxString& text, double angleDeg = 0 );
        wxRealRect drawTTFText( double x, double y, const wxString& text, double height, double weight, const wxString& fontfilename, double angleDeg = 0, double width = -1 );
        wxRealRect getTTFTextBound(const wxString& text, double angleDeg = 0, double x = 0, double y = 0);

        void drawInnerRectShadow(int width, int alpha, const wxRect& rect, const wxColor& color = wxColor(0,0,0));
        void drawInnerRectShadow(int width, int alpha, const wxRealRect& rect, const wxColor& color = wxColor(0,0,0));

        void aggTest1();
        void aggTest2();

    protected:
    private:
        typedef agg::font_engine_freetype_int32 font_engine_type;
        typedef agg::font_cache_manager<font_engine_type> font_manager_type;
        // Pipeline to process the vectors glyph paths (curves + contour)

        wxRect              m_clippingRect;

        font_engine_type    m_font_engine;
        double              m_font_weight;

        //wxDC&               m_dc;
        wxBitmap*           m_bmp;
        int                 m_width;
        int                 m_height;
        unsigned char*      m_buffer;

        pixfmt*                 m_pixf;
        agg::rendering_buffer*  m_rb;
        renderer_base_type*     m_ren_base;
        renderer_scanline_type* m_ren_sl;
        scanline_type           m_sl;
        rasterizer_type         m_ras;

        //alpha channel
        unsigned char*          m_alphaBuffer;
        agg::rendering_buffer   m_alpha_mask_rbuf;
        agg::pixfmt_gray8*      m_pixfg;
        ren_base_alpha*         m_rba;
        renderer_alpha*         m_rena;

        agg::rgba               m_penColor;
        agg::rgba               m_brushColor;

        static agg::rgba   convertToRGBA( const wxColour& c );
        static wxColor     convertToColor( const agg::rgba& c );
        void               renderAlphaMask( const double a );

        void        fillColorArray( color_array_type& array, agg::rgba8 begin, agg::rgba8 end );
        void        applyClipping();
};

#endif // WXAGG_H
