#ifndef WXAGGBRUSH_H
#define WXAGGBRUSH_H

#include <wx/wx.h>
#include "agg_color_rgba.h"

#include "aggHelper.h"

enum wxAggBrushType {
    wxAGGBRUSH_SOLID,
    wxAGGBRUSH_TRANSPARENT,
    wxAGGBRUSH_BDIAGONAL_HATCH,
    wxAGGBRUSH_CROSSDIAG_HATCH,
    wxAGGBRUSH_FDIAGONAL_HATCH,
    wxAGGBRUSH_CROSS_HATCH,
    wxAGGBRUSH_HORIZONTAL_HATCH,
    wxAGGBRUSH_VERTICAL_HATCH,
    wxAGGBRUSH_GRADIENT,
    wxAGGBRUSH_BITMAP
};

class wxAggBrush : public wxAggObject
{
    public:
        wxAggBrush();
        wxAggBrush( const wxColour& color, int style = wxAGGBRUSH_SOLID );
        wxAggBrush( const wxBrush& brush );
        wxAggBrush( const wxAggBrush& brush );
        wxAggBrush( const wxBitmap& bitmap );

        ~wxAggBrush();

        bool        IsOk() const;

        wxColour    GetWxColor() const;
        agg::rgba   GetAggColor() const;
        agg::rgba   GetAggEndColor() const;
        int         GetStyle() const;
        wxBitmap&   GetBitmap();
        double      GetGradientOrientation() const;

        void        SetColor( const wxColour& color );
        void        SetColor( const agg::rgba& color );

        void        SetEndColor( const wxColour& color );
        void        SetEndColor( const agg::rgba& color );
        void        SetGradientOrientation( const double& degree );

        void        SetBitmap( const wxBitmap& bitmap );
        void        SetStyle( int style );

        void        SetBrush( const wxBrush& brush );
        void        SetBrush( const wxAggBrush& brush );

        wxAggBrush operator = ( const wxAggBrush& brush );



    protected:

    private:
        bool        m_isOk;
        agg::rgba   m_color;
        agg::rgba   m_endcolor;
        int         m_style;

        double      m_graddegree;

        wxBitmap    m_bitmap;

        DECLARE_DYNAMIC_CLASS( wxAggBrush )
};

extern const wxAggBrush wxDefaultAggBrush;
extern const wxAggBrush wxTransparentAggBrush;

#endif // WXAGGBRUSH_H
