#include "wxAggFont.h"

#include "../UnicodeHelper.h"

IMPLEMENT_DYNAMIC_CLASS( wxAggFont, wxAggObject )

wxAggFont::wxAggFont()
{
    m_fontOk = false;
}

wxAggFont::wxAggFont( const wxAggFont& font )
{
    loadFont( font.m_fontfilename, font.m_font_height, font.m_font_weight, font.m_font_width );
}

wxAggFont& wxAggFont::operator = ( const wxAggFont& font )
{
    if ( &font == this )
        return *this;

    m_fontfilename = font.m_fontfilename;
    m_font_height = font.m_font_height;
    m_font_weight = font.m_font_weight;
    m_font_width = font.m_font_width;

    reloadFont();
    return *this;
}

bool wxAggFont::loadFont( const wxString& fontFileName, double height, double weight, double width )
{
    const char* fname = UnicodeHelper::U2C( fontFileName );

    agg::glyph_rendering gren = agg::glyph_ren_outline;

    m_font_engine.gamma( agg::gamma_power( 1.0 ) );

    bool ret = false;

    if ( width == 0 )
    {
        width = height;
    }

    if ( m_font_engine.load_font( fname, 0, gren ) )
    {
        m_font_engine.hinting( true );
        m_font_engine.height( height );
        m_font_engine.width( width );
        m_font_engine.flip_y( true );
        m_font_engine.char_map( FT_ENCODING_UNICODE );

        m_font_weight = weight;

        m_font_height = height;
        m_font_width = width;
        m_fontfilename = fontFileName;

        ret = true;
        m_fontOk = true;
    }

    return ret;
}

bool wxAggFont::reloadFont()
{
    return loadFont( m_fontfilename, m_font_height, m_font_weight, m_font_width );
}

void wxAggFont::setFont( const wxAggFont& font )
{
    m_fontfilename = font.m_fontfilename;
    m_font_height = font.m_font_height;
    m_font_weight = font.m_font_weight;
    m_font_width = font.m_font_width;

    reloadFont();
}

double wxAggFont::getHeight() const
{
    if ( m_fontOk )
        return m_font_height;
    else
        return -1.0;
}

void wxAggFont::setHeight( double h )
{
    if ( h <= 0 )
        return;

    if ( m_fontOk )
    {
        m_font_engine.height( h );
        m_font_height = h;
        m_font_engine.width( h );
        m_font_width = h;
    }
}

void wxAggFont::setWidth( double w )
{
    if ( w <= 0 )
        return;

    if ( m_fontOk )
    {
        m_font_engine.width( w );
        m_font_width = w;
    }
}

double wxAggFont::getWidth() const
{
    if ( m_fontOk )
        return m_font_width;
    else
        return -1.0;
}

double wxAggFont::getFontWeight() const
{
    if ( m_fontOk )
        return m_font_weight;
    else
        return -1.0;
}

font_engine_type& wxAggFont::getFontEngine()
{
    return m_font_engine;
}

