#include "wxAggPen.h"
/// initialization

const wxAggPen wxDefaultAggPen = wxAggPen( wxColor(255,255,255,255), 1.0, wxAGGPEN_SOLID );

IMPLEMENT_DYNAMIC_CLASS( wxAggPen, wxAggObject )

wxAggPen::wxAggPen()
{
    m_color = agg::rgba( 1.0, 1.0, 1.0, 1.0 );
    m_linewidth = 1.0;

    m_linestyle = wxAGGPEN_SOLID;

    m_cap = agg::butt_cap;

    createDashes();

    m_isOk = true;
}

wxAggPen::wxAggPen( const wxColour& color, double width, int style )
{
    m_color = Agg::convertToRGBA( color );

    if ( width <= 0 )
        m_linewidth = 1.0;

    m_linewidth = width;

    if ( style < 0 || style > 6 ) m_linestyle = wxAGGPEN_SOLID;
    else m_linestyle = style;

    m_cap = agg::butt_cap;

    createDashes();

    m_isOk = true;
}

wxAggPen::wxAggPen( const wxAggPen& pen )
{
    if ( pen.m_isOk == false )
    {
        m_isOk = false;
        m_dashcount = 0;
        m_dashes = NULL;
    }
    else
    {
        m_color = pen.m_color;
        m_linewidth = pen.m_linewidth;
        m_linestyle = pen.m_linestyle;
        m_cap = pen.m_cap;

        m_dashcount = pen.m_dashcount;
        m_dashes = new double[m_dashcount];
        for ( int i = 0; i < m_dashcount; ++i )
        {
            m_dashes[i] = pen.m_dashes[i];
        }

        m_isOk = true;
    }
}

wxAggPen::wxAggPen( const wxPen& pen )
{
    m_color = Agg::convertToRGBA( pen.GetColour() );
    m_linewidth = (double) pen.GetWidth();

    switch ( pen.GetCap() )
    {
        case wxCAP_BUTT:        m_cap = agg::butt_cap; break;
        case wxCAP_PROJECTING:  m_cap = agg::square_cap; break;
        case wxCAP_ROUND:
        default:                m_cap = agg::round_cap; break;
    }

    switch ( pen.GetStyle() )
    {
        case wxTRANSPARENT: m_linestyle = wxAGGPEN_TRANSPARENT; break;
        case wxDOT:         m_linestyle = wxAGGPEN_DOT; break;
        case wxLONG_DASH:   m_linestyle = wxAGGPEN_LONG_DASH; break;
        case wxSHORT_DASH:  m_linestyle = wxAGGPEN_SHORT_DASH; break;
        case wxDOT_DASH:    m_linestyle = wxAGGPEN_DOT_DASH; break;
        case wxSOLID:
        default:            m_linestyle = wxAGGPEN_SOLID;
    }

    createDashes();

    m_isOk = true;
}

wxAggPen::~wxAggPen()
{
    delete[] m_dashes;

    m_isOk = false;
}

wxAggPen& wxAggPen::operator = ( const wxAggPen& pen )
{
    if ( &pen == this )
        return *this;

    if ( pen.m_isOk == false )
    {
        m_isOk = false;
        m_dashes = NULL;
        m_dashcount = 0;
        return *this;
    }

    m_color = pen.m_color;
    m_linewidth = pen.m_linewidth;
    m_linestyle = pen.m_linestyle;
    m_cap = pen.m_cap;

    m_dashcount = pen.m_dashcount;

    if ( m_dashes != NULL )
        delete[] m_dashes;

    m_dashes = new double[m_dashcount];
    for ( int i = 0; i < m_dashcount; ++i )
    {
        m_dashes[i] = pen.m_dashes[i];
    }

    m_isOk = true;

    return *this;
}

/// getters

bool wxAggPen::IsOk() const
{
    return m_isOk;
}

agg::line_cap_e wxAggPen::GetCap() const
{
    return m_cap;
}

wxColour wxAggPen::GetWxColor() const
{
    return Agg::convertToColor( m_color );
}

agg::rgba wxAggPen::GetAggColor() const
{
    return m_color;
}

int wxAggPen::GetStyle() const
{
    return m_linestyle;
}

double wxAggPen::GetWidth() const
{
    return m_linewidth;
}

int wxAggPen::GetDashes( double **dashes )
{
    if ( m_linestyle != wxAGGPEN_USER_DASH )
        createDashes();

    *dashes = m_dashes;
    return m_dashcount;
}

/// setters
void wxAggPen::SetCap( agg::line_cap_e cap )
{
    if ( cap == m_cap )
        return;

    m_cap = cap;
}

void wxAggPen::SetColor( const wxColour& color )
{
    m_color = Agg::convertToRGBA( color );
}

void wxAggPen::SetColor( agg::rgba color )
{
    m_color = color;
}

void wxAggPen::SetDashes( int dashcount, double* dashes )
{
    if ( dashcount < 0 )
        return;

    m_dashcount = dashcount;

    if ( m_dashes != NULL )
        delete[] m_dashes;

    m_dashes = new double[2*m_dashcount];

    for ( int j = 0; j < m_dashcount*2; ++j )
    {
        m_dashes[j] = dashes[j];
    }
}

void wxAggPen::SetStyle( int style )
{
    if ( style == m_linestyle )
        return;

    if ( style < 0 || style > 6 ) m_linestyle = wxAGGPEN_SOLID;
    else m_linestyle = style;

    /*if ( m_linestyle != wxAGGPEN_USER_DASH )
        createDashes();*/
}

void wxAggPen::SetWidth( double width )
{
    if ( width <= 0 )
        return;

    m_linewidth = width;

    /*if ( m_linestyle != wxAGGPEN_USER_DASH )
        createDashes();*/
}

void wxAggPen::SetPen( const wxAggPen& pen )
{
    m_color = pen.m_color;
    m_linewidth = pen.m_linewidth;
    m_linestyle = pen.m_linestyle;
    m_cap = pen.m_cap;

    m_dashcount = pen.m_dashcount;
    m_dashes = new double[m_dashcount];
    for ( int i = 0; i < m_dashcount; ++i )
    {
        m_dashes[i] = pen.m_dashes[i];
    }

    m_isOk = true;
}

void wxAggPen::SetPen( const wxPen& pen )
{
    m_color = Agg::convertToRGBA( pen.GetColour() );
    m_linewidth = (double) pen.GetWidth();

    switch ( pen.GetCap() )
    {
        case wxCAP_BUTT:        m_cap = agg::butt_cap; break;
        case wxCAP_PROJECTING:  m_cap = agg::square_cap; break;
        case wxCAP_ROUND:
        default:                m_cap = agg::round_cap; break;
    }

    switch ( pen.GetStyle() )
    {
        case wxTRANSPARENT: m_linestyle = wxAGGPEN_TRANSPARENT; break;
        case wxDOT:         m_linestyle = wxAGGPEN_DOT; break;
        case wxLONG_DASH:   m_linestyle = wxAGGPEN_LONG_DASH; break;
        case wxSHORT_DASH:  m_linestyle = wxAGGPEN_SHORT_DASH; break;
        case wxDOT_DASH:    m_linestyle = wxAGGPEN_DOT_DASH; break;
        case wxSOLID:
        default:            m_linestyle = wxAGGPEN_SOLID;
    }

    createDashes();

    m_isOk = true;
}

void wxAggPen::SetPen( const wxColour& color, double width, int style )
{
    m_color = Agg::convertToRGBA( color );

    m_linewidth = width;

    if ( style < 0 || style > 6 ) m_linestyle = wxAGGPEN_SOLID;
    else m_linestyle = style;

    m_cap = agg::butt_cap;

    createDashes();

    m_isOk = true;
}

/// private

void wxAggPen::createDashes()
{
    if ( m_dashes != NULL )
        delete[] m_dashes;

    double mw = ( m_linewidth < 1 ) ? 1.0 : m_linewidth;

    if ( m_linestyle == wxAGGPEN_SOLID )
    {
        m_dashcount = 1; m_dashes = new double[2];
        m_dashes[0] = 1.0; m_dashes[1] = 0;
    }

    if ( m_linestyle == wxAGGPEN_DOT )
    {
        m_dashcount = 1; m_dashes = new double[2];
        m_dashes[0] = mw; m_dashes[1] = mw + 2.0;
    }

    if ( m_linestyle == wxAGGPEN_DOT_DASH )
    {
        m_dashcount = 2; m_dashes = new double[4];
        m_dashes[0] = 9.0; m_dashes[1] = 6.0; m_dashes[2] = 3.0; m_dashes[3] = 3.0;
    }

    if ( m_linestyle == wxAGGPEN_LONG_DASH )
    {
        m_dashcount = 1; m_dashes = new double[2];
        m_dashes[0] = 19.0; m_dashes[1] = 9.0;
    }

    if ( m_linestyle == wxAGGPEN_SHORT_DASH )
    {
        m_dashcount = 1; m_dashes = new double[2];
        m_dashes[0] = 9.0; m_dashes[1] = 6.0;
    }

    if ( m_linestyle == wxAGGPEN_TRANSPARENT )
    {
        m_dashcount = 0;
        m_dashes = NULL;
    }
}
