#include "AimValuesPanel.h"

#include <wx/dcbuffer.h>
#include "../agg/wxAgg2.h"
#include "../common/types/RealPointArray.h"
#include "../DirHelper.h"
#include "../logger.h"

BEGIN_EVENT_TABLE( AIMValuesPanel, wxPanel )
    EVT_PAINT( AIMValuesPanel::onPaint )
    EVT_LEFT_DOWN( AIMValuesPanel::onClick )
END_EVENT_TABLE()

AIMValuesPanel::AIMValuesPanel( wxWindow* parent, int id ) //: wxPanel( parent, id, wxDefaultPosition, wxSize( 350, 450) )
{
    m_maxh = 500;
    Create( parent, id, wxDefaultPosition, wxSize( 350, m_maxh ) );
    m_topvalueTop = 999;
    m_bottomvalueTop = 9999;

    m_topvalueBottom = 999;
    m_bottomvalueBottom = 9999;

    m_topvalueTS = m_topvalueBS = _T("");
    m_bottomvalueTS = m_bottomvalueBS = _T("");

    m_topvalueFormatTop = _T("No Dot");
    m_bottomvalueFormatTop = _T("No Dot");

    m_topvalueFormatBottom = _T("No Dot");
    m_bottomvalueFormatBottom = _T("No Dot");

    m_topRSlot = m_bottomRSlot = -1;

    setDisplayColors( RED_GREEN );

    m_toptop = m_topbottom = m_bottombottom = m_bottomtop = 0;

    m_onDispBB = m_onDispBT = m_onDispTB = m_onDispTT = _T("");

    Refresh( false );
}

void AIMValuesPanel::redraw()
{
    formatValues( m_tvT, m_bvT, false ); // not on round (top)
    formatValues( m_tvB, m_bvB, true ); // on round ( bottom )

    // if something changed -> refresh
    if ( m_onDispTT != m_tvT || m_onDispTB != m_tvB || m_onDispBT != m_bvT || m_onDispBB != m_bvB )
        Refresh( false );
}

void AIMValuesPanel::setSlots( int topRound, int bottomRound )
{
    bool needrefresh = false;

    if ( m_topRSlot != topRound )
    {
    m_topRSlot = topRound;
        needrefresh = true;
    }

    if ( m_bottomRSlot != bottomRound )
    {
        m_bottomRSlot = bottomRound;
        needrefresh = true;
    }

    if ( needrefresh )
        Refresh( false );
    }

void AIMValuesPanel::setTopDisplayFormat( const wxString& format, bool isDisplayedOnRound )
{
    if ( isDisplayedOnRound )
        m_topvalueFormatBottom = format;
    else
        m_topvalueFormatTop = format;
}

void AIMValuesPanel::setBottomDisplayFormat( const wxString& format, bool isDisplayedOnRound )
{
    if ( isDisplayedOnRound )
        m_bottomvalueFormatBottom = format;
    else
        m_bottomvalueFormatTop = format;
}

void AIMValuesPanel::setTopValue( double value, bool isDisplayedOnRound )
{
    if ( isDisplayedOnRound )
        m_topvalueBottom = value;
    else
        m_topvalueTop = value;
}

void AIMValuesPanel::setBottomValue( double value, bool isDisplayedOnRound )
{
    if ( isDisplayedOnRound )
        m_bottomvalueBottom = value;
    else
        m_bottomvalueTop = value;
}

void AIMValuesPanel::setTopValue( const wxString& value, bool isDisplayedOnRound )
{
    if ( isDisplayedOnRound )
    {
        m_topvalueBottom = -999999999;
        m_topvalueBS = value;
    }
    else
    {
        m_topvalueTop = -999999999;
        m_topvalueTS = value;
    }
}

void AIMValuesPanel::setBottomValue( const wxString& value, bool isDisplayedOnRound )
{
    if ( isDisplayedOnRound )
    {
        m_bottomvalueBottom = -999999999;
        m_bottomvalueBS = value;
    }
    else
    {
        m_bottomvalueTop = -999999999;
        m_bottomvalueTS = value;
    }
}

void AIMValuesPanel::setValues( double topvalue, double bottomvalue, bool isDisplayedOnRound )
{
    if ( isDisplayedOnRound )
    {
        m_topvalueBottom = topvalue;
        m_bottomvalueBottom = bottomvalue;
    }
    else
    {
        m_topvalueTop = topvalue;
        m_bottomvalueTop = bottomvalue;
    }
}

void AIMValuesPanel::onClick( wxMouseEvent& event )
{
    wxPoint pt = event.GetPosition();
    double dist = sqrt( pow( pt.x - m_buttonPos.x, 2 ) + pow( pt.y - m_buttonPos.y, 2 ) );
    if ( dist < 10 )
    {
        observerNotify( _T("button") );
    }

}

void AIMValuesPanel::onPaint( wxPaintEvent& event )
{
    wxBufferedPaintDC dc( this );
    dc.SetBackground( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    dc.Clear();

    wxAgg2 canvas( this, wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE) );

    wxRealSize r = GetSize();
    double h = r.h;

    if ( h > (double)m_maxh )
        h = (double)m_maxh;

    h = 450;

    double cxtop, cytop;
    double cxbottom, cybottom;
    double translY = 12.0;

    cxtop = h / 4 * 0.86 + 15; cytop = h / 4 + translY;

    double roundsize = h/4 * 0.86;

    cxbottom = cxtop;// + roundsize + 30;

    cybottom = 3 * h / 4 + translY;

    canvas.setBrush( wxDefaultAggBrush );
    canvas.setBrush( wxColor( 130, 130, 130 ) );
    canvas.setPen( wxColor( 130, 130, 130 ) );
    canvas.drawCircle( cxtop, cytop, h / 4 * 0.86 );
    canvas.drawCircle( cxbottom, cybottom, h / 4 * 0.86 );

    canvas.setBrush( wxColor( 0,0,0 ) );
    canvas.setPen( wxColor( 130,130,130 ) );
    canvas.drawCircle( cxtop, cytop, h / 4 * 0.83 );
    canvas.drawCircle( cxbottom, cybottom, h / 4 * 0.83 );

//    LOGP( tvL + bvL ) LOGP( tvR + bvR )

    // draw top values
    canvas.setPen( m_topColor );
    canvas.loadTTFont( DirHelper::GetPath(dFONTS) + _T("LED.ttf"), 68.0, 0.0 );

    wxRealRect rect = canvas.getTTFTextBound( m_tvT, 0 );
    canvas.drawTTFText( m_tvT, cxtop - rect.w / 2, cytop - 23 + translY );

    rect = canvas.getTTFTextBound( m_tvB, 0 );
    canvas.drawTTFText( m_tvB, cxbottom - rect.w / 2, cybottom - 23 + translY);

    // draw bottom values
    canvas.setFontHeight( 54.0 );
    canvas.setPen( m_bottomColor );

    rect = canvas.getTTFTextBound( m_bvT, 0 );
    canvas.drawTTFText( m_bvT, cxtop - rect.w / 2, cytop + 23 + translY);

    rect = canvas.getTTFTextBound( m_bvB, 0 );
    canvas.drawTTFText( m_bvB, cxbottom - rect.w / 2, cybottom + 23 + translY);

    // draw 'buttons'

    wxAggBrush* brush = canvas.getBrush();
    brush->SetStyle( wxAGGBRUSH_GRADIENT );
    brush->SetEndColor( wxColor( 30, 30, 60 ) );
    brush->SetColor( wxColor( 255, 255, 255 ) );
    brush->SetGradientOrientation( 45 );
    canvas.setTransparentPen();

    m_buttonPos.x = cxbottom;
    m_buttonPos.y = cybottom + 55;
    m_buttonr = 12;
    canvas.drawCircle( cxtop, cytop + 55, m_buttonr );
    canvas.drawCircle( cxbottom, cybottom + 55, m_buttonr );

    canvas.setPen( wxDefaultAggPen );
    canvas.setPen( wxColor( 0, 0, 0 ) );
    canvas.loadTTFont(  DirHelper::GetPath(dFONTS) + _T("arial.ttf"), 14.0, 0 );
    canvas.drawTTFText( wxString::Format( _T("Values of the selected slot (Slot %d)"), m_topRSlot ), 10, 14 );
    canvas.drawTTFText( wxString::Format( _T("Displayed on the Round (Slot %d)"), m_bottomRSlot ), 15, 244 );

    wxRealPoint pt1( cxbottom + roundsize, m_buttonPos.y + 10 ), pt2( cxbottom + m_buttonr + 2, m_buttonPos.y + 2 );

    canvas.setPen( wxColor( 255, 0, 0, 196 ) );
    canvas.setPenWidth( 5.0 );
    drawArrow( &canvas, pt1, pt2 );

    canvas.setFontHeight( 13 );

    canvas.drawTTFText( _T("Slot selector\nbutton"), pt1.x + 5, pt1.y );

    canvas.setPen( wxColor( 85, 85, 85, 200) );

    pt1.x = cxtop + roundsize + 15;
    pt2.x = cxtop + roundsize / 2.0 + 20;

    if ( m_toptop >= 0 )
    {
        pt1.y = pt2.y = 91;
        drawArrow( &canvas, pt1, pt2, 0.4 );
        canvas.drawTTFText( m_topvarnames[m_toptop], pt1.x + 5, pt1.y + 5 );
    }

    if ( m_topbottom >= 0 )
    {
        pt1.y = pt2.y = 141;
        drawArrow( &canvas, pt1, pt2, 0.4 );
        canvas.drawTTFText( m_bottomvarnames[m_topbottom], pt1.x + 5, pt1.y + 5 );
    }

    if ( m_bottomtop >= 0 )
    {
        pt1.y = pt2.y = 311;
        drawArrow( &canvas, pt1, pt2, 0.4 );
        canvas.drawTTFText( m_topvarnames[m_bottomtop], pt1.x + 5, pt1.y + 5 );
    }

    if ( m_bottombottom >= 0 )
    {
        pt1.y = pt2.y = 361;
        drawArrow( &canvas, pt1, pt2, 0.4 );
        canvas.drawTTFText( m_bottomvarnames[m_bottombottom], pt1.x + 5, pt1.y + 5 );
    }

    //
    canvas.setPen( wxColor( 0, 0, 0 ) );
    canvas.loadTTFont( DirHelper::GetPath(dFONTS) + _T("arial.ttf"), 13, 0.0 );
    wxString str = wxString::Format( _T("If you press Burn,\nthe selected slot(%d) on the bottom\nround display will be set to the default slot."), m_bottomRSlot );
    canvas.drawTTFText( str, 15, 470 );
    canvas.render( dc );

    m_onDispTT = m_tvT;
    m_onDispTB = m_tvB;
    m_onDispBT = m_bvT;
    m_onDispBB = m_bvB;
}

void AIMValuesPanel::setDisplayColors( int displayColor )
{
    wxColor red( 226, 73, 61, 224 );
    wxColor green( 173, 252, 56, 224 );

    switch ( displayColor )
    {
        case RED_GREEN:
            m_topColor = red;
            m_bottomColor = green;
            break;

        case YELLOW_ORANGE:
            break;

        case RED_ORANGE:
            m_topColor = red;
            break;

        case ORANGE_GREEN:
            m_bottomColor = green;
            break;

        case BLUE_RED:
            m_bottomColor = red;
            break;

        case BLUE_GREEN:
            m_bottomColor = green;
            break;

        case RED_RED:
        default:
            m_topColor = red;
            m_bottomColor = red;
            break;

    }
}

/*
wxString AIMValuesPanel::formatAIMValue( double& val, bool isTopRow, int dotpos )
{
    ///
    /// TODO: negative numbers
    ///
    int value = (int)val;
    bool valid = true;

    wxString formatted;
    wxString hexarr[6] = { _T("A"), _T("B"), _T("C"), _T("D"), _T("E"), _T("F") };

    if ( isTopRow )
    {
        // negative value
        if ( value < 0 )
        {
            int absval = (int)( fabs( value ) );
            if ( absval < 100 )
                formatted = wxString::Format(_T("-%02d"), absval );
            else {
                int firstD = absval / 10, rem = absval % 10;
                if ( firstD >= 16 )
                {
                    valid = false; formatted = _T("-FU");
                }
                else
                {
                    formatted = _T("-") + hexarr[firstD - 10] + wxString::Format( _T("%d"), rem );
                }
            }
        }
        // positive value
        else {
            if ( value < 1000 ) formatted = wxString::Format( _T("%03d"), value );
            else
            {
                int firstD = value / 100, rem = value % 100;
                if ( firstD >= 16 ) {
                    valid = false; formatted = _T("FF0");
                }
                else
                {
                    formatted = hexarr[ firstD - 10 ] + wxString::Format( _T("%02d"), rem );
                }
            }
        }
    }
    else
    {
        // negative number
        if ( value < 0 )
        {
            int absval = (int)( fabs( value ) );

            if ( absval < 1000 )
                formatted = wxString::Format( _T("-%03d"), absval );
            else
            {
                int firstD = absval / 100, rem = absval % 100;
                if ( firstD >= 16 )
                {
                    valid = false;
                    formatted = _T("-FFU");
                }
                else
                {
                    formatted = _T("-") + hexarr[firstD-10] + wxString::Format(_T("%02d"), rem );
                }
            }
        }
        // positive number
        else
        {
            if ( value < 10000 ) formatted = wxString::Format( _T("%04d"), value );
            else
            {
                int firstD = value / 1000, rem = value % 1000;
                if ( firstD >= 16 ) {
                    valid = false;; formatted = _T("FFF0");
                }
                else
                {
                    formatted = hexarr[ firstD - 10 ] + wxString::Format( _T("%03d"), rem );
                }
            }
        }
    }
    // overflow
    if ( !valid )
        return formatted;

    // place dot
    if ( isTopRow )
    {
        switch ( dotpos )
        {
            case 0: formatted = formatted.Left( 2 ) + _T(".") + formatted.Right( 1 ); break;
            case 1: formatted = formatted.Left( 1 ) + _T(".") + formatted.Right( 2 ); break;
            case 3:
            default: break;
        }
    }
    else
    {
        switch ( dotpos )
        {
            case 0: formatted = formatted.Left( 3 ) + _T(".") + formatted.Right( 1 ); break;
            case 1: formatted = formatted.Left( 2 ) + _T(".") + formatted.Right( 2 ); break;
            case 2: formatted = formatted.Left( 1 ) + _T(".") + formatted.Right( 3 ); break;
            case 3:
            default: break;
        }
    }

    return formatted;
}*/

void AIMValuesPanel::formatValues( wxString& topval, wxString& bottomval, bool isDisplayedOnRound )
{
    int fval;
    int ival;

    double value = ( isDisplayedOnRound ) ? m_topvalueBottom : m_topvalueTop;
    wxString format = ( isDisplayedOnRound ) ? m_topvalueFormatBottom : m_topvalueFormatTop;

    if ( value <= -999999999 )
    {
        topval = ( isDisplayedOnRound ) ? m_topvalueBS : m_topvalueTS;
    }
    else
    {
        ival = (int)value;

        if ( format == _T("No Dot") )
            topval = wxString::Format(_T("%03d%04d"), ival, fval );
        else if ( format == _T("XX.X") )
        {
            fval = (value - ival) * 10.0;
            topval = wxString::Format(_T("%02d.%01d"), ival, fval );
        }
        else if ( format == _T("X.XX") )
        {
            fval = (value - ival) * 100.0;
            topval = wxString::Format(_T("%01d.%02d"), ival, fval );
        }

        if ( format == _T("No Dot") )
            topval = topval.Left( 3 );
        else
            topval = topval.Left( 4 );
    }

    ///

    value = ( isDisplayedOnRound ) ? m_bottomvalueBottom : m_bottomvalueTop;
    format = ( isDisplayedOnRound ) ? m_bottomvalueFormatBottom : m_bottomvalueFormatTop;

    if ( value <= -999999999 )
    {
        bottomval = ( isDisplayedOnRound ) ? m_bottomvalueBS : m_bottomvalueTS;
    }
    else
    {
        ival = (int)value;

        if ( format == _T("No Dot") )
            bottomval = wxString::Format(_T("%04d%5d"), ival, fval );
        else if ( format == _T("X.XXX") )
        {
            fval = (value - ival)*1000;
            bottomval = wxString::Format(_T("%01d.%03d"), ival, fval );
        }
        else if ( format == _T("XX.XX") )
        {
            fval = (value - ival)*100;
            bottomval = wxString::Format(_T("%02d.%2d"), ival, fval );
        }
        else if ( format == _T("XXX.X") )
        {
            fval = (value - ival)*10;
            bottomval = wxString::Format(_T("%03d.%5d"), ival, fval );
        }


        if ( format == _T("No Dot") )
            bottomval = bottomval.Left( 4 );
        else
            bottomval = bottomval.Left( 5 );
    }
}

void AIMValuesPanel::drawArrow( wxAgg2* canvas, wxRealPoint start,  wxRealPoint end, double arrheadlenfactor, double theta, bool fillit )
{
    canvas->drawLine( start, end );

    double arrlen = sqrtf( pow( start.x - end.x, 2 ) + pow( start.y - end.y, 2 ) );

    double arrheadlen;
    if ( arrheadlenfactor < 0 )
        arrheadlen = arrlen * 0.25;
    else
        arrheadlen = arrlen * arrheadlenfactor;

    double angle = atan2 ( end.y - start.y, end.x - start.x) + M_PI;

    double x1, y1, x2, y2;

    x1 = end.x + arrheadlen * cos( angle - theta );
    y1 = end.y + arrheadlen * sin( angle - theta );
    x2 = end.x + arrheadlen * cos( angle + theta );
    y2 = end.y + arrheadlen * sin( angle + theta );

    if ( fillit )
    {
        RealPointArray points;
        points.Add( wxRealPoint( x1, y1 ) ); points.Add( wxRealPoint( x2, y2 ) ); points.Add( end );
        canvas->drawPolygon( points );
    }
    else
    {
        canvas->drawLine( x1, y1, end.x, end.y );
        canvas->drawLine( x2, y2, end.x, end.y );
    }
}

void AIMValuesPanel::setSelectedValues( int toptop, int topbottom, int bottomtop, int bottombottom )
{
    m_toptop = toptop; m_topbottom = topbottom;

    if ( bottomtop != -1 )
        m_bottomtop = bottomtop;
    if ( bottombottom != -1 )
        m_bottombottom = bottombottom;

    Refresh( false );
}
