#ifndef AIMVALUESPANEL_H
#define AIMVALUESPANEL_H

#include <wx/wx.h>
#include "../Observable.h"

class AIMDisplayFrame;
class wxAgg2;

class AIMValuesPanel : public wxPanel, public Observable
{
    public:
        AIMValuesPanel( wxWindow* parent, int id = wxID_ANY );

    public:
        void        setTopDisplayFormat( const wxString& format, bool isDisplayedOnRound );
        void        setBottomDisplayFormat( const wxString& format, bool isDisplayedOnRound );
        void        setTopValue( double value, bool isDisplayedOnRound );
        void        setBottomValue( double value, bool isDisplayedOnRound );

        void        setTopValue( const wxString& value, bool isDisplayedOnRound );
        void        setBottomValue( const wxString& value, bool isDisplayedOnRound );

        void        setValues( double topvalue, double bottomvalue, bool isDisplayedOnRound );
        void        redraw();

        void        setVarNames( const wxArrayString& top, const wxArrayString& bottom ) { m_topvarnames = top; m_bottomvarnames = bottom; }
        void        setSelectedValues( int toptop, int topbottom, int bottomtop = -1, int bottombottom = -1);

        void        setSlots( int topRound, int bottomRound );

        enum DisplayColors { RED_RED, RED_GREEN, YELLOW_ORANGE, RED_ORANGE, ORANGE_GREEN, BLUE_RED, BLUE_GREEN };

        void        setDisplayColors( int displayColor );

    private:

        int             m_maxh; // needed to be fixed (resize problems)

        int             m_topRSlot, m_bottomRSlot;

        wxColor         m_topColor, m_bottomColor;

        wxString        m_topvalueFormatTop;
        wxString        m_bottomvalueFormatTop;

        wxString        m_topvalueFormatBottom;
        wxString        m_bottomvalueFormatBottom;

        double          m_topvalueTop;
        double          m_bottomvalueTop;

        double          m_topvalueBottom;
        double          m_bottomvalueBottom;

        wxString        m_topvalueTS;
        wxString        m_bottomvalueTS;
        wxString        m_topvalueBS;
        wxString        m_bottomvalueBS;

        wxRealPoint     m_buttonPos;
        double          m_buttonr;

        wxArrayString   m_topvarnames;
        wxArrayString   m_bottomvarnames;

        int             m_toptop, m_topbottom;
        int             m_bottomtop, m_bottombottom;

        wxString        m_onDispTT, m_onDispTB, m_onDispBT, m_onDispBB; // values displaying

        wxString        m_tvT, m_bvT; // values to display
        wxString        m_tvB, m_bvB; // values to display

    private:

        void    onPaint( wxPaintEvent& event );

        void    onClick( wxMouseEvent& event );

        void    formatValues( wxString& topval, wxString& bottomval, bool isDisplayedOnRound );
        wxString    formatAIMValue( double& val, bool isTopRow, int dotpos ); // helper

        void    drawArrow( wxAgg2* canvas, wxRealPoint start, wxRealPoint end, double arrheadlenfactor = -1, double theta = M_PI/10, bool fillit = false );

        friend class AIMDisplayPanel;

        DECLARE_EVENT_TABLE()
};

#endif // AIMVALUESPANEL_H
