#include "app.h"

#include "DirHelper.h"
#include "LogDirManager.h"
#include "DisplayHelper.h"
#include <wx/xrc/xmlres.h>
#include <wx/image.h>
#include <wx/socket.h>

#include "logger.h"


#include "configmanager/ConfigHelper2.h"
#include "configmanager/ConfigMgrMainGui.h"
#include "configmanager/MainGuiConfig.h"
#include "ConfigBase.h"

#include "project/ProjectChooserFrame.h"
#include "project/ProjectHelper.h"
#include "project/ProjectManager.h"


#include "common/VemsContext.h"
#include "fileparser/ConfigCriteriaParser.h"
#include "firmwareconfig/ConfigCriteriaData.h"
#include "common/VemsMessageHashHelper.h"
#include "bcWidgets/BitmapCanvasSystemSettings.h"

#include "common/HelpUtil.h"
#include "common/IconSetsHelper.h"
#include "vemsgauge/GaugeGroupManager.h"
#include "version.h"
#include "MTConfigHelper.h"


#include "gui/VemsSplashScreen.h"

#include "common/wxBitmapTypeHelper.h"

#include "sqlite/wxSQLite3ExceptionMessageBox.h"

#include <wx/app.h>

#include "bcWidgets/bcProgressDialog.h"

IMPLEMENT_APP(MyApp);

BEGIN_EVENT_TABLE(MyApp, wxApp)

      //EVT_KEY_DOWN(MyApp::OnKeyDown)


END_EVENT_TABLE()

MyApp::MyApp(){
    m_frame = NULL;
    m_timer = NULL;
    m_splashFrame = NULL;
}

void MyApp::eventBlocker(int blocktime)
{
    //m_isEventDisabled = true;
    //m_timer =
    m_timer->Start(blocktime, wxTIMER_ONE_SHOT );
}

void MyApp::OnAssertFailure(const wxChar *file, int line, const wxChar *func, const wxChar *cond, const wxChar *msg)
{
}

void MyApp::OnAssert(const wxChar *file, int line, const wxChar *func, const wxChar * msg)
{
}

int MyApp::FilterEvent(wxEvent& evt)
{
    wxWindow* activeWindow = wxWindow::FindFocus();

    if ( !activeWindow ) return -1;

    if ( activeWindow->IsKindOf(CLASSINFO(bcProgressDialog) ) ) {
        return -1;
    }

    if ( activeWindow->IsKindOf(CLASSINFO(wxDialog) ) ) {
        if ( ((wxDialog*)activeWindow)->IsModal() ) return -1;
    }

    //Logger::logf(LOG_MSG,_T("MyApp::FilterEvent START"));
    wxObject* obj = evt.GetEventObject();

    if(!obj) return -1;
    if(obj->IsKindOf(  CLASSINFO(wxTextCtrl)  ) ){
        return -1;
    }
//    if(obj->IsKindOf(  CLASSINFO(wxSlider)  ) ){
//        return -1;
//    }
    if(obj->IsKindOf(  CLASSINFO(wxComboBox)  ) ){
        return -1;
    }

    int evtType = evt.GetEventType();
    if( evtType == wxEVT_KEY_DOWN)
    {
            wxKeyEvent& keyEvt = (wxKeyEvent&)evt;
            //Logger::logf(LOG_MSG,_T("MyApp::FilterEvent keycode: %i"), keyCode);
            /// global key must use modifier key!!!
            int keyModifier = keyEvt.GetModifiers();
            int b = (keyModifier & wxMOD_ALT) | (keyModifier & wxMOD_CONTROL ) | (keyModifier & wxMOD_SHIFT);
            if(!b){
                return -1;
            }
            int keyCode = keyEvt.GetKeyCode();
            if(keyCode == WXK_ALT){
                 /// only modifier pressed
                return -1;
            }
            wxString keyCodeStr = wxKeyBind::KeyCodeToString(keyCode);
            if( (keyCodeStr == wxEmptyString) || (keyCodeStr.size() == 0) ){
                /// only modifier pressed
                return -1;
            }

            if ( ( keyModifier & wxMOD_ALT) && ( (char)keyCode == '0' ) )
            {
                m_frame->loadLastLogFile();

                eventBlocker();

                return true;
            }

            if( (keyModifier& wxMOD_SHIFT) && keyCode == WXK_F11 ) {
                if(m_timer->IsRunning())
                    return true;

                eventBlocker();
                m_frame->ToggleFullScreen();
                return true;
            }

#if AUI_MDI_FRAME
            if( (keyModifier& wxMOD_CONTROL) && keyCode == WXK_TAB ) {
                if(m_timer->IsRunning())
                    return true;

                eventBlocker();

                if( (keyModifier& wxMOD_SHIFT) )
                    m_frame->ChangePage(-1);
                else
                    m_frame->ChangePage(1);

                return true;
            }
#endif
            //Logger::logf(LOG_MSG,_T("MyApp::FilterEvent keycodeSTR:")+keyCodeStr+_T("||"));
            wxString modifierStr = wxKeyBind::KeyModifierToString(keyModifier);
            /*
            modifierStr.UpperCase();
            modifierStr = modifierStr.BeforeLast('+');

            wxString fullKey =  modifierStr +_T("-") + keyCodeStr;
            */
            wxString fullKey =  modifierStr + keyCodeStr;

            //Logger::logf(LOG_MSG,_T("MyApp::FilterEvent keycode: %i: ")+fullKey, keyCode);

            GaugeGroupManager* ggm = m_frame->getGaugeGroupManager();

            if( ggm->showGroupByHotkey(fullKey) ){
                if(m_timer->IsRunning() ){
                    return true;
                }
                eventBlocker();
                return true;
            } else if( fullKey == _T("Shift+Tab") ){
                if(m_timer->IsRunning() ){
                    return true;
                }
                eventBlocker();

                ggm->SlideFocusToNextTable( m_frame );

            }

            //}
    }
    else if( evtType == wxEVT_COMMAND_MENU_SELECTED)
    {
            if(m_timer->IsRunning() ){
                //Logger::logf(LOG_MSG,_T("MyApp::FilterEvent wxEVT_COMMAND_MENU_SELECTED disabled"));
                return true;
            }
            //eventBlocker();
            //Logger::logf(LOG_MSG,_T("MyApp::FilterEvent wxEVT_COMMAND_MENU_SELECTED"));
            //return true;
    }
    else if( evtType == wxEVT_SET_FOCUS )
    {/*
        if( obj != m_frame ) return -1;

        GaugeGroupManager* ggm = m_frame->getGaugeGroupManager();

        Logger::logf(LOG_MSG,_T("MyApp::FilterEvent wxEVT_SET_FOCUS to MainFrame, ggm->IsGroupChanged()=%i"),ggm->IsGroupChanged());

        if(ggm->IsGroupChanged())
            return true;
        else
            return -1;
*/
    }


    return -1;
}

void MyApp::OnKeyDown(wxKeyEvent &evt)
{
    wxObject* obj = evt.GetEventObject();
    bool isFound = obj->IsKindOf(CLASSINFO(MainFrame));
    if(isFound){
         Logger::logf(LOG_MSG,_T("MyApp::OnKeyDown FROM mainfarme = %i"), evt.GetKeyCode());
    }

    Logger::logf(LOG_MSG,_T("MyApp::OnKeyDown = %i"), evt.GetKeyCode());

    evt.Skip();
}

size_t MyApp::GetRunningTimeMs()
{
    return m_runningTime.Time();
}

bool MyApp::OnInit()
{
    m_runningTime.Start();

    wxSocketBase::Initialize();

    BitmapCanvasSystemSettings::Init();
    ::wxInitAllImageHandlers();
    wxBitmapTypeHelper::Init();

    IconSetsHelper::Init();

    m_timer = new wxTimer();

    setlocale(LC_ALL, "C");

     wxFileName prgName = wxFileName(argv[0]);
     wxString path = prgName.GetPath();
     if(path.StartsWith(_T("."))){
         path  = wxFileName::GetCwd();
     }
     DirHelper::SetProgramRootDir(path);
	 LogDirManager::SetDbPath( DirHelper::GetPath(dCONFIG, true) );

    ShowSplashScreen();

    // parse the cmd line

    wxString logfile = DirHelper::GetPath(dDEBUGLOG) + _T("filelog.txt");
    Logger::initLog( logfile );
    Logger::logf(LOG_MSG, AutoVersion::getFullVersion() );

    HelpUtil::addBookToDialog( DirHelper::GetPath( dHELP ) + _T("vemstune.hhp") );
    HelpUtil::addBookToDialog( DirHelper::GetPath( dHELP ) + _T("dialogs/dialogs.hhp") );
    HelpUtil::addBook( DirHelper::GetPath( dHELP ) +  _T("../help2/vemstune.hhp") );//to dialog and help too

    ConfigHelper2::init();

    MainGuiConfig mainGuiCfg;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_LAYOUT);
    ConfigMgrMainGui cfgMgr(cfg);
    cfgMgr.load(&mainGuiCfg);

    wxSize ds = DisplayHelper::GetDisplaySize();

    if( mainGuiCfg.m_x >= ds.x ) mainGuiCfg.m_x = 50;
    if( mainGuiCfg.m_y >= ds.y ) mainGuiCfg.m_y = 50;

    if( mainGuiCfg.m_x < 0 - mainGuiCfg.m_width ) mainGuiCfg.m_x = 50;
    if( mainGuiCfg.m_y < 0 - mainGuiCfg.m_height ) mainGuiCfg.m_y = 50;

    //calling parameters
    StringHash params;

    setCwd(argv[0]);
    if(argc >1)
    {
        if( wxString( argv[1] ) == _T("-convert") )
        {
            params[ _T("-convert") ] = wxString( argv[2] );
        }
        else if( wxString( argv[1] ) == _T("-play") )
        {
            params[ _T("-play") ] = wxString( argv[2] );
        }
        else if ( wxString( argv[1] ) == _T("-quickupload") )
        {
            params[ _T("-quickupload") ] = wxString( argv[2] );
        }
        else if ( wxString( argv[1] ) == _T("-quickfirmwareupload") )
        {
            params[ _T("-quickfirmwareupload") ] = wxString( argv[2] );
        }
        else if ( wxString( argv[1] ) == _T("-quickconfigupload") )
        {
            params[ _T("-quickconfigupload") ] = wxString( argv[2] );
        }
        else
        {
            prepareSession(argv[1]);
        }
    }

	bool wxsOK = true;


    wxObject* obj = NULL;

    obj = VemsContext::initObject(_T("VemsMessageHashClass"));
    VemsMessageHashHelper::Parse( *((VemsMessageHashClass*)obj) );

    ProjectHelper::Init();
    Logger::logf(LOG_MSG, _T("MyApp::OnInit() after ProjectHelper::Init()") );

    wxString mainCaption = AutoVersion::getAppName();

    m_frame = new MainFrame(mainCaption, mainGuiCfg.m_x, mainGuiCfg.m_y, mainGuiCfg.m_width, mainGuiCfg.m_height, params );
    m_frame->Maximize( mainGuiCfg.m_isMaximized );

#if MDI_FRAME

    wxMDIClientWindow * win = m_frame->GetClientWindow();
/*
    layoutConf->SetPath(_T("/layout/maingui"));
    long h = -1;
    layoutConf->Read(_T("mdi_win_height"),_T("-1")).ToLong(&h);
*/
    win->SetSize(wxSize(200,300));

#endif

    m_frame->Show(true);
    m_frame->SetFullScreen( mainGuiCfg.m_isFullScreen );

    wxString *tmp1 = new wxString(), *tmp2 = new wxString(), *tmp3 = new wxString();
    prgName.SplitPath(argv[0], tmp1, tmp2, tmp3);

    m_frame->SetRootDir(wxString(*tmp1));
    m_frame->SendSizeEvent();

    delete m_splashFrame;

	return wxsOK;
}

void MyApp::setCwd(wxChar * path_c)
{
    wxString path = wxString::Format(_T("%s"), path_c);
    wxFileName fileName = wxFileName(path);
    wxString dir = fileName.GetPath();
    wxFileName::SetCwd(dir);
}

/**
Session name preparation
example:
c:\vems\car1-firmware1.0.27
c:\vems\car1\firmware1.0.27
Sesion name will be: car1-firmware1.0.27
*/
void MyApp::prepareSession(wxChar * path_c)
{
    wxString path = wxString::Format(_T("%s"), path_c);
    wxFileName fileName = wxFileName(path);
    wxString baseDir = fileName.GetPath();

    const wxArrayString a = fileName.GetDirs();

    wxString sessionName = a.Last();
    size_t idx = a.GetCount();
    if((sessionName.StartsWith(_T("firmware")) ||
       sessionName.StartsWith(_T("v3")))
        && (idx > 1) ){

        sessionName = a.Item(idx-2) +_T("-") + sessionName;
    }

//    Helper::setConsoleSession(sessionName, baseDir, fileName.GetFullName() );

    //wxString headerFile = _T("global.h");
    //wxString hexFile = _T("vems.hex");

//    fprintf(stdout, " MyApp::prepareSession() baseDir=%s, sessionName=%s\n", baseDir.c_str(), sessionName.c_str() );
    //patchFile_air
    //patchFile_coolant

    //configFile1
    //configFile2
}

int MyApp::OnExit()
{
//        Logger::log(LOG_MSG,_T("MyApp::OnExit"));
    m_timer->Stop();
    fprintf(stdout, " MyApp::OnExit() " );

    ConfigHelper2::cleanup();
	MTConfigHelper::cleanup();

	//VemsContext::cleanup();

    return 0;
}

void MyApp::HandleEvent(wxEvtHandler *handler, wxEventFunction func, wxEvent& event) const
{
    try {
        wxApp::HandleEvent(handler, func, event);
    }
    catch ( wxSQLite3Exception& e )
    {
        wxString msg;
//        int code = e.GetErrorCode();
//        int code2 = e.GetExtendedErrorCode();
//        msg += wxString::Format(_T("code: %i, msg=")+e.ErrorCodeAsString(code), code);
//        msg += wxString::Format(_T("\nextcode: %i, msg=")+e.ErrorCodeAsString(code2), code2)+_T("\n");
        msg += e.GetMessage();



        //int ret = wxMessageBox(wxT("Unhandled sqlite exception caught.\n\nMessage:\n\n")+msg+_T("\n\nAbort the program?"), wxT("VemsTune"), wxYES_NO  | wxICON_ERROR);
        wxSQLite3ExceptionMessageBox dlg(NULL,e);

        int ret = dlg.ShowModal();

        if(ret == wxYES) {
            wxExit();
        }
    }
    catch ( ... )
    {
        int ret = wxMessageBox(wxT("Unhandled exception caught. Abort the program?"), wxT("VemsTune"), wxYES_NO  | wxICON_ERROR);
        if(ret == wxYES) {
            wxExit();
        }
    }
}

/*
bool MyApp::OnExceptionInMainLoop()
{
    printf("OnExceptionInMainLoop\n");
    return false;
}

void MyApp::OnFatalException()
{
    printf("OnFatalException\n");
}

int MyApp::OnRun()
{
    int ret = 0;

    try {

        if ( m_exitOnFrameDelete == Later )
        {
            m_exitOnFrameDelete = Yes;
        }
        //else: it has been changed, assume the user knows what he is doing

        return MainLoop();

    } catch (...) {
        printf("OnRun catch\n");
        ret = 1;
    }

    return ret;
}

void MyApp::OnUnhandledException()
{
    printf("OnUnhandledException\n");
}
*/
wxFrame* MyApp::getMainFrame()
{
    return m_frame;
}

void MyApp::ShowSplashScreen()
{
    m_splashFrame = new wxFrame(NULL,-1,_T(""),wxDefaultPosition,wxDefaultSize,wxFRAME_NO_TASKBAR|wxBORDER_NONE);

    VemsSplashScreen* splashScreen = new VemsSplashScreen(m_splashFrame);

    m_splashFrame->SetBackgroundColour( wxColor(42,42,42) );
    m_splashFrame->ClearBackground();

    wxBoxSizer* sfsizer = new wxBoxSizer(wxVERTICAL);
    sfsizer->Add(splashScreen,0,wxALL,2);
    m_splashFrame->SetSizer(sfsizer);
    sfsizer->SetSizeHints(m_splashFrame);

    m_splashFrame->Centre();
    m_splashFrame->Show();

    splashScreen->Update();
    m_splashFrame->Update();
}
