#include "bcButton.h"

#include "../agg/wxAgg2.h"

#include "../DirHelper.h"
#include "../logger.h"

#include "../wxHSLColor.h"
#include "../wxImageHelper.h"
#include "BitmapCanvasParentBase.h"
#include "BitmapCanvasSystemSettings.h"

BEGIN_EVENT_TABLE(bcButton, BitmapCanvasWidgetBase)

    EVT_LEFT_DOWN(bcButton::OnLeftDown)
    EVT_LEFT_UP(bcButton::OnLeftUp)

    EVT_ENTER_WINDOW(bcButton::OnEnter)
    EVT_LEAVE_WINDOW(bcButton::OnLeave)

    //EVT_SET_FOCUS(bcButton::OnSetFocus)
    EVT_KILL_FOCUS(bcButton::OnKillFocus)

    EVT_KEY_DOWN(bcButton::OnKeyDown)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(bcButton, BitmapCanvasWidgetBase);

bcButton::bcButton(BitmapCanvasParentBase* parent, int id, const wxString& label, const wxPoint& pos, const wxSize& size, long style)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
{
    //ctor
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(80,18);

    m_imgPos = wxPoint(0,0);

    m_label = label;

    m_state = BCB_NORMAL;
    m_prev_state = BCB_NORMAL;

    m_textOffset = wxRealPoint(0,0);

    m_groupId = -1;

    DrawWidget();
}

bcButton::bcButton(BitmapCanvasParentBase* parent, int id, const wxImage& label, const wxPoint& pos, const wxSize& size, long style)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
{
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(80,18);

    m_style |= bcBUTTON_AUTOSIZE;

    m_imgPos = wxPoint(0,0);

    m_imgLabelStates[bcBUTTON_IMAGE_NORMAL] = label.Copy();
    m_label = wxEmptyString;

    m_state = BCB_NORMAL;
    m_prev_state = BCB_NORMAL;

    m_textOffset = wxRealPoint(0,0);

    m_groupId = -1;

    DrawWidget();
}

bcButton::bcButton(BitmapCanvasParentBase* parent, int id, const wxString& label, const wxImage& imglabel, const wxPoint& pos, const wxSize& size, long style)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
{
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(80,18);

    m_imgPos = wxPoint(0,0);

    m_imgLabelStates[bcBUTTON_IMAGE_NORMAL] = imglabel.Copy();
    m_label = label;

    m_state = BCB_NORMAL;
    m_prev_state = BCB_NORMAL;

    m_textOffset = wxRealPoint(0,0);

    m_groupId = -1;

    DrawWidget();
}

bcButton::~bcButton()
{
    //dtor
}
/// TODO: rethink and implement
void bcButton::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{

}

bool bcButton::IsStateChanged()
{
    return m_prev_state == m_state ? false : true;
}

void bcButton::SetGroupId(long id)
{
    m_groupId = id;
}

void bcButton::CheckGroupButtons(bool isCheckingSelf)
{
    if( !(m_style & bcBUTTON_TOGGLE) ) return;

    BitmapCanvasWidgetBaseArray arr;

    m_parent->GetWidgetList(arr);

    for(size_t a=0; a<arr.GetCount();a++){
        BitmapCanvasWidgetBase* widget = arr[a];

        if(widget == this) continue;

        if(!widget->IsKindOf(CLASSINFO(bcButton))) continue;

        bcButton* bcb = wxDynamicCast(widget,bcButton);

        if( !(bcb->GetStyle() & bcBUTTON_TOGGLE) ) continue;

        if(bcb->GetGroupId() == -1 || m_groupId != bcb->GetGroupId()) continue;

        if(isCheckingSelf && bcb->GetToggleValue()) {
            SetToggleValue(false);
            return;
        }

        bcb->SetToggleValue(false);
    }
}

void bcButton::OnKeyDown(wxKeyEvent& evt)
{
    if(evt.GetKeyCode() == WXK_RETURN) {
        DoPress(true);
    }
    else
    {
        m_parent->SetWidgetFocus(NULL);
        DropEvent(evt);
    }
}

void bcButton::OnSetFocus(wxFocusEvent& evt)
{
    LOG
}

void bcButton::OnKillFocus(wxFocusEvent& evt)
{
    Refresh();
}

void bcButton::OnLeftDown(wxMouseEvent& evt)
{
    bool press = false;

    if(m_style & bcBUTTON_TOGGLE){
        press = GetToggleValue();
        m_state ^= BCB_TOGGLED;
    }

    m_state |= BCB_CLICK_START;

    DoPress(press);
}

void bcButton::OnLeftUp(wxMouseEvent& evt)
{
    if(!(m_style & bcBUTTON_TOGGLE) &&  (m_state & BCB_PRESSED)) {
        m_state &= ~BCB_CLICK_START;
        DoPress(true);
    }
}

void bcButton::DoPress(bool isup)
{

    if(isup) {
        wxCommandEvent devt(wxEVT_COMMAND_BUTTON_CLICKED, GetId());
        DropEvent(devt);

        m_state &= ~BCB_PRESSED;

    } else {
        m_state |= BCB_PRESSED;
    }

    if(m_style & bcBUTTON_TOGGLE && m_groupId != -1) {

        CheckGroupButtons(false);

    }

    Refresh();
}

void bcButton::OnEnter(wxMouseEvent& evt)
{
    evt.Skip(1);

    // problem in toggle mode
    //if(m_state & BCB_CLICK_START) m_state |= BCB_PRESSED;

    if((m_style & bcBUTTON_BORDER_HIGHLIGHT) || (m_style & bcBUTTON_HOVER_OUTER_GLOW) ) {
        m_state |= BCB_HIGHLIGHTED;
    }

    if(IsStateChanged())
        Refresh();
}

void bcButton::OnLeave(wxMouseEvent& evt)
{
    evt.Skip(1);

    if(!(m_state & BCB_TOGGLED)) m_state &= ~BCB_PRESSED;

    if((m_style & bcBUTTON_BORDER_HIGHLIGHT) || (m_style & bcBUTTON_HOVER_OUTER_GLOW) )
        m_state &= ~BCB_HIGHLIGHTED;

    Refresh();
}

wxRect bcButton::GetTextBox()
{
    wxAgg2 aggCanvas(10,10);

    wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
    if(aggCanvas.loadTTFont(fontfile, GetSysDouble(BCBS_TEXT_HEIGHT), GetSysDouble(BCBS_TEXT_WEIGHT) ) == false) return wxRect(0,0,0,0);

    return aggCanvas.getTTFTextBound(m_label).GetOuterBox();
}

void bcButton::SetLabel(const wxString& label)
{
    m_label = label;
    Refresh();
}

void bcButton::SetLabel(const wxBitmap& label, bcButtonImageStates state, const wxPoint& pos)
{
    m_imgLabelStates[state] = label.ConvertToImage();
    if(pos != wxDefaultPosition) m_imgPos = pos;
    Refresh();
}

void bcButton::SetImgLabel(const wxImage& label, bcButtonImageStates state, const wxPoint& pos)
{
    m_imgLabelStates[state] = label.Copy();
    if(pos != wxDefaultPosition) m_imgPos = pos;
    Refresh();
}

void bcButton::SetToggleValue(bool state)
{
    if( !(m_style | bcBUTTON_TOGGLE) ) return;

    if(state == (m_state & BCB_TOGGLED)) return;

    m_state ^= BCB_TOGGLED;
    m_state ^= BCB_PRESSED;

    Refresh();
}

bool bcButton::GetToggleValue()
{
    return m_state & BCB_TOGGLED ? true : false;
}

void bcButton::SetBitmapPosition(const wxPoint& pos)
{
    m_imgPos = pos;
    Refresh();
}

void bcButton::SetTextOffset(double x, double y)
{
    m_textOffset.w = x;
    m_textOffset.h = y;
    Refresh();
}

wxRealSize bcButton::GetFitSize()
{
    bool isBitmap = IsBitmap();
    bool isText = IsText();

    wxRealSize size;

    if(isText)
        size = GetTextBox().GetSize();

    if(isBitmap) {
        wxRealSize imgSize;
        imgSize.w = m_imgPos.x + m_imgLabelStates[bcBUTTON_IMAGE_NORMAL].GetWidth()+1;
        imgSize.h = m_imgPos.y + m_imgLabelStates[bcBUTTON_IMAGE_NORMAL].GetHeight();
        size.w = wxMax(size.w,imgSize.w);
        size.h = wxMax(size.h,imgSize.h);
    }

//    Logger::logf(LOG_MSG,_T("size w=%f, h=%f"),size.w,size.h);

    double padding = GetSysDouble(BCBS_PADDDING);
    double border_width = GetSysDouble(BCBS_BORDER_WIDTH);

    size.IncBy(padding*2+border_width*2+1);

    if(m_style & bcBUTTON_TEXT_PUSH_AFTER_BITMAP_BYX)
        m_textOffset.w = m_imgLabelStates[bcBUTTON_IMAGE_NORMAL].GetWidth() + 2;

    size.IncBy(m_textOffset);

    if(m_style & bcBUTTON_ALIGN_CENTER_HORIZONTAL) size.IncBy(m_textOffset);

    return size;
}

bool bcButton::IsAutoSize()
{
    return m_style & bcBUTTON_AUTOSIZE ? true : false;
}

bool bcButton::IsBitmap()
{
    return (m_imgLabelStates[bcBUTTON_IMAGE_NORMAL].IsOk() && m_imgLabelStates[bcBUTTON_IMAGE_NORMAL].GetWidth() && m_imgLabelStates[bcBUTTON_IMAGE_NORMAL].GetHeight()) ? true : false;
}

bool bcButton::IsText()
{
    return (m_label == wxEmptyString || (m_style & bcBUTTON_NO_DRAW_TEXT_WHEN_BITMAP_LABEL && IsBitmap()) ) ? false : true;
}

wxString bcButton::GetState()
{
    wxString statestr;
    if(m_state == BCB_NORMAL)
        return _T("BCB_NORMAL");

    if(m_state & BCB_HIGHLIGHTED)
        statestr += _T("BCB_HIGHLIGHTED ");

    if(m_state & BCB_PRESSED)
        statestr += _T("BCB_PRESSED ");

    if(m_state & BCB_TOGGLED)
        statestr += _T("BCB_TOGGLED");

    return statestr;
}

void bcButton::DrawWidget()
{

    //Logger::logf(LOG_MSG,_T("m_imgLabelStates.size=%i, this=%p, state: ")+GetState(),m_imgLabelStates.size(),this);

// define flags
    bool isAutoSize = IsAutoSize();

    bool isBitmap = IsBitmap();

    bool isText = IsText();

    double padding = GetSysDouble(BCBS_PADDDING);
    double border_width = GetSysDouble(BCBS_BORDER_WIDTH);
    double blur_radius = 0.0;

    wxRealSize size;

// define widget size

    wxRealSize fitsize = GetFitSize();

    if((m_style & bcBUTTON_HOVER_OUTER_GLOW)) {
        blur_radius = GetPropDouble(BCBP_GLOW_RADIUS,3.0);
        fitsize.IncBy(blur_radius*2);
    }

    if(isAutoSize) {

        m_parent->AddRefreshRect( GetRect() );

        size = fitsize;

    } else {


        wxRealSize widgetsize = GetSize();
        size.w = wxMax(widgetsize.w,fitsize.w);
        size.h = wxMax(widgetsize.h,fitsize.h);
    }

    m_properties[BCWB_SIZE] << size;

    //LOGP(wxString::Format(_T("size w=%.1f,h=%.1f, m_label=")+m_label,size.w,size.h));

    double d = GetPropDouble(BCBP_GRADIENT_LIGHTNESS_DIFF,0.4);

    wxAgg2 aggCanvas(size.Floor());

    if(!aggCanvas.isOk()) return;

// define colors
    wxColor sbgcolor = GetSysColor(BCBS_BACKGROUND_COLOR);
    wxColor sfgcolor = GetSysColor(BCBS_FOREGROUND_COLOR);

    wxColor bgcolor = m_state & BCB_PRESSED ? sfgcolor : sbgcolor;
    wxColor fgcolor = m_state & BCB_PRESSED ? sbgcolor : sfgcolor;

    wxAggBrush* brush = aggCanvas.getBrush();
    brush->SetStyle( wxAGGBRUSH_GRADIENT );
    brush->SetColor( bgcolor );

// set 3D effect
    wxHSLColor bg_hsl(bgcolor);

    if(m_state & BCB_PRESSED)
        bg_hsl.m_lum += d;
    else
        bg_hsl.m_lum -= d;



// prepare face img
    wxImage faceImg;
    if(isBitmap){

        if((m_state&BCB_HIGHLIGHTED) && m_imgLabelStates.find(bcBUTTON_IMAGE_HOVER) != m_imgLabelStates.end())
            faceImg = m_imgLabelStates[bcBUTTON_IMAGE_HOVER];

        else if((m_state&BCB_PRESSED) && m_imgLabelStates.find(bcBUTTON_IMAGE_PRESSED) != m_imgLabelStates.end())
            faceImg = m_imgLabelStates[bcBUTTON_IMAGE_PRESSED];

        else
            faceImg = m_imgLabelStates[bcBUTTON_IMAGE_NORMAL];
    }

// prepare drawing when state is disabled
    if ( IsEnabled() == false )
    {
        bgcolor = wxColor( 190, 190, 190, 255 );
        fgcolor = wxColor( 140, 140, 140, 255 );
        bg_hsl.m_lum += 0.5 * d;

        if(isBitmap){
            if(m_imgLabelStates.find(bcBUTTON_IMAGE_DISABLED) == m_imgLabelStates.end()) {
                faceImg = m_imgLabelStates[bcBUTTON_IMAGE_NORMAL].ConvertToGreyscale();
                wxImageHelper::ShiftLuminance(faceImg,0.3);
            } else
                faceImg = m_imgLabelStates[bcBUTTON_IMAGE_DISABLED];
    }
    }

// prepare drawing
    chkminmax(bg_hsl.m_lum,0,1);

    brush->SetEndColor( bg_hsl.ToRGB() );
    brush->SetGradientOrientation( 90 );

    wxColor bordercolor = (m_state & BCB_HIGHLIGHTED && m_style & bcBUTTON_BORDER_HIGHLIGHT) ? GetSysColor(BCBS_BORDER_HIGHLIGHT_COLOR) : GetPropColor(BCBS_BORDER_COLOR,wxColor(80,80,80));

    wxAggPen * pen = aggCanvas.getPen();
    pen->SetColor( bordercolor );
    pen->SetWidth( border_width );

    double border_radius = GetSysDouble(BCBS_BORDER_RADIUS);

    if(m_style & bcBUTTON_NO_BORDER) pen->SetStyle(wxAGGPEN_TRANSPARENT);

    wxRealRect borderRect(border_width/2, border_width/2, size.w-border_width, size.h-border_width);

    if((m_style & bcBUTTON_HOVER_OUTER_GLOW)) {
        borderRect.Deflate(blur_radius);
        if(m_state & BCB_HIGHLIGHTED) {
            wxColor blur_color = GetPropColor(BCBP_GLOW_COLOR,wxColor(80,80,80));
            aggCanvas.setBlur( wxAggBlur( 0, blur_radius, blur_color) );
        }
    }

    borderRect.SubPixelNormalize(border_width);

// draw border
    aggCanvas.drawRoundedRectangle( borderRect, border_radius);

    aggCanvas.disableBlur();

//    if(IsFocused()) {
//        double dashes[2] = { 3,3 };
//        wxHSLColor fcHSL(bordercolor);
//        fcHSL.m_lum += 0.5;
//        chkminmax(fcHSL.m_lum,0,1);
//        pen->SetColor( fcHSL.ToRGB() );
//        pen->SetStyle( wxAGGPEN_USER_DASH );
//        pen->SetDashes(2,dashes);
//        aggCanvas.drawRoundedRectangle( borderRect.Deflate( border_width + 1 ), border_radius);
//        pen->SetStyle( wxAGGPEN_SOLID );
//    }

    wxRealRect lineBBox;
    wxRealPointEx linePos;

// prepare text drawing
    if(isText) {

        wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
        if(aggCanvas.loadTTFont(fontfile, GetSysDouble(BCBS_TEXT_HEIGHT), 0 ) == false) return;

// calculate text position
        lineBBox = aggCanvas.getTTFTextBound(m_label, 0,0,0);

        linePos.x = padding + border_width + blur_radius;

        if(m_style & bcBUTTON_ALIGN_CENTER_HORIZONTAL)
            linePos.x = (size.w - lineBBox.w)/2;

        if(m_style & bcBUTTON_ALIGN_RIGHT)
            linePos.x = lineBBox.w - padding - border_width;


        linePos.y = lineBBox.h + padding/2 + blur_radius;

        if(m_style & bcBUTTON_ALIGN_CENTER_VERTICAL)
            linePos.y = (size.h - lineBBox.h)/2 + lineBBox.h/2 + padding + border_width;

        if(m_style & bcBUTTON_ALIGN_BOTTOM)
            linePos.y = size.h - padding - border_width;

    }

    if(m_style & bcBUTTON_NO_BORDER) pen->SetStyle(wxAGGPEN_SOLID);

    wxPointEx imgPos = m_imgPos;
    linePos.IncBy(m_textOffset.w,m_textOffset.h);

// shift positions when the state is pressed
    if(m_state & BCB_PRESSED) {
        linePos.IncBy(0,1);
        imgPos.IncBy(0,1);
    }

    linePos = linePos.Ceil();

    imgPos.IncBy( (int)ceil(border_width + padding + blur_radius)+1,(int)ceil(border_width + padding+blur_radius) );

    wxRealPointEx gravenTextPos = linePos;
    gravenTextPos.IncBy( 1 );

    wxColor gravenColor = GetSysColor(BCBS_TEXT_GRAVEN_COLOR);

// text under bitmap
    if( (m_style & bcBUTTON_BITMAP_LABEL_ABOVE_TEXT_LABEL) && isText ) {
        if(m_style & bcBUTTON_TEXT_GRAVEN && !(m_state & BCB_PRESSED) ) {
            pen->SetColor(gravenColor);
            aggCanvas.drawTTFText(gravenTextPos.x, gravenTextPos.y, m_label,0);
        }
        pen->SetColor( fgcolor );
        aggCanvas.drawTTFText(linePos.x, linePos.y, m_label,0);

    }

    if(isBitmap && faceImg.IsOk())
        aggCanvas.drawImage(&faceImg, imgPos);

// text above bitmap
    if( !(m_style & bcBUTTON_BITMAP_LABEL_ABOVE_TEXT_LABEL) && isText ) {
        if(m_style & bcBUTTON_TEXT_GRAVEN && !(m_state & BCB_PRESSED) ) {
            pen->SetColor(gravenColor);
            aggCanvas.drawTTFText(gravenTextPos.x, gravenTextPos.y, m_label,0);
        }
        pen->SetColor( fgcolor );
        aggCanvas.drawTTFText(linePos.x, linePos.y, m_label,0);
    }


    render(aggCanvas);

    m_prev_state = m_state;
}
