#include "bcLabel.h"

#include <wx/tokenzr.h>

#include "BitmapCanvasParentBase.h"

#include "../agg/wxAgg2.h"
#include "../DirHelper.h"
#include "../logger.h"
#include "../wxRealPointEx.h"
#include "../wxSizeEx.h"
#include "../common/types/Uint8tHash.h"

const TextDesc DefaultTextDesc = TextDesc(wxEmptyString,0,wxALIGN_LEFT,12.0,wxRealPoint(0,0),wxColor(0,0,0,255),0.0,0.0,_T("arial.ttf"));


IMPLEMENT_DYNAMIC_CLASS(bcLabel, BitmapCanvasWidgetBase);

bcLabel::bcLabel()
{

}

bcLabel::bcLabel(BitmapCanvasParentBase* parent, int id, const TextDesc& desc, const wxPoint& pos, const wxSize& size, long style, bool isPreDraw)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
{
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(100,100);

    m_labels[desc.Level] = desc;

    if(isPreDraw)
        DrawWidget();
}

bcLabel::bcLabel(BitmapCanvasParentBase* parent, int id, const wxString& text, const wxPoint& pos, const wxSize& size, long style, bool isPreDraw)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
{
    //ctor
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(100,100);

    m_labels[0] = TextDesc(text,0);

    if(isPreDraw)
    DrawWidget();

}

bcLabel::bcLabel(BitmapCanvasParentBase* parent, const wxString& text, wxVariantIntHash& properties, bool isPreDraw)
{
    CreateBcWidget(parent,properties);
    m_labels[0] = TextDesc(text,0);

    if(isPreDraw)
    DrawWidget();
}

bcLabel::~bcLabel()
{
    //dtor
}
/// TODO: rethink and implement
void bcLabel::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{

}

bool bcLabel::Focusable()
{
    return false;
}

void bcLabel::WriteLabel(TextDesc desc)
{
    ChangeLabel(desc);
    DrawWidget();
}

void bcLabel::WriteLabel(const wxString& label, uint8_t level)
{
    WriteLabel(TextDesc(label,level));
}

void bcLabel::ChangeLabel(TextDesc desc)
{
    if(desc.Text == wxEmptyString) m_labels.erase(desc.Level);

    else m_labels[desc.Level] = desc;
}

void bcLabel::ChangeLabel(const wxString& label, uint8_t level)
{
    ChangeLabel(TextDesc(label,level));
}

void bcLabel::SetLabel(TextDesc desc)
{
    if(desc.Text == wxEmptyString) m_labels.erase(desc.Level);

    else m_labels[desc.Level] = desc;

    Refresh();
}

void bcLabel::SetLabel(const wxString& label, uint8_t level)
{
    SetLabel(TextDesc(label,level));
}

TextDesc bcLabel::GetLabelDesc(uint8_t level)
{
    if( m_labels.find(level) == m_labels.end() ) return DefaultTextDesc;

    return m_labels[level];
}

wxString bcLabel::GetLabel(uint8_t level)
{
    if(m_labels.find(level) == m_labels.end()) return wxEmptyString;

    return m_labels[level].Text;
}

wxRect bcLabel::GetTextBox(TextDesc& desc)
{
    wxAgg2 aggCanvas(10,10);

    wxString fontfile = DirHelper::GetPath(dFONTS) + desc.Fontfile;
    if(aggCanvas.loadTTFont(fontfile, desc.Height, desc.Weight ) == false) return wxRect(0,0,0,0);

    wxArrayString lines = wxStringTokenize(desc.Text,_T("\n"),wxTOKEN_RET_EMPTY_ALL);

    wxRealRect box;
    wxRealRect crect(0,0,0,0);

    for(size_t a = 0; a < lines.GetCount(); a++) {
        crect = aggCanvas.getTTFTextBound(lines[a], desc.Angle, desc.Position.x, desc.Position.y + crect.h * a );
        box.Union(crect);
    }

    return box.GetOuterBox();
}

wxRect bcLabel::GetTextBox(int level)
{
    wxRect box(0,0,0,0);

    if(level == -1) {

        for(size_t a = 0; a < 256; a++) {

            if( m_labels.find(a) == m_labels.end() ) continue;

            TextDesc desc = m_labels[a];

            box.Union( GetTextBox(desc) );
        }

    } else {
        if(m_labels.find(level) != m_labels.end()) {
            box.Union( GetTextBox(m_labels[level]) );
        }
    }

    return box.GetSize();
}

void bcLabel::DrawWidget()
{

    int shadowSizeDiff = m_style & BCLS_DROPSHADOW ? 3 : 0;
    wxColor shadowColor(0,0,0,50);

    double padding = GetSysDouble(BCBS_PADDDING);
    double border_width = GetSysDouble(BCBS_BORDER_WIDTH);

    wxRealSize size;

    if(m_style & BCLS_AUTO_SIZE) {
        m_parent->AddRefreshRect( GetRect() );
        size = GetTextBox().GetSize();
        size.IncBy(padding*2+border_width*2);
        if(m_style & BCLS_DROPSHADOW)
            size.IncBy(shadowSizeDiff);
        m_properties[BCWB_SIZE] << size;
    } else
        size = GetSize();

    wxAgg2 aggCanvas(size.Ceil());

    if(!aggCanvas.isOk()) return;

    wxAggBrush* brush = aggCanvas.getBrush();
    brush->SetStyle( wxAGGBRUSH_SOLID );


    wxAggPen * pen = aggCanvas.getPen();

    pen->SetWidth( border_width );

    double border_radius = GetSysDouble(BCBS_BORDER_RADIUS);

    wxRealRect border_rect(border_width/2, border_width/2, size.w-border_width-shadowSizeDiff, size.h-border_width-shadowSizeDiff);

    if(border_width < 0.001)
        pen->SetStyle(wxAGGPEN_TRANSPARENT);

    if(!(m_style & BCLS_TRANSPARENT_BG)) {
        if(m_style & BCLS_DROPSHADOW) {
            wxRealRect shadowRect = border_rect;
            shadowRect.Offset(shadowSizeDiff,shadowSizeDiff);
            pen->SetColor(shadowColor);
            brush->SetColor(shadowColor);
            aggCanvas.drawRoundedRectangle( shadowRect, border_radius);
        }
        pen->SetColor( GetSysColor(BCBS_BORDER_COLOR) );
        brush->SetColor( GetSysColor(BCBS_BACKGROUND_COLOR) );
        aggCanvas.drawRoundedRectangle( border_rect, border_radius);
    }

    pen->SetStyle(wxAGGPEN_SOLID);

    wxColor gravenColor = GetSysColor(BCBS_TEXT_GRAVEN_COLOR);
    wxColor shadowTextColor = GetSysColor(BCBS_SHADOW_COLOR);

    for(size_t a = 0; a < 256; a++) {

        if( m_labels.find(a) == m_labels.end() ) continue;

        TextDesc desc = m_labels[a];

        wxString fontfile = DirHelper::GetPath(dFONTS) + desc.Fontfile;
        if(aggCanvas.loadTTFont(fontfile, desc.Height, desc.Weight ) == false) continue;

        wxArrayString lines = wxStringTokenize(desc.Text,_T("\n"),wxTOKEN_RET_EMPTY_ALL);

        for(size_t a = 0; a < lines.GetCount(); a++) {

            if(lines[a].Len() < 1) continue;

            wxRealRect lineBBox = aggCanvas.getTTFTextBound(lines[a], desc.Angle,0,0);

            wxRealPoint linePos;

            linePos.x = desc.Position.x + padding + border_width;

            if(desc.Align & wxALIGN_CENTER)
                linePos.x = desc.Position.x + (size.w - lineBBox.w)/2;

            if(desc.Align & wxALIGN_RIGHT)
                linePos.x = (fabs(desc.Position.x) < 0.001 ? size.w : desc.Position.x) - lineBBox.w - padding - border_width;

            linePos.y = desc.Position.y + lineBBox.h * (a+1) + padding/2;

            lineBBox.SetTopLeft(linePos.y-lineBBox.h,linePos.x);

            if(desc.Style & bcLabel_GRAVEN) {
                pen->SetColor( gravenColor );
                aggCanvas.drawTTFText(linePos.x+1, linePos.y+1, lines[a],desc.Angle);
            }

            if(desc.Style & bcLabel_SHADOW) {
                pen->SetColor( shadowTextColor );
                aggCanvas.drawTTFText(linePos.x+1, linePos.y+1, lines[a],desc.Angle);
            }

            pen->SetColor( desc.Color );

            aggCanvas.drawTTFText(linePos.x, linePos.y, lines[a],desc.Angle);

            if(desc.Style & bcLabel_UNDERLINED)
                aggCanvas.drawLine(lineBBox.GetLeft(), ceil(lineBBox.GetBottom())+1.5, lineBBox.GetRight(), ceil(lineBBox.GetBottom())+1.5 );

            if(desc.Style & bcLabel_STRIKE)
                aggCanvas.drawLine(lineBBox.GetLeft(), ceil(lineBBox.GetCenter().y)+1.5, lineBBox.GetRight(), ceil(lineBBox.GetCenter().y)+1.5 );

        }

    }

    render(aggCanvas);
}
