#include "bcLinkLabel.h"

#include "events/bcLinkEvent.h"
#include "../vemsdialog/WindowTipperAgg.h"

#include "../logger.h"

BEGIN_EVENT_TABLE(bcLinkLabel, bcLabel)

    EVT_ENTER_WINDOW(bcLinkLabel::OnEnter)
    EVT_LEAVE_WINDOW(bcLinkLabel::OnLeave)

    EVT_LEFT_UP(bcLinkLabel::OnLeftUp)

END_EVENT_TABLE()


bcLinkLabel::bcLinkLabel(BitmapCanvasParentBase* parent, const bcLinkInfo& info, int id, const wxPoint& pos, const wxSize& size)
    :bcLabel(parent,id,_T(""),pos,size,BCLS_DEFAULT,false)
{
    //ctor

    m_autoJumpURL = true;
    m_isSettedToolTip = false;
    m_linkInfo = info;

    m_normalDesc = GetLabelDesc();
    m_normalDesc.Text = info.GetLabel();
    m_normalDesc.Style = bcLabel_UNDERLINED;
    m_normalDesc.Color = GetSysColor(BCBS_VEMS_COLOR);

    m_hoverDesc = m_normalDesc;
    m_hoverDesc.Color = GetSysColor(BCBS_SELECTION_COLOR);

    SetLabel(m_normalDesc);
}

bcLinkLabel::~bcLinkLabel()
{
    //dtor
}

void bcLinkLabel::SetHoverColor(const wxColor& color)
{
    m_hoverDesc.Color = color;
    Refresh();
}

void bcLinkLabel::SetLinkInfo(const bcLinkInfo& info)
{
    m_linkInfo = info;
    m_normalDesc.Text = info.GetLabel();
    m_hoverDesc.Text = info.GetLabel();
    Refresh();
}

void bcLinkLabel::OnEnter(wxMouseEvent& evt)
{

    SetCursor(wxCURSOR_HAND);

    SetLabel(m_hoverDesc);

    if(!m_isSettedToolTip && m_linkInfo.Title != wxEmptyString) {
        SetToolTip(m_linkInfo.Title, true);
        m_isSettedToolTip = true;
    }

    evt.Skip();
}

void bcLinkLabel::OnLeave(wxMouseEvent& evt)
{
    SetCursor(wxNullCursor);

    SetLabel(m_normalDesc);

    evt.Skip();
}

void bcLinkLabel::OnLeftUp(wxMouseEvent& evt)
{
    WindowTipperAgg* tt = GetToolTip();
    if(tt) tt->HideWin();

    SetCursor(wxNullCursor);

    if(m_autoJumpURL && m_linkInfo.isHyperLink())
        wxLaunchDefaultBrowser(m_linkInfo.HRef);

    SetLabel(m_normalDesc);

    bcLinkEvent linkevt(m_linkInfo, GetId());
    AddPendingEvent(linkevt);
}

/*
    //EVT_MOTION(bcLinkLabel::OnMouseMove)

    for(TextDescHash::iterator it = m_labels.begin(); it != m_labels.end(); it++ ) {
        TextDesc& desc = it->second;
        if(desc.OverCursor == -1) continue;

        desc.m_isChangedOver = false;
    }

    Refresh();

    for(TextDescHash::iterator it = m_labels.begin(); it != m_labels.end(); it++ ) {
        TextDesc& desc = it->second;
        if(desc.OverCursor == -1) continue;
        wxRealRect r = GetTextBox(desc);
        if(r.Contains(evt.GetX(),evt.GetY())) {
            if(!desc.m_isChangedOver) {
                SetCursor(desc.OverCursor);
                desc.m_isChangedOver = true;
                Refresh();
            }
            //evt.Skip();
            //return;
        } else {
            if(desc.m_isChangedOver) {
                desc.m_isChangedOver = false;
                Refresh();
            }
        }
    }



    OverCursor = -1;
    OverColor = Color;
    m_isChangedOver = false;

int         OverCursor;
wxColor     OverColor;

bool        m_isChangedOver;

*/
