#include "bcRadioButton.h"

#include "../agg/wxAgg2.h"

#include "../DirHelper.h"
#include "../logger.h"

#include "../DrawHelper.h"
#include "BitmapCanvasParentBase.h"
#include "BitmapCanvasSystemSettings.h"

BEGIN_EVENT_TABLE(bcRadioButton, BitmapCanvasWidgetBase)

    EVT_LEFT_DOWN(bcRadioButton::OnLeftDown)
    EVT_LEFT_UP(bcRadioButton::OnLeftUp)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(bcRadioButton, BitmapCanvasWidgetBase);

bcRadioButton::bcRadioButton(BitmapCanvasParentBase* parent, int id, const wxString& text, int groupID, const wxPoint& pos, const wxSize& size, long style)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
{
    //ctor
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(100,18);

    m_state = BCCB_NORMAL;
    m_label = text;
    m_style = style;

    m_grpId = groupID;

    DrawWidget();

}

bcRadioButton::~bcRadioButton()
{
    //dtor
}
/// TODO: rethink and implement
void bcRadioButton::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{

}

void bcRadioButton::SetGroupId(int grpid)
{
    m_grpId = grpid;
}

void bcRadioButton::CheckGroupButtons(bool isCheckingSelf)
{
    BitmapCanvasWidgetBaseArray arr;

    m_parent->GetWidgetList(arr);

    for(size_t a=0; a<arr.GetCount();a++){
        BitmapCanvasWidgetBase* widget = arr[a];

        if(widget == this) continue;

        if(!widget->IsKindOf(CLASSINFO(bcRadioButton))) continue;

        bcRadioButton* rb = wxDynamicCast(widget,bcRadioButton);

        if(m_grpId != rb->GetGroupId()) continue;

        if(isCheckingSelf && rb->GetValue()) {
            SetValue(false);
            return;
        }

        rb->SetValue(false);
    }
}

void bcRadioButton::OnLeftDown(wxMouseEvent& evt)
{
    m_state |= BCCB_HOVER;

    Refresh();
}

void bcRadioButton::OnLeftUp(wxMouseEvent& evt)
{
    m_state &= ~BCCB_HOVER;

    if(!( !(m_style & BCRBS_CANOFF) && m_state & BCCB_CHECKED)) {
        m_state ^= BCCB_CHECKED;
        CheckGroupButtons(false);
    }

    Refresh();
}

void bcRadioButton::SetValue(bool isChecked)
{
    bool oval = GetValue();

    if(oval == isChecked) return;

    m_state ^= BCCB_CHECKED;

    Refresh();
}

bool bcRadioButton::GetValue()
{
    return m_state & BCCB_CHECKED ? true : false;
}

void bcRadioButton::DrawWidget()
{

    wxRealSize size = GetSize();

    wxAgg2 aggCanvas(size.Ceil());

    wxAggPen* pen = aggCanvas.getPen();
    wxAggBrush* brush = aggCanvas.getBrush();

    double padding = 2;
    double box_border = 1;


    wxRealRect box( padding + box_border/2, padding + box_border/2, 0, size.h - 2*padding - box_border );
    box.w = box.h;

    pen->SetColor( GetSysColor(BCBS_FOREGROUND_COLOR) );
    pen->SetWidth( box_border );
    brush->SetStyle(wxAGGBRUSH_GRADIENT);

    double bg_grad_diff = -0.3;

    wxColor bgcolor  = GetSysColor( BCBS_BACKGROUND_COLOR );
    wxColor bgcolor2 = DrawHelper::ModLuminance( bgcolor, bg_grad_diff  );

    double gradori = (m_style & BCRBS_INVERTED3D) ? 270 : 90;

    brush->SetColor(bgcolor);
    brush->SetEndColor(bgcolor2);
    brush->SetGradientOrientation( m_state & BCCB_HOVER ? gradori : gradori+180);

    aggCanvas.drawCircle( box.GetCenter(), box.w/2 );

    if(m_state & BCCB_CHECKED) {

        pen->SetStyle( wxAGGPEN_TRANSPARENT );

        wxColor pointcolor  = GetSysColor( BCBS_DYNAMIC_CONTENT_COLOR );
        wxColor pointcolor2 = DrawHelper::ModLuminance( pointcolor, -0.2 );

        brush->SetColor(pointcolor);
        brush->SetEndColor(pointcolor2);

        brush->SetGradientOrientation( m_state & BCCB_HOVER ? gradori-180 : gradori);

        aggCanvas.drawCircle( box.GetCenter(), box.w/4 );

    }

    pen->SetStyle( wxAGGPEN_SOLID );
    pen->SetColor( GetSysColor(BCBS_TEXT_COLOR) );

    wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
    if(aggCanvas.loadTTFont(fontfile, GetSysDouble(BCBS_TEXT_HEIGHT), 0 ) == false) return;

    wxRealRect textbox = aggCanvas.getTTFTextBound(m_label);

    wxRealPoint textpos(box.GetRight() + 2*padding, size.h - (size.h-textbox.h)/2 - padding);

    if(m_style & BCRBS_TEXT_GRAVEN) {
        pen->SetColor( GetSysColor(BCBS_TEXT_GRAVEN_COLOR) );
        aggCanvas.drawTTFText(m_label, textpos.x+1,textpos.y+1 );
    }

    pen->SetColor( GetSysColor(BCBS_TEXT_COLOR) );
    aggCanvas.drawTTFText(m_label, textpos.x,textpos.y );

    render(aggCanvas);
}
