#include "bcScrollBar.h"

#include "../agg/wxAgg2.h"
#include "../wxHSLColor.h"

#include "../logger.h"

int idbcScrollBarTimer = wxNewId();

BEGIN_EVENT_TABLE( bcScrollBar, BitmapCanvasWidgetBase )

    // EVT_KEY_DOWN( bcScrollBar::OnKeyDown )
    EVT_MOUSE_EVENTS( bcScrollBar::OnMouse )

    EVT_TIMER( idbcScrollBarTimer, bcScrollBar::OnTimer )

END_EVENT_TABLE()

bcScrollBar::bcScrollBar( BitmapCanvasParentBase* parent, int winid, const wxPoint& pos, const wxSize& size, long style ) : m_timer( this, idbcScrollBarTimer )
{
    Create( parent, winid, pos, size, style );
}

bcScrollBar::~bcScrollBar()
{
    // nop
}
/// TODO: rethink and implement
void bcScrollBar::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{

}

void bcScrollBar::Create( BitmapCanvasParentBase* parent, int winid, const wxPoint& pos, const wxSize& size, long style )
{
    CreateBcWidget( parent, winid, pos, size );
    m_type = ( style == wxSB_VERTICAL ) ? style : wxSB_HORIZONTAL;

    m_range = 10;
    m_pagesize = 1;
    m_thumbpos = 0;

    m_decIsPressed = false;
    m_incIsPressed = false;
    m_clickedOnBar = false;
    m_isDragging = false;

    m_clickState = 0;

    DrawWidget();
}

void bcScrollBar::DrawWidget()
{
    wxRealSize size = GetSize();

    double w = size.w, h = size.h;

    m_buttonSize = ( w > h ) ? h : w;

    wxAgg2 canvas( size.Floor() );

    if( !canvas.isOk() )
        return;

    wxColor barColor = GetPropColor( BCSB_BARCOLOR, wxColor( 148, 148, 148, 255 ) );
    wxColor thumbColor = GetPropColor( BCSB_THUMBCOLOR, wxColor( 0, 0, 0, 255 ) );
    wxColor arrowColor = GetPropColor( BCSB_ARROWCOLOR, wxColor( 224, 224, 224, 255 ) );

    //
    if ( IsEnabled() == false )
    {
        barColor = wxColor( 190, 190, 190, 255 );
        thumbColor = wxColor( 210, 210, 210, 255 );
        arrowColor = wxColor( 190, 190, 190, 255 );
    }

    double d = 0.4;

    wxHSLColor bg_hsl( thumbColor );
    if ( m_decIsPressed )
        bg_hsl.m_lum -= d;
    else
        bg_hsl.m_lum += d;

    double orient = 90;

    wxAggBrush* brush = canvas.getBrush();
    brush->SetStyle( wxAGGBRUSH_GRADIENT );

    if ( m_decIsPressed )
    {
        brush->SetColor( bg_hsl.ToRGB() );
        brush->SetEndColor( thumbColor );
    }
    else
    {
        brush->SetColor( thumbColor );
        brush->SetEndColor( bg_hsl.ToRGB() );
    }
    brush->SetGradientOrientation( orient );

    wxColor bordercolor = wxColor( 255, 255, 255, 255 );
    wxAggPen * pen = canvas.getPen();
    pen->SetColor( bordercolor ); pen->SetWidth( 1.0 );

    /// draw up/left button
    double border_radius = 2.0;

    m_barSize = ( m_type == wxSB_HORIZONTAL ) ? w - 2 * m_buttonSize : h - 2 * m_buttonSize;

    canvas.drawRoundedRectangle( 0, 0, m_buttonSize, m_buttonSize, border_radius );
    /// draw arrow
    if ( m_type == wxSB_HORIZONTAL )
    {
        RealPointArray arr;
        wxRealPoint p; p.x = 0.33 * m_buttonSize; p.y = 0.5 * m_buttonSize; arr.Add( p );
        p.x = 0.66 * m_buttonSize; p.y = 0.33 * m_buttonSize; arr.Add( p );
        p.x = 0.66 * m_buttonSize; p.y = 0.66 * m_buttonSize; arr.Add( p );
        brush = canvas.getBrush(); brush->SetStyle( wxAGGBRUSH_SOLID ); brush->SetColor( arrowColor );
        pen->SetColor( arrowColor );
        canvas.drawPolygon( arr );
    }
    else
    {
        RealPointArray arr;
        wxRealPoint p; p.x = 0.5 * m_buttonSize; p.y = 0.33 * m_buttonSize; arr.Add( p );
        p.x = 0.33 * m_buttonSize; p.y = 0.66 * m_buttonSize; arr.Add( p );
        p.x = 0.66 * m_buttonSize; p.y = 0.66 * m_buttonSize; arr.Add( p );
        brush = canvas.getBrush(); brush->SetStyle( wxAGGBRUSH_SOLID ); brush->SetColor( arrowColor );
        pen->SetColor( arrowColor );
        canvas.drawPolygon( arr );
    }

    /// draw the bar
    brush->SetColor( barColor ); pen->SetColor( barColor );

    if ( m_type == wxSB_HORIZONTAL )
        canvas.drawRoundedRectangle( m_buttonSize, 0, m_barSize, m_buttonSize, border_radius );
    else
        canvas.drawRoundedRectangle( 0, m_buttonSize, m_buttonSize, m_barSize, border_radius );

    /// draw right/down

    brush->SetStyle( wxAGGBRUSH_GRADIENT );

    bg_hsl.FromRGB( thumbColor );

    if ( m_incIsPressed )
        bg_hsl.m_lum -= d;
    else
        bg_hsl.m_lum += d;

    if ( m_incIsPressed )
    {
        brush->SetColor( bg_hsl.ToRGB() );
        brush->SetEndColor( thumbColor );
    }
    else
    {
        brush->SetColor( thumbColor );
        brush->SetEndColor( bg_hsl.ToRGB() );
    }
    brush->SetGradientOrientation( orient );
    pen->SetColor( bordercolor ); pen->SetWidth( 1.0 );

    if ( m_type == wxSB_HORIZONTAL )
        canvas.drawRoundedRectangle( w - m_buttonSize, 0, m_buttonSize, m_buttonSize, border_radius );
    else
        canvas.drawRoundedRectangle( 0, h - m_buttonSize, m_buttonSize, m_buttonSize, border_radius );

    /// draw arrow
    if ( m_type == wxSB_HORIZONTAL )
    {
        RealPointArray arr;
        wxRealPoint p; p.x = w - 0.33 * m_buttonSize; p.y = 0.5 * m_buttonSize; arr.Add( p );
        p.x = w - 0.66 * m_buttonSize; p.y = 0.33 * m_buttonSize; arr.Add( p );
        p.x = w - 0.66 * m_buttonSize; p.y = 0.66 * m_buttonSize; arr.Add( p );
        brush = canvas.getBrush(); brush->SetStyle( wxAGGBRUSH_SOLID ); brush->SetColor( arrowColor );
        pen->SetColor( arrowColor );
        canvas.drawPolygon( arr );
    }
    else
    {
        RealPointArray arr;
        wxRealPoint p; p.x = 0.5 * m_buttonSize; p.y = h - 0.33 * m_buttonSize; arr.Add( p );
        p.x = 0.33 * m_buttonSize; p.y = h - 0.66 * m_buttonSize; arr.Add( p );
        p.x = 0.66 * m_buttonSize; p.y = h - 0.66 * m_buttonSize; arr.Add( p );
        brush = canvas.getBrush(); brush->SetStyle( wxAGGBRUSH_SOLID ); brush->SetColor( arrowColor );
        pen->SetColor( arrowColor );
        canvas.drawPolygon( arr );
    }

    /// draw thumb
    m_thumbsize = (int)( m_barSize / ( (double)m_range / (double)m_pagesize ) );

    //Logger::logf(LOG_MSG, _T("bar=%f, range=%f, page=%f, thum=%f"), m_barSize, (double)m_range, (double)m_pagesize, (double) m_thumbsize);

    /// minimal width / height
    if ( m_thumbsize < 6 )
        m_thumbsize = 6;

    double pos = (double)m_thumbpos / (double)m_range * m_barSize - m_thumbsize / 2;

    double from = pos; //pos - m_thumbsize / 2;
    double to = pos + m_thumbsize ;

    if ( from < 0 )
    {
         from = 0;
         to = m_thumbsize;
    }
    if ( to >= m_barSize )
    {
        to = m_barSize;
        from = to - m_thumbsize;
    }


    //Logger::logf(LOG_MSG, _T("from=%f, to =%f"), from, to);

    brush->SetColor( thumbColor );

    if ( m_type == wxSB_HORIZONTAL )
    {
        canvas.drawRoundedRectangle( from + m_buttonSize, 0, to - from, m_buttonSize, border_radius );

        m_thumbDrawSize.x = from + m_buttonSize;
        m_thumbDrawSize.y = 0;
        m_thumbDrawSize.w = to - from;
        m_thumbDrawSize.h = m_buttonSize;
    }
    else
    {
        canvas.drawRoundedRectangle( 0, from + m_buttonSize, m_buttonSize, to - from, border_radius );

        m_thumbDrawSize.x = 0;
        m_thumbDrawSize.y = from + m_buttonSize;
        m_thumbDrawSize.w = m_buttonSize;
        m_thumbDrawSize.h = to - from;
    }
    /// draw || lines into thumb
    double middle = m_buttonSize + from + m_thumbsize / 2.0;

    pen->SetColor( arrowColor ); pen->SetWidth( 0.8 );
    if ( m_type == wxSB_HORIZONTAL )
    {
        canvas.drawLine( middle - 1.3, m_buttonSize*0.33, middle - 1.3, m_buttonSize * 0.66 );
        canvas.drawLine( middle + 1.3, m_buttonSize*0.33, middle + 1.3, m_buttonSize * 0.66 );
    }
    else
    {
        canvas.drawLine( m_buttonSize*0.33, middle - 1.3, m_buttonSize*0.66, middle - 1.3 );
        canvas.drawLine( m_buttonSize*0.33, middle + 1.3, m_buttonSize*0.66, middle + 1.3 );
    }

    render( canvas );
}

int bcScrollBar::GetRange() const
{
    return m_range;
}

int bcScrollBar::GetPageSize() const
{
    return m_pagesize;
}

int bcScrollBar::GetThumbPosition() const
{
    return m_thumbpos;
}

int bcScrollBar::GetThumbSize() const
{
    return m_thumbsize;
}

void bcScrollBar::SetThumbPosition( int start )
{
    SetScrollbar( start, m_thumbsize, m_range, m_pagesize );
}

void bcScrollBar::SetScrollbar( int position, int WXUNUSED( thumbsize ), int range, int pagesize )
{
    m_thumbpos = position;
    // m_thumbsize = thumbsize; // will be calculated from size & range & pagesize
    m_range = range;
    m_pagesize = pagesize;

    DrawWidget();
}

/// events
/*
void bcScrollBar::OnKeyDown( wxKeyEvent& event )
{
    if ( !IsEnabled() )
    {
        event.Skip();
        return;
    }
    /// TODO
}
*/

void bcScrollBar::OnMouse( wxMouseEvent& event )
{
    if ( !IsEnabled() )
    {
        event.Skip();
        return;
    }

    wxPoint pt = event.GetPosition();

    int orient = ( m_type == wxSB_HORIZONTAL ) ? wxHORIZONTAL : wxVERTICAL;

    ///
    /// clicking on decrement button (left / up)
    ///
    wxRealRect r( 0, 0, m_buttonSize, m_buttonSize );
    if ( r.Contains( pt.x, pt.y ) )
    {
        if ( ( event.LeftDown() ) || ( m_decIsPressed && event.LeftIsDown() ) )
        {
            m_decIsPressed = true;

            --m_thumbpos;

            if ( m_thumbpos < 0 )
                m_thumbpos = 0;
            else
            {
                Refresh();
                if ( m_thumbpos != 0 )
                {
                    wxScrollEvent evt( wxEVT_SCROLL_LINEUP, GetId(), m_thumbpos, orient );
                    DropEvent( evt );
                }
                else
                {
                    wxScrollEvent evt( wxEVT_SCROLL_TOP, GetId(), 0, orient );
                    DropEvent( evt );
                }
            }

            event.Skip();

            if ( m_timer.IsRunning() == false )
            {
                m_timer.Start( 300 );
            }
            else
            {
                m_timer.Start( 40 );
            }

            return;
        }
        if ( event.LeftUp() )
        {
            if ( m_timer.IsRunning() )
            {
                m_timer.Stop();
            }

            m_decIsPressed = false;
            Refresh();
        }
        return;
    }
    else
    {
        if ( m_decIsPressed && m_timer.IsRunning() )
        {
            m_timer.Stop();
        }
    }

    ///
    /// clicking on increment button ( right / down )
    ///
    if ( m_type == wxSB_HORIZONTAL )
    {
        r.x = m_barSize + m_buttonSize;
        r.y = 0;
    }
    else
    {
        r.x = 0;
        r.y = m_barSize + m_buttonSize;
    }
    if ( r.Contains( pt.x, pt.y ) )
    {
        if ( ( event.LeftDown() ) || ( m_incIsPressed && event.LeftIsDown() ) )
        {
            m_incIsPressed = true;

            ++m_thumbpos;

            if ( m_thumbpos >= m_range )
                m_thumbpos = m_range - 1;
            else
            {
                Refresh();
                if ( m_thumbpos < m_range )
                {
                    wxScrollEvent evt( wxEVT_SCROLL_LINEDOWN, GetId(), m_thumbpos, orient );
                    DropEvent( evt );
                }
                else
                {
                    wxScrollEvent evt( wxEVT_SCROLL_BOTTOM, GetId(), m_range - 1, orient );
                    DropEvent( evt );
                }
            }

            event.Skip();

            if ( m_timer.IsRunning() == false )
            {
                m_timer.Start( 300 );
            }
            else
            {
                m_timer.Start( 40 );
            }


            return;
        }
        if ( event.LeftUp() )
        {
            if ( m_timer.IsRunning() )
            {
                m_timer.Stop();
            }

            m_incIsPressed = false;
            Refresh();
        }

        return;
    }
    else
    {
        if ( m_incIsPressed && m_timer.IsRunning() )
        {
            m_timer.Stop();
        }
    }

    ///
    /// clicking on the thumb & dragging
    ///
    static wxPoint sPoint;
    if ( m_thumbDrawSize.Contains( pt.x, pt.y ) )
    {
        /// TODO
        if ( event.LeftDown() && !m_isDragging)
        {
            m_isDragging = true;
            sPoint = event.GetPosition();
            return;
        }
        if ( event.LeftUp() )
        {
            m_isDragging = false;
            wxScrollEvent evt( wxEVT_SCROLL_THUMBRELEASE, GetId(), m_thumbpos, orient );
            DropEvent( evt );
        }
        return;
    }

    if ( m_isDragging && event.Dragging() )
    {
        wxPoint cPoint = event.GetPosition();
        if ( m_type == wxSB_HORIZONTAL )
            m_thumbpos = (int)( ( cPoint.x - m_buttonSize ) / m_barSize * m_range );
        else
            m_thumbpos = (int)( ( cPoint.y - m_buttonSize ) / m_barSize * m_range );

        sPoint = cPoint;

        wxScrollEvent evt( wxEVT_SCROLL_THUMBTRACK, GetId(), m_thumbpos, orient );
        DropEvent( evt );

        Refresh();
        return;
    }


    ///
    /// clicking on the bar
    ///
    if ( m_type == wxSB_HORIZONTAL )
    {
        r.x = m_buttonSize;
        r.y = 0;
        r.w = m_barSize;
    }
    else
    {
        r.x = 0;
        r.y = m_buttonSize;
        r.h = m_barSize;
    }

    if ( r.Contains( pt.x, pt.y ) )
    {
        if ( ( event.LeftDown() ) || ( m_clickedOnBar && event.LeftIsDown() ) )
        {
            // decide: incrementing or decrementing
            if ( m_clickState == 0 )
            {
                if ( m_type == wxSB_HORIZONTAL )
                {
                    if ( pt.x < m_thumbDrawSize.x )
                    {
                        m_clickState = 2; // decrement
                    }
                    else
                    {
                        m_clickState = 1; // increment
                    }
                }
                else
                {
                    if ( pt.y < m_thumbDrawSize.y )
                    {
                        m_clickState = 2; // decrement
                    }
                    else
                    {
                        m_clickState = 1; // increment
                    }
                }
            }

            // increment or decrement
            if ( m_type == wxSB_HORIZONTAL )
            {
                if ( m_clickState == 2 )
                {
                    // decrement only to the cursor pos
                    if ( m_thumbDrawSize.x + m_thumbDrawSize.w < pt.x )
                        return;
                    else
                        m_thumbpos -= m_pagesize;

                    if ( m_thumbpos < 0 )
                        m_thumbpos = 0;
                }
                if ( m_clickState == 1 )
                {
                    // increment only to the cursor pos
                    if ( m_thumbDrawSize.x > pt.x )
                        return;
                    else
                        m_thumbpos += m_pagesize;

                    if ( m_thumbpos >= m_range )
                        m_thumbpos = m_range - 1;
                }
            }
            else
            {
                if ( m_clickState == 2 )
                {
                    // decrement only to the cursor pos
                    if ( m_thumbDrawSize.y + m_thumbDrawSize.h < pt.y )
                        return;
                    else
                        m_thumbpos -= m_pagesize;

                    if ( m_thumbpos < 0 )
                        m_thumbpos = 0;
                }
                if ( m_clickState == 1 )
                {
                    // increment only to the cursor pos
                    if ( m_thumbDrawSize.y > pt.y )
                        return;
                    else
                        m_thumbpos += m_pagesize;

                    if ( m_thumbpos >= m_range )
                        m_thumbpos = m_range - 1;
                }
            }

            m_clickedOnBar = true;

            // start a timer: checking the continuous left mouse button press
            if ( m_timer.IsRunning() == false )
            {
                m_timer.Start( 300 );
            }
            else
            {
                m_timer.Stop();
                m_timer.Start( 50 );
            }

            // create & drop scrollbar events
            if ( m_clickState == 1 )
            {
                if ( m_thumbpos == m_range - 1 )
                {
                    wxScrollEvent evt( wxEVT_SCROLL_BOTTOM, GetId(),  m_thumbpos, orient );
                    DropEvent( evt );
                }
                else
                {
                    wxScrollEvent evt( wxEVT_SCROLL_PAGEDOWN, GetId(), m_thumbpos, orient );
                    DropEvent( evt );
                }
            }
            else if ( m_clickState == 2 )
            {
                if ( m_thumbpos == 0 )
                {
                    wxScrollEvent evt( wxEVT_SCROLL_TOP, GetId(), 0, orient );
                    DropEvent( evt );
                }
                else
                {
                    wxScrollEvent evt( wxEVT_SCROLL_PAGEUP, GetId(), m_thumbpos, orient );
                    DropEvent( evt );
                }
            }

            Refresh();

            return;
        }
        else
        {
            m_timer.Stop();
            m_clickState = 0;
            return;
        }
    }
    else
    {
        if ( m_clickedOnBar && m_timer.IsRunning() )
        {
            m_timer.Stop();
            m_clickedOnBar = false;
        }
        m_clickState = 0;
    }

    event.Skip();
}

void bcScrollBar::OnTimer( wxTimerEvent& event )
{
    /// TODO
    wxWindow* parent = GetWindowParent();
    wxPoint parentPos = parent->GetPosition();

    wxMouseState state = wxGetMouseState();

    wxRealPointEx pos = GetPosition();

    wxMouseEvent evt;
    if ( state.LeftDown() )
    {
        evt.m_leftDown = true;
        evt.m_x = (int)( state.GetX() - parentPos.x - pos.x - ( parent->GetSize().x - parent->GetClientSize().x ) / 2 );
        evt.m_y = (int)( state.GetY() - parentPos.y - pos.y - ( parent->GetSize().y - parent->GetClientSize().y ) + ( parent->GetSize().x - parent->GetClientSize().x ) / 2 );

        OnMouse( evt ); // addpendingevent and processevent methods do not work!! (?)
    }
}
