#ifndef BCSCROLLBAR_H
#define BCSCROLLBAR_H

#include <wx/wx.h>

#include "BitmapCanvasWidgetBase.h"

class BitmapCanvasParentBase;

class bcScrollBar : public BitmapCanvasWidgetBase
{
    public:
        bcScrollBar( BitmapCanvasParentBase* parent, int winid, const wxPoint& pos = wxDefaultPosition, const wxSize&  size = wxDefaultSize, long style = wxSB_HORIZONTAL );
        virtual ~bcScrollBar();

        virtual void Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties);

        void            Create(  BitmapCanvasParentBase* parent, int winid, const wxPoint& pos = wxDefaultPosition, const wxSize&  size = wxDefaultSize, long style = wxSB_HORIZONTAL );

        int             GetRange() const;
        int             GetPageSize() const;
        int             GetThumbPosition() const;
        int             GetThumbSize() const;

        void            SetThumbPosition( int start );
        virtual void    SetScrollbar( int position, int WXUNUSED(thumbsize), int range, int pagesize );

        enum bcSBProps { BCSB_BARCOLOR, BCSB_THUMBCOLOR, BCSB_ARROWCOLOR };

    protected:
        virtual void        DrawWidget();

        friend class BitmapCanvasParentBase;

        // void    OnKeyDown( wxKeyEvent& event );
        void    OnMouse( wxMouseEvent& event );

    private:
        long            m_type;

        int             m_range;
        int             m_pagesize;
        int             m_thumbpos;

        int             m_thumbsize;

        wxTimer         m_timer;

        wxPoint         m_diffpos;

        /**
            [<][----|=|---------][>]
             ^   ^   ^
             |   |   L  thumbsize
             |   L bar size
             L button size

             buttonsize * 2 + bar size = total size
        **/
        double          m_buttonSize;
        double          m_barSize;

        //
        bool            m_decIsPressed;     // left or up
        bool            m_incIsPressed;     // right or down

        bool            m_clickedOnBar;     // clicking on the bar
        unsigned char   m_clickState;       // incrementing( 1 ) / decrementing ( 2 ) thumbpos while clicking on the bar, default (0)

        bool            m_isDragging;       // dragging the thumb

        wxRealRect      m_thumbDrawSize;    // position of the thumb (in pixels)


        void            OnTimer( wxTimerEvent& event );

        DECLARE_EVENT_TABLE()

};

#endif // BCSCROLLBAR_H
