#include "bcStaticLine.h"

#include "BitmapCanvasSystemSettings.h"
#include "BitmapCanvasParentBase.h"

#include "../agg/wxAgg2.h"

#include "../DirHelper.h"
#include "../DrawHelper.h"
#include "../logger.h"

#include "../common/types/RealPointArrayArray.h"

IMPLEMENT_DYNAMIC_CLASS(bcStaticLine, BitmapCanvasWidgetBase);

bcStaticLine::bcStaticLine(BitmapCanvasParentBase* parent, int id, const wxPoint& pos, const wxSize& size, long style)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
{
    //ctor
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(100,3);


    m_labelRotate = -1;
    m_labelPadding = 3;
    m_labelPos = 0.0;


    DrawWidget();
}

bcStaticLine::~bcStaticLine()
{
    //dtor
}

bool bcStaticLine::Focusable()
{
    return false;
}

void bcStaticLine::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{
}
#if USE_LABEL
void bcStaticLine::SetLabel(const wxString& label)
{
    m_label = label;
    Refresh();
}

wxString bcStaticLine::GetLabel() const
{
    return m_label;
}

void bcStaticLine::SetLabelPos(double pos_or_ratio)
{
    m_labelPos = pos_or_ratio;
    Refresh();
}
#endif
#if USE_LABEL == 2
void bcStaticLine::SetImgLabel(const wxImage& label)
{
    m_imgLabel = label.Copy();
    Refresh();
}

const wxImage& bcStaticLine::GetImgLabel() const
{
    return m_imgLabel;
}

void bcStaticLine::SetLabelOrinentation(int ori)
{
    m_labelOri = ori;
    Refresh();
}
#endif
wxRect bcStaticLine::GetTextBox()
{
    wxAgg2 aggCanvas(10,10);

    wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
    if(aggCanvas.loadTTFont(fontfile, GetSysDouble(BCBS_TEXT_HEIGHT), GetSysDouble(BCBS_TEXT_WEIGHT) ) == false) return wxRect(0,0,0,0);

    return aggCanvas.getTTFTextBound(m_label).GetOuterBox();
}

void bcStaticLine::DrawWidget()
{
    wxColor fgColor = GetSysColor( BCBS_FOREGROUND_COLOR );
    wxColor textColor = GetSysColor( BCBS_TEXT_COLOR );
    wxColor gravenColor = GetSysColor( BCBS_TEXT_GRAVEN_COLOR );

    wxRealSize size = GetSize();

    bool isHorizontal = true;
    if( m_style & bcSL_VERTICAL ) isHorizontal = false;

#if USE_LABEL == 2

    double linerad = isHorizontal ? 0 : M_PI_2;

    bool isLabel = m_label != wxEmptyString;
    bool isImgLabel = m_imgLabel.IsOk();

    int labelrot;
    if(m_labelOri == ORI_AUTO)
        labelrot = m_style & bcSL_HORIZONTAL ? 0 : M_PI_2;
    else
        labelrot = m_labelOri * M_PI_2;

    if(isLabel) {
        wxRect textBox = GetTextBox();
        if( isHorizontal )
            size.h = wxMax(size.h, ( fabs(sin(labelrot))<0.0001 ? textBox.height : textBox.width));
        else
            size.w = wxMax(size.w, ( fabs(sin(labelrot))<0.0001 ? textBox.width : textBox.height));
    }

    if(isImgLabel) {
        wxSize imsize(m_imgLabel.GetWidth(), m_imgLabel.GetHeight());
        if( isHorizontal )
            size.h = wxMax(size.h, ( fabs(sin(labelrot))<0.0001 ? imsize.y : imsize.x));
        else
            size.w = wxMax(size.w, ( fabs(sin(labelrot))<0.0001 ? imsize.x : imsize.y));
    }

    wxSize isize = size.Ceil();

    wxAgg2 aggCanvas(isize);

    if(!aggCanvas.isOk()) return;

    m_parent->AddRefreshRect( GetRect() );
    m_properties[BCWB_SIZE] << isize;

    wxRealPoint linePos;

    if(m_style & bcSL_HORIZONTAL) {
        if(m_style & bcSL_ALIGN_BOTTOM) linePos.y = isize.y-2+0.5;
        if(m_style & bcSL_ALIGN_TOP) linePos.y = 0.5;
        if(m_style & bcSL_ALIGN_CENTER) linePos.y = floor(size.h/2)+0.5;
    }
    if(m_style & bcSL_VERTICAL) {
        if(m_style & bcSL_ALIGN_RIGHT) linePos.x = isize.x-2+0.5;
        if(m_style & bcSL_ALIGN_LEFT) linePos.x = 0.5;
        if(m_style & bcSL_ALIGN_CENTER) linePos.x = floor(size.w/2)+0.5;
    }

    double line_wide = isHorizontal ? isize.x : isize.y;
    double line_thick = isHorizontal ? isize.y : isize.x;

    double label_pos = (m_labelPos > 0.0001 && m_labelPos < 1) ? (m_labelPos * ) : m_labelPos;

    wxAggPen * pen = aggCanvas.getPen();

    pen->SetColor( wxColor(62,62,62) );

//    m_labelPadding


#elif USE_LABEL == 1

    bool isLabel = m_label != wxEmptyString;

    wxRect textBox;

    if(isLabel) {
        m_parent->AddRefreshRect( GetRect() );

        textBox = GetTextBox();
        size.h = wxMax(size.h, textBox.height );

        m_properties[BCWB_SIZE] << size;
    }

    wxAgg2 aggCanvas(size.Ceil());

    if(!aggCanvas.isOk()) return;

    wxColor normalcolor(62,62,62);

    RealPointArrayArray lineSections;

    double label_pos = 0;

    if(m_style & bcSL_HORIZONTAL) {

        double line_y = floor(size.h*0.66)+0.5;

        label_pos = (m_labelPos > 0.0001 && m_labelPos < 1) ? (m_labelPos * size.w) : m_labelPos;


        if(isLabel) {
            RealPointArray line1,line2;
            line1.Add( wxRealPoint(0,line_y) );
            line1.Add( wxRealPoint(label_pos,line_y) );
            lineSections.Add(line1);
            line2.Add( wxRealPoint(label_pos+2*m_labelPadding+textBox.width,line_y) );
            line2.Add( wxRealPoint(size.w-1,line_y) );
            lineSections.Add(line2);
        } else {
            RealPointArray line1;
            line1.Add( wxRealPoint(0,line_y) );
            line1.Add( wxRealPoint(size.w-1,line_y) );
            lineSections.Add(line1);
        }

    } else if(m_style & bcSL_VERTICAL) {

        double line_x = floor(size.w/2)+0.5;

        RealPointArray arr;

        arr.Add( wxRealPoint(line_x,0) );
        arr.Add( wxRealPoint(line_x,size.h-1) );

        lineSections.Add(arr);
    }

    wxAggPen * pen = aggCanvas.getPen();

    pen->SetColor( normalcolor );
    for(size_t a=0;a<lineSections.GetCount();a++){
        aggCanvas.drawLine(lineSections[a]);
    }

    pen->SetColor( gravenColor );
    for(size_t a=0;a<lineSections.GetCount();a++){
        RealPointArray& arr = lineSections[a];
        DrawHelper::OffsetPath(arr, wxRealPoint(1,1));
        aggCanvas.drawLine(arr);
    }

    if(isLabel) {
        wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
        if(aggCanvas.loadTTFont(fontfile, GetSysDouble(BCBS_TEXT_HEIGHT), GetSysDouble(BCBS_TEXT_WEIGHT) )) {
            pen->SetColor( gravenColor );
            wxRealPoint labelpos(ceil(label_pos+m_labelPadding),ceil(textBox.height*0.7));
            aggCanvas.drawTTFText(m_label,labelpos.x+1,labelpos.y+1);
            pen->SetColor( GetSysColor(BCBS_TEXT_COLOR) );
            aggCanvas.drawTTFText(m_label,labelpos.x,labelpos.y);
        }
    }

#else
    wxAgg2 aggCanvas(this);

    if(!aggCanvas.isOk()) return;

    wxRealPoint lineStart, lineEnd;

    if(m_style & bcSL_HORIZONTAL) {
        lineStart.y = floor(size.h/2)+0.5;
        lineStart.x = 0;
        lineEnd.y = lineStart.y;
        lineEnd.x = size.w-1;
    } else if(m_style & bcSL_VERTICAL) {
        lineStart.y = 0;
        lineStart.x = floor(size.w/2)+0.5;
        lineEnd.y = size.h-1;
        lineEnd.x = lineStart.x;
    }

    wxAggPen * pen = aggCanvas.getPen();

    pen->SetColor( wxColor(62,62,62) );
    aggCanvas.drawLine( lineStart, lineEnd );

    lineStart.x++; lineStart.y++; lineEnd.x++; lineEnd.y++;
    pen->SetColor( gravenColor );
    aggCanvas.drawLine( lineStart, lineEnd );

#endif

    render(aggCanvas);

}
