#include "IconSetsHelper.h"

#include <wx/filename.h>
#include <wx/artprov.h>

#include "../DirHelper.h"

Int2StringHash IconSetsHelper::m_iconsFilename;
Int2StringHash IconSetsHelper::m_setsDirname;
IconSetsType IconSetsHelper::m_defaultIconSet;

void IconSetsHelper::Init()
{
    m_setsDirname[IS_BLACK_GRAVEN] = _T("graven-black");
    m_setsDirname[IS_GREY_GRAVEN] = _T("graven-grey");
    m_setsDirname[IS_WHITE_RAISED] = _T("raised-white");

    m_iconsFilename[IS_LEFT] = _T("left");
    m_iconsFilename[IS_RIGHT] = _T("right");
    m_iconsFilename[IS_HOME] = _T("home");
    m_iconsFilename[IS_REFRESH] = _T("refresh");
    m_iconsFilename[IS_SEARCH] = _T("search");
    m_iconsFilename[IS_FOLDER] = _T("folder");
    m_iconsFilename[IS_RECYCLE_EMPTY] = _T("recycle-empty");
    m_iconsFilename[IS_X] = _T("x");

    m_defaultIconSet = IS_BLACK_GRAVEN;
}

bool IconSetsHelper::SetDefaultSet(IconSetsType set)
{
    if( m_setsDirname.find(set) == m_setsDirname.end() ) return false;

    m_defaultIconSet = set;

    return true;
}

wxString IconSetsHelper::GetIconPath(IconSetsType setname, IconSetIconName icon, IconSetSize size)
{
    if( m_setsDirname.find(setname) == m_setsDirname.end() ) return wxEmptyString;
    if( m_iconsFilename.find(icon) == m_iconsFilename.end() ) return wxEmptyString;

    wxString filename = DirHelper::GetPath(dICON) + _T("sets") + DirHelper::Separ();
    filename += m_setsDirname[setname] + DirHelper::Separ();
    filename += wxString::Format(_T("%i"),size) + DirHelper::Separ();
    filename += m_iconsFilename[icon]+ _T(".png");

    if(!wxFileName::FileExists(filename)) return wxEmptyString;

    return filename;
}

wxImage IconSetsHelper::GetImage(IconSetIconName icon, IconSetSize size)
{
    wxString fp = GetIconPath(m_defaultIconSet,icon,size);

    if( wxFileName::FileExists(fp) ) {
        wxImage img;
        img.LoadFile( fp , wxBITMAP_TYPE_PNG);
        return img;
    }

    return wxArtProvider::GetBitmap(wxART_MISSING_IMAGE,wxART_OTHER, wxSize(size,size)).ConvertToImage();
}

wxBitmap IconSetsHelper::GetBitmap(IconSetIconName icon, IconSetSize size)
{
    wxString fp = GetIconPath(m_defaultIconSet,icon,size);

    if( wxFileName::FileExists(fp) ) {
        wxBitmap bmp;
        bmp.LoadFile( fp , wxBITMAP_TYPE_PNG);
        return bmp;
    }

    return wxArtProvider::GetBitmap(wxART_MISSING_IMAGE,wxART_OTHER, wxSize(size,size));
}

wxImage IconSetsHelper::GetImage(IconSetsType setname, IconSetIconName icon, IconSetSize size)
{
    wxString fp = GetIconPath(setname,icon,size);

    if( wxFileName::FileExists(fp) ) {
        wxImage img;
        img.LoadFile( fp , wxBITMAP_TYPE_PNG);
        return img;
    }

    return wxArtProvider::GetBitmap(wxART_MISSING_IMAGE,wxART_OTHER, wxSize(size,size)).ConvertToImage();
}

wxBitmap IconSetsHelper::GetBitmap(IconSetsType setname, IconSetIconName icon, IconSetSize size)
{
    wxString fp = GetIconPath(setname,icon,size);

    if( wxFileName::FileExists(fp) ) {
        wxBitmap bmp;
        bmp.LoadFile( fp , wxBITMAP_TYPE_PNG);
        return bmp;
    }

    return wxArtProvider::GetBitmap(wxART_MISSING_IMAGE,wxART_OTHER, wxSize(size,size));
}
