#include "KeyBinderHelper.h"

#include <wx/menuutils.h>
//#include "../configmanager/ConfigHelper2.h"
#include "../confighelper.h"

#include "../logger.h"

wxKeyProfileArray KeyBinderHelper::s_keyProfileArray;

KeyBinderHelper::KeyBinderHelper()
{
    //ctor
}

KeyBinderHelper::~KeyBinderHelper()
{
    //dtor
}


void KeyBinderHelper::Update(wxWindow* window)
{
	// detach all
	s_keyProfileArray.DetachAll();

	// enable & attach to this window only one
	s_keyProfileArray.GetSelProfile()->Enable(TRUE);

#ifndef wxUSE_BINDERAPP
	// VERY IMPORTANT: we should not use this function when we
	//                 have temporary children... they would
	//                 added to the binder and when they will be
	//                 deleted, the binder will reference invalid memory...
	s_keyProfileArray.GetSelProfile()->AttachRecursively(window);
	//r.UpdateAllCmd();		// not necessary
#endif

}

int KeyBinderHelper::Load(wxMenuBar* m)
{
    Logger::logf(LOG_MSG, _T("KeyBinderHelper::Load() START---------------------------\n----------------------"));
    // before loading we must register in wxCmd arrays the various types
	// of commands we want wxCmd::Load to be able to recognize...
	wxMenuCmd::Register(m);
	// clear our old array
	s_keyProfileArray.Cleanup();


	wxConfig *cfg = ConfigHelper::getConfig(CONFIG_TYPE_KEYBINDER);
	if (s_keyProfileArray.Load(cfg)) {
		// get the cmd count
		int total = 0;
		for (int i=0; i<s_keyProfileArray.GetCount(); i++)
			total += s_keyProfileArray.Item(i)->GetCmdCount();

		if (total == 0) {
		    wxString msg = wxString::Format(
					wxT("No keyprofiles have been found...\n")
					wxT("A default keyprofile will be set."));
            Logger::logf(LOG_MSG, _T("KeyBinderHelper::Load() ")+ msg );
            /*
			wxMessageBox();
            */
			wxKeyProfile *p = new wxKeyProfile(wxT("Default"));
			p->ImportMenuBarCmd(m);
			s_keyProfileArray.Add(p);
		} else {
		    Logger::logf(LOG_MSG, _T("KeyBinderHelper::Load() normal"));
            /*
			wxMessageBox(wxString::Format(
					wxT("All the [%d] keyprofiles have been correctly loaded ")
					wxT("(%d commands in total).\n")
					wxT("The %d-th loaded profile ('%s') will be applied."),
					arr.GetCount(), total, arr.GetSelProfileIdx()+1,
					arr.GetSelProfile()->GetName().c_str()),
						wxT("Success"));
            */
		}
		// reattach this frame to the loaded keybinder
		//Update();
        return total;
	} else {
	     Logger::logf(LOG_MSG, _T("KeyBinderHelper::Load() error"));
	     wxKeyProfile *p = new wxKeyProfile(wxT("Default"));
            p->ImportMenuBarCmd(m);
			s_keyProfileArray.Add(p);

/*
		wxMessageBox(wxT("Something wrong while loading !"),
			wxT("Error"), wxOK | wxICON_ERROR);
        */
         return -1;
	}
}

int KeyBinderHelper::Save()
{
	//wxConfig *cfg = new wxConfig(wxGetApp().GetAppName());
	wxConfig *cfg = ConfigHelper::getConfig(CONFIG_TYPE_KEYBINDER);
	if (s_keyProfileArray.Save(cfg, wxEmptyString, TRUE)) {
	    ConfigHelper::save(CONFIG_TYPE_KEYBINDER);
		// get the cmd count
		int total = 0;
		for (int i=0; i<s_keyProfileArray.GetCount(); i++)
			total += s_keyProfileArray.Item(i)->GetCmdCount();
        /*
		wxMessageBox(wxString::Format(wxT("All the [%d] keyprofiles ([%d] commands ")
			wxT("in total) have been correctly saved."), arr.GetCount(), total),
			wxT("Success"));
        */
        return 1;
	} else {
/*
		wxMessageBox(wxT("Something wrong while saving !"), wxT("Error"),
			wxOK | wxICON_ERROR);
			*/
        return 0;
	}
}
