#include "AIMDetector.h"

#include "../../aimdisplay/AIMDetectorEvent.h"
#include "../../common/FSMAIM.h"
#include "../../common/SIPR2AIMConverter.h"

#include "../../vemsprotocol/SIPRTypes.h"
#include "../../vemsprotocol/SIPRPacket.h"
#include "../../events/InputEvent.h"

#include "../../logger.h"

#define AIM_PACKET_SIZE 5

#include "../../aimdisplay/AIMDisplayHelper.h"

AIMDetector::AIMDetector(wxEvtHandler* eventHandler)
{
    //m_clientEventHandler    = clientEventHandler;
    m_eventHandler          = eventHandler;

    //m_aimParser = new AIMParser(m_eventHandler);
    m_fsm = new FSMAIM();

    m_timeoutCtr = 0;

    m_txCurrentSize = 0;

    m_timeout = 40;

    Connect( wxEVT_INPUT, InputEventHandler( AIMDetector::OnSIPRInputEvent ), NULL, this );

}


AIMDetector::~AIMDetector()
{
    //dtor
    //delete m_aimParser;
}

void AIMDetector::OnSIPRInputEvent(InputEvent& evt)
{
    SIPRPacket p = evt.getPacket();

    if(p.type() != SIPR_AIM) return;

    if( m_txCurrentSize + AIM_PACKET_SIZE > TX_BUFFER_SIZE ) {
        Logger::logf(LOG_MSG,_T("AIMDetector::OnSIPRInputEvent m_txBuff is full!!!"));
        return;
}

    uint8_t* outbuffer = new uint8_t[AIM_PACKET_SIZE];

    bool res = SIPR2AIMConverter::Convert(p,outbuffer);

    if(res) {
        memcpy(m_txBuff+m_txCurrentSize,outbuffer,AIM_PACKET_SIZE);
        m_txCurrentSize += AIM_PACKET_SIZE;
    }

    delete[] outbuffer;
}

void AIMDetector::write( uint8_t* buffer, int& size )
{
    if(m_txCurrentSize == 0) return;

    memcpy(buffer,m_txBuff,m_txCurrentSize);
    size = m_txCurrentSize;

    m_txCurrentSize = 0;
}

void AIMDetector::read( uint8_t* buffer, int size )
{
    if(size == 0) {

        ++m_timeoutCtr;

        wxThread::Sleep( 2 );

        if(m_timeoutCtr > m_timeout) {
            m_state = TIMEOUT;
            return;
}

    } else {

        m_timeoutCtr = 0;

    }

    for( int i = 0; i < size; ++i )
    {
        int state = m_fsm->feed( buffer[i] );

        if(state == FSMAIM::END)
        {

            AIMPacket& aimp = m_fsm->getData();



            m_siprbuffer[0] = aimp.channel;
            m_siprbuffer[1] = aimp.data >> 8;
            m_siprbuffer[2] = aimp.data & 0xFF;

            IntArray& reqChannels = AIMDisplayHelper::requiredChannels;

            if ( ( reqChannels.GetCount() > 0 ) && ( reqChannels.Index( aimp.channel ) == wxNOT_FOUND ) ){
                return;
            }
            //if(aimp.channel == 45)Logger::logf(LOG_MSG,_T("AIMDetector::read channel=%i, data=%i"),aimp.channel,aimp.data);

            SIPRPacket packet( 0, SIPR_AIM, 3 , m_siprbuffer );
            InputEvent event;
            event.setPacket( packet );
            if(m_eventHandler) m_eventHandler->ProcessEvent( event );

        }
    }


}

/*void AIMDetector::addReqChannel( int channelid )
{
    if ( m_reqChannels.Index( channelid ) == wxNOT_FOUND )
        m_reqChannels.Add( channelid );
}*/

void AIMDetector::finish()
{
    if ( m_state == TIMEOUT )
    {
        if ( m_eventHandler )
        {
            AIMDetectorEvent e;
            e.SetString(_T("TIMEOUT"));
            m_eventHandler->AddPendingEvent( e );
        }
    }
    //m_clientEventHandler->handleEvent( CommunicationClientEvent::TIMEOUT );
}


bool AIMDetector::isFinished()
{
    if ( m_state == TIMEOUT )
        return true;
    //return m_state == FINISHED;
    return false;
}

