#ifndef CONFIGDOWNLOADER_H
#define CONFIGDOWNLOADER_H


#include "DeviceDetectorListener.h"
#include "../CommunicationControllerClient.h"


class SiprPacketReceiver;
class ConfigDownloadListener;
class TimeoutListener;


class ConfigDownloader : public CommunicationControllerClient
    , public DeviceDetectorListener
{
    public:
        ConfigDownloader(   ConfigDownloadListener* downloadListener,
                            TimeoutListener* timeoutListener,
                            SiprPacketReceiver* receiver  );
        virtual ~ConfigDownloader();

        /// CommunicationControllerClient methods
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();

        /// DeviceDetectorListener methods
        void OnDeviceFound( ECUInfo::Device device, ECUInfo::EcuMode mode );
        void OnDeviceNotFound();

    private:
        const size_t DOWNLOAD_MIN_LINE_COUNT;
        const size_t DOWNLOAD_SKIP_EMPTY_LINES;

        enum State { SEND, WAIT, CLIENT_MODE, FINISHED, TIMEDOUT };
        const size_t TIMEOUT;

        ConfigDownloadListener*     m_downloadListener;
        TimeoutListener*            m_timeoutListener;
        SiprPacketReceiver*         m_receiver;
        State                       m_state;

        uint8_t                     m_rxBuffer[256];
        size_t                      m_rxDataLength;

        size_t                      m_lineCount;
        size_t                      m_emptyLineCount;
        bool                        m_isDownloadingConfig;

        CommunicationControllerClient* m_client;

    private:
        void send( uint8_t* buffer, int& size );
        void wait( uint8_t* buffer, int size );
        void sendClientMode( uint8_t* buffer, int& size );
        void waitClientMode( uint8_t* buffer, int size );

        void sendLinePacket();
        bool isEOL() const;
};

#endif // CONFIGDOWNLOADER_H
