#ifndef EEPROMREADERCLIENT_H
#define EEPROMREADERCLIENT_H

#include "DeviceModeSelectorListener.h"
#include "TimeoutListener.h"
#include "../CommunicationControllerClient.h"
#include "../../vemsprotocol/ECUInfo.h"

#include <wx/file.h>

// escape char (end indicator)
#define HDLC_END 0x7e

class wxFile;
class CommunicationBase;
class IntArray;
class CommunicationClientEventHandler;

class EepromReaderClient
    : public CommunicationControllerClient
    , public DeviceModeSelectorListener
    , public TimeoutListener
{
    public:
        EepromReaderClient( CommunicationClientEventHandler* clientEventHandler,
                            wxEvtHandler* eventHandler,
                            CommunicationBase* commBase,
                            const IntArray& baudrates,
                            ECUInfo::EcuMode sourceMode = ECUInfo::UNKNOWN );
        virtual ~EepromReaderClient();

        void stop();

//        void handleEvent( CommunicationClientEvent::Type eventType );

        /// CommunicationControllerClient methods
        void write( uint8_t* buffer, int& size );

        void read( uint8_t* buffer, int size );

        void finish();

        bool isFinished();

        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );
        void OnTimeout();

    private:
    enum State
        {
            SELECT_MAN_MODE,
            WAIT_MAN_MODE,

            SEND_COMMAND,
            WAIT_DATA,

            FINISHED,
            TIMEDOUT,
            CRC_FAILED,
        };

        enum { MIN_SIZE = 244, BUFFER_SIZE = 256 *2 + 10 }; // extreme case: all values are escaped
        enum { MAX_CRCERR = 5 };    // no. of maximum crc errors

        bool                                m_writeToFile;

        CommunicationClientEventHandler*    m_clientEventHandler;
        wxEvtHandler*                       m_eventHandler;
        State                               m_state;

        size_t                              m_rxDataLength;
        size_t                              m_timeoutCount;

        uint8_t                             m_rxBuffer[BUFFER_SIZE];

        ECUInfo::EcuMode                    m_sourceMode;

        CommunicationControllerClient*      m_client;

        size_t                              m_address;

        wxFile                              m_file;

        unsigned int                        m_fileaddress;

        wxMemoryBuffer*                     m_membuffer;

        unsigned char                       m_crcErrors;

    private:
        void    selectManMode( uint8_t* buffer, int& size );
        void    waitManMode( uint8_t* buffer, int size );

        void    sendCommand( uint8_t* buffer, int& size );
        void    waitData( uint8_t* buffer, int size );

        void sendEvent(size_t WXUNUSED(size));

        // helper
        size_t  unescapeData( uint8_t* buffer, size_t buflen, uint8_t** outbuffer );
        void    writeToFile( uint8_t* data, size_t datalen );
        void    addToBuffer( uint8_t* data, size_t datalen );

        void    sendNoValidData();
};

#endif // EEPROMREADERCLIENT_H
