#ifndef FIRMWAREUPLOADER_H
#define FIRMWAREUPLOADER_H


#include "../CommunicationControllerClient.h"
#include "../../vemsprotocol/ECUInfo.h"

class FirmwareData;
class CommunicationBase;
class IntArray;
class FirmwareUploaderListener;
class TimeoutListener;


class FirmwareUploader : public CommunicationControllerClient
{
    public:
        FirmwareUploader(   FirmwareUploaderListener* uploaderListener,
                            TimeoutListener* timeoutListener,
                            FirmwareData* fwData );

        virtual ~FirmwareUploader();

        /// CommunicationControllerClient methods
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();

    private:
        enum State {    ERASE_ALL_PAGES,
                        WAIT_ALL_PAGE_ERASED,
                        SEND_LEAVE_BOOT_MODE_COMMAND,
                        WAIT_AUTO_ENTER_BOOT_MODE,
                        CLEARING,
                        SEND_PAGE_ADDRESS,
                        WAIT_PAGE_ADDRESS_OK,
                        SEND_PAGE,
                        WAIT_SEND_PAGE_OK,
                        FINISHED,
                        TIMEDOUT };

        enum { ERASED_MAX_TRY_COUNT = 30 };
        enum { ERASED_SLEEP_TIME = 100 };
        enum { MAX_PAGE_NUMBER = 503 };
        enum { V3_MAX_PAGE_IDX = 503 };
        enum { AUTO_ENTER_BOOT_MODE_SLEEP_TIME = 20 };

    private:
        FirmwareUploaderListener*   m_uploaderListener;
        TimeoutListener*            m_timeoutListener;
        FirmwareData*               m_firmwareData;

        State                       m_state;

        size_t                      m_currentPage;
        size_t                      m_erasedTryCount;
        size_t                      m_timeoutCount;
        bool                        m_currentPageCommandSent;
        size_t                      m_currentPageBlockIdx;
        size_t                      m_clearCount;
        size_t                      m_clearCountMax;

    private:
        void eraseAllPages( uint8_t* buffer, int& size );
        void waitAllPageErased( uint8_t* buffer, int size );
        void sendLeaveBootModeCommand( uint8_t* buffer, int& size );
        void waitAutoEnterBootMode( uint8_t* buffer, int size );
        void clearing( uint8_t* buffer, int size );
        void sendPageAddress( uint8_t* buffer, int& size );
        void waitPageAddressOk( uint8_t* buffer, int size );
        void sendPage( uint8_t* buffer, int& size );
        void waitSendPageOk( uint8_t* buffer, int size );

        void sendFirmwareUploadEvent( int eventType );

};

#endif // FIRMWAREUPLOADER_H
