#ifndef OFFSETREADERCLIENT_H
#define OFFSETREADERCLIENT_H

#include "DeviceModeSelectorListener.h"
#include "TimeoutListener.h"
#include "../CommunicationControllerClient.h"
#include "../CommunicationClientEventHandler.h"
#include "../../Interruptable.h"
#include "../../vemsprotocol/ECUInfo.h"
#include "../../common/types/StringHash.h"
#include "../../common/types/IntArray.h"

class wxTextFile;
class CommunicationBase;




class OffsetReaderClient : public CommunicationControllerClient, public CommunicationClientEventHandler, public Interruptable
    , public DeviceModeSelectorListener
    , public TimeoutListener
{
    public:
        // csv writer
        OffsetReaderClient( CommunicationBase* commBase,
                            const IntArray& baudrates,
                            ECUInfo::EcuMode deviceMode,
                            wxArrayString& offsets,
                            bool hasCRC );

        // SIPR
        OffsetReaderClient( ECUInfo::EcuMode deviceMode, wxArrayString& addresses, wxEvtHandler* evtHandler, bool hasCRC );

        // sendevent mode
        OffsetReaderClient( ECUInfo::EcuMode deviceMode, StringHash& variable, wxEvtHandler* evtHandler, bool hasCRC );
        ~OffsetReaderClient();

    public:
        // Interruptable virtual methods
        void    Abort();

        // CommunicationControllerClient virtual methods
        void    write( uint8_t* buffer, int& size );
        void    read( uint8_t* buffer, int size );
        void    finish();
        bool    isFinished();

        // CommunicationClientEventHandler virtual methods
        void    handleEvent( CommunicationClientEvent::Type eventType );

        // DeviceModeSelectorListener virtual methods
        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );

        void OnTimeout() {} ///TODO

    private:
        bool m_crc;

        enum { TIME_OUT = 1500 };

        enum Mode { CSV_Writer, Evt_Sender, SIPRPacket_Creator } m_mode;

        enum State {
            GO_MAN,
            WAIT_MAN,

            SEND_COMMAND,
            GET_RESPONSE,

            WRITETO_CSV,
            SEND_EVENT,
            CREATE_SIPR,

            RESET_DEVMODE,
            WAIT_RESET,

            ABORTED,

            TIMEOUT,
            FINISHED

        } m_state;

        ECUInfo::EcuMode                m_deviceMode;
        CommunicationControllerClient*  m_client;

        wxEvtHandler*                   m_evtHandler;
        StringHash                      m_variable;
        int                             m_varOffset;

        wxArrayString                   m_offsets;
        size_t                          m_offsetCounter;
        unsigned char*                  m_buffer;
        size_t                          m_bufLen;
        size_t                          m_maxBufLen;

        size_t                          m_timeoutCtr;

        wxTextFile*                     m_csvfile;

        CommunicationBase*              m_commBase;
        IntArray                        m_baudrates;

    private:
        // Select MAN mode
        void        goMan( uint8_t* buffer, int& size );
        void        waitMan( uint8_t* buffer, int size );

        void        sendCommand( uint8_t* buffer, int& size );
        void        getResponse( uint8_t* buffer, int size );

        void        writeToCsv();
        void        closeFile();

        void        sendEvent();

        void        createSipr();

        void        resetDevMode( uint8_t* buffer, int& size );
        void        waitResetDevMode( uint8_t* buffer, int size );

};

#endif // OFFSETREADERCLIENT_H
