#include "SDCardFileHelper.h"

#include "../../logger.h"
#include "../../Math.h"

uint16_t getCRC( uint8_t* data, size_t len );

bool SDCardFile::isCRCValid( uint8_t* data, size_t len )
{
    uint16_t crccalc = getCRC( data, len - 2 );

    uint16_t crcrecv = ( data[len - 2 ] << 8 ) + data[len - 1]; // big endian

    if ( crccalc == crcrecv )
    {
        return true;
    }
    else
    {
        Logger::logf(LOG_MSG, _T("SDCard block invalid CRC, calculated=%04x, received=%04x"), crccalc, crcrecv );
        return false;
    }
}

bool SDCardFile::isDataValid( uint8_t* data, size_t len )
{
    int data_or = 0;
    int data_and = 0xff;

    for ( unsigned i = 0; i < len; ++i )
    {
        data_or |= data[i];
        data_and &= data[i];
    }

    if ( ( data_or == 0 ) || ( data_and == 0xff ) )
    {
        return false;
    }

    /* if ( data[511] != 64 )
        return false;*/

    return true;
}

bool SDCardFile::checkSectorIsEmpty( uint8_t * data, size_t len )
{
    size_t sum = 0;

    for ( size_t i = 0; i < len; ++i )
    {
        sum += data[i];
    }

    return ( ( sum == 0 ) || ( sum == ( len * 255 ) ) );
}

size_t SDCardFile::CalcLogTimeLength( size_t len, size_t datalen )
{
    return (size_t)( ((int) (512.0 / (double)datalen) ) * 0.1 * len );
    //return (size_t)( (double)len * 51.2f / (double)datalen );
}

size_t SDCardFile::CalcLogTimeLength( size_t len )
{
    // assumed 7 RTs in a block
    return (size_t)( (double)len * 0.1f * 7.0 );
}

uint16_t getCRC( uint8_t* data, size_t len )
{
    uint16_t crc = 0;

    for ( unsigned int i = 0; i < len; ++i )
    {
        crc = Math::crc16( crc, data[i] );
    }

    return crc;
}

bool SDCardFile::isVEMSFilesystem( uint8_t* data )
{
    wxString vh = wxString::From8BitData( (const char*)data, 4 );

    if ( vh == _T("VEMS") ) return true;

    return false;
}

FirmwareVersion SDCardFile::getFwVersion( uint8_t* data )
{
    uint16_t rver = data[4] << 8;
    rver += data[5];

    FirmwareVersion fv;

    fv.build = rver % 100;
    rver /= 100;
    fv.minor = rver % 100;
    rver /= 100;
    fv.major = rver;

    return fv;
}

unsigned int SDCardFile::getFWVersionAsInt( uint8_t* data )
{
    uint16_t rver = data[4] << 8;
    rver += data[5];

    return rver;
}

//1 : old 3 byte
//2 ? new 4 byte
uint8_t SDCardFile::getInodeVersion( uint8_t* data )
{
    return data[7];
}
