#ifndef SDCARDFILESEARCH_H
#define SDCARDFILESEARCH_H

#include "../CommunicationControllerClient.h"
#include "../../common/MMCFilesDesc.h"

WX_DEFINE_ARRAY_INT( uint32_t, UInt32Arr );


/// ------------------------------------------------------
/// SDStaticInfo - store MMCFiles descriptors in array
/// m_dirty - if there was no start/stop/erase/ECU restart then no searching is required
/// if it is dirty -> clear the array
class SDStaticInfo
{
    public:
        static bool isDirty() { return m_dirty; }

        static void setDirty( bool dirty )
        {
            m_dirty = dirty;
            if ( m_dirty )
            {
                m_mmcdescarr.Clear();
            }
        }

        static MMCFilesDescA m_mmcdescarr;
    private:
        static bool m_dirty;
};

/// ------------------------------------------------------
/// searching on v3 - fills SDStaticInfo::m_mmcdescarr

class SDCardFileSearch : public CommunicationControllerClient
{
    public:
        SDCardFileSearch(wxEvtHandler * evthandler );
        virtual ~SDCardFileSearch();

    public:
        /// CommunicationControllerClient virtual methods
        void    write( uint8_t* buffer, int& size );
        void    read( uint8_t* buffer, int size );
        void    finish();
        bool    isFinished();

        // see m_cardstate:
        // 0        : ok
        // -1       : card empty
        // -2       : not valid VEMS filesystem
        // -21/-22  : msd/mss failed
        // -42      : timeout

    private:

        /// TODO
        enum State {
            INVALID,                // invalid

            GET_STOPIDX,            // get & interpret stopidx
            CHECK_STOPIDX,

            GET_DATASIZE,           // get & interpret datasize
            CHECK_DATASIZE,

            GET_INODE,              // get the inode table, fill descriptors
            CHECK_INODE,
            FINISH_INODE,

            CARD_STATE_FINISHED,    // searching finished

            TIMEOUT,                // timeout
            TIMEOUT_FALLBACK,

            GET_STATUS_BGET,        // before every msg/G we need to check status
            CHECK_STATUS_BGET,

            SEND_MSD,               // send msd in neccessary
            CHECK_MSD,

            FINISHED                // FSM finished state
        };


        // max_try_count: max. number of sent msg#addr
        // max_mss_tries: max number of sent mss (if result of mss is 'u')
        enum { MAX_TRY_COUNT = 10, MAX_MSS_TRIES = 20 };

        // timeout in ms
        enum { TIMEOUT_VAL = 400 };

        // m_state - the actual state
        // m_nextstate - after mss which state must be selected
        State                       m_state;
        State                       m_nextstate;

        // evthandler - sending sdcardevents
        wxEvtHandler*               m_evthandler;

        // current address, the min.&stop addr of the data
        unsigned int                m_address;
        unsigned int                m_minIdx;
        unsigned int                m_stopIdx;

        // datasize (64)
        int                         m_datasize;

        // needed for searching
        short                       m_count;
        uint32_t                    m_mask;
        bool                        m_upward;
        bool                        m_nextBlock;

        // 514 bytes (sector: 512+2 crc) & the len of the buffer
        unsigned char*              m_buffer;
        unsigned int                m_bufLen;

        int                         m_cardState;

        UInt32Arr                   m_addrarr;
        UInt32Arr                   m_rttypearr;

        // &SDStaticInfo::m_mmcdescarr
        MMCFilesDescA*              m_mmcdescarr;

        // timeout counter (should be <TIMEOUT_VAL)
        size_t                      m_timeoutCounter;

        // # of sent mss before msg
        size_t                      m_tryCount;

        // # of ('u') - result of mss (see MAX_MSS_TRIES)
        size_t                      m_msscounter;

        // if msd is sent m_iswaiting => true, do not resend msd for again
        bool                        m_isWaiting;

    private:
        // get stop idx
        void        getStopIdx( uint8_t* buffer, int& size );
        void        checkStopIdx( uint8_t* buffer, int size );

        // get data size
        void        getDataSize( uint8_t* buffer, int& size );
        void        checkDataSize( uint8_t* buffer, int size );

        // get inode
        void        getInode( uint8_t* buffer, int& size );
        void        checkInode( uint8_t* buffer, int size );

        // send mss
        void        getStatusBeforeGet( uint8_t* buffer, int& size );
        void        checkStatusBeforeGet( uint8_t* buffer, int size );

        // send msd
        void        sendDisable( uint8_t* buffer, int& size );
        void        checkDisable( uint8_t* buffer, int size );

        // interpret indode
        void        finishInode();
        void        checkCardState();

        /// helpers
        // for raw rt
        size_t      CalcLogTimeLen( size_t len, size_t datalen );
        // for triggerframe-format (not exact result!)
        size_t      CalcLogTimeLen( size_t len );

};

#endif // SDCARDFILESEARCH_H
