#include "confighelper.h"

#include <wx/wfstream.h>
#include <wx/fileconf.h>
#include <wx/filename.h>

#include <wx/regex.h>


#include "logger.h"
#include "DirHelper.h"


#include "vemsprotocol/ECUInfo.h"

wxConfig * ConfigHelper::globalConfig; //= new wxConfig(_T("V3_CONFIG") );
wxConfig * ConfigHelper::m_guiLayoutConfig;
wxFileInputStream* ConfigHelper::globalFileInputStream = NULL;
wxFileInputStream* ConfigHelper::keybindingsFileInputStream = NULL;
wxFileInputStream* ConfigHelper::layoutFileInputStream = NULL;

#if defined(__WINCE_STANDARDSDK__)
wxString * ConfigHelper::guiConfigFile = new wxString(_T("v3cockpit.cfg") );
#elif defined(__WXMSW__)
wxString * ConfigHelper::guiConfigFile = new wxString(_T("v3gui.cfg") );
#elif defined(__UNIX__)
wxString * ConfigHelper::guiConfigFile = new wxString(_T("v3guil.cfg") );
#endif

//wxString * ConfigHelper::guiLayoutConfigFile = new wxString(_T("guilayout.cfg") );
wxString * ConfigHelper::guiLayoutConfigFile = new wxString(_T("gui.cfg") );

wxString ConfigHelper::CFG_PORT_NAME = wxString(_T("port"));
wxString ConfigHelper::CFG_AUTO_RAW_LOG = wxString(_T("autoRawLog"));

SerialConfig * ConfigHelper::m_serialConfig = new SerialConfig();
SerialConfig * ConfigHelper::m_serialConfigGPS = new SerialConfig();


wxString * ConfigHelper::s_keyBindingsFile = new wxString(_T("keybindings.cfg") );
wxConfig * ConfigHelper::s_keyBindingsCfg;

//wxSize ConfigHelper::m_refDisplaySize = wxDefaultSize;

ConfigHelper::ConfigHelper()
{
    //ctor
}

ConfigHelper::~ConfigHelper()
{
    //dtor
}






void ConfigHelper::initConfig(char mode)
{
    //const wxString fileName = getConfigFile();
    const wxString fileName = getConfigFile(mode);
	Logger::logf(LOG_MSG, _T("ConfigHelper::initConfig(): ")+fileName);
    checkAndCreateFile(fileName);

    switch(mode) {
        case CONFIG_TYPE_GUI:
			globalFileInputStream = new wxFileInputStream(fileName);
            globalConfig = (wxConfig *) new wxFileConfig(  *globalFileInputStream,  wxConvUTF8);
          break;

        case CONFIG_TYPE_LAYOUT:
			layoutFileInputStream = new wxFileInputStream(fileName);
            m_guiLayoutConfig = (wxConfig *) new wxFileConfig(  *layoutFileInputStream,  wxConvUTF8);
          break;
        case CONFIG_TYPE_KEYBINDER:
			keybindingsFileInputStream = new wxFileInputStream(fileName);
            s_keyBindingsCfg = (wxConfig *) new wxFileConfig(  *keybindingsFileInputStream,  wxConvUTF8);
            break;
    }


}

void ConfigHelper::cleanup()
{
	if ( globalConfig )
	{
		delete globalConfig;
		delete globalFileInputStream;
	}

	if ( m_guiLayoutConfig )
	{
		delete m_guiLayoutConfig;
		delete layoutFileInputStream;
	}

	if ( s_keyBindingsCfg )
	{
		delete s_keyBindingsCfg;
		delete keybindingsFileInputStream;
	}
}

wxString ConfigHelper::getConfigFile(char mode)
{
    wxString dir = DirHelper::GetPath(dCONFIG);

    switch(mode){
        case CONFIG_TYPE_GUI:
            dir.Append(*guiConfigFile);
          break;
        case CONFIG_TYPE_LAYOUT:
            dir.Append(*guiLayoutConfigFile);
          break;

          break;
    }

    return dir;
}

wxConfig * ConfigHelper::getConfig(char mode)
{
    switch(mode){
        case CONFIG_TYPE_GUI:
            return globalConfig;
          break;
        case CONFIG_TYPE_LAYOUT:
            return m_guiLayoutConfig;
          break;

    }

    return NULL;
}

void ConfigHelper::checkAndCreateFile(const wxString& file)
{
    bool isExist = wxFileName::FileExists(file);
    if(! isExist){
        wxFile tmpFile;
        tmpFile.Create(file);
        tmpFile.Close();

            //wxFileName tmpFile(file);
            //tmpFile.Touch();
        //bool Create(const char* filename, bool overwrite = false, int access = wxS_DEFAULT)
    }

}

void ConfigHelper::save(char mode)
{
    wxFileConfig * fileConfig  = (wxFileConfig *) getConfig(mode);

    const wxString fileName = getConfigFile(mode);
    //Logger::logf(LOG_MSG,_T("--------------ConfigHelper::save(char mode)fileName:")+fileName);
    wxOutputStream * fos = (wxOutputStream *) new wxFileOutputStream(fileName);

    fileConfig->Save(*fos, wxConvUTF8);
}

//void ConfigHelper::SaveFrameLayout(wxObject* frame_withRTTI)
//{
//    wxString classname = frame_withRTTI->GetClassInfo()->GetClassName();
//
//    SaveFrameLayout(frame_withRTTI,classname);
//}
//
//void ConfigHelper::SaveFrameLayout(wxObject* frame, const wxString& name)
//{
//    wxConfig * v3conf = ConfigHelper::getConfig( CONFIG_TYPE_LAYOUT );
//    if(!v3conf) return;
//
//    v3conf->SetPath(_T("/autosavedlayout/")+name);
//
//    wxWindow * win = wxDynamicCast(frame, wxWindow);
//
//    wxRect winrect = win->GetRect();
//
//    if(win) {
//        v3conf->Write(_T("x"), winrect.GetLeft() );
//        v3conf->Write(_T("y"), winrect.GetTop() );
//        v3conf->Write(_T("width"), winrect.GetWidth() );
//        v3conf->Write(_T("height"), winrect.GetHeight() );
//    }
//
//    wxTopLevelWindow * topwin = wxDynamicCast(frame, wxTopLevelWindow);
//
//    if(topwin) {
//
//        v3conf->Write(_T("fullscreen"), topwin->IsFullScreen() ? 1 : 0 );
//        v3conf->Write(_T("maximized"),topwin->IsMaximized() ? 1 : 0 );
//
//    }
//
//    v3conf->Flush();
//    save( CONFIG_TYPE_LAYOUT );
//}

//void ConfigHelper::LoadFrameLayout(wxObject* frame_withRTTI, const wxRect& bbrect)
//{
//    wxString classname = frame_withRTTI->GetClassInfo()->GetClassName();
//
//    LoadFrameLayout(frame_withRTTI,classname,bbrect);
//}
//
//void ConfigHelper::LoadFrameLayout(wxObject* frame, const wxString& name, const wxRect& bbrect)
//{
//    wxConfig * v3conf = ConfigHelper::getConfig( CONFIG_TYPE_LAYOUT );
//    if(!v3conf) return;
//
//    v3conf->SetPath(_T("/autosavedlayout/")+name);
//
//    long x = v3conf->Read(_T("x"),50);
//    long y = v3conf->Read(_T("y"),50);
//    long width = v3conf->Read(_T("width"),500);
//    long height = v3conf->Read(_T("height"),400);
//    bool isMaximized = v3conf->Read(_T("maximized"),0L);
//    bool isFullScreen = v3conf->Read(_T("fullscreen"),0L);
//
//    wxRect brect = bbrect;
//
//    if( brect == wxDefaultRect ) {
//
//        brect.SetTopLeft( wxPoint(0,0) );
//        brect.SetSize( DisplayHelper::GetDisplaySize() );
//
//    }
//
//    if( x >= brect.GetRight() || x <= brect.GetLeft() ) x = 50;
//    if( y >= brect.GetBottom() || y <= brect.GetTop() ) y = 50;
//
//    if( width > brect.GetWidth() ) width = 500;
//    if( height > brect.GetHeight() ) height = 400;
//
//    wxWindow * win = wxDynamicCast(frame, wxWindow);
//
//    if(win) {
//        win->SetSize( width, height );
//        win->Move( x, y );
//    }
//
//    wxTopLevelWindow * topwin = wxDynamicCast(frame, wxTopLevelWindow);
//
//    if(topwin) {
//        topwin->Maximize( isMaximized );
//        topwin->ShowFullScreen( isFullScreen );
//    }
//}
