#include "ConfigMgrFrameLayout.h"

#include "../ConfigBase.h"
#include "MainGuiConfig.h"

#include "../logger.h"
#include "../DisplayHelper.h"

ConfigMgrFrameLayout::ConfigMgrFrameLayout(ConfigBase* configBase, const wxString& name)
:ConfigMgrBase(configBase)
{
    //ctor
    m_path = _T("/autosavedlayout/");
    m_name = name;
}

ConfigMgrFrameLayout::~ConfigMgrFrameLayout()
{
    //dtor
}


void ConfigMgrFrameLayout::loadTo(wxObject* frame_withRTTI, const wxRect& bbrect)
{
    wxString classname = frame_withRTTI->GetClassInfo()->GetClassName();

    loadTo(frame_withRTTI,classname,bbrect);
}

void ConfigMgrFrameLayout::loadTo(wxObject* frame, const wxString& name, const wxRect& bbrect)
{
    m_name = name;

    MainGuiConfig cfg;
    load(&cfg);

    wxRect brect = bbrect;

    if( brect == wxDefaultRect ) {
        brect.SetTopLeft( wxPoint(0,0) );
        brect.SetSize( DisplayHelper::GetDisplaySize() );
    }

    if( cfg.m_x >= brect.GetRight() || cfg.m_x <= brect.GetLeft() ) cfg.m_x = 50;
    if( cfg.m_y >= brect.GetBottom() || cfg.m_y <= brect.GetTop() ) cfg.m_y = 50;

    if( cfg.m_width > brect.GetWidth() ) cfg.m_width = 500;
    if( cfg.m_height > brect.GetHeight() ) cfg.m_height = 400;

    wxWindow * win = wxDynamicCast(frame, wxWindow);

    if(win) {
        win->SetSize( cfg.m_width, cfg.m_height );
        win->Move( cfg.m_x, cfg.m_y );
    }

    wxTopLevelWindow * topwin = wxDynamicCast(frame, wxTopLevelWindow);

    if(topwin) {
        topwin->Maximize( cfg.m_isMaximized );
        topwin->ShowFullScreen( cfg.m_isFullScreen );
    }
}

void ConfigMgrFrameLayout::load(wxObject* obj)
{
    Logger::log(LOG_MSG,_T("ConfigMgrFrameLayout::load START 10"));
    MainGuiConfig* config = dynamic_cast<MainGuiConfig*>( obj );
    if(!config){
         Logger::log(LOG_MSG,_T("ConfigMgrFrameLayout::load START 20 cast failed"));
        return;
    }
    Logger::log(LOG_MSG,_T("ConfigMgrFrameLayout::load START 30"));

    m_configBase->setPath(m_path + m_name);
    m_configBase->read(_T("x"), &config->m_x, 50 );
    m_configBase->read(_T("y"), &config->m_y, 50 );
    m_configBase->read(_T("width"), &config->m_width, 500 );
    m_configBase->read(_T("height"), &config->m_height, 100 );
    long tmp;
    m_configBase->read(_T("maximized"), &tmp, 0 );
    config->m_isMaximized = tmp;
    m_configBase->read(_T("fullscreen"), &tmp, 0 );
    config->m_isFullScreen = tmp;
}

void ConfigMgrFrameLayout::save(wxObject* obj)
{
    MainGuiConfig* config = dynamic_cast<MainGuiConfig*>( obj );
    if(!config) return;

    m_configBase->setPath(m_path + m_name);
    m_configBase->write( _T("fullscreen"), wxString::Format(_T("%i"), config->m_isFullScreen) );
    if( !config->m_isFullScreen ){
        m_configBase->write( _T("maximized"), wxString::Format(_T("%i"), config->m_isMaximized) );
        if( !config->m_isMaximized ) {
            m_configBase->write( _T("x"), wxString::Format(_T("%i"), config->m_x) );
            m_configBase->write( _T("y"), wxString::Format(_T("%i"), config->m_y) );
            m_configBase->write( _T("width"), wxString::Format(_T("%i"), config->m_width) );
            m_configBase->write( _T("height"), wxString::Format(_T("%i"), config->m_height) );
        }
    }

    m_configBase->flush();
    m_configBase->save();
}

void ConfigMgrFrameLayout::saveFrom(wxObject* frame_withRTTI)
{
    wxString classname = frame_withRTTI->GetClassInfo()->GetClassName();

    saveFrom(frame_withRTTI,classname);
}

void ConfigMgrFrameLayout::saveFrom(wxObject* frame, const wxString& name)
{
    m_name = name;
    MainGuiConfig cfg;
    wxWindow * win = wxDynamicCast(frame, wxWindow);
    if(win) {
        wxRect winrect = win->GetRect();
        cfg.setRect(winrect);
    }

    wxTopLevelWindow * topwin = wxDynamicCast(frame, wxTopLevelWindow);

    if(topwin) {
        cfg.m_isFullScreen = topwin->IsFullScreen();
        cfg.m_isMaximized = topwin->IsMaximized();
    }

    save(&cfg);
}
