#include "SerialConfigMgr.h" 

#include "../ConfigBase.h"
#include "../serialconfig.h"
#include "../logger.h"

SerialConfigMgr::SerialConfigMgr(ConfigBase* configBase, const wxString& path)
:ConfigMgrBase(configBase)
{
    //ctor
    m_path = path;
}

SerialConfigMgr::~SerialConfigMgr()
{
    //dtor
}

void SerialConfigMgr::load(wxObject* obj)
{
    Logger::log(LOG_MSG,_T("SerialConfigMgr::load START 10"));
    SerialConfig* serialConfig = dynamic_cast<SerialConfig*>( obj );
    if(!serialConfig){
         Logger::log(LOG_MSG,_T("SerialConfigMgr::load START 20 cast failed"));
        return;
    }
    Logger::log(LOG_MSG,_T("SerialConfigMgr::load START 30"));
    getConfig(m_path, *serialConfig);
/*
    try{
        <SerialConfig> serialConfig = dynamic_cast<<SerialConfig> &>( obj );
        getConfig(m_path, serialConfig);
    }catch(){
    exception std::bad_cast (from the <typeinfo> header).
    }
*/
}

void SerialConfigMgr::save(wxObject* obj)
{
    SerialConfig* serialConfig = dynamic_cast<SerialConfig*>( obj );
    if(!serialConfig) return;

    m_configBase->setPath( m_path );
    m_configBase->write( _T("port"), serialConfig->getPortName() );
    m_configBase->write( _T("baudrate"), wxString::Format(_T("%i"), serialConfig->getBaudRate()) );
    m_configBase->write( _T("enabled"), wxString::Format(_T("%i"), serialConfig->isEnabled()) );
    m_configBase->setPath( _T("/") );
    m_configBase->flush();
    m_configBase->save();
}



void SerialConfigMgr::getConfig(const wxString& path, SerialConfig& serialConfig)
{

    wxString portName = m_configBase->getValue(path, _T("port"), _T("COM1") );
    wxString portRate = m_configBase->getValue(path, _T("baudrate"), _T("9600") );
    wxString enabled = m_configBase->getValue(path, _T("enabled"), _T("1") );

    serialConfig.setPortName(portName);

	long baudRate = wxAtoi(portRate);
    serialConfig.setBaudRate(baudRate);

	long isEnabled = wxAtoi(enabled);
	serialConfig.setEnabled(isEnabled!=0?true:false);

	Logger::logf(LOG_MSG, _T("SerialConfigMgr::getConfig(): path: ") +path+ _T(" port: ")+portName);
	Logger::logf(LOG_MSG, _T("SerialConfigMgr::getConfig(): baud:%i"), baudRate);
	Logger::logf(LOG_MSG, _T("SerialConfigMgr::getConfig(): enabled:%i"), isEnabled);

}
