#ifndef BUBBLESTATUSBAR_H
#define BUBBLESTATUSBAR_H

#include <wx/wx.h>
#include "../common/types/wxColorHash.h"

//#define DefaultBubbleState (const)BubbleStateDesc(wxColor(0,0,0,255),wxColor(0,0,0,0),wxColor(0,0,0,255))

class StatusBubble;

WX_DEFINE_ARRAY(StatusBubble*, StatusBubbleArray);

class BubbleStateDesc
{
    public:
        BubbleStateDesc() {
            m_fgColor = wxColor(0,0,0,0);
            m_bgColor = wxColor(0,0,0,0);
            m_borderColor = wxColor(0,0,0,0);
            m_borderSize = 0.0;
            m_roundedSize = 0.0;
        };
        BubbleStateDesc(const wxColor& fgColor, const wxColor& bgColor = wxColor(0,0,0,0), const wxColor& borderColor = wxColor(0,0,0,255), double bordersize = 1.0, double roundedsize = 2.0){
            m_fgColor = fgColor;
            m_bgColor = bgColor;
            m_borderColor = borderColor;
            m_borderSize = bordersize;
            m_roundedSize = roundedsize;
        };

        wxColor& GetBackgroundColor() { return m_bgColor; }
        wxColor& GetForegroundColor() { return m_fgColor; }
        wxColor& GetBorderColor() { return m_borderColor; }
        double   GetBorderSize() { return m_borderSize; }
        double   GetRoundedSize() { return m_roundedSize; }

        void SetBackgroundColor(const wxColor& c) { m_bgColor = c; }
        void SetForegroundColor(const wxColor& c) { m_fgColor = c; }
        void SetBorderColor(const wxColor& c) { m_borderColor = c; }
        void SetBorderSize(double s) { m_borderSize = s; }
        void SetRoundedSize(double s) { m_roundedSize = s; }

    private:

        wxColor     m_fgColor;
        wxColor     m_bgColor;
        wxColor     m_borderColor;
        double      m_borderSize;
        double      m_roundedSize;

};

extern BubbleStateDesc DefaultBubbleState;

WX_DECLARE_STRING_HASH_MAP(BubbleStateDesc,BubbleStateDescHash);

class BubbleStatusBar
{
    public:

        BubbleStatusBar();
        ~BubbleStatusBar();

        StatusBubble* AddBubble(const wxString& symbol, int align, BubbleStateDescHash statedescs = BubbleStateDescHash() );
        StatusBubble* InsertBubble(int idx, const wxString& symbol, int align, BubbleStateDescHash statedescs = BubbleStateDescHash() );

        bool RemoveBubble(size_t idx);
        bool RemoveBubble(const wxString& symbol);

        void SetBarRect(const wxRect& rect){ m_rect = rect; };
        void SetBarRect(int x, int y, int w, int h){ m_rect = wxRect(x,y,w,h); };


        void DrawStatusBar(wxDC& dc);

        void SetParent(wxWindow * win) { m_parent = win; }

        void ReDrawBubbles();

        void SetPadding(int p) { m_padding = p; };
        void SetSpacing(int s) { m_spacing = s; };


        void RefreshParent() { if(m_parent) m_parent->Refresh(false); } ;

        bool SetBubbleState(const wxString& symbol, const wxString& state, bool refreshParent = true);
        bool SetBubbleText(const wxString& symbol, const wxString& text, bool refreshParent = true);
        bool SetBubbleStateAndText(const wxString& symbol, const wxString& state,const wxString& text, bool refreshParent = true);

    private:

        wxWindow * m_parent;

        wxRect  m_rect;

        StatusBubbleArray   m_bubbles;

        int m_padding;
        int m_spacing;

};



class StatusBubble
{
    public:
        StatusBubble(const wxString& name, int height, int align, BubbleStatusBar* mgr);
        ~StatusBubble();

        bool SetStateAndText(const wxString& state,const wxString& text, bool refreshParent = true);

        bool SetState(const wxString& state, bool refreshParent = true);
        void SetText(const wxString& text, bool refreshParent = true);

        int Draw();

        BubbleStateDesc& GetStateDesc(const wxString& state);

        wxBitmap* GetBitmap() { return m_bitmap; }

        void SetAlign(int a) { m_align = a; };
        int GetAlign() { return m_align; };

        wxString& GetText() { return m_text; };


        void SetStateDescriptors(BubbleStateDescHash states) { m_stateDescs = states; };

        void SetName(const wxString& name) { m_name = name; };
        wxString& GetName() { return m_name; };

        int GetWidth() { return m_width; };

        void SetHeight(int height) { m_height = height; }

    private:
        int         m_align;
        int         m_height;
        int         m_width;

        wxBitmap *  m_bitmap;
        wxString    m_text;
        wxString    m_state;
        wxString    m_name;

        BubbleStateDescHash m_stateDescs;

        BubbleStatusBar*    m_mgr;
};

#endif // BUBBLESTATUSBAR_H
