#include "InfoWindow.h"
#include "InfoWindowHelper.h"
#include "../DirHelper.h"
#include "../appHelper.h"

Stacker InfoWindow::m_stacker;
int InfoWindow::m_screenWidth = -1;
int InfoWindow::m_screenHeight = -1;
std::list<wxString> InfoWindow::m_active_messages;

BEGIN_EVENT_TABLE( InfoWindow, wxFrame )
    EVT_TIMER( -1, InfoWindow::OnTimer )
    EVT_MOTION( InfoWindow::OnMove )
    EVT_LEFT_DOWN( InfoWindow::OnClick )
    EVT_RIGHT_DOWN( InfoWindow::OnClick )
END_EVENT_TABLE()

/** static */
void InfoWindow::Display( wxWindow* parent, const wxString& title, const wxString& message, int flags, unsigned int delay, unsigned int fadein_time )
{
    if ( std::find( m_active_messages.begin(), m_active_messages.end(), message) != m_active_messages.end() )
    {
        // dont display a visible message again
        return;
    }
    // wxWindow* p = ( parent == NULL ) ? appHelper::getMainFrameWindow() : parent;
    wxWindow* p = parent;
    new InfoWindow( p, title, message, flags, delay, fadein_time );
}

/* */
InfoWindow::InfoWindow( wxWindow* parent, const wxString& title, const wxString& message, int flags, unsigned int delay, unsigned int fadein_time )
        : m_timer(this ), m_status( 0 ), m_delay(delay), m_fadeout( 2 )
{
    int msglen = 10 * message.length();
    int titlelen = 12 * title.length() + 28 * 2 ; // 2x icons
    wxSize minsize = ( msglen < titlelen ) ? wxSize(titlelen,-1) : wxDefaultSize;

    if(parent!=NULL)
		Create( parent, wxID_ANY, wxEmptyString, wxPoint( -21, -21 ), wxDefaultSize, wxSIMPLE_BORDER | wxWS_EX_TRANSIENT | wxCLIP_CHILDREN | wxSTAY_ON_TOP | wxFRAME_NO_TASKBAR | wxFRAME_FLOAT_ON_PARENT );
	else
		Create( parent, wxID_ANY, wxEmptyString, wxPoint( -21, -21 ), wxDefaultSize, wxSIMPLE_BORDER | wxWS_EX_TRANSIENT | wxCLIP_CHILDREN | wxSTAY_ON_TOP | wxFRAME_NO_TASKBAR );

    wxInitAllImageHandlers();

    #ifdef WIN32
    m_scrollmillis = 50;
    #else
    m_scrollmillis = 50;
    #endif

    wxString iconname;

    wxString vemsIcon = DirHelper::GetPath( dICON ) + _T("vemslogo-icon-24.png");

    // action
    m_fadein = flags & IW_FADE_IN;
    m_moving = (flags & IW_MOVE)!=0?true:false;
    // default action: moving
    if ( m_fadein == false && m_moving == false )
    {
        m_moving = true;
    }

    // colors
    if ( flags & IW_ERROR )
    {
        m_titleBackground.Set( 82, 0, 0 );
        m_textBackground.Set( 255, 255, 255 );
        m_textColor.Set( 82, 0, 0 );
        iconname = _T("ul-error-l.png");
    }
    else
    {
        if ( flags & IW_WARNING )
        {
            m_titleBackground.Set( 216, 143, 1 );
            m_textBackground.Set( 255, 255, 255 );
            m_textColor.Set( 0, 0, 0 );
            iconname = _T("ul-warning-l.png");
        }
        // default: IW_MESSAGE
        else
        {
            m_titleBackground.Set( 32, 83, 188 );
            m_textBackground.Set( 255, 255, 255 );
            m_textColor.Set( 40, 44, 163 );
            iconname = _T("ul-info-l.png");
        }
    }


    m_it = m_active_messages.insert( m_active_messages.begin(), message );

    wxBoxSizer* bs = new wxBoxSizer( wxVERTICAL );

    wxWindow* o = NULL;

    #ifndef WIN32

        wxBoxSizer* pbs = new wxBoxSizer( wxVERTICAL );

        wxPanel* pnl = new wxPanel( this, -1, wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE );
        pnl->SetBackgroundColour( m_titleBackground );

        OwnStaticText* titleC = new OwnStaticText( pnl, -1, title, wxDefaultPosition, minsize, wxALIGN_CENTRE );

        pbs->Add( titleC, 0, wxEXPAND|wxALL|wxALIGN_CENTER, 5 );
        pnl->SetSizer(pbs);
        pbs->SetSizeHints(pnl);

        o = pnl;
    #else
        OwnStaticText* titleC = new OwnStaticText( this, -1, title, wxDefaultPosition, minsize, wxALIGN_CENTRE );
        titleC->SetBackgroundColour( m_titleBackground );
        o = titleC;
    #endif

    titleC->SetForegroundColour( *wxWHITE );
    titleC->SetFont( wxFont( 10, wxDEFAULT, wxNORMAL, wxBOLD ) );

    bs->Add( o, 0, wxGROW | wxALIGN_CENTER_VERTICAL, 5 );

    OwnStaticText* text = new OwnStaticText( this, -1, message, wxDefaultPosition, wxDefaultSize, 0 );

    text->SetForegroundColour( m_textColor );
    text->SetBackgroundColour( m_textBackground );

    bs->Add( text, 0, wxALIGN_CENTER | wxALL, 12 );

    SetBackgroundColour( m_textBackground );
    SetSizer( bs );

    bs->SetSizeHints( this );
    Layout();

    iconname = DirHelper::GetPath( dICON ) + iconname;

    #ifdef WIN32
    wxPoint p( 4, o->GetRect().GetBottom() - 13 );
    wxPoint p2( o->GetRect().GetRight() - 28, o->GetRect().GetBottom() - 13 );
    IconBitmap* icon = new IconBitmap( this, -1, p, wxSize( 24, 24 ), iconname );
    IconBitmap* icon2 = new IconBitmap( this, -1, p2, wxSize( 24, 24 ), vemsIcon );
    icon->SetColors( m_titleBackground, m_textBackground );
    icon2->SetColors( m_titleBackground, m_textBackground );
    #endif

    wxCoord w, h;
    GetClientSize( &w, &h );

    m_pos = m_stacker.StackMe( w );

    if ( m_screenWidth == -1 )
        m_screenWidth = wxSystemSettings::GetMetric( wxSYS_SCREEN_X );

    if ( m_screenHeight == -1 )
        m_screenHeight = wxSystemSettings::GetMetric( wxSYS_SCREEN_Y );

    m_left = m_screenWidth - m_pos;
    m_hMin = m_screenHeight - h;
    m_top = m_screenHeight;

    Move( m_left, m_top );

    m_alpha = 0;

    Show();

    m_timer.Start( fadein_time, false );
}


InfoWindow::~InfoWindow()
{

    m_stacker.ReleaseMe( m_pos );
    m_active_messages.erase( m_it );
}

void InfoWindow::OnTimer( wxTimerEvent& event )
{
    switch( m_status )
    {
        case 0:
            if ( m_fadein )
            {
                SetTransparent( m_alpha );
            }
            else m_alpha = 255;

            if ( m_moving == false )
            {
                Move( m_left, m_hMin );
                m_top = m_hMin;
            }

            m_status = 1;
            m_timer.Start( m_scrollmillis, false );

            break;

        case 1:
            if ( m_fadein )
            {
                SetTransparent( m_alpha );
                m_alpha += 10;
                if ( m_alpha>255 ) m_alpha = 255;
            }
            if ( m_moving )
            {
                m_top -= 6;
                if ( m_top <= m_hMin ) m_top = m_hMin;
                Move( m_left, m_top);
            }

            if( m_top <= m_hMin && m_alpha == 255 )
            {
                m_status = 2;
                m_timer.Start( m_delay, true );
            }

            break;

        case 2:

            m_status = 3;
            m_timer.Start( m_scrollmillis, false );

            break;

        case 3:

            if ( m_fadein )
            {
                m_alpha -= 10;
                if ( m_alpha < 0 ) m_alpha = 0;
                SetTransparent( m_alpha );
            }
            if ( m_moving )
            {
                m_top += m_fadeout;
                Move( m_left, m_top );
            }
            if( m_top > m_screenHeight && m_alpha == 0 )
            {
                Hide();
                Destroy();
            }

            break;
    }
};

void InfoWindow::OnMove( wxMouseEvent& event )
{
    if ( m_status == 2)
    {
        m_timer.Start( m_delay, true );
    }
}

void InfoWindow::OnClick( wxMouseEvent& event )
{
    m_fadeout = 6;
    m_status = 3;
    m_timer.Start( m_scrollmillis, false );
}
