#include "DirHelper.h"

#include <wx/filename.h>
#include <wx/tokenzr.h>

#define ICONDIRNAME              _T("icons")
#define GRAPHICSDIRNAME          _T("graphics")
//#define LOGDIRNAME              _T("log")
#define DEBUGLOGDIRNAME          _T("logs")
#define LOGDIRNAME               _T("VEMS_Files")
#define CACHEDIRNAME             _T("cache")
#define CONFIGDIRNAME            _T("config")
#define PROJECTSDIRNAME          _T("vemsprojects")
#define WEBCACHEDIRNAME          _T("webcache")
#define CONFIGLETSDIRNAME        _T("configlets")
#define CONFIGFILTERSDIRNAME     _T("configfilters")
#define HOTKEYDATABASESDIRNAME   _T("hotkeys")
#define DEFAULTCONFIGS           _T("defaultfirmwareconfigs")
#define HELPDIRNAME              _T("help")
#define DEFAULTSTATEMENTSDIRNAME _T("defaultstatements")
#define GPSMAPCACHEDIRNAME       _T("maps")
#define GPSMAPTILECACHEDIRNAME   _T("tiles")
#define FONTDIRNAME              _T("fonts")

wxString DirHelper::m_OSPathSeparator = wxFileName::GetPathSeparator();

wxPathStringsHashMap DirHelper::m_paths;

DirHelper::DirHelper()
{
    //ctor
}

DirHelper::~DirHelper()
{
    //dtor
}

wxString DirHelper::Separ()
{
    return m_OSPathSeparator;
}

void DirHelper::SetProgramRootDir(const wxString &dir)
{

    m_paths.clear();
    m_paths[dPROGRAMROOT]   = dir;
    m_paths[dCACHE]         = dir + m_OSPathSeparator + GRAPHICSDIRNAME + m_OSPathSeparator + CACHEDIRNAME;
    m_paths[dICON]          = dir + m_OSPathSeparator + GRAPHICSDIRNAME + m_OSPathSeparator + ICONDIRNAME;
    m_paths[dGRAPHICS]      = dir + m_OSPathSeparator + GRAPHICSDIRNAME;
    m_paths[dWEBCACHE]      = dir + m_OSPathSeparator + WEBCACHEDIRNAME;
    m_paths[dPROJECTS]      = dir + m_OSPathSeparator + PROJECTSDIRNAME;
    m_paths[dCONFIG]        = dir + m_OSPathSeparator + CONFIGDIRNAME;
    m_paths[dLOG]           = dir + m_OSPathSeparator + LOGDIRNAME;

    m_paths[dFONTS]         = dir + m_OSPathSeparator + FONTDIRNAME;

    m_paths[dGPSMAPSCACHE]  = m_paths[dCACHE] + m_OSPathSeparator + GPSMAPCACHEDIRNAME;
    m_paths[dGPSMAPTILESCACHE] = m_paths[dGPSMAPSCACHE] + m_OSPathSeparator + GPSMAPTILECACHEDIRNAME;

    //LogDirManager::SetDbPath(GetPath(dCONFIG));
    //m_paths[dWORKINGDIR]           = LogDirManager::GetDefaultWorkDir();
    m_paths[dWORKINGDIR]    = dir + m_OSPathSeparator + LOGDIRNAME;

    m_paths[dCONFIGLETS]    = dir + m_OSPathSeparator + CONFIGLETSDIRNAME;
    m_paths[dCONFIGFILTERS] = dir + m_OSPathSeparator + CONFIGFILTERSDIRNAME;
    m_paths[dHOTKEYS]       = dir + m_OSPathSeparator + CONFIGDIRNAME + m_OSPathSeparator + HOTKEYDATABASESDIRNAME;
    m_paths[dDEFAULTCONFIGS]= dir + m_OSPathSeparator + DEFAULTCONFIGS;
    m_paths[dDEBUGLOG]      = dir + m_OSPathSeparator + DEBUGLOGDIRNAME;
    m_paths[dHELP]          = dir + m_OSPathSeparator + HELPDIRNAME;

    m_paths[dDEFAULTSTATEMENTS] = dir + m_OSPathSeparator + CONFIGDIRNAME + m_OSPathSeparator + DEFAULTSTATEMENTSDIRNAME;


    //CheckDirs();
}

bool DirHelper::MkDir(const wxString& path, int perm)
{
    if(wxFileName::DirExists(path)) return false;

#ifdef __WIN32__
    #ifdef __WINCE_STANDARDSDK__
        CreateDirectory(path,NULL);
    #else
        wxMkDir(path);
    #endif
#else
    wxMkDir(path.mb_str(),perm);
#endif
	return true;
}

void DirHelper::CheckDir(const wxString& dirpath)
{
    wxArrayString pathparts = wxStringTokenize(dirpath,m_OSPathSeparator);

    if(pathparts.GetCount() < 1) return;

    wxString actualpath = pathparts[0];

    for(size_t a = 1; a < pathparts.GetCount(); a++) {
        actualpath += m_OSPathSeparator + pathparts[a];

        if(!wxFileName::DirExists( actualpath ))
            MkDir( actualpath );
    }
}

void DirHelper::CheckDirs()
{
    for(wxPathStringsHashMap::iterator it = m_paths.begin(); it != m_paths.end(); it++ ) {

        // do not create VEMS_Files in program root dir
        if ( ( it->first == dLOG ) || ( it->first == dWORKINGDIR ) )
            continue;

        wxString path = it->second;

        CheckDir( path );
    }
}

void DirHelper::setLogdirPath(long id, const wxString& name)
{
    if ( id == 0 ) return;
	wxString logDir;
	if(!!name)
		logDir = GetPath(dWORKINGDIR) + name + wxString::Format(_T("-%i"), id);
	else
		logDir = GetPath(dWORKINGDIR) + wxString::Format(_T("%i"), id);

    SetPath(dLOG, logDir);
}

void DirHelper::setLogdirPath( const wxString& id, const wxString& name)
{
    if ( id == wxEmptyString ) return;
    wxString logDir;

    if (!!name)
        logDir = GetPath(dWORKINGDIR) + name + _T("-") + id;

    SetPath(dLOG, logDir);
}

void DirHelper::SetPath(DirHelperTypes type, wxString path)
{
    if ( wxDirExists(path) ){
        m_paths[type] = path;
    }else{
#ifdef __WIN32__
#ifdef __WINCE_STANDARDSDK__
			CreateDirectory(path,NULL);
#else
			wxMkDir(path);
#endif
#else
            wxMkDir(path.mb_str(),0777 );   // octal!!
#endif
         if ( wxDirExists(path) ){
            m_paths[type] = path;
         }
    }
}

wxString DirHelper::GetPath(DirHelperTypes type, bool WithPathSeparatorOnEnd)
{
    if(m_paths.find(type) == m_paths.end()) return wxEmptyString;

    wxString tmp = m_paths[type];

    if(WithPathSeparatorOnEnd)
        tmp += m_OSPathSeparator;

    return tmp;
}


wxString DirHelper::CreateNewFileName(const wxString &defaultFileName, const wxString &path, bool isTimeStamp, bool isDateStamp)
{
    wxString TimeStamp = _T("");
    wxString DateStamp = _T("");

    wxDateTime tnow = wxDateTime::Now();


    if(isTimeStamp) {
        TimeStamp = wxString::Format(_T("-%.2i.%.2i.%.2i"),tnow.GetHour(),tnow.GetMinute(),tnow.GetSecond());
    }

    if(isDateStamp) {
        DateStamp = wxString::Format(_T("-%i.%.2i.%.2i"),tnow.GetYear(),tnow.GetMonth()+1,tnow.GetDay());
    }

    wxFileName tmp = wxFileName();
    wxString tmp1, tmp2, tmp3;
    tmp.SplitPath(defaultFileName, &tmp1, &tmp2, &tmp3);

    wxString tmpFileName  =  tmp2 + DateStamp + TimeStamp;
    wxString tmpFileExt = _T("");
    if(!(tmp3.IsEmpty()))
        tmpFileExt = _T(".") + tmp3;

    wxString tmpFileNameC = tmpFileName + tmpFileExt;
    int i=0;

    while(wxFileExists(path+tmpFileNameC)){
        tmpFileNameC = tmpFileName + wxString::Format(_T("-%i"),i) + tmpFileExt;
        i++;
    }

    //delete tmp1; delete tmp2; delete tmp3;

    return wxString(path + tmpFileNameC);
}
