/***************************************************************
 * Name:      HotkeyManagerWizardMain.cpp
 * Purpose:   Code for Application Frame
 * Author:     ()
 * Created:   2010-07-08
 * Copyright:  ()
 * License:
 **************************************************************/

#include <wx/filename.h>

#include "HotkeyManagerWizardMain.h"
#include "DirHelper.h"
#include "wx/wxsqlite3.h"

BEGIN_EVENT_TABLE(HotkeyManagerWizardFrame, wxFrame)
    EVT_CLOSE(HotkeyManagerWizardFrame::OnClose)
    EVT_MENU(idMenuQuit, HotkeyManagerWizardFrame::OnQuit)
    EVT_MENU(idMenuAbout, HotkeyManagerWizardFrame::OnAbout)

    EVT_TEXT(idTargetClassCtrl, HotkeyManagerWizardFrame::OnTargetClassUpdated)

    EVT_BUTTON(idButtonBrowse, HotkeyManagerWizardFrame::OnButton)
    EVT_BUTTON(idButtonClose, HotkeyManagerWizardFrame::OnButton)
    EVT_BUTTON(idButtonOK, HotkeyManagerWizardFrame::OnButton)

END_EVENT_TABLE()

HotkeyManagerWizardFrame::HotkeyManagerWizardFrame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE | wxTAB_TRAVERSAL )
{

    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

#if wxUSE_MENUS
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idMenuQuit, _("&Quit\tAlt-F4"), _("Quit the application"));
    mbar->Append(fileMenu, _("&File"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuAbout, _("&About\tF1"), _("Show info about this application"));
    mbar->Append(helpMenu, _("&Help"));

    SetMenuBar(mbar);
#endif // wxUSE_MENUS

#if wxUSE_STATUSBAR
    // create a status bar with some information about the used wxWidgets version
    CreateStatusBar(2);
    SetStatusText(_("Hello bello!"),0);
    SetStatusText( _T("dummy"), 1);
#endif // wxUSE_STATUSBAR

    BuildGUI();

}

HotkeyManagerWizardFrame::~HotkeyManagerWizardFrame()
{
}

void HotkeyManagerWizardFrame::OnTargetClassUpdated(wxCommandEvent& event)
{
    wxString str = m_targetClassName->GetValue();

    m_symbolsFilename->SetValue(str.Lower()+_T("events.db"));

    m_profileSymbolsFilename->SetValue(str.Lower()+_T("hkeys.db"));
}

void HotkeyManagerWizardFrame::BuildGUI()
{
    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    wxStaticBoxSizer* sb1 = new wxStaticBoxSizer(wxVERTICAL,this,_T("Target properties"));


        m_titleCtrl = new wxTextCtrl(this, wxID_ANY, _T(""));
        m_targetClassName = new wxTextCtrl(this, idTargetClassCtrl, _T(""));

        m_symbolsFilename = new wxTextCtrl(this, wxID_ANY, _T(""));

        wxFlexGridSizer* tpropsizer = new wxFlexGridSizer(2);
        tpropsizer->AddGrowableCol(1);

        tpropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Title:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
        tpropsizer->Add( m_titleCtrl , 0, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, 5 );
        tpropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Class symbol:\n(for example the name of the class)")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
        tpropsizer->Add( m_targetClassName , 0, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, 5 );

        tpropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Eventsymbols table file:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
        tpropsizer->Add( m_symbolsFilename , 0, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, 5 );

        sb1->Add(tpropsizer,0,wxALL | wxEXPAND,5);


    wxStaticBoxSizer* sb2 = new wxStaticBoxSizer(wxVERTICAL,this,_T("Profiles"));


        m_profileTitleCtrl = new wxTextCtrl(this, wxID_ANY, _T("Default"));
        m_profileSymbolsFilename = new wxTextCtrl(this, wxID_ANY, _T(""));

        wxFlexGridSizer* ppropsizer = new wxFlexGridSizer(2);
        ppropsizer->AddGrowableCol(1);

        ppropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Title:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
        ppropsizer->Add( m_profileTitleCtrl , 0, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, 5 );

        ppropsizer->Add(new wxStaticText(this, wxID_ANY, _T("Profile file:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
        ppropsizer->Add( m_profileSymbolsFilename , 0, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, 5 );

        sb2->Add(ppropsizer,0,wxALL | wxEXPAND,5);

    mainsizer->Add(sb1,0,wxALL | wxEXPAND,5);
    mainsizer->Add(sb2,0,wxALL | wxEXPAND,5);

    wxStaticBoxSizer* sb3 = new wxStaticBoxSizer(wxVERTICAL,this,_T("Path"));

        m_path = new wxTextCtrl(this, wxID_ANY, _T(""));

        wxBoxSizer* pathsizer = new wxBoxSizer(wxHORIZONTAL);

        pathsizer->Add(m_path,1,wxALL | wxALIGN_CENTER_VERTICAL,5);
        pathsizer->Add(new wxButton(this, idButtonBrowse,_T("Browse")), 0, wxALL | wxALIGN_CENTER_VERTICAL,5);

        sb3->Add(pathsizer,0,wxALL|wxEXPAND,5);

    mainsizer->Add(sb3,0,wxALL | wxEXPAND,5);

    wxBoxSizer* buttonsizer = new wxBoxSizer(wxHORIZONTAL);

    buttonsizer->Add(new wxButton(this, idButtonOK, _T("Do it!")), 0, wxALL, 5);
    buttonsizer->Add(new wxButton(this, idButtonClose, _T("Close")), 0, wxALL, 5);

    mainsizer->Add(buttonsizer, 0, wxALL | wxALIGN_CENTER_HORIZONTAL , 5);

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);
}

void HotkeyManagerWizardFrame::OnButton(wxCommandEvent& event)
{
    switch(event.GetId()) {
        case idButtonBrowse:{
            wxDirDialog dlg(this);
            dlg.SetPath( m_path->GetValue() == wxEmptyString ? DirHelper::GetPath() : m_path->GetValue() );
            if( dlg.ShowModal() != wxID_OK ) return;
            m_path->SetValue( dlg.GetPath() );
        }break;

        case idButtonClose:
            Close();
            break;

        case idButtonOK:
            CreateDatabases();
            break;
    }


}

void HotkeyManagerWizardFrame::CreateDatabases()
{
    wxString path = m_path->GetValue();

    if(!wxFileName::DirExists(path)) {
        wxMessageBox(_T("The selected path is not exists!"),_T("Error"),wxICON_ERROR);
        return;
    }

    wxString eventsdb = path + DirHelper::Separ() + m_symbolsFilename->GetValue();
    wxString profiledb = path + DirHelper::Separ() + m_profileSymbolsFilename->GetValue();

    wxString targetsDB = path + DirHelper::Separ() + _T("targets.db");
    wxString profsDB = path + DirHelper::Separ() + _T("profiles.db");

    if(!wxFileName::FileExists(targetsDB)) {
        wxMessageBox(_T("Hotkey targets database is not exists!\n(")+targetsDB+_T(")"),_T("Error"),wxICON_ERROR);
        return;
    }

    if(!wxFileName::FileExists(profsDB)) {
        wxMessageBox(_T("Hotkey profiles database is not exists!\n(")+profsDB+_T(")"),_T("Error"),wxICON_ERROR);
        return;
    }


    if(wxFileName::FileExists(eventsdb)) {
        int res = wxMessageBox(_T("The selected events db file is exists!\nDo you want to overwrite?"),_T("Error"),wxYES_NO | wxICON_QUESTION);
        if(res == wxYES)
            wxRemoveFile(eventsdb);
        else
            return;
    }

    if(wxFileName::FileExists(profiledb)) {
        int res = wxMessageBox(_T("The selected profile db file is exists!\nDo you want to overwrite?"),_T("Error"),wxYES_NO | wxICON_QUESTION);
        if(res == wxYES)
            wxRemoveFile(profiledb);
        else
            return;
    }

    wxSQLite3Database db;
    db.Open(eventsdb);

    db.ExecuteUpdate(_T("CREATE TABLE [hotkeyevent] ( \
                        [event_id] INTEGER  NOT NULL PRIMARY KEY AUTOINCREMENT, \
                        [symbol] VARCHAR(64)  NULL, \
                        [title] VARCHAR(256)  NULL ); "));

    db.Close();


    wxSQLite3Database db2;
    db2.Open(profiledb);

    db2.ExecuteUpdate(_T("CREATE TABLE [hotkeys] ( \
                            [id] INTEGER  NOT NULL PRIMARY KEY AUTOINCREMENT, \
                            [event_id] INTEGER  NULL, \
                            [keystring] VARCHAR(64)  NULL );"));

    db2.Close();


    wxSQLite3Database db3;
    db3.Open(targetsDB);

    db3.ExecuteUpdate(wxString::Format(_T("insert into targets (title,class_symbol,symboltablefile) values ('%s','%s','%s')")
                                            ,m_titleCtrl->GetValue().c_str()
                                            ,m_targetClassName->GetValue().c_str()
                                            ,m_symbolsFilename->GetValue().c_str() ));


    long target_id = db3.GetLastRowId().ToLong();

    db3.Close();

    wxSQLite3Database db4;
    db4.Open(profsDB);

    db4.ExecuteUpdate(wxString::Format(_T("insert into profiles (target_id,title,filename,isuse) values (%i,'%s','%s',%i)")
                                        ,target_id
                                        ,m_profileTitleCtrl->GetValue().c_str()
                                        ,m_profileSymbolsFilename->GetValue().c_str()
                                        ,1 ));

    db4.Close();

    wxString msg = _T("The eventsymbols and profile databases is created. Added the specified target to targets database and the specified profile to the profiles database");

    wxMessageBox(msg,_T("Info"));
}

void HotkeyManagerWizardFrame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void HotkeyManagerWizardFrame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void HotkeyManagerWizardFrame::OnAbout(wxCommandEvent &event)
{

}
