#ifndef SDCARDEVENT_H
#define SDCARDEVENT_H

#include <wx/wx.h>

#include "../common/MMCFilesDesc.h"

extern const wxEventType wxEVT_SDCARD;    /// without this MinGW gives undifened referenced linker error

#define EVT_SDCARD(func) \
    wx__DECLARE_EVT0(wxEVT_SDCARD, SDCardEventHandler(func))

namespace SDCardEventTypes {
    enum  {
        TIMEOUT,

        // search-download
        SEARCHING_PROGRESS,
        SEARCHING_FINISHED,
        DOWNLOADING_START,
        DOWNLOADING_PROGRESS,
        DOWNLOADING_FINISHED,
        SUMMARY_START,
        SUMMARY_PROGRESS,
        SUMMARY_FINISHED,
        INTERRUPTED,

        COMM_FAILED,

        // start-stop-erase
        STARTLOG_FAILED,
        STARTLOG_WAITING,
        LOGGING_STARTED,

        STOPLOG_FAILED,
        LOGGING_STOPPED,

        ERASE_WAITING,
        ERASE_NOTEMPTY,
        ERASE_SUCCESS,
        ERASE_FAILED,

        MSx_FAILED,

        FIRMWARE_VERSION,
    };
}

class SDCardEvent : public wxEvent
{
    public:
        SDCardEvent( int id = 0 );
        virtual wxEvent *Clone() const { return new SDCardEvent(*this); }

        void            SetFiles( const MMCFilesDescA& desc );
        MMCFilesDescA   GetFiles() const;

        void            SetInt( int value );
        int             GetInt() const;

        void            SetString( const wxString& str );
        wxString        GetString() const;

        void            SetVariant( const wxVariant& var );
        wxVariant       GetVariant() const;

        void            SetType( unsigned int type );
        unsigned int    GetType() const;

        void            SetDataType(unsigned int dataType);
        unsigned int    GetDataType()const;

    private:

        DECLARE_DYNAMIC_CLASS( SDCardEvent );

        MMCFilesDescA   m_sdfiles;
        int             m_intvalue;
        wxString        m_stringvalue;
        unsigned int    m_type;
        wxVariant       m_variant;
        unsigned int    m_dataType;
};

typedef void (wxEvtHandler::*SDCardEventFunction)(SDCardEvent&);

#define SDCardEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent(SDCardEventFunction, &func)

#endif // SDCARDEVENT_H
