#ifndef VEMSTABLEEDITOREVENTS_H
#define VEMSTABLEEDITOREVENTS_H
#include "wx/wx.h"

#include <wx/event.h>

extern const wxEventType EVT_TABLE_EDITOR;    /// without this MinGW gives undifened referenced linker error

#define TABLE_EDITOR_EVENT(id, func) \
    wx__DECLARE_EVT1(EVT_TABLE_EDITOR, id, TableEditorEventHandler(func))

class TableEditorEvent : public wxEvent
{
    public:
        enum EventType
        {
            CELL_CHANGED,
            ROW_HEADER_CHANGED,
            ROW_HEADER_MOVED,
            ROW_HEADER_INSERTED,
            COL_HEADER_CHANGED,
            COL_HEADER_MOVED,
            COL_HEADER_INSERTED,
            TABLE_NAME_CHANGED,
            CELL_COPIED,
            UNDO_CELL_MODIFICATION,
            UNDO_ALL_CELLS_MODIFICATION,
            CELL_INTERPOLATED,
            CELLS_INTERPOLATED,
            CELLS_CHANGED,
            WRITE_FW_CONFIG,
            BURN_FW_CONFIG,
            REFRESH_FW_CONFIG,
            IMPORT,
            REMAP,
            CURSOR_MOVED,
        };

    public:
        TableEditorEvent( int id = 0);
        TableEditorEvent( EventType type, int row = 0, int col = 0, double d = 0.0 );

        virtual wxEvent *Clone() const { return new TableEditorEvent(*this); }

        DECLARE_DYNAMIC_CLASS(TableEditorEvent)

        int         row;
        int         col;
        double      value;
        wxString    name;
        bool        readAfterWrite;
        bool        fast;               // true - cell was changed by Q,W,E,R
    protected:
    private:
};

typedef void (wxEvtHandler::*TableEditorEventFunction)(TableEditorEvent&);

#define TableEditorEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent(TableEditorEventFunction, &func)

#endif // VEMSTABLEEDITOREVENTS_H
