#ifndef EXPRESSIONBASE_H
#define EXPRESSIONBASE_H

#include <wx/dynarray.h>

#include "RPNTokenBase.h"

class ExpressionBase : public RPNTokenBase
{
    public:
        enum AssocType { ASSOC_NONE=0, ASSOC_LEFT, ASSOC_RIGHT };
        enum TokenType { NUMBER = 0, OPERATOR, STRING, VARIABLE, FUNCTION, UNKNOWN, LP, RP, COMMA, COLON };

        ExpressionBase()
        {
            m_prec = 0;
            m_assoc = ASSOC_NONE;
            m_type = UNKNOWN;
            m_operator = wxEmptyString;
        }
        virtual ~ExpressionBase();

        virtual void    interpret( RPNStack& queue ) = 0;
        virtual wxString toString() = 0;

        uint32_t        getPrec() { return m_prec; }
        AssocType       getAssoc() { return m_assoc; }
        TokenType       getType() { return m_type; }
        wxString        getOperator() { return m_operator; }
    protected:
        void setPrec( uint32_t prec_in ) { m_prec = prec_in; }
        void setAssoc( AssocType assoc_in ) { m_assoc = assoc_in; }
        void setType( TokenType type_in ) { m_type = type_in; }
        void setOperator( const wxString& op_in ) { m_operator = op_in; }
        void delItem( RPNStackItem** item, ... );
    private:
        uint32_t    m_prec;
        AssocType   m_assoc;
        TokenType   m_type;
        wxString    m_operator;
};

WX_DEFINE_ARRAY( ExpressionBase*, InfixTokenA );

#endif // EXPRESSIONBASE_H
