#include "ExpressionFvGetGPSCourse.h"

#include <math.h>
#include "../DrawHelper.h"
#include "../logger.h"


ExpressionFvGetGPSCourse::ExpressionFvGetGPSCourse()
{
    setType( FUNCTION );
    m_latitude      = 0.0;
    m_longnitude    = 0.0;
    d = 0.0;
}

ExpressionFvGetGPSCourse::~ExpressionFvGetGPSCourse()
{
    //dtor
}

void ExpressionFvGetGPSCourse::interpret( RPNStack& rpnStack )
{
    RPNStackItem* a = rpnStack.pop();
    RPNStackItem* b = rpnStack.pop();

    if( a->getType() == RPNStackItem::NUMBER && b->getType() == RPNStackItem::NUMBER )
    {
        RPNStackItem* item = new RPNStackItem();


        double latitude     = b->getNumber();
        double longnitude   = a->getNumber();

        if( m_latitude != latitude && m_longnitude != longnitude )
        {
            if( m_latitude != 0 && m_longnitude != 0 )
            {
                d = getCourse( m_latitude, m_longnitude, latitude, longnitude );
            }

            m_latitude      = latitude;
            m_longnitude    = longnitude;
        }

        item->setNumber( d );

        rpnStack.push( item );
    }
    else
    {
        // throw exception
    }

    delete a;
    delete b;
}

double ExpressionFvGetGPSCourse::getCourse( double lat1Deg, double lon1Deg, double lat2Deg, double lon2Deg )
{
    //convert latitude and longitude to radians
    double lat1 = degreesToRadians( lat1Deg );
    double lon1 = degreesToRadians( lon1Deg );
    double lat2 = degreesToRadians( lat2Deg );
    double lon2 = degreesToRadians( lon2Deg );

    //compute latitude and longitude differences
    double dlat = lat2 - lat1;
    double dlon = lon2 - lon1;

    double distanceNorth    = dlat;
    double distanceEast     = dlon * cos( lat1 );
/*
    //compute the distance in radians
    dist = Math.Sqrt(distanceNorth * distanceNorth + distanceEast * distanceEast)

    //and convert the radians to meters
    dist = RadiansToMeters(dist)

    //add the elevation difference to the calculation
    Dim dele As Double = CDbl(pt2.ele - pt1.ele)
    dist = Math.Sqrt(dist * dist + dele * dele)
*/
    //compute the course
    double course =fmod(  atan2( distanceEast, distanceNorth ), 2 * M_PI );

    course = radiansToDegrees( course );

    if( course < 0 )
    {
        course += 360.0;
    }

    return course;
}


double ExpressionFvGetGPSCourse::degreesToRadians( double degrees )
{
    return degrees * M_PI / 180.0;
}


double ExpressionFvGetGPSCourse::radiansToDegrees( double radians )
{
    return radians * 180.0 / M_PI;
}

/*
Public Class Form1
    Structure wptType
        Dim lat As Double
        Dim lon As Double
        Dim ele As Double
    End Structure

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Dim Distance As Double = 0
        Dim Course As Double = 0
        Dim Source As wptType
        Dim Destination As wptType
        Source.lat = 56
        Source.lon = 12
        Destination.lat = 56
        Destination.lon = 11
        GetCourseAndDistance(Source, Destination, Course, Distance)
        MsgBox(Course)
    End Sub

    Sub GetCourseAndDistance(ByVal pt1 As wptType, ByVal pt2 As wptType, ByRef course As Double, ByRef dist As Double)
        ' convert latitude and longitude to radians
        Dim lat1 As Double = DegreesToRadians(CDbl(pt1.lat))
        Dim lon1 As Double = DegreesToRadians(CDbl(pt1.lon))
        Dim lat2 As Double = DegreesToRadians(CDbl(pt2.lat))
        Dim lon2 As Double = DegreesToRadians(CDbl(pt2.lon))

        ' compute latitude and longitude differences
        Dim dlat As Double = lat2 - lat1
        Dim dlon As Double = lon2 - lon1

        Dim distanceNorth As Double = dlat
        Dim distanceEast As Double = dlon * Math.Cos(lat1)

        ' compute the distance in radians
        dist = Math.Sqrt(distanceNorth * distanceNorth + distanceEast * distanceEast)

        ' and convert the radians to meters
        dist = RadiansToMeters(dist)

        ' add the elevation difference to the calculation
        Dim dele As Double = CDbl(pt2.ele - pt1.ele)
        dist = Math.Sqrt(dist * dist + dele * dele)

        ' compute the course
        course = Math.Atan2(distanceEast, distanceNorth) Mod (2 * Math.PI)
        course = RadiansToDegrees(course)
        If course < 0 Then
            course += 360
        End If
    End Sub

    Function DegreesToRadians(ByVal degrees As Double) As Double
        Return degrees * Math.PI / 180.0
    End Function

    Function RadiansToDegrees(ByVal radians As Double) As Double
        Return radians * 180.0 / Math.PI
    End Function

    Function RadiansToNauticalMiles(ByVal radians As Double) As Double
        ' There are 60 nautical miles for each degree
        Return radians * 60 * 180 / Math.PI
    End Function

    Function RadiansToMeters(ByVal radians As Double) As Double
        ' there are 1852 meters in a nautical mile
        Return 1852 * RadiansToNauticalMiles(radians)
    End Function

End Class
*/
