#include "ExpressionFvGetGPSDistance.h"

#include <math.h>
#include "../DrawHelper.h"

ExpressionFvGetGPSDistance::ExpressionFvGetGPSDistance()
{
    setType( FUNCTION );
    m_latitude      = 0.0;
    m_longnitude    = 0.0;
}

ExpressionFvGetGPSDistance::~ExpressionFvGetGPSDistance()
{
    //dtor
}

void ExpressionFvGetGPSDistance::interpret( RPNStack& rpnStack )
{
    RPNStackItem* a = rpnStack.pop();
    RPNStackItem* b = rpnStack.pop();

    if( a->getType() == RPNStackItem::NUMBER && b->getType() == RPNStackItem::NUMBER )
    {
        RPNStackItem* item = new RPNStackItem();

        double d = 0.0;
        double latitude     = a->getNumber();
        double longnitude   = b->getNumber();

        if( m_latitude != latitude && m_longnitude != longnitude )
        {
            if( m_latitude != 0 && m_longnitude != 0 )
            {
                d = calculateDistance( m_latitude, m_longnitude, latitude, longnitude );
            }

            m_latitude      = latitude;
            m_longnitude    = longnitude;
        }

        item->setNumber( d );

        rpnStack.push( item );
    }
    else
    {
        // throw exception
    }

    delete a;
    delete b;
}

//http://www.tma.dk/gps/#calcdist
//http://www.codeproject.com/KB/cs/distancebetweenlocations.aspx
double ExpressionFvGetGPSDistance::calculateDistance( double Lat1, double Long1, double Lat2, double Long2 )
{
    /*
        The Haversine formula according to Dr. Math.
        http://mathforum.org/library/drmath/view/51879.html

        dlon = lon2 - lon1
        dlat = lat2 - lat1
        a = (sin(dlat/2))^2 + cos(lat1) * cos(lat2) * (sin(dlon/2))^2
        c = 2 * atan2(sqrt(a), sqrt(1-a))
        d = R * c

        Where
            * dlon is the change in longitude
            * dlat is the change in latitude
            * c is the great circle distance in Radians.
            * R is the radius of a spherical Earth.
            * The locations of the two points in
                spherical coordinates (longitude and
                latitude) are lon1,lat1 and lon2, lat2.
    */
    double dDistance    = 0.0;
    double dLat1InRad   = Lat1 * ( M_PI / 180.0 );
    double dLong1InRad  = Long1 * ( M_PI / 180.0 );
    double dLat2InRad   = Lat2 * ( M_PI / 180.0 );
    double dLong2InRad  = Long2 * ( M_PI / 180.0 );

    double dLongitude   = dLong2InRad - dLong1InRad;
    double dLatitude    = dLat2InRad - dLat1InRad;

    // Intermediate result a.

    double a = pow( sin( dLatitude / 2.0 ), 2.0 ) +
               cos( dLat1InRad ) * cos( dLat2InRad ) *
               pow( sin( dLongitude / 2.0 ), 2.0 );

    // Intermediate result c (great circle distance in Radians).

    double c = 2.0 * atan2( sqrt( a ), sqrt( 1.0 - a ) );

    // Distance.

    // const Double kEarthRadiusMiles = 3956.0;

    const double kEarthRadiusKms = 6376.5;
    dDistance = kEarthRadiusKms * c * 1000.0;

    return dDistance;
}
