#include "ExpressionFvInterpolate3D.h"
#include "VariableData.h"
#include "../VemsTuneHelper.h"
#include "../logger.h"
#include "../common/VemsMatrixHelper.h"

ExpressionFvInterpolate3D::ExpressionFvInterpolate3D()
{
    setType( FUNCTION );
}


void ExpressionFvInterpolate3D::interpret( RPNStack& rpnStack )
{
    RPNStackItem* itemY     = rpnStack.pop();
    RPNStackItem* itemX     = rpnStack.pop();
    RPNStackItem* itemTable = rpnStack.pop();
    RPNStackItem* itemRow   = rpnStack.pop();
    RPNStackItem* itemCol   = rpnStack.pop();


    if (    itemY->getType() != RPNStackItem::NUMBER || itemX->getType() != RPNStackItem::NUMBER ||
            itemTable->getType() != RPNStackItem::STRING || itemRow->getType() != RPNStackItem::STRING ||
            itemCol->getType() != RPNStackItem::STRING )
    {
        delItem( &itemX, &itemY, &itemTable, &itemRow, &itemCol, NULL );
        throw ExpError::VariableException( _T("function interpolate2d parameter mismatch") );
    }

    if ( !VemsTuneHelper::staticVariables )
    {
        delItem( &itemX, &itemY, &itemTable, &itemRow, &itemCol, NULL );
        Logger::logf( LOG_MSG, _T("-----ExpressionFvInterpolate3D::interpret() - VemsTuneHelper::staticVariables is NULL!!!") );
        return;
    }

    VemsMatrix<double> col;

    if( !VemsTuneHelper::staticVariables->getValue( itemCol->getString(), col ) )
    {
		wxString varname = itemCol->getString();
        delItem( &itemX, &itemY, &itemTable, &itemRow, &itemCol, NULL );
        throw ExpError::VariableException( _T("variable not found : ") + varname );
    }

    VemsMatrix<double> row;

    if( !VemsTuneHelper::staticVariables->getValue( itemRow->getString(), row ) )
    {
		wxString varname = itemRow->getString();
        delItem( &itemX, &itemY, &itemTable, &itemRow, &itemCol, NULL );
        throw ExpError::VariableException( _T("variable not found : ") + varname );
    }

    VemsMatrix<double> table;

    if( !VemsTuneHelper::staticVariables->getValue( itemTable->getString(), table ) )
    {
		wxString varname = itemTable->getString();
        delItem( &itemX, &itemY, &itemTable, &itemRow, &itemCol, NULL );
        throw ExpError::VariableException( _T("variable not found : ") + varname );
    }

    double x = itemX->getNumber();
    double y = itemY->getNumber();

    double result = VemsMatrixHelper::interpolate3D( col, row, table, x ,y );

    RPNStackItem* item = new RPNStackItem();
    item->setNumber( result );
    rpnStack.push( item );

    delItem( &itemX, &itemY, &itemTable, &itemRow, &itemCol, NULL );
}

