#include "ExpressionFvTable.h"

#include <wx/arrimpl.cpp>
#include <wx/textfile.h>
#include <wx/tokenzr.h>

#include "../DirHelper.h"
#include "../logger.h"

TableHash ExpressionFvTable::tables;

ExpressionFvTable::~ExpressionFvTable()
{

}

void ExpressionFvTable::interpret( RPNStack& rpnStack )
{
    RPNStackItem * a = rpnStack.pop();
    RPNStackItem * b = rpnStack.pop();

    double result = 0.0;

    if( a->getType() == RPNStackItem::STRING && b->getType() == RPNStackItem::NUMBER )
    {
        wxString    name = a->getString();
        int         key = (int)b->getNumber();

        if( tables.find( name ) == tables.end() )
        {
            if( !doRead( name ) )
            {
                Logger::log( LOG_ERROR, _T("file read error!") );
                return; //throw exception
            }; //throw exception
        }

        result = get( name, key );
    }
    else
    {
        //throw exception parameter mismatch
    }
    delete a;
    delete b;

    RPNStackItem* item = new RPNStackItem();
    item->setNumber( result );
    rpnStack.push( item );
}

bool ExpressionFvTable::doRead( const wxString& fileName )
{
    //const wxString dir = _T("resources");
    //const wxString dir = _T("config");
    wxTextFile file;

    if (!file.Open(DirHelper::GetPath(dCONFIG) + fileName, wxConvLibc)){
        return false; //FC_CANNOT_OPEN_FILE; need exception?
    }

    wxString    line = file.GetFirstLine();
    bool        firstline = true;

    DoubleArray* da = new DoubleArray();

    do{
        if (firstline){
            firstline = false;
        }else{
            line = file.GetNextLine();
        }

        line.Trim(false);
        if(line[0] == _T(';')) continue;

        wxArrayString as = parseLine(line);

        if(as.GetCount() == 4){
            double d = -1;
            as[1].ToDouble(&d);
            da->Add(d);
        }

    }while(!file.Eof());

    tables[fileName] = da;

    return true; //FC_OK;
}

wxArrayString ExpressionFvTable::parseLine( const wxString& line )
{
    wxStringTokenizer   tkz(line, _T(" ;\t"), wxTOKEN_STRTOK);
    wxArrayString       tokens;

    while(tkz.HasMoreTokens()){
        tokens.Add(tkz.GetNextToken());
    }

    return tokens;
}

double ExpressionFvTable::get( const wxString& name, int key )
{
    if(tables.find(name) == tables.end())
    {
        Logger::log( LOG_ERROR, _T("table not found!") );
        return -1; //need exception
    }

    DoubleArray* da = tables[name];

    if(key < 0 || key > (int)da->GetCount() || da->GetCount() == 0)
    {
        Logger::log( LOG_ERROR, _T("invalid key!") );
        return -1; //need exception
    }

    return da->Item(key);
}
