#include "InfixTokenizer.h"
#include "../logger.h"
#include "StringUtil.h"
#include "../Utils.h"

InfixTokenizer::InfixTokenizer()
{
    m_ops.Add( new InfixOperator( _T("*"), 8, InfixOperator::ASSOC_LEFT ) );
    m_ops.Add( new InfixOperator( _T("/"), 8, InfixOperator::ASSOC_LEFT ) );
    m_ops.Add( new InfixOperator( _T("+"), 5, InfixOperator::ASSOC_LEFT ) );
    m_ops.Add( new InfixOperator( _T("-"), 5, InfixOperator::ASSOC_LEFT ) );
    m_ops.Add( new InfixOperator( _T("("), 0, InfixOperator::ASSOC_NONE ) );
    m_ops.Add( new InfixOperator( _T(")"), 0, InfixOperator::ASSOC_NONE ) );

    m_prevType = InfixToken::UNKNOWN;
}

InfixTokenizer::~InfixTokenizer()
{
    //dtor
}


void InfixTokenizer::parse( const wxString& str, InfixTokenA& tokenA_out)
{
    const wxString delim =_T(" ");
    wxArrayString a;

    Utils::tokenizeToArray( (wxString&)str, delim, a);

    for(uint32_t i=0; i < a.GetCount(); i++)
    {
        wxString token = a.Item(i);

        if( getOperator( token, tokenA_out ) )
        {
            continue;
        }
        else if( getNumber( token, tokenA_out ) )
        {
            continue;
        }
        else
        {
            getVariable( token, tokenA_out );
        }

    }
}

bool InfixTokenizer::getNumber( const wxString& str, InfixTokenA& tokenA_out )
{
    double d;

    if( str.ToDouble( &d ) )
    {
        InfixToken* token = new InfixToken();
        token->setType( InfixToken::NUMBER );
        token->setNumber( d );
        token->setValue( str );
        tokenA_out.Add( token );
        m_prevType = InfixToken::NUMBER;
        return true;
    }
    else
    {
        return false;
    }
}

bool InfixTokenizer::getOperator( const wxString& str, InfixTokenA& tokenA_out)
{
    for( uint32_t i = 0; i < m_ops.GetCount(); i++ ){
        wxString op = m_ops[i]->getOp();
        if( str == op )
        {

            InfixToken* token = new InfixToken();
            token->setType( InfixToken::OPERATOR );
            token->setValue( op );
            token->setPrec( m_ops[i]->getPrec() );
            tokenA_out.Add( token );
            m_prevType = InfixToken::OPERATOR;

            return true;
        }
    }

    return false;
}

bool InfixTokenizer::getVariable( const wxString& str, InfixTokenA& tokenA_out)
{
    InfixToken* token = new InfixToken();
    token->setType( InfixToken::VARIABLE );
    token->setValue( str );
    tokenA_out.Add( token );
    m_prevType = InfixToken::VARIABLE;

    return true;
}

/*
void InfixTokenizer::parse( const wxString& str, InfixTokenA& tokenA_out)
{
    if( str.IsEmpty() ){
        return;
    }

    m_string = str;

    while( !m_string.IsEmpty() )
    {
        m_string.Trim( false );

        if( m_string.IsEmpty() )
        {
            continue;
        }

        //Logger::log( LOG_MSG, m_string );

        if( getOperator( tokenA_out) )
        {
            continue;
        }
        else if( getNumber( tokenA_out ) )
        {
            continue;
        }
        else
        {
            m_string = m_string.Mid( 1 );
        }
    }
}
*/

/*
bool InfixTokenizer::getOperator( InfixTokenA& tokenA_out)
{
    for( uint32_t i = 0; i < m_ops.GetCount(); i++ ){
        wxString op = m_ops[i]->getOp();
        if( m_string.StartsWith( op ) )
        {

            InfixToken* token = new InfixToken();
            token->setType( InfixToken::OPERATOR );
            token->setValue( op );
            token->setPrec( m_ops[i]->getPrec() );
            tokenA_out.Add( token );

            m_string = m_string.Mid( op.length() );

            m_prevType = InfixToken::OPERATOR;

            //Logger::logf( LOG_WARNING, _T("InfixTokenizer::getOperator -> Operator found : ") + token->getValue() );

            return true;
        }
    }

    return false;
}
*/

/*
bool InfixTokenizer::getNumber( InfixTokenA& tokenA_out )
{
    wxString number = wxEmptyString;

    while( !m_string.IsEmpty() )
    {
        if( StringUtil::isNumber( m_string[0] ) )
        {
            number += m_string[0];
            m_string = m_string.Mid( 1 );
        }
        else
        {
            break;
        }
    }

    if( number.IsEmpty() )
    {
        return false;
    }
    else
    {
        double d;

        if( number.ToDouble( &d ) )
        {
            InfixToken* token = new InfixToken();
            token->setType( InfixToken::NUMBER );
            token->setNumber( d );
            token->setValue( number );
            tokenA_out.Add( token );
        }

        //hiba
        m_prevType = InfixToken::NUMBER;

        return true;
    }
}
*/

uint32_t InfixTokenizer::getPrec(const wxString& t)
{
    for(uint32_t i = 0; i < m_ops.GetCount(); i++){
            InfixOperator *op =  m_ops.Item(i);
            if( op->getOp() ==  t) { return op->getPrec(); }
    }
    return 0;
}
