#include "ShuntingYard.h"
#include "ExpressionFactory.h"
#include "../logger.h"

ShuntingYard::ShuntingYard()
{
    //ctor
}

ShuntingYard::~ShuntingYard()
{
    //dtor
}

void ShuntingYard::toRpn( InfixTokenA& tokenA_in, ExpressionA& exprA_out )
{
    InfixTokenA opStack;
    ExpressionFactory* expFactory = ExpressionFactory::createFactory( NULL );

    for(uint32_t i = 0; i < tokenA_in.GetCount(); i++)
    {
        //Logger::logf( LOG_WARNING, _T("OperatorStack : ") + opStackToString( opStack ) );

        InfixToken* token = tokenA_in[i];

        if( token->getType() == InfixToken::NUMBER )
        {
            //Logger::logf( LOG_WARNING, _T("ShuntingYard::toRpn -> found number: ") + token->getValue() );
            exprA_out.Add( expFactory->create( token->getValue() ) );
            continue;
        }

        if( token->getType() == InfixToken::VARIABLE )
        {
            Logger::logf( LOG_WARNING, _T("ShuntingYard::toRpn -> found variable: ") + token->getValue() );
            exprA_out.Add( expFactory->create( token->getValue() ) );
            continue;
        }



        if( token->getType() == InfixToken::OPERATOR )
        {
            //Logger::logf( LOG_WARNING, _T("ShuntingYard::toRpn -> found operator: ") + token->getValue() );

            if( token->getValue() == _T("(") )
            {
                //Logger::logf( LOG_WARNING, _T("ShuntingYard::toRpn -> found operator: ") + token->getValue() );
                opStack.Add( token );
            }
            else if( token->getValue() == _T(")") )
            {
                //Logger::logf( LOG_WARNING, _T("ShuntingYard::toRpn -> found operator: ") + token->getValue() );
                bool foundLP = false;

                while( !opStack.IsEmpty() )
                {
                    InfixToken* lastToken = opStack.Last();

                    if( lastToken->getValue() != _T("(") )
                    {
                        opStack.RemoveAt( opStack.GetCount()-1 );
                        exprA_out.Add( expFactory->create( lastToken->getValue() ) );
                        continue;
                    }
                    else
                    {
                        opStack.RemoveAt( opStack.GetCount()-1 );
                        foundLP = true;
                        break;
                    }
                }

                if( !foundLP )
                {
                    Logger::log( LOG_WARNING, _T("Missing (") );
                }
            }
            else
            {
                //Logger::logf( LOG_WARNING, _T("ShuntingYard::toRpn -> found operator: ") + token->getValue() );
                while( !opStack.IsEmpty() )
                {
                    InfixToken* lastToken = opStack.Last();

                    if( token->getPrec() <= lastToken->getPrec() )
                    {
                        opStack.RemoveAt( opStack.GetCount()-1 );
                        exprA_out.Add( expFactory->create( lastToken->getValue() ) );
                        continue;
                    }
                    else
                    {
                        break;
                    }

                }

                opStack.Add( token );
            }
        }

   }

   while( !opStack.IsEmpty() )
   {
        exprA_out.Add( expFactory->create( opStack.Last()->getValue() ) );
        opStack.RemoveAt( opStack.GetCount()-1 );
   }

    Logger::logf( LOG_WARNING, _T("ShuntingYard::toRpn -> output: ") + toString( exprA_out ) );

}

wxString ShuntingYard::opStackToString( const InfixTokenA& opStack_in )
{
    wxString str = wxEmptyString;

    for( uint32_t i = 0; i < opStack_in.GetCount(); i++ )
    {
        str += opStack_in.Item( i )->getValue() + _T(" ");
    }

    return str;
}

wxString ShuntingYard::toString( ExpressionA& exprA )
{
    wxString str = wxEmptyString;

    for(uint32_t i = 0; i < exprA.GetCount(); i++ )
    {
        ExpressionBase* eb = exprA.Item(i);
//        str += eb->toString() + _T(" ");
    }

    return str;
}
