#include "ConfigParserMSQ.h"

#include "../firmwareconfig/FirmwareConfigData.h"
#include "../Utils.h"
#include "../logger.h"

ConfigParserMSQ::ConfigParserMSQ(FirmwareConfigData* fcData, bool isHighBoost )
{
    m_ptrFcData = fcData;
    m_isHighBoost = isHighBoost;
    m_inConstant = false;
    m_tableState = FIND_TABLE;
    m_tableRowIdx = 0;
}

ConfigParserMSQ::~ConfigParserMSQ()
{
    //dtor

}

bool ConfigParserMSQ::isEndOfConstant()
{
    if( m_line.StartsWith(_T("</constant>")) ) return true;
    return false;

}

bool ConfigParserMSQ::isStartOfConstant()
{
    if( m_line.StartsWith(_T("<constant")) ) return true;
    return false;
}

void ConfigParserMSQ::parseTableData()
{
    wxArrayString lineTokens;
    Utils::tokenizeToArray( m_line, _T(" "), lineTokens );
    m_ptrFcData->addTableLineMSQ( _T("j"), m_tableRowIdx, lineTokens );
    m_tableRowIdx++;
}

void ConfigParserMSQ::parseAxis(const wxString& axisName)
{
    m_ptrFcData->addTableLineMSQ( axisName, 0, m_tmpAxisToken );
    m_tmpAxisToken.Clear();
}

wxString ConfigParserMSQ::getTableName()
{
///     <constant name="veTable" rows="14" cols="16" units="%" offset="0">
/// return veTable
    int idx = m_line.Find( _T("name=") );
    if(idx == wxNOT_FOUND) return wxEmptyString;

    wxString tmp = m_line.Mid(idx);
    wxString s = tmp.AfterFirst( _T('"') );
    return s.BeforeFirst( _T('"') );
}

wxString ConfigParserMSQ::convertX(const wxString& str)
{
    long value;
    str.ToLong(&value);
    value /= 100;
    wxString newStr = wxString::Format(_T("%i"), value);
    return newStr;
}

wxString ConfigParserMSQ::convertY(const wxString& str)
{
    int divider = 1;
    if(m_isHighBoost){
        divider = 2;
    }

    long value;
    str.ToLong(&value);
    value /= divider;
    wxString newStr = wxString::Format(_T("%i"), value);
    return newStr;

}

void ConfigParserMSQ::parseLine()
{
    m_line.Trim(false);
    m_line.Trim(true);

    if(m_tableState == END) return;

    Logger::logf( LOG_MSG, _T("ConfigParserMSQ::parseLine() %i line: ")+m_line, m_inConstant );
    if(m_inConstant){
        if( isEndOfConstant() ){
            m_inConstant = false;
            switch(m_tableState){
                case TABLE_DATA:
                    m_tableState = X_AXIS_DATA;
                    break;
                case X_AXIS_DATA:
                    Logger::logf( LOG_MSG, _T("ConfigParserMSQ::parseLine() parse j_r[0]:") + m_tmpAxisToken.Item(0));
                    parseAxis(_T("j_r"));
                     m_tableState = Y_AXIS_DATA;
                     break;
                case Y_AXIS_DATA:
                    Logger::logf( LOG_MSG, _T("ConfigParserMSQ::parseLine() parse j_k[0]:") + m_tmpAxisToken.Item(0));
                    parseAxis(_T("j_k"));
                    m_tableState = END;
                    break;
            }
            return;
        }
        /// Parse Table data
        switch(m_tableState){
            case TABLE_DATA:
                parseTableData();
                break;
            case X_AXIS_DATA:
                m_tmpAxisToken.Add(convertX(m_line));
                break;
            case Y_AXIS_DATA:
                m_tmpAxisToken.Add(convertY(m_line));
                break;
        }

    }else{
        if( !isStartOfConstant() ) return;
        /// isStartOfConstant
        m_inConstant = true;
        switch(m_tableState){
           case FIND_TABLE:{
                wxString tableName = getTableName();
                Logger::logf( LOG_MSG, _T("ConfigParserMSQ::parseLine() tableName: ")+tableName );
                if(tableName == _T("veTable")){
                    m_tableState = TABLE_DATA;
                }
                break;
            }
            default:
            break;
        }
    }
}

