#include "V3iniTokenizer.h"
#include <wx/tokenzr.h>

#include "../logger.h"

//#include <wx/arrimpl.cpp>
//WX_DEFINE_OBJARRAY(TokenArray);


Token::Token(wxString s, type t, double d) : str(s), typ(t), num(d)
{
    //Logger::logf( LOG_MSG, _T("Token::Token()") );
}

Token::~Token()
{
    //Logger::logf( LOG_MSG, _T("Token::~Token()") );
}

Tokenizer::Tokenizer()
{
    //Logger::logf( LOG_MSG, _T("Tokenizer::Tokenizer()") );
}

Tokenizer::Tokenizer( const Tokenizer& tokenizer )
{
    for ( size_t i = 0; i < tokenizer.m_tokens.GetCount(); ++i ) {
        Token* token = tokenizer.m_tokens[i];
        m_tokens.Add( new Token( *token ) );
    }
}

Tokenizer::~Tokenizer()
{
    //Logger::logf( LOG_MSG, _T("Tokenizer::~Tokenizer()") );
    WX_CLEAR_ARRAY(m_tokens);
}

size_t Tokenizer::tokenize(const wxString& s)
{
    wxString::const_iterator it = s.begin();

    while(it != s.end()){
        //eat spaces
        if(isspace(*it)){
            it++;
            continue;
        }
        //drop comment
        if(*it == _T(';')) break;


        //if(*it == _T('=') || *it == _T(',')) it++;

        //Syntactic  sugar, ignore it.
        if(*it == _T(',') || *it == _T('='))
        {
            it++;
            continue;
        }

        // Syntax
        if(*it == _T(':'))
        {
            m_tokens.Add( new Token(_T(':'), T_COL) );
            it++;
            continue;
        }

        //# directive
        if(*it == _T('#')){
            m_tokens.Add( new Token(_T('#'), T_DIR) );
            it++;

            while(it != s.end() ){
                if(isspace(*it)){
                    it++;
                    continue;
                }

                wxString str;

                //while( it != s.end() && ( isalnum(*it) || (*it == _T('_')) || (*it == _T('"') ) || (*it == _T('/') ) ) ){
                while( it != s.end() &&  !isspace(*it)  ){
                    str.append(*it);
                    it++;
                }
                m_tokens.Add( new Token(str, T_ID) );
                if(it != s.end()) it++;
            }
            continue;
        }

        //Parse alpha-numeric id.
        if(isalpha(*it))
        {
            wxString str;

            while((isalnum(*it)) || (*it == _T('_')))
            {
                str += *it;
                it++;
            }
            m_tokens.Add( new Token(str, T_ID) );
            //wxLogMessage(wxString::Format(_T("add id : %s"), str.c_str()));
            continue;
        }

        // Floats with no exponent.
        if((isdigit(*it)) || (*it == _T('-')))
        {
            wxString str;
            str += *it;
            it++;

            while(isdigit(*it) || *it == _T('.'))
            {
                str += *it;
                it++;
            }

            double d = 0.0;
            str.ToDouble(&d);
            m_tokens.Add( new Token(str, T_NUM, d) );
            //wxLogMessage(wxString::Format(_T("add num : %s"), str.c_str()));
            continue;
        }

        //Quoted string, quotes removed.
        if(*it == _T('"'))
        {
            it++;
            wxString str;
            while((it != s.end()) && (*it != _T('"'))){

                str += *it;
                it++;
            }

            //if (*s != '"') throw CString("Closing quote not found.");
            if (*it != '"') Logger::log(LOG_MSG,_T("Tokenizer::tokenize() Closing quote not found: ")+s);
            it++; //todo exception handling!!!
            m_tokens.Add( new Token(str, T_STR) );
            //wxLogMessage(wxString::Format(_T("add string : %s"), str.c_str()));
            continue;
        }

        // Block identifier
        if(*it == _T('['))
        {
            it++;
            wxString str;
            while((it != s.end()) && (*it != _T(']')))
            {
                str += *it;
                it++;
            }

            m_tokens.Add( new Token(str, T_BLK) );
            //wxLogMessage(wxString::Format(_T("add block : %s"), str.c_str()));
            continue;
        }

        // Expression
        if(*it == _T('{'))
        {
            it++;
            wxString str;
            while((it != s.end()) && (*it != _T('}')))
            {
                str += *it;
                it++;
            }
            m_tokens.Add( new Token(str, T_EXP));
            //wxLogMessage(wxString::Format(_T("add expr : %s"), str.c_str()));
            continue;
        }

        if(it != s.end()) it++;

    }//while(it != s.end())

    //wxLogMessage(wxString::Format(_T("add line: %s"), line.c_str()));
    //Logger::logf( LOG_MSG, _T("Tokenizer::tokenize() - m_tokens.GetCount(): %d"), m_tokens.GetCount() );
    return getSize();
}
