#include "parseheader.h"
#include "parserutil.h"

#include "../logger.h"

const wxString CONFIG_T_START_STR       = _T("struct config_t {");
const wxString CONFIG_EXT_T_START_STR   = _T("struct config_ext_t {");
const wxString CONFIG_SHARED_T_START_STR   = _T("struct config_shared_t {");
const wxString CONFIG_END_STR           = _T("};");
const wxString UNIT_UINT8_T             = _T("uint8_t");
const wxString UNIT_UINT16_T             = _T("uint16_t");
const wxString UNIT_UINT32_T             = _T("uint32_t");


//const int PAGE_CONFIG_T     = 5;
//const int PAGE_CONFIG_EXT_T = 9;
//const int PAGE_SHARED_T     = 16;

enum State
{
  STATE_IN_CONFIG_T = 5,
  STATE_IN_CONFIG_EXT_T = 9,
  STATE_IN_CONFIG_SHARED_T = 16,
  STATE_FINISHED,
};

ParseHeader::ParseHeader( SymbolTable& symbols )
    : m_symbolTable ( symbols )
{
    m_state     = -1;
    m_offset    = 0;
}

void ParseHeader::init()
{
    m_state     = -1;
    m_offset    = 0;
    m_symbols.clear();
}

void ParseHeader::parseLine()
{
    //Logger::logf( LOG_MSG, _T("ParseHeader::parseLine()") );
    int mult = 1;
    switch( m_state )
    {
        case STATE_FINISHED : return;
        case STATE_IN_CONFIG_T :
        case STATE_IN_CONFIG_EXT_T :
        case STATE_IN_CONFIG_SHARED_T:
        {
            m_line.Trim( false );   //removes leading space

            if ( m_line.StartsWith( CONFIG_END_STR ) ) { m_state = -1; }

            if ( m_line.StartsWith( ParserUtil::COMMENT ) ){ return; }
            ParserUtil::truncateComment( m_line );  // strip comment from the end of the line

            wxString unit, symbolName;
            ParserUtil::getUnitAndValue( m_line, unit, symbolName );
            if ( unit.CompareTo( UNIT_UINT8_T ) == 0 ){
                mult = 1;
            }else if( unit.CompareTo( UNIT_UINT16_T ) == 0){
                mult = 2;
            }else if( unit.CompareTo( UNIT_UINT32_T ) == 0){
                mult = 4;
            }else{
                return;
            }

            m_page = m_state;

            //Logger::logf( LOG_MSG, _T("ParseHeader::parseLine() - page : %d"), page );

            wxString arrayName, sizeStr;

            long size = 1;
            if( ParserUtil::getArrayAndSize( symbolName, arrayName, sizeStr ) == 0){ //array symbol
                sizeStr.ToLong(&size);
                size *= mult;
                //Logger::logf( LOG_MSG, _T("ParseHeader::parseLine() array name: ")+ symbolName+_T(" ")+ arrayName+_T(" size:%d,(%d)"), size, m_offset);
                createSymbol(arrayName, size);

            }else{
                size *= mult;
                //Logger::logf( LOG_MSG, _T("ParseHeader::parseLine() scalar name: ")+ symbolName+_T(" ")+_T(" size:%d,(%d)"), size, m_offset);
                createSymbol(symbolName, size);
            }

            //m_symbolTable
        }break;
        default :
        {
            m_line.Trim( false );   //removes leading space
            if ( m_line.StartsWith( CONFIG_T_START_STR ) )
            {
                m_state     = STATE_IN_CONFIG_T;
                m_offset    = 0;
            }
            else if ( m_line.StartsWith( CONFIG_EXT_T_START_STR ) )
            {
                m_state     = STATE_IN_CONFIG_EXT_T;
                m_offset    = 0;
            }
            else if ( m_line.StartsWith( CONFIG_SHARED_T_START_STR ) )
            {
                m_state     = STATE_IN_CONFIG_SHARED_T;
                m_offset    = 0;
            }
            else { return; }
        }
    }
}

void ParseHeader::addSymbol(const wxString& name)
{
        m_symbols.add( name, _T("") );
        Symbol* symbol = Symbol::createScalar( name, m_page, m_offset );
        m_symbolTable[ name ] = symbol;
        symbolOrder.Add( symbol );
        ++m_offset;
}

void ParseHeader::createSymbol(const wxString& name, size_t size)
{
    if(size == 1){
        addSymbol(name);
        return;
    }

    for ( size_t i = 0; i < size; ++i )
    {
        wxString key;
        key.Printf(_T("[%X]"), i);
        key = name + key;

        addSymbol(key);
    }
}

