#ifndef PARSEHEADER_H
#define PARSEHEADER_H

/** \class ParseHeader
  *
  * \~english
  * \brief  Parse "global.h - config_t struct" scalar and array symbols to a wxArrayString and a SymbolTable
  * \detail struct config_t symbols page = 5
  *         struct config_ext_t symbols page = 9
  *         offset = order in the struct, it starts from 0
  *
  * \~hungarian
  * \brief  Beolvassa a "global.h" allomany "config_t" strukturajanak valtozoit egy string tombbe( wxArrayString )
  *
  */

#include "parserbase.h"
#include "../Symbol.h"
#include "../common/OrderedStringHash.h"

#include <wx/arrstr.h>

class SymbolTable;

class ParseHeader : public ParserBase
{
    public:
        ParseHeader( SymbolTable& symbols );
        virtual ~ParseHeader() {}

        void init();
        void parseLine();
        const wxArrayString& getHeaderArray() { return m_symbols.getArray(); }

        SymbolArray         symbolOrder;
    protected:
    private:
        SymbolTable&        m_symbolTable;
        OrderedStringHash   m_symbols;      //parsed symbols ordered hash
        int                 m_state;
        int                 m_offset;
        int                 m_page;

        void addSymbol(const wxString& name);
        void createSymbol(const wxString& name, size_t size);
};

#endif // PARSEHEADER_H

