#include "parserutil.h"


#include "../logger.h"
#include <wx/regex.h>


IntKeyHash ParserUtil::formatHash;
wxString ParserUtil::COMMENT = _T("//");


ParserUtil::ParserUtil()
{
    //ctor
    formatHash[START_parseHeaderFile] = _T("Start header file parsing...");
    formatHash[END_parseHeaderFile] = _T("Finised header file parsing");

    formatHash[START_parseConfigFile] = _T("Start config file parsing...");
    formatHash[END_parseConfigFile] = _T("Finised config file parsing");

    formatHash[START_parseTableFile] = _T("Start table file parsing...");
    formatHash[END_parseTableFile] = _T("Finised table file parsing");


    formatHash[SYMBOL_ALL_COUNT] = _T("%d symbol found");
    formatHash[SYMBOL_SCALAR_COUNT] = _T("%d scalar symbol found");
    formatHash[SYMBOL_ARRAY_COUNT] = _T("%d array symbol found");

    formatHash[FC_OK] = _T("OK");
    formatHash[FC_INVALID_FILENAME] = _T("Invalid Filename: %s");
    formatHash[FC_CANNOT_OPEN_FILE] = _T("Cannot open file: %s");
    formatHash[FC_INVALID_TABLE_LINE] = _T("Invalid table line: %s found!");
    formatHash[FC_INVALID_TABLE_KEY] = _T("Invalid table: %s found!");
    formatHash[FC_HIGH_ROW_COUNT] = _T("Row length is too high (max/curr:%d/%d) in table:%s!");
    formatHash[FC_ALTERING_ROW_COUNT] = _T("Row length is altering (exp/curr:%d/%d) in table:%s!");
    formatHash[FC_HIGH_COL_COUNT] = _T("Column index is too high (max/curr:%d/%d) in table: %s!");

    formatHash[FC_UNDEFINED_SYMBOL] =_T("Symbol: %s in header file is not found in config.txt!");
    formatHash[FC_EXTRA_SYMBOL] =_T("Symbol: %s in config.txt not found in global.h!");
}

ParserUtil::~ParserUtil()
{
    //dtor
}

int ParserUtil::truncateComment(wxString& line)
{
    int index = line.Find(COMMENT);
    if (  index == -1 ) return -1;
    line.Truncate(index);
    return index;
}

int ParserUtil::getUnitAndValue(const wxString& line, wxString& unit, wxString& value)
{
    wxString regexp = _T("^\\s*(\\S+)\\s+([^;]+)"); // unit8_t cwl;
    return getCommonNameValue(line, regexp, unit, value);

}

// retut  0 if  match
int ParserUtil::getCommonNameValue(const wxString& line, const wxString& regexp, wxString& key, wxString& value)
{

    //fprintf(stdout,"line: %s \n", line.c_str());
    wxRegEx reKV;
    reKV.Compile( regexp, wxRE_ADVANCED); // wxRE_EXTENDED,wxRE_ADVANCED, wxRE_BASIC
    //bool isValid =
    reKV.IsValid();
    //fprintf(stdout,"regexp: %i \n", isValid);
    int res =-1;

    if ( reKV.Matches(line) )
    {
        key = reKV.GetMatch(line, 1);
        value = reKV.GetMatch(line, 2);
        //fprintf(stdout,"111 key: %s value: %s \n", key.c_str(), value.c_str());
        res = 0;
    }
    return res;
}

//line: key=value
int ParserUtil::getKeyAndValue(const wxString& line, wxString& key, wxString& value)
{
    wxString regexp;
    //regexp = _T("([^=]+)=([[:alnum:]]+)");
    regexp = _T("^\\s*([^=]+)=(\\S+)"); // key=value
    return getCommonNameValue(line, regexp, key, value);
}

// retut  0 if  match
int ParserUtil::getArrayAndSize(const wxString& line, wxString& array, wxString& size)
{
    wxString regexp;
    //regexp = _T("([^=]+)=([[:alnum:]]+)");
    regexp = _T("^\\s*([^\\[]+)\\[(\\d+)\\]"); // cwl[23]
    return getCommonNameValue(line, regexp, array, size);
}

//line: name[2,6]
//line: name[A,11]
int ParserUtil::getArrayColumnAndRow(const wxString& line, wxString& col, wxString& row)
{
    wxString tmp = line.AfterFirst('[');
    col = tmp.BeforeFirst(','); //A

    tmp = line.AfterFirst(',');
    row = tmp.BeforeFirst(']'); //11
    return 0;
}

void ParserUtil::addProcessInfo(firmwareConfig_codes code, ...)
{
    // logType level = LOG_MSG; //geLevel(FC_INVALID_FILENAME);

    wxString format = getFormat(code);

    //_T("Cannot open File: %s")+strFile

    va_list argptr;//, argcp;
    va_start(argptr, code);
    //va_copy(argcp, argptr);

    wxString msg = wxString::FormatV(format, argptr);
    va_end(argptr);

    //Logger::log(level, _T("ParserUtil::addProcessInfo() ") + msg );
    //Helper::Message(msg);         //cause wince
}

wxString ParserUtil::getFormat(firmwareConfig_codes format_id)
{
        return formatHash[format_id];
}
