#include "firmware.h"


#include "SerialException.h"

#include "Math.h"
#include "Utils.h"
#include "logger.h"
#include "UnicodeHelper.h"


#include <wx/textfile.h>
#include <wx/tokenzr.h>
#include <wx/config.h>
#include <wx/wfstream.h>
#include <wx/stream.h>

#include "firmwareconfig.h"
#include "firmwareconfigitem.h"
#include "hashcompare.h"

#include "vemsprotocol.h"
#include "FirmwareInfo.h"
#include "vemsprotocol/ECUInfo.h"

// NTCFinder* Firmware::ntcf = new NTCFinder();

Firmware::Firmware(VemsProtocol* vemsProtocol)
{
//    fprintf( stdout, "START Firmware ctor \n");
//    ihex = new IntelHex();
    vp = vemsProtocol;
    //int state = vp->getState();
    //ctor
    CONFIG_INDEX_MSG =  _T("Config line index:");
}

Firmware::~Firmware()
{
    //if(m_observer != NULL) delete m_observer;
    //dtor
}

int Firmware::sendFile(wxString& strFile)
{

    if ( strFile.Length()  > 0)
    {
        wxTextFile file;
        wxString line;
        if (  file.Open(strFile) )
        {
            bool firstline = true;
            do
            {
                if ( firstline ) firstline = false;
                else line = file.GetNextLine();

                vp->sendString( line );
            }
            while ( !file.Eof() );

            printf("Opened file: %s!\n", strFile.c_str());
        }
        else
        {
            printf("Could not open file: %s!\n", strFile.c_str());
            return -1;
        }

    }
    else
    {
        printf("Could not open file: %s!\n", strFile.c_str());
        return -1;
    }

    return 0;
}

FirmwareConfig* Firmware::getConfig(int type, wxOutputStream *os)
{
     Logger::log(LOG_MSG,_T("Firmware::getConfig() 1"));
    vp->setState(ECUInfo::MAN);
    Logger::log(LOG_MSG,_T("Firmware::getConfig() 2"));
    //vp->setState(ECUInfo::MAN);
/*
    int st = vp->getState();
    if( st != ECUInfo::MAN){
        fprintf(stdout, "Could not set state error code=%d!\n", st);
        Helper::Message( vp->getErrorMessage(st) );
        return NULL;
    }
*/
    //write info
    /*
    if(os != NULL){
        FirmwareInfo * fwInfo = vp->getFirmwareInfo();
        if(fwInfo != NULL){
              Logger::logf(LOG_MSG,_T("Firmware::getConfig() 3 hhh"));
            wxString infoStr = fwInfo->getAsFileString();
              Logger::logf(LOG_MSG,_T("Firmware::getConfig() 4 hhh"));
            observerNotify(infoStr);
            const char * str = UnicodeHelper::U2C(infoStr);
            os->Write(str, infoStr.Length());


        }
    }
*/
    int res;
    if(type & TYPE_MCD) res = vp->sendString("mcd");
    if(type & TYPE_MCT) res = vp->sendString("mct");

    FirmwareConfig *fc = new FirmwareConfig();

    int MAX_CONFIG_LINE_NUM = 300;

    observerInit(_T("Config fetch Progress"), CONFIG_INDEX_MSG, MAX_CONFIG_LINE_NUM);

    int result;
    int line_idx = 0;


    // write config
    do
    {
        char * readBuff = new char[255]; //NULL;//[32];
        //result = vp->readString(readBuff, 32);
        result = vp->ReadUntilEOS(&readBuff, "\r\n", 100L);
        readBuff[result] ='\0';

        //fprintf(stdout,"READBUFF: %s\n", readBuff);

        //TODOO: put to vemsprotocol.ccp
        char* t = new char[result + 1];
        memcpy(t,readBuff, result);
        delete[] readBuff;
        readBuff = t;
        readBuff[result] ='\0';

        wxString tmp(readBuff, wxConvUTF8); // should work on ANSI env

        //tmp.Printf(_T("%s"),readBuff);
        observerNotify(tmp);
        //Logger::log(LOG_MSG,tmp);

        //tmp.Printf(_T("Firmware::getConfig() line length: %d"),result);
        //Logger::log(LOG_MSG,tmp);
        wxString line(readBuff, wxConvUTF8);
        if(os != NULL){
            //FirmwareInfo * fwInfo = vp->getFirmwareInfo();
            //wxString infoStr = fwInfo->getAsFileString();
            //const char * str = UnicodeHelper::U2C(infoStr);
            //os->Write(str, infoStr.Length());
            os->Write(readBuff, result);
            os->Write("\n", 1);
        }
        fc->parseLine(line);

        observerNotify(line_idx, line_idx);
        line_idx++;
    }
    while (result != 0);

    observerNotify(MAX_CONFIG_LINE_NUM, MAX_CONFIG_LINE_NUM);

    wxString tmp = wxString::Format(_T("Config Line idx: %d"), line_idx);
    Logger::log(LOG_MSG,tmp);
    return fc;
}

StringHash* Firmware::writeEcuTextFile(wxString& fileName)
{

    StringHash* hash = new StringHash();

    wxTextFile textFile(fileName);

    if ( textFile.Open() )
    {
        textFile.Clear();
    }
    else
    {
        textFile.Create();
    }

    int result = 0;
    do
    {
        char * readBuff = new char[255]; //NULL;//[32];
        //result = vp->readString(readBuff, 32);
        result = vp->ReadUntilEOS(&readBuff, "\r\n", 100L);
        readBuff[result] ='\0';

        //fprintf(stdout,"READBUFF: %s\n", readBuff);

        //TODOO: put to vemsprotocol.ccp
        char* t = new char[result + 1];
        memcpy(t,readBuff, result);
        delete[] readBuff;
        readBuff = t;
        readBuff[result] ='\0';

        wxString tmp(readBuff, wxConvUTF8); // should work on ANSI env

        //tmp.Printf(_T("%s"),readBuff);
        observerNotify(tmp);
        Logger::log(LOG_MSG,tmp);

        tmp.Printf(_T("Result: %d"),result);
        Logger::log(LOG_MSG,tmp);

        wxString line(readBuff, wxConvUTF8);
        //line.Printf(_T("%s"), readBuff);
        textFile.AddLine(line);

        //use key: h[0]
        //Helper::parseKeyValuePairToHash(line,_T("="), *hash);

        //use key: h[0,0..row]
        wxArrayString tokens;
        Utils::tokenizeToArray(line, wxT("="), tokens);

        if (tokens.GetCount() == 2)
        {
            wxString arrayDef = tokens.Item(0);
            wxString valueList = tokens.Item(1);

            wxArrayString row;
            Utils::tokenizeToArray(valueList, wxT(" "), row);
//            FirmwareConfig::add2CommpareHash(*hash, arrayDef, row);
        }


    }
    while (result != 0);

    textFile.Write();
    textFile.Close();

    return hash;
}

wxString Firmware::getOneWireID()
{
    wxArrayString oneWireA = wxArrayString();
    wxString hexList = _T("c1 e1 33 ff ff ff ff ff ff ff ff e3 c1");

    wxStringTokenizer tkz(hexList, wxT(" "));
    for (int i=0; i< 100;i++)
    {
        while ( tkz.HasMoreTokens() )
        {
            wxString command = _T("mxw");
            wxString token = tkz.GetNextToken();
            //Helper::Message(token);
            // process token here
            //oneWireA.Add(token);
            unsigned char binValue = (unsigned char) Math::hex2int(token); //2digit
            //command.Printf(_T("mxw%c"), binValue);
            command.Append(binValue);
            vp->sendString(command);
            observerNotify(token + command);
        }
    }
    //Helper::Message(hexList);
    uint8_t readBuff[32];
    //char readBuff = new  char(16);

    int result = vp->readString(readBuff, 30); //8 byte should be received

    wxString tmp;
    tmp.Printf(_T("One wire read Result: %i"), result);
    observerNotify(tmp);

    wxString asStr;
    asStr.Printf(_T("Result as str: %s"), readBuff);
    observerNotify(asStr);

    return Math::char2hex(readBuff, 16);
}


void Firmware::printResult(CompareResult * cr, StringHash& fileConfigHash, StringHash& ecuConfigHash)
{
    //Diff values
    wxArrayString diffValue = cr->getDiffValueKey();
    diffValue.Sort();

    observerNotify(_T("--Diff Value Found--(Config name: File| |ECU)") );

    int errorCnt = 0;
    for(uint16_t i=0; i < diffValue.Count(); i++){
        wxString key = diffValue.Item(i);
        wxString msg = key +_T(": ")+fileConfigHash[key] + _T("| |") +ecuConfigHash[key] ;
        observerNotify(msg);
        errorCnt++;
    }

    if(errorCnt == 0){ observerNotify(_T("None! GOOD!!!")); }

    // Missing keys:
    errorCnt =0;
    wxArrayString missingKeys = cr->getMissingKey();
    missingKeys.Sort();

    observerNotify(_T("--Missing Keys in config file--") );
    for(uint16_t i=0; i < missingKeys.Count(); i++){
        wxString key = missingKeys.Item(i);
        //wxString msg = miss;
        observerNotify(key);
        errorCnt++;
    }
    if(errorCnt == 0){ observerNotify(_T("None! GOOD!!!")); }
}


int Firmware::configDownloadMgr(FirmwareConfigItemArray* configItemA)
{
     vp->setState(ECUInfo::MAN);
    //vp->setState(ECUInfo::MAN);
/*
    int st = vp->getState();
    if( st != ECUInfo::MAN){
        fprintf(stdout, "Could not set state error code=%d!\n", st);
        Helper::Message( vp->getErrorMessage(st) );
        return -1;
    }
*/
    int config = 0;
    FirmwareConfig * fileConfig = NULL;//; = new FirmwareConfig();
    for(uint16_t i = 0; i <configItemA->Count(); i++ ){
            FirmwareConfigItem* configItem = (*configItemA)[i];

             if( configItem->type == _T("config_download") ){
                 configItem->toString();
                Logger::log(LOG_MSG,_T("Config download file path: ")+ configItem->getAbsolutePath() );


                    if(config == 0){
                        //mcd
                        wxString mcdFile = configItem->getAbsolutePath();
                        Logger::log(LOG_MSG, _T("Firmware::configDownloadMgr() mcdFile: ")+ mcdFile);

                        wxOutputStream *os = new wxFileOutputStream(mcdFile);
                        fileConfig = getConfig(TYPE_MCD, os);
                        os->Close();
                        if(fileConfig == NULL) return -1;

                    }else if( config == 1){
                        //mct
                         wxString mctFile = configItem->getAbsolutePath();
                          Logger::log(LOG_MSG, _T("Firmware::configDownloadMgr() mctFile: ")+ mctFile);

                        wxOutputStream *os = new wxFileOutputStream(mctFile);
                        fileConfig = getConfig(TYPE_MCT, os);
                        os->Close();
                        if(fileConfig == NULL) return -1;
                    }
                    config++;
            }
    }
    return 0;
}

FirmwareConfig* Firmware::getFirmwareConfig(FirmwareConfigItemArray* configItemA)
{
    FirmwareConfig * fileConfig = new FirmwareConfig();
        for(uint16_t i = 0; i <configItemA->Count(); i++ ){
            FirmwareConfigItem* configItem = (*configItemA)[i];

            if( configItem->type == _T("header") ){
                Logger::logf(LOG_MSG,_T("Header file path: ")+ configItem->getAbsolutePath() );
                configItem->toString();
                fileConfig->parseHeaderFile( configItem->getAbsolutePath() );
            }
        }

        const wxArrayString& headerA = fileConfig->getHeaderArray();
        if(headerA.Count() == 0){
            Logger::logf(LOG_ERROR,_T("Firmware::configMgr() HEADER size == 0"));
            return NULL;
        }

        wxString path;
        for(uint16_t i = 0; i <configItemA->Count(); i++ ){
            FirmwareConfigItem* configItem = (*configItemA)[i];

            if( configItem->type == _T("config") ){
                 configItem->toString();
                 path = configItem->getPath();
                //Logger::log(LOG_MSG,_T("Config file path: ")+ configItem->getAbsolutePath() );

                if(configItem->active){
                    Logger::log(LOG_MSG,_T("Firmware::getFirmwareConfig() parsing: ")+ configItem->getAbsolutePath() );
                    fileConfig->parseFile( configItem->getAbsolutePath() );
                }else{
                    Logger::log(LOG_MSG,_T("Firmware::getFirmwareConfig() ignored: ")+ configItem->getAbsolutePath() );

                }
            }
        }
    //TEST
        StringHash sh = fileConfig->getConfigHash();
        if(sh.size() == 0){
            Logger::logf(LOG_ERROR,_T("Firmware::getFirmwareConfig() configHash is EMPTY!"));
        }
    ////
    //// invoke replace
        Logger::logf(LOG_ERROR,_T("Firmware::getFirmwareConfig() path = ")+path);
        return fileConfig;
}

int Firmware::configMgr(FirmwareConfig * fileConfig, bool isWrite)
{

        Logger::log(LOG_ERROR,_T("Firmware::configMgr() START ") );
        fileConfig->createMtt(false);
        fileConfig->getUndefinedSymbol();
        fileConfig->Message();

        wxArrayString mttA = fileConfig->getMTTA();
        fprintf( stdout, "Firmware::configMgr() %d\n", mttA.Count() );
        int res;
        if(isWrite){
             Logger::log(LOG_MSG,_T("UPLOAD CONFIG!!"));
            res = sendConfig(mttA);
            if ( res < 0){
                Logger::log(LOG_ERROR,_T("Firmware::configMgr() send config error: ") + vp->getErrorMessage(res) );
                return -2; // serial error
            }
             Logger::log(LOG_ERROR, wxString::Format(_T("Firmware::configMgr upload res: %d"), res) );
        }else{
              Logger::log(LOG_MSG,_T("VERIFY CONFIG!!"));
        }

        Logger::log(LOG_MSG,_T("FETCH CONFIG!!"));
        FirmwareConfig* ecuConfig = getConfig(TYPE_MCD | TYPE_MCT, NULL);
        if(  ecuConfig == NULL ){
             //Helper::Message(vp->getErrorMessage(SP_CANNOT_OPEN) );
             return -3; // serial error
        }
        //ecuConfig->printKeyValueHash();

        StringHash configFileHash = fileConfig->getSymbolKeyValueHash();
        StringHash configEcuHash = ecuConfig->getSymbolKeyValueHash();
        CompareResult * cr = HashCompare::compare(configFileHash, configEcuHash);
        printResult(cr, configFileHash, configEcuHash);


        wxArrayString* undefSymbol = fileConfig->getUndefinedSymbol();
        wxArrayString* extraSymbol = fileConfig->getExtraSymbol();

        Logger::logf(LOG_MSG,_T("Firmware::configMgr() Undefined symbols: %i"), undefSymbol->Count() );

        if( !cr->isIdentical() ){
            Logger::log(LOG_MSG, cr->toString() );
            return 1; // not identical
        }
        return 0; // identical

}

wxString Firmware::getHexFile(FirmwareConfigItemArray* configItemA)
{
    FirmwareConfigItem* configItem = new FirmwareConfigItem();
    for(uint16_t i = 0; i <configItemA->Count(); i++ ){
        configItem = configItemA->Item(i);

        if( configItem->type == _T("firmware") ){
            return configItem->getAbsolutePath();
        }
    }
    return wxEmptyString;
}

wxArrayString* Firmware::getPatchFiles(FirmwareConfigItemArray* configItemA)
{
    wxArrayString * A = new wxArrayString();

    FirmwareConfigItem* configItem = new FirmwareConfigItem();
    for(uint16_t i = 0; i <configItemA->Count(); i++ ){
        configItem = configItemA->Item(i);
         Logger::logf(LOG_MSG, _T("Firmware::getPatchFiles: ")+configItem->getPath() +_T(" active: %i"), configItem->active );
        if( configItem->type == _T("firmware_patch") ){
            if(configItem->active){
                A->Add(configItem->getAbsolutePath());
            }
        }
    }
    return A;
}

int Firmware::sendConfig(wxArrayString& array)
{
    int res = vp->setState(ECUInfo::MAN);

    if(res < 0){
        Logger::log(LOG_ERROR, _T("Firmware::sendConfig() error! ") + vp->getErrorMessage(res) );
        return res;
    }

    res = vp->getState();
    if( res != ECUInfo::MAN){
        Logger::log(LOG_ERROR, _T("Firmware::sendConfig() error2! ") + vp->getErrorMessage(res));
        return SP_CANNOT_INIT_ECU;
    }
     Logger::log(LOG_MSG,_T("Firmware::sendConfig() send array"));
    return sendArray(array);
    //vp->setState(ECUInfo::MEGATUNE);

}

int Firmware::sendArray(wxArrayString& array)
{
    int maxIdx = array.GetCount();
     observerInit(_T("Config upload Progress"), CONFIG_INDEX_MSG,  maxIdx);
    int res;
     fprintf( stdout, "Firmware::sendArray() %d\n", array.Count() );
    for(uint32_t i = 0; i< array.Count(); i++){
       // vp->sendString((char *)line.c_str());
       observerNotify(i,i);
        wxString line = array.Item(i);
        //fprintf( stdout, "VemsProtocol::sendArray() line for sendString: %s\n", UnicodeHelper::U2C( line ) ); //GOOD!
        res = vp->sendString( line );
        if(res < 0){
             observerNotify(maxIdx, maxIdx);
             return res;
        }
    }
    observerNotify(maxIdx, maxIdx);
    return 0;
}

