#ifndef FIRMWARECONFIG_H
#define FIRMWARECONFIG_H

#include "Symbol.h"
#include "common/wxhashdef.h"
#include "fileparser/parseheader.h"

WX_DECLARE_STRING_HASH_MAP( wxArrayString*, wxArrayStringH );

class SymbolFileDesc
{
    public:
        SymbolFileDesc(const wxString& name, int lineNumber)
        {
            m_fileName = name;
            m_lineNumber = lineNumber;
        }
        int      getLineNumber(){ return m_lineNumber; };
        wxString getFileName(){ return m_fileName; };
        void setLineNumber(int n){ m_lineNumber = n; };
        void setFileName(const wxString& name){ m_fileName = name; };
    private:
        int m_lineNumber;
        wxString m_fileName;
};

WX_DECLARE_STRING_HASH_MAP( SymbolFileDesc*, SymbolFileDescH );

class FirmwareConfig
{
    public:
        FirmwareConfig();
        virtual ~FirmwareConfig();

        int parseHeaderFile(const wxString& strFile);
        const wxArrayString& getHeaderArray() { return m_parseHeader.getHeaderArray(); }

        int parseFile(const wxString& strFile);
        int parseLine(wxString& line);

        ///STDataIOFile functions
        void writeSymbols( DoubleHash* symbolHash, bool isTable = false );
        void writeTableValue( wxString& line, const wxString& symbol, uint8_t value );
        DoubleHash& getConfigValueHash();

        int init();
        void Message();

        void createMtt(bool isLog);
        wxArrayString& getMTTA();

//        ByteArray& getByteBuffer();
        /// contains all config key type (scalar, array) array key example: j[1,2]
        StringHash& getSymbolKeyValueHash();
        ///contains only the scalar key value pairs
        StringHash& getConfigHash();
//        wxArrayString& getConfigArray();

        /// header not contains but found in cfg most porbobly it is an old config value
        wxArrayString * getExtraSymbol();
        /// header contains but not found in cfg
        wxArrayString * getUndefinedSymbol();

        void replaceKey(const wxString& oldKey, const wxString& newKey, int value);
        void saveConfig(const wxString& strFile, bool force);

        void resetReplaceCount();
        void addReplaceCount(int i);

          /// Debug
        void printKeyValueHash();
        const String2IntHash& getTableSizeH() const { return m_tableSizeH; }
        const String2CharRefHash& getTableDataH() const { return m_tableDataH; }
        const SymbolTable& getSymbolTable() const { return m_symbolTable; }
    protected:
    private:
        String2CharRefHash  m_tableDataH;
        SymbolFileDescH m_symbolFileDescH;
        String2IntHash  m_tableSizeH;       //table name, table size
        wxArrayString   configArray;        //symbol names in order
        wxArrayString   mttA;
        wxArrayString   undefinedSymbolA;   // exist in header but not in config
        wxArrayString   extraSymbolA;       //exist in config but not in header
        wxArrayString   configkeyArray;
        SymbolTable     m_symbolTable;
        ParseHeader     m_parseHeader;
        StringHash      configHash;
        DoubleHash      m_configDoubleHash;
        StringHash      symbolKeyValueHash;
        wxString        m_fileName;

        int m_replaceCount;
        int m_lineNumber;

        void calculateTablesOffset();
        int parseTableLine(wxString& line);
        int parseConfigLine(wxString& line);
        wxString getSymbol(wxString& key);
        int addRow2Table(const wxString& key, const wxString& indexStr, const wxArrayString& stringA);
        wxString getMTTTableCommand(wxString& key, const wxString& value);
        wxString getMTTConfigCommand(wxString& key, const wxString& value);
        int getTableIndex(wxString& key);
        void add2CommpareHash(StringHash& h, const wxString& arrayDef, const wxArrayString & row);
        void configAutoUpgrade(wxArrayString* undefSymbols);
};

#endif // FIRMWARECONFIG_H
