#include "ConfigValidatorHelper.h"
#include "../FirmwareInfo.h"
#include "../MTConfigHelper.h"
#include "../UserLog.h"
#include "../VemsMsgListCtrl.h"
#include "../stdatamanager/STDataHelper.h"
#include "../stdatamanager/STDataModelBase.h"
#include "../firmwareconfig/ConfigCriteriaData.h"
#include "../firmwareconfig/ConfigValidatorMgr.h"
#include "../fileparser/ConfigCriteriaParser.h"
#include "../vemsprotocol/ECUInfo.h"
#include "../common/VemsMessageHashHelper.h"
#include "../common/types/DoubleHash.h"


ConfigValidatorMgr::ButtonState ConfigValidatorHelper::validate(    FirmwareInfo* fwInfo,
                                                                    STDataModelBase* stDataModel,
                                                                    UserDialogBase* userDialog,
                                                                    int style )
{
    /// TODO: get a model and check if it is valid or not
    /// valid means:
    /// 1. ecu online,
    /// 2. project loaded,
    /// 3. vemslog loaded
    /// 4. Simple vemscfg is loaded

    wxString fwVersion = fwInfo->getFirmwareVersionAsString();

    if ( fwVersion == wxEmptyString ) {
        wxString msg = VemsMessageHashHelper::GetVemsMsg( _T("config_not_found_str") ).getHtmlMsg();
        wxMessageBox(msg);
        return ConfigValidatorMgr::CFV_UNKNOWN;
    }

    /// check the appropriate global.h if it is not exist give a message how to fix it.
    MTConfigData* mtConfigData = MTConfigHelper::getMTConfigData();
    SymbolTable& cfgSymbolH = mtConfigData->getSymbolMgr().getCfgSymbolTable();
    if (  cfgSymbolH.empty() ) {
        wxString headerFile = fwInfo->getHeaderFile();
        if ( headerFile.IsEmpty() ) {
            UserLog::vLog(_T("header_not_found"));
            StringHash hash;
            hash[_T("firmware_version")] = fwVersion;
            wxString msg = VemsMessageHashHelper::GetVemsMsg(_T("header_not_found_str"),&hash).getHtmlMsg();
            wxMessageBox(msg);
            return ConfigValidatorMgr::CFV_UNKNOWN;
        } else {
            /// should be moved to web-tool on install
            /// if fw == downloaded fw then parse
            MTConfigHelper::parseFirmwareHeader( *fwInfo );
            cfgSymbolH = mtConfigData->getSymbolMgr().getCfgSymbolTable();
        }
    }

    /// Create doubleHash from model and symbolMgr
    VariableDataDH variableDataH;
    STDataHelper::fillVariableHashFromModel( *stDataModel, cfgSymbolH, variableDataH );

    /// get ConfigcriteriaData
    ConfigCriteriaData criteriaData;
    int device = fwInfo->getDevice();
    wxString file;
    switch(device){
        case ECUInfo::DEVICE_V3:
            file = _T("ConfigCriteria.xml");
            break;
        case ECUInfo::DEVICE_ROUND:
            file = _T("Round-ConfigCriteria.xml");
            break;
        default:
            break;
    }
    ConfigCriteriaParser::parse( criteriaData, file );

    ConfigValidatorMgr cfValidator(&criteriaData, mtConfigData );

    VemsMsgListCtrl* messageViewer = VemsMsgListCtrl::deleteAndCreate(userDialog, _T("Config Sanity check") );

    /// validate the model
    cfValidator.registerVemsMessageViewer(messageViewer);
    cfValidator.validate(variableDataH);
    cfValidator.checkChannelCollisions( stDataModel );
    cfValidator.checkInvalidChannels( stDataModel );
    cfValidator.checkInvalidBits( stDataModel );

    switch ( style )
    {
        case CVH_SHOW_RESULT_ALWAYS:
            messageViewer->Show();
            break;

        case CVH_SHOW_RESULT_IF_COLLISION:
            if ( ( cfValidator.getInvalidChannelCount() + cfValidator.getChannelCollisionCount() ) > 0 ) {
                messageViewer->Show();
            } else {
                messageViewer->Destroy();
            }
            break;

        case CVH_NOT_SHOW_RESULT:
            messageViewer->Destroy();
            break;
    }

    return cfValidator.getButtonState();
}
