#ifndef CONFIGVALIDATORMGR_H
#define CONFIGVALIDATORMGR_H

#include "../common/types/StringHash.h"
#include "../common/types/IntArray.h"
#include "../common/VemsMessageViewerBase.h"
#include "../expression/RPNParser.h"
#include "../expression/VariableData.h"

class ConfigCriteriaData;
class ConfigCriteria;
class ConfigCriteriaRange;
class SymbolTable;
class MTConfigData;
class STDataModelBase;
class FirmwareInfo;

class ConfigValidatorMgr
{
    public:
        ConfigValidatorMgr(ConfigCriteriaData* criteriaData, MTConfigData* mtconfigData);
        ConfigValidatorMgr(ConfigCriteriaData* criteriaData, MTConfigData* MTConfigData, STDataModelBase* stdata_model, FirmwareInfo* );
        virtual ~ConfigValidatorMgr();
        void registerVemsMessageViewer(VemsMessageViewerBase* vemsMessageViewer);

        ///------------
        // see ConfigValidator & ConfigValidatorBase
        void validate( StringHash& symbols ); // with internally created VarDH
        void checkInvalidBits( StringHash& symbols ); // with internally created VarDH
        ///------------


        void validate(VariableDataDH& data);
        void checkChannelCollisions(STDataModelBase * stdata_model = NULL );
        void checkInvalidChannels(STDataModelBase * stdata_model = NULL );
        void checkInvalidBits( STDataModelBase * stdata_model  = NULL );

        enum ButtonState { CFV_UNKNOWN, CFV_GREEN, CFV_RED, CFV_RED_BLINKING, CFV_YELLOW, CFV_YELLOW_BLINKING };
        ButtonState getButtonState() const;

        int getInvalidChannelCount() const { return m_invalidChannelCount; }
        int getChannelCollisionCount() const { return m_channelCollisionCount; }

    protected:
    private:
        int m_okCount;
        int m_warningCount;
        int m_errorCount;
        int m_invalidChannelCount;
        int m_channelCollisionCount;

        VariableDataDH      m_variableDH;
        STDataModelBase*    m_stModel;
        ConfigCriteriaData* m_criteriaData;
        MTConfigData*       m_mtconfigData;
        FirmwareInfo*       m_fwInfo;

        IntArray    m_criteriaTypeA;        /// idx= criteriaDataIdx, value result: ok, error, etc
        IntArray    m_criteriaRangeIdxA;     /// idx= criteriaDataIdx, value result: criteria rangeidx
        RPNParser m_rpn;
        VemsMessageViewerBaseArray m_viewerA;

        void setVariableData(VariableDataDH& vd, SymbolTable& variableHash);
        void show(const VemsMessage& vemsMsg);

        wxString validate(const ConfigCriteria& criteria, int& resultType, int& rangeIdx);
        int validate(const ConfigCriteriaRange& criteriaRange);
        void validate(const ConfigCriteria& criteria, IntArray& resultA);

        void show( const ConfigCriteria* criteria, int type, const wxString& expression);
        void show( const ConfigCriteria* criteria, const IntArray& resultA);
        void reset();
        //wxString getLink(const wxString& href, const wxString& name);
        //void setParam(const StringHashSet& variablesSet, const wxString& expression, StringHash& paramH);
        void setParam(const wxArrayString& variablesA, StringHash& paramH);
        int compareInts(int* n1, int* n2);
        //wxString getMenuLink(const wxString& expression);

};

#endif // CONFIGVALIDATORMGR_H
