#ifndef FIRMWARECONFIGMANAGER_H
#define FIRMWARECONFIGMANAGER_H


#include "../common/types/StringSet.h"
#include "../vemsprotocol/DeviceInfo.h"
#include "../common/ConfigSwitchTargetTypes.h"

class FWConfigUploadManager;
class ObserverBase;
class SIPRPacket;
class wxProgressDialog;
class InputEvent;
class UserDialogBase;
class VemsDeviceControllerBase;
class bcProgressDialog;
class STDataModelBase;
class VemsCommand;
class CommunicationEvent;
class VemsDeviceControllerEvent;


extern const wxEventType VEMS_FIRMWARE_CONFIG_UPLOADED;


class FirmwareConfigManager : public wxEvtHandler
{
    public:
        FirmwareConfigManager(  wxWindow* parent,
                                VemsDeviceControllerBase* deviceController,
                                STDataModelBase* deviceModel,
                                ObserverBase* observer,
                                UserDialogBase* userDialog );

        ~FirmwareConfigManager();

        void setDeviceInfo( const DeviceInfo& deviceInfo ) { m_deviceInfo = deviceInfo; }

        void uploadConfigFile( const wxString& filename, int currentConfigType = CFG_UNKNOWN, int uploadConfigType = CFG_BOTH );
        void easyUploadConfig( int currentConfigType = CFG_UNKNOWN, int uploadConfigType = CFG_BOTH );
        void uploadConfigFromProject( int currentConfigType = CFG_UNKNOWN, int uploadConfigType = CFG_BOTH );
        void verifyConfigFromProject();

        void verifyConfigFromFile( const wxString& filename = wxEmptyString );
        void filteredConfigUpload( int currentConfigType = CFG_UNKNOWN, int uploadConfigType = CFG_BOTH );

        wxString getUploadedConfigFileName() { return m_cfgfilename; }

    private:
        FWConfigUploadManager*  m_pUploadManager;
        bcProgressDialog*       m_pProgressDialog;
        ObserverBase*           m_observer;
        wxWindow*               m_parent;
        int                     m_iLineCount;
        bool                    m_bIsFilteredUpload;
        wxString                m_cfgfilename;
        wxString                m_filterFile;

        DeviceInfo                  m_deviceInfo;
        VemsDeviceControllerBase*   m_deviceController;
        size_t                      m_dirtyCount;
        size_t                      m_sentCount;
        size_t                      m_burnCount;

        STDataModelBase*            m_deviceModel;
        VemsCommand*                m_loadCmd;

    private:
        void uploadConfig( const wxArrayString& configFiles, int currentConfigType, int uploadConfigType );
        size_t getUploadRequestCount( int currentConfigType = CFG_UNKNOWN, int uploadConfigType = CFG_BOTH ) const;
        void verifyConfig(  bool connectInputEvent = true );
        void showFirmwareConfigResult( int result );
        void reset();
        void parseConfigFilterFile( const wxString& name, StringSet& keys );

        bool isHeaderFileExist();
        void showHeaderFileMissingMessage();
        bool getConfigFile();
        bool getFilterFile();
        void createUploadConfig();
        void createProgressDialog( const wxString& title, const wxString& message = wxEmptyString );
        void destroyProgressDialog();

        void showConfigDownloadResult();
        void showFilterParsingResult();
        void showConfigUploadResult();
        void processConfigUpload( const SIPRPacket& packet );

        bool isEnabled() const;
        void showErrorMessage() const;
        void setLoadCmd( int currentConfigType, int uploadConfigType );

        void connectEventHandlers();
        void disconnectEventHandlers();

        void onCommunicationEvent( CommunicationEvent& event );
        void onSIPR( InputEvent& event );
        void onDeviceDisconnected( VemsDeviceControllerEvent& event );
};


#endif // FIRMWARECONFIGMANAGER_H
