#include "DiffGaugeDescManager.h"

#include "../wxSqliteHelper.h"

#define chkdb(x) if( ! (m_db.IsOpen()) ) return x;

DiffGaugeDesc::DiffGaugeDesc(long id, const wxString& name, double minval, double maxval)
{
    m_properties[_T("id")] = id;
    m_properties[_T("name")] = name;
    m_properties[_T("min_value")] = minval;
    m_properties[_T("max_value")] = maxval;

}

DiffGaugeDesc::DiffGaugeDesc(wxSQLite3ResultSet& profileLineResult)
{
    wxSqliteHelper::SQLLineToVariantHash(profileLineResult, m_properties);
}

int DiffGaugeDesc::GetId()
{
    return m_properties[_T("id")].GetLong();
}

wxString DiffGaugeDesc::GetName()
{
    return m_properties[_T("name")].GetString();
}

double DiffGaugeDesc::GetMinValue()
{
    return m_properties[_T("min_value")].GetDouble();
}

double DiffGaugeDesc::GetMaxValue()
{
    return m_properties[_T("max_value")].GetDouble();
}

wxVariantHash& DiffGaugeDesc::GetSymbol(int id)
{
    for(size_t a=0; a<m_symbols.GetCount(); a++) {
        if( m_symbols[a][_T("id")].GetLong() == id ) return m_symbols[a];
    }

    return m_dummy;
}

bool DiffGaugeDesc::SetProp(const wxString& key, const wxVariant& val)
{
    m_properties[key] = val;
    return true;
}

/************************************************************
                    DiffGaugeDescManager
*/

DiffGaugeDescManager::DiffGaugeDescManager()
{
    //ctor
    m_lastDatabaseFile = wxEmptyString;

    m_defaultDesc = DiffGaugeDesc(0, _T("dafault"), 0.7, 1.3);

    wxVariantHash hash;
    hash[_T("id")] = 0L;
    hash[_T("profile_id")] = 0L;
    hash[_T("symbol_name")] = _T("lambdatarget");
    m_defaultDesc.AddSymbolProps(hash);

    wxVariantHash hash2;
    hash2[_T("id")] = 0L;
    hash2[_T("profile_id")] = 0L;
    hash2[_T("symbol_name")] = _T("lambda");
    m_defaultDesc.AddSymbolProps(hash2);
}

DiffGaugeDescManager::~DiffGaugeDescManager()
{
    //dtor
}

bool DiffGaugeDescManager::CheckDB()
{
    chkdb(false);

    ColumnDescArrayHash desc;

    ColumnDescArray arr1;
    arr1.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr1.Add( ColumnDesc(_T("name"), _T("VARCHAR(64)")) );
    arr1.Add( ColumnDesc(_T("min_value"), _T("FLOAT")) );
    arr1.Add( ColumnDesc(_T("max_value"), _T("FLOAT")) );
    desc[_T("profiles")] = arr1;

    ColumnDescArray arr2;
    arr2.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr2.Add( ColumnDesc(_T("profile_id"), _T("INTEGER")) );
    arr2.Add( ColumnDesc(_T("symbol_name"), _T("VARCHAR(64)")) );
    arr2.Add( ColumnDesc(_T("color"), _T("VARCHAR(32)")) );
    desc[_T("symbols")] = arr2;

    wxSqliteHelper::DatabaseCheck(m_db,desc);

    return true;
}

bool DiffGaugeDescManager::LoadFromFile(const wxString& filename)
{
    if(m_db.IsOpen()) m_db.Close();

    m_db.Open(filename);

    CheckDB();

    m_profiles.clear();

    wxString qs = _T("select * from profiles;");

    wxSQLite3ResultSet res = m_db.ExecuteQuery(qs);

    while(res.NextRow()){
        DiffGaugeDesc desc(res);

        int pid = desc.GetId();

        wxSQLite3ResultSet pres = m_db.ExecuteQuery(wxString::Format(_T("select * from symbols where profile_id=%i order by id asc;"),pid));

        while(pres.NextRow()){
            wxVariantHash sprop;
            wxSqliteHelper::SQLLineToVariantHash(pres, sprop);
            desc.AddSymbolProps(sprop);
        }

        m_profiles[pid] = desc;
    }

    m_lastDatabaseFile = filename;

    return true;
}

DiffGaugeDesc& DiffGaugeDescManager::GetProfile(const wxString& name)
{
    for(Int2DiffGaugeDescHash::iterator it = m_profiles.begin(); it != m_profiles.end(); it++ ) {

        DiffGaugeDesc& desc = it->second;

        if( desc.GetName() == name ) return desc;
    }

    return m_defaultDesc;
}

DiffGaugeDesc& DiffGaugeDescManager::GetProfile(int id)
{
    if( m_profiles.find(id) == m_profiles.end() ) return m_defaultDesc;

    return m_profiles[id];
}

bool DiffGaugeDescManager::IsProfileExists(const wxString& name)
{
    for(Int2DiffGaugeDescHash::iterator it = m_profiles.begin(); it != m_profiles.end(); it++ ) {

        DiffGaugeDesc& desc = it->second;

        if( desc.GetName() == name ) return true;
    }

    return false;
}

bool DiffGaugeDescManager::IsProfileExists(int id)
{
    return m_profiles.find(id) == m_profiles.end() ? false : true;
}

