#include "GaugeDescManager.h"

#include <wx/xml/xml.h>
#include <wx/fileconf.h>
#include <wx/wfstream.h>
#include <wx/tokenzr.h>
#include <wx/filename.h>

#include "../ThemesHelper.h"
#include "../common/VemsMessageHashHelper.h"
#include "../UserLog.h"

#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"
#include "../configmanager/DeviceConfigHelper.h"

#include "../DirHelper.h"
#include "../logger.h"
#include "../MTConfigHelper.h"
#include "../common/FunctionMeter.h"
//#include "../DrawHelper.h"
#include "../vemsgauge/GaugeType.h"


#include "MultiGraphDescManager.h"
#include "VemsFlagDescManager.h"
#include "DiffGaugeDescManager.h"
#include "MultiSymbolGaugeDescMgr.h"

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(SymbolGaugeDescArrayArray);

GaugeDescManager::GaugeDescManager(MTConfigData* mtConfigData)
{
    //ctor
    m_gaugeGroupMgr = NULL;
    m_isLockGauges= false;
    m_mgMgr = NULL;
    m_flagDescMgr = NULL;

    m_mtConfigData = mtConfigData;

    m_lastError = 0;
    m_lastErrorStr = wxEmptyString;
}

GaugeDescManager::~GaugeDescManager()
{
    //dtor
}

MTConfigData* GaugeDescManager::getMTConfigData()
{
    return m_mtConfigData;
}

bool GaugeDescManager::CheckSymbolName(const wxString &symbolname)
{
    Symbol * s = getMTConfigData()->getSymbol( symbolname );

    if(s != NULL)
        return true;

    return false;
}

void GaugeDescManager::Init()
{
    if(!(ThemesHelper::parseWidgetThemeFile()))
        UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("theme_file_error")) );

    //GaugeDescEditorDesc
    wxString gddFile = DirHelper::GetPath(dCONFIG) + _T("GDDesc.xml");
    LoadGDDs( gddFile );

    m_mgMgr = new MultiGraphDescManager();

    m_flagDescMgr = new VemsFlagDescManager();


    //m_diffGaugeMgr = new DiffGaugeDescManager();

    InitDiffGaugeDesc();

    InitColsGaugeDesc();

}

void GaugeDescManager::InitDiffGaugeDesc()
{
    wxVariantHash props;
    props[_T("id")] = 0L;
    props[_T("name")] = _T("dafault");
    props[_T("min_value")] = 0.7;
    props[_T("max_value")] = 1.3;

    MultiSymbolGaugeDesc defaultDiffDesc(props);

    wxVariantHash hash;
    hash[_T("id")] = 0L;
    hash[_T("profile_id")] = 0L;
    hash[_T("symbol_name")] = _T("lambdatarget");
    defaultDiffDesc.AddSymbolProps(hash);

    wxVariantHash hash2;
    hash2[_T("id")] = 0L;
    hash2[_T("profile_id")] = 0L;
    hash2[_T("symbol_name")] = _T("lambda");
    defaultDiffDesc.AddSymbolProps(hash2);

    ColumnDescArrayHash dbdesc;

    ColumnDescArray arr1;
    arr1.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr1.Add( ColumnDesc(_T("name"), _T("VARCHAR(64)")) );
    arr1.Add( ColumnDesc(_T("min_value"), _T("FLOAT")) );
    arr1.Add( ColumnDesc(_T("max_value"), _T("FLOAT")) );
    dbdesc[_T("profiles")] = arr1;

    ColumnDescArray arr2;
    arr2.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr2.Add( ColumnDesc(_T("profile_id"), _T("INTEGER")) );
    arr2.Add( ColumnDesc(_T("symbol_name"), _T("VARCHAR(64)")) );
    arr2.Add( ColumnDesc(_T("custom_symbol_title"), _T("VARCHAR(64)")) );
    arr2.Add( ColumnDesc(_T("color"), _T("VARCHAR(32)")) );
    dbdesc[_T("symbols")] = arr2;

    m_diffGaugeMgr = new MultiSymbolGaugeDescMgr(defaultDiffDesc, dbdesc);
}

void GaugeDescManager::InitColsGaugeDesc()
{
    wxVariantHash props;
    props[_T("id")] = 0L;
    props[_T("name")] = _T("dafault");

    MultiSymbolGaugeDesc defaultColsDesc(props);

    wxVariantHash hash;
    hash[_T("id")] = 0L;
    hash[_T("profile_id")] = 0L;
    hash[_T("symbol_name")] = _T("rpm");
    defaultColsDesc.AddSymbolProps(hash);

    wxVariantHash hash2;
    hash2[_T("id")] = 0L;
    hash2[_T("profile_id")] = 0L;
    hash2[_T("symbol_name")] = _T("throttle");
    defaultColsDesc.AddSymbolProps(hash2);

    ColumnDescArrayHash dbdesc;

    ColumnDescArray arr1;
    arr1.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr1.Add( ColumnDesc(_T("name"), _T("VARCHAR(64)")) );
    arr1.Add( ColumnDesc(_T("min_value"), _T("FLOAT")) );
    arr1.Add( ColumnDesc(_T("max_value"), _T("FLOAT")) );
    dbdesc[_T("profiles")] = arr1;

    ColumnDescArray arr2;
    arr2.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr2.Add( ColumnDesc(_T("profile_id"), _T("INTEGER")) );
    arr2.Add( ColumnDesc(_T("symbol_name"), _T("VARCHAR(64)")) );
    arr2.Add( ColumnDesc(_T("custom_symbol_title"), _T("VARCHAR(64)")) );
    arr2.Add( ColumnDesc(_T("color"), _T("VARCHAR(32)")) );
    dbdesc[_T("symbols")] = arr2;

    m_colsGaugeMgr = new MultiSymbolGaugeDescMgr(defaultColsDesc,dbdesc);
}

void GaugeDescManager::LoadDescriptors( int device )
{
    Logger::logf(LOG_MSG,_T("GaugeDescManager::LoadDescriptors() START") );
    ConfigBase* cfgBase = ConfigHelper2::getConfig();
    wxString descFile = DeviceConfigHelper::getDeviceFile(cfgBase, device, _T("descriptorFile"));

    //bool res = LoadDescriptors( DirHelper::GetPath(dCONFIG) + descFile );
    LoadDescriptors( DirHelper::GetPath(dCONFIG) + descFile );
    wxString mgdescFile = DirHelper::GetPath(dCONFIG) +
        DeviceConfigHelper::getDeviceFile(cfgBase, device, _T("multigraphDescFile") );

    wxString mgdefdescFile = DirHelper::GetPath(dCONFIG) +
        DeviceConfigHelper::getDeviceFile(cfgBase, device, _T("multigraphDefaultDescFile") );

    CheckConfigFile(mgdescFile, mgdefdescFile);

    m_mgMgr->InitMgr(mgdescFile);

    wxString flagDescFile = DirHelper::GetPath(dCONFIG) + DeviceConfigHelper::getDeviceFile(cfgBase, device, _T("flagFile") );

    LOGP( _T("flagDescFile: ") + flagDescFile );

    m_flagDescMgr->LoadDescriptorFromFile( flagDescFile );

    wxString diffdescfile = DirHelper::GetPath(dCONFIG) + DeviceConfigHelper::getDeviceFile(cfgBase, device, _T("diffGaugeFile") );
    wxString colsdescfile = DirHelper::GetPath(dCONFIG) + DeviceConfigHelper::getDeviceFile(cfgBase, device, _T("colsGaugeFile") );

    m_diffGaugeMgr->LoadFromFile( diffdescfile, DeviceConfigHelper::getDevicePath(device)+_T("/diffGaugeFile") );
    m_colsGaugeMgr->LoadFromFile( colsdescfile, DeviceConfigHelper::getDevicePath(device)+_T("/colsGaugeFile") );
    Logger::logf(LOG_MSG,_T("GaugeDescManager::LoadDescriptors() START") );
}

void GaugeDescManager::CheckConfigFile(const wxString& file, const wxString& deffile)
{
    if( wxFileName::FileExists(file) ) return;

    if( !wxFileName::FileExists(deffile) ) return;

    wxCopyFile(deffile, file);
}

const WidgetThemeTableArray* GaugeDescManager::GetWidgetThemes()
{
    return ThemesHelper::GetWidgetThemes();
}

void GaugeDescManager::GetGaugeTypesFromStr(const wxString& str, IntArray& arr, const wxString& delimiter)
{
    wxArrayString strarr = wxStringTokenize(str, delimiter);

    for(size_t a=0; a < strarr.GetCount(); a++){

        wxString type = strarr[a];

        int gt = GaugeType::getType(type);

        Logger::logf(LOG_MSG,_T("GetGaugeTypesFromStr ")+type+_T(" id=%i"),gt);

        if(gt != -1) arr.Add(gt);
    }

}

void GaugeDescManager::LoadGDDs(const wxString &filename)
{
    wxXmlDocument doc;

    doc.Load( DirHelper::GetPath(dCONFIG) + _T("GDDesc.xml"));

    wxXmlNode *root = doc.GetRoot();

    if( root->GetName() != _T("GDDs")) return;

    GDDesc* gdd;
    //long tmp;

    for( wxXmlNode * pnode = root->GetChildren() ; pnode ; pnode = pnode->GetNext())
    {

        if( pnode->GetName() != _T("Page") ) continue;

        wxString pagename = pnode->GetPropVal(_T("name"),wxEmptyString);

        IntArray globvalids;
        globvalids.Clear();

        if(pnode->HasProp(_T("valid"))){
            GetGaugeTypesFromStr( pnode->GetPropVal( _T("valid"),_T("")), globvalids);
        }

        for( wxXmlNode * gdnode = pnode->GetChildren() ; gdnode ; gdnode = gdnode->GetNext())
        {

            if( gdnode->GetName() != _T("DescriptorItem") ) continue;

            gdd = new GDDesc();

            gdd->Name  = gdnode->GetPropVal( _T("name"),  wxEmptyString );
            gdd->Title = gdnode->GetPropVal( _T("title"), wxEmptyString );
            gdd->Type  = gdnode->GetPropVal( _T("type"),  _T("slider")  );
            gdd->PageName = pagename;

            gdd->ValidGauges.Clear();

            if(gdnode->HasProp(_T("valid"))){
                GetGaugeTypesFromStr( gdnode->GetPropVal( _T("valid"),_T("")), gdd->ValidGauges);
            } else {
                WX_APPEND_ARRAY(gdd->ValidGauges, globvalids);
            }

            gdnode->GetPropVal(_T("min"),_T("0")).ToLong(&(gdd->Min));
            gdnode->GetPropVal(_T("max"),_T("0")).ToLong(&(gdd->Max));
            gdnode->GetPropVal(_T("factor"),_T("1")).ToLong(&(gdd->Factor));

            m_gaugeTableDesc.Add(gdd);

        }

    }

}

wxString GaugeDescManager::GetGaugeSymbolType(const wxString &symbolname)
{
    if( m_symbolDesc.find( symbolname ) != m_symbolDesc.end() )
        return m_symbolDesc[symbolname]->Type;

    return wxEmptyString;
}

wxString GaugeDescManager::GetGaugeSymbolTitle(const wxString &symbolname)
{
    if( m_symbolDesc.find( symbolname ) != m_symbolDesc.end() )
        return m_symbolDesc[symbolname]->Title;

    return wxEmptyString;
}

void GaugeDescManager::GetSymbolDescs(SymbolGaugeDescArray &arr)
{
    for( String2SymbolGaugeDescHash::iterator it = m_symbolDesc.begin(); it != m_symbolDesc.end(); it++){
        if( CheckSymbolName( it->first) )
            arr.Add( it->second );
    }

    arr.Sort( &SymbolGaugeDesc::compare_function );
}

/// 18us / P4@3GHz
void GaugeDescManager::GetGrouppedSymbolDescs(SymbolGaugeDescArrayHash &hash)
{
    for( String2SymbolGaugeDescHash::iterator it = m_symbolDesc.begin(); it != m_symbolDesc.end(); it++){

        if( CheckSymbolName( it->first) == false ) continue;

        SymbolGaugeDesc * desc = it->second;

        hash[desc->Groupname].Add(desc);
    }

//    arr.Sort( &SymbolGaugeDesc::compare_function );
}

size_t GaugeDescManager::GetSymbolGroupListM(wxSortedArrayString &arr)
{
    for( String2SymbolGaugeDescHash::iterator it = m_symbolDesc.begin(); it != m_symbolDesc.end(); it++){

        if( CheckSymbolName( it->first ) == false ) continue;

        SymbolGaugeDesc * desc = it->second;

        if(arr.Index(desc->Groupname) == wxNOT_FOUND) arr.Add(desc->Groupname);
    }

    return arr.GetCount();
}

size_t GaugeDescManager::GetSymbolDescs(const wxString& grpname, SymbolGaugeDescArray &arr)
{
    for( String2SymbolGaugeDescHash::iterator it = m_symbolDesc.begin(); it != m_symbolDesc.end(); it++){

        if( CheckSymbolName( it->first ) == false ) continue;

        SymbolGaugeDesc * desc = it->second;

        if(desc->Groupname == grpname) arr.Add(desc);
    }

    arr.Sort( &SymbolGaugeDesc::compare_function );

    return arr.GetCount();
}

/*********************** SymbolGaugeDesc
*/
int SymbolGaugeDesc::compare_function(SymbolGaugeDesc **A,SymbolGaugeDesc **B)
{
    return ((*A)->Title).CmpNoCase((*B)->Title);
}

