#ifndef MULTIGRAPHDESCMANAGER_H
#define MULTIGRAPHDESCMANAGER_H

#include <wx/wx.h>
#include <wx/wxsqlite3.h>
#include "MultiGraphDesc.h"
#include "MultiGraphTheme.h"
#include "GUIDescStructItem.h"

#include "../common/types/String2IntHash.h"

class MultiGraphDescManager
{
    public:
        MultiGraphDescManager();
        virtual ~MultiGraphDescManager();

        void InitMgr(const wxString& descDBFileName = wxEmptyString, const wxString& themeDBFileName = wxEmptyString);

        bool LoadMultiGraphDesc();
        void GetMultiGraphDescProfiles(wxVariantHashArray& profiles);
        void SetMultiGraphDescProfileTitle(size_t id, const wxString& title);
        void SetMultiGraphDesc(MultiGraphDesc* desc, size_t id);
        void SaveMultiGraphDesc(size_t id);
        void DelMultiGraphDescProfile(size_t id);
        wxString GetMultiGraphDescFile() { return m_mvgDescFile; };
        wxString GetMultiGraphDescProfileTitle(size_t id);
        MultiGraphDesc* GetMultiGraphDesc(size_t id);
        MultiGraphDesc* SaveAsMultiGraphDesc(size_t id, const wxString& title);

        GUIDescStructItemArray& GetMGDescStruct() { return m_MGdescStruct; };



        MultiGraphTheme* GetMultiGraphTheme(size_t id);
        MultiGraphTheme* SaveAsMultiGraphTheme(size_t id, const wxString& title);
        wxString GetMultiGraphThemeProfileTitle(size_t id);
        void SetMultiGraphTheme(MultiGraphTheme* theme, size_t id);
        void SetMultiGraphThemeProfileTitle(size_t id, const wxString& title);
        void SaveMultiGraphTheme(size_t id);
        void GetMultiGraphThemeProfiles(wxVariantHashArray& profiles);
        void DelMultiGraphThemeProfile(size_t id);

        MultiGraphThemePropertyArray& GetMultiGraphThemePropertyArray() { return m_mgThemePropDesc; }

        void UpdateSymbol(wxVariantHash &hash);

    protected:

        void CheckDescDB();
        void CheckThemeDB();

        MultiGraphDesc* LoadMultiGraphDescProfile(int profilID);
        MultiGraphTheme* LoadMultiGraphThemeProfile(int profile_id);

        void LoadMultiGraphThemes();

    private:

        int InsertNode(MultiGraphDescNode &nodeDesc, int profile_id);
        int InsertSymbol(wxVariantHash& hash, int node_id);

        void UpdateNode(MultiGraphDescNode &nodeDesc);


        void CreateMGDescStruct();

        MultiGraphDescHash      m_multiGraphDescHash;
        MultiGraphThemeHash     m_multiGraphThemeHash;
        wxString                m_mvgDescFile;
        wxString                m_mvgThemeFile;
        GUIDescStructItemArray     m_MGdescStruct;

        wxSQLite3Database       m_descDB;
        wxSQLite3Database       m_themeDB;

        wxString                m_defaultThemePropStatements;

        MultiGraphThemePropertyArray m_mgThemePropDesc;
        String2IntHash          m_themePropNameToId;
};


#endif // MULTIGRAPHDESCMANAGER_H
