#include "MultiSymbolGaugeDescMgr.h"

#include "../wxSqliteHelper.h"
#include "../logger.h"
#include "../UserLog.h"

#define chkdb(x) if( ! (m_db.IsOpen()) ) return x;

MultiSymbolGaugeDesc::MultiSymbolGaugeDesc(wxVariantHash& properties)
{
    m_properties = properties;
}

MultiSymbolGaugeDesc::MultiSymbolGaugeDesc(long id, const wxString& name)
{
    m_properties[_T("id")] = id;
    m_properties[_T("name")] = name;
}

MultiSymbolGaugeDesc::MultiSymbolGaugeDesc(wxSQLite3ResultSet& profileLineResult)
{
    wxSqliteHelper::SQLLineToVariantHash(profileLineResult, m_properties);
}

int MultiSymbolGaugeDesc::GetId()
{
    return m_properties[_T("id")].GetLong();
}

wxString MultiSymbolGaugeDesc::GetName()
{
    return m_properties[_T("name")].GetString();
}

wxVariantHash& MultiSymbolGaugeDesc::GetSymbol(const wxString& name)
{
    for(size_t a=0; a<m_symbols.GetCount(); a++) {
        if( m_symbols[a][_T("symbol_name")].GetLong() == name ) return m_symbols[a];
    }

    return m_dummy;
}

wxVariantHash& MultiSymbolGaugeDesc::GetSymbol(int id)
{
    for(size_t a=0; a<m_symbols.GetCount(); a++) {
        if( m_symbols[a][_T("id")].GetLong() == id ) return m_symbols[a];
    }

    return m_dummy;
}

bool MultiSymbolGaugeDesc::SetProp(const wxString& key, const wxVariant& val)
{
    m_properties[key] = val;
    return true;
}

wxVariant MultiSymbolGaugeDesc::GetProp(const wxString& key, const wxVariant& defval)
{
    if(m_properties.find(key) == m_properties.end()) return defval;

    return m_properties[key];
}

/************************************************************
                    MultiSymbolGaugeDescMgr
*/

MultiSymbolGaugeDescMgr::MultiSymbolGaugeDescMgr(MultiSymbolGaugeDesc defaultdesc, ColumnDescArrayHash dbStruct)
{
    //ctor
    m_lastDatabaseFile = wxEmptyString;

    m_defaultDesc = defaultdesc;

    m_dbStruct = dbStruct;
}

MultiSymbolGaugeDescMgr::~MultiSymbolGaugeDescMgr()
{
    //dtor
}

bool MultiSymbolGaugeDescMgr::CheckDB()
{
    chkdb(false);

    wxSqliteHelper::DatabaseCheck(m_db,m_dbStruct);

    return true;
}

bool MultiSymbolGaugeDescMgr::LoadFromFile(const wxString& filename, const wxString& inipath)
{
    if(m_db.IsOpen()) m_db.Close();

    try {

    m_db.Open(filename);

    } catch (...) {

        Logger::logf(LOG_MSG,_T("MultiSymbolGaugeDescMgr::LoadFromFile exception!!"));

    }

    if(!m_db.IsOpen()) {

        wxString msg = _T("Database opening failed. Filename: ")+filename;
        msg += _T("<br>Please check the settings in v3gui.cfg!");
        if(inipath != wxEmptyString)
            msg += _T(" (") + inipath + _T(")");

        UserLog::uLog(ULOG_ERROR, msg);
        return false;

    }


    CheckDB();

    m_profiles.clear();

    wxString qs = _T("select * from profiles;");

    wxSQLite3ResultSet res = m_db.ExecuteQuery(qs);

    while(res.NextRow()){
        MultiSymbolGaugeDesc desc(res);

        int pid = desc.GetId();

        wxSQLite3ResultSet pres = m_db.ExecuteQuery(wxString::Format(_T("select * from symbols where profile_id=%i order by id asc;"),pid));

        while(pres.NextRow()){
            wxVariantHash sprop;
            wxSqliteHelper::SQLLineToVariantHash(pres, sprop);
            desc.AddSymbolProps(sprop);
        }

        m_profiles[pid] = desc;
    }

    m_lastDatabaseFile = filename;

    return true;
}

MultiSymbolGaugeDesc& MultiSymbolGaugeDescMgr::GetProfile(const wxString& name)
{
    for(Int2MultiSymbolGaugeDescHash::iterator it = m_profiles.begin(); it != m_profiles.end(); it++ ) {

        MultiSymbolGaugeDesc& desc = it->second;

        if( desc.GetName() == name ) return desc;
    }

    return m_defaultDesc;
}

MultiSymbolGaugeDesc& MultiSymbolGaugeDescMgr::GetProfile(int id)
{
    if( m_profiles.find(id) == m_profiles.end() ) return m_defaultDesc;

    return m_profiles[id];
}

bool MultiSymbolGaugeDescMgr::IsProfileExists(const wxString& name)
{
    for(Int2MultiSymbolGaugeDescHash::iterator it = m_profiles.begin(); it != m_profiles.end(); it++ ) {

        MultiSymbolGaugeDesc& desc = it->second;

        if( desc.GetName() == name ) return true;
    }

    return false;
}

bool MultiSymbolGaugeDescMgr::IsProfileExists(int id)
{
    return m_profiles.find(id) == m_profiles.end() ? false : true;
}

