#include "OutputChannelSQL.h"

#include "../logger.h"
#include "../DirHelper.h"
#include "../Math.h"

wxString OutputChannelSQL::s_dbFilename =_T("outputchannels.s3db");
wxSQLite3Database OutputChannelSQL::m_db;

unsigned short OutputChannelSQL::m_validtypes = 0;

OutputType::OutputType(const OutputType& ot)
{
    channel = ot.channel; pin = ot.pin; current = ot.current; connector_type = ot.connector_type;
    name = ot.name; title = ot.title; type = ot.type;
}

OutputChannelSQL::OutputChannelSQL()
{

}

OutputChannelSQL::~OutputChannelSQL()
{
}

void OutputChannelSQL::openDB()
{
    if ( m_db.IsOpen() )
        return;

    wxString filename = DirHelper::GetPath(dCONFIG) + s_dbFilename;
    m_db.Open( filename );
}

OutputType* OutputChannelSQL::getOutputType(const wxString& str)
{
    wxString tmp = str.BeforeFirst(';');
    if ( str == str_invalid_value ) return NULL;
    if (tmp == str)
    {
        OutputType* ot = new OutputType();
        ot->channel = 0xff; // disabled
        return ot;
    }

    wxString ochannel = tmp.AfterFirst('-'); wxString otype = tmp.BeforeFirst('-');

    openDB();

    try
    {
        wxString query = _T("SELECT types_id FROM output_types WHERE name = '")+ otype + _T("'");
        wxSQLite3ResultSet res = m_db.ExecuteQuery(query);
        int typenum = -1; long channum; ochannel.ToLong(&channum);
        while (res.NextRow()) { typenum = res.GetInt(0); }
        if ( typenum == -1 ) return NULL;

        uint8_t _type = 16 * channum + (uint8_t)typenum;
        OutputType* ot = getOutputType(_type);
        return ot;
    }
    catch(wxSQLite3Exception e)
    {
        return NULL;
    }
}

wxString OutputChannelSQL::getInjGroupPins()
{
    wxString result;

    openDB();

    try
    {
        wxString query = _T("SELECT bit,pin FROM injgrouppins ORDER BY bit");
        wxSQLite3ResultSet res = m_db.ExecuteQuery(query);
        uint8_t ctr = 0;
        while (res.NextRow())
        {
            int bit = res.GetInt(_T("bit")); wxString pin = res.GetString(_T("pin"));
            if (bit == ctr) { result += (pin + _T("|")); ctr++; continue; }
            else {
                for (int i=ctr;i<bit;++i) result += _T("|");
                result += (pin + _T("|")); ctr++; continue;
            }
        }
        return result;
    }
    catch(wxSQLite3Exception e)
    {
        return wxEmptyString;
    }
}

OutputType* OutputChannelSQL::getOutputType(uint8_t channel)
{
    uint8_t type_id = channel & 15;
    uint8_t channel_id = channel >> 4;

    // inverted?
    if ( channel_id > 7) channel_id -= 8;

    OutputType* outputtype = NULL;

    if ( type_id == 0x0F )
    {
        outputtype = new OutputType;
        outputtype->channel = channel; outputtype->name = str_disabled_value;
        outputtype->connector_type = 0;
        outputtype->current = _T("?");
        outputtype->pin = _T("0");
        return outputtype;
    }

    openDB();

    try
    {
        wxString query = _T("SELECT * FROM output_types WHERE types_id = ")+ wxString::Format(_T("%d"), type_id);
        wxSQLite3ResultSet res = m_db.ExecuteQuery(query);
        while (res.NextRow())
        {
            outputtype = new OutputType;
            outputtype->channel = channel;
            outputtype->connector_type = res.GetInt(_T("connector_type"));
            outputtype->current = res.GetString(_T("current"));
            outputtype->name = res.GetString(_T("name"));
            outputtype->title = res.GetString(_T("title"));
            outputtype->type = res.GetString(_T("type"));
        }
    }
    catch (wxSQLite3Exception e)
    {

    }

    if (outputtype != NULL)
    {
        outputtype->pin = OutputChannelSQL::getPin(channel);
    }
    return outputtype;
}

wxString OutputChannelSQL::getPin(uint8_t channel)
{
    openDB();

    uint8_t channelid = channel>>4, type_id = channel & 15;
    if (channelid > 7) channelid -= 8;// inverted
    wxString q = wxString::Format(_T("select pin from output_channels where type_id=%i and channel_id=%i"), type_id, channelid);
    wxSQLite3ResultSet res = m_db.ExecuteQuery(q);

    wxString pin;
    while (res.NextRow())
    {
        pin = res.GetString(_T("pin"));
    }
    return pin;

}

wxObjectArray OutputChannelSQL::getSymbols(bool onlyVisible)
{
    openDB();
    wxString query;

    if ( onlyVisible ) query =  wxT("SELECT * FROM symbols WHERE s_order>-1 ORDER BY s_order ASC");
    else query = wxT("SELECT * FROM symbols ORDER BY s_order ASC");

    wxObjectArray result;

    try
    {
        wxSQLite3ResultSet res = m_db.ExecuteQuery(query);
        while (res.NextRow())
        {
            dbsymbol* sym = new dbsymbol;
            sym->name = res.GetString(_T("name"));
            sym->fullname = res.GetString(_T("fullname"));
            result.Add(sym);
        }
    }
    catch (wxSQLite3Exception e)
    {
    }
    return result;
}

void OutputChannelSQL::getValidTypes()
{
    if ( m_validtypes == 0 )
    {
        openDB();
        wxString query = wxT("SELECT type_id FROM output_channels");
        try{
            wxSQLite3ResultSet res = m_db.ExecuteQuery(query);
            while (res.NextRow())
            {
                int type_id = res.GetInt(_T("type_id"));
                m_validtypes |= Math::powInt(2, type_id);
            }
        }
        catch (wxSQLite3Exception e){ }
    }
}

IntArray OutputChannelSQL::getChannels()
{
    IntArray result;
    openDB();

    wxString query = wxT("SELECT * FROM output_channels");
    try{
        wxSQLite3ResultSet res = m_db.ExecuteQuery(query);
        while (res.NextRow())
        {
            int channelid = res.GetInt(_T("channel_id")), type_id = res.GetInt(_T("type_id"));
            result.Add(16*channelid+type_id);

            m_validtypes |= Math::powInt(2, type_id);
        }
    }
    catch (wxSQLite3Exception e)
    {

    }
    return result;
}

bool OutputChannelSQL::isValidType(int hexid)
{
    if (hexid<0 || hexid>255) return false;

    uint8_t type_id = hexid & 15;

    if (type_id == 0x0F) return true; // disabled

    getValidTypes();

    if ( m_validtypes & Math::powInt(2, type_id))
    {
        return true;
    }
    else return false;
}

bool OutputChannelSQL::getIgnitionMapping(bool isDual, uint8_t h2xvalue, uint8_t& type_id, uint8_t& channel_id)
{
    openDB();

    try
    {
        wxString query;
        if ( isDual )
            query = wxString::Format(_T("SELECT type_id, channel_id FROM ignitionmap WHERE dual = 1 AND h2x = %d"), h2xvalue);
        else
            query = wxString::Format(_T("SELECT type_id, channel_id FROM ignitionmap WHERE dual = 0 AND h2x = %d"), h2xvalue);

        wxSQLite3ResultSet res = m_db.ExecuteQuery(query);

        if ( res.NextRow() == false )
            return false;

        type_id = res.GetInt(_T("type_id"));
        channel_id = res.GetInt(_T("channel_id"));

        return true;
    }
    catch(wxSQLite3Exception e)
    {
        return false;
    }
}

IntArray OutputChannelSQL::getIgnitionOutputs(bool isDual)
{
    openDB();
    IntArray result;
    result.Clear();

    try {
        wxString query = wxString::Format(_T("SELECT type_id, channel_id FROM ignitionmap WHERE dual = %d"), (int) isDual);

        wxSQLite3ResultSet res = m_db.ExecuteQuery(query);

        while ( res.NextRow() )
        {
            int type_id = res.GetInt(_T("type_id"));
            int ch_id = res.GetInt(_T("channel_id"));

            int hex = 16 * ch_id + type_id;
            result.Add( hex );

            if ( isDual )
            {
                hex = 16 * ( ch_id - 1 ) + type_id;
                result.Add( hex );
            }
        }

        return result;
    }
    catch ( wxSQLite3Exception e )
    {
        return result;
    }
}

int OutputChannelSQL::getIgnh2x(int channel_id, int type_id, bool isDual )
{
    openDB();
    int result = -1;

    try {
        wxString query = wxString::Format(_T("SELECT h2x FROM ignitionmap WHERE dual = %d AND type_id = %d AND channel_id =%d"),
            (int) isDual, type_id, channel_id );

        wxSQLite3ResultSet res = m_db.ExecuteQuery(query);

        while ( res.NextRow() )
        {
            result = res.GetInt(_T("h2x"));
        }

        return result;
    }
    catch ( wxSQLite3Exception e )
    {
        return result;
    }
}
/*
getTitle(uint8_t channel);

bool OutputChannelSQL::LoadDescriptors(const wxString &filename)
{
    FunctionMeter fm(_T("GaugeDescManagerSQL::LoadDescriptors"));
    m_dbFilename = filename;

    db.Open( filename );
    if( !db.IsOpen() )
        return false;

    wxSQLite3ResultSet res = db.ExecuteQuery( _T("select * from descriptors order by name asc;") );

    VEMSGaugeDesc * gd;

    while( res.NextRow() ){

        gd = new VEMSGaugeDesc();

        for(int a = 0; a < res.GetColumnCount(); a++) {

            switch( res.GetColumnType(a) ){
                case WXSQLITE_INTEGER:
                    gd->Properties[ res.GetColumnName(a) ] = (long)(res.GetInt(a)); break;
                case WXSQLITE_FLOAT:
                    gd->Properties[ res.GetColumnName(a) ] = res.GetDouble(a); break;
                case WXSQLITE_TEXT:
                    gd->Properties[ res.GetColumnName(a) ] = res.GetString(a); break;
                default:
                    break;
            }
        }

        wxSQLite3ResultSet cares = db.ExecuteQuery( wxString::Format(_T("select * from colored_areas where desc_id=%i order by min asc;"),res.GetInt(_T("id"))) );

        while( cares.NextRow() ){
            gd->coloredA.Add( new ColoredArea(cares.GetDouble(_T("min")),cares.GetDouble(_T("max")),cares.GetString(_T("colorname")),cares.GetInt(_T("id"))) );
        }

        m_gaugeTable.Add( gd );

    }

    wxSQLite3ResultSet sres = db.ExecuteQuery( _T("select * from symbols;") );

    while( sres.NextRow() ){
        m_symbolDesc[ sres.GetString(_T("name")) ] = new SymbolGaugeDesc(sres.GetString(_T("name")),sres.GetString(_T("type")),sres.GetString(_T("title")));
    }

    db.Close();

    return true;
}
*/
