#ifndef VEMSFLAGDESCMANAGER_H
#define VEMSFLAGDESCMANAGER_H

#include <wx/wx.h>
#include <wx/wxsqlite3.h>
#include "../common/types/wxVariantHashArray.h"

class VemsFlagStateDesc
{
    public:
        VemsFlagStateDesc();
        VemsFlagStateDesc(long value, const wxString& text);
        virtual ~VemsFlagStateDesc();

        void ParseColors();

        bool SetValue(const wxString &value);
        bool SetValue(long value);
        bool SetText(const wxString &text);
        bool SetForegroundColor(const wxString &fgcstring);
        bool SetBackgroundColor(const wxString &bgcstring);
        bool SetImage(const wxString &imagename);

        int  GetValue();
        wxString GetText();
        wxColor GetForegroundColor();
        wxColor GetBackgroundColor();
        wxColor GetForegroundColor2();
        wxColor GetBackgroundColor2();
        bool IsFlash();
        long GetFlashInterval();
        wxBitmap GetImage();

        wxVariantHash   Properties;

};

WX_DECLARE_HASH_MAP(int, VemsFlagStateDesc, wxIntegerHash, wxIntegerEqual, VemsFlagStates);


extern VemsFlagStateDesc DefaultVemsFlagStateDesc;

extern VemsFlagStateDesc DisabledVemsFlagStateDesc;

class VemsFlagDesc
{
    public:
        VemsFlagDesc() {};
        VemsFlagDesc(int id, const wxString& name){
            Properties[_T("id")] = (long)id;
            Properties[_T("name")] = name;
            Properties[_T("autosize")] = 0L;
        };
        virtual ~VemsFlagDesc() {};

        wxVariantHash   Properties;
        VemsFlagStates  States;

};

WX_DECLARE_STRING_HASH_MAP(VemsFlagDesc, VemsFlagDescHash);
WX_DECLARE_OBJARRAY(VemsFlagDesc, VemsFlagDescArray);

class VemsFlagDescManager
{
    public:
        VemsFlagDescManager();
        virtual ~VemsFlagDescManager();

        bool LoadDescriptorFromFile(const wxString& filepath);
        bool LoadGroups();

        bool CheckDB();

        VemsFlagStates& GetVemsFlagDescStates(const wxString &name);
        VemsFlagDesc& GetVemsFlagDesc(const wxString &name);
        VemsFlagDesc& GetVemsFlagDesc(int id);
        VemsFlagDescHash& GetAllDesc() { return m_vemsFlagDesc; };
        size_t GetAllDesc(VemsFlagDescArray& arr);

        wxString GetLastDbFile() { return m_lastDatabaseFile; }

        VemsFlagStateDesc& GetVemsFlagStateDescById(int state_id);

        wxVariantHashArray& GetGroups() { return m_flagGroups; };

        static int flag_compare_function_title(VemsFlagDesc **A,VemsFlagDesc **B);

    protected:

    private:

        wxSQLite3Database   m_db;
        VemsFlagDescHash    m_vemsFlagDesc;
        wxVariantHashArray  m_flagGroups;

        wxString            m_lastDatabaseFile;

        VemsFlagDesc        m_nullDesc;

};

#endif // VEMSFLAGDESCMANAGER_H
