#ifndef HEADERPARSER_H
#define HEADERPARSER_H

#include <wx/wx.h>

#include "../fileparser/parserbase.h"

#include "../common/types/StringHash.h"

#include "expression/ExpressionParser.h"

#include "VariableDescriptor.h"

#define MAX_IFDEF_LEVEL 30

class VariableList;

class HeaderParser : public ParserBase
{
    public:
        HeaderParser( const wxString& mymake_filename );

    public:
        void                    parseLine();
        String2StructVarHash    getParsedStructs() const { return m_parsedStructVariables; }

    private:
        bool                    m_insideComment;
        bool                    m_ifdeflevels[MAX_IFDEF_LEVEL];

        bool                    m_parsingStruct;

        size_t                  m_currentifdeflevel;

        // doublehash [#define] => val
        VariableList*           m_defines;
        ExpressionParser        m_exprParser;

        bool                    m_in_function;
        int                     m_function_opened_brackets;

        StructArray             m_parsedStructTypes;

        String2StructVarHash    m_parsedStructVariables;

        StructDescriptor*       m_currentStruct;

    private:
        void    parseMyMake( const wxString& filename );

        void    trim();
        void    trim( wxString& line );
        void    stripComments();
        void    stripMultispaces( wxString & line );

        bool    checkCommentEnd();
        void    checkCommentBegin();

        void    doIfdefCheck( bool ifdef ); // ifdef is true if directive is #ifdef, false if directive is #ifndef

        void    createDefine();
        void    removeDefine();
        void    checkIfCondition();
        void    showErrWarn( bool isError );

        void    createNewStruct( const wxString& structName );
        void    parseStructLine();
        StructDescriptor* findStructDescr( const wxString& structname );

        inline
        int     countChars( wxString str_in, wxString str_what );
};

#endif // HEADERPARSER_H
