#include "SymbolTableFiller.h"

#include "../Symbol.h"
#include "../SymbolMgr.h"
#include "VariableDescriptor.h"
#include "../logger.h"

SymbolTableFiller::SymbolTableFiller( SymbolMgr* symbolMgr )
{
    m_symbolMgr = symbolMgr;
    m_offset    = 0;
}

void SymbolTableFiller::fillSymbolTable( String2StructVarHash parsedData, const wxString& tablename, unsigned long startAddress )
{
    m_offset = startAddress;

    if ( m_symbolMgr == NULL )
        return;

    String2StructVarHash::const_iterator it = parsedData.find( tablename );

    if ( it != parsedData.end() )
    {
        StructureVar strV = it->second;

        int size = strV.size;

        if ( size == 1 )
        {
            StructDescriptor sd = strV.structType;

            wxString varname = tablename + _T("@");

            createSymbols( &sd, varname );
        }
        else
        {
            StructDescriptor sd = strV.structType;

            for ( size_t j = 0; j < size; ++j )
            {
                wxString varname = tablename + wxString::Format( _T("[%d]"), j ) + _T("@");
                createSymbols( &sd, varname );
            }
        }
    }
}

void SymbolTableFiller::createSymbols( StructDescriptor* structdescr, wxString parentname )
{
    for ( size_t i = 0; i < structdescr->fields.GetCount(); ++i )
    {
        VariableDescriptor vd = structdescr->fields[i];

        if ( vd.isElementary )
        {
            wxString varname = parentname + vd.name;

            Symbol* sym = NULL;

            if ( vd.isArray )
            {
                sym = Symbol::createArray( varname, vd.arraySize, 1, -1, m_offset );
                sym->setType( false, vd.elemsize );
            }
            else
            {
                sym = Symbol::createScalar( varname, -1, m_offset );
            }

            m_symbolMgr->addOrReplaceSymbol( sym );
            m_offset += vd.getSize();
        }
        else
        {
            // struct
            if ( vd.isArray == false )
            {
                wxString structname = parentname + vd.name + _T("@");
                createSymbols( vd.structDescriptor, structname );
            }
            else
            {
                for ( size_t j = 0; j < vd.arraySize; ++j )
                {
                    wxString structname = parentname + vd.name + wxString::Format( _T("[%d]"), j ) + _T("@");
                    createSymbols( vd.structDescriptor, structname );
                }
            }
        }
    }
}
