#include "ExpressionException.h"


ExpressionException::ExpressionException( const int row, const int col, const int id, const wxString& expr, const char* name )
{
    m_expr = expr;

    if ( name )
        m_name = wxString::From8BitData( name );
    else
        m_name = wxEmptyString;

    m_col = col;

    errcode = id;

    if ( m_row >= 0 )
        m_row = row;
    else
        m_row = -1;

    createErr();
}

wxString ExpressionException::getExprErr() const
{
    wxString ret = _T("\n") + m_expr + _T("\n");

    for ( int i = 0; i < m_col-1; ++i )
        ret += _T(" "); ret += _T("^");

    return ret;
}

void ExpressionException::setRow( int row )
{
    m_row = row;
    createErr();
}

void ExpressionException::setCol( int col )
{
    m_col = col;
    createErr();
}

void ExpressionException::createErr()
{
    wxString errMsg = _T("");

    switch ( errcode )
    {
        case 1:
        case 2: errMsg = _T("Syntax error"); break;
        case 3: errMsg = _T("Parentesis ) missing"); break;
        case 4: errMsg = _T("Empty expression"); break;
        case 5:
        case 6: errMsg = _T("Unexpected end of expression"); break;
        case 7: errMsg = _T("Value expected"); break;
        case 101: errMsg = _T("Unknown operator '")+m_name+_T("'"); break;
        case 102: errMsg = _T("Unknown function '")+m_name+_T("'"); break;
        case 103: errMsg = _T("Unknown variable '")+m_name+_T("'"); break;
        case 200: errMsg = _T("Too long expression, maximum number of characters exceeded"); break;
        case 300: errMsg = _T("Defining variable failed"); break;
        case 400: errMsg = _T("Integer value expected in function"); break;
    }

    if ( m_row >= 0 )
        err = errMsg + wxString::Format( _T(" in line %d, column %d"), m_row, m_col );
    else
        err = errMsg + wxString::Format( _T(" in column %d"), m_col );
}
