#include "GpsCommClient.h"
#include "../Math.h"
#include "../vemsprotocol/SIPRPacket.h"
#include "../vemsprotocol/SIPRTypes.h"
#include "../events/InputEvent.h"
#include "../events/GPSEvent.h"
#include "../events/GPSCommEvent.h"


GpsCommClient::GpsCommClient( wxEvtHandler* eventHandler )
{
    m_eventHandler      = eventHandler;

    m_seqence           = 1;
    m_packedGpsData[0]  = 0;
    m_packedGpsData[1]  = 0;
    m_packedGpsData[2]  = SiprSensorType::GPS;
    m_packedGpsData[16] = 255;

    m_state         = GPSInfo::NOT_CONNECTED;
    m_noDataCount   = 0;
}

GpsCommClient::~GpsCommClient()
{
}

void GpsCommClient::write( uint8_t* buffer, int& size )
{
    return;
}

void GpsCommClient::read( uint8_t* buffer, int size )
{
    // because GPS receivedBytes = 0 with some BlueTooth driver
    // check received data  with buffer[0] != 0 instead of size > 0

    if ( buffer[0] == 0 ) {
        m_noDataCount++;
        wxThread::Sleep( 5 );

        if( (m_noDataCount == 400)  ){
            m_state = GPSInfo::NO_GPS_DATA;
            sendEvent(GPSInfo::NO_GPS_DATA);
            wxThread::Sleep( 1000 );
        }
    } else {

        int parserStatus = m_parser.feedGPScharacter( buffer[0] );

        if ( parserStatus == END ) {
            m_noDataCount = 0;

            if( m_state != GPSInfo::CONNECTED ) {
                m_state = GPSInfo::CONNECTED;
                sendEvent(GPSInfo::CONNECTED);
            }

            sendInputEvent();
        }

    }
}

void GpsCommClient::finish()
{
    sendEvent(GPSInfo::OFFLINE);
}

bool GpsCommClient::isFinished()
{
    return false;
}

void GpsCommClient::sendInputEvent()
{
    if ( !m_eventHandler ) { return; }

    packData(&m_packedGpsData[3]);

    InputEvent e;
    SIPRPacket gpsPacket( m_seqence++, SIPR_READ_SENSORS, 17, m_packedGpsData );
    e.setPacket( gpsPacket );
    m_eventHandler->AddPendingEvent( e );
}

void GpsCommClient::sendEvent(GPSInfo::CommState state)
{
    GPSCommEvent event;
    event.state = state;
    m_eventHandler->AddPendingEvent( event );
}

void GpsCommClient::sendEvent(GPSInfo::State state)
{
    GPSEvent event;
    event.state = state;
    m_eventHandler->AddPendingEvent( event );
}

void GpsCommClient::packData(uint8_t* packedGpsData)
{
    GPSDataStruct gpsData = m_parser.gpsdata;

    if( gpsData.errorbyte )
    {
        return;
    }

    int  lat = gpsData.latitude;

    //N = 0 , S = -1
    if( gpsData.latitudeindicator == 'S' )
    {
        lat *= -1;
    }

    Math::getBytes( lat, &packedGpsData[0] );

    int lon = gpsData.longnitude;

    //E = 0, W = -1
    if( gpsData.longnitudeindicator == 'W' )
    {
        lon *= -1;
    }

    Math::getBytes( lon, &packedGpsData[4] );

    int height = gpsData.antennaheight;

    height += 10000;
    Math::getBytes( height, &packedGpsData[8], 3);

    if( gpsData.dataFormat == NMEAParse::GPS_FORMAT_GPRMC )
    {
        Math::getBytes( gpsData.speed , &packedGpsData[11], 2);
        //0 = Valid
        //1 = Invalid
        if( gpsData.status == 'A' )
        {
            packedGpsData[8] &= 127;  /// 7.bit 0 Valid GPS Data
        }
        else
        {
            packedGpsData[8] |= 128; ///7.bit 1 Invalid GPS Data
        }
    }

}
