#include "ConfigDownloadDirector.h"
#include "../vemsprotocol/ConfigDownloadController.h"
#include "../vemsprotocol/VemsDeviceControllerBase.h"
#include "../vemsprotocol/SIPRTypes.h"
#include "../vemsprotocol/SIPRPacket.h"
#include "../communication/clients/ConfigSaver2.h"
#include "../events/InputEvent.h"
#include "../events/CommunicationEvent.h"
#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"
#include "../DirHelper.h"
#include "../RawLogHelper.h"
#include "../logger.h"
#include "../UserLog.h"


const wxEventType ConfigDownloadDirectorEvent = wxNewEventType();


ConfigDownloadDirector::ConfigDownloadDirector( wxWindow* parent, VemsDeviceControllerBase* deviceController )
{
//    m_configDownloadController = new ConfigDownloadController(this);
    m_configDownloadController = new ConfigDownloadController();
    m_configSaver = new ConfigSaver2();
    m_parent = parent;
    m_deviceController = deviceController;
//    deviceController->setEv
    Connect( wxEVT_INPUT, InputEventHandler( ConfigDownloadDirector::onInputEvent ), NULL, this );
    Connect( wxEVT_INPUT, InputEventHandler( ConfigSaver2::onInputEvent ), NULL, m_configSaver );
}

ConfigDownloadDirector::~ConfigDownloadDirector()
{
    delete m_configDownloadController;
    delete m_configSaver;
}

void ConfigDownloadDirector::saveDeviceConfig( bool showDialog )
{
    if ( !isConfigDownloadEnabled() ) {
        showErrorMessage();
        return;
    }

    wxString path = DirHelper::GetPath(dLOG);

    if ( showDialog ) {
        showFileDialog( path );
    }

    DeviceInfo deviceInfo       = m_deviceController->getDeviceInfo();
    FirmwareInfo& firmwareInfo  = deviceInfo.firmwareInfo;

    m_deviceController->stop( VDC_FLAG_NOEVENT );

    m_configDownloadController->setDeviceController( m_deviceController );
    m_configDownloadController->setEventHandler( this );

    wxString baseFileName   = firmwareInfo.getSerialNumber() + _T(".vemscfg");
    m_fileName       = DirHelper::CreateNewFileName( baseFileName, path, true, true );
    wxString header         = RawLogHelper::getVemsCfgHeader(firmwareInfo);

    LOGP( m_fileName );
    LOGP( header );

    connectCommunicationEventHandler();

    m_configSaver->init( m_fileName, header );
//    m_configDownloadController->start( deviceInfo );
    m_configDownloadController->start();
}

void ConfigDownloadDirector::showFileDialog( wxString& path )
{
    wxDirDialog dlg(m_parent);

    ConfigBase* cfg = ConfigHelper2::getConfig( ConfigHelper2::CONFIG_LAYOUT );

    dlg.SetPath( cfg->getValue(_T("/dialogpaths"), _T("configdownload_fw"), path ) );
    if( dlg.ShowModal() == wxID_OK ) {
        path = dlg.GetPath() + DirHelper::Separ();
        cfg->saveValue(_T("/dialogpaths"),_T("configdownload_fw"), path );
    }
}

bool ConfigDownloadDirector::isConfigDownloadEnabled() const
{
    return ( m_deviceController->isConnected() && ( m_deviceController->isV3() ) );
}

void ConfigDownloadDirector::showErrorMessage() const
{
        UserLog::vLog( _T("ecu_not_found_1") );
        wxString msg = _T("ECU is not detected!");
        wxMessageBox(msg);
}

void ConfigDownloadDirector::onInputEvent( InputEvent& event )
{
    SIPRPacket packet = event.getPacket();

    if ( packet.type() == SIPR_TABLES_END ) {
        //m_deviceController->resume();
        disconnectCommunicationEventHandler();
        wxMessageBox( _T("Config download completed successfully!\n") + m_fileName, _T("Information"), wxICON_INFORMATION );
        wxCommandEvent directorEvent( ConfigDownloadDirectorEvent, wxID_ANY );
        m_parent->AddPendingEvent( directorEvent );
    }
}

void ConfigDownloadDirector::onCommunicationEvent( CommunicationEvent& event )
{
    LOGP( event.toString() );

    disconnectCommunicationEventHandler();

    m_configSaver->close();
    m_configSaver->deleteFile();

    wxMessageBox( _T("Config download failed!"), _T("Error"), wxICON_ERROR );


    if ( event.GetId() == CommunicationEvent::TIMEOUT ) {

        m_deviceController->timeout();
    }
}

void ConfigDownloadDirector::connectCommunicationEventHandler()
{
    Connect(    wxID_ANY,
                wxEVT_COMMUNICATION,
                CommunicationEventHandler( ConfigDownloadDirector::onCommunicationEvent ),
                NULL, this );
}

void ConfigDownloadDirector::disconnectCommunicationEventHandler()
{
    Disconnect( wxID_ANY,
                wxEVT_COMMUNICATION,
                CommunicationEventHandler( ConfigDownloadDirector::onCommunicationEvent ),
                NULL, this );
}
