#include "ECUConfigPanel.h"
#include "../SerialPortHelper.h"
#include "../logger.h"
#include "../ConfigBase.h"
#include "../VemsTuneController.h"
#include "../SerialException.h"
#include "../configmanager/ConfigHelper2.h"
#include "../wxSystemHelper.h"
#include "../configmanager/CommunicationConfig.h"
#include "../configmanager/ConfigMgrCommunication.h"
#include "../vemsprotocol/ECUInfo.h"
#include "../customwidgets/InfoWindow.h"
#include "../communication/clients/SerialPortScanner.h"
#include "../events/SerialPortScannerEvent.h"

#include <wx/statline.h>

#include "../customwidgets/InfoWindow.h"
#include "../common/HelpUtil.h"

enum {
    IDC_SERIAL = 3000,
    IDC_TCP ,
    IDC_COMPORT,
    IDC_BAUDRATE,
    IDC_COMTIMEOUT ,
    IDC_RT_ERROR_COUNT,
    IDC_SDCARD,
    IDC_IPPORT,
    IDC_TCPTIMEOUT,
    IDC_DEFPROT,
    IDC_DEFREADC,
    IDC_DETECTBUTTON,
    IDC_SAVEBUTTON,
    IDC_CLOSEBUTTON,
    IDC_FWUPLOAD,
    IDC_DEVMGRBUTTON
};

BEGIN_EVENT_TABLE( ECUConfigPanel, wxDialog )
    EVT_SERIAL_PORT_SCANNER( SerialPortScannerEvent::SPS_EVT_DEVICE_FOUND, ECUConfigPanel::OnDeviceFound )
    EVT_SERIAL_PORT_SCANNER( SerialPortScannerEvent::SPS_EVT_DEVICE_NOT_FOUND, ECUConfigPanel::OnDeviceNotFound )
    EVT_SERIAL_PORT_SCANNER( SerialPortScannerEvent::SPS_EVT_CANNOT_OPEN_PORT, ECUConfigPanel::OnCannotOpenPort )
END_EVENT_TABLE();

ECUConfigPanel::ECUConfigPanel( wxWindow* parent, VemsTuneController* vemscontroller )
{
    m_controller    = vemscontroller;
    m_portScanner   = NULL;
    m_detecting     = false;
    m_restartControllerAfterClose = false;

    buildGUI( parent );
}


ECUConfigPanel::~ECUConfigPanel()
{
}

void ECUConfigPanel::findAndShow(wxWindow* parent, VemsTuneController* controller)
{
    wxWindow* win = wxFindWindowByName(_T("ECU Communication"),parent);
    if(!win) {
        ECUConfigPanel* f = new ECUConfigPanel( parent, controller );
        f->Show();
    } else
        win->SetFocus();
}

// Build GUI
void ECUConfigPanel::buildGUI( wxWindow* parent )
{
    CommunicationConfig commConfig;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrCommunication cfgMgr(cfg);
    cfgMgr.load(&commConfig);

    wxStaticText* m_ctext1, * m_ctext2, * m_ctext3, * m_ctext4, * m_ctext5, *currBaudrateLabel;
    wxStaticText* m_rtErrorText;

    Create( parent, wxID_ANY, _T("ECU Communication") );
    this->SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    this->ClearBackground();

    wxFlexGridSizer* mainSizer = new wxFlexGridSizer( 2, 1, 0, 0 );
    mainSizer->AddGrowableCol( 0 );  mainSizer->AddGrowableRow( 0 );

//    ConfigBase* mainCfg  = ConfigHelper2::getConfig();

    wxString value = wxEmptyString;

    // -----------------------
    // COMMUNICATION PANEL
    // -----------------------
    m_CommunicationPanel = new wxPanel( this );
    m_serialPanel = new wxPanel( m_CommunicationPanel );
    m_tcpPanel = new wxPanel( m_CommunicationPanel );

    wxStaticBox* box = new wxStaticBox( m_CommunicationPanel, wxID_STATIC, _T("Communication") );
    wxFont font = box->GetFont();  font.SetWeight( wxFONTWEIGHT_BOLD ); box->SetFont( font ); box->SetForegroundColour( *wxBLACK );

    wxStaticBoxSizer * sizer1 = new wxStaticBoxSizer( box, wxVERTICAL );
    wxBoxSizer* csizer1 = new wxBoxSizer( wxHORIZONTAL ); wxBoxSizer* csizer2 = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* csizer3 = new wxBoxSizer( wxHORIZONTAL ); wxBoxSizer* csizer4 = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* csizer5 = new wxBoxSizer( wxHORIZONTAL ); wxBoxSizer* csizer6 = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer* currBaudrateSizer = new wxBoxSizer( wxHORIZONTAL );

    wxBoxSizer* m_rtErrorSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* interWriteDelaySizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* preferredBaudrateSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* fwuploadBrsizer = new wxBoxSizer( wxHORIZONTAL );

    m_serial = new wxRadioButton( m_CommunicationPanel, IDC_SERIAL, _T("Serial:") );

    m_ctext1 = new wxStaticText( m_serialPanel, -1, _T("Port:") );

    currBaudrateLabel = new wxStaticText( m_serialPanel, -1, _T("Current Baudrate:") );
    wxString currBaudrateStr = m_controller->getDeviceInfo().getBaudrateAsString();
    wxTextCtrl* currBaudrateTextCtrl = new wxTextCtrl( m_serialPanel, wxID_ANY, currBaudrateStr );
    currBaudrateTextCtrl->Enable(false);


    wxStaticText* preferredBaudrateLabel = new wxStaticText( m_serialPanel, -1, _T("Preferred baudrate:") );

    m_ctext2 = new wxStaticText( m_serialPanel, -1, _T("Timeout(ms):") );

    m_rtErrorText = new wxStaticText( m_serialPanel, -1, _T("Ignored realtime error count:") );

    wxStaticText* interWriteDelayText = new wxStaticText( m_serialPanel, -1, _T("Inter Write Delay(ms):") );
    wxStaticText* fwuploadBrlabel = new wxStaticText( m_serialPanel, -1, _T("Firmware upload baudrate") );

    m_ctext3 = new wxStaticText( m_serialPanel, -1, _T("SD Card baud rate:") ); m_ctext3->SetFont( font );
    m_com_timeout = new wxTextCtrl( m_serialPanel, IDC_COMTIMEOUT, wxEmptyString );
    m_rt_error_count = new wxTextCtrl( m_serialPanel, IDC_RT_ERROR_COUNT, wxEmptyString );
    m_rtInterWriteDelay = new wxTextCtrl( m_serialPanel, wxID_ANY, wxEmptyString );

    wxArrayString ports  = SerialPortHelper::GetCOMPortList();
    wxArrayString sdspeed; sdspeed.Add( wxT("57600") ); sdspeed.Add( wxT("115200") );

    // Serial ports
//    mainCfg->read( _T("/port"), &value );

    // add config value to the drop down list if not found
    if ( ports.Index( commConfig.port, false ) == wxNOT_FOUND )
    {
        ports.Add( commConfig.port );
    }

    /// ONLY FOR TEST
    //ports.Insert(_T("COM1"),0);
    //ports.Insert(_T("COM2"),1);  ports.Add(_T("COM6")); ports.Add(_T("COM7"));
    //ports.Add(_T("COM12"));
    //ports.Remove(L"COM3");
    //ports.Remove(L"COM5");
    /// ----

    ports.Sort();

    m_comport = new wxComboBox( m_serialPanel, IDC_COMPORT, commConfig.port,
        wxDefaultPosition, wxDefaultSize, ports, wxTE_PROCESS_ENTER );

    wxArrayString baudrates = SerialPortHelper::getSerialBaudrates();
//    baudrates.Add( _T("9600") );
//    baudrates.Add( _T("19200") );

    value = _T("19200");

    //mainCfg->read( _T("/preferredBaudrate"), &value );
    value = wxString::Format( _T("%d"), commConfig.preferredBaudrate );

    m_preferredBaudrateComboBox = new wxComboBox( m_serialPanel, IDC_BAUDRATE, value,
        wxDefaultPosition, wxDefaultSize, baudrates, wxTE_PROCESS_ENTER | wxCB_READONLY );

    // Serial port time-out
//    mainCfg->read( _T("/timeout"), &value );
    value = wxString::Format( _T("%d"), commConfig.timeout );
    m_com_timeout->SetValue( value );

//    mainCfg->read( _T("/rterrorcount"), &value );
    value = wxString::Format( _T("%d"), commConfig.rtErrorCount );
    m_rt_error_count->SetValue( value );

//    mainCfg->read( _T("/interWriteDelay"), &value );
    value = wxString::Format( _T("%d"), commConfig.interWriteDelay );
    m_rtInterWriteDelay->SetValue( value );
//
//    mainCfg->read( _T("/firmwareUploadBaudrate"), &value, _T("19200") );
    value = wxString::Format( _T("%d"), commConfig.fwUploadBaudrate );

    m_fwuploadBrComboBox = new wxComboBox( m_serialPanel, IDC_FWUPLOAD, value, wxDefaultPosition, wxDefaultSize,
        SerialPortHelper::getSerialBaudrates(), wxCB_READONLY );

    // SD Card Baud rate
//    mainCfg->read( _T("/sdcard/baudrate"), &value );
    value = wxString::Format( _T("%d"), commConfig.sdcardBaudrate );
    m_sdcard_b = new wxComboBox( m_serialPanel, IDC_SDCARD, value,
        wxDefaultPosition, wxDefaultSize, sdspeed );

    // Add controls to sizers
    currBaudrateSizer->Add( 60, 10, 0 );
    currBaudrateSizer->Add( currBaudrateLabel, 0, wxRIGHT, 5 );
    currBaudrateSizer->Add( currBaudrateTextCtrl, 1,  wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    csizer1->Add( m_ctext1, 0, wxRIGHT, 5 );
    csizer1->Add( m_comport, 1,  wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    preferredBaudrateSizer->Add( 60,10,0 ); preferredBaudrateSizer->Add( preferredBaudrateLabel, 0, wxRIGHT , 5 );
    preferredBaudrateSizer->Add( m_preferredBaudrateComboBox, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    fwuploadBrsizer->Add( 60, 10, 0 ); fwuploadBrsizer->Add( fwuploadBrlabel, 0, wxRIGHT , 5 );
    fwuploadBrsizer->Add( m_fwuploadBrComboBox, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    csizer2->Add( 60,10,0 ); csizer2->Add( m_ctext2, 0, wxRIGHT , 5 );
    csizer2->Add( m_com_timeout, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    m_rtErrorSizer->Add( 60,10,0 ); m_rtErrorSizer->Add( m_rtErrorText, 0, wxRIGHT , 5 );
    m_rtErrorSizer->Add( m_rt_error_count, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    interWriteDelaySizer->Add( 60,10,0 );
    interWriteDelaySizer->Add( interWriteDelayText, 0, wxRIGHT | wxALIGN_CENTER_VERTICAL , 5 );
    interWriteDelaySizer->Add( m_rtInterWriteDelay, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    wxBoxSizer* helpsizer = new wxBoxSizer(wxHORIZONTAL);
    helpsizer->Add( 60,10,0 );

    wxButton* devmgrbtn = new wxButton(m_serialPanel,IDC_DEVMGRBUTTON,_T("Open Device Manager"));
    helpsizer->Add(devmgrbtn, 0, wxALIGN_RIGHT);

    csizer3->Add( m_ctext3, 0, wxLEFT | wxRIGHT, 10 ); csizer3->Add( m_sdcard_b, 1, wxEXPAND | wxALIGN_RIGHT | wxRIGHT, 5 );

    // TCP
    m_tcp = new wxRadioButton( m_CommunicationPanel, IDC_TCP, _T("TCP:") );
//    m_tcp->Enable( false );

    m_ctext4 = new wxStaticText( m_tcpPanel, -1, _T("IP:port") );
    m_ctext5 = new wxStaticText( m_tcpPanel, -1, _T("Timeout") );
    m_ipport = new wxTextCtrl( m_tcpPanel, IDC_IPPORT, wxEmptyString );
    m_tcp_timeout =  new wxTextCtrl( m_tcpPanel, IDC_TCPTIMEOUT, wxEmptyString );

    csizer4->Add( m_ctext4, 0, wxLEFT | wxRIGHT, 5 );
    csizer4->Add( m_ipport, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 ); csizer5->Add( 52, 10, 0 );
    csizer5->Add(m_ctext5, 0, wxRIGHT, 10 ); csizer5->Add( m_tcp_timeout, 1, wxEXPAND | wxALIGN_RIGHT | wxRIGHT, 5 );

    wxStaticLine * line = new wxStaticLine( m_CommunicationPanel, -1, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );

    m_ipport->SetValue( commConfig.ip );
    value = wxString::Format( _T("%d"), commConfig.tcpTimeout );
    m_tcp_timeout->SetValue( value );

    // Detect button
    m_txtDetect = new wxStaticText( m_CommunicationPanel, wxID_ANY, _T("Click on Detect to detect ECU automatically"));
    m_progressbar = new ProgressBar( m_CommunicationPanel, wxID_ANY, wxDefaultPosition,wxSize(-1,20) );
    m_progressbar->SetWindowStyle( wxBORDER_SUNKEN );
    m_progressbar->SetProgressStyle(PB_PERCENTTEXT_INNER);
    m_progressbar->Update(0);

    m_progressbar->SetValueFormat(_T("%.1f"));
    m_progressbar->SetValueLimits(0, 100.0, 1.0);

    m_detectButton = new wxButton( m_CommunicationPanel, IDC_DETECTBUTTON, wxT("Detect") );

    csizer6->Add( line, 0, wxEXPAND | wxLEFT | wxRIGHT | wxALIGN_CENTER, 10 );
    csizer6->Add( m_txtDetect, 0, wxEXPAND | wxALIGN_CENTER | wxALL, 5);
    csizer6->Add( m_progressbar, 1, wxEXPAND | wxALL | wxALIGN_CENTER, 5);
    csizer6->Add( m_detectButton, 0, wxALL | wxALIGN_CENTER, 5 );


    bool isSerialMode = commConfig.mode.IsSameAs( _T("SERIAL") );

    /// Serial Panel sizers
    wxBoxSizer* serialSizer = new wxBoxSizer( wxVERTICAL );

    serialSizer->Add( csizer1, 0, wxEXPAND);
    serialSizer->Add( currBaudrateSizer, 0, wxEXPAND | wxTOP, 10 );
    serialSizer->Add( preferredBaudrateSizer, 0, wxEXPAND | wxTOP, 10 );
    serialSizer->Add( fwuploadBrsizer, 0, wxEXPAND | wxTOP, 10 );
    serialSizer->Add( csizer2, 0,  wxEXPAND | wxTOP, 10 );
    serialSizer->Add( m_rtErrorSizer, 0,  wxEXPAND | wxTOP, 10 );
    serialSizer->Add( interWriteDelaySizer, 0,  wxEXPAND | wxTOP, 10 );
    serialSizer->Add( helpsizer, 0,  wxEXPAND | wxTOP, 10 );
    serialSizer->Add( csizer3, 0,  wxEXPAND | wxTOP, 10 );

    m_serialPanel->SetSizer( serialSizer );
    m_serialPanel->Enable( isSerialMode );
    m_serial->SetValue( isSerialMode );

    /// TCP Panel sizers
    wxBoxSizer* tcpSizer = new wxBoxSizer( wxVERTICAL );

    tcpSizer->Add( csizer4, 0, wxEXPAND | wxTOP, 10 );
    tcpSizer->Add( csizer5, 0, wxEXPAND | wxTOP, 10 );

    m_tcpPanel->SetSizer( tcpSizer );
    m_tcpPanel->Enable(!isSerialMode);
    m_tcp->SetValue( !isSerialMode );

    /// Communication Panel sizers
    sizer1->Add( m_serial, 0, wxLEFT | wxRIGHT, 5 );
    sizer1->Add( m_serialPanel, 0, wxEXPAND);

    sizer1->Add( m_tcp, 0, wxLEFT, 5 );
    sizer1->Add( m_tcpPanel, 0, wxEXPAND);

    sizer1->Add( csizer6, 0, wxEXPAND | wxTOP, 10 );

    m_CommunicationPanel->SetSizer( sizer1 );

    wxBoxSizer* savesizer = new wxBoxSizer( wxHORIZONTAL );
    m_SaveButton = new wxButton( this, IDC_SAVEBUTTON, wxT("Save") );
    wxButton* closeBtn = new wxButton( this, IDC_CLOSEBUTTON, wxT("Close") );
    savesizer->Add( m_SaveButton, 0, wxEXPAND | wxALL | wxALIGN_CENTRE, 5 );
    savesizer->Add( closeBtn, 0, wxEXPAND | wxALL | wxALIGN_CENTRE, 5 );

    mainSizer->Add( m_CommunicationPanel,1, wxALL | wxEXPAND , 8 );
    mainSizer->Add( savesizer, 0, wxALL | wxALIGN_CENTER, 2 );

	this->SetSizer( mainSizer );
	mainSizer->Fit( this );
	mainSizer->SetSizeHints( this );

    CenterOnScreen();


    // add control event handlers
    m_CommunicationPanel->Connect( IDC_SERIAL, wxEVT_COMMAND_RADIOBUTTON_SELECTED,
        wxCommandEventHandler(ECUConfigPanel::OnSelectCommMode), 0, this );

    m_CommunicationPanel->Connect( IDC_TCP, wxEVT_COMMAND_RADIOBUTTON_SELECTED,
        wxCommandEventHandler(ECUConfigPanel::OnSelectCommMode), 0, this );

    m_CommunicationPanel->Connect( IDC_COMPORT, wxEVT_COMMAND_COMBOBOX_SELECTED,
        wxCommandEventHandler(ECUConfigPanel::OnCOMChange), 0, this );

    m_CommunicationPanel->Connect( IDC_COMPORT, wxEVT_COMMAND_TEXT_ENTER,
        wxCommandEventHandler(ECUConfigPanel::OnCOMChange), 0, this );

    m_CommunicationPanel->Connect( IDC_BAUDRATE, wxEVT_COMMAND_COMBOBOX_SELECTED,
        wxCommandEventHandler(ECUConfigPanel::OnChangeBaudrate), 0, this );

    m_CommunicationPanel->Connect( IDC_BAUDRATE, wxEVT_COMMAND_TEXT_ENTER,
        wxCommandEventHandler(ECUConfigPanel::OnChangeBaudrate), 0, this );

    Connect( IDC_SAVEBUTTON, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ECUConfigPanel::OnButton), 0, this );
    Connect( IDC_CLOSEBUTTON, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ECUConfigPanel::OnButton), 0, this );
    Connect( IDC_DETECTBUTTON, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ECUConfigPanel::OnButton), 0, this );
    Connect( IDC_DEVMGRBUTTON, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ECUConfigPanel::OnButton), 0, this );

    Connect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler(ECUConfigPanel::OnClose), 0, this );

    connectKeyDownEvent(this);
}

void ECUConfigPanel::saveValues()
{
    // serial port and timeout, sdcard
//    ConfigBase* mainCfg  = ConfigHelper2::getConfig();
//
//    mainCfg->write( _T("/port"), m_comport->GetValue() );
//    mainCfg->write( _T("/preferredBaudrate"), m_preferredBaudrateComboBox->GetValue() );
//    mainCfg->write( _T("/firmwareUploadBaudrate"), m_fwuploadBrComboBox->GetValue() );
//    mainCfg->write( _T("/timeout"), m_com_timeout->GetValue() );
//    mainCfg->write( _T("/rterrorcount"), m_rt_error_count->GetValue() );
//    mainCfg->write( _T("/interWriteDelay"), m_rtInterWriteDelay->GetValue() );
//    mainCfg->write( _T("/sdcard/baudrate"), m_sdcard_b->GetValue() );

    CommunicationConfig commConfig;

    if ( m_serial->GetValue() ) {
        commConfig.mode = _T("SERIAL");
    } else {
        commConfig.mode = _T("TCP");
    }

    commConfig.port = m_comport->GetValue();
    m_preferredBaudrateComboBox->GetValue().ToLong( &commConfig.preferredBaudrate );
    m_fwuploadBrComboBox->GetValue().ToLong( &commConfig.fwUploadBaudrate );
    m_com_timeout->GetValue().ToLong( &commConfig.timeout );
    m_rt_error_count->GetValue().ToLong( &commConfig.rtErrorCount );
    m_rtInterWriteDelay->GetValue().ToLong( &commConfig.interWriteDelay );
    m_sdcard_b->GetValue().ToLong( &commConfig.sdcardBaudrate );

    commConfig.ip = m_ipport->GetValue();
    m_tcp_timeout->GetValue().ToLong( &commConfig.tcpTimeout );

    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrCommunication cfgMgr(cfg);
    cfgMgr.save(&commConfig);




    //read
    // TODO:  save TCP IP, port, and time-out
    // ...


//    if ( mainCfg->save() )
//    {
//        // wxMessageBox( wxT("Settings saved"), wxT("Information"), wxICON_INFORMATION);
//        InfoWindow::Display( NULL, _T("Information"), _T("Settings have been saved"),
//            IW_FADE_IN | IW_MOVE | IW_MESSAGE );
//    }
//    else
//    {
//        InfoWindow::Display( NULL, _T("Error"), _T("Could not save settings"),
//            IW_FADE_IN | IW_MOVE | IW_ERROR );
//        // wxMessageBox( wxT("Could not save settings"), wxT("Error"), wxICON_EXCLAMATION );
//    }
    this->Close();
    this->Destroy();
}

void ECUConfigPanel::OnClose( wxCloseEvent& event )
{
    // disconnect event handlers
    this->Disconnect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( ECUConfigPanel::OnClose ), 0, this );

    if ( m_restartControllerAfterClose ) {

        m_controller->setECUOnline(true);
        m_controller->setGPSOnline(true);
    }

    event.Skip();
}

void ECUConfigPanel::OnButton(wxCommandEvent& event)
{
    switch(event.GetId()){
        case IDC_DETECTBUTTON:
            Detect();
            break;

        case IDC_SAVEBUTTON:
            m_restartControllerAfterClose = true;

            // stop threads
            m_controller->setECUOnline( false, true );
            m_controller->setGPSOnline( false );

            saveValues();
            break;

        case IDC_CLOSEBUTTON:
            Close();
            break;

        case IDC_DEVMGRBUTTON:
            wxSystemHelper::LaunchDeviceManager();
            break;
    }
}

void ECUConfigPanel::Detect()
{
    m_detecting = true;
    // disable controls while searching
    enablePanels(false);
    m_progressbar->Update(0);

    // looking for available com ports
    wxArrayString ports = m_comport->GetStrings();

    if ( ports.IsEmpty() )
    {
        wxMessageBox( wxT("No available COM port"), wxT("Error"), wxOK | wxICON_EXCLAMATION );
        return;
    }

    IntArray bauds = SerialPortHelper::getSerialBaudratesInt();

    m_restartControllerAfterClose = true;

    // stop threads
    m_controller->setECUOnline( false, true );
    m_controller->setGPSOnline( false );

    CommunicationConfig commConfig;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrCommunication cfgMgr(cfg);
    cfgMgr.load(&commConfig);

    m_portScanner = new SerialPortScanner( this, ports, bauds, commConfig.timeout, commConfig.interWriteDelay );
}

void ECUConfigPanel::OnDeviceFound( SerialPortScannerEvent& event )
{
    delete m_portScanner;
    m_portScanner = NULL;

    ECUInfo::Device device = event.getDevice();
    wxString port = event.getPortName();

    wxString devStr;

    if ( device == ECUInfo::DEVICE_ROUND ) devStr = _T("round");
    if ( device == ECUInfo::DEVICE_V3 ) devStr = _T("v3");
    wxMessageBox( wxT("ECU (") + devStr + _T(") connected on ") + port, wxT("ECU found"), wxOK | wxICON_INFORMATION);

    enablePanels();
    m_comport->SetValue(port);
    saveValues();
}

void ECUConfigPanel::OnDeviceNotFound( SerialPortScannerEvent& event )
{
    LOG

    if ( !m_detecting ) {

        if ( event.isScanningFinished() ) {
            wxMessageBox( wxT("ECU not found on ") + event.getPortName(),  wxT("Error"), wxOK | wxICON_HAND);
            enablePanels();
            SetFocus();
        }

        return;
    }

    float max = event.getNumPorts() * event.getNumBauds();
    float curr = event.getCurrPortIdx() * event.getNumBauds() + event.getCurrBaudIdx() + 1;
    float percent = curr / max;

    m_progressbar->Update( percent * 100.0 );

    if ( event.isScanningFinished() ) {

        delete m_portScanner;
        m_portScanner = NULL;

        m_txtDetect->SetLabel( wxT("ECU not found") );

        enablePanels();
        wxMessageBox( wxT("Could not find ECU"),  wxT("Information"), wxOK | wxICON_HAND);
        SetFocus();

    } else {
        m_txtDetect->SetLabel(_T("Trying to connect on ") + event.getNextPortName() );
    }
}

void ECUConfigPanel::OnCannotOpenPort( SerialPortScannerEvent& event )
{
    LOG

    if ( !m_detecting ) {
        wxMessageBox( wxT("Could not open port: ") + event.getPortName(),  wxT("Error"), wxOK | wxICON_HAND);
        enablePanels();
        SetFocus();
        return;
    }

    float max = event.getNumPorts();
    float curr = event.getCurrPortIdx() + 1;
    float percent = curr / max;

    m_progressbar->Update( percent * 100.0 );

    if ( event.isScanningFinished() ) {

        delete m_portScanner;
        m_portScanner = NULL;

        m_txtDetect->SetLabel( wxT("ECU not found") );

        enablePanels();
        wxMessageBox( wxT("Could not find ECU"),  wxT("Information"), wxOK | wxICON_HAND);
        SetFocus();

    } else {
        m_txtDetect->SetLabel(_T("Trying to connect on ") + event.getNextPortName() );
    }
}

void ECUConfigPanel::OnCOMChange( wxCommandEvent& event )
{
    // disable controls
    enablePanels(false);

    m_restartControllerAfterClose = true;

    // if ECU online -> stop
    m_controller->setECUOnline( false, true );
    m_controller->setGPSOnline( false );

    // Update serial port
    wxString ComPort = m_comport->GetValue();

    wxArrayString ports;
    ports.Add( ComPort );

    IntArray bauds = SerialPortHelper::getSerialBaudratesInt();

    m_detecting = false;

    CommunicationConfig commConfig;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrCommunication cfgMgr(cfg);
    cfgMgr.load(&commConfig);

    delete m_portScanner;
    m_portScanner = new SerialPortScanner( this, ports, bauds, commConfig.timeout, commConfig.interWriteDelay );

}


void ECUConfigPanel::OnChangeBaudrate(wxCommandEvent& event)
{

}

void ECUConfigPanel::OnSelectCommMode(wxCommandEvent& event)
{
    switch ( event.GetId() )
    {
        case IDC_SERIAL:
            m_serialPanel->Enable(true);
            m_tcpPanel->Enable(false);
            break;

        case IDC_TCP:
            m_serialPanel->Enable(false);
            m_tcpPanel->Enable(true);
            break;
    }
}



void ECUConfigPanel::enablePanels(bool enabled)
{
    m_CommunicationPanel->Enable( enabled );
    m_SaveButton->Enable( enabled );
}
///---------------------------------------------------------------------------------------

void ECUConfigPanel::OnKeyDown( wxKeyEvent& event )
{
    if ( event.GetKeyCode() == WXK_F1 ) {
        HelpUtil::showHelp( _T("ECU Communication Dialog") );
    } else {
        event.Skip();
    }
}

void ECUConfigPanel::connectKeyDownEvent( wxWindow* window )
{
    if( window )
    {
        window->Connect( wxID_ANY,
                         wxEVT_KEY_DOWN,
                         wxKeyEventHandler( ECUConfigPanel::OnKeyDown ),
                         (wxObject*) NULL,
                         this );

        wxWindowListNode* node = window->GetChildren().GetFirst();

        while( node )
        {
            wxWindow* child = node->GetData();
            this->connectKeyDownEvent( child );

            node = node->GetNext();
        }
    }
}
