#include "ECUConfigPanel2.h"
#include "../SerialPortHelper.h"
#include "../configmanager/ConfigHelper2.h"
#include "../configmanager/VemsDeviceControllerConfig.h"
#include "../configmanager/ConfigMgrVemsDeviceController.h"
#include "../logger.h"
#include "../vemsprotocol/VemsDeviceController.h"
#include "../events/VemsDeviceControllerEvent.h"
#include <wx/statline.h>
///TODO popup window
//#include "../customwidgets/InfoWindow.h"

enum { IDC_SERIAL = 3000, IDC_TCP , IDC_COMPORT, IDC_CURRENT_BAUDRATE, IDC_BAUDRATE, IDC_COMTIMEOUT , IDC_RT_ERROR_COUNT, IDC_SDCARD,
    IDC_IPPORT, IDC_TCPTIMEOUT, IDC_DEFPROT, IDC_DEFREADC, IDC_DETECTBUTTON, IDC_SAVEBUTTON };

BEGIN_EVENT_TABLE( ECUConfigPanel, wxFrame )
    //EVT_VEMS_DEVICE_CONTROLLER( wxID_ANY, ECUConfigPanel::OnVemsDeviceControllerEvent )
END_EVENT_TABLE();

ECUConfigPanel::ECUConfigPanel( wxWindow* parent, VemsDeviceController* vemscontroller )
{
    m_deviceController = vemscontroller;
    m_deviceController->stop();

    /// VemsDeviceControllerEvent handlers
    //m_deviceController->Connect( wxID_ANY, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( ECUConfigPanel::OnVemsDeviceControllerEvent ), NULL, this );

    buildGUI( parent );
    m_tryCount = 0;
}

ECUConfigPanel::~ECUConfigPanel()
{

}

void ECUConfigPanel::findAndShow(wxWindow* parent, VemsDeviceController* controller)
{
    wxWindow* win = wxFindWindowByName(_T("ECU Communication"),parent);
    if ( !win )
         new ECUConfigPanel( parent, controller );
    else win->SetFocus();
}

void ECUConfigPanel::buildGUI( wxWindow* parent )
{
    VemsDeviceControllerConfig config;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrVemsDeviceController cfgMgr(cfg);
    cfgMgr.load(&config);

    Create( parent, wxID_ANY, _T("ECU Communication") );
    this->SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    this->ClearBackground();

    wxFlexGridSizer* sizer = new wxFlexGridSizer( 2, 1, 0, 0 );
    sizer->AddGrowableCol( 0 );  sizer->AddGrowableRow( 0 );

    wxString value = wxEmptyString;

    // -----------------------
    // COMMUNICATION PANEL
    // -----------------------
    m_CommunicationPanel = new wxPanel( this );

    wxStaticBox* box = new wxStaticBox( m_CommunicationPanel, wxID_STATIC, _T("Communication") );
    wxFont font = box->GetFont();  font.SetWeight( wxFONTWEIGHT_BOLD ); box->SetFont( font ); box->SetForegroundColour( *wxBLACK );

    wxStaticBoxSizer * sizer1 = new wxStaticBoxSizer( box, wxVERTICAL );
    wxBoxSizer* csizer1 = new wxBoxSizer( wxHORIZONTAL ); wxBoxSizer* csizer2 = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* csizer3 = new wxBoxSizer( wxHORIZONTAL ); wxBoxSizer* csizer4 = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* csizer5 = new wxBoxSizer( wxHORIZONTAL ); wxBoxSizer* csizer6 = new wxBoxSizer( wxVERTICAL );

    wxBoxSizer* m_rtErrorSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* interWriteDelaySizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* currentBaudrateSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* preferredBaudrateSizer = new wxBoxSizer( wxHORIZONTAL );

    m_ctext1 = new wxStaticText( m_CommunicationPanel, -1, _T("Port:") );

    m_currentBaudrateLabel = new wxStaticText( m_CommunicationPanel, -1, _T("Baudrate:") );

    m_preferredBaudrateLabel = new wxStaticText( m_CommunicationPanel, -1, _T("Preferred baudrate:") );

    m_ctext2 = new wxStaticText( m_CommunicationPanel, -1, _T("Timeout(ms):") );

    m_rtErrorText = new wxStaticText( m_CommunicationPanel, -1, _T("Ignored realtime error count:") );

    m_interWriteDelayText = new wxStaticText( m_CommunicationPanel, -1, _T("Inter Write Delay(ms):") );

    m_ctext3 = new wxStaticText( m_CommunicationPanel, -1, _T("SD Card baud rate:") ); m_ctext3->SetFont( font );
    m_serial = new wxRadioButton( m_CommunicationPanel, IDC_SERIAL, _T("Serial:") );
    m_com_timeout = new wxTextCtrl( m_CommunicationPanel, IDC_COMTIMEOUT, wxEmptyString );
    m_rt_error_count = new wxTextCtrl( m_CommunicationPanel, IDC_RT_ERROR_COUNT, wxEmptyString );
    m_rtInterWriteDelay = new wxTextCtrl( m_CommunicationPanel, wxID_ANY, wxEmptyString );

    wxArrayString ports  = SerialPortHelper::GetCOMPortList();
    wxArrayString sdspeed; sdspeed.Add( wxT("57600") ); sdspeed.Add( wxT("115200") );

    value = config.port;
    // Serial ports
    //mainCfg->read( _T("/port"), &value );

    // add config value to the drop down list if not found
    if ( ports.Index( value, false ) == wxNOT_FOUND )
    {
        ports.Add( value );
    }

    /// ONLY FOR TEST
    //ports.Insert(_T("COM1"),0);
    //ports.Insert(_T("COM2"),1);  ports.Add(_T("COM6")); ports.Add(_T("COM7"));
    //ports.Add(_T("COM12"));
    //ports.Remove(L"COM3");
    //ports.Remove(L"COM5");
    /// ----

    ports.Sort();

    m_comport = new wxComboBox( m_CommunicationPanel, IDC_COMPORT, value,
        wxDefaultPosition, wxDefaultSize, ports, wxTE_PROCESS_ENTER );

//    wxArrayString baudrates;
//    baudrates.Add( _T("9600") );
//    baudrates.Add( _T("19200") );

    value = wxString::Format( _T("%d"), config.baudrate );

    m_currentBaudrateTextCtrl = new wxTextCtrl( m_CommunicationPanel, wxID_ANY, value );
    m_currentBaudrateTextCtrl->Enable( false );

    value = wxString::Format( _T("%d"), config.preferredBaudrate );

    m_preferredBaudrateComboBox = new wxComboBox( m_CommunicationPanel, IDC_BAUDRATE, value,
        wxDefaultPosition, wxDefaultSize, SerialPortHelper::getSerialBaudrates(), wxTE_PROCESS_ENTER | wxCB_READONLY );

    value = wxString::Format( _T("%d"), config.timeout );
    m_com_timeout->SetValue( value );

    value = wxString::Format( _T("%d"), config.rtErrorCount );
    m_rt_error_count->SetValue( value );

    value = wxString::Format( _T("%d"), config.interWriteDelay );
    m_rtInterWriteDelay->SetValue( value );

    value = wxString::Format( _T("%d"), config.sdcardBaudrate );
    m_sdcard_b = new wxComboBox( m_CommunicationPanel, IDC_SDCARD, value,
        wxDefaultPosition, wxDefaultSize, sdspeed );

    // Add controls to sizers
    csizer1->Add( m_serial, 0, wxLEFT | wxRIGHT, 5 ); csizer1->Add( m_ctext1, 0, wxRIGHT, 5 );
    csizer1->Add( m_comport, 1,  wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    currentBaudrateSizer->Add( 60,10,0 ); currentBaudrateSizer->Add( m_currentBaudrateLabel, 0, wxRIGHT , 5 );
    currentBaudrateSizer->Add( m_currentBaudrateTextCtrl, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    preferredBaudrateSizer->Add( 60,10,0 ); preferredBaudrateSizer->Add( m_preferredBaudrateLabel, 0, wxRIGHT , 5 );
    preferredBaudrateSizer->Add( m_preferredBaudrateComboBox, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    csizer2->Add( 60,10,0 ); csizer2->Add( m_ctext2, 0, wxRIGHT , 5 );
    csizer2->Add( m_com_timeout, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    m_rtErrorSizer->Add( 60,10,0 ); m_rtErrorSizer->Add( m_rtErrorText, 0, wxRIGHT , 5 );
    m_rtErrorSizer->Add( m_rt_error_count, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    interWriteDelaySizer->Add( 60,10,0 );
    interWriteDelaySizer->Add( m_interWriteDelayText, 0, wxRIGHT , 5 );
    interWriteDelaySizer->Add( m_rtInterWriteDelay, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 );

    csizer3->Add( m_ctext3, 0, wxLEFT | wxRIGHT, 10 ); csizer3->Add( m_sdcard_b, 1, wxEXPAND | wxALIGN_RIGHT | wxRIGHT, 5 );

    // TCP
    m_tcp = new wxRadioButton( m_CommunicationPanel, IDC_TCP, _T("TCP:") );
    m_ctext4 = new wxStaticText( m_CommunicationPanel, -1, _T("IP:port") );
    m_ctext5 = new wxStaticText( m_CommunicationPanel, -1, _T("Timeout") );

    value = config.ip;
    m_ip = new wxTextCtrl( m_CommunicationPanel, IDC_IPPORT, value );

    value = wxString::Format( _T("%d"), config.tcpTimeout );
    m_tcp_timeout =  new wxTextCtrl( m_CommunicationPanel, IDC_TCPTIMEOUT, value );

    csizer4->Add( m_tcp, 0, wxLEFT, 5 ); csizer4->Add( m_ctext4, 0, wxLEFT | wxRIGHT, 5 );
    csizer4->Add( m_ip, 1, wxRIGHT | wxEXPAND | wxALIGN_RIGHT, 5 ); csizer5->Add( 52, 10, 0 );
    csizer5->Add(m_ctext5, 0, wxRIGHT, 10 ); csizer5->Add( m_tcp_timeout, 1, wxEXPAND | wxALIGN_RIGHT | wxRIGHT, 5 );

    wxStaticLine * line = new wxStaticLine( m_CommunicationPanel, -1, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );

    size_t portsCount = ports.GetCount();
    size_t baudrateCount = SerialPortHelper::getSerialBaudrates().GetCount();

    m_maxTryCount = portsCount * baudrateCount;

    // Detect button
    m_txtDetect = new wxStaticText( m_CommunicationPanel, wxID_ANY, _T("Click on Detect to detect ECU automatically"));
    m_detectButton = new wxButton( m_CommunicationPanel, IDC_DETECTBUTTON, wxT("Detect") );

    m_progressbar = new ProgressBar( m_CommunicationPanel, wxID_ANY, wxDefaultPosition,wxSize(-1,20) );
    m_progressbar->SetWindowStyle( wxBORDER_SUNKEN );
    m_progressbar->SetProgressStyle(PB_PERCENTTEXT_INNER);
//    m_progressbar->Update(0);

//    m_progressbar->SetValueFormat(_T("%.1f"));
    m_progressbar->SetValueLimits(0, 100, 1.0);



    csizer6->Add( line, 0, wxEXPAND | wxLEFT | wxRIGHT | wxALIGN_CENTER, 10 );
    csizer6->Add( m_txtDetect, 0, wxEXPAND | wxALIGN_CENTER | wxALL, 5);
    csizer6->Add( m_progressbar, 1, wxEXPAND | wxALL | wxALIGN_CENTER, 5);
    csizer6->Add( m_detectButton, 0, wxALL | wxALIGN_CENTER, 5 );

    // Communication Panel sizers
    sizer1->Add( csizer1, 0, wxEXPAND);
    sizer1->Add( currentBaudrateSizer, 0, wxEXPAND | wxTOP, 10 );
    sizer1->Add( preferredBaudrateSizer, 0, wxEXPAND | wxTOP, 10 );
    sizer1->Add( csizer2, 0,  wxEXPAND | wxTOP, 10 );
    sizer1->Add( m_rtErrorSizer, 0,  wxEXPAND | wxTOP, 10 );
    sizer1->Add( interWriteDelaySizer, 0,  wxEXPAND | wxTOP, 10 );


    sizer1->Add( csizer3, 0,  wxEXPAND | wxTOP, 10 ); sizer1->Add( csizer4, 0, wxEXPAND | wxTOP, 10 );
    sizer1->Add( csizer5, 0, wxEXPAND | wxTOP, 10 ); sizer1->Add( csizer6, 0, wxEXPAND | wxTOP, 10 );
    m_CommunicationPanel->SetSizer( sizer1 );

    // -----------------------
    // PROTOCOL PANEL
    // -----------------------
    m_ProtocolPanel = new wxPanel( this );
    wxStaticBox* box2 = new wxStaticBox( m_ProtocolPanel, wxID_STATIC, _T("Default Protocol") );
    box2->SetFont( font ); box2->SetForegroundColour( *wxBLACK );

    wxStaticBoxSizer* sizer2 = new wxStaticBoxSizer( box2, wxVERTICAL );
    wxBoxSizer* psizer1 = new wxBoxSizer( wxHORIZONTAL );wxBoxSizer* psizer11 = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer* psizer2 = new wxBoxSizer( wxHORIZONTAL );wxBoxSizer* psizer21 = new wxBoxSizer( wxHORIZONTAL );
    wxStaticText* ptext1 = new wxStaticText( m_ProtocolPanel, -1, _T("Default protocol") ); ptext1->SetFont( font );
    wxStaticText* ptext2 = new wxStaticText( m_ProtocolPanel, -1, _T("protocol") );
    wxStaticText* ptext3 = new wxStaticText( m_ProtocolPanel, -1, _T("Default read realtime command") ); ptext3->SetFont( font );
    wxStaticText* ptext4 = new wxStaticText( m_ProtocolPanel, -1, _T("read_realtime") );
    wxArrayString pStr1, pStr2;pStr1.Add( _T("MEGATUNE") );
    pStr1.Add( _T("SIPR") ); pStr2.Add( _T("SIPR_RT_MT_NORMAL") ); pStr2.Add( _T("SIPR_RT_MT_EXT") );

    value = config.protocol;
    m_defprot = new wxComboBox( m_ProtocolPanel, IDC_DEFPROT, value, wxDefaultPosition, wxDefaultSize, pStr1 );

    value = config.readRealtime;
    m_defreadc = new wxComboBox( m_ProtocolPanel, IDC_DEFREADC, value, wxDefaultPosition, wxDefaultSize, pStr2);

    psizer1->Add( ptext1, 0, wxLEFT, 15 );
    psizer11->Add( ptext2, 0, wxLEFT , 15 ); psizer11->Add( 10, 0, 0 );
    psizer11->Add( m_defprot, 1, wxEXPAND | wxALIGN_RIGHT | wxRIGHT, 5 );
    psizer2->Add( ptext3, 0, wxLEFT, 15 );
    psizer21->Add( ptext4, 0, wxLEFT, 15 ); psizer21->Add( 10, 0, 0 );
    psizer21->Add( m_defreadc,  1, wxEXPAND | wxALIGN_RIGHT | wxRIGHT, 5 );

    sizer2->Add( psizer1,0, wxEXPAND );
    sizer2->Add( psizer11, 0, wxEXPAND | wxTOP, 10 );
    sizer2->Add( psizer2, 0, wxEXPAND | wxTOP, 10 );
    sizer2->Add( psizer21, 0, wxEXPAND | wxTOP, 10 );
    m_ProtocolPanel->SetSizer( sizer2 );

    wxBoxSizer* savesizer = new wxBoxSizer( wxHORIZONTAL );
    m_SaveButton = new wxButton( this, IDC_SAVEBUTTON, wxT("Save") );
    savesizer->Add( m_SaveButton, 0, wxEXPAND | wxALL | wxALIGN_CENTRE, 5 );

    sizer->Add( m_CommunicationPanel,1, wxALL | wxEXPAND , 8 );
    sizer->Add( m_ProtocolPanel, 1, wxBOTTOM | wxRIGHT | wxLEFT | wxEXPAND , 8 );
    sizer->Add( savesizer, 0, wxALL | wxALIGN_CENTER, 2 );

    ///select mode
    bool serialEnabled = config.mode.IsSameAs( _T("SERIAL") );
    setEnabledSerialSettings( serialEnabled );
    setEnabledTCPSettings( !serialEnabled );

	this->SetSizer( sizer );
	sizer->Fit( this );
	sizer->SetSizeHints( this );

    this->CenterOnScreen();
    this->Show( true );

    // add control event handlers
    m_CommunicationPanel->Connect( IDC_DETECTBUTTON, wxEVT_COMMAND_BUTTON_CLICKED,
        wxCommandEventHandler(ECUConfigPanel::OnDetectButton), 0, this );

    m_CommunicationPanel->Connect( IDC_COMPORT, wxEVT_COMMAND_COMBOBOX_SELECTED,
        wxCommandEventHandler(ECUConfigPanel::OnCOMChange), 0, this );

    m_CommunicationPanel->Connect( IDC_COMPORT, wxEVT_COMMAND_TEXT_ENTER,
        wxCommandEventHandler(ECUConfigPanel::OnCOMChange), 0, this );

    m_CommunicationPanel->Connect( IDC_BAUDRATE, wxEVT_COMMAND_COMBOBOX_SELECTED,
        wxCommandEventHandler(ECUConfigPanel::OnChangeBaudrate), 0, this );

    m_CommunicationPanel->Connect( IDC_BAUDRATE, wxEVT_COMMAND_TEXT_ENTER,
        wxCommandEventHandler(ECUConfigPanel::OnChangeBaudrate), 0, this );

    m_CommunicationPanel->Connect( IDC_SERIAL, wxEVT_COMMAND_RADIOBUTTON_SELECTED,
        wxCommandEventHandler(ECUConfigPanel::OnChangeCommunicationMode), 0, this );

    m_CommunicationPanel->Connect( IDC_TCP, wxEVT_COMMAND_RADIOBUTTON_SELECTED,
        wxCommandEventHandler(ECUConfigPanel::OnChangeCommunicationMode), 0, this );

    this->Connect( IDC_SAVEBUTTON, wxEVT_COMMAND_BUTTON_CLICKED,
        wxCommandEventHandler(ECUConfigPanel::OnSaveButton), 0, this );

    this->Connect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler(ECUConfigPanel::OnClose), 0, this );
}

void ECUConfigPanel::setEnabledSerialSettings( bool enable )
{
    m_serial->SetValue( enable );

    m_ctext1->Enable( enable );
    m_comport->Enable( enable );

    m_preferredBaudrateLabel->Enable( enable );
    m_preferredBaudrateComboBox->Enable( enable );

    m_ctext2->Enable( enable );
    m_com_timeout->Enable( enable );

    m_rtErrorText->Enable( enable );
    m_rt_error_count->Enable( enable );

    m_interWriteDelayText->Enable( enable );
    m_rtInterWriteDelay->Enable( enable );

    m_ctext3->Enable( enable );
    m_sdcard_b->Enable( enable );
}

void ECUConfigPanel::setEnabledTCPSettings( bool enable )
{
    m_tcp->SetValue( enable );

    m_ctext4->Enable( enable );
    m_ctext5->Enable( enable );
    m_ip->Enable( enable );
    m_tcp_timeout->Enable( enable );
}


void ECUConfigPanel::saveConfig()
{
    VemsDeviceControllerConfig config;

    config.mode                 = m_serial->GetValue() ? _T("SERIAL") : _T("TCP");

    config.port                 = m_comport->GetValue();
    config.preferredBaudrate    = wxAtoi( m_preferredBaudrateComboBox->GetValue() );
    config.timeout              = wxAtoi( m_com_timeout->GetValue() );
    config.rtErrorCount         = wxAtoi( m_rt_error_count->GetValue() );
    config.interWriteDelay      = wxAtoi( m_rtInterWriteDelay->GetValue() );

    config.sdcardBaudrate       = wxAtoi( m_sdcard_b->GetValue() );

    config.protocol             = m_defprot->GetValue();
    config.readRealtime         = m_defreadc->GetValue();

    config.ip                   = m_ip->GetValue();
    config.tcpTimeout           = wxAtoi( m_tcp_timeout->GetValue() );

    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrVemsDeviceController cfgMgr(cfg);
    cfgMgr.save(&config);

//    //
//    // TODO:  save TCP IP, port, and time-out
//    // ...
//
//    // Protocol
//    mainCfg->write( _T("/devices/protocol"), m_defprot->GetValue() );
//    mainCfg->write( _T("/devices/read_realtime"), m_defreadc->GetValue() );
//
////    if ( mainCfg->save() )
////    {
////        // wxMessageBox( wxT("Settings saved"), wxT("Information"), wxICON_INFORMATION);
////        InfoWindow::Display( NULL, _T("Information"), _T("Settings have been saved"),
////            IW_FADE_IN | IW_MOVE | IW_MESSAGE );
////    }
////    else
////    {
////        InfoWindow::Display( NULL, _T("Error"), _T("Could not save settings"),
////            IW_FADE_IN | IW_MOVE | IW_ERROR );
////        // wxMessageBox( wxT("Could not save settings"), wxT("Error"), wxICON_EXCLAMATION );
////    }

}

void ECUConfigPanel::OnClose( wxCloseEvent& event )
{
    this->Disconnect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( ECUConfigPanel::OnClose ), 0, this );

    //m_deviceController->registerHandler( GetParent() );
    //m_deviceController->resume();
    VemsDeviceControllerConfig config;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrVemsDeviceController cfgMgr(cfg);
    cfgMgr.load(&config);

    m_deviceController->start(config);

    event.Skip();
}

void ECUConfigPanel::OnDetectButton( wxCommandEvent& event )
{
    wxArrayString ports = m_comport->GetStrings();

    if ( ports.IsEmpty() ) {
        wxMessageBox( wxT("No available COM port"), wxT("Error"), wxOK | wxICON_EXCLAMATION );
        return;
    } else {
//        m_portIdx = 0;
//        m_numPorts = ports.GetCount();
        m_tryCount = 0;

        enablePanels(false);

        m_txtDetect->SetLabel( _T("Detecting...") );

//        m_deviceController->addEventHandler( this );
        m_deviceController->scanPorts( ports );
    }
}

void ECUConfigPanel::OnVemsDeviceControllerEvent( VemsDeviceControllerEvent& event )
{
    LOG
    switch ( event.GetId() )
    {
        case VDC_EVENT_DEVICE_FOUND:
        {
            ECUInfo::Device device = event.getDevice();
            wxString devStr = ECUInfo::getDeviceAsString( device );
            devStr.LowerCase();

            wxString title = _T("ECU found");
            wxString message =  wxT("ECU (") + devStr + _T(") connected on ") + event.getPortName();

            m_txtDetect->SetLabel( message );
            LOGP( wxString::Format( _T("m_maxTryCount: %d"), m_maxTryCount ) );
            m_progressbar->Update( 100.0, false );

            wxMessageBox( message, title, wxOK | wxICON_INFORMATION );
            enablePanels();
            //saveValues();
        } break;

        case VDC_EVENT_DEVICE_NOT_FOUND:
        {
            ++m_tryCount;
            m_progressbar->Update( m_tryCount );
            wxString label = _T("Trying to connect on ") + event.getPortName() + _T(" at ");
            label += wxString::Format( _T("%d"), event.getBaudrate() );
            m_txtDetect->SetLabel( label );

            if ( event.isScanningFinished() ) {
                m_progressbar->Update( 100.0, false );

                enablePanels();
                wxMessageBox( wxT("Could not find ECU"),  wxT("Information"), wxOK | wxICON_HAND);
                m_txtDetect->SetLabel( wxT("ECU not found") );
                SetFocus();
            }
        } break;

        case VDC_EVENT_CANNOT_OPEN_PORT:
        {
            size_t baudrateCount = SerialPortHelper::getSerialBaudrates().GetCount();

            m_tryCount += baudrateCount;
            m_progressbar->Update( m_tryCount );
            wxString label = _T("Trying to connect on ") + event.getPortName() + _T(" at ");
            label += wxString::Format( _T("%d"), event.getBaudrate() );
            m_txtDetect->SetLabel( label );

            if ( event.isScanningFinished() ) {
                m_progressbar->Update( 100.0, false );

                enablePanels();
                wxMessageBox( wxT("Could not find ECU"),  wxT("Information"), wxOK | wxICON_HAND);
                m_txtDetect->SetLabel( wxT("ECU not found") );
                SetFocus();
            }
        } break;
    }
}


void ECUConfigPanel::OnSaveButton( wxCommandEvent& event )
{
    saveConfig();

    Close();
    Destroy();
}

void ECUConfigPanel::OnCOMChange( wxCommandEvent& event )
{
//    // disable controls
//    enablePanels(false);
//
//    m_connecting = true; m_detecting = false;
//
//    // if ECU online -> stop
//    m_deviceController->setECUOnline( false, true );
//
//
//    // Update serial port
//    wxString ComPort = m_comport->GetValue();
//
//    // get ECUStateMgr
//    ECUStateMgrBase* mgr = m_deviceController->getEcuStateMgr();
//    CommunicationBase* combase = mgr->getCommunicationBase();
//
//    SerialHelper::setECUPort( ComPort);
//    combase = SerialHelper::getECUComm();
//
//    try
//    {
//        mgr->getState();
//        int device = mgr->getDevice();
//        wxString devStr;
//        switch ( device )
//        {
//            case ECUInfo::DEVICE_V3:
//            case ECUInfo::DEVICE_ROUND:
//                if ( device == ECUInfo::DEVICE_ROUND ) devStr = _T("round");
//                if ( device == ECUInfo::DEVICE_V3 ) devStr = _T("v3");
//                wxMessageBox( wxT("ECU (") + devStr + _T(") connected on ") + ComPort, wxT("ECU found"), wxOK | wxICON_INFORMATION);
//                enablePanels();
//                m_connecting = false;
//                saveValues();
//
//                //Try to restart ECU threads
//                m_deviceController->startThreads();
//                m_deviceController->setECUOnline(true);
//
//                break;
//            default:
//                wxMessageBox( wxT("ECU not found on ") + ComPort,  wxT("Error"), wxOK | wxICON_HAND);
//                enablePanels();
//                m_connecting = false;
//                break;
//        }
//    }
//    catch (SerialException& e)
//    {
//        wxMessageBox( wxT("Could not open port: ") + ComPort,  wxT("Error"), wxOK | wxICON_HAND);
//        enablePanels();
//    }
//    SetFocus();
}

void ECUConfigPanel::OnChangeBaudrate(wxCommandEvent& event)
{

}

void ECUConfigPanel::OnChangeCommunicationMode( wxCommandEvent& event )
{
    bool serialEnabled = ( event.GetId() == IDC_SERIAL );
    setEnabledSerialSettings( serialEnabled );
    setEnabledTCPSettings( !serialEnabled );
}

void ECUConfigPanel::enablePanels(bool enabled)
{
    m_CommunicationPanel->Enable( enabled );
    m_ProtocolPanel->Enable( enabled );
    m_SaveButton->Enable( enabled );
}

