#ifndef FILEOPENDLG_H
#define FILEOPENDLG_H

#include "FileOpenSQL.h"

#include <wx/bmpcbox.h>
#include <wx/filename.h>
#include <wx/listctrl.h>
#include <wx/progdlg.h>

#include "../DirHelper.h"
#include "../ObserverBase.h"
#include "../logger.h"

#include "OwnBitmapComboBox.h"
#include <wx/dnd.h>

#ifdef WIN32
#include <wx/volume.h>


class VolumeInfo {
    public:
        wxString        Name;
        wxFSVolumeKind  Kind;
        wxString        DisplayName;
        wxBitmap        Icon;
};

WX_DEFINE_ARRAY( VolumeInfo*, VolumeInfoA );

#endif

struct columnOrder {
    uint8_t column;
    bool    ascending;
};

class QuickAnalyze;

class FileOpenDlg : public wxDialog, public ObserverBase
{
    public:
        FileOpenDlg( wxWindow* parent, const wxString& caption = _T("Open VEMS related files"),
                    wxString defaultDir = wxEmptyString, int defaultFiles = VF_VEMS_ALL );
        ~FileOpenDlg();

        wxString        GetFileName( bool fullPath = true );

//        bool    AcceptDrop();
//        bool    OnDropFiles( wxCoord x, wxCoord y, const wxArrayString& filenames );
        bool onDropFiles( const wxArrayString& filenames );


    private:
        enum { FDLG_TIMERID = 9999 };

        ColumnInfoA         m_columnInfoA;

        uint8_t             m_numcolumns;
        float*              m_widthf;

        wxFileName          m_defaultDir;
        int                 m_defaultFiles;

        OwnBitmapComboBox*  m_dirList;

        wxListCtrl*         m_fileList;
        wxToolBar*          m_fo_tb;

        wxTextCtrl*         m_fileName;
        wxComboBox*         m_fileFilter;

        wxDialog*           m_progressDlg;

        columnOrder         m_currColOrder;

        #ifdef WIN32
        VolumeInfoA         m_drives;
        #endif

        wxArrayString       m_comboRawDirs;

        FileInfoA           m_analyzeInfo;
        QuickAnalyze*       m_analyzer;
        wxProgressDialog*   m_progressDlg2;
        size_t              m_aIndex;

        wxMenu*             m_popupMenu;
        bool                m_configLoaded;

        wxStaticBitmap*     m_gpspreview;

        wxSizer*            m_listsizer;
        wxPanel*            m_gpsroutepanel;

        bool                m_canClose;
        wxTimer             m_timer;

    private:
        void buildGUI( wxWindow* parent, const wxString& caption );
        void initListCtrl();
        void fillListWithFiles( uint8_t colOrder = 0, bool ascending = true );

        void AddLine( FileInfo* finfo );
        void AddDir( const wxString& dirname );
        void updateDirCombo();

        void OnDirComboBox( wxCommandEvent& event );
        void OnFilterCombo( wxCommandEvent& event );
        void OnSize( wxSizeEvent& event );
        void OnListHeaderClick( wxListEvent& event );
        void OnSelectListItem( wxListEvent& event );
        void OnDoubleClickList( wxListEvent& event );

        void OnGoParentFolder( wxCommandEvent& event );
        void OnAnalyzeFiles( wxCommandEvent& event );
        void OnSearchFiles( wxCommandEvent& event );
        void OnExportGPS( wxCommandEvent& event );
        void OnExportGPSMenu( wxCommandEvent& event );
        void OnShowGPSRoute( wxCommandEvent& event );

        void OnOpenSepButton( wxCommandEvent& event );
        void OnOpenButton( wxCommandEvent& event );
        void OnCancelButton( wxCommandEvent& event );
        void OnClose( wxCloseEvent& event );

        void SelectColumns( wxListEvent& event );
        void OnDrag( wxListEvent& event );

        // helpers
        void calcColumnWidth( float w, int* cw );

        void enumDrives();
        void rebuildList( FileInfoA* fileList = NULL );
        void showSearchResult( FileInfoA* filelist );

        // observerBase - parsing directory
        void notify( int i );
        void notify( int i, int idx ) {}
        void notify( const wxString& str );
        void Init( const wxString& name, const wxString& msg, const int max ) {}

        void OnTimer( wxTimerEvent& event );

        DECLARE_EVENT_TABLE()
};

#endif // FILEOPENDLG_H
