#ifndef FIRMWAREDIRECTOR_H
#define FIRMWAREDIRECTOR_H


#include "wx/progdlg.h"
#include "../common/types/StringSet.h"


class VemsDeviceControllerBase;
class FirmwareVerifyController;
class InputEvent;
class FirmwareVerifyEvent;
class FirmwareUploadEvent;
class CommunicationEvent;
class FirmwareConfigItemArray;
class FirmwareData;
class Project;


class FirmwareDirector : public wxEvtHandler
{
    public:
        FirmwareDirector( wxWindow* parent, VemsDeviceControllerBase* deviceController );
        virtual ~FirmwareDirector();

        void verifyFirmware( Project* proj = NULL );
        void uploadFirmware( Project* proj = NULL );

    private:
        VemsDeviceControllerBase*   m_deviceController;
        FirmwareVerifyController*   m_firmwareVerifyController;
        wxWindow*                   m_parent;
        wxProgressDialog*           m_progressDialog;
        StringSet                   m_filepatches;

        wxString getHexFile( FirmwareConfigItemArray* configItemA );
        bool isEnabled() const;
        void showErrorMessage() const;

        void startVerifyFirmware( FirmwareData* firmwareData );
        void startUploadFirmware( FirmwareData* firmwareData );

        void connectCommunicationEventHandler();
        void disconnectCommunicationEventHandler();

        void connectFirmwareVerifyEventHandlers();
        void disconnectFirmwareVerifyEventHandlers();

        void connectFirmwareUploadEventHandlers();
        void disconnectFirmwareUploadEventHandlers();

        void createProgressBar( int processType, int max );

        void cleanupVerifyFirmware();
        void cleanupUploadFirmware();

        // CommunicationEvent handler
        void onCommunicationEvent( CommunicationEvent& event );

        // FirmwareVerifyEvent handlers
        void onFirmwarePageVerified( FirmwareVerifyEvent& event );
        void onFirmwareVerifingSucceeded( FirmwareVerifyEvent& event );
        void onFirmwareVerifingFailed( FirmwareVerifyEvent& event );

        // FirmwareUploadEvent handlers
        void onFirmwareUploadingStarted( FirmwareUploadEvent& event );
        void onFirmwarePagesErasing( FirmwareUploadEvent& event );
        void onFirmwarePagesErasingSucceeded( FirmwareUploadEvent& event );
        void onFirmwareUploadingPagesStarted( FirmwareUploadEvent& event );
        void onFirmwarePageUploaded( FirmwareUploadEvent& event );
        void onFirmwareUploadingSucceeded( FirmwareUploadEvent& event );
        void onFirmwareUploadingFailed( FirmwareUploadEvent& event );

        int addPatchFiles( FirmwareConfigItemArray* configItemA, FirmwareData*  fwData );
        wxArrayString* getPatchFiles(FirmwareConfigItemArray* configItemA);
};

#endif // FIRMWAREDIRECTOR_H
