#include "SerialConfigPanel.h"

#include "../serialconfig.h"
#include "../serialhelper.h"
#include "../configmanager/ConfigHelper2.h"
#include "../configmanager/SerialConfigMgr.h"

#include "../logger.h"

BEGIN_EVENT_TABLE( SerialConfigPanel, wxPanel )
    EVT_BUTTON( wxID_ANY, SerialConfigPanel::OnButton )
END_EVENT_TABLE()

SerialConfigPanel::SerialConfigPanel( wxWindow* parent ) : wxPanel( parent )
{
    buildGUI();
    initData();
}

SerialConfigPanel::~SerialConfigPanel()
{
    //dtor
    Logger::logf(LOG_MSG, _T("SerialConfigPanel dtor"));
}

void SerialConfigPanel::buildGUI()
{
    const wxString strECUBoxLabel   = _T("ECU");
    const wxString strECUPortLabel  = _T("Port");
/*
    const wxString strGPSBoxLabel   = _T("GPS");
    const wxString strGPSPortLabel  = _T("Port");
    const wxString strGPSSpeedLabel = _T("Speed");
*/
    const wxString strOKButtonLabel     = _T("Ok");
    const wxString strCancelButtonLabel = _T("Cancel");

    wxStaticBoxSizer* sizerECU  = new wxStaticBoxSizer( wxHORIZONTAL, this, strECUBoxLabel );
    wxStaticText* labelECUPort  = new wxStaticText( this, wxID_ANY,strECUPortLabel );
    m_textECUPort = new wxTextCtrl( this, wxID_ANY );
    sizerECU->Add( labelECUPort, 1, wxRIGHT, 5 );
    sizerECU->Add( m_textECUPort, 2 );
/*
    wxStaticBoxSizer* sizerGPS  = new wxStaticBoxSizer( wxVERTICAL, this, strGPSBoxLabel );
    wxStaticText* labelGPSPort  = new wxStaticText( this, wxID_ANY, strGPSPortLabel );
    m_textGPSPort = new wxTextCtrl( this, wxID_ANY );
    wxBoxSizer* sizerGPSPort    = new wxBoxSizer( wxHORIZONTAL );
    sizerGPSPort->Add( labelGPSPort, 1, wxRIGHT, 5  );
    sizerGPSPort->Add( m_textGPSPort, 2 );

    wxStaticText* labelGPSSpeed = new wxStaticText( this, wxID_ANY, strGPSSpeedLabel );
    m_textGPSSpeed = new wxTextCtrl( this, wxID_ANY );
    wxBoxSizer* sizerGPSSpeed   = new wxBoxSizer( wxHORIZONTAL );
    sizerGPSSpeed->Add( labelGPSSpeed, 1, wxRIGHT, 5 );
    sizerGPSSpeed->Add( m_textGPSSpeed, 2);

    sizerGPS->Add( sizerGPSPort, 0, wxEXPAND | wxDOWN, 5 );
    sizerGPS->Add( sizerGPSSpeed, 0, wxEXPAND, 0 );
*/
    wxButton* buttonOK      = new wxButton( this, wxOK, strOKButtonLabel );
    wxButton* buttonCancel  = new wxButton( this, wxCANCEL, strCancelButtonLabel );

    wxBoxSizer*  sizerButtons = new wxBoxSizer( wxHORIZONTAL );

    sizerButtons->Add( buttonOK, 0, wxRIGHT, 5 );
    sizerButtons->Add( buttonCancel );

    wxBoxSizer* sizerMain = new wxBoxSizer( wxVERTICAL );

    sizerMain->Add( sizerECU, 0, wxEXPAND | wxALL, 5 );
//    sizerMain->Add( sizerGPS, 0, wxEXPAND | wxALL, 5 );
    sizerMain->Add( sizerButtons, 0, wxEXPAND | wxALL, 5 );

    SetSizer( sizerMain );

    SetInitialSize( GetBestSize() );

    SerialConfig * sc = SerialHelper::getECUConfig();
    wxString portName = sc->getPortName();

    wxString defValue;

    if( portName.length() < 1 )
    {
        defValue = _T("COM1");
    }
    else
    {
        defValue = portName;
    }
}

void SerialConfigPanel::initData()
{
    SerialConfig * sc = SerialHelper::getECUConfig();
    wxString portName = sc->getPortName();
    if( portName.length() < 1 )
    {
        portName = _T("COM1");
    }

    m_textECUPort->SetValue( portName );

//    SerialConfig* scGPS = ConfigHelper::getGPSSerialConfig();

//    m_textGPSPort->SetValue( scGPS->getPortName() );
//    m_textGPSSpeed->SetValue( wxString::Format( _T("%i"), scGPS->getBaudRate() ) );
}

void SerialConfigPanel::setData()
{
    SerialConfig * sc = SerialHelper::getECUConfig();
    sc->setPortName( m_textECUPort->GetValue() );
    SerialConfigMgr scm(ConfigHelper2::getConfig(), _T("/"));
    scm.save(sc);

//    SerialConfig* scGPS = ConfigHelper::getGPSSerialConfig();
//    scGPS->setPortName( m_textGPSPort->GetValue() );
//    long l = 0;
//    m_textGPSSpeed->GetValue().ToLong( &l );
//    scGPS->setBaudRate( l );
//    ConfigHelper::writeSerialConfigGPS();
}



void SerialConfigPanel::OnButton( wxCommandEvent& event )
{
    Logger::logf( LOG_MSG, _T("SerialConfigPanel::OnButton()") );
    int id = event.GetId();

    if( id == wxOK )
    {
        setData();
    }

   wxDialog* parent = (wxDialog*)GetParent(); //->Close();
   parent->EndModal(id);
}


SerialConfigDialog::SerialConfigDialog( wxWindow* parent )
//: wDialog( parent, wxID_ANY, _T("Set serial") )
{
    Create(parent, wxID_ANY, _T("Serial Port Setup"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("id"));
    SetIcon(wxIcon(_T("vemsLogoIcon2")));
    buildGUI();
}


SerialConfigDialog::~SerialConfigDialog()
{
    //dtor
    Logger::logf(LOG_MSG, _T("SerialConfigDialog dtor"));
}


void SerialConfigDialog::buildGUI()
{
    SerialConfigPanel* panel = new SerialConfigPanel( this );
    SetClientSize( panel->GetSize() );
    SetMinSize( GetSize() );
}
