#include "VemsWelcomeScreen.h"

#include <wx/dcbuffer.h>
#include <wx/filename.h>

#include "../DrawHelper.h"
#include "../agg/wxAgg2.h"

#include "../logger.h"
#include "../DirHelper.h"
#include "../FileHelper.h"

#include "../bcWidgets/bcWrapper.h"
#include "../bcWidgets/bcStaticBitmap.h"
#include "../bcWidgets/bcButton.h"
#include "../bcWidgets/bcLabel.h"
#include "../bcWidgets/bcLinkLabel.h"
#include "../bcWidgets/bcStaticLine.h"

#include "../version.h"

#include "../common/MainFrameGUICommandTypes.h"
#include "../events/VemsGUICommandEvent.h"

#include "../RecentVemsFiles.h"

#include "../StringHelper.h"
#include "../wxHSLColor.h"

#include "../common/FunctionMeter.h"

BEGIN_EVENT_TABLE(VemsWelcomeScreen, wxPanel)

    EVT_PAINT(VemsWelcomeScreen::OnPaint)
    EVT_SIZE(VemsWelcomeScreen::OnSize)

    EVT_BUTTON(idButtonOpenLogFile, VemsWelcomeScreen::OnButton)
    EVT_BUTTON(idButtonOpenConfigFile, VemsWelcomeScreen::OnButton)
    EVT_BUTTON(idButtonAdvOpenLogFile, VemsWelcomeScreen::OnButton)
    EVT_BUTTON(idButtonOpenLogFileExternal, VemsWelcomeScreen::OnButton)
    EVT_BUTTON(idButtonOpenECUComm, VemsWelcomeScreen::OnButton)
    EVT_BUTTON(idButtonOpenGPSComm, VemsWelcomeScreen::OnButton)

END_EVENT_TABLE()

VemsWelcomeScreen::VemsWelcomeScreen(wxWindow * parent, int id)
{
    //ctor
    FUNCTIME
    m_bgBitmap = NULL;
    m_eventHandler = NULL;
    m_headHeight = 0;

    Create(parent,id,wxDefaultPosition,wxDefaultSize, wxBORDER_SUNKEN);

    CreateBcParent(this);

    BuildGUI();

    DrawBackground();
}

VemsWelcomeScreen::~VemsWelcomeScreen()
{
    //dtor
    LOG
}

/***************************
        Building gui
*/

void VemsWelcomeScreen::BuildGUI()
{


    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer* contentSizer = new wxBoxSizer(wxVERTICAL);

// put head image
    wxImage img = DrawHelper::GetIconImg(_T("vemstune-banner-black.png"));

    int headPadding = 5;

    m_headHeight = img.GetHeight() + headPadding;

    bcStaticBitmap* logo = new bcStaticBitmap(this, wxID_ANY, img);
    mainsizer->Add(new bcWrapper(logo),0, wxALL | wxALIGN_CENTER_HORIZONTAL, headPadding );

    wxString ver = AutoVersion::getVersion();
    wxString date = AutoVersion::getDateTime();

// put version info
    bcLabel* title = new bcLabel(this);
    TextDesc desc(_T("VEMS Tuning application, version: ")+ver+_T(" (")+date+_T(")"));
    desc.Height = 12;
    title->SetLabel(desc);
    mainsizer->Add(new bcWrapper(title) ,0, wxBOTTOM | wxALIGN_CENTER_HORIZONTAL, 5 );


    bcLinkLabel* link = new bcLinkLabel(this, bcLinkInfo(_T("http://vems.hu/vt"),_T("Official website: vems.hu"),_T("Open with default browser")));
    mainsizer->Add( new bcWrapper(link), 0, wxALIGN_CENTER_HORIZONTAL );

// put line of buttons


    wxImage logImg = DrawHelper::GetIconImg(_T("logfile-22.png"));
    wxImage cfgImg = DrawHelper::GetIconImg(_T("cfgfile-22.png"));

    bcButton* openLogButton = new bcButton(this, idButtonOpenLogFile, _T("Open log"), logImg); openLogButton->SetToolTip(_T("Standard open dialog for log opening"),true);
    bcButton* openCfgButton = new bcButton(this, idButtonOpenConfigFile, _T("Open config"), cfgImg); openCfgButton->SetToolTip(_T("Standard open dialog for config opening"),true);
    bcButton* openLogExtButton = new bcButton(this, idButtonOpenLogFileExternal, _T("To new window"), logImg); openLogExtButton->SetToolTip(_T("Open log with separated frame"),true);
    bcButton* openAdvLogButton = new bcButton(this, idButtonAdvOpenLogFile, _T("Adv. open"), logImg); openAdvLogButton->SetToolTip(_T("Open advanced open dialog"),true);

    bcButton* openECUComm = new bcButton(this, idButtonOpenECUComm, _T("ECU"), DrawHelper::GetIconImg(_T("ecu-22.png")));
    bcButton* openGPSComm = new bcButton(this, idButtonOpenGPSComm, _T("GPS"), DrawHelper::GetIconImg(_T("icon_sat-22.png")));

    openLogButton->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);
    openCfgButton->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);
    openLogExtButton->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);
    openAdvLogButton->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);
    openECUComm->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);
    openGPSComm->SetStyleFlag(bcBUTTON_HOVER_OUTER_GLOW);

    wxFlexGridSizer* buttonSizers = new wxFlexGridSizer(3);
    buttonSizers->AddGrowableCol(0);

    wxBoxSizer* linesizer1 = new wxBoxSizer(wxHORIZONTAL);
    linesizer1->Add(new bcWrapper(new bcStaticLine(this,-1,wxDefaultPosition,wxSize(20,3))), 0, wxBOTTOM | wxALIGN_BOTTOM, 8);
        linesizer1->Add(new bcWrapper(new bcStaticBitmap(this, wxID_ANY, DrawHelper::GetIconImg(_T("open-48.png")))),0, wxRIGHT, 3 );
        TextDesc opentitleDesc(_T("File open dialogs")); opentitleDesc.Style = bcLabel_GRAVEN;
        bcLabel * opentitle = new bcLabel(this); opentitle->SetLabel(opentitleDesc);
        linesizer1->Add(new bcWrapper(opentitle), 0, wxRIGHT | wxALIGN_BOTTOM, 5);
    linesizer1->Add(new bcWrapper(new bcStaticLine(this)), 1, wxBOTTOM | wxALIGN_BOTTOM, 8);
    buttonSizers->Add(linesizer1, 0, wxEXPAND);

    buttonSizers->AddSpacer(10);

        wxBoxSizer* linesizer1b = new wxBoxSizer(wxHORIZONTAL);
        linesizer1b->Add(new bcWrapper(new bcStaticLine(this,-1,wxDefaultPosition,wxSize(20,3))), 1, wxBOTTOM | wxALIGN_BOTTOM, 8); //linesizer1b->Add(new bcWrapper(new bcStaticLine(this)), 0, wxBOTTOM | wxALIGN_BOTTOM, 8);
        TextDesc commTitleDesc(_T("Communication settings")); commTitleDesc.Style = bcLabel_GRAVEN;
        bcLabel * commtitle = new bcLabel(this); commtitle->SetLabel(commTitleDesc);
        linesizer1b->Add(new bcWrapper(commtitle), 0, wxRIGHT | wxALIGN_BOTTOM, 5);
        linesizer1b->Add(new bcWrapper(new bcStaticBitmap(this, wxID_ANY, DrawHelper::GetIconImg(_T("laptop-48.png")))),0, wxLEFT | wxRIGHT, 3 );
        linesizer1b->Add(new bcWrapper(new bcStaticLine(this,-1,wxDefaultPosition,wxSize(20,3))), 0, wxBOTTOM | wxALIGN_BOTTOM, 8);
    buttonSizers->Add(linesizer1b, 0, wxEXPAND);

        wxFlexGridSizer* openButtonsSizer = new wxFlexGridSizer(4);
        openButtonsSizer->Add(new bcWrapper(openLogButton), 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
        openButtonsSizer->Add(new bcWrapper(openLogExtButton), 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
        openButtonsSizer->Add(new bcWrapper(openAdvLogButton), 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
        openButtonsSizer->Add(new bcWrapper(openCfgButton), 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
    buttonSizers->Add(openButtonsSizer);

    buttonSizers->AddSpacer(10);

        wxBoxSizer* commButtonSizer = new wxBoxSizer(wxHORIZONTAL);
        commButtonSizer->Add(new bcWrapper(openECUComm), 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
        commButtonSizer->Add(new bcWrapper(openGPSComm), 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
    buttonSizers->Add(commButtonSizer, 0, wxALIGN_RIGHT);

    contentSizer->Add(buttonSizers, 0, wxALIGN_CENTER_HORIZONTAL | wxEXPAND, 5);


// put file history
    wxBoxSizer* linesizer2 = new wxBoxSizer(wxHORIZONTAL);

    linesizer2->Add(new bcWrapper(new bcStaticLine(this,-1,wxDefaultPosition,wxSize(20,3))), 0, wxBOTTOM | wxALIGN_BOTTOM, 8);
    linesizer2->Add(new bcWrapper(new bcStaticBitmap(this, wxID_ANY, DrawHelper::GetIconImg(_T("open-48-history.png")))),0, wxRIGHT, 3 );
    linesizer2->Add(new bcWrapper(new bcStaticLine(this)), 1, wxBOTTOM | wxALIGN_BOTTOM, 8);

    contentSizer->Add(linesizer2, 0, wxEXPAND | wxTOP, 10);

    wxArrayString recentLogs = RecentVemsFiles::getRecentFiles(true,5);
    wxArrayString recentConfigs = RecentVemsFiles::getRecentFiles(false,5);

    /// recent logs, recent configs, default configs

    wxGridBagSizer* recentSizer = new wxGridBagSizer();
    wxGridBagSizer* defaultSizer = new wxGridBagSizer();
    int line = 0;

    AddHistoryCaption(recentSizer, line, _T("Recent log files"));
    for(size_t a=0; a < recentLogs.GetCount(); a++ ){
        AddHistoryLine(recentSizer, line, recentLogs[a], logImg );
    }


    line = 0; // start new column (#2)
    AddHistoryCaption(recentSizer, line, _T("Recent config files"), 2);
    for(size_t a=0; a < recentConfigs.GetCount(); a++ ){
        AddHistoryLine(recentSizer, line, recentConfigs[a], cfgImg, 2);
    }

    contentSizer->Add(recentSizer, 0, wxEXPAND | wxALL ,5);


    wxBoxSizer* linesizer3 = new wxBoxSizer(wxHORIZONTAL);
    linesizer3->Add(new bcWrapper(new bcStaticLine(this,-1,wxDefaultPosition,wxSize(20,3))), 0, wxBOTTOM | wxALIGN_BOTTOM, 8);
    linesizer3->Add(new bcWrapper(new bcStaticBitmap(this, wxID_ANY, DrawHelper::GetIconImg(_T("open-48-vems.png")))),0, wxRIGHT, 3 );
    //linesizer3->Add(new bcWrapper(new bcLabel(this, wxID_ANY, _T("Open default configs"))),0, wxALIGN_BOTTOM, 3 );
    linesizer3->Add(new bcWrapper(new bcStaticLine(this)), 1, wxBOTTOM | wxALIGN_BOTTOM, 8);

    contentSizer->Add(linesizer3, 1, wxEXPAND | wxTOP, 5 );

    line = 0;
    wxArrayString v3SampleLogs;
    FileHelper::GetFilesExp(DirHelper::GetPath(dDEFAULTCONFIGS),v3SampleLogs,_T(".+\\.vemslog"), NO_FILE_LIMIT, ORDER_DESC);
    AddHistoryCaption(defaultSizer, line, _T("Example data log files"));
    for(size_t a=0; a < v3SampleLogs.GetCount(); a++ ){
        AddHistoryLine(defaultSizer, line, v3SampleLogs[a], logImg, 0 );
    }

    line = 0;
    wxArrayString v3DefConfigs;
    FileHelper::GetFilesExp(DirHelper::GetPath(dDEFAULTCONFIGS),v3DefConfigs,_T("config_[0-9]{1,}\\.[0-9]{1,}\\.[0-9]{2,}\\.vemscfg"), NO_FILE_LIMIT, ORDER_DESC);
    AddHistoryCaption(defaultSizer, line, _T("Default V3 config files"), 2);
    for(size_t a=0; a < v3DefConfigs.GetCount(); a++ ){
        AddHistoryLine(defaultSizer, line, v3DefConfigs[a], cfgImg, 2 );
    }

    line = 0;
    wxArrayString roundDefConfigs;
    FileHelper::GetFilesExp(DirHelper::GetPath(dDEFAULTCONFIGS),roundDefConfigs,_T("round_[0-9]{1,}\\.[0-9]{1,}\\.[0-9]{1,}\\.vemscfg"), NO_FILE_LIMIT, ORDER_DESC);
    AddHistoryCaption(defaultSizer, line, _T("Default round config files"), 4);
    for(size_t a=0; a < roundDefConfigs.GetCount(); a++ ){
        AddHistoryLine(defaultSizer, line, roundDefConfigs[a], cfgImg, 4 );
    }

    contentSizer->Add(defaultSizer, 0, wxEXPAND | wxALL ,5);


    mainsizer->Add(contentSizer, 0, wxALIGN_CENTER_HORIZONTAL);


    mainsizer->AddSpacer(30);

    SetSizer(mainsizer);
    Refresh(false);
}

void VemsWelcomeScreen::AddHistoryCaption(wxGridBagSizer* recentSizer, int& line, const wxString& label, int col)
{
    bcLabel * bclab = new bcLabel(this);
    TextDesc desc(label);
    desc.Style = bcLabel_GRAVEN;
    //desc.Color = wxColor(255,255,255);
    desc.Height = 13;
    bclab->SetLabel(desc);
    recentSizer->Add(new bcWrapper(bclab),wxGBPosition(line,col), wxGBSpan(1,2), wxALL, 5 );
    line++;
}

void VemsWelcomeScreen::AddHistoryLine(wxGridBagSizer* recentSizer, int& line, const wxString& label, const wxImage& iconimage, int startCol)
{
    recentSizer->Add(new bcWrapper(new bcStaticBitmap(this, wxID_ANY, iconimage)), wxGBPosition(line,startCol), wxDefaultSpan, wxALIGN_CENTER_VERTICAL | wxLEFT, 20);

    wxFileName fn(label);

    bcLinkInfo link(label,fn.GetFullName(),label);
    link.LabelLimit = 95;

    bcLinkLabel* bclab = new bcLinkLabel(this, link);
    recentSizer->Add(new bcWrapper(bclab), wxGBPosition(line,startCol+1), wxDefaultSpan, wxALL, 5);
    bclab->Connect(wxEVT_BCLINK, bcLinkEventHandler(VemsWelcomeScreen::OnLink), NULL, this );

    line++;
}


/***************************
        Controlling
*/

void VemsWelcomeScreen::SendCommand(int cmd, const wxString& msg)
{
    if(!m_eventHandler) return;

    VemsGUICommandEvent event(cmd);
    event.SetMessage(msg);

    m_eventHandler->AddPendingEvent(event);
}

void VemsWelcomeScreen::OnLink(bcLinkEvent& evt)
{
    wxString filename = evt.GetLinkInfo().HRef;

    wxFileName fn(filename);

    if(fn.GetExt() == _T("vemslog")) {

        SendCommand(MFCMD_OPEN_LOGFILE, filename);

    } else if(fn.GetExt() == _T("vemscfg")) {

        SendCommand(MFCMD_OPEN_CONFIGFILE, filename);

}
}


void VemsWelcomeScreen::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()) {
        case idButtonOpenLogFile:
            SendCommand(MFCMD_OPENDLG_LOGFILE);
            break;

        case idButtonOpenConfigFile:
            SendCommand(MFCMD_OPENDLG_CONFIGFILE);
            break;

        case idButtonAdvOpenLogFile:
            SendCommand(MFCMD_OPENDLG_LOGFILEADV);
            break;

        case idButtonOpenLogFileExternal:
            SendCommand(MFCMD_OPENDLG_LOGFILETOSEPAR);
            break;

        case idButtonOpenECUComm:
            SendCommand(MFCMD_OPENDLG_COMM_ECU);
            break;

        case idButtonOpenGPSComm:
            SendCommand(MFCMD_OPENDLG_COMM_GPS);
            break;

    }
}

void VemsWelcomeScreen::OnSize(wxSizeEvent& evt)
{
    DrawBackground();
    Refresh(false);

    evt.Skip();
}


/***************************
          Drawing
*/


void VemsWelcomeScreen::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);
    dc.Clear();

    if(m_bgBitmap) dc.DrawBitmap(*m_bgBitmap,0,0);

    PaintWidgets(dc);
}

void VemsWelcomeScreen::DrawBackground()
{
    wxAgg2 canvas(this,wxColor(0,0,0,255));

    if(!canvas.isOk()) return;

    wxAggPen* pen = canvas.getPen();
    pen->SetStyle(wxAGGPEN_TRANSPARENT);
    wxAggBrush* brush = canvas.getBrush();

    brush->SetStyle(wxAGGBRUSH_GRADIENT);
    brush->SetColor(wxColor(250,250,250));
    brush->SetEndColor(wxColor(170,170,170));
    brush->SetGradientOrientation(-90);

    wxRealRect contentRect(0,m_headHeight,GetClientSize().x,GetClientSize().y-m_headHeight);

    canvas.drawRectangle( contentRect );

    double lw = 3;
    pen->SetStyle(wxAGGPEN_SOLID);
    pen->SetWidth(lw);
    pen->SetColor(wxColor(42,42,42));
    canvas.drawLine(0,m_headHeight+lw/2,GetClientSize().x,m_headHeight+lw/2);

    ptrdel(m_bgBitmap);

    m_bgBitmap = canvas.renderToBitmap(false);
}


