#include "wxHotKeyManagerGUI.h"

#include <wx/keybinder.h>
#include <wx/statline.h>
#include <wx/regex.h>

#include "../logger.h"
#include "../DirHelper.h"
#include "wxHotKeyManager.h"
#include "../wxTextCtrlHotKey.h"
#include "../wxSqliteHelper.h"

BEGIN_EVENT_TABLE(wxHotKeyManagerGUI, wxFrame)

    EVT_LISTBOX(idLBHotKey, wxHotKeyManagerGUI::OnListBox)
    EVT_LISTBOX(idLBEvents, wxHotKeyManagerGUI::OnListBox)

    EVT_BUTTON(idBtnRemove, wxHotKeyManagerGUI::OnButton)
    EVT_BUTTON(idBtnRemAll, wxHotKeyManagerGUI::OnButton)
    EVT_BUTTON(idBtnAdd, wxHotKeyManagerGUI::OnButton)
    EVT_BUTTON(idBtnClose, wxHotKeyManagerGUI::OnButton)
    EVT_BUTTON(idBtnSetProf, wxHotKeyManagerGUI::OnButton)
    EVT_BUTTON(idBtnAddNewProfile, wxHotKeyManagerGUI::OnButton)
    EVT_BUTTON(idBtnBrwProfile, wxHotKeyManagerGUI::OnButton)
    EVT_BUTTON(idBtnDelProfile, wxHotKeyManagerGUI::OnButton)


    EVT_COMBOBOX(idCBProfileList, wxHotKeyManagerGUI::OnCombo)

END_EVENT_TABLE()

wxHotKeyManagerGUI::wxHotKeyManagerGUI( wxWindow * parent, const wxString &title )
{
    //ctor
    Create( parent, -1, title);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    m_profilesFile = DirHelper::GetPath(dHOTKEYS) + _T("profiles.db");

    m_title = new wxStaticText(this, -1, title);
    m_usedText = new wxStaticText(this, -1, _T("used"));

    wxFont font = m_title->GetFont();
    font.SetPointSize( font.GetPointSize()+2 );
    font.SetWeight(wxFONTWEIGHT_BOLD);
    m_title->SetFont( font );

    m_profileList = new wxComboBox(this, idCBProfileList, _T(""), wxDefaultPosition, wxDefaultSize,0,0, wxCB_READONLY | wxCB_SORT);

    m_eventsListBox = new wxListBox(this, idLBEvents);
    m_hotkeyListBox = new wxListBox(this, idLBHotKey);

    m_hotkeyCtrl = new wxTextCtrlHotKey(this, idTCHotKey, _T(""));
    m_hotkeyCtrl->registerObserver(this);

    m_hotkeyFeedBack = new wxStaticText(this,-1,_T(""), wxDefaultPosition, wxSize(-1,20), wxBORDER_SIMPLE | wxALIGN_CENTRE | wxST_NO_AUTORESIZE);
    m_hotkeyFeedBack->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNHIGHLIGHT));
    m_hotkeyFeedBack->ClearBackground();

    m_newProfileCtrl = new wxTextCtrl(this,-1,_T(""));

    m_btnRem = new wxButton(this, idBtnRemove, _T("Remove"));
    m_btnRemall = new wxButton(this, idBtnRemAll, _T("Remove All"));
    m_btnAdd = new wxButton(this, idBtnAdd, _T("Add"));
    m_btnSetProf = new wxButton(this, idBtnSetProf, _T("Activate profile"));
    m_btnSetProf->Enable(false);
    //m_btnSHCreater = new wxButton(this,idBtnSHCreater,_T("New"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);

    m_addProfText = new wxStaticText(this,-1,_T("Add"));

    int linecnt = 0;

    m_mainsizer = new wxGridBagSizer();

    m_mainsizer->AddGrowableCol(0);
    m_mainsizer->AddGrowableCol(1);

    wxString note = _T("Imortant to know that if there is a collision with global hotkeys\n");
    note += _T("(for example gauge group hotkeys) the global hokey will be used and this will be ignored!");

    m_mainsizer->Add( new wxStaticText(this, -1, note), wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALL| wxALIGN_CENTER_VERTICAL|wxALIGN_CENTER_HORIZONTAL,5);
    linecnt++;

    wxBoxSizer * proflistsizer = new wxBoxSizer(wxHORIZONTAL);

    proflistsizer->Add( new wxStaticText(this, -1, _T("Profiles:")),0,wxALL|wxALIGN_CENTER_VERTICAL,5);
    proflistsizer->Add( m_profileList, 0, wxALL, 5);

    m_mainsizer->Add( proflistsizer, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALL| wxALIGN_CENTER_VERTICAL);
    linecnt++;

    m_mainsizer->Add( m_usedText, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALL| wxALIGN_CENTER_VERTICAL,5);
    //m_mainsizer->Add( m_btnSHCreater, wxGBPosition(linecnt,1), wxGBSpan(1,1), wxALL| wxALIGN_CENTER_VERTICAL,5);
    linecnt++;

    wxBoxSizer * s = new wxBoxSizer(wxVERTICAL);
    s->Add(new wxStaticLine(this,-1,wxDefaultPosition,wxSize(2,2)),0,wxALL|wxEXPAND,5);
    m_mainsizer->Add( s, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL | wxEXPAND);
    linecnt++;

    m_mainsizer->Add( m_title, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALL| wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL, 5);
    linecnt++;

    m_mainsizer->Add( new wxStaticText(this, -1, _T("Commands:")), wxGBPosition(linecnt,0), wxGBSpan(1,1), wxALL| wxALIGN_CENTER_HORIZONTAL, 5);
    m_mainsizer->Add( new wxStaticText(this, -1, _T("Shortcuts:")), wxGBPosition(linecnt,1), wxGBSpan(1,1), wxALL| wxALIGN_CENTER_HORIZONTAL, 5);
    linecnt++;

    m_mainsizer->AddGrowableRow(linecnt);

    m_mainsizer->Add( m_eventsListBox, wxGBPosition(linecnt,0), wxDefaultSpan, wxALL| wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL | wxEXPAND, 5);

        wxBoxSizer * hksizer = new wxBoxSizer(wxVERTICAL);
        hksizer->Add( m_hotkeyListBox, 1, wxALL| wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL | wxEXPAND, 5);

            wxBoxSizer * hkbsizer = new wxBoxSizer(wxHORIZONTAL);
            hkbsizer->Add( m_btnRem, 0, wxALL, 5);
            hkbsizer->Add( m_btnRemall, 0, wxALL, 5);

        hksizer->Add( hkbsizer, 0, wxALIGN_CENTER_HORIZONTAL);
        hksizer->Add( new wxStaticText(this,-1,_T("New Shortcut:")), 0, wxTOP|wxRIGHT|wxLEFT, 5);
        hksizer->Add( m_hotkeyCtrl, 0, wxALL | wxEXPAND,5);
        hksizer->Add( new wxStaticText(this,-1,_T("Currently assigned to:")), 0, wxTOP|wxRIGHT|wxLEFT , 5);
        hksizer->Add( m_hotkeyFeedBack, 0, wxALL | wxEXPAND,5);
        hksizer->Add( m_btnAdd, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5);

    m_mainsizer->Add( hksizer, wxGBPosition(linecnt,1), wxDefaultSpan, wxALL| wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL | wxEXPAND, 0);
    linecnt++;



    s = new wxBoxSizer(wxVERTICAL);
    s->Add(new wxStaticLine(this,-1,wxDefaultPosition,wxSize(2,2)),0,wxALL|wxEXPAND,5);
    m_mainsizer->Add( s, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL | wxEXPAND);
    linecnt++;

    m_mainsizer->Add( m_btnSetProf, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALL| wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL, 5);
    linecnt++;

    s = new wxBoxSizer(wxVERTICAL);
    s->Add(new wxStaticLine(this,-1,wxDefaultPosition,wxSize(2,2)),0,wxALL|wxEXPAND,5);
    m_mainsizer->Add( s, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL | wxEXPAND);
    linecnt++;

        wxBoxSizer * profmgrsizer = new wxBoxSizer(wxHORIZONTAL);

            m_createProfilSizer = new wxGridBagSizer();
            m_createProfilSizer->Add( new wxStaticText(this, -1, _T("Make a copy from this profile:")),wxGBPosition(0,0), wxGBSpan(1,3) ,wxALL|wxALIGN_CENTER_VERTICAL,3);
            m_createProfilSizer->Add( new wxStaticText(this, -1, _T("Title:")),wxGBPosition(1,0), wxGBSpan(1,1) ,wxALL|wxALIGN_CENTER_VERTICAL,3);
            m_createProfilSizer->Add( m_newProfileCtrl,wxGBPosition(1,1), wxGBSpan(1,1) ,wxALL|wxALIGN_CENTER_VERTICAL,3);
            m_createProfilSizer->Add( new wxButton(this,idBtnAddNewProfile,_T("Make")),wxGBPosition(1,2), wxGBSpan(1,1) ,wxALL|wxALIGN_CENTER_VERTICAL,3);

        profmgrsizer->Add( m_createProfilSizer, 0, wxALL| wxALIGN_CENTER_VERTICAL, 5);
        profmgrsizer->Add(new wxStaticLine(this,-1,wxDefaultPosition,wxSize(2,2)),0,wxALL|wxEXPAND,5);

            wxBoxSizer * addprofs = new wxBoxSizer(wxVERTICAL);
            addprofs->Add( m_addProfText, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,3);
            addprofs->Add( new wxButton(this,idBtnBrwProfile, _T("Browse")), 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,3);

        profmgrsizer->Add( addprofs, 0, wxALL| wxALIGN_CENTER_VERTICAL, 5);
        profmgrsizer->Add(new wxStaticLine(this,-1,wxDefaultPosition,wxSize(2,2)),0,wxALL|wxEXPAND,5);

            wxBoxSizer * remprofs = new wxBoxSizer(wxVERTICAL);
            remprofs->Add( new wxStaticText(this,-1,_T("Remove current profile")), 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,3);
            remprofs->Add( new wxButton(this,idBtnDelProfile, _T("Remove")), 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL,3);

        profmgrsizer->Add( remprofs, 0,   wxALL| wxALIGN_CENTER_VERTICAL, 5);

    m_mainsizer->Add( profmgrsizer, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALL| wxALIGN_CENTER_VERTICAL |wxALIGN_CENTER_HORIZONTAL| wxEXPAND, 0);
    linecnt++;

    s = new wxBoxSizer(wxVERTICAL);
    s->Add(new wxStaticLine(this,-1,wxDefaultPosition,wxSize(2,2)),0,wxALL|wxEXPAND,5);
    m_mainsizer->Add( s, wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL | wxEXPAND);
    linecnt++;

    m_mainsizer->Add( new wxButton(this, idBtnClose, _T("Close")), wxGBPosition(linecnt,0), wxGBSpan(1,2), wxALL| wxALIGN_CENTER_HORIZONTAL| wxALIGN_CENTER_VERTICAL, 5);
    linecnt++;

	SetSizer(m_mainsizer);
	m_mainsizer->Fit(this);
	m_mainsizer->SetSizeHints(this);

    StartState();


}

wxHotKeyManagerGUI::~wxHotKeyManagerGUI()
{
    //dtor
}


void wxHotKeyManagerGUI::StartState()
{
    m_btnRem->Enable(false);
    m_btnRemall->Enable(false);
    m_btnAdd->Enable(false);

    m_eventsListBox->Clear();
    m_hotkeyCtrl->SetValue(_T(""));
    m_hotkeyFeedBack->SetLabel(_T(""));
    m_hotkeyListBox->Clear();
}

void wxHotKeyManagerGUI::OnListBox(wxCommandEvent &evt)
{
    switch( evt.GetId() ){
        case idLBEvents:
            UpdateHotKeyList();
            break;

        case idLBHotKey:
            break;


    }
}

void wxHotKeyManagerGUI::LoadProfile(const wxString &title)
{

    wxSQLite3Database db;

    try{
        db.Open( m_profilesFile );
    } catch (wxSQLite3Exception &ex) {
        Logger::logf(LOG_MSG,_T("wxHotKeyManagerGUI::LoadTarget sqllite exception msg=")+ex.GetMessage());
        return;
    }

    wxString tf = DirHelper::GetPath(dHOTKEYS) + _T("targets.db");
    db.ExecuteQuery(wxString::Format(_T("attach database \"%s\" as targetdb;"),tf.c_str()));

    wxSQLite3ResultSet res = db.ExecuteQuery( wxString::Format(_T("select * from profiles where title='%s';"),title.c_str() ) );

    m_title->SetLabel( _T("Edit profile: ") + res.GetString(_T("title")) );

    db.Close();

    if( res.GetInt(_T("isuse")) == 1 )
        m_btnSetProf->Enable(false);
    else
        m_btnSetProf->Enable(true);

    LoadFile( DirHelper::GetPath(dHOTKEYS) + res.GetString(_T("filename")) );

}

void wxHotKeyManagerGUI::OnCombo(wxCommandEvent &evt)
{
    wxString str = m_profileList->GetValue();

    LoadProfile( str );
}

void wxHotKeyManagerGUI::UpdateHotKeyList()
{
    PutHotKeys( m_eventsListBox->GetStringSelection() );
}

void wxHotKeyManagerGUI::PutHotKeys(const wxString &title)
{
    wxString qs = wxString::Format(_T("select keystring from hotkeys where \
                            event_id=(select event_id from hotkeyevent where title=\"%s\") order by id asc;"), title.c_str());

    wxSQLite3ResultSet res = m_db.ExecuteQuery( qs );

    m_hotkeyListBox->Clear();

    while( res.NextRow() ){
        m_hotkeyListBox->Append( res.GetString(0) );
    }

    if( m_hotkeyListBox->GetCount() ) {
        m_hotkeyListBox->Select(0);
        m_btnRem->Enable(true);
        m_btnRemall->Enable(true);
    } else {
        m_btnRem->Enable(false);
        m_btnRemall->Enable(false);
    }

}

void wxHotKeyManagerGUI::OnButton(wxCommandEvent &evt)
{
    switch( evt.GetId() ){
        case idBtnAdd:
            AddHotKey();
            break;

        case idBtnRemAll:
            RemoveAllHotKey();
            break;

        case idBtnRemove:
            RemoveHotKey();
            break;

        case idBtnClose:
            Close();
            break;

        case idBtnSetProf:
            SetProfile();
            break;

        case idBtnSHCreater:

            break;

        case idBtnAddNewProfile:
            MakeNewProfile();
            break;

        case idBtnBrwProfile:
            BrwProfile();
            break;

        case idBtnDelProfile:
            DelProfile();
            break;
    }
}

void wxHotKeyManagerGUI::BrwProfile()
{
    wxFileDialog dlg(this);
    dlg.SetPath( DirHelper::GetPath(dHOTKEYS) );
    dlg.SetWildcard(_T("Hotkey profile database files (*.db;*.s3db)|*.db;*.s3db"));
    if(dlg.ShowModal() != wxID_OK) return;

    wxTextEntryDialog tdlg(this,_T("The profile's title:"));

    wxSQLite3Database db;
    db.Open( m_profilesFile );

    wxString tf = DirHelper::GetPath(dHOTKEYS) + _T("targets.db");
    db.ExecuteQuery(wxString::Format(_T("attach database \"%s\" as targetdb;"),tf.c_str()));

    int target_id = db.ExecuteScalar( wxString::Format(_T("select id from targets where title='%s'"),m_targetTitle.c_str()) );

    int sc = 0;

    wxString title;
    wxString resp;

    do{

        if(tdlg.ShowModal() != wxID_OK) return;

        title = tdlg.GetValue();

        sc = db.ExecuteScalar(wxString::Format(_T("select count(*) from profiles where target_id=%i and title='%s';"),target_id,title.c_str()));

    }while(sc);


    wxString qs = wxString::Format(_T("insert into profiles (target_id,title,filename,isuse) values (%i,'%s','%s',0);"),
                                        target_id,title.c_str(),dlg.GetFilename().c_str() );

    db.ExecuteUpdate( qs );

    m_profileList->Append( title );
    db.Close();
}

void wxHotKeyManagerGUI::DelProfile()
{
    if(m_profileList->GetCount() == 1) {
        wxMessageBox(_T("You can't remove the only profile for ")+m_targetTitle);
        return;
    }

    wxSQLite3Database db;
    db.Open( m_profilesFile );

    wxString tf = DirHelper::GetPath(dHOTKEYS) + _T("targets.db");
    db.ExecuteQuery(wxString::Format(_T("attach database \"%s\" as targetdb;"),tf.c_str()));

    int target_id = db.ExecuteScalar( wxString::Format(_T("select id from targets where title='%s'"),m_targetTitle.c_str()) );

    db.ExecuteUpdate(wxString::Format(_T("delete from profiles where target_id=%i and title='%s'"),target_id,m_profileList->GetValue().c_str() ));
    db.Close();

    m_profileList->Delete( m_profileList->FindString(m_profileList->GetValue(),true) );
    m_profileList->Select(0);

    LoadProfile( m_profileList->GetValue() );
}

void wxHotKeyManagerGUI::MakeNewProfile()
{
    wxSQLite3Database db;
    db.Open( m_profilesFile );

    wxString tf = DirHelper::GetPath(dHOTKEYS) + _T("targets.db");
    db.ExecuteQuery(wxString::Format(_T("attach database \"%s\" as targetdb;"),tf.c_str()));

    wxString title = m_newProfileCtrl->GetValue();

    wxSQLite3ResultSet rest = db.ExecuteQuery( wxString::Format(_T("select id from targets where title='%s';"),m_targetTitle.c_str()) );

    int target_id = rest.GetInt(0);

    wxSQLite3ResultSet res = db.ExecuteQuery( wxString::Format(_T("select count(*) from profiles where target_id=%i and title='%s';"),target_id,title.c_str()) );

    if( res.GetInt(0) > 0 ) {
        db.Close();
        wxMessageBox(_T("This title is alredy exists!\nPlease select an other one."),_T("Error"));
        return;
    }

    wxString filename = m_targetTitle + _T("_") + title;

    wxRegEx exp1;
    exp1.Compile(_T("[^a-zA-Z0-9_]"));
    exp1.ReplaceAll(&filename,_T(""));

    wxString ext = _T(".db");

    if( wxFileExists( DirHelper::GetPath( dHOTKEYS ) + filename + ext ) ){
        size_t a=0;
        wxString fn;
        while(1){
            fn = filename + wxString::Format(_T("-%i"),a);

            if( !wxFileExists( DirHelper::GetPath( dHOTKEYS ) + fn + ext) ) break;

            a++;
        }

        filename = fn;
    }

    filename += ext;

    db.ExecuteUpdate( wxString::Format(_T("insert into profiles (target_id,title,filename,isuse) values \
        (%i,'%s','%s',0);"),target_id,title.c_str(),filename.c_str() ) );

    wxSQLite3ResultSet res3 = db.ExecuteQuery(wxString::Format(_T("select filename from profiles where title='%s' \
                                    and target_id=%i;"),m_profileList->GetValue().c_str(),target_id) );

    wxCopyFile( DirHelper::GetPath(dHOTKEYS) + res3.GetString(0), DirHelper::GetPath(dHOTKEYS) + filename);

    m_profileList->Append( title );

    m_newProfileCtrl->SetValue(_T(""));

}

void wxHotKeyManagerGUI::SetProfile()
{
    wxString str = m_profileList->GetValue();

    wxSQLite3Database db;
    db.Open( m_profilesFile );

    wxString tf = DirHelper::GetPath(dHOTKEYS) + _T("targets.db");
    db.ExecuteQuery(wxString::Format(_T("attach database \"%s\" as targetdb;"),tf.c_str()));

    wxSQLite3ResultSet res = db.ExecuteQuery(wxString::Format(_T("select target_id from profiles where title='%s';"),str.c_str()));

    db.ExecuteUpdate( wxString::Format(_T("update profiles set isuse=0 where target_id=%i;"),res.GetInt(0)) );
    db.ExecuteUpdate( wxString::Format(_T("update profiles set isuse=1 where title='%s' and target_id=%i;"),str.c_str(),res.GetInt(0)) );

    m_usedText->SetLabel( _T("Currently used profile on ") + m_targetTitle + _T(": ") + str );

    db.Close();

    m_btnSetProf->Enable(false);
}

void wxHotKeyManagerGUI::RemoveHotKey()
{
    wxString hkstr = m_hotkeyListBox->GetStringSelection();

    m_db.ExecuteUpdate( wxString::Format(_T("delete from hotkeys where keystring='%s';"),hkstr.c_str()  ) );

    UpdateHotKeyList();
}

void wxHotKeyManagerGUI::RemoveAllHotKey()
{
    wxArrayString arr = m_hotkeyListBox->GetStrings();

    m_db.Begin();

    for( size_t a=0; a<arr.GetCount();a++ ) {
        m_db.ExecuteUpdate( wxString::Format(_T("delete from hotkeys where keystring='%s';"),arr[a].c_str()) );
    }

    m_db.Commit();

    UpdateHotKeyList();
}

void wxHotKeyManagerGUI::AddHotKey()
{

    wxString evtstr = m_eventsListBox->GetStringSelection();
    wxString hkstr = m_hotkeyCtrl->GetValue();

    m_db.ExecuteUpdate( wxString::Format(_T("insert into hotkeys (event_id,keystring) \
                        values((select event_id from hotkeyevent where title=\"%s\"),'%s');"),evtstr.c_str(),hkstr.c_str()) );


    UpdateHotKeyList();
}

bool wxHotKeyManagerGUI::LoadTarget(const wxString &targetTitle)
{
    wxSQLite3Database  db;

    SetTitle(_T("HotKey managing for ")+targetTitle);

    m_targetTitle = targetTitle;

    m_addProfText->SetLabel(_T("Add profile for ")+m_targetTitle);

    try{
        db.Open( m_profilesFile );
    } catch (wxSQLite3Exception &ex) {
        Logger::logf(LOG_MSG,_T("wxHotKeyManagerGUI::LoadTarget sqllite exception msg=")+ex.GetMessage());
        return false;
    }

    m_profileList->Clear();

    wxString tf = DirHelper::GetPath(dHOTKEYS) + _T("targets.db");
    db.ExecuteQuery(wxString::Format(_T("attach database \"%s\" as targetdb;"),tf.c_str()));

    wxString qs = wxString::Format(_T("select * from profiles inner join targets on profiles.target_id=targets.id \
                                       where targets.title='%s' order by profiles.title asc;"),targetTitle.c_str());

    wxSQLite3ResultSet res = db.ExecuteQuery( qs );

    wxString file;

    m_eventsDbFile = DirHelper::GetPath( dHOTKEYS ) + res.GetString(_T("symboltablefile"));

    while( res.NextRow() ) {
        m_profileList->Append( res.GetString(_T("title")) );
        if( res.GetInt(_T("isuse")) == 1 ) {
            file = DirHelper::GetPath( dHOTKEYS ) + res.GetString(_T("filename"));
            m_title->SetLabel( _T("Edit profile: ") + res.GetString(_T("title")) );
            m_profileList->SetStringSelection( res.GetString(_T("title")) );
            m_usedText->SetLabel( _T("Currently used profile on ") + targetTitle + _T(": ") + res.GetString(_T("title")) );
        }
    }

    db.Close();

    Logger::logf(LOG_MSG,_T("m_eventsDbFile=")+m_eventsDbFile);

    LoadFile(file);

    return true;
}

bool wxHotKeyManagerGUI::LoadFile(const wxString &dbfile)
{
    m_dbFile = dbfile;

    if(m_db.IsOpen())
        m_db.Close();

    //wxSQLite3Database db;

    try{
        m_db.Open(m_dbFile);
    } catch (wxSQLite3Exception &ex) {
        Logger::logf(LOG_MSG,_T("wxHotKeyManagerGUI::LoadFile sqllite exception msg=")+ex.GetMessage());
        return false;
    }

    m_db.ExecuteQuery(wxString::Format(_T("attach database \"%s\" as eventsdb;"),m_eventsDbFile.c_str()));

    if( !m_db.TableExists(_T("hotkeys")) ){
        Logger::logf(LOG_MSG,_T("wxHotKeyManagerGUI::LoadFile tables not exists"));
        return false;
    }

    wxString qs = _T("select title from hotkeyevent order by title asc;");

    wxSQLite3ResultSet res = m_db.ExecuteQuery(qs);

    StartState();

    while( res.NextRow() ){
        m_eventsListBox->Append( res.GetString(0) );
    }


  //  m_mainsizer->Fit(this);
    m_mainsizer->SetSizeHints(this);

    wxSize es = m_eventsListBox->GetSize();

    m_hotkeyFeedBack->SetMinSize(wxSize(es.x+10,20));

//    m_mainsizer->Fit(this);
  //  m_mainsizer->SetSizeHints(this);

    //m_eventsListBox->SetSize(wxSize(es.x,es.x*1.5));
    m_eventsListBox->SetMinSize(wxSize(es.x,(int)(es.x*1.3)));

    m_mainsizer->Fit(this);

    m_eventsListBox->SetMinSize(wxSize(es.x,100));


    return true;
}

void wxHotKeyManagerGUI::notify( int i )
{
    Logger::logf(LOG_MSG,_T("wxHotKeyManagerGUI::notify"));

    if( i == 1 ) {
        wxString qs = wxString::Format( _T("select title from hotkeyevent inner join hotkeys on \
        hotkeys.event_id=hotkeyevent.event_id where hotkeys.keystring='%s';"), m_hotkeyCtrl->GetValue().c_str() );

        wxSQLite3ResultSet res = m_db.ExecuteQuery( qs );
        m_hotkeyFeedBack->SetLabel( _T(" ") + res.GetString(0) + _T(" ") );
        m_btnAdd->Enable(true);
    } else if( i == 2 ) {
        m_btnAdd->Enable(false);
        m_hotkeyFeedBack->SetLabel(_T(""));
    }
}
