#include "logger.h"

#include <wx/log.h>
#include <wx/datetime.h>

#if defined (__WINCE_STANDARDSDK__)
   #include <mmsystem.h>
#endif

wxLog* Logger::fileLogTarget = NULL;
unsigned int Logger::m_attrib = 0;

void Logger::initLog( const wxString& targetFileFullPath )
{	
#if defined(__WINCE_STANDARDSDK__)
	FILE *fp;
	fp  = fopen(targetFileFullPath.mb_str(),"w");
    wxLog::SetActiveTarget( new wxLogStderr(fp) );
#else
    #if defined(VEMSDEBUG)
        FILE *fp;
        fp  = fopen(targetFileFullPath.mb_str(),"w");
        fileLogTarget = new wxLogStderr(fp);
        wxLog::SetActiveTarget(fileLogTarget);
        new wxLogChain( new wxLogStderr() );
    #else
        wxLog::SetActiveTarget( new wxLogStderr() );
    #endif
#endif

    saveConsoleAttrib();

    logf(LOG_MSG, _T("Logger::initLog() Finished! - ") + targetFileFullPath );
}

void Logger::cleanup()
{
	delete wxLog::SetActiveTarget( fileLogTarget );
}

// szFormat should NOT contain unicode garbage: use "cleanformat %s", wx("unicodegarbage")
void Logger::logf(logType type, const wxChar *szFormat, ...)
{

if( type == LOG_MSG){
    #if !defined (VEMSDEBUG)
        return;
    #endif
}
    va_list argptr;
    va_start(argptr, szFormat);
/*
#if wxUSE_UNICODE
    wxMB2WXbuf warg = wxConvertMB2WX(szFormat);
    wxChar *tmp = wxStrdup(warg);
#else // !wxUSE_UNICODE
    #define wxArgv argv
#endif // wxUSE_UNICODE/!wxUSE_UNICODE
*/
    //wxLogError(szFormat, argptr );
    //fprintf(stdout, (const char*)szFormat, argptr );


    wxVLogMessage( szFormat, argptr ); // nem megy (empty output)
//    wxVLogMessage( _T("haha %i %i %i"), argptr ); // ez megy

//    vsprintf( buffer,szFormat, argptr );

    va_end(argptr);

    setTextAttrib( m_attrib );
  //  fprintf(stdout, buffer );

    // wxString ws = wxString::Format( _T("%s"), buffer );
    // Logger::log(type, ws );
}

void Logger::logC(logType type, char* buff)
{

     //fprintf(stdout, "TEST LOG %s\n",(const char*)wxConvUTF8.cWX2MB(str.c_str() ) );
     //const char * tmp = UnicodeHelper::U2C(str);

     switch(type)
     {
        case LOG_MSG:
            wxLogMessage(_T("%s"), buff );
            //fprintf(stdout, "%s\n",(const char*)wxConvUTF8.cWX2MB(str.c_str() ) );
            break;
        case LOG_WARNING:
             wxLogWarning(_T("%s"), buff );
             break;
        case LOG_ERROR:
            wxLogError(_T("%s"),  buff );
            break;
        default:
            wxLogMessage(_T("%s"), buff );

     }

     //delete tmp;
}

void Logger::log(logType type, const wxString& str)
{

     //fprintf(stdout, "TEST LOG %s\n",(const char*)wxConvUTF8.cWX2MB(str.c_str() ) );
     //const char * tmp = UnicodeHelper::U2C(str);

     switch(type)
     {
        case LOG_MSG:
            wxLogMessage(_T("%s"), str.c_str() );
            //fprintf(stdout, "%s\n",(const char*)wxConvUTF8.cWX2MB(str.c_str() ) );
            break;
        case LOG_WARNING:
             wxLogWarning(_T("%s"), str.c_str() );
             break;
        case LOG_ERROR:
            wxLogError(_T("%s"),  str.c_str() );
            break;
        default:
            wxLogMessage(_T("%s"), str.c_str() );

     }

     //delete tmp;
}

void Logger::saveConsoleAttrib()
{
    #if defined(__WXMSW__)
    HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);

    CONSOLE_SCREEN_BUFFER_INFO con_info;
    GetConsoleScreenBufferInfo( hConsole, &con_info );
    m_attrib = con_info.wAttributes;
    #endif
}

void Logger::setTextColor( const wxColor& color )
{
    #if defined (__WXMSW__)
    size_t attrib = 0;

    unsigned char red = color.Red();
    unsigned char green = color.Green();
    unsigned char blue = color.Blue();

    if ( red > 0 )
        attrib |= FOREGROUND_RED;

    if ( green > 0 )
        attrib |= FOREGROUND_GREEN;

    if ( blue > 0 )
        attrib |= FOREGROUND_BLUE;

    if ( ( red >= 128 ) || ( blue >= 128 ) || ( green >= 128 ) )
        attrib |= FOREGROUND_INTENSITY;

    setTextAttrib( attrib );
    #endif
}

void Logger::setTextAttrib( unsigned int attrib )
{
    #if defined(__WXMSW__)
    HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
    SetConsoleTextAttribute( hConsole, attrib );
    #endif
}

void Logger::logMilli(logType type, const wxString& str)
{
	wxString tmp;

#if defined (__WINCE_STANDARDSDK__)

    long time = timeGetTime();

	tmp = wxString::Format(_T("%i "),time) + str;

#else
    wxDateTime now = wxDateTime::UNow();
        //wxDateTime_t milli = now::GetMillisecond();
    wxString nowStr = now.Format(_T("%l"));
	tmp = nowStr + str;

#endif

	log(type, tmp);

}
