#include "ExpressionSQL.h"

#include "../DirHelper.h"
#include "../logger.h"
#include "../gaugedesc/GaugeDescHelper.h"
#include "../gaugedesc/GaugeDescManager.h"


wxSQLite3Database ExpressionSQL::db;

/// main

StringHashA ExpressionSQL::getFilters(bool orderByName)
{
    StringHashA result;
    result.Clear();
    try {
        result = _get(false,orderByName);
        return result;
    }
    catch (wxSQLite3Exception e)
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::getFilters() database error"));
        wxMessageBox( _T("ExpressionSQL::getFilters() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return result;
    }
}

StringHashA ExpressionSQL::getExpressions(bool orderByName)
{
    StringHashA result;
    result.Clear();
    try {
        result = _get(true,orderByName);
        return result;
    }
    catch (wxSQLite3Exception e)
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::getFilters() database error"));
        wxMessageBox( _T("ExpressionSQL::getFilters() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return result;
    }
}

wxString ExpressionSQL::getFilterByName( const wxString& filterName )
{
    wxString result;
    try {
        result = _getbyname( filterName, false );
        return result;
    }
    catch ( wxSQLite3Exception e )
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::getFilterByName() database error"));
        wxMessageBox( _T("ExpressionSQL::getFilterByName() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return result;
    }
}

wxString ExpressionSQL::getExpressionByName( const wxString& expressionName )
{
    wxString result;
    try {
        result = _getbyname( expressionName, true );
        return result;
    }
    catch ( wxSQLite3Exception e )
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::getExpressionByName() database error"));
        wxMessageBox( _T("ExpressionSQL::getExpresisonByName() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return result;
    }
}

bool ExpressionSQL::saveFilter(const wxString& filter_name, const wxString& filter)
{
    try {
        bool res = _save( filter_name, filter, false );
        return res;
    }
    catch (wxSQLite3Exception e)
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::saveFilter() database error"));
        wxMessageBox( _T("ExpressionSQL::saveFilter() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return false;
    }

}

bool ExpressionSQL::saveExpression(const wxString& expression_name, const wxString& expression)
{
    try {
        bool res = _save( expression_name, expression, true );
        return res;
    }
    catch (wxSQLite3Exception e)
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::saveExpression() database error"));
        wxMessageBox( _T("ExpressionSQL::saveExpression() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return false;
    }

}

bool ExpressionSQL::deleteExpression(const wxString& expression_name)
{
    checkDb();
    try
    {
        wxString query = _T("DELETE FROM expressions WHERE expression_name ='") + expression_name + _T("'");
        return (db.ExecuteUpdate( query ) > 0);
    }
    catch ( wxSQLite3Exception e )
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::deleteExpression() database error"));
        wxMessageBox( _T("ExpressionSQL::deleteExpression() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return false;
    }
}

wxArrayString ExpressionSQL::getSymbols()
{
    wxArrayString result;
    GaugeDescManager* gdmanager = GaugeDescHelper::GetGDMgr();
    SymbolGaugeDescArray arr;
    gdmanager->GetSymbolDescs(arr);

    unsigned count = arr.GetCount();

    for ( size_t i = 0; i< count; ++i )
    {
        result.Add(arr[i]->Name);
        //int id = wxNewId();
    }

    return result;
}

DoubleHashArray ExpressionSQL::getExpressionColor( const int& coloring_id )
{
    checkDb();
    DoubleHashArray result;
    result.Clear();

    try {
        wxString query = _T("SELECT value, color FROM expression_colors WHERE coloring_type = ");
        query << coloring_id;
        query += _T(" ORDER BY value ASC");

        wxSQLite3ResultSet res = db.ExecuteQuery( query );
        while ( res.NextRow() )
        {
            DoubleHash* h = new DoubleHash;
            double value = res.GetDouble( _T("value") );
            (*h)[_T("value")] = value;

            wxString colorS = _T("#") + res.GetString(_T("color"));
            wxColor color(colorS);
            int r = color.Red(), g = color.Green(), b = color.Blue();

            double dcolor = 256.0*256.0*(double)r + 256.0* (double)g + (double)b;
            (*h)[_T("color")] = dcolor;
            result.Add(h);
        }
        return result;
    }
    catch ( wxSQLite3Exception e )
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::getExpressionColor() database error"));
        wxMessageBox( _T("ExpressionSQL::getExpressionColor() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return result;
    }

}

DoubleHashArray ExpressionSQL::getExpressionColor( const wxString& expression )
{
    checkDb();
    DoubleHashArray result;
    result.Clear();
    try {
        wxString query = _T("SELECT value, color FROM expression_colors c JOIN expressions e ON ( e.coloring_type = c.coloring_type) WHERE e.expression = '") +
            expression + _T("' ORDER BY value ASC");

        wxSQLite3ResultSet res = db.ExecuteQuery( query );
        while ( res.NextRow() )
        {
            DoubleHash* h = new DoubleHash;
            double value = res.GetDouble( _T("value") );
            (*h)[_T("value")] = value;

            wxString colorS = _T("#") + res.GetString(_T("color"));
            wxColor color(colorS);
            int r = color.Red(), g = color.Green(), b = color.Blue();

            double dcolor = 256.0*256.0*(double)r + 256.0* (double)g + (double)b;
            (*h)[_T("color")] = dcolor;
            result.Add(h);
        }
        return result;
    }
    catch ( wxSQLite3Exception e )
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::getExpressionColor() database error"));
        wxMessageBox( _T("ExpressionSQL::getExpressionColor() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        return result;
    }
}

/// helpers


void ExpressionSQL::checkDb()
{
    openDb();

    if ( ! db.TableExists(_T("expressions")) )
    {
        wxString query = _T("CREATE TABLE expressions ( coloring_type INTEGER , expression_name VARCHAR(255) NOT NULL, expression TEXT, isexpression INTEGER DEFAULT '0' NULL)");
        try {
            db.ExecuteQuery( query );
        }
        catch ( wxSQLite3Exception e)
        {
            Logger::logf(LOG_MSG, _T("ExpressionSQL::checkDb() database error"));
            wxMessageBox( _T("ExpressionSQL::checkDb() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        }
    }

    if ( ! db.TableExists(_T("expression_colors")) )
    {
        wxString query = _T("CREATE TABLE expression_colors ( coloring_type INTEGER, value FLOAT, color VARCHAR(6) )");
        try {
            db.ExecuteQuery( query );
        }
        catch ( wxSQLite3Exception e )
        {
            Logger::logf(LOG_MSG, _T("ExpressionSQL::checkDb() database error"));
            wxMessageBox( _T("ExpressionSQL::checkDb() database error"), _T("Error"), wxOK | wxICON_EXCLAMATION );
        }
    }
}

void ExpressionSQL::openDb()
{
    if ( db.IsOpen() ) return;
    wxString dbfile = DirHelper::GetPath(dCONFIG) + _T("expressiondb.db");
    db.Open ( dbfile );
}

StringHashA ExpressionSQL::_get( bool isExpression, bool orderByName )
{
    checkDb();
    StringHashA result;
    result.Clear();

    wxString query;

    if ( orderByName )
        query = wxString::Format(_T("SELECT * FROM expressions WHERE isexpression = %d ORDER BY expression_name"), (int)isExpression);
    else
        query = wxString::Format(_T("SELECT * FROM expressions WHERE isexpression = %d ORDER BY expression"), (int)isExpression);

    wxSQLite3ResultSet res = db.ExecuteQuery( query );

    while ( res.NextRow() )
    {
        wxString ename = res.GetString( _T("expression_name") );
        wxString expr = res.GetString( _T("expression") );
        StringHash* h = new StringHash;
        (*h)[_T("name")] = ename;
        (*h)[_T("expr")] = expr;
        result.Add(h);
    }

    return result;

}

wxString ExpressionSQL::_getbyname(const wxString& name, bool isExpression )
{
    checkDb();
    wxString result = wxEmptyString;

    wxString query = wxString::Format(_T("SELECT * FROM expressions WHERE isexpression = %d AND expression_name = '") + name + _T("'"), (int)isExpression);

    wxSQLite3ResultSet res = db.ExecuteQuery( query );
    while ( res.NextRow() )
    {
        result = res.GetString( _T("expression") );
    }
    return result;
}

bool ExpressionSQL::_save( const wxString& expression_name, const wxString& expression, bool isExpression )
{
    checkDb();

    wxString query;

    if ( checkForExpression(expression_name) )
    {
        if ( isExpression )
            query = _T("UPDATE expressions SET isexpression=1, expression = '") + expression + _T("' WHERE expression_name = '") + expression_name + _T("'");
        else
            query = _T("UPDATE expressions SET isexpression=0, expression = '") + expression + _T("' WHERE expression_name = '") + expression_name + _T("'");
    }

    else
        query = wxString::Format(_T("INSERT INTO expressions (expression_name, expression, isexpression) VALUES ('") +
                expression_name + _T("','") + expression + _T("', %d)"), (int)isExpression);

    return ( db.ExecuteUpdate( query ) > 0 );
}

bool ExpressionSQL::checkForExpression(const wxString& expression_name)
{
    checkDb();
    try
    {
        wxString query = _T("SELECT * FROM expressions WHERE expression_name ='") + expression_name + _T("'");

        wxSQLite3ResultSet res = db.ExecuteQuery( query );
        if ( res.Eof() )
            return false;
        else
            return true;

    }
    catch ( wxSQLite3Exception e)
    {
        Logger::logf(LOG_MSG, _T("ExpressionSQL::checkForExpression() database error"));
    }
    return false;
}
