#include "GPSHelper.h"

#include <wx/arrimpl.cpp>
#include <wx/textfile.h>

#include <wx/filename.h>

#include "LogHistoryMgr.h"
#include "LogHistoryMgrHelper.h"

#include "../agg/wxAgg.h"

#include "../logger.h"

WX_DEFINE_OBJARRAY( GPSDataA )

void SIPR2GPSConverter::getBounds( double* minLat, double* maxLat, double* minLon, double* maxLon )
{
    /// TODO
    // lon 180??
    *minLat = m_minLat; *maxLat = m_maxLat;
    *minLon = m_minLon; *maxLon = m_maxLon;
}

void SIPR2GPSConverter::require()
{
    if ( m_isregistered )
        return;
    m_rtDataMgr->registerSymbol( _T("latitudeDec") );
    m_rtDataMgr->registerSymbol( _T("longitudeDec") );
    m_rtDataMgr->registerSymbol( _T("height") );
    m_rtDataMgr->registerSymbol( _T("gpsSpeed") );
    m_rtDataMgr->registerSymbol( _T("distanceSum") );

    m_isregistered = true;
}

void SIPR2GPSConverter::unregisterSymbols()
{
    if ( !m_isregistered )
        return;
    m_rtDataMgr->unregisterSymbol( _T("latitudeDec") );
    m_rtDataMgr->unregisterSymbol( _T("longitudeDec") );
    m_rtDataMgr->unregisterSymbol( _T("height") );
    m_rtDataMgr->unregisterSymbol( _T("gpsSpeed") );
    m_rtDataMgr->unregisterSymbol( _T("distanceSum") );

    m_isregistered = false;
}

void SIPR2GPSConverter::refresh( DoubleHash* table )
{
    GPSData _data;
    _data.latitude = (*table)[_T("latitudeDec")];
    _data.longitude = (*table)[_T("longitudeDec")];
    _data.height = (*table)[_T("height")];
    _data.speed = (*table)[_T("gpsSpeed")];
    _data.distance = (*table)[_T("distanceSum")];

    if ( _data.latitude < m_minLat ) m_minLat = _data.latitude;
    if ( _data.latitude > m_maxLat ) m_maxLat = _data.latitude;

    if ( _data.longitude < m_minLon ) m_minLon = _data.longitude;
    if ( _data.longitude > m_maxLon ) m_maxLon = _data.longitude;

    m_data.Add( _data );
}

GPSDataA SIPR2GPSConverter::getData()
{
    return m_data;
}

///

GPSDataA GPSHelper::m_gpsdata;
double GPSHelper::m_minLon, GPSHelper::m_minLat, GPSHelper::m_maxLon, GPSHelper::m_maxLat;

void GPSHelper::analyzeGPS( const wxString& filename, int exportType )
{
    LogHistoryMgr* mgr = new LogHistoryMgr ( *LogHistoryMgrHelper::getInstance() );

    mgr->emptyHistory();
    mgr->loadHistoryFromLog( filename );

    if ( ! mgr->hasGPSdata() )
    {
        delete mgr;
        return;
    }

    SIPR2GPSConverter conv;
    mgr->getRTDataMgr()->RegisterViewer(&conv);
    conv.SetDataMgr( mgr->getRTDataMgr() );
    conv.require();

    mgr->runGPSData();

    m_gpsdata = conv.getData();

    conv.UnRegisterViewer();

    wxFileName fname( filename );
    fname.SetExt( _T("kml") );

    switch ( exportType )
    {
        case -1: break;
        case 1: exportToGPX( fname.GetFullPath(), filename, filename );
                break;
        case 0:
        default:
                exportToKML( fname.GetFullPath(), filename, filename );
                break;
    }

    conv.getBounds( &m_minLat, &m_maxLat, &m_minLon, &m_maxLon );

    delete mgr;

    fname.SetExt( _T("png") );

    sketchRoute( fname.GetFullPath() );
}


void GPSHelper::calculateGPSData()
{
    m_minLat = m_minLon = 0.0;
    m_maxLat = m_maxLon = 0.0;

    m_gpsdata.Empty();

    LogHistoryMgr* mgr = new LogHistoryMgr ( *LogHistoryMgrHelper::getInstance() );

    if ( ! mgr->hasGPSdata() )
    {
        delete mgr;
        return;
    }

    SIPR2GPSConverter conv;
    mgr->getRTDataMgr()->RegisterViewer(&conv);
    conv.SetDataMgr( mgr->getRTDataMgr() );
    conv.require();

    mgr->runGPSData();

    m_gpsdata = conv.getData();

    conv.UnRegisterViewer();

    conv.getBounds( &m_minLat, &m_maxLat, &m_minLon, &m_maxLon );

    delete mgr;
}

GPSDataA GPSHelper::getGPSData()
{
    return m_gpsdata;
}

void GPSHelper::getBounds( double* minLat, double* maxLat, double* minLon, double* maxLon )
{
    *minLat = m_minLat; *maxLat = m_maxLat;
    *minLon = m_minLon; *maxLon = m_maxLon;
}

void GPSHelper::exportToGPX( const wxString& filename, const wxString& name, const wxString& descr  )
{
    wxString pName = ( name == wxEmptyString ) ? _T("path") : name;
    wxString pDescr = ( descr == wxEmptyString ) ? _T("") : descr;

    wxTextFile gpxfile( filename );

    if ( gpxfile.Exists() )
    {
        gpxfile.Open();
        gpxfile.Clear();
    }
    else
        gpxfile.Create();

    gpxfile.AddLine( _T("<?xml version=\"1.0\"?>") );
    gpxfile.AddLine( _T("<gpx creator=\"VemsTune\" version=\"1.0\" xmlns=\"http://www.topografix.com/GPX/1/0\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd\">") );
    gpxfile.AddLine( _T("<trk>") );
    gpxfile.AddLine( _T("  <name>") + pName + _T("</name>") );
    gpxfile.AddLine( _T("  <trkseg>") );

    for ( size_t i = 0; i < m_gpsdata.GetCount(); ++i )
    {
        GPSData _data = m_gpsdata[i];
        wxString coord = wxString::Format( _T("    <trkpt lat =\"%f\" lon=\"%f\">"), _data.latitude, _data.longitude );
        gpxfile.AddLine( coord );
        coord = wxString::Format( _T("      <ele>%.1f</ele>"), _data.height );
        gpxfile.AddLine( coord );
        gpxfile.AddLine(_T("    </trkpt>") );
    }

    gpxfile.AddLine( _T("  </trkseg>") );
    gpxfile.AddLine( _T("</trk>") );
    gpxfile.AddLine( _T("</gpx>") );

    gpxfile.Write();

    gpxfile.Close();
}

void GPSHelper::exportToKML( const wxString& filename, const wxString& name, const wxString& descr )
{
    wxString pName = ( name == wxEmptyString ) ? _T("path") : name;
    wxString pDescr = ( descr == wxEmptyString ) ? _T("") : descr;

    wxTextFile kmlfile( filename );

    if ( kmlfile.Exists() )
    {
        kmlfile.Open();
        kmlfile.Clear();
    }
    else
        kmlfile.Create();

    // header
    kmlfile.AddLine( _T("<?xml version=\"1.0\" encoding=\"UTF-8\"?>") );
    kmlfile.AddLine( _T("<kml xmlns=\"http://www.opengis.net/kml/2.2\">") );
    kmlfile.AddLine( _T("  <Document>") );
    kmlfile.AddLine( _T("    <name>") + pName + _T("</name>") );
    kmlfile.AddLine( _T("    <description>") + pDescr + _T("</description>") );
    kmlfile.AddLine( _T("    <Style id=\"e_linestyle\">") );
    kmlfile.AddLine( _T("      <LineStyle>") );
    kmlfile.AddLine( _T("        <color>7f00ffff</color>") );
    kmlfile.AddLine( _T("        <width>4</width>") );
    kmlfile.AddLine( _T("      </LineStyle>") );
    kmlfile.AddLine( _T("      <PolyStyle>") );
    kmlfile.AddLine( _T("        <color>7f00ff00</color>") );
    kmlfile.AddLine( _T("      </PolyStyle>") );
    kmlfile.AddLine( _T("    </Style>") );
    kmlfile.AddLine( _T("    <Placemark>") );
    kmlfile.AddLine( _T("      <name>") + pName + _T("</name>") );
    kmlfile.AddLine( _T("      <description>") + pDescr + _T("</description>") );
    kmlfile.AddLine( _T("      <styleUrl>#e_linestyle</styleUrl>") );
    kmlfile.AddLine( _T("      <LineString>") );
    kmlfile.AddLine( _T("        <extrude>1</extrude>") );
    kmlfile.AddLine( _T("        <tessellate>1</tessellate>") );
    kmlfile.AddLine( _T("        <altitudeMode>absolute</altitudeMode>") );
    kmlfile.AddLine( _T("        <coordinates>") );

    for ( size_t i = 0; i < m_gpsdata.GetCount(); ++i )
    {
        GPSData _data = m_gpsdata[i];
        wxString coord = _T("          ") + wxString::Format( _T("%f,%f,%.1f"), _data.longitude, _data.latitude, _data.height );
        kmlfile.AddLine( coord );
    }

    kmlfile.AddLine( _T("        </coordinates>") );
    kmlfile.AddLine( _T("      </LineString>") );
    kmlfile.AddLine( _T("    </Placemark>") );
    kmlfile.AddLine( _T("  </Document>") );
    // footer
    kmlfile.AddLine( _T("</kml>") );

    kmlfile.Write();

    kmlfile.Close();
}

void GPSHelper::sketchRoute( const wxString& filename )
{
    double w, h;
    double x, y;
    double latitude, longitude;
    double left, right, top, bottom;

    RealPointArray path;

    getBounds( &top, &bottom, &left, &right );

    if ( ( bottom - top == 0 ) || ( m_gpsdata.GetCount() == 0 ) )
        return;

    double r = fabs( right - left ) / fabs( bottom - top );

    w = 200;
    if ( w / r > 150 )
    {
        h = 150; w = h * r;
    }
    else if ( 150 * r > w )
    {
        w = 200; h = w / r;
    }
    else h = w / r;

    double padding = 0.1 * w;

    wxAgg aggCanvas( (int) (w + 2*padding), (int) (h + 2*padding) );

    aggCanvas.setBrush( wxColor( 141, 197,108 ) );
    aggCanvas.setPen( wxColor( 101, 157,88 ) );
    aggCanvas.drawFilledRectangle( 0, 0, w + 2*padding, h+2*padding, 1.5 );


    for ( size_t i = 0; i < m_gpsdata.GetCount(); ++i )
    {
        GPSData _data = m_gpsdata[i];
        latitude = _data.latitude;
        longitude = _data.longitude;


        x = fabs( longitude - left ) / fabs( right - left ) * w + padding;
        y = h - ( fabs( latitude - top ) / fabs( bottom - top ) * h ) + padding;

        path.Add( wxRealPoint( x, y ) );
    }


    aggCanvas.setPen( wxColor(255, 255, 255) );
    aggCanvas.drawLine( path, 3.2 );
    aggCanvas.setPen( wxColor(128, 0, 0) );
    aggCanvas.drawLine( path, 2.0 );

    wxBitmap* bmp = aggCanvas.renderToBitmap();

    bmp->SaveFile( filename , wxBITMAP_TYPE_PNG );

    delete bmp;
    return;
}
