#include "LogHistoryMgr.h"

#include "GPSHelper.h"

#include "../vemsprotocol/SIPRPacket.h"
#include "../vemsprotocol/SIPRTypes.h"

#include "../rtdatamanager/RTDataMgrFacade.h"
#include "../stdatamanager/STDataMgrFacade.h"

#include "../rtdatamanager/RT2ViewerConverter.h"
#include "../rtdatamanager/RTDataInputSIPR.h"

#include "../logger.h"
#include "../MTConfigHelper.h"

#include "../expression/RPNHelper.h"
#include "../expression/RPNParser.h"
#include "../expression/VariableData.h"
#include "LogFilter.h"

#include "../common/FunctionMeter.h"

#include <wx/progdlg.h>

#ifndef SIZE_MAX
	#define SIZE_MAX 0xffffffff
#endif

unsigned LogHistoryMgr::m_counter = 0;

LogHistoryMgr::LogHistoryMgr(RTDataMgrBase* rtMgr, STDataMgrBase* stMgr, SymbolMgr* symbolMgr)
:LogHistoryBase(rtMgr, stMgr, symbolMgr)
{
    //ctor
    m_converter = new RT2ViewerConverter(symbolMgr, &m_viewerModel );
    m_vemslogbuffer = new VEMSLogBuffer();
    m_fromlog = false;
    m_oldidx = 0;
    m_logversion = 0;
    m_headerHash.clear();

    m_stdatamodel = NULL;
    m_prevGPSPtr = NULL;

    m_packetdelay = 0;
    m_counter++;

    m_gpsDistance.Clear();
    m_gpscalculated = false;
    m_canSliderEnabled = false;

    m_currentIdx = -1;

    m_logFileName = _T("");

    m_st_belonging2rtidx = -1;
}

LogHistoryMgr::LogHistoryMgr( const LogHistoryMgr& mgr ) : LogHistoryBase(NULL,NULL,NULL)
{
    RTDataMgrFacade* rtfacade = new RTDataMgrFacade();

    RTDataInputSIPR* inputSIPR = new RTDataInputSIPR( mgr.getSymbolMgr(), &m_viewerModel );

    rtfacade->registerInput( inputSIPR );
    rtfacade->selectInput( INPUT_TYPE_SIPR );

    m_rtMgr = rtfacade;
    m_stMgr = new STDataMgrFacade();
    //m_stMgr = mgr.m_stMgr;
    m_symbolMgr = mgr.getSymbolMgr();

    m_st_belonging2rtidx = -1;

    // copy required DH (not needed?)

    /*
    DoubleHash dh = mgr.m_rtMgr->getRequiredDH();

    for ( DoubleHash::iterator it = dh.begin(); it != dh.end(); ++it )
    {
        m_rtMgr->registerSymbol( it->first );
        Logger::logf(LOG_MSG, _T("---> ") + it->first );
    }
    */

    m_converter = NULL;

    m_vemslogbuffer = mgr.m_vemslogbuffer;
    m_stdatamodel = NULL;
    m_prevGPSPtr = NULL;
    m_counter++;

    m_gpsDistance.Clear();
    m_gpscalculated = false;
    m_canSliderEnabled = true;
    m_fromlog = mgr.m_fromlog;

    m_oldidx = mgr.m_oldidx;
    m_logversion = mgr.m_logversion;
    m_headerHash = mgr.m_headerHash;

    m_logFileName = mgr.m_logFileName;

    m_currentIdx = mgr.m_currentIdx;

    // initialize ST
    initST( *MTConfigHelper::getMTConfigData()->getSIPRDescriptor() );
}

LogHistoryMgr::~LogHistoryMgr()
{
    //dtor
    if ( m_vemslogbuffer != NULL && m_counter == 1 )
    {
        delete m_vemslogbuffer;
        m_vemslogbuffer = NULL;
    }

    if ( m_converter != NULL )
    {
        delete m_converter;
        m_converter = NULL;
    }

    if ( m_stdatamodel ) { delete m_stdatamodel; }
    Logger::logf(LOG_MSG, _T("LogHistoryMgr destructor called"));

    purgeCache();

    --m_counter;
}

void LogHistoryMgr::clear()
{
    delete m_stMgr;
    delete m_rtMgr;

    m_gpsDistance.Clear();
    m_gpscalculated = false;
    m_rt_idx = -1;
    m_rt_time = -1;
    m_currentIdx = -1;

    m_st_belonging2rtidx = -1;

    m_logFileName = wxEmptyString;

    purgeCache();
}

void LogHistoryMgr::purgeCache()
{
    WX_CLEAR_HASH_MAP( Int2DoubleHashHash, m_cache )
    Logger::logf( LOG_MSG, _T("LogHistoryMgr::purgeCache() - cached purged") );
}

void LogHistoryMgr::initST(const SIPRDescriptor& siprDesc)
{
    m_viewerModel.clear();
    RPNHelper::setUsedStatic(*m_symbolMgr);
    m_symboltable = m_symbolMgr->getUsedStaticAndBit();

    if ( !m_stdatamodel )
        m_stdatamodel = new STDataModelSerial( m_symboltable, &siprDesc );
    else
    {
        //delete m_stdatamodel;
    }
}

void LogHistoryMgr::calcgpsdist()
{
    if ( hasGPSdata() )
    {
        DoubleHashArray arr;
        getGPSData( arr );

        for ( size_t i = 0; i < arr.GetCount(); ++i )
        {
            DoubleHash* h = arr[i];
            m_gpsDistance.Add( (*h)[_T("distanceSum")] );
        }
        WX_CLEAR_ARRAY( arr );
    }

    m_gpscalculated = true;
}

void LogHistoryMgr::calculateModel(size_t idx, DoubleHash** data)
{

    // static int rec_level = 0;

    // Logger::logf( LOG_MSG, _T("Recursion level = %d"), rec_level );

    if ( m_gpscalculated == false )
    {
        calcgpsdist();
    }

    RPNParser parser;
    Int2ArrayStringHash pvars;

    // iterate on data: searching for and fill previous values
    DoubleHash::iterator it = (**data).begin();
    for ( ; it != (**data).end(); ++it )
    {
        Symbol* sym = m_symbolMgr->getSymbol( it->first );

        // if required symbol doesnot exist in symboltable/INI -> continue
        if ( !sym )
        {
            continue;
        }

        wxString expr = sym->getExpr();

        // does the expression depend on previous values? - if no continue
        if ( expr.Find('[') == wxNOT_FOUND )
            continue;
        pvars = parser.getPVariables( expr );

        // iterate on 'time'
        for ( Int2ArrayStringHash::iterator timeit = pvars.begin(); timeit != pvars.end(); ++timeit )
        {
            wxArrayString vars = timeit->second;

            int shift = timeit->first;

            if ( shift == 0 )
            {
                continue;   // current value will be calculated without recursion
            }

            DoubleHash* dh = new DoubleHash;
            for ( size_t n = 0; n < vars.GetCount(); ++n )
            {
                (*dh)[vars[n]] = 0;
            }

            // rec_level++;

            calculateModel( idx + shift, &dh );

            for ( size_t n = 0; n < vars.GetCount(); ++n )
            {
                // cheat: 'register' symbol (eg. rpm[-1]) & add to DH
                wxString reqS = vars[n] + wxString::Format(_T("[%d]"), shift);
                (**data)[reqS] = (*dh)[vars[n]];
            }

            wxString reqS = wxString::Format( _T("time[%d]"), shift );
            (**data)[reqS] = (*dh)[_T("time")];

            delete dh;
        }
    }

    // update static values
    updateConfigModel(idx);
    updateViewerModelWithConfig();

    ///
    /// !!! CACHING TEMPORARILY REMOVED
    ///
    /*
    /// CACHING
    // check if all required data already exists in cache
    bool foundMissing = false;

    if ( m_cache.find( idx ) != m_cache.end() )
    {
        for ( DoubleHash::iterator it = (*data)->begin(); it != (*data)->end(); ++it )
        {
            if ( m_cache[idx]->count( it->first ) == 0 )
            {
                foundMissing = true;
                delete m_cache[idx];
                break;
            }
        }
        if ( foundMissing == false && idx >= 0 && idx < getHistoryCount() )
        {
            delete *data;
            *data = new DoubleHash(*m_cache[idx]);
            return;
        }
    }
    */
    /// <--- CACHING

    int gpsidx;

    SIPRPacket* packet = m_vemslogbuffer->getPacketByType( idx, IDX_RTDATA_NORMAL );
    SIPRPacket* gpspacket = m_vemslogbuffer->getGPSData( idx, &gpsidx );

    if ( packet == NULL )
    {
        Logger::logf(LOG_MSG, _T("Packet NULL!"));
    }
    else
    {
        (**data)[_T("time")] = m_vemslogbuffer->getPacketTime( idx );
        (**data)[strLogIdxPos] = (double)(idx);

        // calculate GPS data
        if ( gpspacket != NULL )
            m_converter->convert( *gpspacket, **data );
        // and other RT values
        m_converter->convert(*packet, **data);

        if ( gpsidx > 0 )
        {
            (**data)[_T("distanceSum")] = m_gpsDistance[gpsidx];
        }

        // add data to cache
        /// !! CACHING
        /*DoubleHash* cacheData = new DoubleHash(**data);
        m_cache[idx] = cacheData;*/
        /// <--- CACHING
    }

//    if ( rec_level>0) rec_level--;
//    Logger::logf( LOG_MSG, _T("rec.level=%d"), rec_level );
}

void LogHistoryMgr::getHistory(const IntArray& idxA, DoubleHashArray& viewerData)
{
    //FUNCTIME
    const DoubleHash& requiredH = m_rtMgr->getRequiredDH();

    m_converter->refreshDependency(requiredH);

    IntArray::const_iterator it;
    for ( it = idxA.begin(); it != idxA.end(); ++it )
    {
        size_t idx = *it;
        DoubleHash* data = new DoubleHash(requiredH);
        calculateModel(idx, &data);
        viewerData.Add(data);
    }
}

void LogHistoryMgr::analyzeHistory()
{
    m_fromlog = true;
    long old_stidx = -1;
    for ( size_t i = 0; i < getHistoryCount(); ++i )
    {
        long stidx = m_vemslogbuffer->getSTIndexByRTIdx( i );
        if ( ( stidx > -1 ) &&  ( stidx > old_stidx ) )
        {
        // update ST
        updateConfigModel( i );
        updateViewerModelWithConfig();

            old_stidx = stidx;
        }

        SIPRPacket* packet = m_vemslogbuffer->getPacketByType( i , IDX_RTDATA_NORMAL );

        if ( packet == NULL )
            continue;

        m_rt_idx = i;
        m_rt_time = m_vemslogbuffer->getPacketTime( i );

        process( *packet );
        observerNotify(i, this);
    }
}

void LogHistoryMgr::runGPSData()
{
    m_fromlog = true;
    for ( size_t i = 0; i < getHistoryCount( IDX_RTDATA_GPS ); ++i )
    {
        SIPRPacket* packet = m_vemslogbuffer->getPacketByType( i , IDX_RTDATA_GPS );

        if ( packet == NULL )
            continue;

        m_rt_idx = i;
        m_rt_time = m_vemslogbuffer->getPacketTime( i );

        process( *packet );
    }
}

void LogHistoryMgr::getHistory(size_t startIdx, size_t endIdx, DoubleHashArray& viewerA)
{
    //FUNCTIME

    const DoubleHash& requiredH = m_rtMgr->getRequiredDH();

    m_converter->refreshDependency(requiredH);

    size_t hist_count = getHistoryCount();
    if( startIdx >= hist_count || endIdx >= hist_count ) return;
    int buflen = ( endIdx - startIdx + 1 );

    for ( size_t i = startIdx; i <= endIdx;++i )
    {
        DoubleHash* data = new DoubleHash(requiredH);
        calculateModel(i, &data);
        viewerA.Add(data);
        // notify observer (%)
        int position = ( i - startIdx + 1);
        int percent = (int)( ( (double) position * 100.0 )/ (double) buflen );

        observerNotify(percent);
    }
}
/// ProcessFilter - DoubleHashArray - ver.2

void LogHistoryMgr::processFilter(size_t startIdx, size_t endIdx)
{
//    FunctionMeter fm(_T("LogHistoryMgr::processFilter"));
//    Logger::logf(LOG_MSG, _T("processfilter: start=%i,end=%i"),startIdx,endIdx);
    int temp = m_currentIdx;

    refresh(startIdx,false);

    m_currentIdx = temp;

    m_rtMgr->requireChanged();
    m_rtMgr->getRequiredSymbols();
    const DoubleHash& requiredH = m_rtMgr->getRequiredDH();
	Logger::logf(LOG_MSG, _T("LogHistoryMgr::processFilter() -  requiredH.size() = %d"), requiredH.size() );
    m_converter->refreshDependency(requiredH);

    int buflen = ( endIdx - startIdx + 1);
//    Logger::logf(LOG_MSG, _T("processfilter: start bufflen: %i"), buflen);

    RPNParser rpn;
    //DoubleHash data(requiredH);
    DoubleHashArray dataArr;
    IntArray    indexArray;
    VariableDataDH vardata;

    // pass1 - pre-calculate all values - if it is not calculated yet
    int prev_precent = 0;

    observerNotify(0);

    for ( size_t i = startIdx; i <= endIdx;++i )
    {
        if(m_isStopProcessFlag) {
            m_isStopProcessFlag = false;
            break;
        }

        DoubleHash* pdata = new DoubleHash(requiredH);
        vardata = VariableDataDH( pdata );
        rpn.setVariableData( &vardata );

        calculateModel(i, &pdata);

        dataArr.Add( pdata );
        indexArray.Add(i);

        // notify observer (%)
        int position = ( i - startIdx + 1);
        int percent = (int)( ( (double) position * 100.0 )/ (double) buflen );

        if(percent != prev_precent)
            observerNotify(percent);

        prev_precent = percent;
    }

    // pass2 - filtering
    for ( size_t i = 0; i < indexArray.GetCount(); ++i ){
        //Logger::logf(LOG_MSG, _T("LogHistoryMgr::processFilter(): i= %i"), i);

        int packetIdx = indexArray.Item(i);


        for ( size_t filters = 0; filters < m_logfilterA.GetCount(); ++filters )
        {
            DoubleHash data_ = *(dataArr[i]);
            // lambda delay
            if ( m_packetdelay != 0 )
            {
                size_t idx = findPacketByDelay( packetIdx, m_packetdelay );
                // overwrite current lambda
                if ( idx != SIZE_MAX )
                {
                    DoubleHash delayed = *(dataArr[idx]);
                    data_[_T("lambda")] = delayed[_T("lambda")];

//                    Logger::logf(LOG_MSG,_T("idx = %d (time=%.2f), delayed lambda idx = %d (time=%.2f)"), i, data_[_T("time")], idx, delayed[_T("time")]);
                }
            }

            vardata = VariableDataDH( &data_ );
            rpn.setVariableData( &vardata );

            double cond = 1.0;
            wxString filter = m_logfilterA[filters]->getFilter();
            if (  filter != wxEmptyString )
                cond = rpn.calculate(filter);

            wxString expression = m_logfilterA[filters]->getExpression();
            if( expression != wxEmptyString){
                double value = rpn.calculate( expression );
                data_[expressionValue] = value;
            }
            bool passedFilter = ( cond > 0 );

            m_logfilterA[filters]->processData(data_,passedFilter);

            //Logger::logf(LOG_MSG,_T("rpm=%0.f, passedFilter=%i, i=%i, rtidx=%f"),data_[_T("rpm")], passedFilter, i, data_[_T("rtidx")]);
        }
    }
     observerNotify(100);
    //Logger::logf(LOG_MSG, _T("LogHistoryMgr::processFilter() 300"));
    WX_CLEAR_ARRAY( dataArr );
}

void LogHistoryMgr::emptyHistory()
{
    if ( m_vemslogbuffer != NULL )
        m_vemslogbuffer->clearBuffer();
    else m_vemslogbuffer = new VEMSLogBuffer();

    if ( m_converter != NULL )
        delete m_converter;

    m_converter = new RT2ViewerConverter( m_symbolMgr, &m_viewerModel );

    m_fromlog = false;
    m_logFileName = wxEmptyString;
    m_canSliderEnabled = false;
    m_prevGPSPtr = NULL;
    m_packetdelay = 0;
    m_currentIdx = -1;
    observerNotify( -2, 0 );
    m_st_belonging2rtidx = -1;
}

size_t LogHistoryMgr::getHistoryCount( size_t pType )
{
    // SIPR_READ_SENSORS:
    // - SIPR_RT_MT_NORMAL, SIPR_RT_MT_EXT
    return m_vemslogbuffer->getHistoryCount( pType );
}

void LogHistoryMgr::refreshLastPosition()
{
    if ( m_currentIdx >= 0 )
        refresh( m_currentIdx, false );
}

void LogHistoryMgr::refresh(size_t idx, bool notifySlider)
{
    //FunctionMeter fm(_T("LogHistoryMgr::refresh"));
    if ( m_gpscalculated == false )
    {
        calcgpsdist();
    }


    wxObjectArray siprpackets;

    long stidx = m_vemslogbuffer->getSTIndexByRTIdx( idx );

//    Logger::logf(LOG_MSG, _T("stidx=%d"), stidx);

    if ( stidx != -1 && stidx != m_st_belonging2rtidx )
    {

    m_vemslogbuffer->getSTData(idx, siprpackets);

    // send SIPR_LOG_MARKER/SIPR_REFRESH_START packet before send pages ( disable STDataMgrFacade updateViewers )
    if ( siprpackets.GetCount() > 0 ) {
        uint8_t packetData = SIPR_REFRESH_START;
        SIPRPacket packet( 0, SIPR_LOG_MARKER, 1, &packetData );
        ((STDataMgrFacade*)m_stMgr)->process( packet );
    }

    for ( size_t i = 0; i < siprpackets.GetCount(); ++i )
    {
        SIPRPacket* packet = (SIPRPacket*) siprpackets[i];
        ((STDataMgrFacade*)m_stMgr)->process( *packet );
    }

    // send SIPR_LOG_MARKER/SIPR_REFRESH_STOP packet after send pages ( enable STDataMgrFacade updateViewers )
    if ( siprpackets.GetCount() > 0 ) {
        uint8_t packetData = SIPR_REFRESH_STOP;
        SIPRPacket packet( 0, SIPR_LOG_MARKER, 1, &packetData );
        ((STDataMgrFacade*)m_stMgr)->process( packet );
    }
        //LOG
        m_st_belonging2rtidx = stidx;
    }

    m_vemslogbuffer->setPosition( m_vemslogbuffer->getIdxByDIdx( idx, IDX_RTDATA_NORMAL ) );

    int gpsidx;

    SIPRPacket* packet = m_vemslogbuffer->getPacket();
    SIPRPacket* gpspacket = m_vemslogbuffer->getGPSData( idx, &gpsidx );

    if ( packet == NULL )
        return;

    switch ( packet->type() )
    {
        case SIPR_READ_SENSORS:
        case SIPR_READ_MEMORY_32:
        {
            int subType = 0;

            if ( packet->type() == SIPR_READ_SENSORS ) subType = packet->getSubType();
            if ( packet->type() == SIPR_READ_MEMORY_32 ) subType = SiprSensorType::SIPR_READ_MEMORY_32;

            switch ( subType )
            {
                case SiprSensorType::SIPR_RT_MT_NORMAL:
                case SiprSensorType::SIPR_RT_MT_EXT:
                case SiprSensorType::SIPR_RT_ROUND:
                case SiprSensorType::SIPR_READ_MEMORY_32:
                {
                    ((RTDataMgrFacade*)m_rtMgr)->process( *packet );

                    DoubleHash& dh = m_rtMgr->getRequiredDH_RW();
                    if ( gpsidx > -1 )
                    {
                        dh[_T("distanceSum")] = m_gpsDistance[gpsidx];
                    }

                    //m_rtMgr->notifyViewers();
                    break;
                }
                default:
                    break;
            }
            break;
        }
        default:
            break;
    }

    // GPS
    if ( gpspacket != NULL && gpspacket != m_prevGPSPtr )
    {
        ((RTDataMgrFacade*)m_rtMgr)->process( *gpspacket );
        DoubleHash& dh = m_rtMgr->getRequiredDH_RW();
        if ( gpsidx > -1 )
        {
            dh[_T("distanceSum")] = m_gpsDistance[gpsidx];
        }


        m_prevGPSPtr = gpspacket;
    }
    m_rtMgr->notifyViewers();
    // dont delete it!!
    siprpackets.clear();

    if ( notifySlider )
    {
        observerNotify( idx, getHistoryCount() );
        observerNotify( (int)(m_vemslogbuffer->getPacketTime(idx)), this );
    }

    m_currentIdx = idx;
}

bool LogHistoryMgr::loadHistoryFromLog(const wxString& filename)
{
    purgeCache();

    m_rtMgr->reset();

    bool ret = m_vemslogbuffer->loadLogFromFile(filename, m_headerHash, m_logversion );
    m_fromlog = true;
    m_logFileName = filename;

    m_canSliderEnabled = true;

    observerNotify( -1, getHistoryCount()-1 );

    double mtime = m_vemslogbuffer->getPacketTime( getHistoryCount()-1 );
    wxString strTime = wxString::Format( _T("%.2f"), mtime );
    observerNotify( strTime );

    m_gpsDistance.Clear();
    m_gpscalculated = false;

    m_currentIdx = 0;

    return ret;
}

void LogHistoryMgr::process(const SIPRPacket& p)
{
    switch ( p.type() )
    {
        case SIPR_READ_SENSORS:
        case SIPR_READ_MEMORY_32:
        {
            if ( !m_fromlog )
            {
                m_vemslogbuffer->addPacket(p);
            }

            if( !((RTDataMgrFacade*)m_rtMgr)->process( p ) ) return;

            DoubleHash& dh = m_rtMgr->getRequiredDH_RW();
            if ( m_rt_time >= 0 )
            {
                dh[_T("time")] = m_rt_time;
            }
            if ( m_rt_idx >= 0 )
            {
                dh[strLogIdxPos] = m_rt_idx;
            }

//            DoubleHash& dh = m_rtMgr->getRequiredDH_RW();
//            dh[_T("rpm")] = rand() * 3000.0 / RAND_MAX;

            m_rtMgr->filteredNotifyViewers();

            break;
        }
        case SIPR_GET_TABLEDATA:
        case SIPR_GET_CONFIG_1:
        case SIPR_GET_CONFIG_2:
        case SIPR_READ_MEMORY:
            if ( !m_fromlog ) m_vemslogbuffer->addPacket(p);
            ((STDataMgrFacade*)m_stMgr)->process( p );
            break;

        case SIPR_LOG_MARKER:
            if ( !m_fromlog ) m_vemslogbuffer->addPacket(p);
            ((STDataMgrFacade*)m_stMgr)->process( p );
            break;

        case SIPR_AIM:
            if ( !m_fromlog ) m_vemslogbuffer->addPacket(p);

            if( !((RTDataMgrFacade*)m_rtMgr)->process( p ) ) return;

            m_rtMgr->filteredNotifyViewers();

            break;
    }
}

/**
File log Slider invoke this!
*/
void LogHistoryMgr::notify(int i)
{
    int position = (int)((double)i / 100.0 * (double)m_vemslogbuffer->getHistoryCount(IDX_RTDATA_NORMAL));

    if ( m_fromlog )
        position = i;

//    Logger::logf(LOG_MSG, _T("LogHistory::notify() RTidx = %d"), position);
    refresh( position,false );
}

size_t LogHistoryMgr::getNextPacket(SIPRPacket** packet)
{
    // get next packet
    SIPRPacket * temp = m_vemslogbuffer->getNextPacket();
    // calc sleep time
    size_t tsleep = 5;
    if ( temp == NULL )
    {
        *packet = NULL; return tsleep;
    }

    size_t position = m_vemslogbuffer->getPosition()-1;
    if ( m_oldidx+1 >= getHistoryCount() ) m_oldidx = 0;

    if( temp->type() == SIPR_READ_SENSORS)
    {
        switch ( temp->getSubType() )
        {
            case SiprSensorType::SIPR_RT_MT_NORMAL:
            case SiprSensorType::SIPR_RT_MT_EXT:
            case SiprSensorType::SIPR_RT_ROUND:
            {
                size_t idx = m_vemslogbuffer->findRTidxByIdx( position, m_oldidx );
                double ntime = m_vemslogbuffer->getPacketTime( idx + 1 );
                if (ntime == -1) tsleep = 80;
                else {
                    double ttime = m_vemslogbuffer->getPacketTime(idx);
                    tsleep = (size_t)((ntime - ttime) * 1000.0);
                }
                m_oldidx = (idx>=0)?idx:m_oldidx;
                // notify slider
                observerNotify(idx, getHistoryCount());

                break;
            }

            default:
                tsleep = 80;
                break;
        }
    }
    *packet = temp;
    return tsleep;
}

size_t LogHistoryMgr::getPacket( size_t idx, SIPRPacket** packet )
{
// get next packet
    SIPRPacket * temp = m_vemslogbuffer->getPacket( idx );
    // calc sleep time
    size_t tsleep = 5;
    if ( temp == NULL )
    {
        *packet = NULL; return tsleep;
    }

    if( temp->type() == SIPR_READ_SENSORS)
    {
        switch ( temp->getSubType() )
        {
            case SiprSensorType::SIPR_RT_MT_NORMAL:
            case SiprSensorType::SIPR_RT_MT_EXT:
            case SiprSensorType::SIPR_RT_ROUND:
            {
                long rtidx = m_vemslogbuffer->findRTidxByIdx( idx );
                if ( rtidx < 0 )
                {
                    tsleep = 80;
                    break;
                }
                double ntime = m_vemslogbuffer->getPacketTime( rtidx + 1 );
                double ttime = -1;
                if ( ntime < 0 ) tsleep = 80;
                else {
                    ttime = m_vemslogbuffer->getPacketTime(rtidx);
                    tsleep = (size_t)((ntime - ttime) * 1000.0);
                }
                // notify slider
                observerNotify(rtidx, getHistoryCount());
                if ( ttime > 0 )
                    observerNotify( (int)ttime, this );
                break;
            }

            default:
                tsleep = 80;
                break;
        }
    }
    *packet = temp;
    return tsleep;
}

void LogHistoryMgr::updateConfigModel(size_t idx)
{
    wxObjectArray stpackets;
    m_vemslogbuffer->getSTData( idx, stpackets );
    for ( size_t j = 0; j < stpackets.GetCount(); ++j ){
        SIPRPacket* p = (SIPRPacket*)stpackets[j];
        m_stdatamodel->process(*p);
    }
    stpackets.clear();
}

void LogHistoryMgr::updateViewerModelWithConfig()
{
    const SymbolTable& m_usedConfigHash = m_symbolMgr->getUsedStaticAndBit();
    SymbolTable::const_iterator it = m_usedConfigHash.begin();
    for ( ; it != m_usedConfigHash.end(); ++it )
    {
        Symbol* sym = it->second;
        m_viewerModel[sym->getName()] = m_stdatamodel->getViewerData(*sym);
    }
}


size_t LogHistoryMgr::findPacketByDelay( size_t startIdx, long delay )
{
    if ( delay == 0 )
        return startIdx;

    double ddelay = (double)delay / 1000.0;

    size_t hist_count = getHistoryCount();
    if ( startIdx < 0 || startIdx >= hist_count )
        return SIZE_MAX;
    else
    {
        double startTime = m_vemslogbuffer->getPacketTime( startIdx ), dtime = startTime + ddelay;
        double prevdiff = fabsf( dtime - startTime );

        if ( delay > 0 )
        {
            for ( size_t i = startIdx + 1; i < hist_count; ++i )
            {
                double ptime = m_vemslogbuffer->getPacketTime( i );
                double diff = fabsf( ptime - dtime );
                if ( ptime >= dtime )
                {
                    if ( prevdiff < diff )
                        return ( i - 1 );
                    else
                        return i;
                }
                prevdiff = diff;
            }
            return SIZE_MAX;
        }
        else
        {
            size_t i = startIdx - 1;
            while ( i > 0 )
            {
                double ptime = m_vemslogbuffer->getPacketTime( i );
                double diff = fabsf( ptime - dtime );

                if ( ptime <= dtime )
                {
                    if ( prevdiff > diff )
                        return i;
                    else
                        return (i + 1);
                }

                prevdiff = diff;
                --i;
            }
            return SIZE_MAX;
        }
    }
}

void LogHistoryMgr::setPacketDelay( long delay )
{
    m_packetdelay = delay;
}

size_t LogHistoryMgr::getMdxFrameMarkersCount()
{
    size_t framesCnt = m_vemslogbuffer->getHistoryCount( IDX_FRAME_END );

    return framesCnt;
}

void LogHistoryMgr::getMdxSiprIndices( size_t idx, IntArray& indices )
{
    indices.Clear();
    if ( idx < 0 || idx >= m_vemslogbuffer->getHistoryCount( IDX_FRAME_END ) )
    {
        return;
    }
    if ( idx == 0 )
    {
        int bufidx = m_vemslogbuffer->getIdxByDIdx( 0, IDX_FRAME_END );

        if ( bufidx <= 0 )
            return;

        for ( int i = 0; i < bufidx - 1; ++i )
            indices.Add( i );
    }
    else
    {
        int bufidx1 = m_vemslogbuffer->getIdxByDIdx( idx, IDX_FRAME_END );
        int bufidx0 = m_vemslogbuffer->getIdxByDIdx( idx-1, IDX_FRAME_END );

        if ( bufidx0 <= 0 || bufidx1 <= 0 )
            return;

        for ( int i = bufidx0+1; i < bufidx1; ++i )
            indices.Add( i );
    }
}

void LogHistoryMgr::getMdxSiprPackets( size_t idx, SIPRPacket*** packets, unsigned int& numberOfPacketsInFrame )
{
    if ( idx < 0 || idx >= m_vemslogbuffer->getHistoryCount( IDX_FRAME_END ) )
    {
        packets = NULL;
        numberOfPacketsInFrame = 0;
        return;
    }
    if ( idx == 0 )
    {
        int bufidx = m_vemslogbuffer->getIdxByDIdx( 0, IDX_FRAME_END );

        if ( bufidx <= 0 )
        {
            *packets = NULL;
            numberOfPacketsInFrame = 0;
            return;
        }

        *packets = new SIPRPacket*[bufidx-1];

        // packets[0] = m_vemslogbuffer->getFirstPacket();

        for ( int i = 0; i < bufidx - 1; ++i )
        {
            (*packets)[i] = m_vemslogbuffer->getNextPacket();
        }
        numberOfPacketsInFrame = bufidx - 1;
    }
    else
    {
        int bufidx1 = m_vemslogbuffer->getIdxByDIdx( idx, IDX_FRAME_END );
        int bufidx0 = m_vemslogbuffer->getIdxByDIdx( idx-1, IDX_FRAME_END );

        if ( bufidx0 <=0 || bufidx1 <= 0 )
        {
            *packets = NULL;
            numberOfPacketsInFrame = 0;
            return;
        }

        numberOfPacketsInFrame = ( bufidx1 - bufidx0 - 1 );

        *packets = new SIPRPacket*[numberOfPacketsInFrame];

        m_vemslogbuffer->setPosition( bufidx0 + 1 );
        for ( size_t i = 0; i < numberOfPacketsInFrame; ++i )
        {
            (*packets)[i] = m_vemslogbuffer->getNextPacket();
        }
    }
}

void LogHistoryMgr::getMarkers( IntArray& idxA, uint8_t markerType )
{
    idxA.Clear();

    uint8_t packettype = IDX_LOGMARKER;

    switch ( markerType )
    {
        case LH_MARKER_USER:
            packettype = IDX_LOGMARKER;
            break;

        case LH_MARKER_STATIC:
            packettype = IDX_STDATA;
            break;
    }

    size_t markersCount = m_vemslogbuffer->getHistoryCount( packettype );
    size_t rtCount = m_vemslogbuffer->getHistoryCount( IDX_RTDATA_NORMAL );

    for ( size_t i = 0; i < markersCount; ++i )
    {
        int buf_idx = m_vemslogbuffer->getIdxByDIdx( i, packettype );
        long rt_idx = m_vemslogbuffer->getNextRTidxByIdx( buf_idx );

        if ( rt_idx < 0 || rt_idx >= (int)rtCount )
            continue;

        // do not duplicate indices
        if ( idxA.Index( rt_idx ) == wxNOT_FOUND )
            idxA.Add( rt_idx );
    }
}

bool LogHistoryMgr::hasGPSdata()
{
    return ( m_vemslogbuffer->getHistoryCount( IDX_RTDATA_GPS ) > 0 );
}

void LogHistoryMgr::getGPSData( DoubleHashArray& gpsdata )
{
    GPSDataA t_gpsdata;

    if ( ! hasGPSdata() )
    {
        return;
    }

    SIPR2GPSConverter conv;
    //RTDataMgrFacade* mgr = new RTDataMgrFacade( *((RTDataMgrFacade*)getRTDataMgr()) );

    RTDataMgrFacade* tempMgr = new RTDataMgrFacade();
    RTDataInputSIPR* inputSIPR = new RTDataInputSIPR( m_symbolMgr, NULL );

    tempMgr->registerInput( inputSIPR );
    tempMgr->selectInput( INPUT_TYPE_SIPR );


    tempMgr->RegisterViewer(&conv);
    conv.SetDataMgr( tempMgr );
    conv.require();

    for ( size_t i = 0; i < getHistoryCount( IDX_RTDATA_GPS ); ++i )
    {
        SIPRPacket* packet = m_vemslogbuffer->getPacketByType( i , IDX_RTDATA_GPS );

        if ( packet == NULL )
            continue;

        tempMgr->process( *packet );
        tempMgr->notifyViewers();
    }

    t_gpsdata = conv.getData();

    conv.UnRegisterViewer();
    delete inputSIPR;
    delete tempMgr;

//----

    gpsdata.Clear();

    for ( size_t i = 0; i < t_gpsdata.GetCount(); ++i )
    {
        GPSData _data = t_gpsdata[i];

        DoubleHash* dh = new DoubleHash;
        (*dh)[_T("latitudeDec")] = _data.latitude;
        (*dh)[_T("longitudeDec")] = _data.longitude;
        (*dh)[_T("gpsSpeed")] = _data.speed;
        (*dh)[_T("distanceSum")] = _data.distance;

        int idx = m_vemslogbuffer->getIdxByDIdx( i, IDX_RTDATA_GPS );
        int rtidx = m_vemslogbuffer->getNextRTidxByIdx( idx );
        rtidx--;

        (*dh)[strLogIdxPos] = rtidx;

        (*dh)[_T("time")] = m_vemslogbuffer->getPacketTime( rtidx );

        gpsdata.Add( dh );
    }
}

void LogHistoryMgr::enableSlider( bool enable )
{
    if ( enable == false || m_canSliderEnabled == false )
    {
        observerNotify( -2, 0 );
        return;
    }
    else
    {
        observerNotify( -1, getHistoryCount()-1 );

        double mtime = m_vemslogbuffer->getPacketTime( getHistoryCount()-1 );
        wxString strTime = wxString::Format( _T("%.2f"), mtime );
        observerNotify( strTime );
    }
}

wxString LogHistoryMgr::getLogFileName()
{
    if ( m_fromlog == false )
        return wxEmptyString;
    else
        return m_logFileName;
}

long int LogHistoryMgr::getIdxByRTIdx( size_t idx )
{
    if ( m_vemslogbuffer )
    {
        return m_vemslogbuffer->getIdxByDIdx( idx );
    }
    return -1;
}
