#include "SearchDialog.h"

#include "LogHistoryBase.h"
#include "LogHistoryProcessFilterThread.h"
#include "../logger.h"
#include "../DrawHelper.h"

BEGIN_EVENT_TABLE(SearchDialog, wxDialog)

    //EVT_TEXT(idComboSearch, SearchDialog::OnComboText)
    //EVT_TEXT_ENTER(idComboSearch, SearchDialog::OnComboEnter)

    EVT_CHECKBOX(idisSearchBetweenSectionCB, SearchDialog::OnCheck)

    EVT_BUTTON(idButtonSearch, SearchDialog::OnButton)
    EVT_BUTTON(idButtonCancel, SearchDialog::OnButton)
    EVT_BUTTON(idButtonClose, SearchDialog::OnButton)
    EVT_BUTTON(idButtonHelp, SearchDialog::OnButton)

    EVT_COMMAND_ENTER(idInternalProcess, SearchDialog::OnInternalCommand)
    EVT_COMMAND_ENTER(idInternalEndProcess, SearchDialog::OnInternalCommand)

    EVT_CLOSE(SearchDialog::OnClose)

    EVT_KEY_DOWN(SearchDialog::OnKeyDown)

    EVT_MENU(idMenuClose, SearchDialog::OnMenu)

    EVT_MOVE(SearchDialog::OnMove)

    EVT_LISTBOX_DCLICK(idSymbolCompList,SearchDialog::OnSymbolCompDClick)

END_EVENT_TABLE()

SearchDialog::SearchDialog(wxWindow* parent, LogHistoryBase* loghistory, const wxString& title, wxArrayString searchHistory, wxArrayString symbolList )
    :m_filter(NULL)
    ,m_progressStaticBox(NULL)
    ,m_isCancelledSearch(false)
    ,m_helpPanel(NULL)
{
    //ctor
    m_lastInstertPoint = 0;

    m_logMgr = loghistory;

    m_searchHistory = searchHistory;

    m_symbolList = symbolList;

    m_symbolList.Sort();

    Create(parent, wxID_ANY, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE);

    m_limitSection = DefaultSection;

    m_searchProcesser = new LogHistoryProcessFilterThread(loghistory);

    m_searchProcesser->registerObserver(this);

    BuildGUI();

//    wxAcceleratorEntry entries[1];
//    entries[0].Set(wxACCEL_NORMAL,  WXK_ESCAPE, idMenuClose);
//    wxAcceleratorTable accel(1, entries);
//    SetAcceleratorTable(accel);
}

void SearchDialog::OnMenu(wxCommandEvent& evt)
{
    LOG
}

SearchDialog::~SearchDialog()
{
    //dtor
    m_searchProcesser->stop();
    m_searchProcesser->unRegisterObserver(this);
    delete m_searchProcesser;

    GetParent()->SetFocus();
}

void SearchDialog::BuildGUI()
{
    if(m_logMgr->getHistoryCount() < 1) {
        new wxStaticText(this, wxID_ANY, _T("Loghistory is empty!"));
        return;
    }

    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

        wxStaticBoxSizer* sb1 = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("Expression to search for"));

        wxString first = m_searchHistory.GetCount() ? m_searchHistory[ m_searchHistory.GetCount()-1 ] : _T("");

        m_searchCB = new wxComboBox(this, idComboSearch, first, wxDefaultPosition, wxSize(300,-1), 0, NULL, wxTE_PROCESS_ENTER);

        m_searchCB->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(SearchDialog::OnSearchBoxKeyDown), NULL, this);
        m_searchCB->Connect(wxEVT_KEY_UP, wxKeyEventHandler(SearchDialog::OnSearchBoxKeyUp), NULL, this);
        m_searchCB->Connect(wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(SearchDialog::OnComboText), NULL, this);


        for(size_t a=0; a<m_searchHistory.GetCount();a++) m_searchCB->Insert(m_searchHistory[a],0);

        sb1->Add(m_searchCB, 1, wxALL, 5 );

        wxBitmapButton* helpbutton = new wxBitmapButton(this, idButtonHelp, DrawHelper::GetBitmap(_T("ul-info-m.png")));

        sb1->Add(helpbutton, 0, wxALL, 5);

    mainsizer->Add(sb1, 0, wxALL | wxEXPAND, 5);

        wxStaticBoxSizer* sb2 = new wxStaticBoxSizer(wxVERTICAL, this, _T("Options"));

        m_isSection = new wxCheckBox(this, idisSearchBetweenSectionCB, _T("Search between..."));

        wxString beginIdxStr = wxString::Format(_T("%i"), m_limitSection == DefaultSection ? 0 : m_limitSection.beginIdx );
        wxString endIdxStr = wxString::Format(_T("%i"), m_limitSection == DefaultSection ? m_logMgr->getHistoryCount()-1 : m_limitSection.endIdx );

        wxTextValidator tv(wxFILTER_NUMERIC);
        m_beginIdxCtrl = new wxTextCtrl(this, wxID_ANY, beginIdxStr);
        m_beginIdxCtrl->SetValidator(tv);
        m_endIdxCtrl= new wxTextCtrl(this, wxID_ANY, endIdxStr);
        m_endIdxCtrl->SetValidator(tv);
        m_beginIdxCtrl->Enable(false);
        m_endIdxCtrl->Enable(false);

        sb2->Add(m_isSection, 0, wxALL, 5);

            wxFlexGridSizer* sectionsizer = new wxFlexGridSizer(3);
            sectionsizer->AddSpacer(20);
            sectionsizer->Add(new wxStaticText(this, wxID_ANY, _T("Start index:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
            sectionsizer->Add(m_beginIdxCtrl, 1, wxALL, 2 );
            sectionsizer->AddSpacer(20);
            sectionsizer->Add(new wxStaticText(this, wxID_ANY, _T("End index:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
            sectionsizer->Add(m_endIdxCtrl, 1, wxALL, 2 );

        sb2->Add(sectionsizer, 0, wxALL | wxEXPAND, 5);

    mainsizer->Add(sb2, 0, wxALL | wxEXPAND, 5);

    wxStaticBoxSizer* sb3 = new wxStaticBoxSizer(wxVERTICAL, this, _T("Searching progress"));

    m_progressStaticBox = sb3->GetStaticBox();

        m_progressBar = new wxGauge(this, wxID_ANY, 100, wxDefaultPosition, wxSize(-1,20), wxGA_SMOOTH | wxGA_HORIZONTAL);
        sb3->Add(m_progressBar, 0, wxALL | wxEXPAND, 5);

    mainsizer->Add(sb3, 0, wxALL | wxEXPAND, 5);

        wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);

        buttonSizer->Add(new wxButton(this, idButtonSearch, _T("Search")), 0, wxALL, 5);
        buttonSizer->Add(new wxButton(this, idButtonCancel, _T("Cancel")), 0, wxALL, 5);
        buttonSizer->Add(new wxButton(this, idButtonClose, _T("Close")), 0, wxALL, 5);

    mainsizer->Add(buttonSizer, 0, wxALL | wxALIGN_RIGHT, 5 );

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    m_symbolCompletionFrame = new wxFrame(this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(200,150), wxFRAME_NO_TASKBAR | wxFRAME_FLOAT_ON_PARENT);

    m_symbolCompletionList = new wxListBox(m_symbolCompletionFrame, idSymbolCompList, wxDefaultPosition, wxSize(200,150), m_symbolList, wxLB_SORT | wxBORDER_NONE );

    m_symbolCompletionList->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(SearchDialog::OnSymbolCompKeyDown), NULL, this );
}

void SearchDialog::OnSymbolCompDClick(wxCommandEvent& evt)
{
    //m_searchCB->SetFocus();

    SymbolCompletion(m_symbolCompletionList->GetStringSelection());

    evt.Skip();
}

void SearchDialog::OnSymbolCompKeyDown(wxKeyEvent& evt)
{
    if( evt.GetKeyCode() == WXK_RETURN ) SymbolCompletion(m_symbolCompletionList->GetStringSelection());

    evt.Skip();
}

void SearchDialog::EnableSectionCtrls(bool isEnable)
{
    m_beginIdxCtrl->Enable(isEnable);
    m_endIdxCtrl->Enable(isEnable);
}
// id checkbox checked, the start,end textctrl set to enable
void SearchDialog::OnCheck(wxCommandEvent& evt)
{
    EnableSectionCtrls(evt.IsChecked());
}

void SearchDialog::SetSearchSection(const Section& section)
{
    if(!section.isOk()) return;

    m_limitSection = section;

    m_beginIdxCtrl->SetValue( wxString::Format(_T("%i"),section.beginIdx) );
    m_endIdxCtrl->SetValue( wxString::Format(_T("%i"),section.endIdx) );

    EnableSectionCtrls(true);

    m_isSection->SetValue(true); //will cause OnCheck
}

void SearchDialog::Search()
{
    if(m_logMgr->getHistoryCount() < 1) return;

    wxString searchstr = m_searchCB->GetValue();

    int len = m_logMgr->getHistoryCount() - 1;

    m_resultSections.Clear();

    //moving expression string to the first position
    if(m_searchHistory.Index(searchstr) != wxNOT_FOUND) {
        m_searchCB->Delete( m_searchCB->FindString(searchstr) );
        m_searchHistory.Remove(searchstr);
    }

    m_searchCB->Insert(searchstr,0);
    m_searchHistory.Add(searchstr);

    m_searchCB->SetValue(searchstr);


    // set the start, end loghistory index
    long start, end;
    if(m_isSection->IsChecked()) {
        m_beginIdxCtrl->GetValue().ToLong(&start);
        m_endIdxCtrl->GetValue().ToLong(&end);
    } else {
        start = 0;
        end = len;
    }



    if(m_filter) delete m_filter;

    m_filter = new LogHistorySearch( searchstr , end, &m_resultSections );

    m_logMgr->registerLogFilter( m_filter );

    m_logMgr->registerObserver(this);

    //wxSafeYield(this, true);

    //Logger::logf(LOG_MSG,_T("start=%i,end=%i"),start,end);

    m_searchProcesser->SetLimits( start, end );

    EnablePanel(false);

    m_progressStaticBox->Enable(true);
    m_isCancelledSearch = false;

    m_retCode = wxID_OK;

    m_searchProcesser->start();

}

void SearchDialog::EnablePanel(bool isEnable)
{
    wxWindowListNode* node = GetChildren().GetFirst();

    while( node )
    {
        wxWindow* child = node->GetData();

        if(child->GetId() != idButtonCancel)
            child->Enable( isEnable );

        node = node->GetNext();
    }

}

void SearchDialog::InsertSymbol(const wxString& symbolname)
{
    SymbolCompletion(symbolname);
}

void SearchDialog::SymbolCompletion(const wxString& symbol)
{
    int insidx = m_lastInstertPoint; //m_searchCB->GetInsertionPoint();

//    Logger::logf(LOG_MSG,_T("insidx=%i"),insidx);

    wxString substr = SearchPartialSymbolName();

    wxString expression = m_searchCB->GetValue();

    int substr_idx = insidx-substr.Len();

    expression.Remove( substr_idx, substr.Len() );

    expression.insert( substr_idx, symbol );

    m_searchCB->SetValue(expression);

    m_searchCB->SetInsertionPoint(insidx-substr.Len()+symbol.Len());

    m_symbolCompletionFrame->Hide();
}

void SearchDialog::OnSearchBoxKeyUp(wxKeyEvent& evt)
{
    evt.Skip();
    m_lastInstertPoint = m_searchCB->GetInsertionPoint();
}

void SearchDialog::OnSearchBoxKeyDown(wxKeyEvent& evt)
{
    if(!m_symbolCompletionFrame->IsShown()) {

        if(evt.GetKeyCode() == WXK_RETURN) {
            Search();
        }

        evt.Skip();
        return;
    }

    switch( evt.GetKeyCode() ) {
        case WXK_RETURN:
            SymbolCompletion(m_symbolCompletionList->GetStringSelection());
            break;

        case WXK_DOWN:
            m_symbolCompletionList->SetSelection( m_symbolCompletionList->GetSelection()+1 );
            break;

        case WXK_UP:
            m_symbolCompletionList->SetSelection( m_symbolCompletionList->GetSelection() ? m_symbolCompletionList->GetSelection()-1 : 0 );
            break;

        case WXK_ESCAPE:
            m_symbolCompletionFrame->Hide();
            break;

        default:
            evt.Skip();
    }

}

void SearchDialog::OnMove(wxMoveEvent& evt)
{
    if(m_symbolCompletionFrame->IsShown()) SetSymbolCompletionFramePos();
}

void SearchDialog::OnKeyDown(wxKeyEvent& evt)
{
    //Logger::logf(LOG_MSG,_T("m_searchProcesser->isRunning()=%i"),m_searchProcesser->isRunning());

    if(evt.GetKeyCode() == WXK_ESCAPE) {
        StopSearch(true, wxID_CANCEL);
    }
}

void SearchDialog::OnClose(wxCloseEvent& evt)
{
    evt.Veto(true);

    StopSearch(false, wxID_CANCEL);
}

void SearchDialog::OnHelpClosed(wxCloseEvent& evt)
{
    m_helpPanel = NULL;
    evt.Skip();
}

void SearchDialog::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()) {
        case idButtonHelp:
            if(!m_helpPanel) {
                m_helpPanel = new SearchDialogHelp(this,m_symbolList);
                m_helpPanel->Connect(wxEVT_CLOSE_WINDOW, wxCloseEventHandler(SearchDialog::OnHelpClosed), NULL, this );
            } else {
                m_helpPanel->Show( !m_helpPanel->IsShown() );
            }
            break;

        case idButtonSearch:
            Search();
            break;

        case idButtonClose:
            StopSearch(false, wxID_CANCEL);
            break;

        case idButtonCancel:
            StopSearch(true, wxID_CANCEL);
            break;
    }
}

void SearchDialog::OnComboEnter(wxCommandEvent& evt)
{
    Search();
}

wxString SearchDialog::SearchPartialSymbolName()
{
    int insidx = m_lastInstertPoint; //m_searchCB->GetInsertionPoint();

    if(insidx < 3) return wxEmptyString;

    int idx = 0;

    wxString str = m_searchCB->GetValue();

    for(int a = insidx-1; a >= 0; a--) {
        wxString c = str[a];
        if(c != _T(" ")) continue;
        idx = a+1;
        break;
    }

    return str.Mid(idx,insidx-idx);
}

void SearchDialog::OnComboText(wxCommandEvent& evt)
{
    if(!m_symbolList.GetCount()) return;

    int insidx = m_searchCB->GetInsertionPoint();

    wxString substr = SearchPartialSymbolName();

    if(substr.Len() < 3) {
        m_symbolCompletionFrame->Hide();
        return;
    }

    int symIdx = -1;

    for(size_t a=0; a<m_symbolList.GetCount();a++) {
        wxString sym = m_symbolList[a];
        if(!sym.StartsWith(substr)) continue;

        symIdx = a;
        break;
    }

    if(symIdx == -1) {
        m_symbolCompletionFrame->Hide();
        return;
    }

    m_symbolCompletionList->SetSelection(symIdx);
    SetSymbolCompletionFramePos();
    m_symbolCompletionFrame->Show();

    m_searchCB->SetFocus();
    m_searchCB->SetInsertionPoint(insidx);

}

void SearchDialog::SetSymbolCompletionFramePos()
{
    wxRect cbRect = m_searchCB->GetRect();
    wxPoint pos = ClientToScreen( wxPoint ( cbRect.GetLeft(), cbRect.GetBottom() + 1) );
    m_symbolCompletionFrame->Move( pos );
}

// retport to loghistorymgr to stopping current process
// this will be causing EndSearch(), if the thread is not running
void SearchDialog::StopSearch(bool isCancel, int retcode)
{
    if(!m_searchProcesser->isRunning()) {
        EndModal(retcode);
        return;
    }

    m_retCode = retcode;
    m_logMgr->StopProcessing();
    m_isCancelledSearch = isCancel;
}

void SearchDialog::EndSearch()
{
    m_logMgr->unregisterLogFilter(m_filter);
    m_logMgr->unRegisterObserver(this);

    delete m_filter;
    m_filter = NULL;

    EnablePanel(true);
    EnableSectionCtrls( m_isSection->IsChecked() );

    m_searchCB->SetFocus();

    m_progressBar->SetValue(0);

    if(!m_isCancelledSearch) {

        if(m_resultSections.GetCount())
            EndModal(m_retCode);
        else{
            wxMessageBox(_T("Not found any data for this expression"), _T("Message"));
        }

    }
}

// thread safe notifies
void SearchDialog::OnInternalCommand(wxCommandEvent& evt)
{
    switch(evt.GetId()) {
        case idInternalProcess:
            m_progressBar->SetValue( evt.GetInt() );
            break;

        case idInternalEndProcess:
            EndSearch();
            break;
    }
}

// will call: OnInternalCommand
void SearchDialog::notify(const wxString& str)
{
    if(str == _T("LogHistoryProcessFilterThreadFinished")) {
        wxCommandEvent evt(wxEVT_COMMAND_ENTER, idInternalEndProcess);
        AddPendingEvent( evt );
    }
}

// will call: OnInternalCommand
void SearchDialog::notify( int i )
{
    wxCommandEvent evt(wxEVT_COMMAND_ENTER, idInternalProcess);
    evt.SetInt( i );
    AddPendingEvent( evt );
}

void SearchDialog::SetSearchString(const wxString& str)
{
    m_searchCB->SetValue(str);
}

/****************************************************************
                        SearchDialogHelp
*/

SearchDialogHelp::SearchDialogHelp(SearchDialog* parent, const wxArrayString& symbolList)
{
    Create(parent, wxID_ANY, _T("Search help"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE | wxFRAME_FLOAT_ON_PARENT );
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_parent = parent;

    wxStaticBoxSizer* slSizer = new wxStaticBoxSizer(wxVERTICAL, this, _T("Symbol list"));

    m_symbolList = new wxListBox(this, wxID_ANY);
    m_symbolList->Append(symbolList);

    m_symbolList->Connect(wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, wxCommandEventHandler(SearchDialogHelp::OnEnter), NULL, this );
    m_symbolList->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(SearchDialogHelp::OnListKeyDown), NULL, this );

    slSizer->Add(m_symbolList, 0, wxALL | wxEXPAND, 5);

    wxStaticBoxSizer* exSizer = new wxStaticBoxSizer(wxVERTICAL, this, _T("Some example"));

    wxString examples = _T("- rpm > 50\n- (rpm < 1500) & (rpm > 1000) & (map < 40)\n- etc...");

    wxStaticText* text = new wxStaticText(this, wxID_ANY, examples);

    exSizer->Add(text, 0, wxALL, 5);

    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    mainsizer->Add(slSizer, 0, wxALL | wxEXPAND, 5);
    mainsizer->Add(exSizer, 0, wxALL | wxEXPAND, 5);

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    if(parent){
        wxPoint parent_pos = parent->GetPosition();
        wxSize parent_size = parent->GetSize();

        Move(parent_pos.x+parent_size.x, parent_pos.y);
}

    Show();
}

SearchDialogHelp::~SearchDialogHelp()
{

}

void SearchDialogHelp::InsertSymbol()
{
    m_parent->InsertSymbol( m_symbolList->GetStringSelection() );
}

void SearchDialogHelp::OnListKeyDown(wxKeyEvent& evt)
{
    if( evt.GetKeyCode() == WXK_RETURN ) InsertSymbol();
    evt.Skip();
}

void SearchDialogHelp::OnEnter(wxCommandEvent& evt)
{
    InsertSymbol();
}
