#include "ProjectChooserFrame.h"
#include "../logger.h"
#include "../DirHelper.h"
#include "ProjectHelper.h"
#include "ProjectManager.h"
#include "../common/typedef.h"

int idProjectList = wxNewId();

BEGIN_EVENT_TABLE(ProjectChooserFrame, wxDialog)

    EVT_BUTTON(idOkButton, ProjectChooserFrame::OnButton)
    EVT_BUTTON(idAddButton, ProjectChooserFrame::OnButton)
    EVT_BUTTON(idCancelButton, ProjectChooserFrame::OnButton)

    EVT_LIST_ITEM_ACTIVATED(idProjectList, ProjectChooserFrame::OnListAcitvated)

END_EVENT_TABLE()

ProjectChooserFrame::ProjectChooserFrame(ProjectManager *prjmgr, wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
//    :wxFrame (parent, id, title, pos, size, style, name)
{
    m_selectedPrj = NULL;
    m_PrjMgr = prjmgr;

    style = 0 |
            wxMINIMIZE_BOX |
            wxRESIZE_BORDER |
            wxSYSTEM_MENU |
            wxCAPTION |
            //wxCLOSE_BOX |
            wxCLIP_CHILDREN;
    //ctor

    Create(parent, id, title, pos, size, style, name);


    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    wxStaticText * stitle = new wxStaticText( this, wxID_ANY, _T("Select a project:"));

    wxFont font = stitle->GetFont();
    font.SetWeight(wxFONTWEIGHT_BOLD);
    stitle->SetFont(font);

    m_projectList = new wxListCtrl(this,idProjectList,wxDefaultPosition, wxSize(200,150),wxLC_REPORT);

    m_projectList->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(ProjectChooserFrame::OnKeyDown));

    wxButton * okbutt   = new wxButton(this, idOkButton, _T("Load"));
    wxButton * addbutt  = new wxButton(this, idAddButton, _T("Create..."));
    wxButton * cancbutt = new wxButton(this, idCancelButton, _T("Cancel"));

    addbutt->Enable(false);

    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    mainsizer->Add(stitle,0,wxALL | wxALIGN_CENTER_VERTICAL |wxALIGN_LEFT, 5);
    mainsizer->Add(m_projectList, 1, wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 5);

        wxBoxSizer * buttonlevelsizer = new wxBoxSizer(wxHORIZONTAL);
        buttonlevelsizer->Add(okbutt,0,wxALL |wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL,5);
        buttonlevelsizer->Add(cancbutt,0,wxALL |wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL,5);
        buttonlevelsizer->Add(addbutt,0,wxALL |wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL,5);

    mainsizer->Add(buttonlevelsizer,0,wxALIGN_RIGHT);


    SetSizer(mainsizer);
	mainsizer->Fit(this);
	mainsizer->SetSizeHints(this);

    InitList();
    LoadRecents();

}

ProjectChooserFrame::~ProjectChooserFrame()
{
    //dtor

}

void ProjectChooserFrame::OnListAcitvated(wxListEvent &evt)
{
    SelectProject();
}

bool ProjectChooserFrame::IsEmpty()
{

    if(m_projectList->GetItemCount())
        return false;
    else
        return true;

}

void ProjectChooserFrame::OnKeyDown(wxKeyEvent &evt)
{
    Logger::logf(LOG_MSG,_T("ProjectChooserFrame::OnListKeyDown = %i"),evt.GetKeyCode());

    wxCommandEvent event;
    event.SetEventType(wxEVT_COMMAND_BUTTON_CLICKED);

    switch(evt.GetKeyCode() )
    {

        case WXK_ESCAPE:
            event.SetId(idCancelButton);
            AddPendingEvent(event);
            break;

    }

    evt.Skip();

}


void ProjectChooserFrame::LoadRecents()
{

    ProjectArray *arr = m_PrjMgr->GetProjects();

    for(uint16_t a=0; a<arr->GetCount();a++){

        AddLineToList(arr->Item(a)->Name,arr->Item(a)->Path);

    }




    SelectLine(0);

}


void ProjectChooserFrame::AddLineToList(const wxString& name, const wxString& path)
{

    wxListItem item0, item1;


    item0.SetId( m_projectList->GetItemCount() );
    item1.SetId( m_projectList->GetItemCount() );

    item0.SetColumn(0);
    item1.SetColumn(1);

    //wxFont font = item0.GetFont();

    item0.SetText(name);
    item1.SetText(path);

    //item0.SetFont(font);


    m_projectList->InsertItem(item0);
    m_projectList->SetItem(item1);


    //window resizes

    m_projectList->SetColumnWidth(0,-1);
    m_projectList->SetColumnWidth(1,-1);


    int c0 = m_projectList->GetColumnWidth(0);
    int c1 = m_projectList->GetColumnWidth(1);

    SetSize( c0+c1+22 , GetSize().y );  /// temporarly
    //Refresh();


}

void ProjectChooserFrame::InitList()
{
/*
    wxImageList *imglist = new wxImageList(16,15,true);
    imglist->Add( wxArtProvider::GetBitmap(wxART_NORMAL_FILE));
    imglist->Add( wxArtProvider::GetBitmap(wxART_FOLDER ));

    m_listCtrl->SetImageList(imglist, wxIMAGE_LIST_SMALL);
*/

    short w = m_projectList->GetSize().x-4;

    m_projectList->InsertColumn(0,_T("Name"),wxLIST_FORMAT_LEFT,w/4);

    m_projectList->InsertColumn(1,_T("Path"),wxLIST_FORMAT_LEFT,(w/4)*3);


}

bool ProjectChooserFrame::SelectLine(long id, bool select)
{

    wxListItem item;
    item.SetId(id);
    bool res = m_projectList->GetItem(item);
    if(!res) return false;

    if(select == true) /// not worx...
        item.SetState(wxLIST_STATE_SELECTED);//item.SetState(item.GetState()|wxLIST_STATE_SELECTED);
    else
        item.SetState(0);//item.SetState(item.GetState()&~wxLIST_STATE_SELECTED);

    m_projectList->SetItem(item);

    Logger::logf(LOG_MSG,_T("ProjectChooserFrame::SelectLine: ")+item.GetText());

    return true;
}

void ProjectChooserFrame::OnButton(wxCommandEvent &evt)
{


    switch(evt.GetId())
    {
        case idOkButton:
            SelectProject();
            break;

        case idCancelButton:
            EndModal(wxID_CANCEL);
            break;

    }

}

void ProjectChooserFrame::SelectProject()
{

    long item = m_projectList->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);



    wxListItem info;
    info.SetColumn(1);
    info.SetId(item);
    info.SetMask(wxLIST_MASK_TEXT);

    m_projectList->GetItem(info);

    m_selectedPrj = m_PrjMgr->GetProjectByPath(info.GetText());

    if( m_selectedPrj )
        EndModal(wxID_OK);
    else
        wxMessageBox(_T("Wrong project file"), _T("Project error"));

}
