#ifndef PROJECTSETTINGSFRAME_H
#define PROJECTSETTINGSFRAME_H

#include <wx/wx.h>
#include <wx/xml/xml.h>
#include <wx/bookctrl.h>
#include <wx/listbox.h>

class Project;
class ProjectManager;
class PropertyGuiItem;

WX_DECLARE_STRING_HASH_MAP( PropertyGuiItem*, PropertyGuiItemHash );
WX_DEFINE_ARRAY( PropertyGuiItemHash* , PropertyGuiItemHashArray);
WX_DECLARE_STRING_HASH_MAP(PropertyGuiItemHashArray*, PropertyGuiItemHashArrayHash);

/*
WX_DECLARE_STRING_HASH_MAP( wxTextCtrl*, wxTextCtrlHash );
WX_DEFINE_ARRAY( wxTextCtrlHash* , wxTextCtrlHashArray);
WX_DECLARE_STRING_HASH_MAP(wxTextCtrlHashArray*, wxTextCtrlHashArrayHash);
*/

WX_DEFINE_ARRAY( wxWindow* , wxWindowArray);
WX_DECLARE_STRING_HASH_MAP( wxWindow*, wxWindowHash );

enum ProjectNodeTypes{
/*
    PNT_NAME = 1,
    PNT_FWVER = 2,
    PNT_CONFFILES = 4,
    PNT_FWFILES = 8,
    PNT_ALL = PNT_NAME | PNT_FWVER | PNT_CONFFILES | PNT_FWFILES
*/
    PNT_MINIMAL = 1,
    PNT_ALL = 2

};

class ProjectSettingsFrame: public wxDialog
{
    public:
        ProjectSettingsFrame(wxWindow *parent, Project *project, ProjectManager *prjmgr, ProjectNodeTypes type = PNT_ALL);
        virtual ~ProjectSettingsFrame();

        void SetProjectMgr(ProjectManager *prjmgr) { m_prjMgr = prjmgr; };

    protected:

        void Init(Project *project);
        void LoadPrj(Project *project, bool firstload = false);
        void FillDlg(Project *project, bool firstload = false);

        void OnButton1(wxCommandEvent &evt);
        void OnButton(wxCommandEvent &evt);
        void OnListBoxSel(wxCommandEvent &evt);

        void EnableTextControls(bool enable);
        void EnablePages(bool enable);


        void HideNotValidPages(Project *prj);
        void ShowValidPages(Project *prj);


        static wxWindow* GetPageByName(const wxString& name, wxBookCtrl *bctrl);
        static void RemovePages(wxBookCtrl *bctrl);
        static void RemovePageByName(const wxString& name, wxBookCtrl *bctrl);



    private:

        void SetProjectToTitle(Project *prj);

        enum {
            idOkButton,
            idAddButton,
            idCancelButton,
            idSaveButton,
            idPrjList,
            idPrjBook
        };


        wxListBox   *   m_projectList;
        wxBookCtrl  *   m_prjBook;
        wxWindowHash    m_pageHash;
        wxStaticText*   m_prjPath;

        ProjectManager *m_prjMgr;


        int m_type;

        //wxTextCtrlHashArrayHash     m_prjContent;
        PropertyGuiItemHashArrayHash m_prjContent;

        DECLARE_EVENT_TABLE();

};

class PropertyGuiItem
{
    public:
        PropertyGuiItem() { Label = NULL; Value = NULL; };
        PropertyGuiItem(wxStaticText * label, wxTextCtrl * value) { Label = label; Value = value; };

        ~PropertyGuiItem() {
            if(Label) Label->Destroy();
            if(Value) Value->Destroy();  };


        void Enable(bool enable) {
            if(Value) Value->Enable(enable);
            if(Label) Label->Enable(enable);  };

        void Show(bool show) {
            if(Value) Value->Show(show);
            if(Label) Label->Show(show); };

        bool IsShown() {
            bool res;
            if(Value) res  = Value->IsShown();
            if(Label) res &= Label->IsShown();
            return res;  };

        wxStaticText *  Label;
        wxTextCtrl *    Value;



};



#endif // PROJECTSETTINGSFRAME_H
