#include "GPSProducerThread.h"
#include "../rtdatamanager/RTDataInputBase.h"
#include "../events/GPSEvent.h"
#include "../logger.h"


GPSProducerThread::GPSProducerThread( RTDataInputBase* rbInput, wxEvtHandler* evt )
{
    m_evtHandler = evt;
    m_rbInput   = rbInput;
    m_state     = GPSInfo::NOT_CONNECTED;
}

GPSProducerThread::~GPSProducerThread()
{
    //dtor
}

void* GPSProducerThread::Entry()
{
    //Logger::logf( LOG_MSG, _T("GPSProducerThread::Entry() - START") );

    int noDataCount = 0;
    int dataReceivedCount = 0;
    while( true )
    {
        //Logger::logf( LOG_MSG, _T("GPSProducerThread::Entry()") );

        waitEnable();
        int result = m_rbInput->updateInputModel();

//        Logger::logf( LOG_MSG, _T("GPSProducerThread::Entry() - 2") );

        switch( result )
        {
            case GPSInfo::NOT_CONNECTED :
            {
                //Logger::logf( LOG_MSG, _T("GPSProducerThread::Entry() : NOT_CONNECTED ") );
                if( m_state != result )
                {
                    sendEvent(GPSInfo::NOT_CONNECTED);
                }
                wxThread::Sleep( 1000 );
            }break;
            case GPSInfo::NO_GPS_DATA :
            {
                //Logger::logf( LOG_MSG, _T("GPSProducerThread::Entry() : NOT_GPS_DATA ") );
                noDataCount++;
                wxThread::Sleep( 1 );
            }break;
            case GPSInfo::DATA_RECEIVED:
            {
                //Logger::logf( LOG_MSG, _T("GPSProducerThread::Entry() : NOT_GPS_DATA ") );
                dataReceivedCount++;
                //wxThread::Sleep( 1 );
            }break;
            case GPSInfo::CONNECTED :
            {
                //Logger::logf( LOG_MSG, _T("GPSProducerThread::Entry() : CONNECTED ") );
                noDataCount = 0;
                dataReceivedCount = 0;
                if( m_state != result )
                {
                    sendEvent(GPSInfo::CONNECTED);
                }
            }
        }

        if( noDataCount == 100 )
        {
            sendEvent(GPSInfo::NO_GPS_DATA);
        }

        if( dataReceivedCount == 200 )
        {
            sendEvent(GPSInfo::NO_GPS_DATA);
        }

    }
    //Logger::logf( LOG_MSG, _T("GPSProducerThread::Entry() - END") );
}

void GPSProducerThread::sendEvent(GPSInfo::State state)
{
    GPSEvent event;
    event.state = state;
    m_evtHandler->AddPendingEvent( event );
    m_state = state;
}
