#include "RingBufferThreadTester.h"

#include "RingBufferReaderThread.h"
#include "../logger.h"

#include "../RTDataMgrBase.h"
#include "../RTDataMgrFacade.h"
#include "../VemsProtocolHelper.h"
//#include "../stdatamanager/STDataMgrFacade.h"

#include "../confighelper.h"
#include "../RTDataInputRingBufferEcu.h"
#include "../RTDataInputRingBufferRawFile.h"
#include "../RTDataInputRingBufferSyncNone.h"
#include "../RTDataInputBase.h"
#include "GUIConsumerThread.h"
#include "EcuPruducerThread.h"
#include "FileProducerThread.h"

RingBufferThreadTester::RingBufferThreadTester()
{
    //ctor
}

RingBufferThreadTester::~RingBufferThreadTester()
{
    //dtor
}

//RingBufferThreadMgr



void RingBufferThreadTester::init(RTDataMgrBase * mgrBase)
{
    uint8_t bsize = 4;

    wxConfig *conf = ConfigHelper::getConfig();
    conf->SetPath(_T("/"));
    bsize = conf->Read(_T("SerialBufferSize"),4);

    if(bsize < 2) bsize = 2;

    m_ringBuffer = new RingBuffer(bsize, 256);
    ThreadConditionBase *tc = new ThreadConditionBase();

    m_ringBufferWriter = new RingBufferWriterThread();
    m_ringBufferWriter->setRingBuffer(m_ringBuffer);
    m_ringBufferWriter->setThreadConditionBase(tc);
    m_ringBufferWriter->setInputMgr((RTDataMgrFacade *)mgrBase);

    m_ringBufferWriter->setVemsProtocol( VemsProtocolHelper::getVemsProtocol() );

    if ( m_ringBufferWriter->Create() != wxTHREAD_NO_ERROR )
    {
        Logger::logf(LOG_MSG, _T("Can't create writer thread!"));
    }

    RingBufferReaderThread * m_ringBufferReader = new RingBufferReaderThread();
    m_ringBufferReader->setRingBuffer(m_ringBuffer);
    m_ringBufferReader->setThreadConditionBase(tc);
    m_ringBufferReader->setRTDataMgrBase(mgrBase);

    if ( m_ringBufferReader->Create() != wxTHREAD_NO_ERROR )
    {
        Logger::logf(LOG_MSG, _T("Can't create reader thread!"));
    }


    m_ringBufferReader->SetPriority(WXTHREAD_DEFAULT_PRIORITY);
    m_ringBufferReader->Run();

    m_ringBufferWriter->SetPriority(WXTHREAD_MAX_PRIORITY);
    m_ringBufferWriter->Run();
    //wxThread::Sleep(500);

}

bool RingBufferThreadTester::isConnected()
{
    return ((EcuPruducerThread *)m_ecuThread)->isConnected();
    //return m_ringBufferWriter->isConnected();
}

RingBuffer * RingBufferThreadTester::getRingBuffer()
{
    return m_ringBuffer;
}

void RingBufferThreadTester::setLocked(bool lock)
{
    //m_ringBufferWriter->setLocked(lock);
    m_ecuThread->setEnable(!lock);
}

void RingBufferThreadTester::setRawDataLog(bool b)
{
    //m_ringBufferWriter->setRawLog(b);
}

////////////////////////////////////////////////////////////
void RingBufferThreadTester::start()
{
    Logger::logf(LOG_MSG, _T("RingBufferThreadTester::start() START"));
    RingBuffer *m_ringbuffer = new RingBuffer(8, 256);
    ThreadConditionBase *tc = new ThreadConditionBase();




/*
    wxMutex mutex;  //= new wxMutex();
    wxMutex Rmutex;
    wxMutex Wmutex;

    wxCondition m_isEmpty(mutex);
    wxCondition m_isFull(mutex);

    wxCondition m_isNotEmpty(Rmutex);
    wxCondition m_isNotFull(Wmutex);
*/
    wxArrayString  queue; // = new wxArrayString();

    //m_isNotEmpty = new wxCondition(m_mutex);
    //m_isNotFull = new wxCondition(m_mutex);

    RingBufferWriterThread * m_ringBufferWriter = new RingBufferWriterThread();
    m_ringBufferWriter->setRingBuffer(m_ringbuffer);
    m_ringBufferWriter->setThreadConditionBase(tc);

    /*
    m_ringBufferWriter->m_mutex = &mutex;
    m_ringBufferWriter->m_Rmutex = &Rmutex;
    m_ringBufferWriter->m_Wmutex = &Wmutex;

    m_ringBufferWriter->m_isFull = &m_isFull;
    m_ringBufferWriter->m_isEmpty = &m_isEmpty;

    m_ringBufferWriter->m_isNotFull = &m_isNotFull;
    m_ringBufferWriter->m_isNotEmpty = &m_isNotEmpty;
*/
//    m_ringBufferWriter->m_queue = &queue;

    if ( m_ringBufferWriter->Create() != wxTHREAD_NO_ERROR )
    {
        Logger::logf(LOG_MSG, _T("Can't create writer thread!"));
    }



    RingBufferReaderThread * m_ringBufferReader = new RingBufferReaderThread();
    m_ringBufferReader->setRingBuffer(m_ringbuffer);
    m_ringBufferReader->setThreadConditionBase(tc);
    /*
    m_ringBufferReader->m_mutex = &mutex;
    m_ringBufferReader->m_Rmutex = &Rmutex;
    m_ringBufferReader->m_Wmutex = &Wmutex;

    m_ringBufferReader->m_isFull = &m_isFull;
    m_ringBufferReader->m_isEmpty = &m_isEmpty;

    m_ringBufferReader->m_isNotFull = &m_isNotFull;
    m_ringBufferReader->m_isNotEmpty = &m_isNotEmpty;
*/
//  m_ringBufferReader->m_queue = &queue;

    if ( m_ringBufferReader->Create() != wxTHREAD_NO_ERROR )
    {
        Logger::logf(LOG_MSG, _T("Can't create reader thread!"));
    }



    char *src;
    m_ringbuffer->Write(src);



    m_ringBufferWriter->Run();
    wxThread::Sleep(500);
    m_ringBufferReader->Run();




}

void RingBufferThreadTester::setActiveInput(RTDataInputType inputType)
{
    if(inputType == INPUT_TYPE_RINGBUFFER_ECU){
        m_fileThread->setEnable(false);
    }else if( inputType == INPUT_TYPE_RINGBUFFER_RAWFILE){
        m_fileThread->setEnable(true);
    }
}


