#include "ThreadLogPlayer.h"

#include "../events/InputEvent.h"
#include "../loghistory/LogHistoryMgr.h"
#include "../loghistory/FilePlayCommander.h"
#include "../vemsprotocol/SIPRTypes.h"
#include "../logger.h"
#include "../events/GeneralEvent.h"

ThreadLogPlayer::ThreadLogPlayer( FilePlayCommander* fpc, LogHistoryMgr* loghistory, wxEvtHandler* eventHandler )
{
    m_logHistory = loghistory;
    m_eventHandler = eventHandler;
    m_initialized = false;
    m_fpc = fpc;
    m_position = 0;
    m_speed = 1;
    m_repeat = false;
}

ThreadLogPlayer::~ThreadLogPlayer()
{
    //dtor
}

bool ThreadLogPlayer::process()
{
    if ( !m_initialized )
    {
        //m_logHistory->setPosition(0);
        m_initialized = true;
        //m_position = 0;
        m_logHistory->setPosition( m_position );
        m_historyLen = m_logHistory->getHistoryCount( IDX_ALL );
        //Logger::logf(LOG_MSG, _T("History length=%d"), m_historyLen );
    }

    SIPRPacket* siprPacket = NULL;

    //Logger::logf(LOG_MSG, _T("pos=%d, len=%d"), m_position, m_historyLen);

    if ( m_position >= m_historyLen )
    {
        if ( m_repeat )
        {
            m_position = 0;
            return true;
        }
        else
        {
            GeneralEvent event;
            event.SetString( _T("SET_STOPPED") );
            m_fpc->AddPendingEvent( event );
            return false;
        }
    }

    size_t tsleep = m_logHistory->getPacket( m_position++, &siprPacket );

    if ( siprPacket == NULL )
    {
        Logger::logf(LOG_MSG, _T("ThreadLogPlayer::process() END - packet is NULL") );

		GeneralEvent event;
		event.SetString( _T("SET_STOPPED") );
		m_fpc->AddPendingEvent( event );


        return false;
    }
    else
    {
        sendEvent(*siprPacket, SIPR_EVENT_TYPE_INPUT);
    }

    if ( siprPacket->type() == SIPR_READ_SENSORS)
    {
        switch (siprPacket->getSubType())
        {
            case SiprSensorType::SIPR_RT_MT_NORMAL:
            case SiprSensorType::SIPR_RT_MT_EXT:
            case SiprSensorType::SIPR_RT_ROUND:
                tsleep = (int)( tsleep / m_speed );
                wxThread::Sleep( tsleep );
                break;

            default:

                tsleep = 80;
                tsleep = (int)( tsleep / m_speed );
                wxThread::Sleep( tsleep );
                break;
        }
    }

    // 5 ms for eg static data
    else
    {
        tsleep = 5;
        wxThread::Sleep( tsleep );
    }

    return true;
}

void ThreadLogPlayer::setLogHistory(LogHistoryMgr * mgr)
{
    m_logHistory = mgr;
    m_initialized = false;
    m_position = 0;
}

void ThreadLogPlayer::sendEvent( SIPRPacket packet, int eventType )
{
    InputEvent e;
    e.setPacket( packet );
    e.setType( eventType );
    if ( m_eventHandler )
    {
        m_eventHandler->AddPendingEvent( e );
    }
}

void ThreadLogPlayer::reset()
{
    m_position = 0;
    m_repeat = false;
    m_initialized = false;
}

void ThreadLogPlayer::stop()
{
    if ( isRunning() )
    {
        ProcessBase::stop();
    }
    m_position = 0;
}

void ThreadLogPlayer::setPosition( size_t pos )
{
    long idx = m_logHistory->getIdxByRTIdx( pos );
    if ( idx < 0 )
    {
        m_position = 0;
    }
    else if ( idx < m_historyLen )
    {
        m_position = idx;
    }
}

void ThreadLogPlayer::setSpeed( unsigned char speed )
{
    m_speed = speed;

    // x1, x2, x4 speeds are supported
    if ( m_speed != 1 && m_speed != 2 && m_speed != 4 )
    {
        m_speed = 1;
    }
}

void ThreadLogPlayer::finished()
{
    LOGP( _T("finished") )
}

void ThreadLogPlayer::setRepeat( bool repeat )
{
    m_repeat = repeat;
}
