#include "ConfigSwitchViewer.h"
#include "../vemsprotocol/CommandRequestBase.h"
#include "../vemsprotocol/VemsCommand.h"

ConfigSwitchViewer::ConfigSwitchViewer( CommandRequestBase* requestHandler )
{
    m_requestHandler = requestHandler;

    m_symbolName = _T("isConfsw");

    m_isSymbolRegistered = false;

    m_isFirstRefresh = true;

    m_isConfsw = false;
}


ConfigSwitchViewer::~ConfigSwitchViewer()
{
}

void ConfigSwitchViewer::require()
{
    if( !m_isSymbolRegistered )
    {
        m_rtDataMgr->registerSymbol( m_symbolName );
        m_isSymbolRegistered = true;
    }
}

void ConfigSwitchViewer::refresh( DoubleHash* table )
{
    DoubleHash::const_iterator it = table->find( m_symbolName );

    if( it == table->end() ) { return; }

    bool isConfsw = (bool)it->second;

    if ( m_isFirstRefresh )
    {
        m_isConfsw = isConfsw;
        m_isFirstRefresh = false;
    }

    if ( m_isConfsw != isConfsw )
    {
        Logger::logf( LOG_MSG, _T("ConfigSwitchViewer::refresh() - CONFIG SWITCH CHANGED - %d"), m_isConfsw );
        m_isConfsw = isConfsw;
        m_requestHandler->request( new ReadAllPageCommand() );
    }
}

void ConfigSwitchViewer::unregisterSymbols()
{
    if( m_isSymbolRegistered )
    {
        m_rtDataMgr->unregisterSymbol( m_symbolName );
        m_isSymbolRegistered = false;
    }
}
