#ifndef RT2VIEWERCONVERTER_H
#define RT2VIEWERCONVERTER_H

#include "../expression/RPNParser.h"
#include "../expression/VariableData.h"

class SymbolMgr;
class DoubleHash;
class SIPRPacket;
class GraphNodeSetArray;
class GraphNodeSet;

class RT2ViewerConverter
{
    public:
        RT2ViewerConverter(SymbolMgr* mt, DoubleHash* viewerModel);
        virtual ~RT2ViewerConverter();

        void refreshDependency(const DoubleHash& requiredData);
        /// old style
        void convert(uint8_t* buff, int size, DoubleHash& requiredData, int type = -1);
        void convertAIM(uint8_t* buff, DoubleHash& requiredData);
        /// new style
        void convert(const SIPRPacket& packet, DoubleHash& requiredData);
        void printViewerModel();

        void resetHash();

    protected:
    private:
        SymbolMgr* m_symbolMgr;
        DoubleHash* m_viewerModel;
        uint8_t* m_buffer;
        uint8_t m_size;
        GraphNodeSetArray* m_dependencyA;

        VariableDataDH  m_variableData;     //DoubleHash variable data for m_rpn
        RPNParser       m_rpn;              //rpn calculator for expression symbols
        bool            m_hasInternalModel;
        int             m_address;          // addres of SIPR_READ_MEMORY_32 type packet

        void fillScalarValues();
        void fillScalarValues(int rtDataType);
        void fillDependentValues();
        void fillRegisteredValues(DoubleHash& requiredData);

        double  getValue(int size, int offset, double scale, double translate);
        int     getValue(int size, int offset);
        bool    getValue(const wxString& symbolname, double& value);

        double  getAIMValue( uint8_t* buffer, double scale, double translation );

        void fillGraphNodeSet(const DoubleHash& requiredData, GraphNodeSet& nodes);
        void purge( GraphNodeSetArray* m_nodeArray);
};

#endif // RT2VIEWERCONVERTER_H
