#include "RTDataInputRingBufferEcu.h"

#include "../logger.h"
#include "../vemsprotocol.h"
#include "../ringbuffer/SyncObj.h"
#include "RTDataRawLogger.h"
#include "../RealTimeDesc.h"
#include "../DirHelper.h"
#include "../RawLogHelper.h"
#include "../GPSData.h"
#include "../vemsprotocol/ECUInfo.h"
#include "../SerialException.h"
#include "../SymbolMgr.h"

RTDataInputRingBufferEcu::RTDataInputRingBufferEcu(RingBuffer * rb, SymbolMgr * mt, SyncObj* syncObj,
    VemsProtocol* vp, RTDataRawLogger* rtDataRawLogger, GPSData* gps, DoubleHash* viewerModel )
: RTDataInputRingBufferSync(rb, mt,  syncObj, viewerModel )
{
    //ctor
    m_vp = vp;
    m_rtDataRawLogger = rtDataRawLogger;
    m_gpsData = gps;
}

RTDataInputRingBufferEcu::~RTDataInputRingBufferEcu()
{
    //dtor
}

RTDataInputType RTDataInputRingBufferEcu::getType()
{
    return INPUT_TYPE_RINGBUFFER_ECU;
}

int RTDataInputRingBufferEcu::updateInputModel()
{
    //Logger::log(LOG_MSG, _T("RTDataInputRingBufferEcu::updateInputModel() start"));
    //return ECU_NOT_CONNECTED;

    m_state = getRtData();
    return m_state;
}

ECUEvent::State RTDataInputRingBufferEcu::getRtData()
{
 //m_buff;
   //Logger::logf(LOG_MSG, _T("RTDataInputRingBufferEcu::Entry() getRtData()"));
    uint8_t* m_buff = (uint8_t*) m_ringBuffer->getHeadPointer(); // not increment the ringbuffer head index
    //Logger::logf(LOG_MSG, _T("RTDataInputRingBufferEcu::Entry() writeRingBuffer() %p"), m_RTBuff);

//     char * tmp = "111111111111111111111111111111111111111111112222222222222222222222222222222222222222222222";
//     memcpy ( m_RTBuff, tmp,  strlen(tmp) );


 //Logger::logf(LOG_MSG, _T("RTDataInputRingBufferEcu::Entry() writeRingBuffer() 1"));
     bool connected = (m_state == ECUEvent::CONNECTED || m_state == ECUEvent::AIM_MODE);
    int res =-1;
    try{
        //Logger::logMilli(LOG_MSG,_T("RTDataInputRingBufferEcu::getRtData() serialRead START"));
//        res=  m_vp->getRuntimeData(m_buff, !connected);
        res=  m_vp->getECUData(m_buff, !connected);
        //if( connected )
        //{
            int rtLength = RealTimeDesc::getDataSize();
            m_symbolMgr->setGroupOffset( _T("GPSData"), rtLength );
            m_gpsData->get( &m_buff[ rtLength ] );
        //}

        //Logger::logMilli(LOG_MSG,_T("RTDataInputRingBufferEcu::getRtData() serialRead END"));
    }catch (SerialException& se){
         Logger::logf(LOG_MSG, _T("RTDataInputRingBufferEcu::Entry() writeRingBuffer() SerialException"));
        //Thread::Sleep(1000);

//        Logger::logf(LOG_MSG,_T("RTDataInputRingBufferEcu::getRtData() serialRead exception ")+ se.toString() );
        return ECUEvent::NOT_CONNECTED;
    }catch (...){
       // Logger::logf(LOG_MSG, _T("RTDataInputRingBufferEcu::Entry() writeRingBuffer() Unknown Exception"));
    }

    //Logger::logf(LOG_MSG, _T("RTDataInputRingBufferEcu::getRtData() res=%i"), res);
    if(res < 0){
        return ECUEvent::NOT_CONNECTED;
    }else if(res == ECUInfo::BOOT){//stay in boot mode
        return ECUEvent::BOOT_MODE;
    }

    //TODO:getFirmwareInfo
    //writeHeader
    //writeData

    if(m_state != ECUEvent::CONNECTED){
         //readFirmwareInfo(5, true);
         m_rtDataRawLogger->setHeaderWritten(false);
    }

    if(m_rtDataRawLogger->isRawLog() && res == ECUInfo::MEGATUNE ){
        if( !m_rtDataRawLogger->isHeaderWritten() ) { writeHeader(); }
        /*memset( m_RTBuff, 'A', 2 );
        memset( &m_RTBuff[2], 'B', 52 );
        memset( &m_RTBuff[54], 'C', 2 );
        m_rtDataRawLogger->Write( m_RTBuff );*/

        m_rtDataRawLogger->Write( (const char*) m_buff );

    }

    if( isActive() && !m_ringBuffer->isFull() ){
        //memcpy(m_buff, m_RTBuff, 256);
        m_ringBuffer->nextHead();
        char* tmp = m_ringBuffer->getHeadPointer();
        memcpy(tmp, m_buff, 256);
        m_isNotEmpty->Broadcast();
        //Logger::logf(LOG_MSG, _T("RTDataInputRingBufferEcu::Entry() getRtData() Broadcast"));
    }

    if(res == ECUInfo::MEGATUNE ) return ECUEvent::CONNECTED;
    if(res == ECUInfo::AIM ) return ECUEvent::AIM_MODE;

    return ECUEvent::CONNECTED;
}
/*
bool RTDataInputRingBufferEcu::readFirmwareInfo(uint8_t trycount, bool force)
{

    bool result = false;

    if(!force && m_fwInfo != NULL) return true;

    m_fwInfo = NULL;

    for(char a=0; a < trycount ; a++){

        try{
             m_fwInfo = m_vp->getFirmwareInfo();
             break;
        }catch(SerialException se){
            continue;
        }

    }

    if(m_fwInfo != NULL) result = true;

    return result;

}
*/

void RTDataInputRingBufferEcu::writeHeader()
{
    Logger::logf(LOG_MSG,_T("RTDataInputRingBufferEcu::writeRingBuffer switched START"));
        m_rtDataRawLogger->Close();
        m_RTLen = RealTimeDesc::getDataSize();

        //readFirmwareInfo(5, true);

//            if(fwInfo == NULL) Logger::logf(LOG_MSG,_T("FirmwareInfo *fwInfo = %p"),fwInfo);

        m_rtDataRawLogger->SetBlockSize(512);
        m_rtDataRawLogger->SetSize(m_RTLen);
        //m_rtDataRawLogger->SetFile(DirHelper::GetPath(dLOG) + _T("rawECUlog.txt"));

        int device = m_vp->getDevice();
        wxString deviceStr = _T("ECU");
        if(device != ECUInfo::DEVICE_UNKNOWN) deviceStr  = ECUInfo::getDeviceAsString( device );

        wxString fileName = DirHelper::CreateNewFileName(deviceStr+_T("-log.vemslog"),DirHelper::GetPath(dLOG),true,true);
         Logger::logf(LOG_MSG,_T("RTDataInputRingBufferEcu::writeHeader() fileName: ")+ fileName);
        m_rtDataRawLogger->SetFile( fileName);
        //m_rtDataRawLogger->SetLog(true);

        m_rtDataRawLogger->AddHeaderItem(RawLogHelper::dataLen,wxString::Format(_T("%i"),m_RTLen));
        m_rtDataRawLogger->AddHeaderItem(RawLogHelper::blockSize,wxString::Format(_T("%i"),512));
/*
        if(m_fwInfo != NULL){
            m_rtDataRawLogger->AddHeaderItem(RawLogHelper::fwVersion,m_fwInfo->getFirmwareVersion());
            m_rtDataRawLogger->AddHeaderItem(RawLogHelper::fwcompTime,m_fwInfo->getCompilationTime());
            m_rtDataRawLogger->AddHeaderItem(RawLogHelper::fwSN,m_fwInfo->getSerialNumber());
        }
*/
        m_rtDataRawLogger->WriteHeader();
}
