#include "RTDataMgrLogSeq.h"

#include "../logger.h"
#include "../Utils.h"

RTDataMgrLogSeq::RTDataMgrLogSeq(const wxString& strFile)
:m_symbolNames (NULL)
{
    //ctor

    m_fileName = strFile;

    initFile();

}

RTDataMgrLogSeq::~RTDataMgrLogSeq()
{
    //dtor
    delete m_symbolNames;
    delete m_row;
}

void RTDataMgrLogSeq::reset()
{
    closeFile();

    initFile();

}

void RTDataMgrLogSeq::initFile()
{

    int res = parse(m_fileName, false); // NON Batch mode

    if(res < 0) return;

    // the header is the first line
    while(m_symbolNames == NULL) parseNextLine(); //fill the headers
    m_row = new DoubleArray();

}

wxArrayString * RTDataMgrLogSeq::getVEMSDataLogHeader()
{
    return m_symbolNames;
}

/** @brief isExist
  *
  * @todo: document this function
  */
bool RTDataMgrLogSeq::isExist(const wxString& symbolname)
{
    if(m_symbolNames == NULL)
        return true;

    if(m_symbolNames->Index(symbolname,false) == wxNOT_FOUND){
        return false;
    }
    return true;
}

/** @brief getValue
  *
  * @todo: document this function
  */
double RTDataMgrLogSeq::getValue(const wxString& symbolname)
{
    int idx = m_symbolNames->Index(symbolname,false);

    if(idx == wxNOT_FOUND){
		Logger::log(LOG_MSG,wxString::Format(_T("symbol not found : %s"), symbolname.c_str()));
        //wxLogMessage(wxString::Format(_T("symbol not found : %s"), symbolname.c_str()));     //c wince
        return -1; //need exception?
    }
    double d = m_row->Item(idx);
    //wxLogMessage(wxString::Format(_T("count %d, max %d, value %f"), m_count, m_maxCount, d));
//	Logger::log(LOG_MSG,wxString::Format(_T("count %d, max %d, value %f"), m_count, m_maxCount, d));
    return d;

}

RTDataInputType RTDataMgrLogSeq::getType()
{
    return INPUT_TYPE_FILE;
}

/** @brief refresh
  *
  * @todo: document this function
  */
int RTDataMgrLogSeq::updateViewerModel()
{
//    Logger::logf(LOG_MSG,_T("RTDataMgrLogSeq::refresh() START"));
//    Logger::logf(LOG_MSG,_T("RTDataMgrLogSeq::refresh() 1"));

    for(DoubleHash::iterator it = m_RegisteredSymbolTable->begin(); it != m_RegisteredSymbolTable->end(); it++){

  //      Logger::logf(LOG_MSG,_T("RTDataMgrLogSeq::refresh() for 2 "));

        wxString key = it->first;

//        Logger::logf(LOG_MSG,_T("RTDataMgrLogSeq::refresh() for key = ") + key );

        int idx = m_symbolNames->Index(key,false);

        //wxLogMessage(it->first);
        if( idx == wxNOT_FOUND){
      //      Logger::logf(LOG_MSG,_T("RTDataMgrLogSeq::refresh() f3"));
           // m_RegisteredSymbolTable->erase(it);

        }else{
            //Logger::logf(LOG_MSG,_T("RTDataMgrLogSeq::refresh() f4"));
            it->second = m_row->Item(idx);
        }
    }

    //Logger::logf(LOG_MSG,_T("RTDataMgrLogSeq::refresh() END"));

    return 0;
}

int RTDataMgrLogSeq::updateInputModel()
{
    parseNextLine();
    return 0;
}

void  RTDataMgrLogSeq::parseLine()
{

    if( m_lineNumber < 1) return;
    //The first line define the Column heeaders
    // parse theheader only once in bach parse
    if( m_lineNumber == 1 && ( m_symbolNames == NULL || m_symbolNames->Count() == 0) ){
        m_symbolNames = new wxArrayString();
        //bool res =
        Utils::tokenizeToArray(m_line,_T("\t"), *m_symbolNames);
        return;
    }

    wxArrayString rowStrArray;
    //bool res =
    Utils::tokenizeToArray(m_line,_T("\t"), rowStrArray);

	// header and values count do not match
	if(rowStrArray.Count() != m_symbolNames->Count())
	{
        return;
    }

    //DoubleArray * rowArray = new DoubleArray();
    // parse strings to double

    m_row->Clear();
    for(uint16_t i= 0; i < rowStrArray.Count(); i++){
        wxString  rowStr = rowStrArray.Item(i);
        double value = -1;
        rowStr.ToDouble(&value);
        m_row->Add(value);
    }
}

bool RTDataMgrLogSeq::isValid()
{
    return true;
}
