#include "RTDataRawLogger.h"

#include "../logger.h"
#include "../RawLogHelper.h"
#include "../GPSData.h"
#include "../FirmwareInfo.h"

RTDataRawLogger::RTDataRawLogger(FirmwareInfo* fwInfo, GPSData* gps)
    : m_isLog (false)
    , m_bufferCount (0)
    , m_buffer (NULL)
    , m_fos (NULL)
{
    //ctor
    m_isHeaderWritten = false;
    //m_gpsDataSize   = 0;
    m_gpsLog        = false;
    m_gpsData       = gps;
    m_fwInfo       = fwInfo;

    //some default values
    m_bufferSize = 512;
    m_dataSize = 56;
    m_RTDataIdx = 0;
}


RTDataRawLogger::~RTDataRawLogger()
{
    //dtor
    delete[] m_buffer;
    delete m_fos;
}

void RTDataRawLogger::SetFile(const wxString& ofileName)
{
    m_fileName = ofileName;

    //m_fos = new wxFileOutputStream(ofileName);
    //m_bfos = new wxBufferedOutputStream(m_bfos);
    //m_bfos->SetBufferIO(512);
}

void RTDataRawLogger::SetSize(int size)
{
    m_dataSize = size;
    calcMaxRTDataCount();
}

void RTDataRawLogger::SetBlockSize(int bSize)
{
    m_bufferSize = bSize;
    if (m_buffer != NULL) delete[] m_buffer;
    m_buffer = new char[m_bufferSize];
    fillBuffer();
    m_bufferCount = 0;
    calcMaxRTDataCount();
}

void RTDataRawLogger::SetLog(bool b)
{
    m_isLog = b;
    m_isHeaderWritten = false;
}

bool RTDataRawLogger::isRawLog()
{
    return m_isLog;
}

void RTDataRawLogger::Close()
{

    if(m_fos == NULL) return;

    if(m_bufferCount  != 0){
        m_fos->Write(m_buffer, m_bufferSize);
    }
    m_fos->Close();

}

void RTDataRawLogger::captureEcuBlock()
{
    //m_CapturedBufferCount = m_bufferCount;
    //m_gpsData->captureEcuBlock(m_bufferCount/m_dataSize-1);

    m_gpsData->captureEcuBlock(m_RTDataIdx);
}

void RTDataRawLogger::calcMaxRTDataCount()
{
    int gpsSize = m_gpsLog ? m_gpsData->length() : 0;
    m_maxRTDataCount = (m_bufferSize - gpsSize) / m_dataSize;
    //Logger::logf(LOG_MSG,_T("RTDataRawLogger::calcMaxRTDataCount() m_maxRTDataCount: %i"), m_maxRTDataCount );
}

int RTDataRawLogger::getRecordCountInBlock() const
{
    return m_maxRTDataCount;
}

int RTDataRawLogger::getRecordSize() const
{
    return m_dataSize;
}

bool RTDataRawLogger::getNextRTDataIdx()
{
    if(m_RTDataIdx == m_maxRTDataCount){
        m_RTDataIdx = 0;
        return true;
    }
    m_RTDataIdx++;
    return false;
}

void RTDataRawLogger::Write(const char * buff)
{
    Write(buff,m_dataSize);
}

void RTDataRawLogger::Write(const char * buff, int size)
{
    if(m_fos == NULL) return;
    if( getNextRTDataIdx() )
    {
        //Logger::logf(LOG_MSG,_T("RTDataRawLogger::Write() 10"));
        WriteGPSData();
        Flush();
        m_gpsData->captureEcuBlock(0xFF); /// Reset
    }
    memcpy( &(m_buffer[m_bufferCount]), buff, size);
    m_bufferCount += size;
}


void RTDataRawLogger::Flush()
{
    m_fos->Write(m_buffer, m_bufferSize);
    fillBuffer();
    m_bufferCount = 0;
}

void  RTDataRawLogger::Flush(uint8_t* data)
{
     m_fos->Write(data, m_bufferSize);
}



void RTDataRawLogger::WriteGPSData()
{
    if( !( m_gpsLog && m_isLog ) || !m_buffer )
    {
        return;
    }

    int gpsIdx  = 0;
    int remains = m_bufferSize % m_dataSize;

    if( remains < m_gpsData->length() )
    {
        gpsIdx = m_bufferSize - m_dataSize - remains;
    }
    else
    {
        gpsIdx = m_bufferSize - remains;
    }

    if( m_gpsData )
    {
        m_gpsData->get( (uint8_t*)&m_buffer[ gpsIdx ] );
    }
    //memcpy( &m_buffer[ gpsIdx ], buffer, m_gpsDataSize );
}

void RTDataRawLogger::fillBuffer()
{
    int i;
    for(i = 0; i< m_bufferSize; i++)
    {
        m_buffer[i] = 0;
    }
    m_buffer[m_bufferSize-1] = '\n';
}


void RTDataRawLogger::WriteHeader()
{
    fillHeader();

    if(m_fos != NULL){ delete m_fos; m_fos = NULL;}
    m_fos = new wxFileOutputStream(m_fileName);

    wxString headerInfo = _T("# Vems Log Header, 512 byte\n");

    WriteHeader(headerInfo.mb_str(),headerInfo.Len());

    AddHeaderItem( RawLogHelper::gpsDataLen, wxString::Format( _T("%i"), ( m_gpsLog?m_gpsData->length():0 ) ) );

    for(StringHash::iterator it = m_headerHash.begin(); it != m_headerHash.end(); it++) {
        wxString tmp = it->first + _T("=") + it->second + _T("\n");
        WriteHeader(tmp.mb_str(),tmp.Len());
    }
    Flush();
    m_isHeaderWritten = true;
}

void RTDataRawLogger::WriteHeader(const char * buff, size_t size)
{
    if(m_fos == NULL) return;
    memcpy( &(m_buffer[m_bufferCount]), buff, size);
    m_bufferCount += size;
}

void RTDataRawLogger::AddHeaderItem(const wxString &key, const wxString &value)
{
    m_headerHash[key] = value;
}

void RTDataRawLogger::fillHeader()
{
    if(m_fwInfo != NULL){
        AddHeaderItem(RawLogHelper::fwVersion,m_fwInfo->getFirmwareVersionAsString());
        AddHeaderItem(RawLogHelper::fwcompTime,m_fwInfo->getCompilationTime());
        AddHeaderItem(RawLogHelper::fwSN,m_fwInfo->getSerialNumber());
        AddHeaderItem(RawLogHelper::signature,m_fwInfo->getSignature());
    }
}
