#ifndef ACCSETMATRIX_H
#define ACCSETMATRIX_H

#include <vector>
#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/weighted_sum.hpp>
#include <boost/accumulators/statistics/weighted_mean.hpp>
#include <boost/accumulators/statistics/weighted_variance.hpp>
#include <boost/accumulators/statistics/weighted_density.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/range/iterator_range.hpp>

enum STAT { STAT_COUNT = 0, STAT_MIN, STAT_MAX, STAT_MEAN, STAT_VARIANCE };

using namespace boost;
using namespace boost::accumulators;

#ifdef _MSC_VER //VC doesn't like droppable density
typedef accumulator_set<double, features< droppable<tag::count>, droppable<tag::max>,
    droppable<tag::weighted_mean>, droppable<tag::min>, droppable<tag::weighted_variance>, stats<tag::weighted_density> >, double > AccSetT;
#else
typedef accumulator_set<double, features< droppable<tag::count>, droppable<tag::max>,
    droppable<tag::weighted_mean>, droppable<tag::min>, droppable<tag::weighted_variance>, droppable<tag::weighted_density> >, double > AccSetT;
#endif

/*typedef accumulator_set<double, features< droppable<tag::count>, droppable<tag::max>,
    droppable<tag::mean>, droppable<tag::min>, droppable<tag::variance>, droppable<tag::density> > > AccSetT;*/

typedef iterator_range < std::vector< std::pair<double, double> >::iterator> histogram_type;

class SimpleIntMatrix
{
    public:
        SimpleIntMatrix();
        SimpleIntMatrix(size_t rows, size_t cols);
        ~SimpleIntMatrix();

        void        setDims(size_t rows, size_t cols);
        void        clear();

        size_t    getCols() { return m_cols; }
        size_t    getRows() { return m_rows; }
        void        setData(size_t index, int value);
        int&        operator[] (size_t index) throw(int);
        int         sumOfElements();
    private:
        size_t      m_rows;
        size_t      m_cols;
        int*        m_data;
};

class AccSetMatrix
{
    public:
        AccSetMatrix(size_t rows, size_t cols);
        ~AccSetMatrix();

        size_t    getCols() { return m_cols; }
        size_t    getRows() { return m_rows; }
        size_t    size() { return (m_cols*m_rows); }

        void      reinit_cell( size_t row, size_t col );

        AccSetT*    getData(size_t row, size_t col);
        AccSetT*    getData(size_t index);
        AccSetT*    operator[] (size_t index);

    private:
        size_t      m_rows;
        size_t      m_cols;
        AccSetT**   m_data;
};

#endif // ACCSETMATRIX_H
