#ifndef RTDATASTAT_H
#define RTDATASTAT_H

#include <wx/progdlg.h>
#include <wx/spinctrl.h>

#include "../Symbol.h"
#include "../stdatamanager/STDataViewerBase.h"
#include "../rtdatamanager/RTDataViewerBase.h"
#include "../vemsdialog/StaticViewersController.h"
#include "../events/InputEvent.h"
#include "../VemsTuneController.h"

#include "../common/VemsMatrix.h"
#include "../common/types/wxObjectArray.h"

#include "../common/types/DoubleHashArray.h"

#include "../loghistory/LogFilter.h"

#include "../vemsprotocol/VEMSLogBuffer.h"
#include "../vemsprotocol/SIPRPacket.h"

#include "../gaugedesc/GaugeDescHelper.h"

#include "AccSetMatrix.h"

#include "RTCellInfo.h"
#include "RTDataStatTable.h"
#include "RTDataStatMath.h"

class STDataMgrBase;
class SymbolMgr;
class LogHistoryProcessFilterThread;
class LogHistoryBase;
class VemsTableDesc;
class Observable;
class RTCellInfo;
class wxToggleButton;

struct cell;

class RTDataStat : public wxFrame, public RTDataViewerBase, public ObserverBase, public LogFilter, public Observable
{
    public:
        RTDataStat(wxWindow * parent, LogHistoryBase* loghistory, Observable* loghistoryobs, VemsTableDesc* tableDesc, bool isTuneMode = false);
        ~RTDataStat();

        void load( unsigned int start = 0, int end = -1);
        void useWeighting( bool useweighting ) { m_useWeighting = useweighting; }
        void useOutlierFiltering( bool usefiltering ) { m_useOutlierFiltering = usefiltering; }

        void notify( int i );
        void notify(int i, int idx);
        void notify(const wxString& str);
        void notify( int i, void* classT );

    protected:

    private:
        bool                      m_tuneMode;
        bool                      m_useWeighting;
        bool                      m_useOutlierFiltering;
        bool                      m_registered;
        LogHistoryBase*           m_loghistory;
        Observable*               m_loghistoryobserv;
        STDataMgrBase*            m_stDataMgr;
        SymbolMgr*                m_symbolMgr;
        VemsTableDesc*            m_tableDesc;
        Symbol*                   m_xBinSymbol;
        Symbol*                   m_yBinSymbol;
        Symbol*                   m_xRTSymbol;
        Symbol*                   m_yRTSymbol;
        unsigned int              m_rows, m_cols;

        int                       m_start, m_end;
        size_t                    m_statLen;

        GaugeDescManager*         m_gdmanager;
        unsigned long             m_index;

        DoubleArray               m_HorizLimits, m_VertLimits;
        DoubleHashArray           m_data;

        VemsMatrix<DoubleArray>*  m_prec_data;
        VemsMatrix<DoubleArray>*  m_prec_data_weight;
        VemsMatrix<double>*       m_prec_data_min;
        VemsMatrix<double>*       m_prec_data_max;
        VemsMatrix<double>*       m_prec_data_mean;
        VemsMatrix<double>*       m_prec_data_variance;

        AccSetMatrix*             m_statMatrix;
        AccSetMatrix*             m_ratioMatrix;
        SimpleIntMatrix*          m_throwedDataM;
        SimpleIntMatrix*          m_outliersDataM;
        unsigned                  m_outofrange;

        double                    m_varianceMult;

        RTDataStatTable*          m_guitable;
        RTCellInfo*               m_cellinfo;


        wxMenu*                   m_popupmenu;
        wxString                  m_requiredSymbol;

        size_t                    m_mincount;
        size_t                    m_maxcount;

        wxStaticText*             m_expressionText;
        wxComboBox*               m_expressionEdit;
        wxStaticText*             m_filterText;
        wxComboBox*               m_filterEdit;
        wxStaticText*             m_dataInfoText;
        wxTextCtrl*               m_dataInfoEdit;
        wxSpinCtrl*               m_minDataSpin;
        wxTextCtrl*               m_varianceMultTxt;

        wxComboBox*               m_delay;
        wxToggleButton*           m_showVEratio;

        cell                      m_displaycell;
        cell                      m_selTopLeft;
        cell                      m_selBottomRight;

        Int2StringHash            m_symbolhash;

        LogHistoryProcessFilterThread*         m_logHistoryProcessFilterThread;

        int                       m_xmin, m_xmax, m_ymin, m_ymax;

        wxProgressDialog*         m_progressdlg;
        bool                      m_showProgdlg;

        StringHashSet             m_requiredSymbols;
        RPNParser                 m_rpnParser;

        void update() {
        }

        /// controller
        void requestRead() {
        }
        void updateModel() {
        }
        void requestWrite() {
        }

        void buildGUI(wxWindow * parent);

        void SetValue(double value) {}
        void refresh(DoubleHash* table);
        void require();
        void unregisterSymbols();
        void SetValueIdx(int idx) {}

        void SetColors();
        void SetColors( const int& coloring_type );

        void fixSelectedCells();

        /// notify -> EVENT
        void OnNotifyEvtHandler( wxCommandEvent& event );

        /// event handlers
        void OnRightClick( wxMouseEvent& event);
        void OnSymbolSelect( wxCommandEvent& event );
        void OnClose( wxCloseEvent& event );
        void OnEvaluate( wxCommandEvent& event );

        void OnExprButton( wxCommandEvent& event );
        void OnSpinCtrl( wxSpinEvent& event );

        void OnRefreshTables( wxCommandEvent& event );
        void OnSelectAll( wxCommandEvent& event );
        void OnFocus( wxFocusEvent& event );
        void OnShowVEcorrRation( wxCommandEvent& event );

        ///
        void insertIntoMatrix( int xval, int yval, double rtDataValue, bool passedFilter );
        void doOutLierFiltering();
        void processData(const DoubleHash& data, bool passedFilter);
        inline double getVE_Value( int row, int col ) const;

        DECLARE_EVENT_TABLE()
};

#endif // RTDATASTAT_H
