#include "RTDataStatDistributionWin.h"

#include <wx/dcbuffer.h>
// #include <wx/gdicmn.h>
#include <wx/arrimpl.cpp>
#include "RTDataStatMath.h"
#include "../UnicodeHelper.h"

#include "../logger.h"

#include "RTPlotter.h"

BEGIN_EVENT_TABLE( RTDataStatDistributionWin, wxFrame )
    EVT_SIZE( RTDataStatDistributionWin::ReDraw )
    EVT_PAINT( RTDataStatDistributionWin::OnPaint )
    EVT_MOTION( RTDataStatDistributionWin::OnMouseMove )
    EVT_CLOSE( RTDataStatDistributionWin::OnClose )
END_EVENT_TABLE()

WX_DEFINE_OBJARRAY( RealPointArrayA )

RTDataStatDistributionWin::RTDataStatDistributionWin( wxWindow *parent, const wxPoint& pos, const wxSize& size )
    : wxFrame( parent, -1, _T("Real-time data - Histogram & distributions"), pos, size, wxSYSTEM_MENU | wxCLOSE_BOX | wxCAPTION | wxRESIZE_BORDER | wxFRAME_FLOAT_ON_PARENT )
{
    m_minV = 0.0;
    m_maxV = 0.0;
    m_startCIdx = 0;
    m_ymax = 1.0;
    SetBackgroundColour( wxColor( 0, 0, 0) );

    m_plotter = new RTPlotter( NULL, this );

    SetIcon(wxIcon(_T("vemsLogoIcon2")));
}

RTDataStatDistributionWin::~RTDataStatDistributionWin()
{
    if ( m_plotter != NULL )
    {
        delete m_plotter;
        m_plotter = NULL;
    }
}

void RTDataStatDistributionWin::SetYAxisMax( double ymax )
{
    m_ymax = ymax;
    if ( m_ymax < 0.1 ) m_ymax = 0.1;
    else if ( m_ymax < 0.3 ) m_ymax = 0.3;
    else if ( m_ymax < 0.5 ) m_ymax = 0.5;
    else if ( m_ymax < 0.75 ) m_ymax = 0.75;
    else m_ymax = 1.0;
}

void RTDataStatDistributionWin::AddCurve( RealPointArray curve, const wxString & curvename, wxColour color )
{
    m_hasData = true;

    m_curves.Add( curve );
    m_curvenames.Add( curvename );

    m_curveColors.Add( color );
}

void RTDataStatDistributionWin::ClearCurves()
{
    m_curves.Clear();
    m_curveColors.Clear();
    m_curvenames.Clear();
    m_ymax = 1.0;

    Refresh( false );
}

void RTDataStatDistributionWin::ReDraw( wxSizeEvent& event )
{
    Refresh( false );
    event.Skip();
}

void RTDataStatDistributionWin::OnPaint( wxPaintEvent& event )
{
    wxBufferedPaintDC dc( this );
    dc.SetBackground( wxColor(255,255,255) );
    dc.Clear();

    m_plotter->SetDC( &dc );

    int height, width;

    GetSize( &width, &height );

    height = height - 20;

    m_plotter->SetViewPortSize( width, height );

    m_plotter->CreateCanvas();
    m_plotter->SetFillColor( wxColor(102,153,204,208));
    m_plotter->ClearCurves();
    m_plotter->SetViewPortLimits( m_minV, m_maxV, 0, 1.0 );

    // no data, no curve
    if ( m_hasData == false )
    {
        wxFont fnt( 24, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD );
        dc.SetFont(fnt);
        dc.SetTextForeground( wxColor(160, 160, 160) );
        const wxString& nd = _T("No data");
        wxSize s = dc.GetTextExtent( nd );
        dc.DrawText( nd, (width-s.x)/2, (height-s.y)/2 );
        return;
    }

    // draw histogram
    if ( m_histCount > 0 )
    {
        double hMin = 9E25, hMax = -9E25;
        for ( size_t i = 0; i < m_histCount; ++i )
        {
            if ( m_histData[i] > hMax ) hMax = m_histData[i];
            if ( m_histData[i] < hMin ) hMin = m_histData[i];
        }
        m_plotter->DrawHistogram( m_histData, m_minV, m_maxV, 15, true, true );
    }

    // draw gaussian curves
    for ( unsigned char c = 0; c < m_curves.GetCount(); ++c )
    {

        m_plotter->AddCurve( m_curves[c], m_curveColors[c], m_curvenames[c], false );
    }
    m_plotter->DrawCurves( false, false );
}

void RTDataStatDistributionWin::SetHistogram( double minval, double maxval, const DoubleArray& data )
{
    m_histMin = minval;
    m_histMax = maxval;
    m_histCount = data.GetCount();

    m_histData = data;

    m_hasData = true;
}

void RTDataStatDistributionWin::DrawGraphs()
{
    Refresh( false );
}

void RTDataStatDistributionWin::DrawEmpty()
{
    m_hasData = false;
    Refresh( false );
}

void RTDataStatDistributionWin::OnMouseMove( wxMouseEvent& event )
{
    int x = event.GetX(),
        y = event.GetY();

    if ( m_plotter != NULL )
    {
        double xval, yval;
        if ( m_plotter->GetXYValue( x, y, xval, yval ) )
        {
            SetCursor( wxCURSOR_CROSS );
            m_plotter->SetMousePos( event.GetPosition() );
        }
        else SetCursor(wxCURSOR_ARROW);
    }
}

void RTDataStatDistributionWin::OnClose( wxCloseEvent& event )
{
    Hide();
}
