#include "RTDataStatTable.h"
#include "../logger.h"
#include "../gaugedesc/GaugeDescHelper.h"
#include "../gaugedesc/GaugeDescManager.h"

#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/mean.hpp>
#include <boost/accumulators/statistics/max.hpp>
#include <boost/accumulators/statistics/min.hpp>
#include <boost/accumulators/statistics/count.hpp>
#include <boost/accumulators/statistics/variance.hpp>
#include <boost/accumulators/statistics/density.hpp>

using namespace boost;
using namespace boost::accumulators;

BEGIN_EVENT_TABLE( RTDataStatTable, wxPanel )
    EVT_PAINT( RTDataStatTable::OnPaint )
    EVT_SIZE ( RTDataStatTable::OnSize )

    EVT_MOUSE_EVENTS( RTDataStatTable::OnMouse )
    EVT_RIGHT_DOWN ( RTDataStatTable::OnRClick )
    EVT_LEFT_DOWN ( RTDataStatTable::OnLeftButton )

    EVT_KEY_DOWN ( RTDataStatTable::OnKeyDown )
    EVT_SET_FOCUS( RTDataStatTable::OnFocusEvent )
END_EVENT_TABLE();

wxString strNA = _T("N/A");

const int idStatMenuCount = wxNewId();
const int idStatMenuMin   = wxNewId();
const int idStatMenuMax   = wxNewId();
const int idStatMenuMean  = wxNewId();
const int idStatMenuVar   = wxNewId();
const int idShowHistogram = wxNewId();

#define SELECTION_COLOR wxColor(128,128,255)

RTDataStatTable::RTDataStatTable(wxWindow* parent, int width, int height, const wxArrayString& xbins, const wxArrayString& ybins, const wxString& reqsymbol, bool tuneMode)
    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxSize(width, height),0)
{
    m_forcedNumFormat = _T("");
    m_tuneMode = tuneMode;
    m_width = width; m_height = height;
    m_xbins = xbins; m_ybins = ybins;
    m_cols = xbins.GetCount();
    m_rows = ybins.GetCount();

    m_FixedCells.setDims( m_rows, m_cols );

    m_cellw = (int) ((width - ( m_cols + 1)*2) / (m_cols + 1));
    m_cellh = (int) ( (height - ( m_rows + 1)*2) / (m_rows + 1));

    m_cellsize.SetWidth(m_cellw); m_cellsize.SetHeight(m_cellh);

    m_labelBkColor = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
    m_matrix = NULL;
    m_statisticalfeat = STAT_MEAN;
    m_calculated = false;

    m_popupmenu = NULL;

    m_descmgr = GaugeDescHelper::GetGDMgr();
    m_symbolname = reqsymbol;

    resetColorGradients();

    m_highlightedCell.col = -1; m_highlightedCell.row = -1;
    m_selectedCell.col = -1; m_selectedCell.row = -1;

    m_mincount = 0; m_maxcount = 9000000;
    m_isSelecting = false;

    m_startcell = invalid_cell;
    m_endcell = invalid_cell;
    m_returnfocus = false;
}

RTDataStatTable::~RTDataStatTable()
{
    //dtor
}

void RTDataStatTable::resetColorGradients()
{
    m_gradientColorArray.Clear();

    m_gradientColorArray.Add(wxColor(32,64,255));
    m_gradientColorArray.Add(wxColor(32, 128, 32));

    m_gradientColorArray.Add(wxColor(224, 224, 32));

    m_gradientColorArray.Add(wxColor(128,64,0));
    m_gradientColorArray.Add(wxColor(224, 128, 64));

    m_customcolors = false;

    if ( m_matrix != NULL )
        Refresh(false);
}

void RTDataStatTable::OnPaint(wxPaintEvent& event)
{
    wxBufferedPaintDC dc(this);

    dc.SetBackground( m_bgColor );
    dc.Clear();

    dc.SetBrush(*wxWHITE_BRUSH); dc.SetPen(*wxWHITE_PEN);
    Draw3DBorder(dc,wxRect(0,0, (m_cellw+2) * (m_cols+1), (m_cellh+2) * (m_rows +1)));
    dc.DrawRectangle(m_cellw+3, m_cellh+3, (m_cellw+2) * m_cols -2, (m_cellh+2) * m_rows -2 );
    Draw3DCells(dc);

    if ( m_matrix != NULL )
        DrawData(dc);

}

void RTDataStatTable::Draw3DCells(wxDC& dc)
{
    wxBrush oldbrush = dc.GetBrush();
    dc.SetPen(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    dc.SetBrush( m_labelBkColor );

    // get text point size
    wxFont font( 1, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD);
    int w1 = CalcFontSize(dc,wxRect(0,0,m_cellw,m_cellh), m_xbins[m_xbins.GetCount()-1],font); /// need to discuss
    int w2 = CalcFontSize(dc,wxRect(0,0,m_cellw,m_cellh), m_ybins[0],font);
    int fontsize = wxMin(w1,w2);
    font.SetPointSize(fontsize);
    dc.SetFont(font);
    int i = 0, xpos = 0;
    while ( i < m_cols+1 )
    {
        Draw3DBorder(dc,wxRect(xpos, 0, m_cellw+2, m_cellh+2));

        // draw
        dc.SetTextForeground(wxColor(0,0,0));
        if ( i > 0)
        {
            wxSize s = dc.GetTextExtent(m_xbins[i-1]);
            dc.DrawText( m_xbins[i-1], xpos + ( m_cellw - s.x)/2, (m_cellh - s.y)/2 );
        }


        xpos += ( m_cellw + 2);
        i++;
        if ( i > 1 && i < m_cols+1)
            dc.DrawLine( xpos, m_cellh+2, xpos, (m_cellh+2)* ( m_rows+1) -1 );
    }
    i = 1; size_t ypos = m_cellh+2;
    while ( i < m_rows+1 )
    {
        Draw3DBorder(dc, wxRect(0, ypos, m_cellw+2, m_cellh+2));

        dc.SetTextForeground(wxColor(0,0,0));
        wxSize s = dc.GetTextExtent(m_ybins[i-1]);
        dc.DrawText( m_ybins[i-1], ( m_cellw - s.x)/2, ypos + (m_cellh - s.y)/2 );

        ypos += ( m_cellh + 2);
        i++;
        if ( i > 1 && i < m_rows+1)
            dc.DrawLine( m_cellw+2, ypos, (m_cellw+2)* ( m_cols+1) -1, ypos );
    }
}

void RTDataStatTable::setBgColor(wxColour color)
{
    m_bgColor = color;
    Refresh();
}

void RTDataStatTable::resizeTo(int width, int height)
{
    m_width = width;
    m_height = height;
    m_cellw = (int) ((width - ( m_cols + 1)*2) / (m_cols + 1));
    m_cellh = (int) ( (height - ( m_rows + 1)*2) / (m_rows + 1));
    m_cellsize.SetWidth(m_cellw); m_cellsize.SetHeight(m_cellh);
    Refresh(false);
}

void RTDataStatTable::Draw3DBorder(wxDC& dc, const wxRect& rc)
{
	wxPen oldPen = dc.GetPen();

	dc.SetPen( wxPen(wxSystemSettings::GetColour(wxSYS_COLOUR_3DSHADOW), 1, wxSOLID));
	dc.DrawLine(rc.x, rc.y, rc.GetRight() + 1, rc.y);
	dc.DrawLine(rc.x, rc.y, rc.x, rc.GetBottom() + 1);
	dc.DrawLine(rc.GetRight(), rc.GetBottom(), rc.GetRight(), rc.y);
	dc.DrawLine(rc.GetRight(), rc.GetBottom(), rc.x, rc.GetBottom());

	dc.SetPen(*wxWHITE_PEN);
	dc.DrawLine(rc.x + 1, rc.y + 1, rc.GetRight(), rc.y + 1);
	dc.DrawLine(rc.x + 1, rc.y + 1, rc.x + 1, rc.GetBottom());

	dc.SetPen(oldPen);
}

void RTDataStatTable::OnSize(wxSizeEvent& event)
{
     unsigned w = event.GetSize().x, h = event.GetSize().y;
     resizeTo(w,h);
     event.Skip();
}

int RTDataStatTable::CalcFontSize(wxDC& dc, wxRect rect, const wxString& str, wxFont font)
{
    int w = rect.width, h = rect.height;
    wxCoord w1, h1;

    int pointsize = 1;
    do
    {
        font.SetPointSize( pointsize );
        dc.GetTextExtent( str, &w1, &h1, NULL, NULL, &font);
        if ( (w1>=w) || (h1>=h) )
            break;
        else ++pointsize;
    } while (1);

    return (pointsize-1);
}

void RTDataStatTable::setMatrix(AccSetMatrix* matrix)
{
    m_matrix = matrix;
    m_calculated = false;

    m_FixedCells.clear();

    Refresh();
}

void RTDataStatTable::selectStatisticalFeature(uint8_t feature)
{
    m_statisticalfeat = feature;
    m_calculated = false;
    Refresh();
}

int RTDataStatTable::CalcCellFontSize(wxDC& dc)
{
    wxFont font( 1, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    int fontsize = CalcFontSize(dc, m_cellsize, strNA, font);
    m_dStr = strNA;

    if ( m_statisticalfeat == STAT_COUNT )
    {
        int fs = CalcFontSize(dc, m_cellsize, wxString::Format(_T("%d"), m_tblmax), font);
        if ( fs < fontsize )
            fontsize = fs;
        return fontsize;
    }


    wxVariant data = m_descmgr->GetGaugeDescFieldBySymbolName( m_symbolname, _T("numformat"));

    if ( m_forcedNumFormat == wxEmptyString )
        m_numformat = data.GetString();
    else
        m_numformat = m_forcedNumFormat;


    data = m_descmgr->GetGaugeDescFieldBySymbolName( m_symbolname, _T("Lo"));
    wxString LoValue = wxString::Format( m_numformat, data.GetDouble());
    data = m_descmgr->GetGaugeDescFieldBySymbolName( m_symbolname, _T("Hi"));
    wxString HiValue = wxString::Format( m_numformat, data.GetDouble());

    int fs = CalcFontSize(dc, m_cellsize, LoValue, font);
    if ( fs < fontsize)
    {
        fontsize = fs; m_dStr = LoValue;
    }

    fs = CalcFontSize(dc, m_cellsize, HiValue, font);
    if ( fs < fontsize)
    {
        fontsize = fs; m_dStr = LoValue;
    }

    return fontsize;
}

void RTDataStatTable::GetTblMinMax(wxDC& dc)
{
    if ( m_matrix == NULL ) return;

    // initialization

    m_tblmin =  9E9;
    m_tblmax = -9E9;

    for ( size_t i = 0; i < m_matrix->getRows(); ++i )
    {
        for ( size_t j = 0; j < m_matrix->getCols(); ++j )
        {

            unsigned matindex = m_matrix->getCols() * i + j;
            AccSetT* acc_cell = m_matrix->getData(matindex);
            if ( count(*acc_cell)>0 || m_statisticalfeat == STAT_COUNT )
            {
                double value;
                size_t cellcount = count(*acc_cell);

                if ( cellcount < m_mincount || cellcount > m_maxcount )
                    continue;

                switch ( m_statisticalfeat )
                {
                    case STAT_MIN:
                        value = extract_result<tag::min>(*acc_cell);
                        break;
                    case STAT_MAX:
                        value = extract_result<tag::max>(*acc_cell);
                        break;
                    case STAT_MEAN:
                        value = extract_result<tag::mean>(*acc_cell);
                        break;
                    case STAT_VARIANCE:
                        value = sqrtf(extract_result<tag::variance>(*acc_cell));
                        break;
                    case STAT_COUNT:
                    default:
                        value = (double)extract_result<tag::count>(*acc_cell);
                        break;
                }
                if (m_tblmin > value) m_tblmin = value;
                if (m_tblmax < value) m_tblmax = value;
            }
        }
    }
    m_calculated = true;
}


void RTDataStatTable::DrawData(wxDC& dc)
{
    if ( m_matrix == NULL ) return;

    if ( !m_calculated )
        GetTblMinMax(dc);

    int fontsize = CalcCellFontSize(dc);

    wxFont font( 1, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    font.SetPointSize( fontsize-1 );
    dc.SetFont( font );

    wxString str;


    for ( size_t i = 0; i < m_matrix->getRows(); ++i )
    {
        for ( size_t j = 0; j < m_matrix->getCols(); ++j )
        {
            unsigned matindex = m_matrix->getCols() * i + j;
            AccSetT* acc_cell = m_matrix->getData(matindex);

            bool isFixed = m_FixedCells[matindex]!=0?true:false;

            size_t cellcount = count(*acc_cell);

            if ( (count (*acc_cell) == 0 && m_statisticalfeat != STAT_COUNT ) )
            {
                dc.SetTextForeground(wxColor(160,160,160));

                if ( isInSelection(i,j) )
                    dc.SetBrush( SELECTION_COLOR );
                else
                    dc.SetBrush(*wxWHITE);

                dc.SetPen(*wxWHITE);
                wxRect r = GetCellRect( i, j );
                dc.DrawRectangle( r.x, r.y, r.width, r.height );
                wxSize s = dc.GetTextExtent( strNA );
                dc.DrawText( strNA , (m_cellw+2)*(j+1)+1 + ( m_cellw - s.x)/2,
                                (m_cellh+2)*(i+1)+1 + (m_cellh - s.y)/2 );
                dc.SetTextForeground(wxColor(0,0,0));
                continue;
            }
            /*if ( cellcount < m_mincount || cellcount > m_maxcount )
            {
                dc.SetTextForeground(wxColor(160,160,160));

                if ( isInSelection(i,j) )
                    dc.SetBrush(SELECTION_COLOR);
                else
                    dc.SetBrush(*wxWHITE);

                dc.SetPen(*wxWHITE);
                wxRect r = GetCellRect( i, j );
                dc.DrawRectangle( r.x, r.y, r.width, r.height );
                wxString tfs = _T("X");

                wxSize s = dc.GetTextExtent( tfs );
                dc.DrawText( tfs , (m_cellw+2)*(j+1)+1 + ( m_cellw - s.x)/2,
                                (m_cellh+2)*(i+1)+1 + (m_cellh - s.y)/2 );
                dc.SetTextForeground(wxColor(0,0,0));
                continue;
            }*/
            double value;

            switch ( m_statisticalfeat )
            {
                case STAT_MIN:
                    value = extract_result<tag::min>(*acc_cell);
                    break;
                case STAT_MAX:
                    value = extract_result<tag::max>(*acc_cell);
                    break;
                case STAT_MEAN:
                    value = extract_result<tag::mean>(*acc_cell);
                    break;
                case STAT_VARIANCE:
                    value = sqrtf(extract_result<tag::variance>(*acc_cell));
                    break;
                case STAT_COUNT:
                default:
                    value = (double)extract_result<tag::count>(*acc_cell);
                    break;
            }


            if ( m_statisticalfeat == STAT_COUNT )
                str = wxString::Format(_T("%.0f"), value);
            /* else if ( m_statisticalfeat == STAT_VARIANCE )
                str = wxString::Format(_T("%.1f"), value); */
            else
            {
                wxColor color = GetCellColor( value );
                if ( isFixed )
                {
                    //str = wxString::Format(_T("%c"), 0x2713);
                    str = _T("OK");
                    color = wxColor(0, 128, 0);
                }
                else str = wxString::Format( m_numformat, value);

                if ( isInSelection(i,j) ) color = SELECTION_COLOR ;
                wxRect r = GetCellRect( i, j);
                dc.SetBrush(color); dc.SetPen(color);
                dc.DrawRectangle( r.x, r.y, r.width, r. height );
            }

            ///
            if ( cellcount < m_mincount || cellcount > m_maxcount )
            {
                dc.SetTextForeground(wxColor(160,160,160));

                if ( isInSelection(i,j) )
                    dc.SetBrush(SELECTION_COLOR);
                else
                    dc.SetBrush(*wxWHITE);

                dc.SetPen(*wxWHITE);
                wxRect r = GetCellRect( i, j );
                dc.DrawRectangle( r.x, r.y, r.width, r.height );
            }
            ///

            wxSize s = dc.GetTextExtent( str );
            dc.DrawText( str , (m_cellw+2)*(j+1)+1 + ( m_cellw - s.x)/2,
                                (m_cellh+2)*(i+1)+1 + (m_cellh - s.y)/2 );
            dc.SetTextForeground(wxColor(0,0,0));
        }
    }
    m_highlightedCell = m_selectedCell;
    SelectCell(dc);

}

void RTDataStatTable::setCustomColors(DoubleArray limits, DoubleArray colors)
{
    m_colorlimits = limits;
    m_customcolorsA = colors;

    m_customcolors = true;
    Refresh();
}

wxColor RTDataStatTable::dbl2Color(double rgb)
{
    unsigned char r,g,b;
    unsigned int irgb = (unsigned)rgb;
    r = ( irgb / (256*256) ); irgb = ( irgb % (256*256) );
    g = ( irgb / 256 ); b = irgb % 256;

    return wxColor( r, g, b );
}

wxColor RTDataStatTable::calcCustomColorCellVal(double value)
{
    if ( m_customcolorsA.GetCount() < 1 || m_colorlimits.GetCount() < 1 )
        return wxColor(255,255,255);

    size_t i;

    int maxindex = m_colorlimits.GetCount()-1;

    if ( value <= m_colorlimits[0] )
        return dbl2Color( m_customcolorsA[0] );

    if ( value >= m_colorlimits[maxindex])
        return dbl2Color( m_customcolorsA[maxindex] );

    for ( i = 0; i < m_colorlimits.GetCount()-1; ++i )
    {
        double limitval = m_colorlimits[i+1];
        if ( value < limitval )
        {
            break;
        }
    }
    // interpolation
    double minval = m_colorlimits[i], maxval = m_colorlimits[i+1];

    double corr = ( value - minval ) / ( maxval - minval );

    wxColor leftColor = dbl2Color(m_customcolorsA[i]);
    wxColor rightColor = dbl2Color(m_customcolorsA[i+1]);

    wxColor interpolatedCol;
    unsigned int red = leftColor.Red() - (int)((double)(leftColor.Red() - rightColor.Red()) * corr);
    unsigned int green = leftColor.Green() - (int)((double)(leftColor.Green() - rightColor.Green()) * corr);
    unsigned int blue = leftColor.Blue() - (int)((double)(leftColor.Blue() - rightColor.Blue()) * corr);
    return wxColor(red,green,blue);
}


wxColor RTDataStatTable::GetCellColor(double value)
{
    if ( m_customcolors )
        return calcCustomColorCellVal(value);


    if( m_gradientColorArray.IsEmpty() ) { return *wxRED; }

    int maxIndex = m_gradientColorArray.Count()-1;

    const float clr_factor = ( value - m_tblmin) / (m_tblmax - m_tblmin);

    if( clr_factor < 0 ) { return *wxRED; }

    float baseIndex = clr_factor * maxIndex;
    int startIndex = (int)baseIndex;
    int endIndex = startIndex + 1;
    if(endIndex >= maxIndex ){  endIndex = maxIndex;}

    wxColor startColor = m_gradientColorArray.Item(startIndex);
    wxColor endColor = m_gradientColorArray.Item(endIndex);

    float reminder = baseIndex - startIndex;

    const int r = startColor.Red() - int((int(startColor.Red()) - int(endColor.Red())) * reminder);
    const int g = startColor.Green() - int((int(startColor.Green()) - int(endColor.Green())) * reminder);
    const int b = startColor.Blue() - int((int(startColor.Blue()) - int(endColor.Blue())) * reminder);

    return wxColor(r,g,b);
}

wxRect RTDataStatTable::GetCellRect(int row, int col)
{
    wxRect r( (m_cellw+2)*(col+1)+1, (m_cellh+2)*(row+1)+1, m_cellw, m_cellh );
    return r;
}

void RTDataStatTable::OnRClick(wxMouseEvent& event)
{
    int x = event.GetPosition().x,
        y = event.GetPosition().y;

    if ( m_tuneMode )
    {
        if ( m_popupmenu == NULL )
        {
            m_popupmenu = new wxMenu;

            int idMenuFixCell = wxNewId();

            m_popupmenu->Append( idMenuFixCell, _T("Fix selected cells\tT"));
            m_popupmenu->Connect( idMenuFixCell, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( RTDataStatTable::OnMenuFix), 0, this );
        }

        x -= (m_cellw+2);
        y -= (m_cellh+2);

        int col = x / ( m_cellw + 2), row = y / ( m_cellh + 2);
        if ( col < 0 || col >= m_cols || row < 0 || row >= m_rows )
        {
            event.Skip();
            return;
        }

        if ( m_startcell == invalid_cell )
        {
            m_selectedCell.row = row; m_selectedCell.col = col;
            m_startcell = m_selectedCell; m_endcell = m_selectedCell;
            observerNotify( idSelectCell, row*m_cols + col );
            Refresh( false );
        }
    }
    else
    {
        if ( m_popupmenu == NULL )
        {
            m_popupmenu = new wxMenu;

            m_popupmenu->Append(idStatMenuCount, _T("Count"), _T("Count"), wxITEM_RADIO );
            Connect( idStatMenuCount, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( RTDataStatTable::OnStatFeatSelect ) );
            m_popupmenu->Append(idStatMenuMin, _T("Min"), _T("Min"), wxITEM_RADIO );
            Connect( idStatMenuMin, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( RTDataStatTable::OnStatFeatSelect ) );
            m_popupmenu->Append(idStatMenuMax, _T("Max"), _T("Max"), wxITEM_RADIO );
            Connect( idStatMenuMax, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( RTDataStatTable::OnStatFeatSelect ) );
            m_popupmenu->Append(idStatMenuMean, _T("Mean"),_T("Mean"), wxITEM_RADIO);
            m_popupmenu->Check( idStatMenuMean, true);
            Connect( idStatMenuMean, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( RTDataStatTable::OnStatFeatSelect ) );
            m_popupmenu->Append(idStatMenuVar, _T("Variance"),_T("Variance"), wxITEM_RADIO);
            Connect( idStatMenuVar, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( RTDataStatTable::OnStatFeatSelect ) );

            m_popupmenu->AppendSeparator();
            m_popupmenu->Append( idShowHistogram, _T("Show Histogram"), _T("Show Histogram"), wxITEM_NORMAL );
            Connect( idShowHistogram, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( RTDataStatTable::OnShowHistogram ) );
        }
    }

    PopupMenu( m_popupmenu, event.GetPosition());
}

void RTDataStatTable::OnStatFeatSelect(wxCommandEvent& event)
{
    int id = event.GetId();

    if ( id == idStatMenuCount ) { observerNotify(42, 0); return; }
    if ( id == idStatMenuMin ) { observerNotify(42, 1); return; }
    if ( id == idStatMenuMax ) { observerNotify(42, 2); return; }
    if ( id == idStatMenuMean ) { observerNotify(42, 3); return; }
    if ( id == idStatMenuVar ) { observerNotify(42, 4); return; }

    event.Skip();
}

void RTDataStatTable::SelectCell(wxDC& dc)
{
    int srow = m_startcell.row, scol = m_startcell.col;

    if ( srow < 0 || srow>= m_rows || scol < 0 || scol >= m_cols )
        return;

    wxBrush oldbrush = dc.GetBrush();
    wxPen oldpen = dc.GetPen();
    // int oldpenw = dc.GetPen().GetWidth();

    wxPen pen(wxColor(0,0,0), 2);
    dc.SetPen(pen);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);

    cell tl, br;
    getSelectionCells(tl, br);

    wxRect r = GetCellRect( tl.row, tl.col );
    wxRect r1 = GetCellRect( br.row, br.col );

    int w = ( tl.col == br.col ) ? r.width + 2 : ( r1.x - r.x + r.width + 2);
    int h = ( tl.row == br.row ) ? r.height + 2 : ( r1.y - r.y + r.height + 2);

    dc.DrawRectangle( r.x - 1, r.y - 1, w, h );

    dc.SetBrush(oldbrush); dc.SetPen(oldpen);

}

void RTDataStatTable::OnLeftButton( wxMouseEvent& event )
{
    long x, y;
    event.GetPosition(&x, &y);

    x -= (m_cellw+2);
    y -= (m_cellh+2);

    int col = x / ( m_cellw + 2), row = y / ( m_cellh + 2);
    if ( col < 0 || col >= m_cols || row < 0 || row >= m_rows )
    {
        event.Skip();
        return;
    }

    if ( event.ShiftDown() )
    {
        m_selectedCell.row = row; m_selectedCell.col = col;
        m_endcell = m_selectedCell;
        if ( m_startcell == invalid_cell )
        {
            m_startcell.col = 0;
            m_startcell.row = 0;
        }

        cell tl, br;

        getSelectionCells(tl, br);

        size_t tblSize = m_cols * m_rows;
        size_t tlCellI = tl.row * m_cols + tl.col;
        size_t brCellI = br.row * m_cols + br.col;
        observerNotify( idMultipleSel, tblSize * tlCellI + brCellI );
    }
    else
    {
        m_selectedCell.row = row; m_selectedCell.col = col;
        m_startcell = m_selectedCell; m_endcell = m_selectedCell;
        observerNotify( idSelectCell, row*m_cols + col );

        m_stDataMgr->updateTableViewers( this, _T("RTDataStatTable"), wxPoint( m_selectedCell.col, m_selectedCell.row ) );

    }
    Refresh(false);
}

void RTDataStatTable::OnKeyDown( wxKeyEvent& event )
{
    if ( m_selectedCell.col < 0 || m_selectedCell.col >= m_cols ||
        m_selectedCell.row < 0 || m_selectedCell.row >= m_rows )
    {
        event.Skip();
        return;
    }

    int keyCode = event.GetKeyCode();
    switch ( keyCode )
    {
        case 'A':
        case 'T':
        case WXK_LEFT:
        case WXK_RIGHT:
        case WXK_UP:
        case WXK_DOWN:
        case WXK_HOME:
        case WXK_END: break;
        default: event.Skip(); return;
    }

    // fix only in tune mode
    if ( keyCode == 'T' )
    {
        if ( m_tuneMode == false || m_selectedCell == invalid_cell )
        {
            event.Skip();
            return;
        }

        observerNotify( idFixCell, 0);
        return;
    }

    bool changed = false; // table has to be refreshed

    if ( keyCode == 'A' )
    {
        if ( event.ControlDown() )
        {
            cell tl, br;
            getSelectionCells(tl, br);

            if ( tl.row == 0 && tl.col == 0 && br.col == m_cols-1 && br.row == m_rows-1 )
            {
                m_startcell = m_selectedCell;
                m_endcell = m_selectedCell;

                m_isSelecting = false;
            }
            else
            {
                m_startcell.row = 0;
                m_startcell.col = 0;
                m_endcell.row = m_rows - 1;
                m_endcell.col = m_cols - 1;

                m_isSelecting = true;
            } // end select/unselect
            changed = true;
        } // end control down
    } // end key A
    else
    {
        if ( !event.ShiftDown() )
        {
            switch ( keyCode )
            {
                case WXK_LEFT:  if ( m_selectedCell.col != 0 )
                                {
                                    --m_selectedCell.col;
                                    changed = true;
                                }
                                break;

                case WXK_RIGHT: if ( m_selectedCell.col < m_cols-1 )
                                {
                                    ++m_selectedCell.col;
                                    changed = true;
                                }
                                break;

                case WXK_UP:    if ( m_selectedCell.row != 0 )
                                {
                                    --m_selectedCell.row;
                                    changed = true;
                                }
                                break;

                case WXK_DOWN:  if ( m_selectedCell.row<m_rows-1 )
                                {
                                    ++m_selectedCell.row;
                                    changed = true;
                                }
                                break;

                case WXK_HOME:  if ( m_selectedCell.col > 0 || m_selectedCell.row > 0 )
                                {
                                    m_selectedCell.row = 0;
                                    m_selectedCell.col = 0;
                                    changed = true;
                                }
                                break;

                case WXK_END:   if ( m_selectedCell.col != m_cols-1 || m_selectedCell.row != m_rows-1 )
                                {
                                    m_selectedCell.row = m_rows-1;
                                    m_selectedCell.col = m_cols-1;
                                    changed = true;
                                }
                                break;
            } // end switch

            m_startcell = m_selectedCell;
            m_endcell = m_selectedCell;

            m_isSelecting = false;

            // notify other tables
            if ( changed )
            m_stDataMgr->updateTableViewers( this, _T("RTDataStatTable"), wxPoint( m_selectedCell.col, m_selectedCell.row ) );

        } // end shift down
        else
        {
            if (!m_isSelecting)
            {
                m_isSelecting = true;
                m_startcell = m_selectedCell;
            }

            if ( keyCode == WXK_LEFT )
            {
                if ( m_selectedCell.col != 0 )
                {
                    --m_selectedCell.col;
                    changed = true;
                    m_endcell = m_selectedCell;
                }
            }

            if ( keyCode == WXK_RIGHT )
            {
                if ( m_selectedCell.col < m_cols-1 )
                {
                    ++m_selectedCell.col;
                    changed = true;
                    m_endcell = m_selectedCell;
                }
            }

            if ( keyCode == WXK_UP )
            {
                if ( m_selectedCell.row != 0 )
                {
                    --m_selectedCell.row;
                    changed = true;
                    m_endcell = m_selectedCell;
                }
            }

            if ( keyCode == WXK_DOWN )
            {
                if ( m_selectedCell.row < m_rows-1 )
                {
                    ++m_selectedCell.row;
                    changed = true;
                    m_endcell = m_selectedCell;
                }
            }

            if ( keyCode == WXK_HOME )
            {
                if ( m_selectedCell.col > 0 || m_selectedCell.row > 0 )
                {
                    m_selectedCell.row = 0;
                    m_selectedCell.col = 0;
                    changed = true;
                    m_endcell = m_selectedCell;
                }
            }

            if ( keyCode == WXK_END )
            {
                if ( m_selectedCell.col != m_cols-1 || m_selectedCell.row != m_rows-1 )
                {
                    m_selectedCell.row = m_rows-1;
                    m_selectedCell.col = m_cols-1;
                    changed = true;
                    m_endcell = m_selectedCell;
                }
            }
        }
    }

    if ( changed )
    {
        if ( m_isSelecting )
        {
            // coding tblSize * top-left cell index + bottom-right cell index
            cell tl, br;

            getSelectionCells(tl, br);

            size_t tblSize = m_cols * m_rows;
            size_t tlCellI = tl.row * m_cols + tl.col;
            size_t brCellI = br.row * m_cols + br.col;
            observerNotify( idMultipleSel, tblSize * tlCellI + brCellI );
        }
        else
            observerNotify( idSelectCell, m_selectedCell.row*m_cols + m_selectedCell.col );

        Refresh(false);
    }

    event.Skip();

}

void RTDataStatTable::getSelectionCells(cell& topleft, cell& bottomright)
{
    topleft.col = ( m_startcell.col < m_endcell.col ) ? m_startcell.col : m_endcell.col;
    topleft.row = ( m_startcell.row < m_endcell.row ) ? m_startcell.row : m_endcell.row;

    bottomright.col = ( m_startcell.col > m_endcell.col ) ? m_startcell.col : m_endcell.col;
    bottomright.row = ( m_startcell.row > m_endcell.row ) ? m_startcell.row : m_endcell.row;
}

void RTDataStatTable::setMinCount(size_t min, bool refresh)
{
    m_mincount = min;

    if ( refresh )
    {
        m_calculated = false;
        Refresh();
    }
}

bool RTDataStatTable::isInSelection(int row, int col)
{
    cell tl, br;

    getSelectionCells(tl, br);

    // if only one cell is selected, use actual color of the cell, not selection color
    if ( tl.row == br.row && tl.col == br.col )
        return false;

    return ( row>=tl.row && row<=br.row && col>=tl.col && col<=br.col );
}

void RTDataStatTable::checkCell( int row, int col )
{
    if ( row < 0 || row >= m_rows || col < 0 || col >= m_cols )
        return;

    m_FixedCells[ row * m_FixedCells.getCols() + col ] = 1;
    Refresh();
}

bool RTDataStatTable::isChecked( int row, int col )
{
    if ( row < 0 || row >= m_rows || col < 0 || col >= m_cols )
        return false;

    return ( m_FixedCells[ row * m_FixedCells.getCols() + col ] )!=0?true:false;
}

void RTDataStatTable::setSelection( cell topLeft, cell bottomRight )
{
    if ( topLeft.row < 0 || topLeft.row >= m_rows || topLeft.col < 0 || topLeft.col >= m_cols )
        return;

    if ( bottomRight.row < 0 || bottomRight.row >= m_rows || bottomRight.col < 0 || bottomRight.col >= m_cols )
        return;

    m_startcell = topLeft;
    m_endcell = bottomRight;

    Refresh();
}

void RTDataStatTable::OnMenuFix( wxCommandEvent& event )
{
    observerNotify( idFixCell, 0);
    Refresh( false );
}

void RTDataStatTable::OnMouse( wxMouseEvent& event )
{
    static bool dragging = false;

    long x, y;
    event.GetPosition(&x, &y);

    x -= (m_cellw+2);
    y -= (m_cellh+2);

    int col = x / ( m_cellw + 2), row = y / ( m_cellh + 2);
    if ( col < 0 || col >= m_cols || row < 0 || row >= m_rows )
    {
        event.Skip();
        return;
    }

    if ( event.LeftIsDown() )
    {
        if ( !dragging )
        {
            dragging = true;
            m_startcell.row = row;
            m_startcell.col = col;
        }
        else
        {
            m_endcell.row = row;
            m_endcell.col = col;

            cell tl, br;

            getSelectionCells(tl, br);

            size_t tblSize = m_cols * m_rows;
            size_t tlCellI = tl.row * m_cols + tl.col;
            size_t brCellI = br.row * m_cols + br.col;
            observerNotify( idMultipleSel, tblSize * tlCellI + brCellI );
        }
        Refresh( false );
        event.Skip();
    }
    else
    {
        dragging = false;
        event.Skip();
    }
}

void RTDataStatTable::updateTable( const wxString& tableName, wxPoint cursorPosition )
{
    int col = cursorPosition.x;
    int row = cursorPosition.y;
    m_selectedCell.col = col; m_selectedCell.row = row;

    m_startcell = m_selectedCell;
    m_endcell = m_selectedCell;

    m_isSelecting = false;

    observerNotify( idSelectCell, m_selectedCell.row*m_cols + m_selectedCell.col );

    m_returnfocus = true;

    Refresh(false);
}

void RTDataStatTable::OnFocusEvent( wxFocusEvent& event )
{
    wxWindow* win = event.GetWindow();
    if ( win && m_returnfocus )
        win->SetFocus();
    else
    {
        event.Skip();
    }
    m_returnfocus = false;
}

void RTDataStatTable::OnShowHistogram( wxCommandEvent& event )
{
    if ( m_selectedCell.col < 0 || m_selectedCell.col >= m_cols ||
        m_selectedCell.row < 0 || m_selectedCell.row >= m_rows )
    {
        wxMessageBox( _T("Select a cell first"), _T("Error"), wxOK | wxICON_ERROR );
        event.Skip();
        return;
    }
    else
    {
        observerNotify( 1, this );
    }
}
