#ifndef RTLIVEANALYZER_H
#define RTLIVEANALYZER_H

#include <wx/wx.h>

#include "../common/types/DoubleArray.h"
#include "../common/VemsMatrix.h"

#include "../expression/RPNParser.h"

#include "../rtdatamanager/RTDataAnalyzerBase.h"

class RTDataMgrBase;
class STDataMgrBase;
class VemsTableDesc;
class Symbol;
class SymbolMgr;
class AccSetMatrix;
class TablePanelBase;

class RTLiveAnalyzer : public wxFrame, public RTDataAnalyzerBase
{
    public:
        RTLiveAnalyzer( wxWindow* parent, RTDataMgrBase* rtMgrBase, STDataMgrBase* stMgrBase, SymbolMgr* symbolMgr, VemsTableDesc* tableDesc );
        virtual ~RTLiveAnalyzer();

    protected:
        void    processData();

    private:
        STDataMgrBase*      m_stDataMgr;
        wxTextCtrl*         m_minData;

        wxString            m_defaultExpr;
        wxString            m_filter;

        VemsTableDesc*      m_tableDesc;
        SymbolMgr*          m_symbolMgr;
        Symbol*             m_xBinSymbol;
        Symbol*             m_yBinSymbol;
        Symbol*             m_xRTSymbol;
        Symbol*             m_yRTSymbol;
        unsigned int        m_rows, m_cols;
        DoubleArray         m_HorizLimits, m_VertLimits;
        int                 m_xmin, m_xmax, m_ymin, m_ymax;

        unsigned int        m_outofrange;
        unsigned int        m_dropped;

    private:
        AccSetMatrix*       m_accSetMatrix;
        VemsMatrix<double>* m_dataMatrix;
        VemsMatrix<double>* m_timeMatrix;

        TablePanelBase*     m_table;

    private:
        void    buildGUI();
        void    getBins( wxArrayString& xbins, wxArrayString& ybins );
        void    updateMatrix( int xvalue, int yvalue, double expr_value );

        DECLARE_EVENT_TABLE()
};

#endif // RTLIVEANALYZER_H
