#include "SDCardSaveDialog.h"
#include "../logger.h"
#include "../DirHelper.h"
#include "../UserLog.h"

///

int idSDCardSaveImg = wxNewId();
int idSDCardDetect = wxNewId();
int idSDRefreshDevices = wxNewId();
int idSDCardCancel = wxNewId();
int idSDCloseDlg = wxNewId();

BEGIN_EVENT_TABLE( SDCardSaveDialog, wxDialog )

    EVT_CLOSE( SDCardSaveDialog::OnClose )
    EVT_IDLE( SDCardSaveDialog::OnIdle )
    EVT_TIMER( wxID_ANY, SDCardSaveDialog::OnTimer)

    EVT_BUTTON( idSDCardSaveImg, SDCardSaveDialog::OnSaveImg )
    EVT_BUTTON( idSDCardDetect, SDCardSaveDialog::OnDetectSD )
    EVT_BUTTON( idSDRefreshDevices, SDCardSaveDialog::OnRefreshDevices )
    EVT_BUTTON( idSDCardCancel, SDCardSaveDialog::OnCancelProgress )
    EVT_BUTTON( idSDCloseDlg, SDCardSaveDialog::OnCloseDlg )

END_EVENT_TABLE()

SDCardSaveDialog::SDCardSaveDialog( wxWindow* parent ) : m_timerIdleWakeUp( this )
{

#ifdef WIN32
    m_ddprog = DirHelper::GetPath() + _T("3rdparty\\dd.exe");
#else
    m_ddprog = _T("./dd");
#endif

    buildGUI( parent );
}

SDCardSaveDialog::~SDCardSaveDialog()
{
    //dtor
}

void SDCardSaveDialog::buildGUI( wxWindow* parent )
{
    Create( parent, -1, _T("SD Card - Save image"));

    SetBackgroundColour( wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    wxBoxSizer* mainsizer = new wxBoxSizer( wxVERTICAL );

    /// devices
    wxStaticBoxSizer* sbox = new wxStaticBoxSizer( wxVERTICAL, this, _T("Removable devices") );
    m_devicesCombo = new wxComboBox( this, wxID_ANY, _T(""), wxDefaultPosition, wxSize(420,-1),
        0, NULL, wxCB_DROPDOWN | wxCB_READONLY );

    wxBoxSizer* buttonsizer = new wxBoxSizer( wxHORIZONTAL );
    m_detectButton = new wxButton( this, idSDCardDetect, _T("Detect SD Card") );
    m_saveButton = new wxButton( this, idSDCardSaveImg, _T("Save image") );

    m_refreshButton = new wxButton( this, idSDRefreshDevices, _T("Refresh list") );

    buttonsizer->Add( m_detectButton, 0, wxALL | wxALIGN_CENTER, 5 );
    buttonsizer->Add( m_saveButton, 0, wxALL | wxALIGN_CENTER, 5 );
    buttonsizer->Add( m_refreshButton, 0, wxALL | wxALIGN_CENTER, 5 );

    sbox->Add( m_devicesCombo, 1, wxALL | wxEXPAND, 5 );
    sbox->Add( buttonsizer, 0, wxALL | wxALIGN_CENTER, 5 );

    /// Progress
    wxStaticBoxSizer* sbox1 = new wxStaticBoxSizer( wxVERTICAL, this, _T("Progress") );
    m_progressbar = new wxGauge( this, wxID_ANY, 100, wxDefaultPosition, wxSize(420, 22) );
    m_progressbar->SetForegroundColour( wxColor(_T("#006699")) );

    m_nrread = new wxStaticText( this, -1, _T("Read: ") );

    m_cancelButton = new wxButton( this, idSDCardCancel, _T("Cancel progress") );
    m_cancelButton->Enable( false );

    sbox1->Add( m_progressbar, 0, wxALL, 5 );
    sbox1->Add( m_nrread, 0, wxALL, 5 );
    sbox1->Add( m_cancelButton, 0, wxALL | wxALIGN_CENTER, 5 );

    /// Messages
    wxStaticBoxSizer* sbox2 = new wxStaticBoxSizer( wxVERTICAL, this, _T("Messages") );
    m_messages = new wxTextCtrl( this, -1, _T(""), wxDefaultPosition, wxSize(350,120),
        wxTE_MULTILINE | wxTE_READONLY );

    sbox2->Add( m_messages, 1, wxALL | wxEXPAND, 5 );

    ///

    m_closeButton = new wxButton( this, idSDCloseDlg, _T("Close dialog"));

    mainsizer->Add( sbox, 0, wxALL | wxEXPAND, 5 );
    mainsizer->Add( sbox1, 0, wxALL | wxEXPAND, 5 );
    mainsizer->Add( sbox2, 0, wxALL | wxEXPAND, 5 );
    mainsizer->Add( m_closeButton, 0, wxALL | wxALIGN_CENTER, 5 );

    SetSizer( mainsizer );
    mainsizer->Fit( this );
    mainsizer->SetSizeHints( this );

    refreshDevices();
}

void SDCardSaveDialog::Notify( const wxString& msg )
{

#ifdef WIN32

    wxString tmp = msg;

    if ( msg.StartsWith(_T("Error")))
    {
        m_messages->AppendText( msg );
        return;
    }

    int pos;

//    if (  (pos = tmp.Find(' ')) != wxNOT_FOUND )
//        tmp = tmp.Left(pos);

    // dd for windows report size in aa,bbb,ccc number format -> convert it to aabbbccc
    while ( ( pos = tmp.Find(',')) != wxNOT_FOUND )
    {
        tmp = tmp.Left( pos ) + tmp.Mid( pos + 1 );
    }

    unsigned long val;

    static int counter = 0;

    // is it a valid number?
    if ( tmp.ToULong( &val ) )
    {
        if ( counter % 25 == 0 )
            m_progressbar->Pulse();

        counter++;

        wxString pf = _T("B");

        double fval = (double)val;
        if ( fval > 1024 ) { fval /= 1024.; pf = _T("kB"); }
        if ( fval > 1024 ) { fval /= 1024.; pf = _T("MB"); }
        if ( fval > 1024 ) { fval /= 1024.; pf = _T("GB"); }

        wxString Msg = wxString::Format( _T("Read: %.2f ") + pf, fval);
        Msg += wxString::Format( _T(" (%d bytes)"), val );

        m_nrread->SetLabel( Msg );
    }

#else
    /// TODO

#endif

}

bool SDCardSaveDialog::DetectVemsSD( const wxString device )
{
#ifdef WIN32

    // redirect output to stdout
    wxString command = m_ddprog + _T(" if=") + device + _T(" of=- bs=512 count=1");

    runSync(command);

    // DD for windows - multiple lines, copyright, version info etc..
    for ( size_t i = 0; i < m_outputA.GetCount(); ++i )
    {
        if ( m_outputA[i].StartsWith(_T("VEMS")))
        {
            return true;
        }
    }
#else

    /// TODO

#endif

    return false;
}

void SDCardSaveDialog::enableButtons( bool enable )
{
    m_detectButton->Enable( enable );
    m_saveButton->Enable( enable );
    m_refreshButton->Enable( enable );
    m_closeButton->Enable( enable );
    m_cancelButton->Enable( !enable );
}

void SDCardSaveDialog::refreshDevices()
{
#ifdef WIN32
    m_devicesCombo->Clear();
    m_devicesName.Clear();

    // get drive list
    wxString cmd = m_ddprog + _T(" --list");
    runSync( cmd );

    wxArrayString comboarr;

    int i = 0;

    wxString tmp;
    wxString prev = _T("");
    wxString devname;

    for ( size_t j = 0; j < m_errorA.GetCount(); ++j )
    {
        wxString s = m_errorA[j];
        // strip whitespaces
        s = s.Trim(0);

        // two consecutive empty lines -> end of the list
        if ( s == _T("") && prev == _T("") )
            break;

        if ( s.StartsWith(_T("link to")))
        {
            s = _T("\\") + s.AfterFirst('\\');
            devname = s;
            tmp += s; i = 1;
        }
        else if ( i == 1 )
        {
            tmp += _T(" (") + s; i = 2;
        }
        else if ( i == 2 )
        {
            tmp += _T(") Mounted on: ")+ s.AfterLast('\\');
            i = 0;
            if ( tmp.Find(_T("(removeable")) != wxNOT_FOUND )
            {
                m_devicesName.Add(devname);
                comboarr.Add( tmp );
            }
            tmp = _T("");
        }
        prev = s;
    }
    m_devicesCombo->Append( comboarr );
#endif
}

/// event handlers of buttons

void SDCardSaveDialog::OnCancelProgress( wxCommandEvent& event )
{
    // only one process should be in the array
    SDProcess* process = m_running[0];

    long PID = process->GetProcessID();

    wxProcess::Kill( PID, wxSIGKILL );
    m_messages->AppendText( wxString::Format(_T("Kill PID = %ld\n"), PID ) );

    m_killed = true;
}

void SDCardSaveDialog::OnCloseDlg( wxCommandEvent& event )
{
    EndModal( 0 );
}

void SDCardSaveDialog::OnRefreshDevices( wxCommandEvent& event )
{
    refreshDevices();
}

void SDCardSaveDialog::OnDetectSD( wxCommandEvent& event )
{
    bool found = false;

    enableButtons( false );

    for ( size_t i = 0;  i < m_devicesName.GetCount(); ++i )
    {
        m_messages->AppendText( _T("Devname=") + m_devicesName[i]  + _T("\n") );
        if ( DetectVemsSD( m_devicesName[i] ) )
        {
            wxString msg = _T("Valid VEMSlogs on: ") + m_devicesCombo->GetString(i);
            msg += _T("\n");

            m_messages->AppendText( msg );
            m_devicesCombo->Select(i);

            found = true;

            break;
        }
        else
        {
            m_messages->AppendText( m_devicesCombo->GetString(i) + _T(" does not contain VEMS log\n"));
        }
    }

    if ( !found )
        m_messages->AppendText( _T("None of the available devices contains valid VEMS log\n"));

    enableButtons();
}

void SDCardSaveDialog::OnSaveImg( wxCommandEvent &event )
{

    int i = m_devicesCombo->GetSelection();

    if ( i == wxNOT_FOUND )
    {
        wxMessageBox( _T("No device has been selected"), _T("Error") );
        return;
    }

    enableButtons( false );

    if ( DetectVemsSD( m_devicesName[i] ) == false )
    {

        wxMessageBox( _T("There is no card attached into the card reader or the card does not contain VEMS Log"),
            _T("Error"), wxOK | wxICON_STOP );

        enableButtons();

        return;
    }


    wxDateTime dt = wxDateTime::Now();
    m_ofname = wxString::Format(_T("%d%02d%02d_%02d%02d%02d.raw"), dt.GetYear(), dt.GetMonth()+1,
        dt.GetDay(), dt.GetHour(), dt.GetMinute(), dt.GetSecond() );

    m_ofname = DirHelper::GetPath( dWORKINGDIR ) + m_ofname;

    wxString command;

#ifdef WIN32
    command = m_ddprog + _T(" if=") + m_devicesName[i];
    command += _T(" of=\"") + m_ofname + _T("\" bs=32768 --progress --stoponempty emptyseek=4");

    runAsync( command, true );

#else

    /// TODO

#endif

}

///

void SDCardSaveDialog::runSync(const wxString& cmd )
{
    m_outputA.Clear();
    m_errorA.Clear();

    int code = wxExecute( cmd, m_outputA, m_errorA );

    if ( code != -1 )
    {
        /// OK
    }
    else
    {
        m_messages->AppendText( _T("Executing command failed: ") + cmd + _T("\n"));
    }
}

void SDCardSaveDialog::runAsync( const wxString& cmd, bool closeParent )
{
    m_killed = false;

    SDProcess *process = new SDProcess( this, closeParent );
    // m_messages->AppendText( cmd );
    int pid = wxExecute( cmd, wxEXEC_ASYNC, process );
    if ( !pid )
    {
        //fail
        delete process;
    }
    else
    {
        process->SetProcessID( pid );
        AddAsyncProcess( process );
        m_messages->AppendText( wxString::Format( _T("Process started, PID = %d"), pid ) );
    }
}

void SDCardSaveDialog::OnClose(wxCloseEvent &event)
{
    if ( m_closeButton->IsEnabled() == false )
        return;

    EndModal( 0 );
}

void SDCardSaveDialog::OnIdle( wxIdleEvent& event )
{
    size_t count = m_running.GetCount();

    for ( size_t n = 0; n < count; n++ )
    {
        if ( m_running[n]->HasInput() )
        {
            event.RequestMore();
        }
    }
}

void SDCardSaveDialog::OnTimer( wxTimerEvent& event )
{
    wxWakeUpIdle();
}

void SDCardSaveDialog::AddAsyncProcess( SDProcess *process )
{
    if ( m_running.IsEmpty() )
    {
        m_timerIdleWakeUp.Start(100);
    }
    m_running.Add( process );
}

void SDCardSaveDialog::RemoveAsyncProcess( SDProcess *process )
{
    m_running.Remove( process );

    if ( m_running.IsEmpty() )
    {
        m_timerIdleWakeUp.Stop();
    }
}

void SDCardSaveDialog::OnProcessTerminated( SDProcess *process )
{
    RemoveAsyncProcess( process );
    m_messages->AppendText( _T("Process ended\n") );
    enableButtons();

    if ( !m_killed && process->ParentClose() )
    {
        UserLog::uLog( ULOG_INFO, _T("SD Card image saved to: ") + m_ofname );
        EndModal( 1 );
    }
}
