#include "sessiondlg.h"

#include "helper.h"
#include "logger.h"

//(*InternalHeaders(SessionDlg)
#include <wx/bitmap.h>
#include <wx/font.h>
#include <wx/fontenum.h>
#include <wx/fontmap.h>
#include <wx/image.h>
#include <wx/intl.h>
#include <wx/settings.h>
#include <wx/xrc/xmlres.h>
//*)

//(*IdInit(SessionDlg)
//*)

BEGIN_EVENT_TABLE(SessionDlg,wxDialog)
	//(*EventTable(SessionDlg)
	//*)
END_EVENT_TABLE()

SessionDlg::SessionDlg(wxWindow* parent,wxWindowID id)
{
     wxString msg = _T("SessionDlg: ");

     wxString defCwd = wxFileName::GetCwd();

     Logger::log(LOG_MSG, msg + _T("Def dir ") + defCwd);


    wxXmlResource::Get()->InitAllHandlers();
    wxXmlResource::Get()->Load(_T("sessiondlg.xrc"));

	//(*Initialize(SessionDlg)
	wxXmlResource::Get()->LoadObject(this,parent,_T("SessionDlg"),_T("wxDialog"));
	StaticText1 = (wxStaticText*)FindWindow(XRCID("ID_STATICTEXT1"));
	CurrSessionCtrl = (wxTextCtrl*)FindWindow(XRCID("ID_CUR_SES_CTRL"));
	SessListCtrl = (wxListCtrl*)FindWindow(XRCID("ID_SESLISTCTRL"));
	ButtonLoad = (wxButton*)FindWindow(XRCID("ID_BUTTON_LOAD"));
	ButtonSave = (wxButton*)FindWindow(XRCID("ID_BUTTON_SAVE"));
	ButtonDelete = (wxButton*)FindWindow(XRCID("ID_BUTTON_DELETE"));
	StaticLine1 = (wxStaticLine*)FindWindow(XRCID("ID_STATICLINE1"));
	ButtonClose = (wxButton*)FindWindow(XRCID("wxID_CANCEL"));
	Connect(XRCID("ID_SESLISTCTRL"),wxEVT_COMMAND_LIST_ITEM_SELECTED,(wxObjectEventFunction)&SessionDlg::OnSessListCtrlItemSelect);
	Connect(XRCID("ID_BUTTON_LOAD"),wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&SessionDlg::OnButtonLoadClick);
	Connect(XRCID("ID_BUTTON_SAVE"),wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&SessionDlg::OnButtonSaveClick);
	Connect(XRCID("ID_BUTTON_DELETE"),wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&SessionDlg::OnButtonDeleteClick);
	//*)

    wxArrayString * sessionArray = new wxArrayString();
    Helper::getGroupArray(_T("/sessions"), sessionArray);
    loadSeession(sessionArray);

    SessListCtrl->SetTextColour(*wxBLUE);
    //SessListCtrl->SetBackgroundColour(*wxLIGHT_GREY);
    //SessListCtrl->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);


    wxSizer*  topSizer = (wxBoxSizer *) this->GetSizer();
    topSizer->Fit(this);
	topSizer->SetSizeHints(this);
}

SessionDlg::~SessionDlg()
{
}

void SessionDlg::OnButtonLoadClick(wxCommandEvent& event)
{
    wxString sesName = CurrSessionCtrl->GetValue();
    if(sesName.length() < 1) return;

//    Helper::loadSession(sesName);
//    Helper::doUpdateStatusBar();
}

void SessionDlg::OnButtonSaveClick(wxCommandEvent& event)
{
    wxString sessName = CurrSessionCtrl->GetValue();
    if(sessName.length() < 1){ return;}

//    Helper::saveSession(sessName);

    wxArrayString * sessionArray = new wxArrayString();
    Helper::getGroupArray(_T("/sessions"), sessionArray);

    SessListCtrl->ClearAll();
    loadSeession(sessionArray);
}

void SessionDlg::OnButtonDeleteClick(wxCommandEvent& event)
{
    wxString sessName = CurrSessionCtrl->GetValue();
    if(sessName.length() < 1){ return;}

    Helper::deleteSession(sessName);

    wxArrayString * sessionArray = new wxArrayString();
    Helper::getGroupArray(_T("/sessions"), sessionArray);

    SessListCtrl->ClearAll();
    loadSeession(sessionArray);

}

void SessionDlg::loadSeession(wxArrayString * groupArray)
{
    wxString _C = _T("SessionDlg ");
    wxString _F = _T("loadSeession ");

    size_t size = groupArray->GetCount();
//    Logger::log(LOG_MSG, wxString::Format(_C+_F+_T("group size: %d") + size) );

    for(size_t x = 0; x< size; x++){
            wxString name = groupArray->Item(x);
             //SessListCtrl->SetItem(x, 0, name);
            SessListCtrl->InsertItem(x, name);
            Logger::log(LOG_MSG, _C+_F+_T("name: ") + name);
    }
}

void SessionDlg::OnSessListCtrlItemSelect(wxListEvent& event)
{
    wxString _C = _T("SessionDlg ");
    wxString _F = _T("OnSessListCtrlItemSelect ");

    int selected = event.m_itemIndex;
    wxString sesName = SessListCtrl->GetItemText(selected);

    Logger::log(LOG_MSG, _C+_F+_T("session name: ") + sesName);
    CurrSessionCtrl->SetValue(sesName);

}

