#include "AudioPanel.h"

#include <wx/wx.h>

#include "../AudioController.h"
#include "../MessageBoxObserver.h"
#include "../logger.h"
#include "../fileparser/RpmFromFile.h"
#include "Main_Pulse_train.h"
#include "portaudiocpp/PortAudioCpp.hxx"
#include "signalgen.h"
#include "../vemsprotocol/SIPRPacket.h"

using namespace portaudio;

enum
{
    ID_TRIGGER_TYPE_COMBO   = 1000,
    ID_RPM_SLIDER,
    ID_VOLUME_SLIDER,
    ID_DELTAMAXCLIMBSLIDER,
    ID_SECTRIG,
    ID_PLAY_BUTTON,
    ID_FILE_PLAY_BUTTON,
    ID_PULSE_TRAIN_BUTTON,
    ID_STOP_BUTTON,
    ID_AUDIO_OUTPUT_COMBO,
    ID_AUDIO_INPUT_COMBO,
    ID_AUDIO_SAMPLE_RATE_COMBO,
    ID_PRIMSIGNAL_MUTE,
    ID_SECSIGNAL_MUTE,
    ID_FLUCTUATING
};

BEGIN_EVENT_TABLE( AudioPanel, wxPanel )
    EVT_COMBOBOX( ID_TRIGGER_TYPE_COMBO, AudioPanel::OnComboBoxSelected )
    EVT_COMBOBOX( ID_AUDIO_OUTPUT_COMBO, AudioPanel::OnAudioOutputSelected )
    EVT_COMBOBOX( ID_AUDIO_INPUT_COMBO, AudioPanel::OnAudioInputSelected )
    EVT_COMBOBOX( ID_AUDIO_SAMPLE_RATE_COMBO, AudioPanel::OnSampleRateSelected )
    EVT_COMMAND_SCROLL( ID_RPM_SLIDER, AudioPanel::OnRpmSliderScroll )
    EVT_COMMAND_SCROLL( ID_VOLUME_SLIDER, AudioPanel::OnVolumeSliderScroll )
    EVT_COMMAND_SCROLL( ID_DELTAMAXCLIMBSLIDER, AudioPanel::OnDeltaMaxClimberSliderScroll )
    EVT_COMMAND_SCROLL( ID_SECTRIG, AudioPanel::OnSecTrigSliderScroll )
    EVT_COMMAND_SCROLL( ID_FLUCTUATING, AudioPanel::OnFluctuatingScroll )
    EVT_BUTTON( ID_PLAY_BUTTON, AudioPanel::OnPlayButton )
    EVT_BUTTON( ID_STOP_BUTTON, AudioPanel::OnStopButton )
    EVT_BUTTON( ID_FILE_PLAY_BUTTON, AudioPanel::OnFilePlayButton )
    EVT_BUTTON( ID_PULSE_TRAIN_BUTTON, AudioPanel::OnPulseTrainButton )
    EVT_TOGGLEBUTTON( ID_PRIMSIGNAL_MUTE, AudioPanel::OnMuteButtons )
    EVT_TOGGLEBUTTON( ID_SECSIGNAL_MUTE, AudioPanel::OnMuteButtons )

END_EVENT_TABLE()


AudioPanel::AudioPanel( wxWindow* parent ) : wxPanel( parent )
{
    m_pulse_train= NULL;
    m_rpmFromFile= NULL;
    m_audioController = NULL;

    //init gui
    initAudioController();

    m_book = new wxBookCtrl( this, wxID_ANY );
    createPlaybackPanel();
    createAuidoIOPanel();
    wxBoxSizer* mainSizer = new wxBoxSizer( wxVERTICAL );
    mainSizer->Add( m_book, 1, wxEXPAND );
    SetSizer( mainSizer );

    //init audio
    m_rpmSlider->SetValue(1000);
    m_volumeSlider->SetValue(1000);
    m_deltaMaxClimbSlider->SetValue((int)1);
    m_SecTrigSlider->SetValue(345);

    initStream();
}

AudioController* AudioPanel::getAudioController()
{
    return m_audioController;
}

void AudioPanel::initAudioController()
{
    m_audioController = new AudioController();

    //ObserverBase* messageObserver = new MessageBoxObserver();
    //m_audioController->registerObserver(messageObserver);
}

void AudioPanel::initStream()
{
    m_audioController->initStream();
    m_audioController->initSignalgen(
        m_triggerTypeCombo->GetValue(),
        m_rpmSlider->GetValue(),
        m_volumeSlider->GetValue());

    m_audioController->initViewerRefreshThread();

    //Logger::logf(LOG_MSG, _T(" AudioPanel::initAudioController() end"));
    //m_audioController->stop();
}

AudioPanel::~AudioPanel()
{
    //dtor
    Logger::logf(LOG_ERROR, _T("DTOR AudioPanel: START "));
     m_audioController->stop();
    delete m_audioController;
}


void AudioPanel::createPlaybackPanel()
{
    wxArrayString types;

    //types.Add(_T("sine"));
    types.Add( _T("c001"));
    types.Add( _T("c004"));
    types.Add( _T("c008"));
    types.Add( _T("c010"));
    types.Add( _T("c024"));
    types.Add( _T("c270"));

    types.Add( _T("m031"));
    types.Add( _T("m111"));
    types.Add( _T("m171"));
    types.Add( _T("m351"));
    types.Add( _T("m582"));
    types.Add( _T("m631"));

    types.Add( _T("f000"));
    types.Add( _T("e324"));
    types.Add( _T("s060"));
    types.Add( _T("s362"));
    types.Add( _T("l008"));

    wxPanel* panel= new wxPanel( m_book );
    wxBoxSizer* mainSizer = new wxBoxSizer( wxVERTICAL );

    panel->SetSizer( mainSizer );

    //trigger type box
    wxStaticBoxSizer* paramSizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("&Signalgen parameters") );
    wxStaticText* textTriggerType = new wxStaticText( panel, wxID_ANY, _T("Trigger type") );

    m_triggerTypeCombo = new wxComboBox( panel, ID_TRIGGER_TYPE_COMBO, _T("c004"),
                                        wxDefaultPosition, wxDefaultSize,
                                        types, wxTE_PROCESS_ENTER);

    m_triggerTypeCombo->SetSelection( 0 );
    paramSizer->Add( textTriggerType, 0, wxALL, 5 );
    paramSizer->Add( m_triggerTypeCombo, 0, wxALL, 5 );
    mainSizer->Add( paramSizer, 0, wxALL | wxEXPAND, 5 );

    //rpm box
    wxStaticBoxSizer *rpmSizer = new wxStaticBoxSizer( wxVERTICAL, panel, _T("&RPM") );
    mainSizer->Add( rpmSizer, 0, wxALL | wxEXPAND, 5 );
    m_rpmSlider = new wxSlider( panel, ID_RPM_SLIDER, 0, 0, 12000,
                             wxDefaultPosition, wxDefaultSize,
                             wxSL_LABELS);

    m_rpmSlider->SetTickFreq(40, 0);

    rpmSizer->Add( m_rpmSlider, 1, wxEXPAND );

    m_book->AddPage( panel, _T("Live Playback") );

    //volume slider
    wxStaticBoxSizer* volumeSizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("&Volume") );
    mainSizer->Add( volumeSizer, 0, wxALL | wxEXPAND, 5 );
    m_volumeSlider = new wxSlider( panel, ID_VOLUME_SLIDER, 0, 0, 1000,
                            wxDefaultPosition, wxDefaultSize,
                             wxSL_LABELS);

    m_volumeSlider->SetTickFreq(40, 0);
    volumeSizer->Add( m_volumeSlider, 1 );

    //Rpm Max Climb Slider
    wxStaticBoxSizer* deltaMaxClimbSizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("Rpm &Max Climb") );
    mainSizer->Add( deltaMaxClimbSizer, 0, wxALL | wxEXPAND, 5 );
    m_deltaMaxClimbSlider = new wxSlider( panel, ID_DELTAMAXCLIMBSLIDER, 0, 0, 10000,
                            wxDefaultPosition, wxDefaultSize,
                             wxSL_LABELS);

    m_deltaMaxClimbSlider->SetTickFreq(40, 0);
    deltaMaxClimbSizer->Add( m_deltaMaxClimbSlider, 1 );

     //Sec trigger Slider
    wxStaticBoxSizer* secTrigSizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("Sec. trigger Pos") );
    mainSizer->Add( secTrigSizer, 0, wxALL | wxEXPAND, 5 );
    m_SecTrigSlider = new wxSlider( panel, ID_SECTRIG, 0, 0, 360,
                            wxDefaultPosition, wxDefaultSize,
                             wxSL_LABELS);

    m_SecTrigSlider->SetTickFreq(40, 0);
    secTrigSizer->Add( m_SecTrigSlider, 1 );

    wxStaticBoxSizer* fluctSizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("Fluctuating (in %)") );
    mainSizer->Add( fluctSizer, 0, wxALL | wxEXPAND, 5 );
    m_fluctuatingSlider = new wxSlider( panel, ID_FLUCTUATING, 0, 0, 100, wxDefaultPosition, wxDefaultSize, wxSL_LABELS );
    fluctSizer->Add( m_fluctuatingSlider, 1 );

    //buttons
    wxBoxSizer* controlSizer = new wxBoxSizer( wxHORIZONTAL );
    wxButton* live_play = new wxButton( panel, ID_PLAY_BUTTON, _T("&Play") );
    wxButton* live_stop = new wxButton( panel, ID_STOP_BUTTON, _T("&Stop") );

    m_mutePrimSignalButton = new wxToggleButton( panel, ID_PRIMSIGNAL_MUTE, _T("Mute primary signal") );
    m_muteSecSignalButton = new wxToggleButton( panel, ID_SECSIGNAL_MUTE, _T("Mute secondary signal") );

    controlSizer->Add(live_play, 0, wxALL, 5);
    controlSizer->Add(live_stop, 0, wxALL, 5);
    controlSizer->Add( m_mutePrimSignalButton, 0, wxALL, 5 );
    controlSizer->Add( m_muteSecSignalButton, 0, wxALL, 5 );

#if defined(VEMSDEBUG)
    wxButton* file_play = new wxButton( panel, ID_FILE_PLAY_BUTTON, _T("Play from .csv &file") );
    wxButton* pulse_train = new wxButton( panel, ID_PULSE_TRAIN_BUTTON, _T("From pulse &train") );
    controlSizer->Add(file_play, 0, wxALL, 5);
    controlSizer->Add(pulse_train, 0, wxALL, 5);
#endif

    mainSizer->Add( controlSizer, 0, wxEXPAND );
    mainSizer->Fit(this);

    FindWindowById( ID_STOP_BUTTON, this )->Disable();
}


void AudioPanel::createAuidoIOPanel()
{
    wxPanel* panel = new wxPanel( m_book );

    m_outputDeviceCombo = new wxComboBox( panel, ID_AUDIO_OUTPUT_COMBO, wxEmptyString, wxDefaultPosition,
                                          wxDefaultSize, 0, NULL, wxCB_READONLY, wxDefaultValidator, _T("comboBox") );

    m_inputDeviceCombo = new wxComboBox(  panel, ID_AUDIO_INPUT_COMBO, wxEmptyString, wxDefaultPosition,
                                            wxDefaultSize, 0, wxCB_READONLY );
    //get audio devices
    //wxArrayString inputDevices;
    //wxArrayString outputDevices;
    //int defaultInputDevice = -1;
    int selectedOutputDeviceComboIdx = 0;
    int selectedInputDeviceComboIdx = 0;

    wxArrayString srarr = m_audioController->getAvailableSampleRate();
    int selectedSR = m_audioController->getSelectedSampleRate();

    m_sampleRateCombo = new wxComboBox(panel,ID_AUDIO_SAMPLE_RATE_COMBO,_T(""),wxDefaultPosition,wxDefaultSize,srarr,wxCB_SORT|wxCB_READONLY);
    m_sampleRateCombo->Select(selectedSR);

    try
    {
        portaudio::AutoSystem autoSys;
        System& paSystem = System::instance();
        int deviceCount = paSystem.deviceCount();

        for( int i = 0; i< deviceCount; i++ )
        {
            Device& ad = paSystem.deviceByIndex( i );
            const char* name = ad.name();
            wxString nameStr = wxString::FromAscii( name ) + wxString::Format( _T(" id : %d"), i );
            Logger::logf(LOG_ERROR, _T("AudioPanel::createAuidoIOPanel(): devices: ")+ nameStr);

            if(ad.isFullDuplexDevice() ){
                nameStr += _T(" (fullduplex)");
                //inputDevices.Add(nameStr);
                m_outputDeviceCombo->Append( nameStr, new int( i ) );
                m_inputDeviceCombo->Append( nameStr, new int( i ) );

            }else if( ad.isInputOnlyDevice() ){
                 m_inputDeviceCombo->Append( nameStr, new int( i ) );
            }else if( ad.isOutputOnlyDevice() ){
                m_outputDeviceCombo->Append( nameStr, new int( i ) );
            }

            /// SET OutputDevice
            if(m_audioController->getOutputDevice() == -1 ){
                 if( ad.isSystemDefaultOutputDevice() ){
                    selectedOutputDeviceComboIdx = m_outputDeviceCombo->GetCount()-1;
                }
            }

            if( m_audioController->getOutputDevice() == i){
                Logger::logf(LOG_ERROR, _T("AudioPanel::createAuidoIOPanel(): m_selectedOutputDevice found =%i"),i );
                selectedOutputDeviceComboIdx = m_outputDeviceCombo->GetCount()-1;
            }

            /// SET InputDevice
            if(m_audioController->getInputDevice() == -1 ){
                 if( ad.isSystemDefaultInputDevice() ){
                    selectedInputDeviceComboIdx = m_inputDeviceCombo->GetCount()-1;
                }
            }

            if( m_audioController->getInputDevice() == i){
                Logger::logf(LOG_ERROR, _T("AudioPanel::createAuidoIOPanel(): m_selectedInputDevice found =%i"),i );
                selectedInputDeviceComboIdx = m_inputDeviceCombo->GetCount()-1;
            }
        }
    }catch( Exception& e ){
        const char* str = e.what();
        //serialErrorMsgDialog((const char *) str);
        wxString msg = wxString::FromAscii(str);
        Logger::log(LOG_MSG,_T("AudioPanel::createAuidoIOPanel: ")+ msg);
        return;
    }

    m_outputDeviceCombo->SetSelection( selectedOutputDeviceComboIdx );
    m_inputDeviceCombo->SetSelection( selectedInputDeviceComboIdx );
    //Audio device settings panel
    //wxPanel* audioDevicePanel = new wxPanel( m_book );
    wxBoxSizer* audioDeviceSizer = new wxBoxSizer( wxVERTICAL );

    wxStaticBoxSizer* playbackSizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("&Playback") );
    wxStaticText* outputDeviceText = new wxStaticText( panel, wxID_ANY, _T("Device :") );

    Logger::logf(LOG_ERROR, _T("AudioPanel::createAuidoIOPanel(): selectedOutputDeviceComboIdx=%i"), selectedOutputDeviceComboIdx );

    playbackSizer->Add( outputDeviceText, 0, wxALL, 5 );
    playbackSizer->Add( m_outputDeviceCombo, 1, wxALL, 5);
    audioDeviceSizer->Add( playbackSizer, 0, wxEXPAND | wxALL, 5 );

    wxStaticBoxSizer* recordingSizer = new wxStaticBoxSizer( wxVERTICAL, panel, _T("&Recording") );
    wxBoxSizer* inputDeviceSizer = new wxBoxSizer( wxHORIZONTAL );
    wxStaticText* inputDeviceText = new wxStaticText( panel, wxID_ANY, _T("Device :") );
    //wxComboBox* m_inputDeviceCombo = new wxComboBox( audioDevicePanel, wxID_ANY );




    wxStaticBoxSizer* samplerateSizer = new wxStaticBoxSizer( wxVERTICAL, panel, _T("&Sample rate") );
    samplerateSizer->Add(m_sampleRateCombo,0,wxALL ,5);

    //wxStaticBoxSizer* recordingSizer = new wxStaticBoxSizer( wxVERTICAL, panel, _T("&SampleRate") );
    //wxBoxSizer* inputDeviceSizer = new wxBoxSizer( wxHORIZONTAL );
    //wxStaticText* inputDeviceText = new wxStaticText( panel, wxID_ANY, _T("Device :") );

    inputDeviceSizer->Add( inputDeviceText, 0, wxALL, 5 );
    inputDeviceSizer->Add( m_inputDeviceCombo, 1, wxALL, 5 );
    recordingSizer->Add( inputDeviceSizer, 0, wxEXPAND );

    audioDeviceSizer->Add( recordingSizer, 0, wxEXPAND | wxALL, 5 );

    audioDeviceSizer->Add( samplerateSizer, 0, wxEXPAND | wxALL, 5 );

    panel->SetSizer( audioDeviceSizer );
    m_book->AddPage( panel, _T("Audio I/O") );
}

//event handlers
void AudioPanel::OnComboBoxSelected( wxCommandEvent &event )
{
    //m_audioController->stop();
    //m_audioController->set
    //m_audioController->start();

     m_audioController->initSignalgen(m_triggerTypeCombo->GetValue(), m_rpmSlider->GetValue(), m_volumeSlider->GetValue() );
}

void AudioPanel::OnSampleRateSelected( wxCommandEvent& event )
{

    long val=0;
    m_sampleRateCombo->GetValue().ToLong(&val);
    int sampleRate = (int)val;
    Logger::logf(LOG_MSG,_T("AudioPanel::OnSampleRateSelected() %i"),val);

    m_audioController->setSampleRate(sampleRate);
    m_audioController->stop();
    wxThread::Sleep(30);
    m_audioController->start();

}

void AudioPanel::OnAudioOutputSelected( wxCommandEvent &event )
{
     Logger::logf(LOG_ERROR, _T("AudioPanel::OnAudioOutputSelected()") );
    wxString value = m_outputDeviceCombo->GetValue();
    long valueIdx = m_outputDeviceCombo->GetSelection();
    //m_selectedOutputDevice = *(int*) m_outputDeviceCombo->GetClientData(valueIdx);

    m_audioController->setOutputDevice( *(int*) m_outputDeviceCombo->GetClientData(valueIdx) );
   // value.ToLong(&valueLong, -1);
    Logger::logf(LOG_ERROR, _T("AudioPanel::OnAudioOutputSelected() out ")+ value);
//    Logger::logf(LOG_ERROR, _T("AudioPanel::OnAudioOutputSelected() out id=%d"), m_selectedOutputDevice );

    //m_audioController->stop();
    //delete m_audioController;
    //initAudioController();

}

void AudioPanel::OnAudioInputSelected( wxCommandEvent &event )
{
     Logger::logf(LOG_ERROR, _T("AudioPanel::OnAudioInputSelected()") );
    wxString value = m_inputDeviceCombo->GetValue();
    long valueIdx = m_inputDeviceCombo->GetSelection();
    m_audioController->setInputDevice( *(int*) m_inputDeviceCombo->GetClientData(valueIdx) );
   // value.ToLong(&valueLong, -1);
    Logger::logf(LOG_ERROR, _T("AudioPanel::OnAudioInputSelected() out ")+ value);
}

void AudioPanel::OnRpmSliderScroll( wxScrollEvent& event )
{
    m_audioController->setRpm( m_rpmSlider->GetValue() );
}


void AudioPanel::OnVolumeSliderScroll( wxScrollEvent& event )
{
    m_audioController->setAmplitude( ((double) m_volumeSlider->GetValue()) );
}

void AudioPanel::OnDeltaMaxClimberSliderScroll( wxScrollEvent& event )
{
    m_audioController->setDeltaMaxClimb( (double) m_deltaMaxClimbSlider->GetValue() );

}

void AudioPanel::OnSecTrigSliderScroll( wxScrollEvent& event )
{
    m_audioController->setSecTrigAngle( (double) m_SecTrigSlider->GetValue() );

}


void AudioPanel::OnPlayButton( wxCommandEvent &event )
{

    m_audioController->initSignalgen(m_triggerTypeCombo->GetValue(),
        m_rpmSlider->GetValue(),
        m_volumeSlider->GetValue(),
        m_audioController->GetSampleRate());
    m_audioController->start();

    FindWindowById( ID_PLAY_BUTTON, this )->Disable();
    //FindWindowById( ID_STOP_BUTTON, this )->Enable();

}

void AudioPanel::OnStopButton( wxCommandEvent &event )
{
    m_audioController->stop();

    FindWindowById( ID_PLAY_BUTTON, this )->Enable();
    FindWindowById( ID_STOP_BUTTON, this )->Disable();

}

void AudioPanel::OnPulseTrainButton( wxCommandEvent& event )
{
    Logger::log(LOG_MSG, _T("AudioPanel::OnPulseTrainButton"));
    if (m_pulse_train==NULL) m_pulse_train=new Main_Pulse_train(this);

}

void AudioPanel::processPulseTrainPacket(SIPRPacket &pulsePacket)
{
    //Logger::logf(LOG_MSG,_T("AudioPanel::processPulseTrainPacket"));
    m_audioController->processPulseTrainPacket(pulsePacket);
}

void AudioPanel::OnFilePlayButton( wxCommandEvent &event )
{
    Logger::logf(LOG_MSG,_T("void AudioPanel::OnFilePlayButton"));

    //m_audioController->initSignalgen(m_triggerTypeCombo->GetValue(), m_rpmSlider->GetValue(), m_volumeSlider->GetValue());
    //m_audioController->start();
    if (m_rpmFromFile == NULL) {
        m_rpmFromFile = new RpmFromFile(this);
        RTDataMgrBase* mgrBase = m_audioController->getRTDataMgrBase();
        m_rpmFromFile->SetDataMgr( mgrBase );
        m_rpmFromFile->RegisterViewer();
        return;
    }
    m_rpmFromFile->SetRpmIdx(0);


/*
    m_rpmFromFile->parseNextLine();
    double rpmvalue = m_rpmFromFile->GetRpmValue();

    m_rpmSlider->SetValue(rpmvalue);
    m_audioController->setRpm(rpmvalue);
    //m_sparkTesterWidgetFactory->ShowWidget(SPARKW_GAUGE);

*/
}

void AudioPanel::OnMuteButtons( wxCommandEvent& event )
{
    int id = event.GetId();

    bool muteit;

    if ( id == ID_PRIMSIGNAL_MUTE )
    {
        muteit = m_mutePrimSignalButton->GetValue();
        m_audioController->muteSignals( (int)muteit , -1 );
    }
    if ( id == ID_SECSIGNAL_MUTE )
    {
        muteit = m_muteSecSignalButton->GetValue();
        m_audioController->muteSignals( -1, (int)muteit );
    }
}

void AudioPanel::OnFluctuatingScroll( wxScrollEvent& event )
{
    int value = m_fluctuatingSlider->GetValue();

    m_audioController->setFluctuation( value );
    //Logger::logf(LOG_MSG, _T("Current position: %d"), m_fluctuatingSlider->GetValue() );
}
