#include "SignalProcessThread.h"

#include "TriggerDetect.h"
#include "Histogram.h"
//#include "FullDuplexTest.h" // only for FRAMES_PER_BUFFER constant
#include "../rtdatamanager/RTDataMgrBase.h"
#include "../ringbuffer/RingBuffer.h"

#include "../logger.h"

// this depends on the phase of the signals, like trigger_tooth, ign_tdcdelay and the output cyl we watch
// when you strobe, you make a white mark on the pulley at certain position
#define WHITEMARK (720)

// fading (*65536) per second
// 65535 is VERY slow fading. 50000 is rather quick.
#define HISTOGRAM_FADING (65535)

//#include "signalgauges/RTSignalViewerBase.h"

SignalProcessThread::SignalProcessThread( RingBuffer *rbp, RTDataMgrBase *mgr,  DoubleHash* viewerModel)
{
    m_sampleIdx = 0; // permanent index; not zeroed for every processBlock(), +=1 for every sample(pair: L+R)
    m_SecTrigPulse = 0;
    m_SecTrigPeriodSampleCount = 0;
    m_SectrigPulseCount = 0;
    m_SparkCount = 0;
    m_DataFallingEdge = 0;
    m_DataRisingEdge = 0;
    m_mgrBase = 0;
    test  = 720;
    m_isRisingEdge = false;
    m_isRisingEdge2 = false;
    m_triggerType = 0;

    m_ringBuffer = rbp;
    m_mgrBase = mgr;
    m_viewerModel = viewerModel;

    //m_RTDataViewerBaseA = new ArrayOfRTDataViewerBase();

    // 23 +2 bin, for 6 .. 48 ( 2 degree resolution )
    sparkPhaseHistogram = new ResultHistogram( 46, 2, 48 );

    // will be: 10 + 2 bin for 1000 .. 6000 msec
    // now 22usec resolution (44100 sps), 3900 usec = 172
    dwellTimeHistogram = new ResultHistogram( 35, 50, 400);

    primtrig.setName( _T("primtrig") );
    localprimtrig.setName( _T("localprimtrig") );
    sectrig.setName( _T(" sectrig") );

    primtrig.setTowardPeak( 255 );
    localprimtrig.setTowardPeak( 255 );
    sectrig.setTowardPeak( 255 );

    primtrig.setTowardMid( 3 );
    localprimtrig.setTowardMid( 0 );
    sectrig.setTowardMid( 3 );

    primtrig.setThresh256( 100 );
    sectrig.setThresh256( 128 );

// THRESHOLD_STRATEGY_AROUND_MEAN does NOT work !!!
   // primtrig.setThresholdStrategy( THRESHOLD_STRATEGY_AROUND_MEAN );
   // sectrig.setThresholdStrategy( THRESHOLD_STRATEGY_AROUND_MEAN ); // use thresholds around average for camsync

    primtrig.setTowardAverage( 3 );
    localprimtrig.setTowardAverage( 3 );
    sectrig.setTowardAverage( 3 );

    sparkPeaks.setTowardAverage(3);
    sparkPeaks.setTowardMid(1000);
    sparkPeaks.setTowardPeak( 255 ); // detect peak immediately

    //const char * fileName = ;
    wxString fileName = _T("/tmp/debugwav.txt");
    m_debugWav = new wxFile( fileName, wxFile::write);

    m_histo = new Histogram();

/// test
//localprimtrig.loadmprevsample();
//localprimtrig.outlierElimination();


}

SignalProcessThread::~SignalProcessThread()
{

    //dtor
}


/**
 * returns 2*60 * 44100.0 / m_SecTrigPeriodSampleCount ;
 * assumes preprocessed data: m_rpm_constant =  (720 << 12 ) / m_SecTrigPeriodSampleCount;
 */
uint16_t SignalProcessThread::getRPM()
{
    //Logger::logf(LOG_MSG, _T("SignalProcessThread::getRPM() m_rpm_constans=%05d"),m_rpm_constant);

    //uint16_t rpm_magic = 16000;// when 96000bit/s
    uint16_t rpm_magic =3065;// when 250000bit/s(pulse train)
    // old value: rpm_magic = 7350;// m_rpm_constant already has 720<<8* but we need 60 * 2 * 44100 *, so ...;
    uint32_t t32 = m_rpm_constant * rpm_magic;
    uint16_t rpm = t32 >> 12 ;
    return rpm;
}

/** ==============
 * For testing: put random circle coordinates into histogram
 */
uint32_t SignalProcessThread::circleOfLifeTest( void )
{
    double angle = (rand() & 0xFFFF) * 2 * 3.14159265358 / 65536.0;
    uint16_t x = AMPLIDUDE_BINS >> 1;//SPARKPHASE_BINS >> 1;              //max 256
    x += (int16_t)((double)x * sin(angle));             // max 720
    uint16_t y = SPARKPHASE_BINS >> 1;//AMPLIDUDE_BINS >> 1;
    y += (int16_t)((double)y * cos(angle));
    /*uint16_t ret =*/ m_histo->putInHistogram(x,y);
    //Logger::logf(LOG_MSG, _T("circleOfLifeTest coords=(%d,%d), %d"), x, y, m_histo->m_hisztogram_array[x][y]);
/*
    uint16_t lrate = rand()%1000+1;
    m_histo->m_hisztogram_array[x][y] = lrate;
    m_histo->CheckMaxRate(lrate);
    NotifySignalViewers(x, y);
*/
    return (y << 16) | x;
}
uint32_t SignalProcessThread::otherDummyTest( void )
{
    double rnd = (rand() & 0xFFFF) / 65536.0;
    uint16_t x = (uint16_t)(rnd * (double)AMPLIDUDE_BINS);
    uint16_t y = (uint16_t)(rnd * (double)SPARKPHASE_BINS);
    /*uint16_t ret =*/ m_histo->putInHistogram(x,y);
    return (y << 16) | x;
}

// processes primtrig (left) and sectrig (right) channels
void SignalProcessThread::processBlock(short * sampleblock, int samplePerChannel, int channel)
{
    Logger::logf(LOG_MSG, _T("SignalProcessThread::processBlock"));

    int index = 0;
    uint16_t sparkP=0;

    // 2 channels per frame (stereo)
    while( index < (samplePerChannel * channel) ){
        int i=0;
        uint8_t prevstate = primtrig.getState();
        // 0x8000 ^ ( same as 0x8000 + )  is for signed short to uint16_t conversion

        //m_buffer[i++] =(sampleblock[ index ]) ;
        uint16_t primTrigSample = 0x8000 ^ (uint16_t)(sampleblock[ index++ ]); // left
        //m_buffer[i++] =(sampleblock[ index ]) ;
        uint16_t secTrigSample = 0x8000 ^ (uint16_t)(sampleblock[ index++ ]); // right

       /*uint16_t primlocalpeakp =*/ localprimtrig.getPeakP();
        // uint16_t primlocalpeakn =  localprimtrig.getPeakN();
        m_prevsampleaverage = primtrig.outlierElimination();
        /*uint16_t primPrevSample =*/ primtrig.getLastSample();
        primtrig.setTowardMid(getRPM());
        primtrig.feedSample( primTrigSample );
        localprimtrig.setTowardMid(getRPM());
        localprimtrig.feedSample(primTrigSample);
        //falling edge
        if( prevstate == 1 && primtrig.getState() == 0 ){ // primtrig falling edge found
//        if( prevstate == 0 && primtrig.getState() == 1 ){ // primtrig falling edge found inverted

            dataFallingEdge();

            //m_prevsampleaverage = prevsampleaverage;
           // m_primValueBeforeFallingedge = primPrevSample;
        }

        // TODO: use usec for this, NOT degree
         if( prevstate == 0 && primtrig.getState() == 1 ){ // primtrig rising edge found
//         if( prevstate == 1 && primtrig.getState() == 0 ){ // primtrig rising edge found inverted
            //Logger::logf(LOG_MSG, _T("in processBlock() after prim rising dege"));

            dataRisingEdge();

            uint16_t primpeakn =  localprimtrig.getPeakN();
            uint16_t diff; // amplitude
            m_SparkCount++; // increase for ALL primtrig rising edge ? might be 5 pulses/period ...

            // calculate dwell TIME (not phase)


            // should we use 1 usec resolution (NOT 4 usec and NOT 22.76 usec that is sampletime for 44100 sps)
            // uint16_t dwelltime = 1000000 / 44100
            dwellTimeHistogram->increment( m_dwellLenSamples );
            dwellPeaks.feedSample( m_dwellLenSamples );    // TODO: display ave and peaks on gauge

            //sparkP = getSparkP();
            uint16_t relative_angle = WHITEMARK - sparkP;
            sparkPhaseHistogram->increment( relative_angle );
            sparkPeaks.feedSample( relative_angle );   // TODO: display ave and peaks on gauge

            //diff = (m_primValueBeforeFallingedge > primpeakn) ? m_primValueBeforeFallingedge - primpeakn : 0;//localprimtrig.diffPeak();
            diff = (m_prevsampleaverage > primpeakn) ? m_prevsampleaverage - primpeakn : 0;
            ///*Logger::logf(LOG_MSG, _T("processBlock rising edge diff = (%d - %d=) %d sparkP = %i")
                ///,m_prevsampleaverage//m_primValueBeforeFallingedge
                ///,primpeakn
                ///,diff
                ///,sparkP);
            uint16_t x;
            if( 0 ){    // just for testing
                uint32_t xy;
                if( 0 ){
                    xy = circleOfLifeTest() ; //
                } else { // another dummy test:
                    xy = otherDummyTest();
                }
                sparkP = xy >> 16;
                x = xy & 0xFFFF;
            } else {
                x = m_histo->putInHistogram(diff >> 7, sparkP);
            // Logger::logf(LOG_MSG, _T("processBlock= rising edge (%d,%d), %d"), x, sparkP, m_histo->m_hisztogram_array[x][sparkP]);
            }

            m_histo->saveAmplitudeSums(diff);
            m_histo->saveSparkphase(sparkP);
            localprimtrig.resetLocalPeak(primTrigSample);

            //NotifySignalViewers(x, sparkP);
/*

           if(test == 50){
                test = 40;
            }else if(test == 40){
                test = 60;
            }else{
                test = 50;
            }
*//*
            test = rand() % 100 + 400;
            table[_T("phase")] = test;
            table[_T("ampl")] = x;

            table[_T("rpm")] = test-200;
            table[_T("map")] = 3* test;
*/


            //table[_T("map")] = 3* test;

            //setSymbolsAtPrimeTrigger();

            //m_mgrBase->notifyViewers();
            ///NotifySignalViewers();

        }

        /* ======================== */
        prevstate = sectrig.getState();
        sectrig.setTowardMid(getRPM());
        sectrig.feedSample( secTrigSample );
        if( prevstate == 1 && sectrig.getState() == 0 ){ // sectrig falling edge
//        if( prevstate == 0 && sectrig.getState() == 1 ){ // sectrig falling edge inverted

            secTrigFallingEdge();

        }

        if( prevstate == 0 && sectrig.getState() == 1 ){ // sectrig falling edge
//        if( prevstate == 1 && sectrig.getState() == 0 ){ // sectrig rising edge inverted;

            //Logger::logf(LOG_MSG, _T("sectrig rising edge inverted "));

            secTrigRisingEdge();


        }
        // write leftSample + rightSample + primtrig.state + sectrig.state
        // into 10 channel RAW (little endian) file. Audacity/Project/import is your friend:
        m_buffer[i++] = (secTrigSample- 32767)/2;
        //m_buffer[i++] = secTrigSample+(m_sampleIdx % 65535) - 32767;//direct loopback
        //m_buffer[i++] = secTrigSample - 32767;//direct loopback
        m_buffer[i++] = (primTrigSample - 32767)/2;//ignition output
        //m_buffer[i++] = 32767 * sectrig.getState();
        //m_buffer[i++] = 32767 * primtrig.getState();
//        m_buffer[i++] = sectrig.getAverage()- 32767;
//        m_buffer[i++] = primtrig.getPeakP()- 32767;
//        m_buffer[i++] = primtrig.getPeakN()- 32767;
        //m_buffer[i++] = sectrig.getUpTreshold()-32767;
        //m_buffer[i++] = sectrig.getLoTreshold()-32767;
//        m_buffer[i++] = primtrig.getUpTreshold()-32767;
//        m_buffer[i++] = primtrig.getLoTreshold()-32767;
        //m_buffer[i++] = sectrig.getPeakP()- 32767;
        //m_buffer[i++] = sectrig.getPeakN()- 32767;
        //m_buffer[i++] = sectrig.getHighCounter()*1000;
        //m_buffer[i++] = sectrig.getLowCounter()*1000;
        //m_buffer[i++] = getRPM();
        //m_buffer[i++] = m_prevSpark*50;

        m_debugWav->Write(m_buffer,4 );

        m_sampleIdx++;

    }

}

void SignalProcessThread::setSymbolsAtPrimeTrigger()
{
    (*m_viewerModel)[_T("Dphase")] = getDwellP();
    (*m_viewerModel)[_T("Dphase2")] = getDwellP2();
    (*m_viewerModel)[_T("phase")] = getSparkP();
    (*m_viewerModel)[_T("phase2")] = getSparkP2();
    (*m_viewerModel)[_T("sparkPos")] =m_SparkPhase;
    (*m_viewerModel)[_T("sparkPos2")] =m_SparkPhase2;
    (*m_viewerModel)[_T("dwellPos")] =m_dwellPhase;
    (*m_viewerModel)[_T("dwellPos2")] =m_dwellPhase2;

    //m_viewerModel[_T("ampl")] = x;
    (*m_viewerModel)[_T("rpm")] = getRPM();
//    Logger::logf(LOG_MSG,_T("SignalProcessThread::setSymbolsAtPrimeTrigger()rpm=%d"),(int)(*m_viewerModel)[_T("rpm")]);
//    uint32_t dwellLenusec = ((256 * 1000000 / 96000) * m_dwellLenSamples ) >> 8;//when audio_record
    uint32_t dwellLenusec = m_dwellLenSamples * 10;//pulse train

    (*m_viewerModel)[_T("dwell")] = dwellLenusec;
    uint32_t dwellLenusec2 = m_dwellLenSamples2 * 10;//pulse train

    (*m_viewerModel)[_T("dwell2")] = dwellLenusec2;

    setSymbolSparkPeaks();
}

void SignalProcessThread::setSymbolsAtSecTrigger()
{
    (*m_viewerModel)[_T("rpm")] = getRPM();
}

void SignalProcessThread::setSymbolSparkPeaks()
{
    wxString symbolName = _T("sparkPeak");
    double avg = (double )  getSparkPeaks()->getAverage();
    double peakN = (double )getSparkPeaks()->getPeakN();
    double peakP = (double )getSparkPeaks()->getPeakP();

    (*m_viewerModel)[symbolName] = avg;
    (*m_viewerModel)[symbolName+_T("_low")] = peakN;
    (*m_viewerModel)[symbolName+_T("_high")] = peakP;
}

/*
void SignalProcessThread::NotifySignalViewers()
{
  //  Logger::logf(LOG_MSG,_T("Notify c=%i"),m_signalViewers.GetCount());
    RTDataViewerBase *rtDataViewer;
    for(uint8_t vs = 0; vs < m_RTDataViewerBaseA->GetCount(); vs++  ){
        //m_RTDataViewerBaseA[vs]->refresh(&table);
        rtDataViewer = m_RTDataViewerBaseA->Item(vs);
        rtDataViewer->refresh(&table);
    }
}
*/

/*
void SignalProcessThread::RegisterSignalViewer( RTSignalViewerBase *viewer )
{
 //   Logger::logf(LOG_MSG,_T("SignalProcessThread::RegisterSignalViewer = %p"),viewer);
    m_signalViewers.Add( viewer );
    viewer->SetValueObj( m_histo );
}

bool SignalProcessThread::UnRegisterSignalViewer( RTSignalViewerBase *viewer )
{
    m_signalViewers.Remove( viewer );
    return true;
}
*/

uint16_t SignalProcessThread::getDwellP()
{
    uint32_t t32;

  if(!m_isRisingEdge){
            t32=m_prevDwell;
    }else{
        uint32_t Dt=m_dwellPhase;
        uint32_t DT=m_SecTrigPulse2-m_PrevSecTrigPulse;
        int Drpm=m_SecTrigPeriodSampleCount-m_PrevSecTrigPeriodSampleCount;
        double rpmt=m_PrevSecTrigPeriodSampleCount+Drpm*((double)Dt/DT);
        double Tt=Dt*(rpmt+m_PrevSecTrigPeriodSampleCount);
        double TT=DT*(m_PrevSecTrigPeriodSampleCount+m_SecTrigPeriodSampleCount);

        t32=(uint32_t)(Tt/TT*720);
        m_prevDwell=(uint16_t)t32;
    }
    return t32;
}
uint16_t SignalProcessThread::getDwellP2()
{
    uint32_t t32;
  if(!m_isRisingEdge){
            t32=m_prevDwell2;
    }else{
        uint32_t Dt=m_dwellPhase2;
        uint32_t DT=m_SecTrigPulse2-m_PrevSecTrigPulse;
        int Drpm=m_SecTrigPeriodSampleCount-m_PrevSecTrigPeriodSampleCount;
        double rpmt=m_PrevSecTrigPeriodSampleCount+Drpm*((double)Dt/DT);
        double Tt=Dt*(rpmt+m_PrevSecTrigPeriodSampleCount);
        double TT=DT*(m_PrevSecTrigPeriodSampleCount+m_SecTrigPeriodSampleCount);

        t32=(uint32_t)(Tt/TT*720);
        m_prevDwell2=(uint16_t)t32;
    }
    return t32;
}
uint16_t SignalProcessThread::getSparkP()
/*
    m_PrevSecTrigPulse//t1
    m_SecTrigPulse2 //t2
    m_PrevSecTrigPeriodSampleCount//rpm1
    m_SecTrigPeriodSampleCount//rpm2
    m_SparkPhase//t
*/

{


    uint32_t t32;

  if(!m_isRisingEdge){
            (*m_viewerModel)[_T("missing")] = 1;
            t32=m_prevSpark;

    }else{
        (*m_viewerModel)[_T("missing")] = 0;

        uint32_t Dt=m_SparkPhase;
        uint32_t DT=m_SecTrigPulse2-m_PrevSecTrigPulse;
        int Drpm=m_SecTrigPeriodSampleCount-m_PrevSecTrigPeriodSampleCount;
        double rpmt=m_PrevSecTrigPeriodSampleCount+Drpm*((double)Dt/DT);
        double Tt=Dt*(rpmt+m_PrevSecTrigPeriodSampleCount);
        double TT=DT*(m_PrevSecTrigPeriodSampleCount+m_SecTrigPeriodSampleCount);

        t32=(uint32_t)(Tt/TT*720);
        m_prevSpark=(uint16_t)t32;
    }
    m_isRisingEdge = false;
    return t32;

    //Logger::logf(LOG_MSG, _T("SignalProcessThread::getSparkP() rpm1 = %i rpm2 = %i"),m_PrevSecTrigPeriodSampleCount,m_SecTrigPeriodSampleCount);
    //Logger::logf(LOG_MSG, _T("SignalProcessThread::getSparkP() Tt = %f TT = %f"),Tt,TT);

/*
//old version
    uint32_t t32 = m_rpm_constant;
    t32 *= m_SparkPhase;
    // /256 because ... see definition of m_rpm_constant
    t32 >>= 12;
    if( t32 > 720) return m_prevSpark;
*/
}
uint16_t SignalProcessThread::getSparkP2()
/*
    m_PrevSecTrigPulse//t1
    m_SecTrigPulse2 //t2
    m_PrevSecTrigPeriodSampleCount//rpm1
    m_SecTrigPeriodSampleCount//rpm2
    m_SparkPhase//t
*/

{


    uint32_t t32;

  if(!m_isRisingEdge2){
            (*m_viewerModel)[_T("missing2")] = 1;
            t32=m_prevSpark2;

    }else{
        (*m_viewerModel)[_T("missing2")] = 0;

        uint32_t Dt=m_SparkPhase2;
        uint32_t DT=m_SecTrigPulse2-m_PrevSecTrigPulse;
        int Drpm=m_SecTrigPeriodSampleCount-m_PrevSecTrigPeriodSampleCount;
        double rpmt=m_PrevSecTrigPeriodSampleCount+Drpm*((double)Dt/DT);
        double Tt=Dt*(rpmt+m_PrevSecTrigPeriodSampleCount);
        double TT=DT*(m_PrevSecTrigPeriodSampleCount+m_SecTrigPeriodSampleCount);

        t32=(uint32_t)(Tt/TT*720);
        m_prevSpark2=(uint16_t)t32;
    }
    m_isRisingEdge2 = false;
    return t32;

    //Logger::logf(LOG_MSG, _T("SignalProcessThread::getSparkP() rpm1 = %i rpm2 = %i"),m_PrevSecTrigPeriodSampleCount,m_SecTrigPeriodSampleCount);
    //Logger::logf(LOG_MSG, _T("SignalProcessThread::getSparkP() Tt = %f TT = %f"),Tt,TT);

/*
//old version
    uint32_t t32 = m_rpm_constant;
    t32 *= m_SparkPhase;
    // /256 because ... see definition of m_rpm_constant
    t32 >>= 12;
    if( t32 > 720) return m_prevSpark;
*/
}
void SignalProcessThread::setTriggerType(const wxString& type)
{
    m_triggerType=0;
    if (type.StartsWith( _T("s") )) m_triggerType=SUBARU_CAMSYNC;
    if (type.StartsWith( _T("l") )) m_triggerType=LANCIA_COSWORTH;

    Logger::logf(LOG_MSG, _T("SignalProcessThread: trigger type: %s"),type.GetData());
}


void SignalProcessThread::consumeRecordedBlocks()
{

            int processedblocks = 0;
            int problembocks = 0;
/*
            while( m_ringBuffer->isEmpty() == false ){
                    //m_histo->loadHistogram();
                    char *buf = m_ringBuffer->getTailPointer(); // it never gives NULL
                    m_ringBuffer->nextTail();

                    // TODO: verify
                    if( buf == NULL ){
                        problembocks ++; // should never happen
                    } else {
                        processBlock( (short*) buf );
                    }
                    processedblocks ++;
            }
*/
            if( processedblocks != 0 ){
                if( problembocks != 0){
                 ///   Logger::logf(LOG_MSG, _T("SignalProcess processed %d, bad %d !"), processedblocks, problembocks);
                }
/**
                Logger::logf(LOG_MSG, _T("Period: %d, DwellSamplePos %d, SparkSamplePos %d!")
                    , m_SecTrigPeriodSampleCount
                    , m_dwellPhase
                    , m_SparkPhase );
*/
                // uint16_t dwellP = (m_rpm_constant * m_dwellPhase)>>12;

                // multiply by 1000000 / 44100  (time of 1 sample in 1 usec unit)
/**

                uint16_t sparkP = getSparkP();
                uint16_t rpm = getRPM();
                Logger::logf(LOG_MSG, _T("Period: %d (rpm=%d), Missed %d (%d-%d), Spark at %d(%d) deg, Dwell=%d usec, ")
                    ,m_SecTrigPeriodSampleCount
                    ,rpm
                    ,m_SectrigPulseCount - m_SparkCount
                    ,m_SectrigPulseCount
                    ,m_SparkCount
                    ,WHITEMARK-sparkP
                    ,sparkP
                    ,dwellLenusec );
   */
                //m_histo->loadHistogram();
               // m_histo->printPacked();

                ///primtrig.printData();
                ///sectrig.printData();

                //dwellTimeHistogram->vectorMult( HISTOGRAM_FADING );
                //sparkPhaseHistogram->vectorMult( HISTOGRAM_FADING );

                sparkPhaseHistogram->vectorPrint();
                dwellTimeHistogram->vectorPrint();

                //Refresh the RTMgr;
                //if(m_mgrBase != NULL ) m_mgrBase->update();
            }

}

/*
void* SignalProcessThread::Entry()
{
        Logger::logf(LOG_MSG, _T("SignalProcessThread START!"));
        while(1){
            consumeRecordedBlocks();
            if(m_mgrBase != NULL ){
                m_mgrBase->updateViewers(); // m_mgrBase will query signalProcess->getRPM()
                // Logger::logf(LOG_MSG, _T("SignalProcess => gauge test!"));
            }
            wxThread::Sleep(100);

          //  Logger::logf(LOG_MSG, _T("SignalProcessThread cycle"));

        }
}
*/

void SignalProcessThread::dataFallingEdge()
{
    m_DataFallingEdge = m_sampleIdx;
    m_dwellPhase = m_DataFallingEdge - m_SecTrigPulse;
    //Logger::logf(LOG_MSG, _T("SignalProcessThread::dataFallingEdge m_dwellLenSamples??:%04X"),(m_DataFallingEdge -m_DataRisingEdge));

    long pw;
    pw=m_DataFallingEdge-m_DataRisingEdge;
    pw=pw*100;
    pw=pw/(m_DataFallingEdge-m_DataFallingEdgeold);
    (*m_viewerModel)[_T("pulse_width")] =(double)pw;

   //Logger::logf(LOG_MSG, _T("SignalProcessThread::dataFallingEdge pulse_width:%04d"),pw);
    m_DataFallingEdgeold=m_DataFallingEdge;

}
void SignalProcessThread::dataRisingEdge()
{


    m_DataRisingEdge = m_sampleIdx;
    m_SparkPhase = m_DataRisingEdge - m_SecTrigPulse;
    m_isRisingEdge = true;
    m_dwellLenSamples = m_DataRisingEdge - m_DataFallingEdge;
    //Logger::logf(LOG_MSG, _T("SignalProcessThread::dataRisingEdge m_dwellLenSamples:%04X"),m_dwellLenSamples);

}void SignalProcessThread::dataFallingEdge2()
{
    m_DataFallingEdge2 = m_sampleIdx;
    m_dwellPhase2 = m_DataFallingEdge2 - m_SecTrigPulse;
    //Logger::logf(LOG_MSG, _T("SignalProcessThread::dataFallingEdge m_dwellLenSamples??:%04X"),(m_DataFallingEdge -m_DataRisingEdge));

    long pw;
    pw=m_DataFallingEdge2-m_DataRisingEdge2;
    pw=pw*100;
    pw=pw/(m_DataFallingEdge2-m_DataFallingEdgeold2);
    (*m_viewerModel)[_T("pulse_width2")] =(double)pw;

   //Logger::logf(LOG_MSG, _T("SignalProcessThread::dataFallingEdge pulse_width:%04d"),pw);
    m_DataFallingEdgeold2=m_DataFallingEdge2;

}
void SignalProcessThread::dataRisingEdge2()
{


    m_DataRisingEdge2 = m_sampleIdx;
    m_SparkPhase2 = m_DataRisingEdge2 - m_SecTrigPulse;
    m_isRisingEdge2 = true;
    m_dwellLenSamples2 = m_DataRisingEdge2 - m_DataFallingEdge2;
    //Logger::logf(LOG_MSG, _T("SignalProcessThread::dataRisingEdge m_dwellLenSamples:%04X"),m_dwellLenSamples);

}
void SignalProcessThread::secTrigFallingEdge()
{
    m_SecTrigPulse = m_sampleIdx;
    m_SectrigPulseCount ++;
}

void SignalProcessThread::secTrigRisingEdge()
{
            if (m_triggerType==SUBARU_CAMSYNC){
//            if (m_triggerType.StartsWith( _T("s") )){

                //Logger::logf(LOG_MSG, _T(" trigger type: %s"),m_triggerType.GetData());

                m_PrevSecTrigPulseShort = m_SecTrigPulse2Short;
                m_SecTrigPulse2Short = m_sampleIdx;
                m_PrevSecTrigPeriodSampleCountShort= m_SecTrigPeriodSampleCountShort;//last period length
                m_SecTrigPeriodSampleCountShort = m_SecTrigPulse2Short - m_PrevSecTrigPulseShort;


                if (m_SecTrigPeriodSampleCountShort > (m_PrevSecTrigPeriodSampleCountShort*5) ){
                    //longpulse
                    m_ShortGapCount = 0;
                }else{
                    //shortpulse
                    m_ShortGapCount++;
                    if ( m_ShortGapCount == 3 ) {
                        // becouse there is 2 long period and the length of the second equal as the first,
                        // so it not more than 5*first, it looks like short

                        //Logger::logf(LOG_MSG, _T(" m_ShortGapCount == 3 "));
                        m_PrevSecTrigPulse = m_SecTrigPulse2;//t1
                        m_SecTrigPulse2 = m_SecTrigPulse2Short; //m_sampleIdx
                        m_PrevSecTrigPeriodSampleCount= m_SecTrigPeriodSampleCount;//rpm1
                        m_SecTrigPeriodSampleCount = m_SecTrigPulse2 - m_PrevSecTrigPulse;//rpm2
                        m_rpm_constant =  (720 << 12) / m_SecTrigPeriodSampleCount; //change m_spakrphase
                        setSymbolsAtPrimeTrigger();
                        m_mgrBase->notifyViewers();
                    }

                }
                //Logger::logf(LOG_MSG, _T(" m_ShortGapCount = %d ,m_SecTrigPeriodSampleCountShort=%d "),m_ShortGapCount,m_SecTrigPeriodSampleCountShort);
            }else if (m_triggerType==LANCIA_COSWORTH){

                m_PrevSecTrigPulseShort = m_SecTrigPulse2Short;
                m_SecTrigPulse2Short = m_sampleIdx;
                m_PrevSecTrigPeriodSampleCountShort= m_SecTrigPeriodSampleCountShort;//last period length
                m_SecTrigPeriodSampleCountShort = m_SecTrigPulse2Short - m_PrevSecTrigPulseShort;


                if (m_SecTrigPeriodSampleCountShort > (m_PrevSecTrigPeriodSampleCountShort*2) ){
                    //longpulse
                }else{
                    //shortpulse
                    m_PrevSecTrigPulse = m_SecTrigPulse2;//t1
                    m_SecTrigPulse2 = m_SecTrigPulse2Short; //m_sampleIdx
                    m_PrevSecTrigPeriodSampleCount= m_SecTrigPeriodSampleCount;//rpm1
                    m_SecTrigPeriodSampleCount = m_SecTrigPulse2 - m_PrevSecTrigPulse;//rpm2
                    m_rpm_constant =  (720 << 12) / m_SecTrigPeriodSampleCount; //change m_spakrphase
                    setSymbolsAtPrimeTrigger();
                    m_mgrBase->notifyViewers();
                }
            }else{
                m_PrevSecTrigPulse = m_SecTrigPulse2;//t1
                m_SecTrigPulse2 = m_sampleIdx;//t2
                m_PrevSecTrigPeriodSampleCount= m_SecTrigPeriodSampleCount;//rpm1
                m_SecTrigPeriodSampleCount = m_SecTrigPulse2 - m_PrevSecTrigPulse;//rpm2
                m_rpm_constant = (uint32_t)((1 << 24) / m_SecTrigPeriodSampleCount); //change m_spakrphase
            Logger::logf(LOG_MSG, _T("SignalProcessThread::  m_rpm_constans=%05X"),m_rpm_constant);
            Logger::logf(LOG_MSG, _T("SignalProcessThread::  m_SecTrigPeriodSampleCount=%05X"),m_SecTrigPeriodSampleCount);
            Logger::logf(LOG_MSG, _T("SignalProcessThread::  m_SecTrigPulse2=%05X"),m_SecTrigPulse2);
            Logger::logf(LOG_MSG, _T("SignalProcessThread::  m_PrevSecTrigPulse=%05X"),m_PrevSecTrigPulse);

                setSymbolsAtPrimeTrigger();
                m_mgrBase->notifyViewers();
            }
}

void SignalProcessThread::processPulseTrainDataBlock(SIPRPacket pulsePacket)
{
    //Logger::logf(LOG_MSG, _T("SignalProcessThread::processPulseTrainDataBlock ,%02X"),pulsePacket.seq);
    uint8_t data_cnt;
    uint32_t time;
    uint8_t pinstate;

    if ((m_seqPulsetrainOld+1)!=pulsePacket.sequence())
        Logger::logf(LOG_MSG, _T("SignalProcessThread::processPulseTrainseq Fail:%02X"),pulsePacket.sequence());
    m_seqPulsetrainOld =pulsePacket.sequence();

    for(data_cnt=0;data_cnt<pulsePacket.size()/4;data_cnt++){

        time=pulsePacket.data()[data_cnt*4+2];
        time |=(pulsePacket.data()[data_cnt*4+1])<<8;
        time |=(pulsePacket.data()[data_cnt*4])<<16;
        pinstate=pulsePacket.data()[data_cnt*4+3];

        if( time < ( m_sampleIdxold & 0x00ffffff )) m_sampleIdx+=0x1000000;
        Logger::logf(LOG_MSG, _T("processPulseTrainDataBlock(2): time:%08X,m_sampleIdx:%02X"),time,m_sampleIdx);
        Logger::logf(LOG_MSG, _T("processPulseTrainDataBlock(2): m_sampleIdxold:%08X"),m_sampleIdxold);

        m_sampleIdx = m_sampleIdx & 0xff000000;

        m_sampleIdx = m_sampleIdx | time;
        //Logger::logf(LOG_MSG, _T("processPulseTrainDataBlock(2): time:%08X,pinstate:%02X"),m_sampleIdx,pinstate);
 /*       if(((m_pinstateold&0x02)==0)&&((pinstate&0x02)==2)){//pin2 rising edge
            secTrigRisingEdge();
        }
        if(((m_pinstateold&0x02)==2)&&((pinstate&0x02)==0)){//pin2 falling edge
            secTrigFallingEdge();
        }*/
        if(((m_pinstateold&0x08)==0)&&((pinstate&0x08)==8)){//pin2 rising edge
            secTrigRisingEdge();
        }
        if(((m_pinstateold&0x08)==8)&&((pinstate&0x08)==0)){//pin2 falling edge
            secTrigFallingEdge();
        }
        if(((m_pinstateold&0x04)==0)&&((pinstate&0x04)==4)){//pin3 rising edge(ch1)
            dataRisingEdge();

        }
        if(((m_pinstateold&0x04)==4)&&((pinstate&0x04)==0)){//pin3 falling edge
            dataFallingEdge();
        }
        /*if(((m_pinstateold&0x08)==0)&&((pinstate&0x08)==8)){//pin4 rising edge(ch2)
            dataRisingEdge2();

        }
        if(((m_pinstateold&0x08)==8)&&((pinstate&0x08)==0)){//pin4 falling edge
            dataFallingEdge2();
        }
*/
        m_pinstateold=pinstate;


        m_sampleIdxold=m_sampleIdx;


    }

}
