#ifndef WX_PRECOMP
    #include "wx/app.h"
    #include "wx/frame.h"
    #include "wx/menu.h"
    #include "wx/msgdlg.h"
    #include "wx/icon.h"
    #include "wx/textctrl.h"
    #include "wx/filedlg.h"
#endif

#include "wx/sound.h"
#include "wx/numdlg.h"
#include "wx/textdlg.h"
#include <wx/xrc/xmlres.h>

#include "SoundPanel.h"
#include "soundFrame.h"
#include "signalgenwx.h"
#include "signalgen.h"

#include "../logger.h"
#include "../UnicodeHelper.h"
#include "../DirHelper.h"

#include "portaudiocpp/PortAudioCpp.hxx"

using namespace portaudio;

//----------------------------------------------------------------------
// SoundPanel
//----------------------------------------------------------------------

const int  ID_BOOK              = 1000;

const int  ID_COMBO1            = 134;
const int  ID_COMBO2            = 135;
const int  ID_COMBO3            = 136;
const int  ID_COMBO4            = 137;
const int  ID_COMBO5            = 138;
const int  ID_COMBO6            = 139;

const int  ID_COMBO_GENERATE    = 143;
const int  ID_LIVE_PLAY        = 144;
const int  ID_LIVE_STOP        = 145;
const int  ID_LIVE_TEST        = 146;

const int  ID_GAUGE             = 180;
const int  ID_RPM_SLIDER        = 181;
const int  ID_VOLUME_SLIDER     = 183;
const int  ID_RPMMAXCLIMBSLIDER = 184;
const int  ID_SPIN              = 182;
const int  ID_SPINCTRL          = 185;
const int  ID_MIN_SPINCTRL          = 186;
const int  ID_MAX_SPINCTRL          = 187;


BEGIN_EVENT_TABLE(SoundPanel, wxPanel)
//EVT_IDLE      (                         SoundPanel::OnIdle)
EVT_BOOKCTRL_PAGE_CHANGING(ID_BOOK,     SoundPanel::OnPageChanging)
EVT_BOOKCTRL_PAGE_CHANGED(ID_BOOK,      SoundPanel::OnPageChanged)

EVT_COMBOBOX  (ID_COMBO1,                SoundPanel::OnCombo)
//EVT_TEXT      (ID_COMBO1,                SoundPanel::OnComboTextChanged)
//EVT_TEXT_ENTER(ID_COMBO1,                SoundPanel::OnComboTextEnter)
EVT_COMBOBOX  (ID_COMBO2,                SoundPanel::OnCombo)
//EVT_TEXT      (ID_COMBO2,                SoundPanel::OnComboTextChanged)
//EVT_TEXT_ENTER(ID_COMBO2,                SoundPanel::OnComboTextEnter)
EVT_COMBOBOX  (ID_COMBO3,                SoundPanel::OnCombo)
//EVT_TEXT      (ID_COMBO3,                SoundPanel::OnComboTextChanged)
//EVT_TEXT_ENTER(ID_COMBO3,                SoundPanel::OnComboTextEnter)
EVT_COMBOBOX  (ID_COMBO4,                SoundPanel::OnCombo)
//EVT_TEXT      (ID_COMBO4,                SoundPanel::OnComboTextChanged)
//EVT_TEXT_ENTER(ID_COMBO4,                SoundPanel::OnComboTextEnter)
EVT_COMBOBOX  (ID_COMBO5,                SoundPanel::OnCombo)
//EVT_TEXT      (ID_COMBO5,                SoundPanel::OnComboTextChanged)
//EVT_TEXT_ENTER(ID_COMBO5,                SoundPanel::OnComboTextEnter)
EVT_COMBOBOX  (ID_COMBO6,                SoundPanel::OnCombo)
//EVT_TEXT      (ID_COMBO6,                SoundPanel::OnComboTextChanged)
//EVT_TEXT_ENTER(ID_COMBO6,                SoundPanel::OnComboTextEnter)
EVT_BUTTON    (ID_LIVE_PLAY,           SoundPanel::OnComboButtons)
EVT_BUTTON    (ID_LIVE_STOP,           SoundPanel::OnComboButtons)
EVT_BUTTON    (ID_LIVE_TEST,           SoundPanel::OnComboButtons)
EVT_BUTTON    (ID_COMBO_GENERATE,      SoundPanel::OnComboButtons)

#if wxUSE_SLIDER
EVT_SLIDER   (ID_RPM_SLIDER,           SoundPanel::OnSliderUpdate)
EVT_SLIDER    (ID_VOLUME_SLIDER,        SoundPanel::OnSliderUpdate)
EVT_SLIDER    (ID_RPMMAXCLIMBSLIDER,        SoundPanel::OnSliderUpdate)
#endif // wxUSE_SLIDER

#if wxUSE_SPINCTRL
EVT_SPINCTRL  (ID_SPINCTRL,             SoundPanel::OnSpinCtrl)
//EVT_SPIN_UP   (ID_SPINCTRL,             SoundPanel::OnSpinCtrlUp)
//EVT_SPIN_DOWN (ID_SPINCTRL,             SoundPanel::OnSpinCtrlDown)
EVT_TEXT      (ID_SPINCTRL,             SoundPanel::OnSpinCtrlText)
EVT_SPINCTRL  (ID_MIN_SPINCTRL,             SoundPanel::OnSpinCtrl)
//EVT_SPIN_UP   (ID_MIN_SPINCTRL,             SoundPanel::OnSpinCtrlUp)
//EVT_SPIN_DOWN (ID_MIN_SPINCTRL,             SoundPanel::OnSpinCtrlDown)
//EVT_TEXT      (ID_MIN_SPINCTRL,             SoundPanel::OnSpinCtrlText)
EVT_SPINCTRL  (ID_MAX_SPINCTRL,             SoundPanel::OnSpinCtrl)
//EVT_SPIN_UP   (ID_MAX_SPINCTRL,             SoundPanel::OnSpinCtrlUp)
//EVT_SPIN_DOWN (ID_MAX_SPINCTRL,             SoundPanel::OnSpinCtrlDown)
//EVT_TEXT      (ID_MAX_SPINCTRL,             SoundPanel::OnSpinCtrlText)
#endif // wxUSE_SPINCTRL

EVT_MOUSEWHEEL(SoundPanel::OnMouseWheel)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MyButton, wxButton)
    //EVT_LEFT_DCLICK(MyButton::OnDClick)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MyComboBox, wxComboBox)
/*    EVT_CHAR(MyComboBox::OnChar)
    EVT_KEY_DOWN(MyComboBox::OnKeyDown)
    EVT_KEY_UP(MyComboBox::OnKeyUp)

    EVT_SET_FOCUS(MyComboBox::OnFocusGot)*/
END_EVENT_TABLE()

// ============================================================================
// implementation
// ============================================================================

SoundPanel::SoundPanel( wxFrame *frame, int x, int y, int w, int h )
       : wxPanel( frame, wxID_ANY, wxPoint(x, y), wxSize(w, h) )
{

    m_combo1 = NULL;
    m_combo2 = NULL;
    m_combo3 = NULL;
    m_combo4 = NULL;
    m_combo5 = NULL;
    m_combo6 = NULL;

    m_gauge = NULL;
    m_gaugeVert = NULL;
    m_slider = NULL;
    m_volume_slider = NULL;
    m_spinbutton = NULL;
    m_deltaMaxClimb_slider= NULL;

    m_spinctrl = NULL;
    m_rpm_max_spinctrl = NULL;
    m_rpm_min_spinctrl = NULL;

    m_book = NULL;

    //m_text = new wxTextCtrl(this, wxID_ANY, _T("This is the log window.\n"),
    //                        wxPoint(0, 250), wxSize(100, 50), wxTE_MULTILINE);

    //m_logTargetOld = wxLog::SetActiveTarget(new wxLogTextCtrl(m_text));

    m_book = new wxBookCtrl(this, ID_BOOK);

    // image ids
    enum
    {
        Image_Combo,
        Image_Gauge,
        Image_Max
    };

    // fill the image list
    wxBitmap bmp(DirHelper::GetPath(dICON) + _T("combo.png"),wxBITMAP_TYPE_PNG);
    wxImageList *imagelist = new wxImageList(bmp.GetWidth(), bmp.GetHeight());
    imagelist-> Add( bmp );
    imagelist-> Add( wxBitmap( DirHelper::GetPath(dICON) + _T("gauge.png"),wxBITMAP_TYPE_PNG ));

    m_book->SetImageList(imagelist);

    //SIGNALGEN TAB
    wxPanel *panel = new wxPanel(m_book);

    wxString types[] =
    {
        _T("c004"),
        _T("c008"),
        _T("c270"),
        _T("m582"),
        _T("m351"),
        _T("f000"),
        _T("e324"),
        _T("s060"),
        _T("l008")
    };

    wxString empty[] =
    {
        _T("")
    };

    /*(void)new wxStaticBox( panel, wxID_ANY, _T("&Signalgen parameters"),
                           wxPoint(5, 5), wxSize(275, 200));*/
    wxBoxSizer *combo_sizer = new wxBoxSizer( wxHORIZONTAL );
    panel->SetSizer( combo_sizer );

    wxStaticBoxSizer *param_sizer = new wxStaticBoxSizer( wxVERTICAL, panel, _T("&Signalgen parameters") );
    combo_sizer->Add(param_sizer);
    m_combo1 = new MyComboBox( panel, ID_COMBO1, _T("c004"),
                              wxDefaultPosition, wxSize(120, wxDefaultCoord),
                              8, types,
                              wxTE_PROCESS_ENTER);
    m_combo1->SetSelection(0);
    wxStaticText *m_combo_text1 = new wxStaticText( panel, wxID_ANY,
                            _T("Trigger type"),
                            wxDefaultPosition,
                            wxSize(150, wxDefaultCoord)
                          );

    m_combo2 = new MyComboBox( panel, ID_COMBO2, _T("180-1740"),
                              wxDefaultPosition, wxSize(120, wxDefaultCoord),
                              1, empty,
                              wxTE_PROCESS_ENTER);
    m_combo2->Delete(0);
    m_combo2->Append(_T("180-1740"));
    m_combo2->SetSelection(0);
    wxStaticText *m_combo_text2 = new wxStaticText( panel, wxID_ANY,
                            _T("RPM"),
                            wxDefaultPosition,
                            wxSize(150, wxDefaultCoord)
                          );
    m_combo3 = new MyComboBox( panel, ID_COMBO3, _T("1"),
                              wxDefaultPosition, wxSize(120, wxDefaultCoord),
                              1, empty,
                              wxTE_PROCESS_ENTER);
    m_combo3->Delete(0);
    m_combo3->Append(_T("1"));
    m_combo3->SetSelection(0);
    wxStaticText *m_combo_text3 = new wxStaticText( panel, wxID_ANY,
                            _T("Seconds"),
                            wxDefaultPosition,
                            wxSize(150, wxDefaultCoord)
                          );
    m_combo4 = new MyComboBox( panel, ID_COMBO4, _T("15-30"),
                              wxDefaultPosition, wxSize(120, wxDefaultCoord),
                              1, empty,
                              wxTE_PROCESS_ENTER);
    m_combo4->Delete(0);
    m_combo4->Append(_T("15-30"));
    m_combo4->SetSelection(0);
    wxStaticText *m_combo_text4 = new wxStaticText( panel, wxID_ANY,
                            _T("Second Trigger position"),
                            wxDefaultPosition,
                            wxSize(150, wxDefaultCoord)
                          );
    m_combo5 = new MyComboBox( panel, ID_COMBO5, _T("100-1000"),
                              wxDefaultPosition, wxSize(120, wxDefaultCoord),
                              1, empty,
                              wxTE_PROCESS_ENTER);
    m_combo5->Delete(0);
    m_combo5->Append(_T("100-1000"));
    m_combo5->SetSelection(0);
    wxStaticText *m_combo_text5 = new wxStaticText( panel, wxID_ANY,
                            _T("Amplitude"),
                            wxDefaultPosition,
                            wxSize(150, wxDefaultCoord)
                          );
    m_combo6 = new MyComboBox( panel, ID_COMBO6, _T("1000-900"),
                              wxDefaultPosition, wxSize(120, wxDefaultCoord),
                              1, empty,
                              wxTE_PROCESS_ENTER);
    m_combo6->Delete(0);
    m_combo6->Append(_T("1000-900"));
    m_combo6->SetSelection(0);
    wxStaticText *m_combo_text6 = new wxStaticText( panel, wxID_ANY,
                            _T("normal_tooth_relative_amplitude"),
                            wxDefaultPosition,
                            wxSize(150, wxDefaultCoord)
                          );

    param_sizer->Add(m_combo_text1, 0, wxLEFT, 10);
    param_sizer->Add(m_combo1, 0, wxLEFT, 10);
    param_sizer->Add(m_combo_text2, 0, wxLEFT, 10);
    param_sizer->Add(m_combo2, 0, wxLEFT, 10);
    param_sizer->Add(m_combo_text3, 0, wxLEFT, 10);
    param_sizer->Add(m_combo3, 0, wxLEFT, 10);
    param_sizer->Add(m_combo_text4, 0, wxLEFT, 10);
    param_sizer->Add(m_combo4, 0, wxLEFT, 10);
    param_sizer->Add(m_combo_text5, 0, wxLEFT, 10);
    param_sizer->Add(m_combo5, 0, wxLEFT, 10);
    param_sizer->Add(m_combo_text6, 0, wxLEFT, 10);
    param_sizer->Add(m_combo6, 0, wxLEFT, 10);
    (void)new wxButton( panel, ID_COMBO_GENERATE, _T("&Generate File"), wxPoint(300,25), wxSize(140,30) );
    m_book->AddPage(panel, _T("SignalGen"), false, Image_Combo);

    panel = new wxPanel(m_book);

    //LIVE TAB
    //wxBoxSizer *main_sizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer *main_sizer = new wxBoxSizer( wxVERTICAL );
    panel->SetSizer( main_sizer );

    //parameters box
    wxStaticBoxSizer* paramSizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("&Signalgen parameters") );

    wxStaticText* textTriggerType = new wxStaticText( panel, wxID_ANY, _T("Trigger type") );

    m_comboTriggerType = new MyComboBox( panel, ID_COMBO1, _T("c004"),
                                        wxDefaultPosition, wxDefaultSize,
                                        8, types, wxTE_PROCESS_ENTER );

    m_comboTriggerType->SetSelection( 0 );
    paramSizer->Add( textTriggerType, 0, wxALL, 5 );
    paramSizer->Add( m_comboTriggerType, 0, wxALL, 5 );
    main_sizer->Add( paramSizer, 0, wxALL | wxEXPAND, 5 );
//RPM SIZER
    wxStaticBoxSizer *gauge_sizer = new wxStaticBoxSizer( wxVERTICAL, panel, _T("&RPM") );
    main_sizer->Add( gauge_sizer, 0, wxALL | wxEXPAND, 5 );
    //wxBoxSizer *sz = new wxBoxSizer( wxVERTICAL );
    //wxBoxSizer *xy = new wxBoxSizer( wxHORIZONTAL );
    //gauge_sizer->Add( sz );
    //m_gauge = new wxGauge( panel, wxID_ANY, 12000, wxDefaultPosition, wxSize(155, 30), wxGA_HORIZONTAL|wxNO_BORDER );
    //sz->Add( m_gauge, 0, wxALL, 10 );
    m_slider = new wxSlider( panel, ID_RPM_SLIDER, 600, 0, 12000,
                             wxDefaultPosition, wxDefaultSize,
                             //wxSL_AUTOTICKS | wxSL_LABELS);
                             wxSL_LABELS);
    m_slider->SetTickFreq(40, 0);
    m_slider->SetToolTip(_T("This is a RPM slider"));

    gauge_sizer->Add( m_slider, 1, wxEXPAND );

    wxBoxSizer* rmpSettingsSizer = new wxBoxSizer( wxHORIZONTAL );
    //sz->Add( m_slider, 0, wxALL, 10 );
    //sz->Add(xy,0, wxALL, 10);
    wxBoxSizer* rpm_min = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer* rpm_max = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer* rpmChangeRate = new wxBoxSizer( wxVERTICAL );
    //xy->Add(rpm_min,0, wxALL, 0);
    //xy->Add(rpm_max,0, wxALL, 0);
    /*m_gaugeVert = new wxGauge( panel, wxID_ANY, 6000,
                               wxDefaultPosition, wxSize(wxDefaultCoord, 90),
                               wxGA_VERTICAL | wxGA_SMOOTH | wxNO_BORDER );
    gauge_sizer->Add( m_gaugeVert, 0, wxALL, 10 );*/

//    m_rpm_min_spinctrl = new wxSpinCtrl( panel, ID_MIN_SPINCTRL, wxEmptyString, wxPoint(120, 160), wxSize(80, wxDefaultCoord) );
    wxStaticText *rpm_min_text = new wxStaticText( panel, wxID_ANY,
                            _T("Minium RPM"),
                            wxDefaultPosition,
                            wxSize(80, wxDefaultCoord)
                          );
    m_rpm_min_spinctrl = new wxSpinCtrl( panel, ID_MIN_SPINCTRL, wxEmptyString, wxDefaultPosition, wxSize(80, wxDefaultCoord) );
    m_rpm_min_spinctrl->SetRange(0,12000);
    m_rpm_min_spinctrl->SetValue(0);
    m_rpm_min_spinctrl->SetToolTip(_T("Select min_rpm for slider"));
    rpm_min->Add( rpm_min_text, 0, wxALL, 5 );
    rpm_min->Add( m_rpm_min_spinctrl, 0, wxALL, 5 );

    rmpSettingsSizer->Add( rpm_min );

//    m_rpm_max_spinctrl = new wxSpinCtrl( panel, ID_MAX_SPINCTRL, wxEmptyString, wxPoint(280, 160), wxSize(80, wxDefaultCoord) );
    wxStaticText *rpm_max_text = new wxStaticText( panel, wxID_ANY,
                            _T("Maximum RPM"),
                            wxDefaultPosition,
                            wxSize(80, wxDefaultCoord)
                          );
    m_rpm_max_spinctrl = new wxSpinCtrl( panel, ID_MAX_SPINCTRL, wxEmptyString, wxDefaultPosition, wxSize(80, wxDefaultCoord) );
    m_rpm_max_spinctrl->SetRange(0,12000);
    m_rpm_max_spinctrl->SetValue(12000);
    m_rpm_max_spinctrl->SetToolTip(_T("Select max_rpm for slider"));
    rpm_max->Add( rpm_max_text, 0, wxALL, 5 );
    rpm_max->Add( m_rpm_max_spinctrl, 0, wxALL, 5 );

    rmpSettingsSizer->Add( rpm_max );

    wxStaticText *rpm_rate_text = new wxStaticText( panel, wxID_ANY,
                            _T("Change rate"),
                            wxDefaultPosition,
                            wxSize(80, wxDefaultCoord)
                          );
    m_rpm_rate_spinctrl = new wxSpinCtrl( panel, wxID_ANY, wxEmptyString,
                            wxDefaultPosition, wxSize(80, wxDefaultCoord) );
    m_rpm_rate_spinctrl->SetRange(1,100);
    m_rpm_rate_spinctrl->SetValue(10);
    m_rpm_rate_spinctrl->SetToolTip(_T("Select rpm rate for slider"));
    rpmChangeRate->Add( rpm_rate_text, 0, wxALL, 5 );
    rpmChangeRate->Add( m_rpm_rate_spinctrl, 0, wxALL, 5 );

    rmpSettingsSizer->Add( rpmChangeRate );

    gauge_sizer->Add( rmpSettingsSizer );
    //sz->Add(rpm_rate_text, 0, wxLEFT, 10 );
    //sz->Add(m_rpm_rate_spinctrl, 0, wxLEFT, 10 );



//RPM SIZER

//VOLUME SIZER
    //wxBoxSizer *vol_ctrl = new wxBoxSizer( wxVERTICAL );
    //wxBoxSizer *vol_ctrl = new wxBoxSizer( wxHORIZONTAL );
    //main_sizer->Add(vol_ctrl, 0, wxALL | wxEXPAND, 5);
    wxStaticBoxSizer *volume_sizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("&Volume") );
    //vol_ctrl->Add( volume_sizer, 0, wxALL, 0 );
    main_sizer->Add( volume_sizer, 0, wxALL | wxEXPAND, 5 );
    //wxBoxSizer *szz = new wxBoxSizer( wxVERTICAL );
    //volume_sizer->Add( szz );
/*
    m_volume_slider = new wxSlider( panel, ID_VOLUME_SLIDER, 500, 0, 1000,
                             wxDefaultPosition, wxSize(155,wxDefaultCoord),
                             wxSL_AUTOTICKS | wxSL_LABELS);
*/
    m_volume_slider = new wxSlider( panel, ID_VOLUME_SLIDER, 500, 0, 1000 );

    m_volume_slider->SetTickFreq(40, 0);
    m_volume_slider->SetToolTip(_T("This is a volume slider"));
    volume_sizer->Add( m_volume_slider, 1 );

    // PLAY STOP BUTTONS
    wxBoxSizer* controlSizer = new wxBoxSizer( wxHORIZONTAL );

    //wxButton *live_play = new wxButton( panel, ID_LIVE_PLAY, _T("&Play"), wxDefaultPosition, wxSize(140,wxDefaultCoord) );
    //wxButton *live_stop = new wxButton( panel, ID_LIVE_STOP, _T("&Stop"), wxDefaultPosition, wxSize(140,wxDefaultCoord) );
    //wxButton *live_test = new wxButton( panel, ID_LIVE_TEST, _T("&TEST"), wxDefaultPosition, wxSize(140,wxDefaultCoord) );
    wxButton* live_play = new wxButton( panel, ID_LIVE_PLAY, _T("&Play") );
    wxButton* live_stop = new wxButton( panel, ID_LIVE_STOP, _T("&Stop") );


    controlSizer->Add(live_play, 0, wxALL, 5);
    controlSizer->Add(live_stop, 0, wxALL, 5);
    //controlSizer->Add(live_test, 0, wxALL, 5);

    main_sizer->Add( controlSizer, 0, wxEXPAND );
//VOLUME SIZER


/*
    int initialSpinValue = 100;
    wxString s;
    s << initialSpinValue;
    m_spintext = new wxTextCtrl( panel, wxID_ANY, s, wxPoint(20,160), wxSize(80,wxDefaultCoord) );
    m_spinbutton = new wxSpinButton( panel, ID_SPIN, wxPoint(103,160) );
    m_spinbutton->SetRange(-40,30);
    m_spinbutton->SetValue(initialSpinValue);

    m_spinctrl = new wxSpinCtrl( panel, ID_SPINCTRL, wxEmptyString, wxPoint(200, 160), wxSize(80, wxDefaultCoord) );
    m_spinctrl->SetRange(-10,30);
    m_spinctrl->SetValue(15);
*/



    m_book->AddPage(panel, _T("Live Playback"), false, Image_Gauge);

    //audio device test
    wxPanel* audioDevicePanel = NULL; audioDevicePanel = new wxPanel( m_book );

    wxComboBox* outputDeviceCombo = NULL;
    outputDeviceCombo = new wxComboBox( audioDevicePanel, wxID_ANY, wxEmptyString, wxDefaultPosition,
                                                    wxDefaultSize, 0, NULL, wxCB_READONLY, wxDefaultValidator, _T("comboBox") );

    //get audio devices
    wxArrayString inputDevices;
    wxArrayString outputDevices;
    int defaultInputDevice = -1;
    int defaultOutputDevice = -1;

    try
    {
        portaudio::AutoSystem autoSys;
        System& paSystem = System::instance();
        int deviceCount = paSystem.deviceCount();

        for( int i = 0; i< deviceCount; i++ )
        {
            Device& ad = paSystem.deviceByIndex( i );
            const char* name = ad.name();
            wxString nameStr = wxString::FromAscii( name ) + wxString::Format( _T(" id : %d"), i );

            if( ad.isInputOnlyDevice() )
            {
                inputDevices.Add( nameStr );
            }
            else if( ad.isOutputOnlyDevice() )
            {
                //outputDevices.Add( nameStr );
                outputDeviceCombo->Append( nameStr, new int( i ) );
            }

            if( ad.isSystemDefaultInputDevice() )
            {
                defaultInputDevice = i;
            }
            else if( ad.isSystemDefaultOutputDevice() )
            {
                defaultOutputDevice = i;
            }
            //wxString msg = wxString::Format(_T("Audio device: %i, name: ") + nameStr, i);
            //Logger::log(LOG_MSG, msg);
        }
    }catch( Exception& e ){
        //const char* str = e.what();
        //serialErrorMsgDialog((const char *) str);
        //return;
    }

    //Audio device settings panel
    //wxPanel* audioDevicePanel = new wxPanel( m_book );
    wxBoxSizer* audioDeviceSizer = new wxBoxSizer( wxVERTICAL );

    wxStaticBoxSizer* playbackSizer = new wxStaticBoxSizer( wxHORIZONTAL, audioDevicePanel, _T("&Playback") );
    wxStaticText* outputDeviceText = new wxStaticText( audioDevicePanel, wxID_ANY, _T("Device :") );
    //wxComboBox* outputDeviceCombo = new wxComboBox( audioDevicePanel, wxID_ANY );
    //wxComboBox* outputDeviceCombo = new wxComboBox( audioDevicePanel, wxID_ANY, wxEmptyString, wxDefaultPosition,
    //                                                wxDefaultSize, outputDevices, wxCB_READONLY );
    outputDeviceCombo->SetSelection( 0 );
    playbackSizer->Add( outputDeviceText, 0, wxALL, 5 );
    playbackSizer->Add( outputDeviceCombo, 1, wxALL, 5);
    audioDeviceSizer->Add( playbackSizer, 0, wxEXPAND | wxALL, 5 );

    wxStaticBoxSizer* recordingSizer = new wxStaticBoxSizer( wxVERTICAL, audioDevicePanel, _T("&Recording") );
    wxBoxSizer* inputDeviceSizer = new wxBoxSizer( wxHORIZONTAL );
    wxStaticText* inputDeviceText = new wxStaticText( audioDevicePanel, wxID_ANY, _T("Device :") );
    //wxComboBox* inputDeviceCombo = new wxComboBox( audioDevicePanel, wxID_ANY );
    wxComboBox* inputDeviceCombo = new wxComboBox(  audioDevicePanel, wxID_ANY, wxEmptyString, wxDefaultPosition,
                                                    wxDefaultSize, inputDevices, wxCB_READONLY );
    inputDeviceCombo->SetSelection( 0 );
    inputDeviceSizer->Add( inputDeviceText, 0, wxALL, 5 );
    inputDeviceSizer->Add( inputDeviceCombo, 1, wxALL, 5 );
    recordingSizer->Add( inputDeviceSizer, 0, wxEXPAND );
    audioDeviceSizer->Add( recordingSizer, 0, wxEXPAND | wxALL, 5 );

    audioDevicePanel->SetSizer( audioDeviceSizer );
    m_book->AddPage( audioDevicePanel, _T("Audio I/O") );


    wxBoxSizer *sizer = new wxBoxSizer( wxVERTICAL );

    // set the sizer for the panel itself
    sizer = new wxBoxSizer(wxVERTICAL);
    //sizer->Add(m_book, wxSizerFlags().Border().Expand());
    //sizer->Add(m_text, wxSizerFlags(1).Border().Expand());
    sizer->Add( m_book, 1, wxEXPAND );
    SetSizer(sizer);
}

SoundPanel::~SoundPanel()
{
    //wxLog::RemoveTraceMask(_T("focus"));
    //delete wxLog::SetActiveTarget(m_logTargetOld);
    delete m_book->GetImageList();
//    if(m_sound)
//        delete m_sound;
}


void SoundPanel::OnPageChanging( wxBookCtrlEvent &event )
{
    /*int selOld = event.GetOldSelection(); */
    /*if ( selOld == 2 )
    {
        if ( wxMessageBox(_T("This demonstrates how a program may prevent the\n")
                          _T("page change from taking place - if you select\n")
                          _T("[No] the current page will stay the third one\n"),
                          _T("Control sample"),
                          wxICON_QUESTION | wxYES_NO, this) != wxYES )
        {
            event.Veto();

            return;
        }
    }*/

/*    *m_text << _T("Book selection is being changed from ") << selOld
            << _T(" to ") << event.GetSelection()
            << _T(" (current page from book is ")
            << m_book->GetSelection() << _T(")\n");*/
}

void SoundPanel::OnPageChanged( wxBookCtrlEvent &event )
{
/*    *m_text << _T("Book selection is now ") << event.GetSelection()
            << _T(" (from book: ") << m_book->GetSelection()
            << _T(")\n");*/
}

void SoundPanel::OnCombo( wxCommandEvent &event )
{
    switch(event.GetId())
    {
        case ID_COMBO1:
                if (m_combo1)
                    wxLogMessage(_T("EVT_COMBOBOX: item %d/%d (event/control), string \"%s\"/\"%s\""),
                             (int)event.GetInt(),
                             m_combo1->GetSelection(),
                             event.GetString().c_str(),
                             m_combo1->GetStringSelection().c_str());
                break;
        case ID_COMBO2:
                if (m_combo2)
                    wxLogMessage(_T("EVT_COMBOBOX: item %d/%d (event/control), string \"%s\"/\"%s\""),
                             (int)event.GetInt(),
                             m_combo2->GetSelection(),
                             event.GetString().c_str(),
                             m_combo2->GetStringSelection().c_str());
                break;
        case ID_COMBO3:
                if (m_combo3)
                    wxLogMessage(_T("EVT_COMBOBOX: item %d/%d (event/control), string \"%s\"/\"%s\""),
                             (int)event.GetInt(),
                             m_combo3->GetSelection(),
                             event.GetString().c_str(),
                             m_combo3->GetStringSelection().c_str());
                break;
        case ID_COMBO4:
                if (m_combo4)
                    wxLogMessage(_T("EVT_COMBOBOX: item %d/%d (event/control), string \"%s\"/\"%s\""),
                             (int)event.GetInt(),
                             m_combo4->GetSelection(),
                             event.GetString().c_str(),
                             m_combo4->GetStringSelection().c_str());
                break;
        case ID_COMBO5:
                if (m_combo5)
                    wxLogMessage(_T("EVT_COMBOBOX: item %d/%d (event/control), string \"%s\"/\"%s\""),
                             (int)event.GetInt(),
                             m_combo5->GetSelection(),
                             event.GetString().c_str(),
                             m_combo5->GetStringSelection().c_str());
                break;
        case ID_COMBO6:
                if (m_combo6)
                    wxLogMessage(_T("EVT_COMBOBOX: item %d/%d (event/control), string \"%s\"/\"%s\""),
                             (int)event.GetInt(),
                             m_combo6->GetSelection(),
                             event.GetString().c_str(),
                             m_combo6->GetStringSelection().c_str());
                break;
    }
}

void SoundPanel::OnComboTextChanged(wxCommandEvent& event)
{
    switch(event.GetId())
    {
        case ID_COMBO1:
                if (m_combo1)
                wxLogMessage(wxT("EVT_TEXT for the combobox: \"%s\" (event) or \"%s\" (control)."),
                     event.GetString().c_str(),
                     m_combo1->GetValue().c_str());
                break;
        case ID_COMBO2:
                if (m_combo2)
                wxLogMessage(wxT("EVT_TEXT for the combobox: \"%s\" (event) or \"%s\" (control)."),
                     event.GetString().c_str(),
                     m_combo2->GetValue().c_str());
                break;
        case ID_COMBO3:
                if (m_combo3)
                wxLogMessage(wxT("EVT_TEXT for the combobox: \"%s\" (event) or \"%s\" (control)."),
                     event.GetString().c_str(),
                     m_combo3->GetValue().c_str());
                break;
        case ID_COMBO4:
                if (m_combo4)
                wxLogMessage(wxT("EVT_TEXT for the combobox: \"%s\" (event) or \"%s\" (control)."),
                     event.GetString().c_str(),
                     m_combo4->GetValue().c_str());
                break;
        case ID_COMBO5:
                if (m_combo5)
                wxLogMessage(wxT("EVT_TEXT for the combobox: \"%s\" (event) or \"%s\" (control)."),
                     event.GetString().c_str(),
                     m_combo5->GetValue().c_str());
                break;
        case ID_COMBO6:
                if (m_combo6)
                wxLogMessage(wxT("EVT_TEXT for the combobox: \"%s\" (event) or \"%s\" (control)."),
                     event.GetString().c_str(),
                     m_combo6->GetValue().c_str());
                break;
    }
}

void SoundPanel::OnComboTextEnter(wxCommandEvent& event)
{
    switch(event.GetId())
    {
        case ID_COMBO1:
                if (m_combo1)
                wxLogMessage(_T("Enter pressed in the combobox: value is '%s'."),
                     m_combo1->GetValue().c_str());;
                break;
        case ID_COMBO2:
                if (m_combo2)
                wxLogMessage(_T("Enter pressed in the combobox: value is '%s'."),
                     m_combo2->GetValue().c_str());;
                break;
        case ID_COMBO3:
                if (m_combo3)
                wxLogMessage(_T("Enter pressed in the combobox: value is '%s'."),
                     m_combo3->GetValue().c_str());;
                break;
        case ID_COMBO4:
                if (m_combo4)
                wxLogMessage(_T("Enter pressed in the combobox: value is '%s'."),
                     m_combo4->GetValue().c_str());;
                break;
        case ID_COMBO5:
                if (m_combo5)
                wxLogMessage(_T("Enter pressed in the combobox: value is '%s'."),
                     m_combo5->GetValue().c_str());;
                break;
        case ID_COMBO6:
                if (m_combo6)
                wxLogMessage(_T("Enter pressed in the combobox: value is '%s'."),
                     m_combo6->GetValue().c_str());;
                break;
    }
}

void SoundPanel::OnComboButtons( wxCommandEvent &event )
{
    switch (event.GetId())
    {
        case ID_COMBO_GENERATE:
            {
                wxArrayString array;
                wxString temp = _T("");
                wxString msg;
                wxString baseDir;
                baseDir = wxFileName::GetCwd();


                array.Add(_T("signalgen"));
                temp = m_combo1->GetValue();
                if(!temp.IsEmpty())
                {
                    array.Add(temp);
                    temp = m_combo2->GetValue();
                    if(!temp.IsEmpty())
                    {
                        array.Add(temp);
                        temp = m_combo3->GetValue();
                        if(!temp.IsEmpty())
                        {
                            array.Add(temp);
                            temp = m_combo4->GetValue();
                            if(!temp.IsEmpty())
                            {
                                array.Add(temp);
                                temp = m_combo5->GetValue();
                                if(!temp.IsEmpty())
                                {
                                    array.Add(temp);
                                    temp = m_combo6->GetValue();
                                    if(!temp.IsEmpty())
                                    {
                                        array.Add(temp);
                                    }
                                }
                            }
                        }
                    }
                }

                if(array.IsEmpty()) // always false "signalgen"
                {
                    Logger::log(LOG_WARNING, _T("empty"));
                    //fprintf (stdout, "empty\n");
                }
                else
                {
                    msg.Printf(_T("String List\n"));

                    //fprintf (stdout, "String List\n");
                    for(uint16_t i=0;i<array.GetCount();i++)
                    {
                        msg.Printf(msg + _T("Array[%d]: "),i);
                        msg.Printf(msg + array.Item(i));
                        msg.Printf(msg + _T("\n"));
                        //fprintf (stdout, "Array[%d]: ",i);
                        //fprintf (stdout, "%s",(const char*)array.Item(i).mb_str());
                        //fprintf (stdout, "\n");
                    }
                    Logger::log(LOG_MSG, msg);
                }

                SignalGenWx obj = SignalGenWx();
                //int ret = obj.DoSignalGen(7, argv);
                //int ret = obj.DoSignalGen(array);
                obj.generateWavFile(array,baseDir);
                array.Clear();
                wxLogMessage(_T("File generated in ")+baseDir);
                break;
            }
        case ID_LIVE_PLAY:
            {
                wxLogMessage(_T("PLAY"));
                wxBusyCursor busy;
                wxString msg;
                long lSize;
                FILE *test;
                wxString filename = _T("test.wav");

                test = fopen(UnicodeHelper::U2C(filename), "rb");
                if(test == NULL) {
                    msg.Printf(_T("cannot open file ") + filename + _T(" for reading, reverting to stdout"));
                    Logger::log(LOG_WARNING, msg);
                    //wavout = stdout;
                } else {
                    msg.Printf(_T("reading from file: ") + filename);
                    Logger::log(LOG_MSG, msg);

                    fseek (test , 0 , SEEK_END);
                    lSize = ftell (test);
                    rewind (test);

                    // allocate memory to contain the whole file.
                    unsigned char *data = (unsigned char*)malloc((unsigned)(lSize*sizeof(unsigned char)));
                    if (data == NULL) return;

                    // copy the file into the buffer.
                    fread (data,1,lSize,test);

                    // Close file
                    fclose(test);
                    if ( !m_sound )
                    {
                        m_sound = new wxSound;
                        m_sound->Create(lSize, data);
                    }

                    if (m_sound->IsOk())
                        m_sound->Play(wxSOUND_ASYNC);

                    if (data != NULL) free (data);
                }
                break;
            }
        case ID_LIVE_STOP:
            {
                wxLogMessage(_T("STOP"));
                wxSound::Stop();
                break;
            }
        case ID_LIVE_TEST:
            {
                wxLogMessage(_T("TEST"));
                GenerateAndPlay();
                break;
            }
    }
}

#if wxUSE_SLIDER

void SoundPanel::OnSliderUpdate( wxCommandEvent &event )
{
    wxString value;
    switch (event.GetId())
    {
        case ID_RPM_SLIDER:
            //m_gauge->SetValue( m_slider->GetValue() );
            //m_gaugeVert->SetValue( m_slider->GetValue() / 2 );
            value.Printf( _T("rpm: %d\n"),
                         m_slider->GetValue());
            wxLogMessage(value);
            break;
        case ID_VOLUME_SLIDER:
            value.Printf( _T("Volume: %d\n"),
                         m_volume_slider->GetValue());
            wxLogMessage(value);
            break;
        case ID_RPMMAXCLIMBSLIDER:
            value.Printf( _T("Rpm max climb: %d\n"),
                         m_deltaMaxClimb_slider->GetValue());
            wxLogMessage(value);
            break;

    }
    GenerateAndPlay();
}

#endif // wxUSE_SLIDER

#if wxUSE_SPINCTRL

void SoundPanel::OnSpinCtrlText(wxCommandEvent& event)
{
   wxString s;
   switch (event.GetId())
    {
        case ID_SPINCTRL:
            {
                if ( m_spinctrl )
                {
                    s.Printf( _T("Spin ctrl text changed: now %d (from event: %s)\n"),
                             m_spinctrl->GetValue(), event.GetString().c_str() );
                    wxLogMessage(s);
                }
                break;
            }
        case ID_MIN_SPINCTRL:
            {
                if ( m_rpm_min_spinctrl )
                {
                    s.Printf( _T("RPM_min spin ctrl text changed: now %d (from event: %s)\n"),
                             m_rpm_min_spinctrl->GetValue(), event.GetString().c_str() );
                    wxLogMessage(s);
                }
                break;
            }
        case ID_MAX_SPINCTRL:
            {
                if ( m_rpm_max_spinctrl )
                {
                    s.Printf( _T("RPM_max spin ctrl text changed: now %d (from event: %s)\n"),
                             m_rpm_max_spinctrl->GetValue(), event.GetString().c_str() );
                    wxLogMessage(s);
                }
                break;
            }
    }
}

void SoundPanel::OnSpinCtrl(wxSpinEvent& event)
{
    wxString s;
    switch (event.GetId())
    {
        case ID_SPINCTRL:
            {
                if ( m_spinctrl )
                {
                    s.Printf( _T("Spin ctrl changed: now %d (from event: %d)\n"),
                             m_spinctrl->GetValue(), event.GetInt() );
                    wxLogMessage(s);
                }
                break;
            }
        case ID_MIN_SPINCTRL:
            {
                if ( m_rpm_min_spinctrl )
                {
                    s.Printf( _T("RPM_min spin ctrl changed: now %d (from event: %d)\n"),
                             m_rpm_min_spinctrl->GetValue(), event.GetInt() );
                    wxLogMessage(s);
                }
                break;
            }
        case ID_MAX_SPINCTRL:
            {
                if ( m_rpm_max_spinctrl )
                {
                    s.Printf( _T("RPM_max spin ctrl changed: now %d (from event: %d)\n"),
                             m_rpm_max_spinctrl->GetValue(), event.GetInt() );
                    wxLogMessage(s);
                }
                break;
            }
    }
    if(m_rpm_min_spinctrl->GetValue()<m_rpm_max_spinctrl->GetValue())
    {
        m_slider->SetRange(m_rpm_min_spinctrl->GetValue(),m_rpm_max_spinctrl->GetValue());
    }
    else
    {
        m_rpm_min_spinctrl->SetValue(m_rpm_max_spinctrl->GetValue());
    }
}

void SoundPanel::OnSpinCtrlUp(wxSpinEvent& event)
{
   wxString s;
   switch (event.GetId())
    {
        case ID_SPINCTRL:
            {
                if ( m_spinctrl )
                {
                    wxLogMessage( wxString::Format(
                        _T("Spin up: %d (from event: %d)\n"),
                        m_spinctrl->GetValue(), event.GetInt() ) );
                }
                break;
            }
        case ID_MIN_SPINCTRL:
            {
                if ( m_rpm_min_spinctrl )
                {
                    wxLogMessage( wxString::Format(
                        _T("RPM_min spin up: %d (from event: %d)\n"),
                        m_rpm_min_spinctrl->GetValue(), event.GetInt() ) );
                }
                break;
            }
        case ID_MAX_SPINCTRL:
            {
                if ( m_rpm_max_spinctrl )
                {
                    wxLogMessage( wxString::Format(
                        _T("RPM_max spin up: %d (from event: %d)\n"),
                        m_rpm_max_spinctrl->GetValue(), event.GetInt() ) );
                }
                break;
            }
    }
}

void SoundPanel::OnSpinCtrlDown(wxSpinEvent& event)
{
   switch (event.GetId())
    {
        case ID_SPINCTRL:
            {
                if ( m_spinctrl )
                {
                    wxLogMessage( wxString::Format(
                        _T("Spin down: %d (from event: %d)\n"),
                        m_spinctrl->GetValue(), event.GetInt() ) );
                }
                break;
            }
        case ID_MIN_SPINCTRL:
            {
                if ( m_rpm_min_spinctrl )
                {
                    wxLogMessage( wxString::Format(
                        _T("RPM_min spin down: %d (from event: %d)\n"),
                        m_rpm_min_spinctrl->GetValue(), event.GetInt() ) );
                }
                break;
            }
        case ID_MAX_SPINCTRL:
            {
                if ( m_rpm_max_spinctrl )
                {
                    wxLogMessage( wxString::Format(
                        _T("RPM_max spin down: %d (from event: %d)\n"),
                        m_rpm_max_spinctrl->GetValue(), event.GetInt() ) );
                }
                break;
            }
    }
}

#endif // wxUSE_SPINCTRL

#if wxUSE_SPINBTN
void SoundPanel::OnSpinUp( wxSpinEvent &event )
{
    wxString value;
    value.Printf( _T("Spin control up: current = %d\n"),
                 m_spinbutton->GetValue());

    if ( event.GetPosition() > 17 )
    {
        value += _T("Preventing the spin button from going above 17.\n");

        event.Veto();
    }

    wxLogMessage(value);
}

void SoundPanel::OnSpinDown( wxSpinEvent &event )
{
    wxString value;
    value.Printf( _T("Spin control down: current = %d\n"),
                 m_spinbutton->GetValue());

    if ( event.GetPosition() < 0 )
    {
        value += _T("Preventing the spin button from going below -17.\n");

        event.Veto();
    }

    wxLogMessage(value);
}

void SoundPanel::OnSpinUpdate( wxSpinEvent &event )
{
    wxString value;
    value.Printf( _T("%d"), event.GetPosition() );
    m_spintext->SetValue( value );

    value.Printf( _T("Spin control range: (%d, %d), current = %d\n"),
                 m_spinbutton->GetMin(), m_spinbutton->GetMax(),
                 m_spinbutton->GetValue());

    wxLogMessage(value);
}

#endif // wxUSE_SPINBTN


void MyComboBox::OnChar(wxKeyEvent& event)
{
    wxLogMessage(_T("MyComboBox::OnChar"));

    if ( event.GetKeyCode() == 'w' )
        wxLogMessage(_T("MyComboBox: 'w' will be ignored."));
    else
        event.Skip();
}

void MyComboBox::OnKeyDown(wxKeyEvent& event)
{
    wxLogMessage(_T("MyComboBox::OnKeyDown"));

    if ( event.GetKeyCode() == 'w' )
        wxLogMessage(_T("MyComboBox: 'w' will be ignored."));
    else
        event.Skip();
}

void MyComboBox::OnKeyUp(wxKeyEvent& event)
{
    wxLogMessage(_T("MyComboBox::OnKeyUp"));

    event.Skip();
}

// this would read rpm and volume slider, generates wavbuffer and plays it
void SoundPanel::GenerateAndPlay()
{
    unsigned int rpm = m_slider->GetValue();
    if(rpm==0) rpm=1;
    unsigned int samples = (unsigned int)((96000.0/rpm)*240.0);
    SignalGen objx = SignalGen();
    char volume_str[6];
    //char rpm_str[6];
    //if(m_sound) delete m_sound;
    m_sound = new wxSound;
    sprintf(volume_str,"%d",m_volume_slider->GetValue());
    //sprintf(rpm_str,"%d",rpm);
    unsigned char *data = (unsigned char*)malloc((unsigned)(samples*4+44)*sizeof(unsigned char));
    if (data == NULL) return;

    // copy the file into the buffer.
    objx.generate_trigger_cycle(data,samples,rpm,UnicodeHelper::U2C(m_combo1->GetValue())
                            ,UnicodeHelper::U2C(m_combo4->GetValue()),volume_str);

    m_sound->Create(samples*4+44, data);
    if (m_sound->IsOk())
    {
        m_sound->Play(wxSOUND_ASYNC | wxSOUND_LOOP);
    }
    if (data) free (data);
}

void SoundPanel::OnMouseWheel(wxMouseEvent& event) {

	bool forwards = (event.GetWheelRotation() > 0);

	if (forwards) {
		m_slider->SetValue(m_slider->GetValue() + m_rpm_rate_spinctrl->GetValue());
	} else {
		m_slider->SetValue(m_slider->GetValue() - m_rpm_rate_spinctrl->GetValue());
	}
	GenerateAndPlay();
}
