#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/app.h"
    #include "wx/frame.h"
    #include "wx/menu.h"
    #include "wx/msgdlg.h"
    #include "wx/icon.h"
    #include "wx/textctrl.h"
    #include "wx/filedlg.h"
#endif

#include "wx/sound.h"
#include "wx/numdlg.h"
#include "wx/textdlg.h"
#include <wx/xrc/xmlres.h>
#include <stdlib.h>

#include "soundFrame.h"
#include "SoundPanel.h"
#include "AudioPanel.h"
#include "../AudioController.h"
#include "../logger.h"

#include "signalgen.h"
#include "signalgenwx.h"

#include "signalgauges/SparkTesterWidgetFactory.h"

// ----------------------------------------------------------------------------
// resources
// ----------------------------------------------------------------------------

// the application icon (under Windows and OS/2 it is in resources)

#define WAV_FILE _T("doggrowl.wav")



// ----------------------------------------------------------------------------
// constants
// ----------------------------------------------------------------------------


//(*InternalHeaders(SoundFrame)
#include <wx/bitmap.h>
#include <wx/font.h>
#include <wx/fontenum.h>
#include <wx/fontmap.h>
#include <wx/image.h>
#include <wx/intl.h>
#include <wx/settings.h>
//*)

// IDs for the controls and the menu commands
//(*IdInit(SoundFrame)
//*)

enum
{
    // menu items
    Sound_SelectFile = wxID_HIGHEST + 1,
#ifdef __WXMSW__
    Sound_SelectResource,
#endif // __WXMSW__
    Sound_SelectMemory,

    Sound_PlaySync,
    Sound_PlayAsync,
    Sound_PlayAsyncOnStack,
    Sound_PlayLoop,
    Sound_Stop,
    Sound_Quit = wxID_EXIT,
    Sound_About = wxID_ABOUT,
    Get_pulseTrain
};

const int idAddGrid = wxNewId();
const int idAdd2DHist = wxNewId();
const int idAddShowspark = wxNewId();
const int idAddGridByTime = wxNewId();
const int idAddVemsGauge= wxNewId();
const int idWriteInFile= wxNewId();

// ----------------------------------------------------------------------------
// event tables and other macros for wxWidgets
// ----------------------------------------------------------------------------

// the event tables connect the wxWidgets events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE(SoundFrame, wxFrame)
	//(*EventTable(SoundFrame)
	//*)

    EVT_MENU(Get_pulseTrain,       SoundFrame::OnGetPulseTrain)


    EVT_MENU(Sound_Quit,             SoundFrame::OnQuit)
    EVT_MENU(Sound_About,            SoundFrame::OnAbout)

    EVT_MENU(idAddGrid, SoundFrame::OnGaugeTest)
    EVT_MENU(idAdd2DHist, SoundFrame::OnGaugeTest)
    EVT_MENU(idAddShowspark, SoundFrame::OnGaugeTest)
    EVT_MENU(idAddGridByTime, SoundFrame::OnGaugeTest)
    EVT_MENU(idAddVemsGauge, SoundFrame::OnGaugeTest)
    EVT_MENU(idWriteInFile, SoundFrame::OnGaugeTest)
END_EVENT_TABLE()

// Create a new application object: this macro will allow wxWidgets to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also implements the accessor function
// wxGetApp() which will return the reference of the right type (i.e. MyApp and
// not wxApp)

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// the application class
// ----------------------------------------------------------------------------


// ----------------------------------------------------------------------------
// main frame
// ----------------------------------------------------------------------------

// frame constructor
SoundFrame::SoundFrame(wxWindow* parent, const wxChar *title, int x, int y)
       : wxFrame(parent, wxID_ANY, title, wxPoint(x, y), wxSize(650, 590), wxDEFAULT_FRAME_STYLE | wxFRAME_FLOAT_ON_PARENT)
       //wxFrame( parent, wxID_ANY, caption, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE | wxFRAME_FLOAT_ON_PARENT )
       ,m_pulse_train(NULL)
{
	wxMenuBar* mbar = new wxMenuBar();
	wxMenu* fileMenu = new wxMenu(_(""));

#if defined (VEMSDEBUG)
    fileMenu->Append(Get_pulseTrain, _T("Get Pulse Train"), _T("Get Pulse Train"));
#endif
	fileMenu->Append(Sound_Quit, _("&Quit\tAlt-F4"), _("Quit the application"));
	mbar->Append(fileMenu, _("&File"));
/*
    wxMenu *playMenu = new wxMenu;
    playMenu->Append(Sound_PlaySync, _T("Play sound &synchronously\tCtrl-S"));
    playMenu->Append(Sound_PlayAsync, _T("Play sound &asynchronously\tCtrl-A"));
    playMenu->Append(Sound_PlayAsyncOnStack, _T("Play sound asynchronously (&object on stack)\tCtrl-K"));
    playMenu->Append(Sound_PlayLoop, _T("&Loop sound\tCtrl-L"));
    playMenu->AppendSeparator();
    playMenu->Append(Sound_Stop, _T("&Stop playing\tCtrl-T"));
    mbar->Append(playMenu, _T("&Play"));
*/
/*
	wxMenu* helpMenu = new wxMenu(_(""));
	helpMenu->Append(Sound_About, _("&About\tF1"), _("Show info about this application"));
	mbar->Append(helpMenu, _("&Help"));
*/
#if defined (VEMSDEBUG)
    wxMenu* gaugeMenu = new wxMenu(_T(""));
    gaugeMenu->Append(idAddGrid, _T("Grid Widget"), _T("Add Grid Widget"));
    gaugeMenu->Append(idAdd2DHist, _T("2D Hist Widget"), _T("Add 2D Hist Widget"));
    gaugeMenu->Append(idAddShowspark, _T("Spark RPM Ploter"), _T("Add Spark Widget"));
    gaugeMenu->Append(idAddGridByTime, _T("Grid By Time"), _T("Add Grid By Time"));
    gaugeMenu->Append(idAddVemsGauge, _T("VemsGauge"), _T("Add VemsGauge"));
    gaugeMenu->Append(idWriteInFile, _T("Write In File"), _T("Start Write In File"));
    mbar->Append(gaugeMenu, _T("&Output Tester Gauges"));
#endif

	SetMenuBar(mbar);

	SetStatusBar(new wxStatusBar(this,-1));

    //m_panel = new SoundPanel( this, 10, 10, 300, 100 );

    AudioPanel* audioPanel = new AudioPanel( this );

    m_sparkTesterWidgetFactory = new SparkTesterWidgetFactory(this,
        audioPanel->getAudioController()->getRTDataMgrBase() );

    Logger::logf(LOG_MSG,_T("SoundFrame::SoundFrame end"));
}

SoundFrame::~SoundFrame()
{

}

void SoundFrame::NotifyUsingFile(const wxString& name)
{
    wxString msg;
    msg << _T("Using sound file: ") << name;
    //m_tc->AppendText(msg);
    Logger::log(LOG_MSG, msg);
    //fprintf(stdout,"%s",(const char*)msg.mb_str());
}


// event handlers
void SoundFrame::OnGetPulseTrain(wxCommandEvent& WXUNUSED(event))
{
   // Logger::log(LOG_MSG, _T("SoundFrame::OnGetPulseTrain"));
   // if (m_pulse_train==NULL) m_pulse_train=new Main_Pulse_train();
}
/*
void SoundFrame::OnSelectFile(wxCommandEvent& WXUNUSED(event))
{
#if wxUSE_FILEDLG
    wxFileDialog dlg(this, _T("Choose a sound file"),
                     wxEmptyString, wxEmptyString,
                     _T("WAV files (*.wav)|*.wav"), wxOPEN|wxCHANGE_DIR);
    if ( dlg.ShowModal() == wxID_OK )
    {
        m_soundFile = dlg.GetPath();
#ifdef __WXMSW__
        m_soundRes.clear();
#endif // __WXMSW__
        m_useMemory = false;

        delete m_sound;
        m_sound = NULL;
        NotifyUsingFile(m_soundFile);
    }
#endif // wxUSE_FILEDLG
}
*/

void SoundFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    // true is to force the frame to close
    Close(true);
}


void SoundFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    wxString msg;
    msg.Printf( _T("This is the About dialog of the sound sample.\n")
                _T("Welcome to %s"), wxVERSION_STRING);

    wxMessageBox(msg, _T("About"), wxOK | wxICON_INFORMATION, this);
}


void SoundFrame::OnGaugeTest(wxCommandEvent& event)
{
    int id = event.GetId();

    if( id == idAdd2DHist) {
        m_sparkTesterWidgetFactory->ShowWidget(SPARKW_HISTOGRAM_2D);
    }else if(id == idAddGrid){
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_GRID);
    }else if( id == idAddShowspark) {
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_PHASE);
    }else if(id == idAddGridByTime){
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_GRID_BY_TIME);
    }else if(id == idAddVemsGauge){
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_GAUGE);
    }else if(id == idWriteInFile){
         m_sparkTesterWidgetFactory->ShowWidget(SPARKW_FILE);
    }
}
