/** \class STDataIOVemsCfg
  *
  * \~english
  * \brief  Save a STDataModel to a file in VemsCfg file format
  * \detail If file is exist overwrite it
  *
  * \~hungarian
  * \brief
  *
  */
#ifndef STDATAIOVEMSCFG_H
#define STDATAIOVEMSCFG_H
#include <wx/wx.h>

class STDataModelBase;
class SymbolArray;
class SymbolTable;
class SymbolMgr;
class Symbol;
class wxFile;
class FirmwareConfigData;

class STDataIOVemsCfg
{
    public:
        STDataIOVemsCfg( STDataModelBase* model )
        {
            m_model = model;
            m_isRawLambdaUsed = false;
        }

        virtual ~STDataIOVemsCfg() {}

        void    saveTo( const wxString& header,
                        const SymbolArray& headerSymbols,
                        SymbolMgr* iniSymbols,
                        const wxString& filename,
                        bool isRawLambdaUsed );


        void    saveTo( FirmwareConfigData* fcData, SymbolMgr* iniSymbols,  bool isRawLambdaUsed );

    private:
        STDataModelBase*    m_model;
        bool                m_isRawLambdaUsed;

        wxString            getTableHeaderLine( const wxString& tableName, const Symbol* headerSymbol );
        //bool                isArrayHeaderSymbol( const Symbol* symbol );
        void                writeScalarSymbolsToFile( wxFile& file, const SymbolArray& headerSymbols );
        void                writeArraySymbolsToFile( wxFile& file, const SymbolArray& orderedConfigTables, const SymbolTable& configTables );
        void                writeArraySymbolToFile( wxFile& file, const Symbol* symbol );
        void                writeArraySymbolHeaderToFile( wxFile& file, const Symbol* symbol, const SymbolTable& configTables );

        void                saveScalarsTo( FirmwareConfigData* fcData, const SymbolTable& headerSymbols );
        void                saveArraysTo( FirmwareConfigData* fcData, const SymbolTable& configTables );
        void                saveArrayTo( FirmwareConfigData* fcData, const Symbol* symbol );
};

#endif // STDATAIOVEMSCFG_H
