#ifndef STDATAMGRBASE_H
#define STDATAMGRBASE_H

#include <wx/event.h>
#include "../common/typedef.h"
#include "../common/types/IntSet.h"
#include "../common/types/StringToIntSetH.h"

WX_DECLARE_STRING_HASH_MAP( wxPoint*, StringToPosition );


class STDataViewerBase;
class STDataViewerBaseSet;
class STDataModelBase;
class VemsCommand;
class Symbol;
class ConfigValidatorBase;


class STDataMgrBase: public wxEvtHandler
{
    public:
        STDataMgrBase();
        virtual ~STDataMgrBase();

        void                setModel( STDataModelBase* model ) { m_stDataModel = model; };
        STDataModelBase*    getModel(){ return  m_stDataModel; }
        void                setDeviceModel( STDataModelBase* model ) { m_deviceModel = model; }
        STDataModelBase*    getDeviceModel() { return  m_deviceModel; }

        bool                wasDataWritten( const Symbol& symbol, size_t row, size_t col );
        bool                wasDataWritten( const Symbol& symbol, size_t offset = 0 );
        bool                wasDataWritten( const wxString& symbolName, size_t offset = 0 );

        int  getRegisteredCount();
        void RegisterViewer( STDataViewerBase* viewer );
        void RegisterTableViewer( STDataViewerBase* viewer );

        void UnRegisterViewer( STDataViewerBase* viewer );
        void UnregisterAll(bool del = false);

        virtual void requestReadPage() = 0;
        virtual void requestBurn() = 0;
        virtual void updateRegisteredViewers() = 0;
        virtual void requestReadAllPage() = 0;
        virtual void requestWriteAllDirty() = 0;

        virtual void requestReadAllDirty() = 0;

        virtual void requestWrite( const Symbol& symbol , size_t row = 0, size_t col = 0 ) = 0;

        void refreshViewerModel(); // copy device model to viewer model, clear data wiating for write or burn

        void    setOnline( bool online = true ) { m_isOnline = online; }
        void    setConnected( bool connected = true ) { m_isConnected = connected; }
        void    setRefreshDeviceModelOnly( bool enable ) { m_isRefreshDeviceModelOnly = enable; }

        bool    isRefreshDeviceModelOnly() const { return m_isRefreshDeviceModelOnly; }
        bool    isOnline() { return m_isOnline; }
        bool    isConnected() { return m_isConnected; }
        bool    isWaitingForWrite() const { return !m_waitingForWrite.empty(); }
        bool    isWaitingForBurn() const { return !m_waitingForBurn.empty(); }

        double  getDataFromViewerModel( const Symbol& symbol, size_t row = 0, size_t col = 0 );
        double  getDataFromDeviceModel( const Symbol& symbol, size_t row = 0, size_t col = 0 );

        virtual void setModelData( const Symbol& symbol, size_t row, size_t col, double data ) = 0;


        /// symbols size must equal to 1
        virtual void setRawModelData( const Symbol& symbol, size_t row, size_t col, uint8_t data ) = 0;
        virtual void updateRawFromDeviceModel( const Symbol& symbol, size_t row, size_t col ) = 0;
        virtual void updateFromDeviceModel( const Symbol& symbol ) = 0;
        virtual void updateFromDeviceModel( const Symbol& symbol, size_t row, size_t col ) = 0;
        double      calculate( const wxString& expr );


        virtual void requestWriteAll() = 0;

        void    setTablePosition( const wxString& tableName, const wxPoint& position );
        wxPoint getTablePosition( const wxString& tableName ) const;

        void addPageNeedRefresh( int page ) { m_pagesNeedRefresh.insert(page); }
        void updateTableViewers( STDataViewerBase* sender, const wxString& tableName, wxPoint cursorPosition );

        void setConfigValidator( ConfigValidatorBase* validator ) { m_configValidator = validator; }

    protected:
        void clearBurningQueue();
        void clearWritingQueue();

        STDataViewerBaseSet*   m_viewers;
        STDataViewerBaseSet*   m_tableViewers;         /// need for table cursor update
        STDataModelBase*        m_stDataModel;
        STDataModelBase*        m_deviceModel;
        StringToIntSetH         m_waitingForWrite;
        StringToIntSetH         m_waitingForBurn;
        bool                    m_isOnline;
        bool                    m_isConnected;
        bool                    m_isRefreshDeviceModelOnly;
        StringToPosition        m_positionsOfTables;
        IntSet                  m_pagesNeedRefresh;
        ConfigValidatorBase*    m_configValidator;
};


#endif // STDATAMGRBASE_H

