#ifndef STDATAMGRFACADE_H
#define STDATAMGRFACADE_H

#include "STDataMgrBase.h"
#include "ModelFilter.h"

class VariableDataDH;
class SymbolTable;
class STDataModelBase;
class SIPRPacket;
class STDataIOBase;
class VemsControlEvent;

WX_DECLARE_HASH_MAP( int, STDataIOBase* , wxIntegerHash, wxIntegerEqual, InputSources );

class STDataMgrFacade : public STDataMgrBase
{
    public:
        STDataMgrFacade();
        virtual ~STDataMgrFacade();

        void setInput( STDataIOBase* input );

        void requestBurn();
        void requestReadPage();
        void requestReadAllPage();

        void process( const SIPRPacket& p );
        void updateRegisteredViewers();
        void requestWriteAllDirty();
        void requestReadAllDirty();

        void updateViewers(); // non blocking (send an event)

        void requestWrite( const Symbol& symbol , size_t row = 0, size_t col = 0 );

        void setModelData( const Symbol& symbol, size_t row, size_t col, double data );

        /// symbols size must equal to 1
        void setRawModelData( const Symbol& symbol, size_t row, size_t col, uint8_t data );
        void updateRawFromDeviceModel( const Symbol& symbol, size_t row, size_t col );
        void updateFromDeviceModel( const Symbol& symbol );
        void updateFromDeviceModel( const Symbol& symbol, size_t row, size_t col );
        void requestWriteAll();
        void clear();

    private:
        STDataIOBase*           m_currInput;
        ModelFilter             m_filter;

        void addToWritingQueue( const Symbol& symbol, size_t row, size_t col );
        void removeFromWritingQueue( const Symbol& symbol, size_t row, size_t col );
        void addToBurningQueue( const Symbol& symbol, size_t row, size_t col );
        void removeFromBurningQueue( const Symbol& symbol, size_t row, size_t col );

        void removeRefreshedFromWritingQueue();
        void removeRefreshedIndiciesFromWritingQueue( const wxString& name, IntSet& set );

        void OnUpdateViewers( VemsControlEvent& event );
        void sendEvent();
};


#endif // STDATAMGRFACADE_H
