#include "TbTbGPS.h"

#include "../DirHelper.h"
#include "../logger.h"
#include "../VemsTuneController.h"
#include "../configmanager/ConfigFrameHelper.h"

/** \class TbTbGPS
  *
  * \~english
  * \brief Show GPS states
  * \details
  *
  *
  * \~hungarian
  * \brief A GPS állapotát mutatja
  * \details
  *
  *
  */

enum ToolBitmapIds
{
    TBI_GPS_ON,
    TBI_GPS_WARNING,
    TBI_GPS_OFF,
    TBI_GPS_OFFLINE
};

IMPLEMENT_DYNAMIC_CLASS(TbTbGPS,VemsToolBarToolBase)

TbTbGPS::TbTbGPS(int id)
        :VemsToolBarToolBase(id)
{
    m_bmpKeyListArray[TBI_GPS_ON].Add( DirHelper::GetPath(dICON) + _T("tbGPS48-true.png"));
    m_bmpKeyListArray[TBI_GPS_OFF].Add( DirHelper::GetPath(dICON) + _T("tbGPS48-false.png"));
    m_bmpKeyListArray[TBI_GPS_WARNING].Add( DirHelper::GetPath(dICON) + _T("tbGPS48-warning.png"));
    m_bmpKeyListArray[TBI_GPS_OFFLINE].Add( DirHelper::GetPath(dICON) + _T("tbGPS48-offline.png"));


    m_bmpKeyListArray[TBI_GPS_ON].Add( DirHelper::GetPath(dICON) + _T("tbGPS-true.png"));
    m_bmpKeyListArray[TBI_GPS_OFF].Add( DirHelper::GetPath(dICON) + _T("tbGPS-false.png"));
    m_bmpKeyListArray[TBI_GPS_WARNING].Add( DirHelper::GetPath(dICON) + _T("tbGPS-warning.png"));
    m_bmpKeyListArray[TBI_GPS_OFFLINE].Add( DirHelper::GetPath(dICON) + _T("tbGPS-offline.png"));

    m_gpsState = GPSInfo::NOT_CONNECTED;
    m_gpsCommState = GPSInfo::OFFLINE;
    m_VemsTuneController = NULL;
}

TbTbGPS::~TbTbGPS()
{
    if ( m_VemsTuneController ) {
        m_VemsTuneController->Disconnect( wxEVT_GPS, GPSEventHandler( TbTbGPS::OnGPSEvent ),NULL, this);
        m_VemsTuneController->Disconnect( wxEVT_GPS_COMM, GPSCommEventHandler( TbTbGPS::OnGPSCommEvent ), NULL, this );
    }
}



void TbTbGPS::OnTool( wxCommandEvent &event )
{
    if( m_VemsTuneController == NULL ) return;
    if( m_VemsTuneController->getSelectedInput() != INPUT_TYPE_SIPR){ return;}

    bool state = m_VemsTuneController->isGPSOnline();

    m_VemsTuneController->setGPSOnline( !state );
}

void TbTbGPS::OnToolR(wxCommandEvent &event)
{
    ConfigFrameHelper::GPSConfigFrame( m_toolBar->GetParent() );
}

void TbTbGPS::Init()
{
    InitBmpRes();

    m_defaultBitmap = m_bmpHash[TBI_GPS_OFFLINE];

    AddRequest(TN_VemsController);
    AddRequest( TN_GPSEvent );
    AddRequest( TN_GPSCommEvent );

    m_shortHelpString = _T("GPS Disabled!");
    m_longHelpString  = _T("Show the GPS State");

}

bool TbTbGPS::SendObject(int id, wxObject *object, const wxString &message)
{
    if( id == TN_VemsController )
    {
        Logger::logf( LOG_MSG, _T("TbTbGPS::SendObject() - id == TN_VemsController") );

        //m_VemsTuneController = (VemsTuneController*)object;
        m_VemsTuneController = dynamic_cast<VemsTuneController*> (object);
            if(!m_VemsTuneController){
                Logger::logf(LOG_MSG,_T("TbTbGPS::SendObject() dynamic_cast failed!"));
                return false;
            }
        m_VemsTuneController->Connect( wxEVT_GPS, GPSEventHandler( TbTbGPS::OnGPSEvent ),NULL, this);
        m_VemsTuneController->Connect( wxEVT_GPS_COMM, GPSCommEventHandler( TbTbGPS::OnGPSCommEvent ), NULL, this );
    }
    return true;
}

void TbTbGPS::SendEvent( ToolBarToolsNeedId event, int value )
{
    switch(event)
    {
        case TN_GPSEvent:
            m_gpsState = value;
            show();
            break;

        case TN_GPSCommEvent:
            m_gpsCommState = value;
            show();
            break;

        default:
            break;
    }
}

void TbTbGPS::OnGPSEvent( GPSEvent& event )
{
    m_gpsState = event.state;
    show();
    event.Skip();
}

void TbTbGPS::OnGPSCommEvent( GPSCommEvent& event )
{
    m_gpsCommState = event.state;
    show();
    event.Skip();
}

void TbTbGPS::show()
{
    LOGP( wxString::Format( _T("m_gpsCommState: %d"), m_gpsCommState ) );

    if( m_gpsCommState == GPSInfo::OFFLINE )
    {
        SetToolBmp( m_bmpHash[ TBI_GPS_OFFLINE ] );
        m_shortHelpString = _T("GPS Disabled!");
    }
    else
    {
        switch( m_gpsState )
        {
            case GPSInfo::CONNECTED :
                SetToolBmp( m_bmpHash[ TBI_GPS_ON ] );
                m_shortHelpString = _T("GPS Connected!");
                break;

            case GPSInfo::NO_GPS_DATA :
                SetToolBmp( m_bmpHash[ TBI_GPS_WARNING ] );
                 m_shortHelpString = _T("No GPS Data!");
                break;

            case GPSInfo::NOT_CONNECTED :
                SetToolBmp( m_bmpHash[ TBI_GPS_OFF] );
                m_shortHelpString = _T("GPS not found!");
                break;
            default:
                break;
        }
    }

    InitHelpStrings();
}

