/** \class TbTbInputSource
  *
  * \~english
  * \brief Choose the input source and shows he current
  * \details
  *
  *
  * \~hungarian
  * \brief Ki lehet választani a bemeneti forrást és mutatja az aktuálisat
  * \details
  *
  *
  */

#include "TbTbInputSource.h"
#include "../logger.h"
#include "../DirHelper.h"

enum ToolBitmapIds
{
    TBI_IS_NONE,
    TBI_IS_ECU,
    TBI_IS_HIGHF,
    TBI_IS_RAWF
};

IMPLEMENT_DYNAMIC_CLASS(TbTbInputSource,VemsToolBarToolBase)


TbTbInputSource::TbTbInputSource(int id)
             :VemsToolBarToolBase (id)
             ,m_selectInputMenu (NULL)
{
    m_bmpKeyListArray[TBI_IS_ECU].Add(DirHelper::GetPath(dICON)   + _T("tbInputEcu48.png"));
    m_bmpKeyListArray[TBI_IS_HIGHF].Add(DirHelper::GetPath(dICON) + _T("tbInputHighFile48.png"));
    m_bmpKeyListArray[TBI_IS_NONE].Add(DirHelper::GetPath(dICON)  + _T("tbInputNone48.png"));
    m_bmpKeyListArray[TBI_IS_RAWF].Add(DirHelper::GetPath(dICON)  + _T("tbInputRawFile48.png"));

    m_bmpKeyListArray[TBI_IS_ECU].Add(DirHelper::GetPath(dICON)   + _T("tbInputEcu.png"));
    m_bmpKeyListArray[TBI_IS_HIGHF].Add(DirHelper::GetPath(dICON) + _T("tbInputHighFile.png"));
    m_bmpKeyListArray[TBI_IS_NONE].Add(DirHelper::GetPath(dICON)  + _T("tbInputNone.png"));
    m_bmpKeyListArray[TBI_IS_RAWF].Add(DirHelper::GetPath(dICON)  + _T("tbInputRawFile.png"));
}

TbTbInputSource::~TbTbInputSource()
{
}

void TbTbInputSource::OnTool(wxCommandEvent &event)
{
    Logger::logf(LOG_MSG,_T("TbTbInputSource::OnTool"));
    if(m_selectInputMenu != NULL) {
        m_toolBar->PopupMenu(m_selectInputMenu);
    }
}


void TbTbInputSource::OnToolR(wxCommandEvent &event)
{

    Logger::logf(LOG_MSG,_T("TbTbInputSource::OnToolR"));

}

void TbTbInputSource::OnToolEnter(wxCommandEvent &event) {}

void TbTbInputSource::Init()
{
    InitBmpRes();
    m_defaultBitmap = m_bmpHash[TBI_IS_ECU];

    AddRequest(TN_SiMenu);
    AddRequest(TN_ChBmpNone);
    AddRequest(TN_ChBmpHFile);
    AddRequest(TN_ChBmpRFile);
    AddRequest(TN_ChBmpEcu);

    m_shortHelpString = _T("Set the Input Source");
    m_longHelpString  = _T("Set the Input Source");
}

bool TbTbInputSource::SendObject(int id, wxObject *object, const wxString &message)
{
    switch(id) {
        case TN_ChBmpAim:
        case TN_ChBmpEcu:
            SetToolBmp(m_bmpHash[TBI_IS_ECU]);
            break;

        case TN_ChBmpRFile:
            SetToolBmp(m_bmpHash[TBI_IS_RAWF]);
            break;

        case TN_ChBmpNone:
            SetToolBmp(m_bmpHash[TBI_IS_NONE]);
            break;

        case TN_ChBmpHFile:
            SetToolBmp(m_bmpHash[TBI_IS_HIGHF]);
            break;

        case TN_SiMenu:
            m_selectInputMenu = (wxMenu*)object;
            break;

        default:
            break;
    }
    return true;
}


void TbTbInputSource::checkMenuItem( const wxString& menuName )
{
    if( m_selectInputMenu )
    {
        int id = m_selectInputMenu->FindItem( menuName );

        if( id != wxNOT_FOUND )
        {
            m_selectInputMenu->Check( id, true );
        }
    }
}
