#include "TbTbStaticControl.h"

#include "../DirHelper.h"
#include "../logger.h"
#include "../stdatamanager/STDataMgrBase.h"

#define DATA_IS_NOT_AVAILABLE -1e6
#define FLASH_TIME 500

enum ToolBitmapIds
{
    /*
    TBI_STATICCONTROL_WRITE_INACTIVE,
    TBI_STATICCONTROL_WRITE_ACTIVE,
    TBI_STATICCONTROL_WRITE_WARNING,
    TBI_STATICCONTROL_READ_INACTIVE,
    TBI_STATICCONTROL_READ_ACTIVE,
    TBI_STATICCONTROL_READ_WARNING,
    TBI_STATICCONTROL_BURN_INACTIVE,
    TBI_STATICCONTROL_BURN_ACTIVE,
    TBI_STATICCONTROL_BURN_WARNING
*/
    TBI_STATICCONTROL_INACTIVE,
    TBI_STATICCONTROL_ACTIVE,
    TBI_STATICCONTROL_WARNING
};

IMPLEMENT_DYNAMIC_CLASS(TbTbStaticControl,VemsToolBarToolBase)

TbTbStaticControl::TbTbStaticControl(int id, const wxString &type)
    : VemsToolBarToolBase(id)
    , m_timer(this)

{
    if(type == _T("Read") ) {

        m_bmpKeyListArray[TBI_STATICCONTROL_ACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbRead48-active.png"));
        m_bmpKeyListArray[TBI_STATICCONTROL_INACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbRead48-inactive.png"));
        m_bmpKeyListArray[TBI_STATICCONTROL_WARNING].Add( DirHelper::GetPath(dICON) + _T("tbRead48-warning.png"));

        m_shortHelpString = _T("Read data from ECU\nHotkey: Alt-R");
        m_longHelpString  = m_shortHelpString;

    } else if(type == _T("Write") ) {

        m_bmpKeyListArray[TBI_STATICCONTROL_ACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbWrite48-active.png"));
        m_bmpKeyListArray[TBI_STATICCONTROL_INACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbWrite48-inactive.png"));
        m_bmpKeyListArray[TBI_STATICCONTROL_WARNING].Add( DirHelper::GetPath(dICON) + _T("tbWrite48-warning.png"));
        m_shortHelpString = _T("Write data to ECU\nHotkey: Alt-W");
        m_longHelpString  = m_shortHelpString;
    } else if(type == _T("Burn") ) {

        m_bmpKeyListArray[TBI_STATICCONTROL_ACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbBurn48-active.png"));
        m_bmpKeyListArray[TBI_STATICCONTROL_INACTIVE].Add( DirHelper::GetPath(dICON) + _T("tbBurn48-inactive.png"));
        m_bmpKeyListArray[TBI_STATICCONTROL_WARNING].Add( DirHelper::GetPath(dICON) + _T("tbBurn48-warning.png"));
        m_shortHelpString = _T("Burn data to ECU\nHotkey: Alt-B");
        m_longHelpString  = m_shortHelpString;
    }


    m_symbolName    = _T("isEEBusy");
    m_type          = type;
    m_stDataMgr     = NULL;
    m_isDataAvailable       = false;
    m_isRTSymbolRegistered  = false;
    m_burningFlag           = false;

    Connect(wxEVT_TIMER, wxTimerEventHandler(TbTbStaticControl::OnTimer));
}

TbTbStaticControl::~TbTbStaticControl()
{
    unregisterViewer();
    UnRegisterViewer();
}


void TbTbStaticControl::OnTool(wxCommandEvent &event)
{
    if ( !m_stDataMgr ) { return; }

    if( m_type == _T("Read") )
    {
        m_toolBar->EnableTool( m_toolID, false );
        if ( m_stDataMgr->isOnline() && m_stDataMgr->isConnected() ) {
            SetToolBmp(TBI_STATICCONTROL_INACTIVE);
        }
        m_stDataMgr->requestReadAllPage();

    }
    else if( m_type == _T("Write") )
    {
        m_stDataMgr->requestWriteAll();
        m_stDataMgr->updateRegisteredViewers();
    }
    else if( m_type == _T("Burn") )
    {
        m_stDataMgr->requestWriteAll();
        m_stDataMgr->requestBurn();
        m_stDataMgr->updateRegisteredViewers();
    }
}


void TbTbStaticControl::Init()
{
    InitBmpRes();

    m_defaultBitmap = m_bmpHash[TBI_STATICCONTROL_ACTIVE];
    m_inactiveBitmap = m_bmpHash[TBI_STATICCONTROL_INACTIVE];

    AddRequest(TN_STDataMgrBase);
}


bool TbTbStaticControl::SendObject(int id, wxObject *object, const wxString &message)
{
    switch(id) {
        case TN_STDataMgrBase:
            m_stDataMgr = dynamic_cast<STDataMgrBase*> (object);
            if(! m_stDataMgr){
                Logger::logf(LOG_MSG,_T("TbTbStaticControl::SendObject() ProcessBase dynamic_cast failed!") );
            } else {
                m_stDataMgr->RegisterViewer( this );
            }
            break;
        default:
            break;
    }
    return true;
}




void TbTbStaticControl::update()
{
    if ( !m_stDataMgr->isOnline() || !m_stDataMgr->isConnected() )
    {
        if ( m_type != _T("Read") ) {
            if ( m_toolBar->GetToolEnabled( m_toolID ) ) {
            m_toolBar->EnableTool( m_toolID, false );
            SetToolBmp(TBI_STATICCONTROL_INACTIVE);
        }
        } else {
            if ( !m_toolBar->GetToolEnabled( m_toolID ) ) {
                m_toolBar->EnableTool( m_toolID, true );
                highlight( false );
            }
        }

        return;
    }

    if ( !m_toolBar->GetToolEnabled( m_toolID ) )
    {
        m_toolBar->EnableTool( m_toolID, true );
    }

    if ( m_type == _T("Read") )
    {
        highlight( false );
    }
    else if( m_type == _T("Write") )
    {
        highlight( m_stDataMgr->isWaitingForWrite() );
    }
    else if( m_type == _T("Burn") )
    {
        highlight( m_stDataMgr->isWaitingForBurn() || m_stDataMgr->isWaitingForWrite() );
    }
}

void TbTbStaticControl::highlight( bool highlight )
{
    if ( highlight )
    {
        if ( m_ToolBmpId != TBI_STATICCONTROL_WARNING )
        {
            SetToolBmp(TBI_STATICCONTROL_WARNING);
        }
    }
    else
    {
        if ( m_ToolBmpId != TBI_STATICCONTROL_ACTIVE )
        {
            SetToolBmp(TBI_STATICCONTROL_ACTIVE);
        }
    }
}


wxString TbTbStaticControl::toString()
{
    return _T("TbTbStaticControl - ") + m_type;
}

void TbTbStaticControl::OnTimer(wxTimerEvent& evt)
{
    if(m_ToolBmpId == TBI_STATICCONTROL_ACTIVE) {
        SetToolBmp(TBI_STATICCONTROL_WARNING);
    } else {
        SetToolBmp(TBI_STATICCONTROL_ACTIVE);
    }
}

///RTDataViewerBase methods
void TbTbStaticControl::SetValue(double value)
{
    if ( m_type != _T("Burn") ) return;

    if(!isDataAvailable()) return;

    bool isBurning      = value;
    bool isBurningPrev  = m_value;

    if ( isBurning && !isBurningPrev ) {
        m_value = value;
        m_timer.Start( FLASH_TIME );
    } else if ( !isBurning && isBurningPrev ){
        m_value = value;
        m_timer.Stop();
        SetToolBmp(TBI_STATICCONTROL_ACTIVE);
    }
}

void TbTbStaticControl::refresh(DoubleHash* table)
{
    if(table->find(getCurrSymbolName()) == table->end()){
        return;
    }

    double value = (*table)[getCurrSymbolName()];

    m_isDataAvailable = value > DATA_IS_NOT_AVAILABLE;

    SetValue( value );
}

void TbTbStaticControl::require()
{
    if( !m_isRTSymbolRegistered ) {
        m_rtDataMgr->registerSymbol( m_symbolName );
        m_isRTSymbolRegistered = true;
    }
}

void TbTbStaticControl::unregisterSymbols()
{
    if( m_isRTSymbolRegistered ) {
        m_rtDataMgr->unregisterSymbol( m_symbolName );
        m_isRTSymbolRegistered = false;
    }
}
