#include "TriggerFrameFSM.h"

#include "../Math.h"
#include "../logger.h"

#define ESC_7E 0x7E
#define ESC_7D 0x7D
//Escaping:
//    0x7e -> 0x7d 0x5e
//    0x7d -> 0x7d 0x5d

TriggerFrameBase::TriggerFrameBase()
{

}

TriggerFrameBase::~TriggerFrameBase()
{

}
//////////////////////////////////////////////////////////////////////
TriggerFrameFSM::TriggerFrameFSM()
{
    //ctor
    m_state = WAIT;
}

TriggerFrameFSM::~TriggerFrameFSM()
{
    //dtor
}


const IntArray& TriggerFrameFSM::getFrame() const
{
    return m_frame;
}


bool TriggerFrameFSM::parse(uint8_t data)
{
    switch(m_state){
        case FRAME_END:
            reset();
        case WAIT:
            switch(data){
                case ESC_7E:
                    // END of frame
                    m_state = FRAME_END;

                    return true;
                case ESC_7D:
                    m_state = RECEIVED_ESC;
                    return false;
            }
            m_frame.Add(data);
            break;
        case RECEIVED_ESC:
            switch(data){
                case 0x5e:
                    m_frame.Add(0x7e);
                    m_state = WAIT;
                    return false;
                 case 0x5d:
                    m_frame.Add(0x7d);
                    m_state = WAIT;
                    return false;
                 default:
                   /// invalid escape;
                   reset();
            }

    }
    return false;
}

int TriggerFrameFSM::getType() const
{
    // frametype, frame end
    size_t size = m_frame.GetCount();
    if(size < 3) return -1;
    return m_frame.Item(size - 1);
}

void TriggerFrameFSM::reset(){
    m_state = WAIT;
    m_frame.Clear();
}

void TriggerFrameFSM::intArrayToByte( uint8_t* desc )
{
    for ( size_t i=0; i < m_frame.GetCount(); ++i )
        desc[i] = m_frame.Item(i);
}

///////////////////////////////////////////////////////////////////////////
TriggerFrameFSMCRC::TriggerFrameFSMCRC()
{

}

TriggerFrameFSMCRC::~TriggerFrameFSMCRC()
{

}

bool TriggerFrameFSMCRC::parse(uint8_t data){
    if( m_trFrameFSM.parse(data) ){
        return checkCRC( m_trFrameFSM.getFrame() );
    }
    return false;
}

const IntArray& TriggerFrameFSMCRC::getFrame() const
{
    return m_trFrameFSM.getFrame();
}

int TriggerFrameFSMCRC::getType() const
{
    // frametype, crc, crc, frame end
    const IntArray& frame = m_trFrameFSM.getFrame();
    size_t size = frame.GetCount();
    if(size < 3) return -1;
    return frame.Item(size - 3);
}

void TriggerFrameFSMCRC::intArrayToByte( uint8_t* desc )
{
    m_trFrameFSM.intArrayToByte( desc );
}

bool TriggerFrameFSMCRC::checkCRC(const IntArray& frame) const
{
     // LITTLE ENDIAN
    int size = frame.GetCount();
    if(size < 3) return false;
    uint16_t crc = frame.Item( size - 2 ) + 256 * frame.Item( size - 1 );

    // calculate crc;
    uint16_t calc_crc = 0xFFFF;

    for( size_t i = 0; i < size-2; ++i )
    {
        calc_crc = Math::crc16_sipr( calc_crc, frame.Item( i ) );
    }

    // CRC error
    if( crc != calc_crc )
    {
        Logger::logf( LOG_ERROR,_T("TriggerFrameFSM::checkCRC() CRC error 0x%04X, 0x%04X"), crc, calc_crc );
        return false;
    }
    return true;

}
