#ifndef TRIGGERLOG_H
#define TRIGGERLOG_H

#include "../FirmwareInfo.h"

#include "../common/types/IntArray.h"
#include "../common/types/Int2IntHash.h"

#include "../common/types/DoubleArray.h"
#include "../common/types/DoubleHash.h"
#include "../common/types/Int2DoubleArrayHash.h"

#include "TriggerLogHeader.h"
#include "TriggerFrame.h"

#include "../Math.h"
#include "../logger.h"
#include "../ObserverBase.h"


class TriggerLogManager;

class TriggerLog : public ObserverBase
{
    public:
        TriggerLog();
        virtual ~TriggerLog();

        IntArray&           getRawTriggerData()     { return m_rawTriggerData; }

        DoubleArray&        getNthTimeLine( unsigned int n );
        Int2DoubleArrayHash& getTimelines()         { return m_timelines; }

        TriggerFrameArray&  getFrameArray()         { return m_rawTriggerFrameA; }

        void                setFirmwareInfo( FirmwareInfo& fwInfo );

        Int2IntHash&        getResult()             { return m_resultH; }
        virtual void        clearData();

        void                getHeaderInfo( wxString& fwVer, wxString& createTime, wxString& serialnum );

        size_t receivedDataCount(){ return m_rawTriggerData.GetCount(); }

        /// DEPRECATED - only prim&sectrig
        /*
        DoubleArray&        getPrimaryTimeLine()    { return m_primaryTrTime; }
        DoubleArray&        getSecondaryTimeLine()  { return m_secondraryTrTime; }
        */
        // changed to:
        DoubleArray&        getPrimaryTimeLine()    { return m_timelines[TriggerFrame::CRC_ONLY]; }
        DoubleArray&        getSecondaryTimeLine()  { return m_timelines[TriggerFrame::SEC_TRIG]; }

    protected:

        double              m_threshold;

        //contains 8bit stream values
        IntArray            m_rawTriggerData;

        TriggerFrameArray   m_rawTriggerFrameA;

        Int2DoubleArrayHash m_timelines;

        /// header
        wxString            m_fwVersion;
        wxString            m_creationTime;
        wxString            m_serialNum;

        Int2IntHash         m_resultH;

        /// DEPRECATED - only prim&sectrig
        /*
        DoubleArray         m_primaryTrTime;
        DoubleArray         m_secondraryTrTime;
        */

    protected:
        void                fillTriggerData();
        void                setThreshold( double threshold ) { m_threshold = threshold; }
        int                 unescape( const IntArray& src, IntArray& out );
        FirmwareInfo        m_fwinfo;

        friend class TriggerLogManager;

    private:

        enum { WAITING_ESC, ESC_FOUND };

        int                 m_prevvalue;
        int                 m_prevdiff;

    private:

        IntArray            getFramesIdx( const IntArray& src );
        IntArray            getFrame( const IntArray& src, const IntArray& idxA, size_t idx );

        void                fillTRDataArray( const IntArray& src, TriggerFrameArray& dest );

        // new version
        void                _fillTimeLine( const TriggerFrameArray& src );

        /// NOT NEEDED OR DEPRECATED
        //void                hexLog( const IntArray& src, int col = 16 );
        // old version (quasi deprecated) - calls _fillTimeLine
        //void                fillTimeLine( const TriggerFrameArray& src );
};

#endif // TRIGGERLOG_H
