#include "TriggerLogController.h"
#include "../communication/clients/DeviceModeSelector.h"
#include "../communication/clients/TriggerLogReader.h"
#include "../logger.h"
#include "../vemsprotocol/VemsDeviceControllerBase.h"
#include "../events/GeneralEvent.h"

BEGIN_EVENT_TABLE( TriggerLogController, wxEvtHandler )
    EVT_COMMUNICATION( wxID_ANY, TriggerLogController::OnCommunicationEvent )
    EVT_GENERAL( wxID_ANY, TriggerLogController::OnGeneralEvent )
END_EVENT_TABLE()

TriggerLogController::TriggerLogController(  VemsDeviceControllerBase* deviceController )
{
    m_deviceController = deviceController;
//    m_evtHandler = evtHandler;
}

TriggerLogController::~TriggerLogController()
{
}

void TriggerLogController::OnTimeout()
{
    // intentionally empty
}

void TriggerLogController::OnDeviceModeSelected( ECUInfo::EcuMode deviceMode )
{
    m_deviceController->registerClient( new TriggerLogReader( this, this ) );
    m_deviceController->startCommControllerNow();
}

void TriggerLogController::start()
{
    if ( m_deviceController->isManMode() ) {
        m_deviceController->registerClient( new TriggerLogReader( this, this ) );
    } else {
        registerDeviceModeSelector();
    }

    m_deviceController->startCommControllerNow();
}

void TriggerLogController::stop()
{
    TriggerLogReader* client = dynamic_cast<TriggerLogReader*>( m_deviceController->getCommClient() );

    if ( client ) {
        client->stop();
    } else {
        LOGP( _T("Cannot cast client to TriggerLogReader!") );
    }

//    m_deviceController->stop( VDC_FLAG_NOEVENT );
}

void TriggerLogController::registerDeviceModeSelector()
{
    m_deviceController->registerClient( new DeviceModeSelector( this,
                                                                this,
                                                                m_deviceController->getCommBase(),
                                                                m_deviceController->getSupportedBaudrates(),
                                                                m_deviceController->getDeviceMode(),
                                                                ECUInfo::MAN,
                                                                &m_deviceMode ) );
}

void TriggerLogController::OnCommunicationEvent( CommunicationEvent& event )
{
    LOG
//    if ( m_evtHandler )
//        m_evtHandler->AddPendingEvent( event );
    m_deviceController->AddPendingEvent( event );
}

void TriggerLogController::OnGeneralEvent( GeneralEvent& event )
{
    if ( event.GetString() == _T("TRIGGER_LOG_STOPPED") )
	{
        m_deviceController->stop( VDC_FLAG_NOEVENT );
	}
}
