#include "TriggerLogGraph.h"

#include <wx/dcbuffer.h>
#include <wx/filedlg.h>
#include <wx/textfile.h>
#include <wx/xml/xml.h>
#include <wx/filename.h>

#include "TriggerFrame.h"
#include "TriggerLog.h"
#include "TriggerLogManager.h"
#include "TriggerLogImg.h"
#include "TriggerLogSaveAudio.h"
#include "TriggerLogSettingsHelper.h"
#include "TriggerLogGraphSettings.h"

#include "../agg/wxAgg2.h"

#include "../bcWidgets/bcButton.h"
#include "../bcWidgets/bcLabel.h"
#include "../bcWidgets/bcScrollBar.h"
#include "../bcWidgets/bcProgressDialog.h"

#include "../vemsprotocol/VemsTuneEventDispatcher.h"

#include "../rtdatastat/RTPlotter.h"

#include "../DirHelper.h"

#include "../logger.h"

#include "../wxVariantHelper.h"

extern int idTLogColApply;
extern int idTLogColSave;

int idTLogGScrollBar            = wxNewId();
int idTLogGSavePngButton        = wxNewId();
int idTLogGSaveWavButton        = wxNewId();
int idTLogGSRTButton            = wxNewId();
int idTLogGPauseButton          = wxNewId();
int idTriggerLogGraphTimer      = wxNewId();
int idTriggerLogGraphCommTimer  = wxNewId();
int idTLogCloseButton           = wxNewId();
int idTLogHelpButton            = wxNewId();
int idTLogColorButton           = wxNewId();
int idTLogGV3StartButton        = wxNewId();
int idTLogGV3StopButton         = wxNewId();
int idTLogGV3ShowResButton      = wxNewId();
int idTLogGMaximizeButton       = wxNewId();

BEGIN_EVENT_TABLE( TriggerLogGraph, wxFrame )
    EVT_PAINT( TriggerLogGraph::OnPaint )
    EVT_SIZE( TriggerLogGraph::OnSizeEvent )
    EVT_ACTIVATE( TriggerLogGraph::OnActivate )

    EVT_LEFT_DCLICK( TriggerLogGraph::OnDblClick )
    EVT_MOUSEWHEEL( TriggerLogGraph::OnMouseWheel )
    EVT_MOUSE_EVENTS( TriggerLogGraph::OnDragging )
    EVT_MOTION( TriggerLogGraph::OnMouse )

    EVT_COMMAND_SCROLL_TOP( idTLogGScrollBar, TriggerLogGraph::OnScroll )
    EVT_COMMAND_SCROLL_BOTTOM( idTLogGScrollBar, TriggerLogGraph::OnScroll )
    EVT_COMMAND_SCROLL_LINEUP( idTLogGScrollBar, TriggerLogGraph::OnScroll )
    EVT_COMMAND_SCROLL_LINEDOWN( idTLogGScrollBar, TriggerLogGraph::OnScroll )
    EVT_COMMAND_SCROLL_PAGEUP( idTLogGScrollBar, TriggerLogGraph::OnScroll )
    EVT_COMMAND_SCROLL_PAGEDOWN( idTLogGScrollBar, TriggerLogGraph::OnScroll )
    EVT_COMMAND_SCROLL_THUMBRELEASE( idTLogGScrollBar, TriggerLogGraph::OnScroll )

    EVT_BUTTON( idTLogGSavePngButton, TriggerLogGraph::OnButton )
    EVT_BUTTON( idTLogGSaveWavButton, TriggerLogGraph::OnButton )
    EVT_BUTTON( idTLogGSRTButton, TriggerLogGraph::OnButton )
    EVT_BUTTON( idTLogGPauseButton, TriggerLogGraph::OnButton )
    EVT_BUTTON( idTLogCloseButton, TriggerLogGraph::OnButton )
    EVT_BUTTON( idTLogHelpButton, TriggerLogGraph::OnButton )
    EVT_BUTTON( idTLogColorButton, TriggerLogGraph::OnButton )
    EVT_BUTTON( idTLogGMaximizeButton, TriggerLogGraph::OnButton )

    EVT_BUTTON( idTLogGV3StartButton, TriggerLogGraph::OnV3Button )
    EVT_BUTTON( idTLogGV3StopButton, TriggerLogGraph::OnV3Button )
    EVT_BUTTON( idTLogGV3ShowResButton, TriggerLogGraph::OnV3Button )

    EVT_TRIGGERLOG( TriggerLogGraph::OnTriggerLogEvent )
    EVT_TIMER(idTriggerLogGraphTimer, TriggerLogGraph::OnTimer)
    EVT_TIMER(idTriggerLogGraphCommTimer, TriggerLogGraph::OnCommTimer)
    EVT_CHAR( TriggerLogGraph::OnKeyEvent )

    EVT_COMMUNICATION( wxID_ANY, TriggerLogGraph::OnCommunicationEvent )

END_EVENT_TABLE()

TriggerLogGraph::TriggerLogGraph( wxWindow* parent, VemsTuneEventDispatcher* evtDispatcher, const wxString& logdir )
: m_triggerlog( NULL ), m_evtHandler( NULL ), m_timer( this, idTriggerLogGraphTimer )
{
    m_evtDispatcher = evtDispatcher;

    if ( m_evtDispatcher )
        m_evtDispatcher->addEventHandler( this );

    TriggerLogSettingsHelper::TriggerLogSettingsHelper();

    m_progdlg = NULL;

    m_tlogmanager = NULL;
    m_created = false;
    m_helpPanelShown = false;
    m_needstoRedraw = true;

    m_tlogsupported = false;

    m_timerTime = 500; // 1000ms update

    m_commtimer.SetOwner( this, idTriggerLogGraphCommTimer );

    m_memDC = new wxMemoryDC;
    m_memDC->SelectObject( wxNullBitmap );

    m_canvasBmp = NULL;

    m_isMaximized = false;

    buildGUI( parent );

    m_plotter = new RTPlotter( NULL, this );

    ResetValues();

    m_logdir = logdir;

    m_noserial = false;

    m_isContinuous = false;

    m_timer.Start( m_timerTime, false );

    CreateBcParent( this );

    clearData();
    Show();
}

void TriggerLogGraph::clearData()
{
    /// DEPRECATED - only prim&sectrig
    /*m_primary.Clear();
    m_secondary.Clear();*/

    // new version
    Int2DoubleArrayHash::iterator it = m_timelines.begin();
    for ( ; it != m_timelines.end(); ++it )
    {
        it->second.Clear();
    }
    m_timelines.clear();

    ResetValues();
}

void TriggerLogGraph::ResetValues()
{
    m_needstoRedraw = true;

    m_min = 0;
    m_max =0;
    m_maxvalue = 0;
    m_cursor = 0;

    if ( m_tlogmanager != NULL )
        m_isContinuous = m_tlogmanager->isContinuous();
    else
        m_isContinuous = false;

    m_showLast = true;
    m_timewindow = 800.0;
    m_paused = false;

    m_clockBase = 0;
    m_prevTime = 0;

    m_prevReceivedData = -1;
}

TriggerLogGraph::~TriggerLogGraph()
{
    if ( m_evtDispatcher )
        m_evtDispatcher->removeEventHandler( this );

    if ( m_memDC )
    {
        m_memDC->SelectObject( wxNullBitmap );
        delete m_memDC;

        if ( m_canvasBmp )
            delete m_canvasBmp;
    }

    delete m_tlogmanager;
}

void TriggerLogGraph::setTriggerLogMgr( TriggerLogManager* mgr )
{
    m_tlogmanager = mgr;

    wxString fwver, ct, snum;

    if ( mgr )
        mgr->getTriggerLog()->getHeaderInfo( fwver, ct, snum );

    m_foundTlogVer = fwver;

    if ( fwver >= _T("1.1.78") )
        m_tlogsupported = true;
    else
    {
        m_tlogsupported = false;
    }


    m_isContinuous = m_tlogmanager->isContinuous();

    setTriggerLog( m_tlogmanager->getTriggerLog() );

    switch ( m_tlogmanager->getType() && m_tlogsupported )
    {
        case 0:  m_v3start->Enable(); break;

        case 1: {
            TriggerFrameArray& trFrameArray = m_triggerlog->getFrameArray();

            for ( size_t i = 0; i < trFrameArray.GetCount(); ++i )
            {
                TriggerFrame* fr = trFrameArray.Item(i);
                add( *fr );
            }
        }
    }

};

void TriggerLogGraph::setTriggerLog( TriggerLog* triggerlog )
{
    m_triggerlog = triggerlog;

    // check if it is a log file
    TriggerLogImg* img = dynamic_cast<TriggerLogImg*>(triggerlog);

    if ( img != NULL )
        m_v3showResultButton->Enable();
}

// loads settings from file or use default values if the file does not exist
void TriggerLogGraph::GetSettings()
{
    wxString filename = DirHelper::GetPath( dCONFIG ) + _T("graphcolors.xml");

    TriggerLogSettingsHelper::loadSettings( filename );

    m_bgColor = TriggerLogSettingsHelper::tLog_bg;
    m_scopeColor = TriggerLogSettingsHelper::tLog_scope;
    m_textcolor = TriggerLogSettingsHelper::tLog_text;
}

// save current settings to file
void TriggerLogGraph::SaveSettings()
{
    wxString filename = DirHelper::GetPath( dCONFIG ) + _T("graphcolors.xml");

    if ( TriggerLogSettingsHelper::saveSettings( filename ) == false )
    {
        wxMessageBox( _T("Could not save color settings"), _T("Error"), wxOK | wxICON_ERROR );
    }
}

void TriggerLogGraph::buildGUI( wxWindow* parent )
{
    GetSettings(); // load colors

    Create( parent, wxID_ANY, _T("Triggers"), wxDefaultPosition, wxSize(400,200),
        wxRESIZE_BORDER |  wxFRAME_FLOAT_ON_PARENT );

    wxSizer* mainsizer = new wxBoxSizer( wxVERTICAL );

    m_scrollBar = new bcScrollBar( this, idTLogGScrollBar, wxDefaultPosition, wxSize( 400, 20 ), wxSB_HORIZONTAL );

    // space for the image
    mainsizer->AddStretchSpacer( 10 );

    m_helpPanel = new bcLabel( this, wxID_ANY, _T(""), wxDefaultPosition, wxSize( 230, 80 ), BCLS_AUTO_SIZE );
    m_helpPanel->SetAlpha(0,false);
    m_helpPanel->SetProperty( BCBS_BACKGROUND_COLOR, wxVariantHelper::ToVariant(wxColor(255,255,255,200)) );
    wxString helpStr = _T("Controls\n\n\u2191\u2193       - Zoom in/out");
    helpStr += _T("\n\u2190 \u2192   - Navigate left/right");
    helpStr += _T("\nF1       - Show/hide this help window");

    double text_height = 13;

    m_helpPanel->SetLabel( TextDesc( helpStr, 0, wxALIGN_LEFT, text_height ) );


    // buttons
    wxSizer* bsizer = new wxBoxSizer( wxHORIZONTAL );

    wxVariant c;

    wxImage savetopng( DirHelper::GetPath( dICON ) + _T("savetopng.png"), wxBITMAP_TYPE_PNG );
    wxImage savetowav( DirHelper::GetPath( dICON ) + _T("savetowav.png"), wxBITMAP_TYPE_PNG );

    m_startButton = new bcButton( this, idTLogGPauseButton, _T("Pause"), wxDefaultPosition, wxSize( 80, 20 ) );
    m_startButton->SetStyleFlag( bcBUTTON_HOVER_OUTER_GLOW );
    m_scrollButton = new bcButton( this, idTLogGSRTButton, _T("Scroll as real-time"), wxDefaultPosition, wxSize( 120, 20 ) );
    m_scrollButton->SetStyleFlag( bcBUTTON_HOVER_OUTER_GLOW );

    m_saveSButton = new bcButton( this, idTLogGSavePngButton, savetopng, wxDefaultPosition, wxSize( 55, 20 ) );
    m_saveWButton = new bcButton( this, idTLogGSaveWavButton, savetowav, wxDefaultPosition, wxSize( 55, 20 ) );

    m_colorSettingsButton = new bcButton( this, idTLogColorButton, _T("Change settings"), wxDefaultPosition, wxSize( 120, 20 ) );
    m_colorSettingsButton->SetStyleFlag( bcBUTTON_HOVER_OUTER_GLOW );

    m_v3start = new bcButton( this, idTLogGV3StartButton, _T("Start V3"), wxDefaultPosition, wxSize( 80, 20 ) );
    m_v3stop = new bcButton( this, idTLogGV3StopButton, _T("Stop V3"), wxDefaultPosition, wxSize( 80, 20 ) );
    m_v3showResultButton = new bcButton( this, idTLogGV3ShowResButton, _T("Show result"), wxDefaultPosition, wxSize( 120, 20 ) );
    m_v3start->Disable();
    m_v3stop->Disable();
    m_v3showResultButton->Disable();
    m_v3start->SetStyleFlag( bcBUTTON_HOVER_OUTER_GLOW );
    m_v3stop->SetStyleFlag( bcBUTTON_HOVER_OUTER_GLOW );
    m_v3showResultButton->SetStyleFlag( bcBUTTON_HOVER_OUTER_GLOW );

    m_closeButton = new bcButton( this, idTLogCloseButton, _T("X"), wxDefaultPosition, wxSize( 16, 16 ), bcBUTTON_ALIGN_CENTER_HORIZONTAL | bcBUTTON_ALIGN_CENTER_VERTICAL & ~bcBUTTON_AUTOSIZE );
    m_closeButton->SetProperty( BCBS_TEXT_HEIGHT, 10.0 );
    m_closeButton->SetProperty( BCBS_PADDDING, 0.5 );
    c << wxColor( 244, 43, 1 );
    m_closeButton->SetProperty( BCBS_BACKGROUND_COLOR, c );
    c << wxColor( 128, 128, 128, 160 );
    m_closeButton->SetProperty( BCBS_BORDER_COLOR, c );
    c << wxColor( 255, 255, 255, 255 );
    m_closeButton->SetProperty( BCBS_FOREGROUND_COLOR, c, true );

    m_helpButton = new bcButton( this, idTLogHelpButton, _T("?"), wxDefaultPosition, wxSize( 16, 16 ), bcBUTTON_ALIGN_CENTER_HORIZONTAL | bcBUTTON_ALIGN_CENTER_VERTICAL  & ~bcBUTTON_AUTOSIZE );
    m_helpButton->SetProperty( BCBS_TEXT_HEIGHT, 10.0 );
    m_helpButton->SetProperty( BCBS_PADDDING, 0.5 );
    c << wxColor( 0, 0, 0 );
    m_helpButton->SetProperty( BCBS_BACKGROUND_COLOR, c );
    c << wxColor( 128, 128, 128, 160 );
    m_helpButton->SetProperty( BCBS_BORDER_COLOR, c );
    c << wxColor( 255, 255, 255, 255 );
    m_helpButton->SetProperty( BCBS_FOREGROUND_COLOR, c, true );

    m_maximizeButton = new bcButton( this, idTLogGMaximizeButton, _T("↕"), wxDefaultPosition, wxSize( 16, 16 ), bcBUTTON_ALIGN_CENTER_HORIZONTAL | bcBUTTON_ALIGN_CENTER_VERTICAL & ~bcBUTTON_AUTOSIZE );
    m_maximizeButton->SetProperty( BCBS_TEXT_HEIGHT, 10.0 );
    m_maximizeButton->SetProperty( BCBS_PADDDING, 0.5 );
    c << wxColor( 0, 0, 0 );
    m_maximizeButton->SetProperty( BCBS_BACKGROUND_COLOR, c );
    c << wxColor( 128, 128, 128, 160 );
    m_maximizeButton->SetProperty( BCBS_BORDER_COLOR, c );
    c << wxColor( 255, 255, 255, 255 );
    m_maximizeButton->SetProperty( BCBS_FOREGROUND_COLOR, c, true );


    m_commerrorTxt = new bcLabel( this, wxID_ANY);
    c << wxColor( 0, 0, 0, 0 );
    TextDesc desc = m_commerrorTxt->GetLabelDesc();
    desc.Color = wxColor( 180, 0, 0 );
    m_commerrorTxt->SetLabel( desc );


    m_helpTxt = new bcLabel( this, wxID_ANY );
    m_helpTxt->SetLabel( _T("Press F1 for help") );
    //
    m_totalTime = new bcLabel( this, wxID_ANY, _T("Total time"), wxDefaultPosition, wxSize( 400, 20 ) );

    //
    m_statistics = new bcLabel( this, wxID_ANY, _T("") );

    //
    bsizer->Add( 10, 20 );
    mainsizer->Add( bsizer, 0, wxALL | wxALIGN_LEFT, 5 );

    //
    SetSizer( mainsizer );

    mainsizer->Fit( this );

    mainsizer->SetSizeHints( this );

    m_created = true;

    SetSize( 720, 630 );

    m_scrollBar->SetScrollbar( 0, 1, 1, 1 );

    m_saveSButton->SetToolTip( _T("Save to PNG"), true );
    m_saveSButton->SetStyleFlag( bcBUTTON_HOVER_OUTER_GLOW );
    m_saveWButton->SetToolTip( _T("Create WAV file from triggerlog"), true );
    m_saveWButton->SetStyleFlag( bcBUTTON_HOVER_OUTER_GLOW );
}

void TriggerLogGraph::OnPaint( wxPaintEvent& event )
{
    wxBufferedPaintDC dc( this );

    dc.SetBackground( m_bgColor );
    dc.Clear();

    int w, h;

    GetClientSize( &w, &h );

    wxAgg2 canvas( w, 20 );

    // draw title bar ( active/inactive case )
    wxAggBrush brush;
    if ( IsActive() )
    {
        brush.SetColor( wxColor( 0, 0, 0, 255 ) );
        brush.SetEndColor( wxColor( 128, 128, 128, 255 ) );
    }
    else
    {
        brush.SetColor( wxColor( 128, 128, 128, 255 ) );
        brush.SetEndColor( wxColor( 228, 228, 228, 255 ) );
    }
    brush.SetStyle( wxAGGBRUSH_GRADIENT );
    canvas.setBrush( brush );

    canvas.setPen( wxAGGPEN_TRANSPARENT );
    canvas.drawRectangle( 0, 0, w, 20 );

    // draw title bar text
    if ( IsActive() )
        canvas.setPen( wxColor( 255, 255, 255, 255 ) );
    else
        canvas.setPen( wxColor( 220, 220, 220, 255 ) );

    canvas.loadTTFont( DirHelper::GetPath( dFONTS ) + _T("arial.ttf"), 12, 1.2 );
    canvas.drawTTFText( _T("Triggers"), 2, 16 );

    canvas.render( dc );

    if ( m_tlogsupported == false )
    {
        wxAgg2 canvas2( w - 10, 110, m_bgColor );
        canvas2.loadTTFont( DirHelper::GetPath( dFONTS ) + _T("arial.ttf"), 24, 1.2 );
        canvas2.setPen( wxColor( 0, 0, 0 ) );
        wxString msg = _T("To display triggerlogs, firmware version 1.1.78 or\n\ngreater is required (found: ") + m_foundTlogVer + _T(")");
        canvas2.drawTTFText( msg, 0, 30 );

        canvas2.render( dc, 20, 240 );
    }
    else
    {
        if ( m_needstoRedraw )
        {
            if ( m_canvasBmp != NULL )
            {
                m_memDC->SelectObject( wxNullBitmap );
                delete m_canvasBmp;
            }
            m_canvasBmp = new wxBitmap( w - 40, h - 130 );
            m_memDC->SelectObject( *m_canvasBmp );

            m_plotter->SetDC( &dc );

            if ( h < 80 )
                return;

            m_plotter->SetViewPortSize( w - 40, h - 130 );

            m_plotter->SetBackgroundColor( m_scopeColor );
            m_plotter->SetLineColor( wxColor( 0, 0, 0, 255 ) );
            m_plotter->SetDivisionColor( wxColor( 80, 80, 80, 200 ) );
            m_plotter->CreateCanvas();

            m_plotter->SetAxisLabels( _T("Time (ms)"), _T("") );

            /// draw signs
            // old ver
            // m_plotter->DrawTriggers( m_primary, m_secondary, m_tlogcolors[0], m_tlogcolors[1], m_min, m_max );

            /// new ver
            m_plotter->DrawTriggers( m_timelines, m_min, m_max, m_triggerlog->getResult() );

            m_memDC->Blit( 0, 0, w - 40, h - 130, &dc, 0, 0 );

            m_needstoRedraw = false;
        }
        else
            dc.Blit( 0, 0, w - 40, h - 130, m_memDC, 0, 0 );
    }

    // paint the buttons & labels
    PaintWidgets( dc );
}

void TriggerLogGraph::OnScroll( wxScrollEvent& event )
{
    // if clicked on scrollbar -> stop continuous drawing
    m_showLast = false;

    m_cursor = m_scrollBar->GetThumbPosition() + m_scrollBar->GetThumbSize() / 2;

    double diff = ( m_max - m_min ) / 2;

    m_min = m_cursor - diff;
    m_max = m_cursor + diff;

    if ( m_min < 0 )
    {
        m_min = 0;
    }

    if ( m_max > m_maxvalue )
    {
        m_max = m_maxvalue;
    }

    // set focus to the frame
    //m_scrollBar->GetParent()->SetFocus();

    m_needstoRedraw = true;

    Refresh( false );

    event.Skip();
}

void TriggerLogGraph::OnSizeEvent( wxSizeEvent& event )
{
    // when initializing scrollbar and the other widgets do not exist yet -> needs to return
    if ( !m_created )
        return;

    m_needstoRedraw = true;

    Refresh( false );

    int w, h;

    GetClientSize( &w, &h );

    int step = (int)( m_max - m_min );
    m_scrollBar->SetSize( w - 40, 20 );
    m_scrollBar->SetScrollbar( m_cursor - m_scrollBar->GetThumbSize(), step, (int)m_maxvalue, step );

    event.Skip();

    // move scrollbar & other widgets to the right position
    m_scrollBar->SetPosition( 10, h - 85 );
    m_startButton->SetPosition( 10, h - 60 );
    m_scrollButton->SetPosition( 100, h - 60 );
    m_saveSButton->SetPosition( 230, h - 60 );
    m_saveWButton->SetPosition( 295, h - 60 );
    m_colorSettingsButton->SetPosition( 360, h - 60 );
    m_totalTime->SetPosition( 490, h - 60 );
    m_closeButton->SetPosition( w - 24, 0 );
    m_helpButton->SetPosition( w - 44, 0 );
    m_maximizeButton->SetPosition( w - 64, 0 );

    m_statistics->SetPosition( 20, h - 170 );

    m_v3start->SetPosition( 10, h - 30 );
    m_v3stop->SetPosition( 100, h - 30 );
    m_v3showResultButton->SetPosition( 230, h - 30 );
    m_helpTxt->SetPosition( 370, h - 30 );
    m_commerrorTxt->SetPosition( 490, h - 30 );

//    m_oldSize.x = w;
//    m_oldSize.y = w;
}

void TriggerLogGraph::OnMouseWheel( wxMouseEvent& event )
{
    int mx = event.GetX(), my = event.GetY();

    if ( m_plotter != NULL )
    {
        double xval, yval;

        if ( m_plotter->GetXYValue( mx, my, xval, yval ) )
        {
            m_showLast = false;

            int dir = event.GetWheelRotation();

            if ( dir > 0 )
            {
                Zoom( true );
            }
            else
            {
                Zoom( false );
            }
        }
        else
        {
            event.Skip();

            return;
        }
    }
    else
    {
        event.Skip();
        return;
    }
}

void TriggerLogGraph::OnKeyEvent( wxKeyEvent & event )
{
    int code = event.GetKeyCode();

    // F1: show/hide help, ESC: hide help
    if ( code == WXK_F1 )
    {
        if ( code == WXK_F1 )
        {
            ToggleHelpPanel();

            m_helpPanelShown = !m_helpPanelShown;
        }

        event.Skip();

        return;
    }

    // arrows
    if ( code == WXK_DOWN || code == WXK_UP || code == WXK_LEFT || code == WXK_RIGHT )
    {
        m_showLast = false;

        /// ZOOM IN&OUT
        if ( code == WXK_DOWN || code == WXK_UP )
        {
            if ( code == WXK_DOWN ) // zoom out
            {
                Zoom( false );
            }
            else // zoom in
            {
                Zoom( true );
            }
        }

        /// MOVING
        if ( code == WXK_RIGHT || code == WXK_LEFT )
        {
            int step = (int)( m_max - m_min );

            switch ( code )
            {
                case WXK_LEFT:
                {
                    m_min -= step/2; m_max -= step/2;
                    if ( m_min < 0 )
                    {
                        m_min = 0;
                        m_max = step;
                    }
                    break;
                }
                case WXK_RIGHT:
                {
                    m_min += step/2; m_max += step/2;
                    if ( m_max > m_maxvalue )
                    {
                        m_min = m_maxvalue - step;
                        m_max = m_maxvalue;
                    }
                    break;
                }
            }

            m_needstoRedraw = true;

            Refresh( false );

            m_cursor = (unsigned long)(( m_max + m_min ) / 2);
            m_scrollBar->SetScrollbar( m_cursor - m_scrollBar->GetThumbSize() / 2, step, (int)m_maxvalue, step );

        }
    }
    event.Skip();
}

void TriggerLogGraph::Zoom( bool zoomin )
{
    if ( m_tlogsupported == false )
    {
        return;
    }

    double mid = ( m_min + m_max  ) * 0.5 ;
    double diff = fabs( mid - m_min );

    if ( zoomin == false )
    {
        m_min = mid - 2 * diff;
        m_max = mid + 2 * diff;
        if ( m_min < 0 )
            m_min = 0;
        if ( m_max > m_maxvalue )
            m_max = m_maxvalue;
    }
    else
    {
        m_min = mid - 0.5 * diff;
        m_max = mid + 0.5 * diff;
    }

    m_needstoRedraw = true;

    Refresh(false);


    int step = (int)(m_max - m_min);

    int step1 = step - 1;
    if ( step1 < 1 ) step1 = 1;

    if ( zoomin == false ) // zoom out
    {
        m_scrollBar->SetScrollbar( m_cursor - m_scrollBar->GetThumbSize(), step, (int)m_maxvalue, step );
    }
    else
    {
        m_scrollBar->SetScrollbar( m_cursor - m_scrollBar->GetThumbSize() / 4, step, (int)m_maxvalue, step );
    }
}

// move the frame or select range
void TriggerLogGraph::OnDragging( wxMouseEvent& event )
{
    static double startPos = 0;
    static bool isDragging = false;
    static wxPoint sPoint;

    long x, y;

    x = event.GetX();
    y = event.GetY();

    int w, h;

    GetClientSize( &w, &h );

    // moving the frame
    if ( isDragging || ( x >=0 && x < w - 70 && y > 0 && y < 20 ) )
    {
        SetCursor( wxCURSOR_ARROW );

        if ( event.LeftIsDown() )
        {
            if ( !isDragging )
            {
                isDragging = true;
                sPoint = event.GetPosition();
                return;
            }
            else
            {
                wxPoint cPoint = event.GetPosition();
                wxPoint pos = GetPosition();
                pos += ( cPoint - sPoint );
                Move( pos );
                return;
            }
        }
        else
        {
            isDragging = false;
        }
    }

    // new ver
    DoubleArray& primary = m_timelines[TriggerFrame::CRC_ONLY];
    DoubleArray& secondary = m_timelines[TriggerFrame::SEC_TRIG];

    /*
    // OLD VERSION, DEPRECATED - only prim&sectrig
    DoubleArray& primary = m_primary;
    DoubleArray& secondary = m_secondary;
    */

    // selecting
    if ( m_plotter != NULL )
    {
        double xval, yval;

        if ( m_plotter->GetXYValue( x, y, xval, yval ) )
        {
            m_needstoRedraw = true;

            if ( event.LeftIsDown() )
            {
                if ( !m_isSelecting )
                {
                    SetCursor( wxCURSOR_SIZEWE );
                    m_isSelecting = true;
                    startPos = xval;
                    m_plotter->SetSelection( startPos, startPos );
                }
                else
                {
                    double endPos = xval;
                    m_plotter->SetSelection( startPos, endPos );

                    size_t selPrim = 0, selSec = 0;

                    for ( size_t i = 0; i < primary.GetCount(); ++i )
                    {
                        if ( primary[i] >= startPos && primary[i] < endPos )
                            ++selPrim;
                        if ( primary[i] > endPos )
                            break;
                    }
                    for ( size_t i = 0; i < secondary.GetCount(); ++i )
                    {
                        if ( secondary[i] >= startPos && secondary[i] < endPos )
                            ++selSec;
                        if ( secondary[i] > endPos )
                            break;
                    }

                    m_plotter->SetExtraText( wxString::Format( _T("Selected: %d primary, %d secondary"), selPrim, selSec ) );
                }
            }
            else
            {
                m_isSelecting = false;
                startPos = -1;
                m_plotter->SetSelection( -1, -1 );
                m_plotter->SetExtraText( _T("") );
            }
        }
    }
    event.Skip();
}

//
void TriggerLogGraph::OnDblClick( wxMouseEvent& event )
{
    long x, y;

    x = event.GetX();
    y = event.GetY();

    int w, h;

    GetClientSize( &w, &h );

    if ( x >=0 && x < w - 70 && y > 0 && y < 20 )
    {
        m_isMaximized = !m_isMaximized;
        Maximize( m_isMaximized );
    }
}

// draw time values onto the graph
void TriggerLogGraph::OnMouse( wxMouseEvent& event )
{
    // in continuous drawing don't display values (not real-time!)
    if ( m_showLast && m_isContinuous )
    {
        event.Skip();
        return;
    }

    int mx = event.GetX(), my = event.GetY();

    if ( m_plotter != NULL )
    {
        double xval, yval;

        if ( m_plotter->GetXYValue( mx, my, xval, yval ) )
        {
            if ( m_isSelecting == false )
                SetCursor( wxCURSOR_CROSS );

            m_needstoRedraw = true;

            m_plotter->SetMousePos( event.GetPosition() );
        }
        else
        {
            if ( m_isSelecting == false )
                SetCursor(wxCURSOR_ARROW);

            m_needstoRedraw = true;

            m_plotter->SetMousePos( wxPoint( -1, -1 ) );
        }
    }
    else event.Skip();
}

void TriggerLogGraph::OnV3Button( wxCommandEvent& event )
{
    /// TODO
    /// send event to the manager
    int id = event.GetId();

    if ( id == idTLogGV3StartButton )
    {
        clearData();
        m_v3start->Disable();
        m_v3stop->Enable();
        m_v3showResultButton->Disable();
        m_isContinuous = true;

        m_noserial = false;

        m_tlogmanager->handleButtons( 1 ); // start

        if ( m_timer.IsRunning() == false )
            m_timer.Start( m_timerTime, false );

    }
    if ( id == idTLogGV3StopButton )
    {
        m_v3start->Enable();
        m_v3stop->Disable();
        m_v3showResultButton->Enable();
        m_showLast = false;
        m_isContinuous = false;

        m_tlogmanager->handleButtons( 2 );

        // new ver
        fillTimeLines( m_triggerlog->getTimelines() );

        /*
        // OLD VERSION, DEPRECATED - only prim&sectrig
        fillTimeLines( m_triggerlog->getPrimaryTimeLine(), m_triggerlog->getSecondaryTimeLine() );
        */
    }

    if ( id == idTLogGV3ShowResButton )
    {
        m_tlogmanager->handleButtons( 3 );
    }
}

void TriggerLogGraph::OnButton( wxCommandEvent& event )
{
    // maximize/restore
    if ( event.GetId() == idTLogGMaximizeButton )
    {
        m_isMaximized = ! m_isMaximized;

        Maximize( m_isMaximized );

        event.Skip();

        return;
    }
    // save screenshot
    if ( event.GetId() == idTLogGSavePngButton )
    {
        wxFileDialog dlg( this, _T("Save screenshot"), m_logdir, _T(""), _T("PNG files (*.png)|*.png"),
            wxFD_SAVE | wxFD_OVERWRITE_PROMPT );

        if ( dlg.ShowModal() == wxID_OK )
        {
            wxString fname = dlg.GetPath();

            wxMemoryDC memdc;

            int w, h;

            GetClientSize( &w, &h );

            w -= 40; h -= 130;

            wxImage img( w, h );

            img.SetRGB( wxRect( 0, 0, w, h ), m_bgColor.Red(), m_bgColor.Green(), m_bgColor.Blue() );

            wxBitmap bmp( img );

            memdc.SelectObject( bmp );

            m_plotter->RenderToMemoryDC( &memdc );

            bmp.SaveFile( fname, wxBITMAP_TYPE_PNG );

            memdc.SelectObject( wxNullBitmap );
        }

        return;
    }

    if ( event.GetId() == idTLogGSaveWavButton )
    {
        wxFileDialog dlg( this, _T("Save WAV file"), m_logdir, _T(""), _T("WAV files (*.wav)|*.wav|RAW audio (*.raw)|*.raw|All files (*.*)|*.*"),
            wxFD_SAVE | wxFD_OVERWRITE_PROMPT );

        if ( dlg.ShowModal() == wxID_OK )
        {
            wxString fname = dlg.GetPath();

            wxFileName fn( fname );
            wxString ext = fn.GetExt();
            ext = ext.Upper();

            /*
            // OLD VERSION, DEPRECATED - only prim&sectrig
            DoubleArray & primary = m_primary;
            DoubleArray & secondary = m_secondary;
            */

            DoubleArray & primary = m_timelines[TriggerFrame::CRC_ONLY];
            DoubleArray & secondary = m_timelines[TriggerFrame::SEC_TRIG];

            TriggerLogSaveAudio exporter( primary, secondary );
            exporter.registerObserver( this );

            m_progdlg = new bcProgressDialog( _T("Exporting to audio file"), _T("Please wait") );
            m_progdlg->Show();

            if ( ext == _T("RAW") )
            {
                exporter.saveToRaw( fname );
            }
            else
            {
                if ( ext != _T("WAV"))
                    fname += _T(".WAV");
                exporter.saveToWaw( fname );
            }
        }
    }

    // continue real-time scrolling from end ( in continuous mode )
    if ( event.GetId() == idTLogGSRTButton && m_isContinuous && m_showLast == false )
    {
        m_showLast = true;

        m_max = m_maxvalue;
        m_min = m_max - m_timewindow;

        if ( m_min < 0 )
        {
            m_min = 0;
            m_max = m_timewindow;
        }

        m_needstoRedraw = true;

        Refresh( false );

        return;
    }

    // pause refreshing ( in continuous mode )
    if ( event.GetId() == idTLogGPauseButton && m_isContinuous )
    {
        m_paused = !m_paused;
        ( m_paused ) ? m_pauseButton->SetLabel( _T("Resume") ) : m_pauseButton->SetLabel( _T("Pause") );
    }

    // close frame - titlebar
    if ( event.GetId() == idTLogCloseButton )
    {
        // we need to notify the manager
        m_tlogmanager->handleButtons( 4 );
        Destroy();
    }

    // show / hide help - titlebar
    if ( event.GetId() == idTLogHelpButton )
    {
        ToggleHelpPanel();
        SetFocus();
    }

    // show color settings dlg - update colors
    if ( event.GetId() == idTLogColorButton )
    {
        TriggerLogGraphSettings dlg( this );

        int ret = dlg.ShowModal();

        if ( ret == idTLogColApply || ret == idTLogColSave )
        {
            if ( ret == idTLogColSave )
            {
                SaveSettings();
            }

            m_bgColor = TriggerLogSettingsHelper::tLog_bg;
            m_scopeColor = TriggerLogSettingsHelper::tLog_scope;
            m_textcolor = TriggerLogSettingsHelper::tLog_text;

            m_needstoRedraw = true;

            Refresh( false );
        }

        return;
    }

    event.Skip();
}

// title bar!
void TriggerLogGraph::OnActivate( wxActivateEvent& event )
{
    Refresh( false );
}

void TriggerLogGraph::UpdateScrollBarPosition()
{
    if ( m_isContinuous == false )
    {
        return;
    }
    else
    {
        if ( m_showLast )
        {
            double pos = m_maxvalue - ( m_maxvalue - m_min ) / m_timewindow;
            m_scrollBar->SetScrollbar( (int)pos, (int)m_timewindow, (int)m_maxvalue, (int)m_timewindow );
        }
        else
        {

            double pos = m_cursor - m_scrollBar->GetThumbSize() / 2;
            m_scrollBar->SetScrollbar( (int)pos, (int)m_timewindow, (int)m_maxvalue, (int)m_timewindow );
        }
    }
}

void TriggerLogGraph::add(TriggerFrame& trFrame)
{
    // new ver
    trFrame.convertFrame2Time( m_timelines, m_clockBase, m_prevTime );

    // old ver, deprecated - only prim&sectrig
    //trFrame.convertFrame2Time( m_primary, m_secondary, m_clockBase, m_prevTime );
}

void TriggerLogGraph::OnTriggerLogEvent( TriggerLogEvent& event )
{
    TriggerFrame& tf = event.m_frame;

    // old ver, deprecated - only prim&sectrig
    //tf.convertFrame2Time( m_primary, m_secondary, m_clockBase, m_prevTime );


    // new ver
    tf.convertFrame2Time( m_timelines, m_clockBase, m_prevTime );

    // if previously serial comm. was broken (and message iss blinking)
    // now comm works -> stop blinking, hide error message
    if ( m_noserial )
    {
        m_noserial = false;
        m_commtimer.Stop();
        m_commerrorTxt->SetLabel( _T("") );
        m_noserial = false;
    }
}

void TriggerLogGraph::OnCommTimer( wxTimerEvent& event )
{
    static bool shown = false;

    if ( m_noserial )
    {
        if ( shown == false )
        {
            m_commerrorTxt->SetLabel(_T("Communication error. Check cables\nand the AC/DC adapter"));
            TextDesc desc = m_commerrorTxt->GetLabelDesc();
            desc.Color = wxColor( 180, 0, 0 );
            m_commerrorTxt->SetLabel( desc );
            shown = true;
        }
        else
        {
            shown = false;
            m_commerrorTxt->SetLabel(_T(""));
        }
        Refresh( false );
    }
}

void TriggerLogGraph::OnCommunicationEvent( CommunicationEvent& event )
{
    int id = event.GetId();
    if ( id == CommunicationEvent::PORT_CLOSED )
    {
        wxString msg = _T("The communication port has been closed / the RS232/USB adapter\nhas been unplugged from the PC.\nPlease plug-in the adapter then press the Stop button,\nthen the Start ");
        msg += _T("button to start the logging again");
        wxMessageBox( msg, _T("Information"), wxICON_EXCLAMATION | wxOK );
    }
    else if ( id == CommunicationEvent::COMMUNICATION_ERROR )
    {
        m_noserial = true;
        if ( m_commtimer.IsRunning() == false ) // flashing red message
            m_commtimer.Start( 1000, false );
    }
    else if ( id == CommunicationEvent::COMM_WORX )
    {
        m_noserial = false;
        m_commtimer.Stop();
        m_commerrorTxt->SetLabel( _T("") );
        Refresh( false );
    }
}

void TriggerLogGraph::ToggleHelpPanel()
{
    wxSize size = GetClientSize();

    wxRect helppanelRect = m_helpPanel->GetRect();

    wxPoint panel_pos = helppanelRect.GetTopLeft();

    wxPoint tpos;

    tpos.x = (size.x - helppanelRect.width) / 2;

    bool isShow;

    tpos.y = (size.y - helppanelRect.height) / 2;

    if( m_helpPanel->GetAlpha() ) {

        tpos.y = 0;
        tpos.x = 0;

        isShow = false;

    } else {

        isShow = true;
    }

    m_needstoRedraw = true;

    m_helpPanel->SlidePosition( tpos );
    m_helpPanel->SlideAlpha( isShow );
}

void TriggerLogGraph::DrawStat()
{
    wxString statTxt = _T("Statistics\n");

    Int2IntHash& res = m_triggerlog->getResult();

    size_t receivedCount = m_triggerlog->receivedDataCount();
    statTxt += wxString::Format( _T("Total number of received bytes: %d\n"), receivedCount );

    if ( res.size() > 0 )
    {
        Int2IntHash::const_iterator it = res.begin();

        size_t totalframes = 0;
        size_t otherframes = 0; // none of CRC_ONLY, SEC_TRIG, SMALL_SIZED, CRC_ERROR, REALTIME

        for ( ; it != res.end(); ++it )
        {
            totalframes += it->second;

            int tftype = it->first;

            Logger::logf(LOG_MSG, _T("tftype=%d"), tftype );

            if ( tftype != TriggerFrame::CRC_ONLY && tftype != TriggerFrame::SEC_TRIG && tftype != TriggerFrame::TRFRAME_SMALL_SIZE &&
                tftype != TriggerFrame::TRFRAME_CRC_ERROR && tftype != TriggerFrame::EVENT_REALTIME )
            {
                otherframes += it->second;
            }
        }

        int primary   = res[TriggerFrame::CRC_ONLY];
        int secondary = res[TriggerFrame::SEC_TRIG];
        int smalls = res[TriggerFrame::TRFRAME_SMALL_SIZE];
        int crcerr = res[TriggerFrame::TRFRAME_CRC_ERROR];
        int rtsize = res[TriggerFrame::EVENT_REALTIME];
        int unsupp = res[TriggerFrame::UNSUPPORTED_FORMAT];

        int df = primary + secondary;
        if ( df > 0 )
        {
            double pp = (double)primary * 100.0 / df;
            double sp = (double)secondary * 100.0 / df;

            statTxt += wxString::Format( _T("Total number of frames: %d\n"), totalframes );
            statTxt += wxString::Format( _T("Primary only frame: %d (%.0f%%), primary + secondary trigger frame: %d (%.0f%%)\n"), primary, pp, secondary, sp );

            statTxt += wxString::Format( _T("Unsupported frames: %d, small sized frames: %d\n"), unsupp, smalls );
            statTxt += wxString::Format( _T("Frames with CRC-error:%d, RT frames=%d, other frames: %d\n"), crcerr, rtsize, otherframes );
        }
        else
        {
            statTxt += wxString::Format( _T("Total number of frames: %d\n"), totalframes );
            statTxt += wxString::Format( _T("Number of real-time data frames: %d\n"), rtsize );
            statTxt += wxString::Format( _T("Small-sized frames: %d, other frames: %d\n"), smalls, otherframes );
            statTxt += wxString::Format( _T("Frames with CRC-error: %d, unsupported frames: %d"), crcerr, unsupp );
        }
    }
    else
    {
        statTxt += _T("No statistics available");
    }

    TextDesc desc;
    desc.Text = statTxt;
    desc.Color = m_textcolor;
    m_statistics->WriteLabel(desc);
}

void TriggerLogGraph::OnTimer( wxTimerEvent& event )
{
    if ( m_paused )return;

    Int2DoubleArrayHash::iterator it = m_timelines.begin();

    for ( ; it != m_timelines.end(); ++it )
    {
        double tmax = 0.0;

        if ( it->second.GetCount() )
            tmax = it->second[it->second.GetCount() - 1];

        m_maxvalue = ( m_maxvalue > tmax ) ? m_maxvalue : tmax;
    }

    size_t receivedCount = m_triggerlog->receivedDataCount();

    if(m_prevReceivedData != (int)receivedCount){
         DrawStat();
         m_prevReceivedData = (int)receivedCount;
        Refresh( false );
    }

    if ( m_maxvalue == 0 ){
        return;
    }

    m_totalTime->SetLabel( wxString::Format( _T("Total time: %.2f"), m_maxvalue ), 0 );



    if ( m_isContinuous )
    {
        if ( m_showLast )
        {
            m_max = m_maxvalue;
            m_min = m_max - m_timewindow;
            if ( m_min < 0 )
            {
                m_min = 0;
                m_max = m_timewindow;
            }
            m_cursor = (unsigned long)( m_max + m_min ) / 2;
        }
    }
    else
    {
        m_max = m_maxvalue;
        m_cursor = (unsigned long)( m_min + m_max ) / 2;
        m_timer.Stop();
    }

    m_needstoRedraw = true;
    Refresh( false );

    UpdateScrollBarPosition();
}

void TriggerLogGraph::notify( int i )
{
    if ( m_progdlg )
        wxSafeYield( m_progdlg );

    if ( i < 100 )
    {
        if ( m_progdlg != NULL )
        {
            m_progdlg->Update( i );
        }
    }
    if ( i >= 100 )
    {
        if ( m_progdlg != NULL )
        {
            m_progdlg->Destroy();
        }
        m_progdlg = NULL;
    }
}
