#include "TriggerLogGraphSettings.h"

#include <wx/spinctrl.h>
#include <wx/colordlg.h>

#include "TriggerLogSettingsHelper.h"

int idTLogGBGColor  = wxNewId();
int idTLogTColor    = wxNewId();
int idTLogScColor   = wxNewId();
int idTLogPColor    = wxNewId();
int idTLogSColor    = wxNewId();
int idTlogColDefault= wxNewId();
int idTLogColSave   = wxNewId();
int idTLogColApply  = wxNewId();
int idTLogColCancel = wxNewId();

BEGIN_EVENT_TABLE( TriggerLogGraphSettings, wxDialog )
    EVT_BUTTON( idTLogGBGColor, TriggerLogGraphSettings::OnButton )
    EVT_BUTTON( idTLogScColor, TriggerLogGraphSettings::OnButton )
    EVT_BUTTON( idTLogTColor, TriggerLogGraphSettings::OnButton )
    EVT_BUTTON( idTLogColSave, TriggerLogGraphSettings::OnButton )
    EVT_BUTTON( idTLogColApply, TriggerLogGraphSettings::OnButton )
    EVT_BUTTON( idTLogColCancel, TriggerLogGraphSettings::OnButton )
    EVT_BUTTON( idTlogColDefault, TriggerLogGraphSettings::OnButton )
END_EVENT_TABLE()

TriggerLogGraphSettings::TriggerLogGraphSettings( wxWindow* parent ) :
        wxDialog( parent, wxID_ANY, _T("Triggerlog Graph - Color settings"), wxDefaultPosition, wxDefaultSize )
{
    size_t colornum = TriggerLogSettingsHelper::tLogEventsOrder.GetCount();

    m_triggerColors = new wxTextCtrl*[colornum];

    m_offsets = new wxSpinCtrl*[colornum];

    //------------------------------------------

    wxSizer* mainsizer = new wxBoxSizer( wxVERTICAL );

    wxFlexGridSizer* fs = new wxFlexGridSizer( 4 );

    wxStaticText* txt = new wxStaticText( this, -1, _T("Background color") );
    m_bgcolor = new wxTextCtrl( this, -1, _T(""), wxDefaultPosition, wxSize( 60, 18 ), wxTE_READONLY );
    m_bgcolor->SetBackgroundColour( TriggerLogSettingsHelper::tLog_bg );
    wxButton* button = new wxButton( this, idTLogGBGColor, _T("Change") );

    fs->Add( txt, 0, wxALL | wxEXPAND, 3 ); fs->Add( m_bgcolor, 0, wxALL | wxEXPAND, 3 ); fs->Add( button, 0, wxALL | wxEXPAND, 3 );
    fs->AddSpacer( 1 );

    //
    txt = new wxStaticText( this, -1, _T("Scope color") );
    m_scopecolor = new wxTextCtrl( this, -1, _T(""), wxDefaultPosition, wxSize( 60, 18 ), wxTE_READONLY );
    m_scopecolor->SetBackgroundColour( TriggerLogSettingsHelper::tLog_scope );
    button = new wxButton( this, idTLogScColor, _T("Change") );

    fs->Add( txt, 0, wxALL | wxEXPAND, 3 ); fs->Add( m_scopecolor, 0, wxALL | wxEXPAND, 3 ); fs->Add( button, 0, wxALL | wxEXPAND, 3 );
    fs->AddSpacer( 1 );

    //
    txt = new wxStaticText( this, -1, _T("Text color") );
    m_textcolor = new wxTextCtrl( this, -1, _T(""), wxDefaultPosition, wxSize( 60, 18 ), wxTE_READONLY );
    m_textcolor->SetBackgroundColour( TriggerLogSettingsHelper::tLog_text );
    button = new wxButton( this, idTLogTColor, _T("Change") );

    fs->Add( txt, 0, wxALL | wxEXPAND, 3 ); fs->Add( m_textcolor, 0, wxALL | wxEXPAND, 3 ); fs->Add( button, 0, wxALL | wxEXPAND, 3 );

    fs->Add( new wxStaticText( this, wxID_ANY, _T("Vertical offset\n(in px)") ), 0, wxALL | wxEXPAND | wxALIGN_CENTER, 3 );

    //

    // create the colored areas
    for ( size_t i = 0; i < colornum; ++i )
    {
        int event_id = TriggerLogSettingsHelper::tLogEventsOrder[i];

        wxString name = TriggerLogSettingsHelper::tLogNames[event_id];

        wxString event_name = name.Upper().Left(1) + name.Mid(1);

        txt = new wxStaticText( this, -1, event_name + _T(" color") );

        m_triggerColors[i] = new wxTextCtrl( this, -1, _T(""), wxDefaultPosition, wxSize( 60, 18 ), wxTE_READONLY );
        m_triggerColors[i]->SetBackgroundColour( TriggerLogSettingsHelper::tLogColors[event_id] );

        int buttonid = wxNewId();

        m_colorButtonIds.Add( buttonid );

        button = new wxButton( this, buttonid, _T("Change") );

        fs->Add( txt, 0, wxALL | wxEXPAND, 3 ); fs->Add( m_triggerColors[i], 0, wxALL | wxEXPAND, 3 );
        fs->Add( button, 0 , wxALL | wxEXPAND, 3 );

        wxString value = wxString::Format(_T("%.0f"), TriggerLogSettingsHelper::tLogVerticalOffsets[event_id] );
        m_offsets[i] = new wxSpinCtrl( this, wxID_ANY, value, wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, -100, 100,
            (int)( TriggerLogSettingsHelper::tLogVerticalOffsets[event_id] ) );
        fs->Add( m_offsets[i], 0, wxALL, 3 );

        Connect( buttonid, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( TriggerLogGraphSettings::OnButton ) );
    }

    mainsizer->Add( fs, 0, wxALL | wxEXPAND, 5 );

    wxSizer* bsizer = new wxBoxSizer( wxHORIZONTAL );
    button = new wxButton( this, idTLogColSave, _T("Save && close") );  bsizer->Add( button, 0, wxALL | wxEXPAND, 5 );
    button = new wxButton( this, idTLogColApply, _T("Apply && close") );  bsizer->Add( button, 0, wxALL | wxEXPAND, 5 );
    button = new wxButton( this, idTLogColCancel, _T("Cancel") );  bsizer->Add( button, 0, wxALL | wxEXPAND, 5 );
    button = new wxButton( this, idTlogColDefault, _T("Default values") ); bsizer->Add( button, 0, wxALL | wxEXPAND, 5 );

    mainsizer->Add( bsizer, 0, wxALL | wxEXPAND | wxALIGN_CENTER, 5 );

    SetSizer( mainsizer );
    mainsizer->Fit( this );
    mainsizer->SetSizeHints( this );
}

TriggerLogGraphSettings::~TriggerLogGraphSettings()
{
    delete[] m_triggerColors;
    delete[] m_offsets;
}

void TriggerLogGraphSettings::OnButton( wxCommandEvent& evt )
{
    const int id = evt.GetId();

    // change colors
    if ( id == idTLogGBGColor || id == idTLogScColor || id == idTLogTColor || ( m_colorButtonIds.Index( id ) != wxNOT_FOUND ) )
    {
        wxColor c;

        if ( id == idTLogGBGColor )
            c = m_bgcolor->GetBackgroundColour();

        else if ( id == idTLogScColor )
            c = m_scopecolor->GetBackgroundColour();

        else if ( id == idTLogTColor )
            c = m_textcolor->GetBackgroundColour();

        else {
            int buttonnr = m_colorButtonIds.Index( id );
            c = m_triggerColors[buttonnr]->GetBackgroundColour();
        }

        wxColourData cdata;
        cdata.SetColour( c );

        wxColourDialog cdlg( this, &cdata );

        if ( cdlg.ShowModal() == wxID_OK )
        {

            c = cdlg.GetColourData().GetColour();

            if ( id == idTLogGBGColor )
                m_bgcolor->SetBackgroundColour( c );

            else if ( id == idTLogScColor )
                m_scopecolor->SetBackgroundColour( c );

            else if ( id == idTLogTColor )
                m_textcolor->SetBackgroundColour( c );

            else {
                int buttonnr = m_colorButtonIds.Index( id );
                m_triggerColors[buttonnr]->SetBackgroundColour( c );
            }

            Refresh( false );

        }
        return;
    }

    // close dialog
    if ( id == idTLogColApply || id == idTLogColSave || id == idTLogColCancel )
    {
        if ( id != idTLogColCancel )
        {
            UpdateSettings();
        }

        EndModal( id );
        return;
    }

    if ( id == idTlogColDefault )
    {
        TriggerLogSettingsHelper::setDefaults();

        m_bgcolor->SetBackgroundColour( TriggerLogSettingsHelper::tLog_bg );
        m_scopecolor->SetBackgroundColour( TriggerLogSettingsHelper::tLog_scope );
        m_textcolor->SetBackgroundColour( TriggerLogSettingsHelper::tLog_text );

        for ( size_t i = 0; i < m_colorButtonIds.GetCount(); ++i )
        {
            int event_id = TriggerLogSettingsHelper::tLogEventsOrder[i];

            m_triggerColors[i]->SetBackgroundColour( TriggerLogSettingsHelper::tLogColors[event_id] );
            m_offsets[i]->SetValue( (int) TriggerLogSettingsHelper::tLogVerticalOffsets[event_id] );
        }

        Refresh( false );

        return;
    }
}

void TriggerLogGraphSettings::UpdateSettings() const
{
    /// TODO
    TriggerLogSettingsHelper::tLog_bg = m_bgcolor->GetBackgroundColour();
    TriggerLogSettingsHelper::tLog_scope = m_scopecolor->GetBackgroundColour();
    TriggerLogSettingsHelper::tLog_text = m_textcolor->GetBackgroundColour();

    for ( size_t i = 0; i < TriggerLogSettingsHelper::tLogEventsOrder.GetCount(); ++i )
    {
        int event_id = TriggerLogSettingsHelper::tLogEventsOrder[i];

         TriggerLogSettingsHelper::tLogColors[event_id] = m_triggerColors[i]->GetBackgroundColour();
         TriggerLogSettingsHelper::tLogVerticalOffsets[event_id] = m_offsets[i]->GetValue();
    }

}
