#include "TriggerLogImg.h"

#include "../logger.h"

#include "../vemsprotocol/VemsLogHeaderParser.h"

#include <wx/filename.h>

TriggerLogImg::TriggerLogImg(const wxString& fileName)
{
    const size_t bufSize = 1024 * 512; // 512kB buffer for read in logfile

    if ( wxFileExists( fileName ) )
    {
        // open file and fill m_rawtriggerdata
        wxFile f( fileName, wxFile::read );

        parseHeader( f, fileName );

        unsigned char buffer[bufSize];

        while ( ! f.Eof() )
        {
            memset( buffer, 0, bufSize );

            size_t nread = f.Read( &buffer, sizeof(buffer) );

            if ( nread != sizeof( buffer ) )
            {
                fillBuffer( buffer, nread );
                break;
            }

            fillBuffer( buffer, sizeof( buffer ) );
        }
        f.Close();

        Logger::logf(LOG_MSG, _T("TriggerLogImg::TriggerLogImg() no of bytes read from file: %d"), m_rawTriggerData.GetCount() );
        fillTriggerData();
    }
    else
        Logger::logf(LOG_MSG, _T("TriggerLogImg::TriggerLogImg() - file does not exist:") + fileName );
}

void TriggerLogImg::parseHeader( wxFile& f, const wxString& fname )
{
    VemsLogHeaderParser parser( f );

    StringHash header = parser.getHeaderHash();

    m_fwVersion = _T("");
    m_creationTime = _T("");
    m_serialNum = _T("");

    StringHash::iterator it = header.find(_T("Version") );
    if ( it == header.end() )
    {
        Logger::logf(LOG_MSG, _T("TriggerLogImg::parseHeader() - no valid header found, use log as raw byte stream"));
        Logger::logf(LOG_MSG, _T("TriggerLogImg::parseHeader() - read data from beginning of the file") );

        f.Seek( 0 );
    }
    else
    {
        m_fwVersion = header[_T("Version")];
        m_creationTime = header[_T("CurrentTime")];

        if ( m_creationTime == wxEmptyString )
        {
            wxFileName fn( fname );
            const wxDateTime& dt = fn.GetModificationTime();
            m_creationTime = dt.FormatISODate() + _T(" ") + dt.FormatISOTime();
        }

        m_serialNum = header[_T("SerialNumber")];

        Logger::logf(LOG_MSG, _T("TriggerLogImg::parseHeader() - TriggerLog header found"));
        Logger::logf(LOG_MSG, _T("Creation date =") + m_creationTime + _T(", serial# = ") +
            m_serialNum + _T(", firmware version = ") + m_fwVersion );
        Logger::logf(LOG_MSG, _T("TriggerLogImg::parseHeader() - read data from pos 512"));
    }
}

inline void TriggerLogImg::fillBuffer( unsigned char* buffer, size_t bufsize )
{
    for ( size_t i = 0; i < bufsize; ++i )
    {
        m_rawTriggerData.Add( buffer[i] );
    }
}
