#include "CarParameters.h"
#include "../logger.h"
#include "../common/types/DoubleHash.h"

CarParameters ::CarParameters ()
:m_wheelradius(0)
,m_aNull(0)
,m_fNull(0)
,m_mass(0)
,m_drivetrain(1)
{
    //ctor
}
CarParameters::CarParameters(CarParameters& in)
{
    m_driveTrainH = new DoubleHash();
    DoubleHash  *dH = in.getDrivetrainH();

    DoubleHash::iterator it;
    for(it = dH->begin(); it != dH->end();it++){
        wxString key = it->first;
        double value = it->second;
        (*m_driveTrainH)[key] = value;
    }

    m_wheelradius = in.getWheelRadius();
    m_aNull = in.getaNull();
    m_fNull = in.getfNull();
    m_mass = in.getMass();
    m_drivetrain = in.getDrivetrain();
}

CarParameters ::~CarParameters ()
{
    //dtor
    delete m_driveTrainH;
}


CarParameters *CarParameters::Clone()
{
    return new CarParameters(*this);
}
/** @brief getMass
  *
  * @todo: document this function
  */
double CarParameters::getMass()
{
    return m_mass;
}

/** @brief getm_fNull
  *
  * @todo: document this function
  */
double CarParameters::getfNull()
{
    return m_fNull;
}

/** @brief getaNull
  *
  * @todo: document this function
  */
double CarParameters::getaNull()
{
    return m_aNull;
}

/** @brief getWheelRadius
  *
  * @todo: document this function
  */
double CarParameters::getWheelRadius()
{
    return m_wheelradius;
}

/** @brief getDrivetrain
  *
  * @todo: document this function
  */
double CarParameters::getDrivetrain()
{
    return m_drivetrain;
}

DoubleHash * CarParameters::getDrivetrainH()
{
    return m_driveTrainH;
}
/** @brief setMass
  *
  * @todo: document this function
  */
void CarParameters::setMass(double value)
{
    m_mass = value;
}

/** @brief setm_fNull
  *
  * @todo: document this function
  */
void CarParameters::setfNull(double value)
{
    m_fNull = value;
}

/** @brief setaNull
  *
  * @todo: document this function
  */
void CarParameters::setaNull(double value)
{
    m_aNull = value;
}

/** @brief setWheelRadius
  *
  * @todo: document this function
  */
void CarParameters::setWheelRadius(double value)
{
    m_wheelradius = value;
}

void CarParameters::setdrivetrainH(DoubleHash *drivetrainH)
{
        m_driveTrainH = drivetrainH;
}

void CarParameters::setCurrentDrivetrain(const wxString& gear){
    if( m_driveTrainH->find(gear) == m_driveTrainH->end() ){
        m_drivetrain =1;
        return;
    }
    m_drivetrain = (*m_driveTrainH)[gear];
}

double CarParameters::getTorque(double drpmPerdt)
{
    double rpm2v = 1.0/60.0/m_drivetrain * 2.0 * 3.14 * m_wheelradius;
    double wheeldVPerdt =  drpmPerdt * rpm2v;
    double wheelForce = m_mass * (wheeldVPerdt + m_aNull) + m_fNull;
    double wheelTorque = wheelForce * m_wheelradius;
    double engineTorque = wheelTorque / m_drivetrain;

    return engineTorque;
}
void CarParameters::print()
{
     Logger::logf(LOG_MSG, _T("--------------CarParameters::print()--------------------"));
    Logger::logf(LOG_MSG, _T("wheelradius= %f, aNull=%f, fNull= %f, mass= %f ")

                    ,m_wheelradius
                    ,m_aNull
                    ,m_fNull
                    ,m_mass);

    DoubleHash::iterator it;
    for(it = m_driveTrainH->begin(); it != m_driveTrainH->end(); it++){
        wxString key = it->first;
        double value = it->second;
        Logger::logf(LOG_MSG, _T("key = ")+key+_T("value = %f"),value );
    }

}
