#include "RacesCache.h"
#include <wx/string.h>

#include <wx/fileconf.h>

#include <wx/file.h>
#include <wx/wfstream.h>
#include <wx/filename.h>

#if !defined(WIN32)
#include <hl_hashwrapper.h>
#include <hl_md5wrapper.h>
#endif

#include <string>
#include <iostream>

#include "../logger.h"
#include "../DirHelper.h"


wxString RacesCache::m_configFile = wxString(_T("racesCache.cfg") );
wxConfig * RacesCache::m_config;

RacesCache::RacesCache()
{
    //ctor
}

RacesCache::~RacesCache()
{
    //dtor
}

wxString RacesCache::getConfigFilePath()
{
    return DirHelper::GetPath(dCONFIG) + m_configFile;
}

void RacesCache::initConfig()
{
   const wxString fileName = getConfigFilePath();
	//Logger::log(LOG_MSG,fileName);

    checkAndCreateFile(fileName);

    wxFileInputStream * fis = new wxFileInputStream(fileName);
    m_config = (wxConfig *) new wxFileConfig(  *fis,  wxConvUTF8);
}

void RacesCache::save()
{
    wxFileConfig * fileConfig  = (wxFileConfig *) m_config;

    const wxString fileName = getConfigFilePath();
    wxOutputStream * fos = (wxOutputStream *) new wxFileOutputStream(fileName);
    fileConfig->Save(*fos, wxConvUTF8);
}

void RacesCache::checkAndCreateFile(const wxString& file)
{
    bool isExist = wxFileName::FileExists(file);
    if(! isExist){
        wxFile tmpFile;
        tmpFile.Create(file);
        tmpFile.Close();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
int RacesCache::getGearNumber(const wxString& fileName, int raceNumber)
{
    //Logger::logf(LOG_MSG,_T("RacesCache::gettGearNumber filename3 ")+ fileName +_T("\n") );
    wxString oldPath = m_config->GetPath();

    wxString checksum = getFileCheckSum(fileName);

    bool isExist = m_config->HasGroup(checksum);
    if( !isExist ){

            return 1;
    }

    m_config->SetPath(checksum);
    long value = 1;
    wxString raceNumStr;
    raceNumStr << raceNumber;
    // bool res = m_config->Read(raceNumStr, &value);
    m_config->Read(raceNumStr, &value);

    m_config->SetPath(oldPath);
    return value;
}

void RacesCache::setGearNumber(const wxString& fileName, int raceNumber, int gearNumber)
{
    wxString oldPath = m_config->GetPath();
    //Logger::logf(LOG_MSG,_T("RacesCache::setGearNumber filename1 ")+ fileName +_T("\n") );
    wxString checksum = getFileCheckSum(fileName);
    m_config->SetPath(checksum);
    wxString raceNumStr;
    raceNumStr << raceNumber;
    // bool res = m_config->Write(raceNumStr, gearNumber);
    m_config->Write(raceNumStr, gearNumber);

    m_config->SetPath(oldPath);
}

/*
CarParameters* RacesCache::getCarParams()
{

    //double drivetrain = getSessionTorqueParam(_T("drivetrain"), 2.13 * 4.11 );

    CarParameters *carParams = new CarParameters();

    double wheelRadius =  getSessionTorqueParam(_T("wheelRadius"), 0.3);
    double aNull = getSessionTorqueParam(_T("aNull"),0.03 * 9.81); //0.03 * 9.81;
    double fNull = getSessionTorqueParam(_T("fNull"), 500.0); //500.0;
    double mass = getSessionTorqueParam(_T("mass"), 1600.0);

   carParams->setWheelRadius(wheelRadius);
   carParams->setaNull(aNull);
   carParams->setfNull(fNull);
   carParams->setMass(mass);

    DoubleHash * drivetrainH = getSessionTorqueDriveTrain();
    carParams->setdrivetrainH(drivetrainH);
    carParams->print();
    return carParams;
    return 1;
}
*/
void RacesCache::test()
{
    /*
     * creating a wrapper object
     */
#if !defined(WIN32)
    hashwrapper *myWrapper = new md5wrapper();

    /*
     * create a hash from a string
     */
    std::string hash1 = myWrapper->getHashFromString("Hello World");

   // wxString tmp;
    //tmp <<  hash1.c_str();
    //std::cout << hash1;
    wxString tmp(hash1.c_str(), wxConvUTF8);
    //wxString(const wchar_t* psz, wxMBConv& conv, size_t nLength = wxSTRING_MAXLEN);

    //prinf("RacesCache::test()1: %s", hash1.c_str());
    Logger::logf(LOG_MSG,_T("RacesCache::test()2: ")+ tmp );
#endif
}

wxString RacesCache::checksum(const wxString& path)
{
    Logger::log(LOG_MSG,_T("---------checksum----------------")  );
    int length = 0;
    int n = 0;
    std::string temp;
    length = path.Length();

    for(n=0; n < length; n++){

        char a = (path.GetChar(n));
        temp.push_back(a);

    }

    std::cout << temp;
#if !defined(WIN32)
    hashwrapper *myWrapper = new md5wrapper();
    std::string hash2 = myWrapper->getHashFromString(temp);

    wxString tmp(hash2.c_str(), wxConvUTF8);

    Logger::logf(LOG_MSG,_T("RacesCache::checksum(): ")+ tmp);
#else
    wxString tmp;
#endif
    return tmp;

}

wxString RacesCache::getFileCheckSum(const wxString& fileName)
{
     //wxString dummyfileName = _T("dummy");
   // Logger::logf(LOG_MSG,_T("------------------------RacesCache::getFileCheckSum file2 ")+ fileName+_T("\n") );
   // Logger::log(LOG_MSG,_T("---------RacesCache::getFileCheckSum()----------------\n")  );
    string temp;

    temp = wxString2stdSrtig(fileName);
    //std::cout << temp;
     wxString tmp;
#if !defined(WIN32)
    hashwrapper *myWrapper = new md5wrapper();
    std::string hash2 = myWrapper->getHashFromFile(temp);
    tmp = stdSrtig2wxString(hash2);
   // Logger::logf(LOG_MSG,_T("getFileCheckSum(): tmp \n")+ tmp);
#endif
    return tmp;


}

wxString RacesCache::stdSrtig2wxString(const string& str)
{

    wxString temp(str.c_str(), wxConvUTF8);
    return temp;

}

string RacesCache::wxString2stdSrtig(const wxString& fileName)
{
    int length = 0;
    int n = 0;
    string temp;
    length = fileName.Length();

    for(n=0; n < length; n++){
        char a = (fileName.GetChar(n));
        temp.push_back(a);
    }
    return temp;

}

