#include "raceloganalyser.h"
#include "CarConfigHelper.h"
#include "CarParameters.h"
#include "RacesCache.h"
#include "../DirHelper.h"
#include "../logger.h"

#include <wx/textfile.h>
#include <wx/arrimpl.cpp>


RaceLogAnalyser::RaceLogAnalyser()
{
    //ctor
    rpmBinArray.Add(0);
    rpmBinArray.Add(1250);
    rpmBinArray.Add(1750);
    rpmBinArray.Add(2250);
    rpmBinArray.Add(2750);
    rpmBinArray.Add(3250);
    rpmBinArray.Add(3750);
    rpmBinArray.Add(4250);
    rpmBinArray.Add(4750);
    rpmBinArray.Add(5250);
    rpmBinArray.Add(5750);
    rpmBinArray.Add(6250);
    rpmBinArray.Add(6750);
    rpmBinArray.Add(7250);


    displayedValueA.Add(_T("Sample count:"));
    //csvRow.Add(_T("Start time:;"));
    //csvRow.Add(_T("End time:;"));
    displayedValueA.Add(_T("(filtered) drmp/dt:"));
    displayedValueA.Add(_T("avg map:"));
    displayedValueA.Add(_T("drmp/dt/avgmap:"));
    displayedValueA.Add(_T("lambda:"));
    displayedValueA.Add(_T("EGT:"));
    displayedValueA.Add(_T("Spark:"));
    displayedValueA.Add(_T("Engine torque:"));
    displayedValueA.Add(_T("raw drmp/dt:"));

    m_carParams = CarConfigHelper::getCarParams();
    m_carParamsH = new CarParametersH();

    m_racesCache = new RacesCache();
    m_racesCache->initConfig();

}

RaceLogAnalyser::~RaceLogAnalyser()
{
    m_racesCache->save();
    delete m_racesCache;
    //dtor
    delete m_carParams;
    delete m_carParamsH;

    WX_CLEAR_ARRAY(raceBin);
    WX_CLEAR_ARRAY(raceArray);
}

void RaceLogAnalyser::setVemsDataLogDesc(VEMSDataLogDesc * vemsDataLogDesc)
{
    m_vemsDataLogDesc = vemsDataLogDesc;

}

void RaceLogAnalyser::print()
{
//print columns
    wxArrayString *columnArray = m_vemsDataLogDesc->getVEMSDataLogHeader();
    for(uint32_t i = 0; i < columnArray->Count(); i++){

        wxString columnName =  columnArray->Item(i);
        Logger::log(LOG_MSG, _T("ColumnName: ")+ columnName);
    }

    int timeIdx = columnArray->Index(_("Time"));
    int rpmIdx = columnArray->Index(_("RPM"));
    int mapIdx = columnArray->Index(_("MAP"));

    if(timeIdx == wxNOT_FOUND) return;
    if(rpmIdx == wxNOT_FOUND) return;
    if(mapIdx == wxNOT_FOUND) return;

    Logger::log(LOG_MSG, wxString::Format(_T("%d,%d,%d"), timeIdx, rpmIdx, mapIdx) );

//0: time
//2: rpm
//3: map

    IntArray * intArray =  new IntArray();
//    double deltaRPMOld = 0.0;

    ECULogItemArray * ecuLogItemArray = m_vemsDataLogDesc->getVEMSDataLog();
	// traverse the log to find races (== monotonously rising RPM, eg. from gearshift to gearshift)
	double deltaRPM_LIMIT = -50;

    for(uint32_t i = 0; i < ecuLogItemArray->Count(); i++){
        DoubleArray * rowArray = ecuLogItemArray->Item(i);
        int size = rowArray->Count();
        //Logger::logf(LOG_MSG, wxString::Format(_T("ROW size: %d"),  size) );
        if(size == 0){
            Logger::log(LOG_MSG, _T("size == 0"));
            continue;
        }

        double timeV = rowArray->Item(timeIdx);
        double rpmV = rowArray->Item(rpmIdx);
//        double mapV = rowArray->Item(mapIdx);

        //if(timeV > 756.0 && timeV < 780.0 ){
        //    Logger::logf(LOG_MSG, wxString::Format(_T("Time: %f Rpm: %f, Map: %f"), timeV, rpmV, mapV) );
        //}

        if(i < 1){
            newFileIndexA.Add(0);
             continue;
        }
        //Logger::log(LOG_MSG, _T("1--------------"));

        DoubleArray * rowArrayPrev = ecuLogItemArray->Item(i-1);
        //Logger::log(LOG_MSG, _T("2--------------"));
        double timeVprev = rowArrayPrev->Item(timeIdx);
        // Logger::log(LOG_MSG, _T("3--------------"));
        if( (timeV - timeVprev) < 0){
            //TODO: new File
             Logger::log(LOG_MSG, _T("New LOG FILE:--------------"));
             newFileIndexA.Add(i);
            continue;
        }
       // Logger::log(LOG_MSG, _T("5--------------"));
        double rpmVprev = rowArrayPrev->Item(rpmIdx);
        double deltaRPM = rpmV - rpmVprev;
       //  if(timeV > 756.0 && timeV < 780.0 )Logger::logf(LOG_MSG, wxString::Format(_T("Time: %f Prev rpm: %f, Curr: %f"), timeV, rpmVprev, rpmV) );
        if(deltaRPM < deltaRPM_LIMIT){ // end of race found

            //if(deltaRPM  != deltaRPMOld){
            // neglect any race that is too short
            double dRPM =getDeltaRPM(intArray);
            // if(timeV > 756.0 && timeV < 780.0 ) Logger::logf(LOG_MSG, wxString::Format(_T("Time: %f Delta rpm: %f dRPM: %f"), timeV, deltaRPM, dRPM) );
                if(dRPM > 1500 ){
                     raceArray.Add(intArray);
                }else{
                    delete intArray;
                }

                intArray = new IntArray();

                //deltaRPMOld = deltaRPM;
            //}

            deltaRPM_LIMIT = -50;
        }else{ // still the same race (RPM climbing), add to intArray for later evaluation
		// do we really need this intArray just to store numbers like 6,7,8,9,10,11 ?????
		// would the start,end (6,11) be enough ?
            if(deltaRPM < 0){deltaRPM_LIMIT = 0;}
        }
        intArray->Add(i);

//        double value = rowArray->Item(0);
        //Logger::log(LOG_MSG, wxString::Format(_T("Value col0: %f"), value) );
    }

    Logger::logf(LOG_MSG, wxString::Format(_T("RaceLogAnalyser::print() final result:")) );
    //print race Numbers
    for(uint32_t i = 0; i< raceArray.Count(); i++){
        intArray = raceArray.Item(i);
        Logger::log(LOG_MSG, wxString::Format(_T("RaceNumber: %d, recordCount %d"), i, intArray->Count()));
    }



}

void RaceLogAnalyser::analyzeRaces()
{
    IntArray *intArray;
    for(uint32_t i = 0; i< raceArray.Count(); i++){
        intArray = raceArray.Item(i);
        analyzeRaces(i, intArray);
    }
}
//intArray values are log indexes (ecuLogItemArray) which belongs to that part. race
void RaceLogAnalyser::analyzeRaces(int raceNum, IntArray* intArray)
{
   // Logger::log(LOG_MSG, _T("VEMSLogParser::analyzeRaces()--------------"));
    RaceArray *raceBinArray = new RaceArray();

    //fill raceBinArray
    for(uint32_t x = 0; x <rpmBinArray.Count()-1; x++){
            raceBinArray->Add( new IntArray() );
    }

    ECULogItemArray * ecuLogItemArray = m_vemsDataLogDesc->getVEMSDataLog();
    for(uint32_t i = 0; i< intArray->Count(); i++){
        int logIdx = intArray->Item(i);
        //Logger::log(LOG_MSG, _T("VEMSLogParser::analyzeRaces()2--------------"));
        DoubleArray * rowArray = ecuLogItemArray->Item(logIdx);
        int binIndex = analyzeRaces(rowArray);
        wxString tmp = wxString::Format(_T("binIndex: %d"), binIndex);
        if(binIndex < 0){
            continue;
        }
        //Logger::log(LOG_MSG, _T("VEMSLogParser::analyzeRaces()3--------------")+tmp);

        IntArray * binArray = raceBinArray->Item(binIndex);
        //if(binArray = NULL) continue;
        //Logger::log(LOG_MSG, _T("VEMSLogParser::analyzeRaces()4--------------"));
        binArray->Add(logIdx);
    }

    //Logger::log(LOG_MSG, _T("VEMSLogParser::analyzeRaces()6--------------"));
    raceBin.Add(raceBinArray);//raceBin.Item(raceNum);
    //Logger::log(LOG_MSG, _T("VEMSLogParser::analyzeRaces()7--------------"));


}

int RaceLogAnalyser::analyzeRaces(DoubleArray * rowArray)
{
     wxArrayString *columnArray = m_vemsDataLogDesc->getVEMSDataLogHeader();
    //for(int i = 0; i< rowArray.Count(); i++){
    int rpmIdx = columnArray->Index(_("RPM"));
    //double timeV = rowArray->Item(timeIdx);
    int rpmV = (int)(rowArray->Item(rpmIdx));
    int binIndex = getBinIndex(rpmV);

    return binIndex;
    /*
    IntArray = raceBinArray.Item(m_raceNum);
    IntArray logBinIndexArray
    binrowArray.Item(binIndex);
    */
}

// returns index where rpmBinArray.Item( index ) < rpmV < rpmBinArray.Item( index+1 )
int RaceLogAnalyser::getBinIndex(int rpmV)
{
    for(uint32_t i = 0; i < rpmBinArray.Count()-1; i++)
    {
        int minValue = rpmBinArray.Item(i);
        int maxValue = rpmBinArray.Item(i+1);

        if(minValue < rpmV && maxValue > rpmV){
                return i;
        }
    }
    return -1;
}

void RaceLogAnalyser::setEmptyValues(ArrayOfArrayString * displayedValuesTable)
{
    for(uint32_t i = 0; i < displayedValuesTable->Count(); i++){
        wxArrayString *row = displayedValuesTable->Item(i);
         row->Add( _T("--") );
    }

}

bool RaceLogAnalyser::isBinValid(RaceArray *raceBinArray, int x, int minCount)
{
    //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-22--------------"));
    if(x < 0 || x > (int)(raceBinArray->Count()-1) ) return false;

    IntArray *binRecord = raceBinArray->Item(x);
            //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-3--------------"));
    if(binRecord == NULL || (int)(binRecord->Count()) < minCount){
        return false;
    }
    return true;
}

ArrayOfArrayString * RaceLogAnalyser::getRacesTable()
{
    ArrayOfArrayString *racesTable = new ArrayOfArrayString();

    // create header
    wxArrayString *colRow = new wxArrayString();
    colRow->Add( _T("Race nums") );
    for(uint32_t x = 0; x < rpmBinArray.Count()-1; x++){
            int rpmMinV = rpmBinArray.Item(x);
            int rpmMaxV = rpmBinArray.Item(x+1);
            colRow->Add( wxString::Format(_T("(%d-%d)"), rpmMinV, rpmMaxV) );
    }
    Logger::log(LOG_MSG, _T("RaceLogAnalyser::getRacesTable()"));
    racesTable->Add(colRow);


    wxArrayString *columnArray = m_vemsDataLogDesc->getVEMSDataLogHeader();
    ECULogItemArray * ecuLogItemArray = m_vemsDataLogDesc->getVEMSDataLog();
	// should we move these 3 vars to object variables ? used pretty often
    int timeIdx = columnArray->Index(_T("Time"));
    int rpmIdx = columnArray->Index(_T("RPM"));
//    int mapIdx = columnArray->Index(_T("MAP"));

//    int lambdaIdx = columnArray->Index(_T("lambda"));
//    int egtIdx = columnArray->Index(_T("EGT #1"));
//    int sparkIdx = columnArray->Index(_T("Spark"));

   //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-1--------------"));
    for(uint32_t i = 0; i < raceBin.Count(); i++){
        //Logger::log(LOG_MSG, wxString::Format(_T("Race Num: %d"), i) );
        RaceArray * raceBinArray = raceBin.Item(i);
        //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-2--------------"));
        wxString binCount;
        IntArray *binRecord;
        int totalRaceRecord = 0;

        ArrayOfArrayString displayedTable;
        initDisplayedValueTable(&displayedTable);
         //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-3--------------"));
        double raceStartTime = -1.0;


        int startIdx=0;
        for(uint32_t x = 0; x < raceBinArray->Count(); x++){
            //int rpmBin = rpmBinArray.Item(x);
            //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-22--------------"));
            //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-3--------------"));
            if(! isBinValid(raceBinArray, x, 1) ){
                //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-333--------------"));
                setEmptyValues( &displayedTable);
                continue;
            }

            bool isPrevBinValid = isBinValid(raceBinArray, x-1, 1);
            bool isNextBinValid = isBinValid(raceBinArray, x+1, 1);

             binRecord = raceBinArray->Item(x);
            //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-4--------------"));
            totalRaceRecord +=binRecord->Count();
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-44--------------"));
            startIdx = binRecord->Item(0);
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-45--------------"));
            int endIdx = binRecord->Last();
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-5--------------"));
            DoubleArray * minRowArray = ecuLogItemArray->Item(startIdx);
            DoubleArray * maxRowArray = ecuLogItemArray->Item(endIdx);
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-55--------------"));
            double startTime = minRowArray->Item(timeIdx);
            double endTime = maxRowArray->Item(timeIdx);
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-6--------------"));
            double startRpm = minRowArray->Item(rpmIdx);

            if(raceStartTime == -1.0){
                raceStartTime = startTime; //wxString::Format(_T(" (%.2f);"), startTime);
            }
            double endRpm = maxRowArray->Item(rpmIdx);
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-7--------------"));
            double drpmPerdt_OLD = (endRpm - startRpm)/(endTime - startTime);

            double drpmPerdt = getFilteredDeltaValue(_T("RPM"), binRecord, isPrevBinValid, isNextBinValid);

            double avgMap = getavgValue(_("MAP"), binRecord);
            double normdrpmPerdt= drpmPerdt/avgMap; // *100 would make the scale better; but without *100, it's more distinguishable from drpmPerdt data
            binCount += wxString::Format(_T(" %d "), binRecord->Count() );
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-8--------------"));

            double lambda =  getavgValue(_T("lambda"), binRecord); //minRowArray->Item(lambdaIdx);
            double egt =  getavgValue(_T("EGT #1"), binRecord); //minRowArray->Item(egtIdx);
            double spark =  getavgValue(_T("Spark"), binRecord);//minRowArray->Item(sparkIdx);
//            double avgRPM =  getavgValue(_T("RPM"), binRecord);

            displayedTable.Item(0)->Add(wxString::Format(_T("%d"), binRecord->Count()) );
            displayedTable.Item(1)->Add(wxString::Format(_T("%.3f"), drpmPerdt ));
            displayedTable.Item(2)->Add(wxString::Format(_T("%.3f"), avgMap ));
            displayedTable.Item(3)->Add(wxString::Format(_T("%.3f"), normdrpmPerdt ));
            displayedTable.Item(4)->Add(wxString::Format(_T("%.3f"), lambda ));
            displayedTable.Item(5)->Add(wxString::Format(_T("%.3f"), egt ));
            displayedTable.Item(6)->Add(wxString::Format(_T("%.3f"), spark ));


            double engineTorque = getEngineToruqe(getFileName( startIdx), i, drpmPerdt);

            displayedTable.Item(7)->Add(wxString::Format(_T("%.3f"), engineTorque ));
            displayedTable.Item(8)->Add(wxString::Format(_T("%.3f"), drpmPerdt_OLD ));


        }
          //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-100--------------"));
        //

        //wxString reportStr = wxString::Format(_T("Race Number %d: (%d); %s\n"), i, totalRaceRecord, vStr.c_str());
        wxString fileName = getFileName( startIdx);
        //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins() filename: ")+ fileName);
        wxString reportStr = wxString::Format(_T("Race Number %d (Start: %0.3f sec)"), i, raceStartTime );

        wxString sampleC = wxString::Format(_T(" %d"), totalRaceRecord);

        //wxString tmp;// = displayedTable.Item(0) + sampleC;
        displayedTable.Item(0)->Item(0) += sampleC;
        //wxString reportStr = wxString::Format(_T("Race Number %d: (%d) (%0.3f);\n"), i, totalRaceRecord, raceStartTime );

        ArrayOfArrayString raceNumberTable;
        wxArrayString * raceNumberRow = new wxArrayString();
        raceNumberRow->Add(reportStr);
        raceNumberRow->Add(fileName);
        raceNumberRow->Add(_T("Gear:"));
        //geaernumber = raceNumberRow->getGearNumber();
        //raceNumberRow->Add(gearNumber);


        // i = raceNumber
       int gearNumber = m_racesCache->getGearNumber(fileName, i);
        wxString gearNumberStr;
         gearNumberStr << gearNumber;
         int temprow =  racesTable->GetCount();
         Logger::logf(LOG_MSG,_T("------------------------------temprow = %d"),temprow);
         m_rowIndex2FileName[racesTable->GetCount()] = fileName;

         raceNumberRow->Add(gearNumberStr);

        raceNumberTable.Add(raceNumberRow );
        WX_APPEND_ARRAY(*racesTable, raceNumberTable)
        WX_APPEND_ARRAY(*racesTable, displayedTable)

        Logger::log(LOG_MSG, wxString::Format(_T("Race Number %d: (%d) %s"), i, totalRaceRecord, binCount.c_str()) );
    }

     return racesTable;
}

double RaceLogAnalyser::getEngineToruqe(const wxString& fileName, int raceNumber, double drpmPerdt)
{
    int gearNumber = m_racesCache->getGearNumber(fileName, raceNumber);
    wxString gearNumberStr;
     gearNumberStr << gearNumber;

    if(m_carParamsH->find(raceNumber) == m_carParamsH->end() ){

        (*m_carParamsH)[raceNumber] = m_carParams->Clone();
        //return m_carParams->getTorque(drpmPerdt);
    }

    CarParameters *carParams = (*m_carParamsH)[raceNumber];
    m_carParams->setCurrentDrivetrain(gearNumberStr);

    return carParams->getTorque(drpmPerdt);
}

void RaceLogAnalyser::printRacesBins()
{
    wxFile * binFile = new wxFile();
    wxString fileName = DirHelper::GetPath(dLOG) + _T("races.csv");
    binFile->Open(fileName, wxFile::write);
    ArrayOfArrayString * raceTable = getRacesTable();

    for(uint32_t i =0; i < raceTable->Count(); i++){
            wxArrayString *row = raceTable->Item(i);
             for(uint32_t y =0; y < row->Count(); y++){
                    wxString cell = row->Item(y);
                    binFile->Write(cell + _T(";"));
             }
             binFile->Write(_T("\n"));
    }
    binFile->Close();
}

/*
void RaceLogAnalyser::printRacesBins_OLD()
{

    //Bin column!
     wxFile * binFile = new wxFile();
    wxString fileName = _T("c:\\races.csv");
    binFile->Open(fileName, wxFile::write);

    wxString binColumnHeader = _T("Race nums;");
    for(int x = 0; x < rpmBinArray.Count()-1; x++){
            int rpmMinV = rpmBinArray.Item(x);
            int rpmMaxV = rpmBinArray.Item(x+1);
            binColumnHeader += wxString::Format(_T("(%d-%d);"), rpmMinV, rpmMaxV);
    }
    Logger::log(LOG_MSG, binColumnHeader);
    binFile->Write(binColumnHeader+_T("\n"));

     wxArrayString *columnArray = m_vemsDataLogDesc->getVEMSDataLogHeader();
    ECULogItemArray * ecuLogItemArray = m_vemsDataLogDesc->getVEMSDataLog();
	// should we move these 3 vars to object variables ? used pretty often
    int timeIdx = columnArray->Index(_("Time"));
    int rpmIdx = columnArray->Index(_("RPM"));
    int mapIdx = columnArray->Index(_("MAP"));

    int lambdaIdx = columnArray->Index(_("lambda"));
    int egtIdx = columnArray->Index(_("EGT #1"));
    int sparkIdx = columnArray->Index(_("Spark"));

   // Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-1--------------"));
    for(int i = 0; i < raceBin.Count(); i++){
        //Logger::log(LOG_MSG, wxString::Format(_T("Race Num: %d"), i) );
        RaceArray * raceBinArray = raceBin.Item(i);
        //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-2--------------"));
        wxString binCount;
        IntArray *binRecord;
        int totalRaceRecord = 0;
        wxString vStr;
        initCSVRow();
        double raceStartTime = -1.0;

        for(int x = 0; x < raceBinArray->Count(); x++){
            //int rpmBin = rpmBinArray.Item(x);
            //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-22--------------"));
            binRecord = raceBinArray->Item(x);
            //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-3--------------"));
            if(binRecord == NULL || binRecord->Count() < 2){
                //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-333--------------"));
                vStr += _T("--;");
                binCount += wxString::Format(_T(" %d "), binRecord->Count() );
                for(int i = 0; i < csvRow.Count(); i++){
                    csvRow.Item(i) += _T("--;");
                }
                continue;
            }

            //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-4--------------"));
            totalRaceRecord +=binRecord->Count();
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-44--------------"));
            int startIdx = binRecord->Item(0);
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-45--------------"));
            int endIdx = binRecord->Last();
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-5--------------"));
            DoubleArray * minRowArray = ecuLogItemArray->Item(startIdx);
            DoubleArray * maxRowArray = ecuLogItemArray->Item(endIdx);
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-55--------------"));
            double startTime = minRowArray->Item(timeIdx);
            double endTime = maxRowArray->Item(timeIdx);
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-6--------------"));
            double startRpm = minRowArray->Item(rpmIdx);

            if(raceStartTime == -1.0){
                raceStartTime = startTime; //wxString::Format(_T(" (%.2f);"), startTime);
            }
            double endRpm = maxRowArray->Item(rpmIdx);
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-7--------------"));
            double drpmPerdt = (endRpm - startRpm)/(endTime - startTime);

            double avgMap = getavgValue(_("MAP"), binRecord);
            double normdrpmPerdt= drpmPerdt/avgMap; // *100 would make the scale better; but without *100, it's more distinguishable from drpmPerdt data
            binCount += wxString::Format(_T(" %d "), binRecord->Count() );
             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-8--------------"));

            double lambda =  getavgValue(_T("lambda"), binRecord); //minRowArray->Item(lambdaIdx);
            double egt =  getavgValue(_T("EGT #1"), binRecord); //minRowArray->Item(egtIdx);
            double spark =  getavgValue(_T("Spark"), binRecord);//minRowArray->Item(sparkIdx);
            double avgRPM =  getavgValue(_T("RPM"), binRecord);

            csvRow.Item(0) += wxString::Format(_T("%d;"), binRecord->Count());
          //  csvRow.Item(1) += wxString::Format(_T("%.2f;"), startTime);
          //  csvRow.Item(2) += wxString::Format(_T("%.2f;"), endTime);
            csvRow.Item(1) += wxString::Format(_T("%.3f;"), drpmPerdt);
            csvRow.Item(2) += wxString::Format(_T("%.3f;"), avgMap);
            csvRow.Item(3) += wxString::Format(_T("%.3f;"), normdrpmPerdt);

            csvRow.Item(4) += wxString::Format(_T("%.3f;"), lambda);
            csvRow.Item(5) += wxString::Format(_T("%.3f;"), egt);
            csvRow.Item(6) += wxString::Format(_T("%.3f;"), spark);

            //double carVelocity = avgRPM/60/2.13/4.11 * 2 * pi * 0.3;

            double drivetrain = 2.13 * 4.11;
            double wheelRadius = 0.3;
            double rpm2v = 1.0/60.0/drivetrain * 2.0 * 3.14 * wheelRadius;
            double wheeldVPerdt =  drpmPerdt * rpm2v;
            double aNull = 0.03 * 9.81;
            double fNull = 500.0;
            double wheelForce = 1600.0 * (wheeldVPerdt + aNull) + fNull;
            double wheelTorque = wheelForce * wheelRadius;
            double engineTorque = wheelTorque / drivetrain;

            csvRow.Item(7) += wxString::Format(_T("%.3f;"), engineTorque);

             //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-9--------------"));
        }
          //Logger::log(LOG_MSG, _T("VEMSLogParser::printRacesBins()-100--------------"));
        //

        //wxString reportStr = wxString::Format(_T("Race Number %d: (%d); %s\n"), i, totalRaceRecord, vStr.c_str());

        wxString reportStr = wxString::Format(_T("Race Number %d: (%d) (%0.3f);\n"), i, totalRaceRecord, raceStartTime );

        for(int i = 0; i < csvRow.Count(); i++){
            reportStr += csvRow.Item(i) +_T("\n");
        }
        binFile->Write(reportStr);

        Logger::log(LOG_MSG, wxString::Format(_T("Race Number %d: (%d) %s"), i, totalRaceRecord, binCount.c_str()) );
    }
    binFile->Close();
}
*/
double RaceLogAnalyser::getavgValue(const wxString& key, IntArray *binRecord)
{
         ECULogItemArray * ecuLogItemArray = m_vemsDataLogDesc->getVEMSDataLog();
         wxArrayString *columnArray = m_vemsDataLogDesc->getVEMSDataLogHeader();
       int mapIdx = columnArray->Index(key);
       double avgMap = 0.0;
       for(uint16_t i = 0; i < binRecord->Count(); i++){
            int idx = binRecord->Item(i);
            DoubleArray * rowArray = ecuLogItemArray->Item(idx);
            double map = rowArray->Item(mapIdx);
            avgMap += map;
       }
       avgMap /= binRecord->Count();
       return avgMap;

}

//use the first-1 and last+1  if available from log record;
//
double RaceLogAnalyser::getFilteredDeltaValue(const wxString& key, IntArray * binRecord, bool isPrevBinValid, bool isNextBinValid)
{


    ECULogItemArray * ecuLogItemArray = m_vemsDataLogDesc->getVEMSDataLog();
    wxArrayString *columnArray = m_vemsDataLogDesc->getVEMSDataLogHeader();
    int varIdx = columnArray->Index(key);
    int timeIdx = columnArray->Index(_T("Time"));

    int maxIdx = ecuLogItemArray->Count()-1;

    int startIdx =  binRecord->Item(0);
    if(isPrevBinValid) startIdx--;

    int endIdx =  binRecord->Last();
    if(isNextBinValid) endIdx++;

    if( startIdx < 0) startIdx = 0;
    if( endIdx > maxIdx ) endIdx = maxIdx;

    DoubleArray * rowStartArray = ecuLogItemArray->Item(startIdx);
    DoubleArray * rowEndArray = ecuLogItemArray->Item(endIdx);
    double startTime = rowStartArray->Item(timeIdx);
    double endTime = rowEndArray->Item(timeIdx);

    //Logger::logf(LOG_MSG, _T("RaceLogAnalyser::getFilteredDeltaValue() isPrevValid: %d, isNextValid: %d, startTime: %f endTime: %f key: ")+key, isPrevBinValid, isNextBinValid, startTime, endTime );

    int sampleCount = endIdx - startIdx+1;
    double deltaTime =  (endTime-startTime) / (sampleCount-1);

    double Sx = 0.0;
    double Sy = 0.0;
    double Sxx = 0.0;
    double Sxy = 0.0;
    double sample = 0.0;

    for(int i = startIdx; i < endIdx + 1; i++){
        DoubleArray * rowArray = ecuLogItemArray->Item(i);
        double yValue = rowArray->Item(varIdx);

        //double xValue = rowArray->Item(timeIdx);

        double xValue = startTime + sample * deltaTime;

        Sx += xValue;
        Sy += yValue;
        Sxx  += xValue* xValue;
        Sxy += xValue* yValue;
        sample ++;
    }

    double slope = (sample * Sxy - Sx*Sy) /(sample* Sxx -Sx*Sx);
    return slope;
}

double  RaceLogAnalyser::getDeltaRPM(IntArray * intArray)
{
    if( intArray->Count() < 3) return -9999.0;

    ECULogItemArray * ecuLogItemArray = m_vemsDataLogDesc->getVEMSDataLog();
    wxArrayString *columnArray = m_vemsDataLogDesc->getVEMSDataLogHeader();
    int rpmIdx = columnArray->Index(_("RPM"));
    int startIdx = intArray->Item(0);
    int endIdx = intArray->Last();
    DoubleArray * startRowArray = ecuLogItemArray->Item(startIdx);
    DoubleArray * endRowArray = ecuLogItemArray->Item(endIdx);
    int startRpm = (int)(startRowArray->Item(rpmIdx));
    int endRpm = (int)(endRowArray->Item(rpmIdx));

     //int timeIdx = columnArray->Index(_("Time"));
//     double timeV = startRowArray->Item(timeIdx);

     // if(timeV > 756.0 && timeV < 780.0 )
     //   Logger::logf(LOG_MSG, wxString::Format(_T("----Time: %f Delta rpm: %d - %d"), timeV, endRpm, startRpm ) );

    return endRpm - startRpm;
}

void RaceLogAnalyser::initDisplayedValueTable(ArrayOfArrayString * displayedValueTable)
{
    for( uint32_t i=0; i < displayedValueA.Count(); i++)
    {
        wxString value = displayedValueA.Item(i);
         //Logger::log(LOG_MSG, _T(" RaceLogAnalyser::initDisplayedValueTable() Displayed values: ") + value);
        wxArrayString *valueRow = new wxArrayString();
        valueRow->Add(value);
        displayedValueTable->Add(valueRow);
    }

}
/*
void RaceLogAnalyser::initCSVRow()
{

    csvRow.Clear();
    csvRow.Add(_T("Sample count:;"));
    //csvRow.Add(_T("Start time:;"));
    //csvRow.Add(_T("End time:;"));
    csvRow.Add(_T("drmp/dt:;"));
    csvRow.Add(_T("avg map:;"));
    csvRow.Add(_T("drmp/dt/avgmap:;"));
    csvRow.Add(_T("lambda:;"));
    csvRow.Add(_T("EGT:;"));
    csvRow.Add(_T("Spark:;"));
    csvRow.Add(_T("Engine torque:;"));
}
*/
wxArrayString * RaceLogAnalyser::getDisplayedValuesArray()
{
        return &csvRow;
}

IntArray * RaceLogAnalyser::getBinArray()
{
        return &rpmBinArray;
}

 wxString RaceLogAnalyser::getFileName( int startIdx)
 {

        if(newFileIndexA.Count() == 1) return  m_vemsDataLogDesc->getFilesArray()->Item(0);

        for(uint32_t i= 0; i < newFileIndexA.Count()-1; i++){

                int idx = newFileIndexA.Item(i);
                int nextIdx = newFileIndexA.Item(i+1);
                //Logger::log(LOG_MSG, wxString::Format(_T("RaceLogAnalyser::getFileName() i=%d, log idx=%d"), i, idx ));
                if(idx < startIdx && nextIdx >startIdx){
                    return m_vemsDataLogDesc->getFilesArray()->Item(i);
                }
        }
        return _T("-NA-");
}

void RaceLogAnalyser::setGearNo(int rowIndex, int raceNum, long gearNo)
{
    rowIndex++;
     Logger::logf(LOG_MSG,_T("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!RaceLogAnalyser::setGearNo"));
    //wxString file = getFileName(rowIndex);
     Logger::logf(LOG_MSG,_T("rowIndex=%d"),rowIndex);
    wxString fileName = m_rowIndex2FileName[rowIndex];
    Logger::logf(LOG_MSG,_T("fileName")+fileName);
    m_racesCache->setGearNumber(fileName, raceNum, gearNo);

    m_carParams->setCurrentDrivetrain(wxString::Format(_T("%i"), gearNo) );

    CarParameters *carParams;
    if(m_carParamsH->find(raceNum)  == m_carParamsH->end() ){
       (* m_carParamsH)[raceNum] = m_carParams->Clone();
    }

    carParams = (*m_carParamsH)[raceNum];
    carParams->setCurrentDrivetrain(wxString::Format(_T("%i"), gearNo) );
}
