#include "AIMSendPanel.h"

#include <wx/statline.h>
#include <wx/xml/xml.h>
#include <wx/gbsizer.h>

#include "../SerialPortHelper.h"
#include "../DirHelper.h"
#include "../configmanager/VemsDeviceControllerConfig.h"
#include "../vemsprotocol/AIMDeviceController.h"
#include "../vemsprotocol/SIPRPacket.h"
#include "../logger.h"
#include "../vemsprotocol/SIPRTypes.h"
#include "../events/InputEvent.h"

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(AIMNodeDescArray);

BEGIN_EVENT_TABLE(AIMSendPanel,wxFrame)

    EVT_RADIOBUTTON(idRadioEvent,AIMSendPanel::OnTargetSelect)
    EVT_RADIOBUTTON(idRadioSerial,AIMSendPanel::OnTargetSelect)

    EVT_BUTTON(idButtonSerialConnect,AIMSendPanel::OnButton)
    EVT_BUTTON(idButtonStart,AIMSendPanel::OnButton)
    EVT_BUTTON(idButtonStop,AIMSendPanel::OnButton)

    EVT_SCROLL(AIMSendPanel::OnChValueChanged)

    EVT_TIMER(idPacketTimer,AIMSendPanel::OnSendTimer)

END_EVENT_TABLE()

AIMSendPanel::AIMSendPanel(wxWindow* parent)
{
    //ctor
    Create(parent, wxID_ANY, _T("Sending AIM to anywhere..."), wxDefaultPosition, wxSize(800,600));
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_packetSinker = NULL;
    m_aimController = NULL;

    m_packetTimer = new wxTimer(this, idPacketTimer);
    m_packetInterval = 200;

    m_variableData = VariableDataDH( &m_model );

    m_rpn.setVariableData(&m_variableData);

    AIMNodeDescArray channels;

    wxString filename = DirHelper::GetPath(dCONFIG) + _T("aimpanel.xml");

    wxXmlDocument doc(filename);

    wxXmlNode* root = doc.GetRoot();

    if(root->GetName() == _T("channels")) {

        root->GetPropVal(_T("interval"),_T("200")).ToLong(&m_packetInterval);

        for(wxXmlNode* node = root->GetChildren(); node; node = node->GetNext()  ) {
            if(node->GetName()!=_T("node")) continue;

            long chanid=0, min, max;
            wxString title, expr;

            node->GetPropVal(_T("chanid"),_T("0")).ToLong(&chanid);
            node->GetPropVal(_T("min"),_T("0")).ToLong(&min);
            node->GetPropVal(_T("max"),_T("0")).ToLong(&max);

            title = node->GetPropVal(_T("title"),_T(""));
            expr = node->GetPropVal(_T("expression"),_T(""));

            if(chanid)
                channels.Add( AIMNodeDesc( chanid,title,min,max,expr ) );

        }
    }


    CreateGUI(channels);

}

AIMSendPanel::~AIMSendPanel()
{
    //dtor
}

void AIMSendPanel::CreateGUI(AIMNodeDescArray& channels)
{
    wxScrolledWindow* slidersPanel = new wxScrolledWindow(this, -1,   wxDefaultPosition, wxSize(700,500), wxVSCROLL | wxBORDER_STATIC);
    slidersPanel->SetScrollRate( 5, 10 );

    wxPanel* controlPanel = new wxPanel(this);

    wxFlexGridSizer* slidersizer = new wxFlexGridSizer(2);
    slidersizer->AddGrowableCol(1);

    for(size_t a=0; a<channels.GetCount();a++){
        AIMNodeDesc& desc = channels[a];

        wxStaticText* label = new wxStaticText(slidersPanel,wxID_ANY,desc.Label);

        wxFont font = label->GetFont();
        font.SetWeight(wxFONTWEIGHT_BOLD);
        label->SetFont(font);

        slidersizer->Add(label,0, wxALL | wxALIGN_CENTER_VERTICAL, 5);

        int id = wxNewId();

        desc.Slider = new wxSlider(slidersPanel, id, desc.Min, desc.Min, desc.Max, wxDefaultPosition, wxDefaultSize, wxSL_LABELS);

        m_channels[id] = desc;

        CalcValue(m_channels[id]);

        slidersizer->Add(desc.Slider,0, wxALL | wxEXPAND, 5);
    }

    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    wxStaticBoxSizer * slidersbox = new wxStaticBoxSizer(wxVERTICAL,this,_T("AIM channels"));


    wxBoxSizer* controlMainSizer = new wxBoxSizer(wxVERTICAL);

    wxStaticBoxSizer * controlpanelbox = new wxStaticBoxSizer(wxHORIZONTAL,controlPanel,_T("Control panel"));

    wxStaticBoxSizer * targettypebox = new wxStaticBoxSizer(wxVERTICAL,controlPanel,_T("Select target"));

    m_radioSerial = new wxRadioButton(controlPanel, idRadioSerial, _T("Serial port"));
    m_radioEvent = new wxRadioButton(controlPanel, idRadioEvent, _T("Event sending"));

    m_radioSerial->SetValue(true);

    targettypebox->Add(m_radioSerial,0, wxALL, 5);
    targettypebox->Add(m_radioEvent,0, wxALL, 5);

    controlpanelbox->Add(targettypebox,0,wxALL|wxEXPAND,5);

    wxArrayString br = SerialPortHelper::getSerialBaudrates();
    wxArrayString coms = SerialPortHelper::GetCOMPortList();

    m_sPorts = new wxComboBox(controlPanel, wxID_ANY, coms.GetCount() ? coms[0] : _T(""), wxDefaultPosition, wxDefaultSize, coms, wxCB_SORT | wxTE_PROCESS_ENTER );
    m_sBauds = new wxComboBox(controlPanel, wxID_ANY, br.GetCount() ? br[0] : _T(""), wxDefaultPosition, wxDefaultSize, br, wxTE_PROCESS_ENTER );

    wxStaticBoxSizer * serialsettingsbox = new wxStaticBoxSizer(wxVERTICAL,controlPanel,_T("Serial settings"));

    wxGridBagSizer* serialsettingssizer = new wxGridBagSizer();

    m_tPorts = new wxStaticText( controlPanel, wxID_ANY, _T("Port:") );
    m_tBauds = new wxStaticText( controlPanel, wxID_ANY, _T("Speed:") );

    serialsettingssizer->Add(m_tPorts, wxGBPosition(0,0), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
    serialsettingssizer->Add(m_sPorts, wxGBPosition(0,1), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, 5 );
    serialsettingssizer->Add(m_tBauds, wxGBPosition(1,0), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
    serialsettingssizer->Add(m_sBauds, wxGBPosition(1,1), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, 5 );

    m_serialConnect = new wxButton(controlPanel,idButtonSerialConnect,_T("Connect"));

    serialsettingssizer->Add(m_serialConnect, wxGBPosition(2,0), wxGBSpan(1,2), wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );

    serialsettingsbox->Add(serialsettingssizer);

    controlpanelbox->Add(serialsettingsbox,0,wxALL|wxEXPAND,5);

    wxStaticBoxSizer * controlsbox = new wxStaticBoxSizer(wxVERTICAL,controlPanel,_T("Control"));

    m_startButton =  new wxButton(controlPanel, idButtonStart, _T("Start"));
    m_stopButton = new wxButton(controlPanel, idButtonStop, _T("Stop"));

    m_stopButton->Enable(false);

    controlsbox->Add( m_startButton, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
    controlsbox->Add( m_stopButton, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );

    controlpanelbox->Add(controlsbox,0,wxALL|wxEXPAND,5);


    controlMainSizer->Add(controlpanelbox,0,wxEXPAND | wxALL, 5 );
    controlPanel->SetSizer(controlMainSizer);
    controlMainSizer->SetSizeHints(controlPanel);

    wxBoxSizer* slidersPanelSizer = new wxBoxSizer(wxVERTICAL);
    slidersPanelSizer->Add(slidersizer,1,wxEXPAND|wxALL,5);
    slidersPanel->SetSizer(slidersPanelSizer);
    slidersPanelSizer->SetSizeHints(slidersPanel);


    slidersbox->Add(slidersPanel,1,wxEXPAND);

    mainsizer->Add(controlPanel);
    mainsizer->Add(slidersbox,1,wxEXPAND | wxALL, 5);

    SetSizer(mainsizer);
    mainsizer->Fit(this);
}

void AIMSendPanel::EnableSerialSettings(bool enable)
{
    m_sBauds->Enable(enable);
    m_sPorts->Enable(enable);
    m_tBauds->Enable(enable);
    m_tPorts->Enable(enable);
    m_serialConnect->Enable(enable);
}

void AIMSendPanel::EnableTargetSelector(bool enable)
{
    m_radioEvent->Enable(enable);
    m_radioSerial->Enable(enable);
}

void AIMSendPanel::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()) {
        case idButtonStart:
            StartSend();
            break;

        case idButtonStop:
            StopSend();
            break;

        case idButtonSerialConnect:
            if(m_serialConnect->GetLabel() == _T("Disconnect")){
                DisconnectSerial();
            } else if(m_serialConnect->GetLabel() == _T("Connect")){
                ConnectSerial();
            }
            break;
    }
}

void AIMSendPanel::StartSend()
{
    m_stopButton->Enable(true);
    m_startButton->Enable(false);

    EnableTargetSelector(false);

    m_packetTimer->Start(m_packetInterval);
}

void AIMSendPanel::StopSend()
{
    m_stopButton->Enable(false);
    m_startButton->Enable(true);

    EnableTargetSelector(true);

    m_packetTimer->Stop();
}

void AIMSendPanel::ConnectSerial()
{
    if(!m_aimController) return;

    VemsDeviceControllerConfig config;

    m_sBauds->GetValue().ToLong(&(config.baudrate));
    config.port = m_sPorts->GetValue();

    m_aimController->reset(config);

    m_serialConnect->SetLabel(_T("Disconnect"));
    EnableTargetSelector(false);
}

void AIMSendPanel::DisconnectSerial()
{
    if(!m_aimController) return;

    m_aimController->stop();

    m_serialConnect->SetLabel(_T("Connect"));
    EnableTargetSelector();
}

void AIMSendPanel::CalcValue(AIMNodeDesc& desc)
{
    if(!desc.Slider) return;

    int slidervalue = desc.Slider->GetValue();

    if(desc.Expression == wxEmptyString) {
        desc.Value = slidervalue;

    } else {

        m_model[_T("x")] = slidervalue;

        double val = m_rpn.calculate( desc.Expression );

        desc.Value = (uint16_t)val;

    }

}

void AIMSendPanel::OnChValueChanged(wxScrollEvent& evt)
{
    //Logger::logf(LOG_MSG,_T("%i"),evt.GetPosition());

    int id = evt.GetId();

    if(m_channels.find(id) == m_channels.end()) return;

    AIMNodeDesc& desc = m_channels[id];

    CalcValue(desc);
}

void AIMSendPanel::OnSendTimer(wxTimerEvent& evt)
{
    bool isSendToSerial = m_radioSerial->GetValue();

    for(Int2AIMNodeDescHash::iterator it = m_channels.begin(); it != m_channels.end(); it++ ){
        AIMNodeDesc& desc = it->second;

        uint8_t* siprbuffer = new uint8_t[3];

        siprbuffer[0] = desc.Channel;
        siprbuffer[1] = desc.Value >> 8;
        siprbuffer[2] = desc.Value & 0xFF;

        SIPRPacket p( 0, SIPR_AIM, 3 , siprbuffer );

        if(isSendToSerial) {

            if(m_aimController) m_aimController->SendSIPRPacket(p);

        } else {
            if(m_packetSinker){
                InputEvent evt;
                evt.setPacket(p);
                m_packetSinker->ProcessEvent(evt);
            }
        }

        //Logger::logf(LOG_MSG,desc.Label + _T(" = %i") , desc.Value);
    }
}

void AIMSendPanel::OnTargetSelect(wxCommandEvent& evt)
{
    if( m_radioEvent->GetValue() ) {

        EnableSerialSettings(false);

    } else if( m_radioSerial->GetValue() ) {

        EnableSerialSettings();

    } else
        return;
}
