#ifndef AIMSENDPANEL_H
#define AIMSENDPANEL_H

#include "../expression/RPNParser.h"
#include "../common/types/DoubleHash.h"
#include "../expression/VariableData.h"


class AIMDeviceController;

class AIMNodeDesc
{
    public:
        AIMNodeDesc() {
            Channel = -1;
            Label = wxEmptyString;
            Expression = wxEmptyString;
            Min = 0;
            Max = 100;
            Slider = NULL;
        }
        AIMNodeDesc(int channel, const wxString& label, int min, int max, const wxString& exp) {
            Channel = channel;
            Label = label;
            Min = min;
            Max = max;
            Slider = NULL;
            Expression = exp;
        }

        int         Channel;
        uint16_t    Value;
        wxString    Label;
        int         Min;
        int         Max;
        wxSlider*   Slider;
        wxString    Expression;
};

WX_DECLARE_OBJARRAY( AIMNodeDesc, AIMNodeDescArray );

WX_DECLARE_HASH_MAP(int, AIMNodeDesc, wxIntegerHash, wxIntegerEqual, Int2AIMNodeDescHash);

class AIMSendPanel: public wxFrame
{
    public:
        AIMSendPanel(wxWindow* parent);
        virtual ~AIMSendPanel();

        void CreateGUI(AIMNodeDescArray& descs);

        void SetPacketSinker(wxEvtHandler* sinker) { m_packetSinker = sinker; };
        void SetAIMDeviceController(AIMDeviceController* aimController) { m_aimController = aimController; }

    protected:

    private:

        void OnTargetSelect(wxCommandEvent& evt);
        void OnButton(wxCommandEvent& evt);
        void OnSendTimer(wxTimerEvent& evt);
        void OnChValueChanged(wxScrollEvent& evt);

        void EnableSerialSettings(bool enable = true);
        void EnableTargetSelector(bool enable = true);

        void ConnectSerial();
        void DisconnectSerial();

        void StartSend();
        void StopSend();

        void CalcValue(AIMNodeDesc& desc);

        enum {
            idPacketTimer,
            idButtonStart,
            idButtonStop,
            idButtonSerialConnect,
            idRadioEvent,
            idRadioSerial
        };

        Int2AIMNodeDescHash     m_channels;

        wxTimer*                m_packetTimer;
        long                    m_packetInterval;

        wxComboBox*             m_sPorts;
        wxComboBox*             m_sBauds;
        wxStaticText*           m_tPorts;
        wxStaticText*           m_tBauds;

        wxButton*               m_startButton;
        wxButton*               m_stopButton;
        wxButton*               m_serialConnect;

        wxRadioButton*          m_radioSerial;
        wxRadioButton*          m_radioEvent;

        AIMDeviceController*    m_aimController;

        RPNParser               m_rpn; //rpn calculator for expression symbols
        DoubleHash              m_model; //model to calculate symbols
        VariableDataDH          m_variableData; //DoubleHash variable data for m_rpn

        wxEvtHandler*           m_packetSinker;

        DECLARE_EVENT_TABLE();
};

#endif // AIMSENDPANEL_H
