#include "ConfigSwitchAdvancedUploadDialog.h"

#include "../DrawHelper.h"

#include "../bcWidgets/bcLabel.h"
#include "../bcWidgets/bcStaticBitmap.h"
#include "../bcWidgets/bcWrapper.h"

BEGIN_EVENT_TABLE(ConfigSwitchAdvancedUploadDialog, wxDialog)

    EVT_RADIOBUTTON(idRBCfgA, ConfigSwitchAdvancedUploadDialog::OnRadio)
    EVT_RADIOBUTTON(idRBCfgB, ConfigSwitchAdvancedUploadDialog::OnRadio)
    EVT_RADIOBUTTON(idRBCfgBoth, ConfigSwitchAdvancedUploadDialog::OnRadio)

END_EVENT_TABLE()

ConfigSwitchAdvancedUploadDialog::ConfigSwitchAdvancedUploadDialog(wxWindow* parent, int configType, int id, const wxString& title)
{
    m_currentConfigType = configType;

    Create(parent,id,title);

    BuildGui();
}

ConfigSwitchAdvancedUploadDialog::~ConfigSwitchAdvancedUploadDialog()
{
    //dtor
}

int ConfigSwitchAdvancedUploadDialog::GetChoosed()
{
    if(m_cfgA->GetValue()) return CFG_A;

    if(m_cfgB->GetValue()) return CFG_B;

    if(m_cfgBoth->GetValue()) return CFG_BOTH;

    return CFG_BOTH;
}

void ConfigSwitchAdvancedUploadDialog::OnRadio(wxCommandEvent& evt)
{
    int target = CFG_BOTH;
    switch(evt.GetId()){
        case idRBCfgA: target = CFG_A; break;
        case idRBCfgB: target = CFG_B; break;
        case idRBCfgBoth: target = CFG_BOTH; break;
    }

    m_headPanel->SetTarget(target);
}

void ConfigSwitchAdvancedUploadDialog::BuildGui()
{
    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    m_headPanel = new CFGSW_HeadPanel(this, wxID_ANY);

    mainsizer->Add(m_headPanel, 0, wxEXPAND);

    wxBoxSizer* radiosizer = new wxBoxSizer(wxVERTICAL);

    m_cfgBoth = new wxRadioButton(this, idRBCfgBoth, _T("Upload to A and B (recommended) "));
    radiosizer->Add(m_cfgBoth, 0 , wxALL , 5 );
    wxString hintBoth = _T("This uploads the same config to table A and B.\n\t Select it if confswitch not in use");
    radiosizer->Add( new wxStaticText(this, wxID_ANY, hintBoth), 0, wxLEFT, 25 );
    radiosizer->AddSpacer(20);

    m_cfgA = new wxRadioButton(this, idRBCfgA, _T("Upload to A only"));
    radiosizer->Add(m_cfgA, 0 , wxALL , 5 );
    wxString hintA = _T("This uploads the config to table A only");
    wxStaticText* cfgatext = new wxStaticText(this, wxID_ANY, hintA);
    radiosizer->Add( cfgatext, 0, wxLEFT, 25 );
    radiosizer->AddSpacer(20);

    m_cfgB = new wxRadioButton(this, idRBCfgB, _T("Upload to B only"));
    radiosizer->Add(m_cfgB, 0 , wxALL , 5 );
    wxString hintB = _T("This uploads the config to table B only");
    wxStaticText* cfgbtext = new wxStaticText(this, wxID_ANY, hintB);
    radiosizer->Add( cfgbtext, 0, wxLEFT, 25 );
    radiosizer->AddSpacer(20);



    mainsizer->Add(radiosizer, 0, wxALL, 15);

    mainsizer->Add( CreateStdDialogButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 10 );

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    m_cfgBoth->SetValue(true);

    bool enable = ( m_currentConfigType == CFG_A || m_currentConfigType == CFG_B );
    m_cfgA->Enable(enable); cfgatext->Enable(enable);
    m_cfgB->Enable(enable); cfgbtext->Enable(enable);
}

wxString CFGSW_HeadPanel::GetTargetIconFile(int target)
{
    switch(target){
        case CFG_A: return _T("cfg_switch_a.png");
        case CFG_B: return _T("cfg_switch_b.png");
        case CFG_BOTH: return _T("cfg_switch_both.png");
    }
    return wxEmptyString;
}

void CFGSW_HeadPanel::SetTarget(int target)
{
    m_icon->SetLabel( DrawHelper::GetIconImg(GetTargetIconFile(target)) );
}

CFGSW_HeadPanel::CFGSW_HeadPanel(wxWindow* parent, int id, int target)
    :bcParentPanel(parent, id)
{
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));

    wxBoxSizer* mainsizer = new wxBoxSizer(wxHORIZONTAL);

    mainsizer->AddSpacer(30);

    bcLabel * label = new bcLabel(this, wxID_ANY, _T("Upload config to table A and to table B \n Do not change config switch input while uploading config!"));
    label->SetProperty( BCBS_BORDER_WIDTH, 0.0, true );

    mainsizer->Add( new bcWrapper(label), 0, wxALIGN_CENTER_VERTICAL );

    //m_icon = new wxStaticBitmap(this, wxID_ANY, DrawHelper::GetBitmap( GetTargetIconFile(target) ) );
    m_icon = new bcStaticBitmap(this, wxID_ANY, DrawHelper::GetIconImg( GetTargetIconFile(target) ) );

    mainsizer->AddStretchSpacer();

    mainsizer->Add( new bcWrapper(m_icon), 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxRIGHT, 10 );

    SetSizer(mainsizer);
}
