#ifndef DIFFGAUGEDESCEDITOR_H
#define DIFFGAUGEDESCEDITOR_H

#include <wx/gbsizer.h>
#include <wx/wx.h>

#include <wx/wxsqlite3.h>

#include "../gaugedesc/GUIDescStructItem.h"
#include "../gaugedesc/GUIDescGUIItem.h"
#include "../common/types/Int2IntHash.h"

class GaugeDescManager;
class wxTextCtrlNullEx;
class DiffGaugeDescManager;
class DifferentialGauge;

class DiffGaugeDescEditor: public wxFrame
{
    public:
        DiffGaugeDescEditor(wxWindow* parent, GaugeDescManager* gdMgr, bool isShowProfiles = true, DifferentialGauge* diffGauge = NULL);
        virtual ~DiffGaugeDescEditor();

        bool BuildGUI(int profileId = -1);


    protected:

        bool LoadProfileProps(const wxString& name, bool checkBlink = true);
        void CreateStateStructDesc();
        void AddPropLine(wxVariantHash& props, int row, int padding);
        void OnProfPropLineDelButton(wxCommandEvent& evt);

        void OnButton(wxCommandEvent& evt);
        void OnProfListBox(wxCommandEvent& evt);
        void OnClose(wxCloseEvent& evt);

        void OnChangeCurrentProfCtrl(wxCommandEvent& evt);

        void DelSelectedProfiles();
        void SaveProfileName();
        void AddNewProfile();
        void AddNewSymbolLine();
        void SaveProfileProps();

        wxString GetSelectedProfile();
        void RefreshProfilePropPanel();
        void RefreshFrameLayout();

    private:

        enum {
            idProfListBox,
            idBtnAppendProf,
            idBtnEditProf,
            idBtnDelProf,
            idBtnSaveProfileProps,
            idBtnAddSymbol,
            idBtnCloseEditor,
            idBtnOK
        };

        wxListBox*          m_profilesList;
        wxTextCtrl*         m_newProfileCtrl;
        wxTextCtrlNullEx*   m_currentProfileCtrl;
        wxButton*           m_setProfNameBtn;
        wxButton*           m_savePropBtn;
        wxPanel*            m_profilePropsPanel;
        wxTextCtrlNullEx*   m_currentProfileMinVal;
        wxTextCtrlNullEx*   m_currentProfileMaxVal;

        GUIDescStructItemArray  m_profPropGuiDesc;
        wxGridBagSizer *        m_currProfSizer;

        GUIDescGUIItemArrayIntHash m_profPropGUIHash;
        Int2IntHash             m_delBtnId2ProfPropId;

        int                     m_profPropLinePadding;
        int                     m_lastProfPropLineRowIdx;
        bool                    m_isShowProfiles;

        wxSQLite3Database       m_db;
        DiffGaugeDescManager*   m_diffMgr;
        GaugeDescManager*       m_gdMgr;

        DifferentialGauge*      m_diffGauge;

        DECLARE_EVENT_TABLE();
};

#endif // DIFFGAUGEDESCEDITOR_H
