#include "NewGaugeDescriptorDialog.h"

#include "../logger.h"

#include "../gaugedesc/GaugeDescManager.h"
#include "../mtconfigdata.h"
#include "../Symbol.h"
#include "../vemsprotocol/SIPRTypes.h"

BEGIN_EVENT_TABLE(NewGaugeDescriptorDialog, wxFrame)
    EVT_BUTTON(idBtnOK,NewGaugeDescriptorDialog::OnButton)
    EVT_BUTTON(idBtnCancel,NewGaugeDescriptorDialog::OnButton)
    EVT_CHECKBOX(idCBFilter,NewGaugeDescriptorDialog::OnCheckBox)
END_EVENT_TABLE()

NewGaugeDescriptorDialog::NewGaugeDescriptorDialog(wxWindow * parent, GaugeDescManager * gdmgr)
{
    //ctor
    Create(parent, wxID_ANY, _T("Create symbol descriptor"));
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    //_T("select distinct type from symbols order by type asc;")

    m_gdMgr = gdmgr;


    m_symbolList = new wxComboBox(this, -1, _T(""),wxDefaultPosition, wxDefaultSize, 0,0, wxCB_READONLY|wxCB_SORT);
    m_typeList = new wxComboBox(this, -1, _T(""),wxDefaultPosition, wxDefaultSize, 0,0, wxCB_SORT);
    m_groupList = new wxComboBox(this, -1, _T(""),wxDefaultPosition, wxDefaultSize, 0,0, wxCB_READONLY|wxCB_SORT);

    m_descName = new wxTextCtrl(this, -1, _T(""));


    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    mainsizer->AddSpacer(15);
    mainsizer->Add( new wxStaticText(this,-1,_T("1. Available symbols:")), 0, wxLEFT|wxRIGHT ,5 );
    mainsizer->Add( m_symbolList, 0, wxALL|wxEXPAND, 5);
    mainsizer->Add( new wxCheckBox(this, idCBFilter, _T("Symbols without descriptor.")), 0, wxLEFT|wxRIGHT|wxBOTTOM, 5);;

    mainsizer->AddSpacer(10);
    mainsizer->Add( new wxStaticText(this,-1,_T("2. Select a symbol group:")), 0, wxLEFT|wxRIGHT ,5 );
    mainsizer->Add( m_groupList, 0, wxALL|wxEXPAND, 5);

    mainsizer->AddSpacer(10);
    mainsizer->Add( new wxStaticText(this,-1,_T("3. Gauge Descriptor types:")), 0, wxLEFT|wxRIGHT ,5 );
    mainsizer->Add( m_typeList, 0, wxALL|wxEXPAND, 5);

    mainsizer->AddSpacer(10);
    mainsizer->Add( new wxStaticText(this,-1,_T("4. New Gauge title:")), 0, wxLEFT|wxRIGHT ,5 );
    mainsizer->Add( m_descName, 0, wxALL|wxEXPAND, 5);

    mainsizer->AddSpacer(15);

    wxBoxSizer * btnsizer = new wxBoxSizer(wxHORIZONTAL);

    btnsizer->Add( new wxButton(this, idBtnOK,_T("OK")), 0, wxALL, 5);
    btnsizer->Add( new wxButton(this, idBtnCancel,_T("Cancel")), 0, wxALL, 5);

    mainsizer->Add( btnsizer,0, wxALIGN_CENTER_HORIZONTAL );


    FillSymbolList(true);
    FillTypeList();
    FillGroupList();

	SetSizer(mainsizer);
	mainsizer->Fit(this);
	mainsizer->SetSizeHints(this);

}

NewGaugeDescriptorDialog::~NewGaugeDescriptorDialog()
{
    //dtor
}

bool NewGaugeDescriptorDialog::CreateType()
{
    return m_gdMgr->AddSymbolDesc( m_symbolList->GetValue(), m_typeList->GetValue(), m_descName->GetValue(), m_groupList->GetValue() );
}

void NewGaugeDescriptorDialog::OnButton(wxCommandEvent &evt)
{
    switch( evt.GetId() ){
        case idBtnOK:
            if(CreateType()) {
                //Close();
            } else {
                wxString msg = _T("An error occured while creating new type.");
                if(m_gdMgr->GetLastError() == 19)
                    msg += _T("\nMaybe the symbol name is alredy exists.");
                wxMessageBox(msg,_T("Error"));
            }
            break;

        case idBtnCancel:
            Close();
            break;
    }
}

void NewGaugeDescriptorDialog::OnCheckBox(wxCommandEvent &evt)
{
    FillSymbolList(!evt.IsChecked());
}

bool NewGaugeDescriptorDialog::FillTypeList()
{
    wxArrayString arr;

    m_gdMgr->GetTypeList(arr);

    m_typeList->Append(arr);

    return true;
}

bool NewGaugeDescriptorDialog::FillSymbolList(bool isShowExists) //if is true, the filter is off
{

    if(!m_gdMgr) return false;

    if(!(m_gdMgr->getMTConfigData())) return false;

    const SymbolTable * tablescl = m_gdMgr->getMTConfigData()->getSymbolMgr().getScalarTable(SiprSensorType::SIPR_RT_MT_NORMAL);
    const SymbolTable & tableexp = m_gdMgr->getMTConfigData()->getSymbolMgr().getExpTable();


    wxArrayString arr;

    if( tablescl != NULL ) {

        SymbolGaugeDescArray symbolDescArr;
        wxArrayString existsymbolsarr;

        if( !isShowExists ) { // filter is on!
            m_gdMgr->GetSymbolDescs( symbolDescArr );
            for ( size_t i = 0; i < symbolDescArr.GetCount(); i++ ){
                existsymbolsarr.Add( symbolDescArr[i]->Name );
            }
        }

        for( SymbolTable::const_iterator it = tablescl->begin(); it != tablescl->end(); it++){

            if( isShowExists || existsymbolsarr.Index(it->second->getName()) == wxNOT_FOUND )
                arr.Add(it->second->getName());
        }

        for( SymbolTable::const_iterator it = tableexp.begin(); it != tableexp.end(); it++){

            if( isShowExists || existsymbolsarr.Index(it->second->getName()) == wxNOT_FOUND )
                arr.Add(it->second->getName());
        }
    }

    m_symbolList->Clear();
    m_symbolList->Append( arr );
    m_symbolList->Select(0);

    return true;
}


bool NewGaugeDescriptorDialog::FillGroupList()
{
    wxArrayString arr;
    m_gdMgr->GetSymbolGroupList(arr);

    m_groupList->Append(arr);
    m_groupList->Select(0);
    return true;
}
