#ifndef STATICVIEWERSCONTROLLER_H
#define STATICVIEWERSCONTROLLER_H


#include "../stdatamanager/STDataViewerBase.h"
#include "../expression/RPNParser.h"


class STDataMgrBase;
class CalibrationBase;
class STDataModelBase;
class UserDialogBase;

class StaticViewersController
{
    public:
        StaticViewersController( wxWindow* helpParent = NULL, const wxString& helpId = wxEmptyString );
        virtual ~StaticViewersController();

        void setController( STDataMgrBase* controller );
        STDataMgrBase* getController(){ return m_controller; }
        void setCalibration(CalibrationBase* calibrationBase );
        void unregisterViewers();
        void addViewer(STDataViewerBase * viewer);
        void removeViewer(STDataViewerBase * viewer);
        void refreshData();
        void writeData();
        void burnData();
        ///
        void startCalibration();
        void stopCalibration();
        void updateData( bool callSetStatus = true );
        bool isHelpShown() const { return m_helpIsShown; }

        ///
        void                requestReadAllDirty();
        void                updateRegisteredViewers();
        STDataModelBase*    getModel();

        virtual void remap() {}     // use only TableMapper
        void showHelp();
        virtual void showRemapDialog() {}
        virtual void showTable3D() {}
        virtual void exportToFile( const wxString& file ) {}
        virtual void importFromFile( const wxString& file ) {}

        void setUserDialogManager( UserDialogBase* dialogManager );
        UserDialogBase* getUserDialogBase(){ return m_dialogManager; }
        void setUserDialogName( const wxString& dialogName ) { m_dialogName = dialogName; }
        void changeToPreviousDialog();
        void changeToNextDialog();

        virtual bool onDropFiles( const wxArrayString& filenames ) { return false; }

    protected:
        STDataViewerBaseA   m_ViewerBaseA;  // TODO - should be private
        STDataMgrBase*      m_controller;   // TODO - should be private

        wxWindow*           m_helpParent;
        wxString            m_helpId;

        static bool         m_helpIsShown;

    private:
        RPNParser           m_rpn;
        CalibrationBase*    m_calibrationBase;
        UserDialogBase*  m_dialogManager;
        wxString            m_dialogName;
};

#endif // STATICVIEWERSCONTROLLER_H
