#include "TableEditor.h"

#include <wx/file.h>
#include <wx/filename.h>


#include "UserDialogFrameBase.h"
#include "TableMapper.h"
#include "StaticControlPanel.h"
#include "../mtconfigdata.h"
//#include "../MTConfigHelper.h"
#include "../3d/TableEditor3D.h"
#include "../events/VEMSTableEditorEvents.h"
#include "../events/UpdateViewerEvent.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../vemsgauge/VEMSTableEditor.h"

#include "../firmwareconfig/FirmwareConfigData.h"
#include "../firmwareconfig/FirmwareConfigLoader.h"
#include "../fileparser/FirmwareConfigParser.h"
#include "../common/VemsMatrixHelper.h"
#include "../vemsgauge/GaugeGroupManager.h"
#include "../vemsgauge/TableEditorPanel.h"

#include "../gaugedesc/GaugeDescManager.h"
#include "../DirHelper.h"

#include "../common/VemsContext.h"
#include "../common/FileDropTarget.h"
#include "../wxSizeEx.h"

/**********************************************************
*   STViewerTableRowHeader methods
**********************************************************/
STViewerTableRowHeader::STViewerTableRowHeader( TableEditor* table )
{
    m_table     = table;
}

void STViewerTableRowHeader::update()
{
    int rows  = m_table->GetNumberRows();

    for( int row = 0; row < rows; row++ )
    {
        int rowInv = rows - 1 - row;

        double viewerData = m_stDataMgr->getDataFromViewerModel( m_symbol, 0, row );
        double deviceData = m_stDataMgr->getDataFromDeviceModel( m_symbol, 0, row );

        m_table->SetRowLabelValue( rowInv, deviceData );
        m_table->AcceptModifiedRowLabelValue( rowInv );
        m_table->SetRowLabelValue( rowInv, viewerData );
    }

    m_table->Refresh();
}

void STViewerTableRowHeader::updateModel()
{
    int rows  = m_table->GetNumberRows();

    for( int r = 0; r < rows; ++r )
    {
        updateSingleData( r );
    }
}


void STViewerTableRowHeader::updateSingleData( int row )
{
    int rows = m_table->GetNumberRows();
    double data = m_table->GetRowLabelValue( row );
    m_stDataMgr->setModelData( m_symbol, 0, rows - 1 - row, data );
}


void STViewerTableRowHeader::requestWriteSingleData( int row )
{
    int rows = m_table->GetNumberRows();
    m_stDataMgr->requestWrite( m_symbol, 0, rows - 1 - row );
}


void STViewerTableRowHeader::requestWrite()
{
    int rows = m_table->GetNumberRows();

    for( int row = 0; row < rows; ++row )
    {
        m_stDataMgr->requestWrite( m_symbol, 0, row );
    }
}


void STViewerTableRowHeader::requestRead()
{
    m_stDataMgr->addPageNeedRefresh( m_symbol.getPage() );
}


wxString STViewerTableRowHeader::toString()
{
    return _T("STViewerTableRowHeader - ") + m_symbol.getName();
}

/**********************************************************
*   STViewerTableColHeader
**********************************************************/
STViewerTableColHeader::STViewerTableColHeader( TableEditor* table )
{
    m_table = table;
}


void STViewerTableColHeader::update()
{
    int cols  = m_table->GetNumberCols();

    for( int c = 0; c < cols; ++c )
    {
        double viewerData = m_stDataMgr->getDataFromViewerModel( m_symbol, 0, c );

        if ( m_stDataMgr->wasDataWritten( m_symbol, 0, c ) )
        {
            m_table->SetColLabelValue( c, viewerData );
            m_table->AcceptModifiedColLabelValue( c );
        }
        else
        {
            double deviceData = m_stDataMgr->getDataFromDeviceModel( m_symbol, 0, c );

            m_table->SetColLabelValue( c, deviceData );
            m_table->AcceptModifiedColLabelValue( c );
            m_table->SetColLabelValue( c, viewerData );
        }
    }

    m_table->Refresh();
}


void STViewerTableColHeader::updateModel()
{
    int cols  = m_table->GetNumberCols();

    for( int c = 0; c < cols; ++c )
    {
        updateSingleData( c );
    }
}


void STViewerTableColHeader::updateSingleData( int col )
{
    // int cols = m_table->GetNumberCols();
    double data = m_table->GetColLabelValue( col );
    m_stDataMgr->setModelData( m_symbol, 0, col, data );
}


void STViewerTableColHeader::requestWriteSingleData( int col )
{
    m_stDataMgr->requestWrite( m_symbol, 0, col );
}


void STViewerTableColHeader::requestWrite()
{
    int cols = m_table->GetNumberCols();

    for( int c = 0; c < cols; ++c )
    {
        m_stDataMgr->requestWrite( m_symbol, 0, c );
    }
}

void STViewerTableColHeader::requestRead()
{
    m_stDataMgr->addPageNeedRefresh( m_symbol.getPage() );
}


wxString STViewerTableColHeader::toString()
{
    return _T("STViewerTableColHeader - ") + m_symbol.getName();
}


/**********************************************************
*   STViewerTable
**********************************************************/
STViewerTable::STViewerTable( TableEditor* table )
{
    m_table = table;
}

void STViewerTable::update()
{
    size_t cols = (size_t)m_table->GetNumberCols();
    size_t rows = (size_t)m_table->GetNumberRows();

    //xorder = true
    for( size_t r = 0; r < rows; ++r )
    {
        for( size_t c = 0; c < cols; ++c  )
        {
            double viewerData = m_stDataMgr->getDataFromViewerModel( m_symbol, r, c );

            if ( m_stDataMgr->wasDataWritten( m_symbol, r, c ) )
            {
                m_table->SetCellValue( rows - 1 - r, c, viewerData );
                m_table->AcceptSingleModifiedCell( rows - 1 - r, c );
            }
            else
            {
                double deviceData = m_stDataMgr->getDataFromDeviceModel( m_symbol, r, c );

                m_table->SetCellValue( rows - 1 - r, c, deviceData );
                m_table->AcceptSingleModifiedCell( rows - 1 - r, c );
                m_table->SetCellValue( rows - 1 - r, c, viewerData );
            }
        }
    }

    //wxPoint cursorPosition = m_stDataMgr->getTablePosition( m_symbol.getName() );

    //m_table->moveCursorToPosition( cursorPosition );

    m_table->Refresh();
}

void STViewerTable::updateTable( const wxString& tableName, wxPoint cursorPosition )
{
//    Logger::logf( LOG_MSG, _T("STViewerTable::updateTable() - tableName: ") + tableName );
//    Logger::logf( LOG_MSG, _T("STViewerTable::updateTable() - m_table->getName(): ") + m_table->getName() );
//
//    if ( m_table->getName().IsSameAs( tableName ) )
//    {
//        m_table->moveCursorToPosition( cursorPosition );
//    }
    m_table->moveCursorToPosition( cursorPosition );
}

void STViewerTable::updateModel()
{
    int rows = m_table->GetNumberRows();
    int cols = m_table->GetNumberCols();

    //xorder = true
    for( int row = 0; row < rows; ++row )
    {
        for( int col = 0; col < cols; ++col  )
        {
            updateSingleData( row, col );
        }
    }

    m_table->Refresh();
}


void STViewerTable::updateSingleData( int row, int col )
{
    int rows = m_table->GetNumberRows();
    // int cols = m_table->GetNumberCols();
    // int offset = ( rows - 1 - row ) * cols + col;

    double data = m_table->GetCellValue( row, col );

    m_stDataMgr->setModelData( m_symbol, rows - 1 - row, col, data );
}


void STViewerTable::requestWriteSingleData( int row, int col )
{
    int rows = m_table->GetNumberRows();
    m_stDataMgr->requestWrite( m_symbol, rows - 1 - row, col );
}


void STViewerTable::requestWrite()
{
    int rows = m_table->GetNumberRows();
    int cols = m_table->GetNumberCols();

    for( int row = 0; row < rows; ++row )
    {
        for( int col = 0; col < cols; ++col )
        {
             m_stDataMgr->requestWrite( m_symbol, row, col );
        }
    }
}

void STViewerTable::requestRead()
{
    m_stDataMgr->addPageNeedRefresh( m_symbol.getPage() );
}


wxString STViewerTable::toString()
{
    return _T("STViewerTable - ") + m_symbol.getName();
}

/**********************************************************
*   TableEditor methods
**********************************************************/
WX_DECLARE_LIST( double, DoubleList );
#include <wx/listimpl.cpp>
WX_DEFINE_LIST( DoubleList );

BEGIN_EVENT_TABLE(TableEditor, wxPanel )
    EVT_UPDATE_VIEWER_EVENT( wxID_ANY, TableEditor::OnUpdateValue )
    EVT_SIZE(TableEditor::OnSize)
    TABLE_EDITOR_EVENT( wxID_ANY, TableEditor::OnTableEditorEvent )
    EVT_KEY_DOWN( TableEditor::OnKeyDown )
    EVT_KEY_UP( TableEditor::OnKeyUp )
//    EVT_SET_FOCUS( TableEditor::OnSetFocus )
  //  EVT_KILL_FOCUS( TableEditor::OnKillFocus )
END_EVENT_TABLE()


TableEditor::TableEditor( wxWindow* parent,GaugeDescManager * dmgr, GaugeProperties &prop, bool showTitle)
//const wxPoint &pos, const wxSize &size, GaugeGroupManager* groupManager, const wxString groupName )
    //:VEMSTableEditor( parent, pos, size, groupManager, groupName )
    :wxPanel( parent, wxID_ANY ,prop.getPos(), prop.getSize(), wxNO_BORDER )
{
    m_hasTitle = showTitle;

    m_focusSinker = NULL;

    m_isRTSymbolRegistered = false;
    m_groupManager = dmgr->GetGaugeGrpMgr();
    m_groupName = prop.groupName;
    SetWindowStyle( wxNO_BORDER );
    m_bUseRealtimeTarget  = false;
    m_table     = NULL;
    m_table3D   = NULL;
    m_prop = prop;
    m_gaugeDescMgr = dmgr;


    // m_table = new VEMSTableEditor( this, wxPoint( 0,0 ), size, groupManager, groupName );
   // m_table->SetWindowStyle( wxNO_BORDER );
   // SetWindowStyle( wxNO_BORDER );
#ifndef __WINCE_STANDARDSDK__
    new FileDropTarget<TableEditor>( this );
#endif

}

TableEditor::~TableEditor()
{
    if ( m_tableViewer )
    {
        m_controller->UnRegisterViewer( m_tableViewer );
        m_ViewerBaseA.Remove( m_tableViewer );
        delete m_tableViewer;
    }


    if ( m_colHeaderViewer )
    {
        m_controller->UnRegisterViewer( m_colHeaderViewer );
        m_ViewerBaseA.Remove( m_colHeaderViewer );
        delete m_colHeaderViewer;
    }

    if ( m_rowHeaderViewer )
    {
        m_controller->UnRegisterViewer( m_rowHeaderViewer );
        m_ViewerBaseA.Remove( m_rowHeaderViewer );
        delete m_rowHeaderViewer;
    }

    UnRegisterViewer();
}

void TableEditor::b_SetSize(const wxSize& size)
{
    GetTable()->SetSize(size);
}

void TableEditor::SetFocusParentSinker(FocusParentSinkerBase* fps)
{
    m_focusSinker = fps;

    if(m_table) m_table->SetFocusParentSinker(fps);

    if(m_table3D) m_table3D->SetFocusParentSinker(fps);
}

StaticViewersController* TableEditor::newInstance(
        wxWindow* parent, const VemsTableDesc& desc, STDataMgrBase* stMgr, RTDataMgrBase* rtMgr,
        MTConfigData* mtconfigdata, GaugeDescManager* gaugeDescMgr, UserDialogBase* udMgr)
{
    wxString name   = desc.name;

    GaugeProperties prop;
    prop.symbolName = name;
    TableEditor* table = new TableEditor( parent, gaugeDescMgr, prop );
    table->init( name, stMgr, rtMgr, mtconfigdata );
    table->setUserDialogName( name );
    table->setUserDialogManager( udMgr );
    table->SetInitialSize( wxSize( 400, 300 ) );

    parent->Connect(    wxID_ANY,
                wxEVT_KEY_DOWN,
                wxKeyEventHandler( TableEditor::OnKeyDown ),
                (wxObject*) NULL,
                table );

    wxSizer* sizer = parent->GetSizer();
    if(sizer){ sizer->Add( table, 1, wxEXPAND ); }
    else{ wxASSERT(_T("Parent must have a sizer!") ); }
    return table;
}

void TableEditor::SetNewTheme(WidgetThemeTable *theme)
{

}

wxRegion TableEditor::GetShape()
{
    return wxRegion();
}

wxRegion TableEditor::GetBorderRegion()
{
    return wxRegion();
}

void TableEditor::init( const wxString& name, STDataMgrBase* stController, RTDataMgrBase* rtController, MTConfigData* mtconfigdata)
{
    m_controller    = stController;
    m_pRTController = rtController;
    m_mtConfigData  = mtconfigdata;

    init( name );
}

void TableEditor::init( const wxString& name )
{
    m_name = name;

    MTConfigData* configData = m_mtConfigData;

    wxASSERT_MSG( ( configData != NULL ), _T("MTConfigData* configData should not be NULL in TableEditor::init()") );

    VemsTableDescH* tableDescH = configData->getTables();

    wxASSERT_MSG( ( tableDescH != NULL ), _T("VemsTableDescH* tableDescH should not be NULL in TableEditor::init()") );

    // if name not found, use veTableTbl
    if( tableDescH->find( m_name ) == tableDescH->end() )
    {
       // m_name = _T("veTable");
        //if ( tableDescH->find( m_name ) == tableDescH->end() ) { return; }
        return;
    }

    //Logger::logf( LOG_MSG, _T("TableEditor::init() - ") + m_name );
    //veTableTbl

    VemsTableDesc* tableDesc = (*tableDescH)[ m_name ];


    m_bUseRealtimeTarget = !tableDesc->zRTSymbol.IsEmpty();

    Symbol* sColH   = configData->getSymbol( tableDesc->xSTSymbol );
    Symbol* sRowH   = configData->getSymbol( tableDesc->ySTSymbol );
    Symbol* sTable  = configData->getSymbol( tableDesc->zSTSymbol );
    Symbol* sLoadH  = configData->getSymbol( tableDesc->xRTSymbol );
    Symbol* sLoadV  = configData->getSymbol( tableDesc->yRTSymbol );
    Symbol* sLoadValue = NULL;

    if ( m_bUseRealtimeTarget )
    {
        sLoadValue = configData->getSymbol( tableDesc->zRTSymbol );
        if ( !sLoadValue ) { return; }
    }

    if( !( sRowH && sColH && sTable && sLoadH && sLoadV ) )
    {
        return; //TODO - handle symbol not found
    }

    m_is3D = tableDesc->is3D;

    if ( m_is3D )
    {
        m_table3D = new TableEditor3D( this, m_gaugeDescMgr->GetGaugeGrpMgr(), m_prop);
        //, m_name, m_groupManager, m_groupName );
        //m_table3D->SetTableName( m_name );
        m_table3D->SetWindowStyle( wxNO_BORDER );
        m_table3D->useTargetMarker( m_bUseRealtimeTarget );
        m_table3D->setSmallStepValue( tableDesc->smallCellStep );
        m_table3D->setLargeStepValue( tableDesc->largeCellStep );
        m_table3D->setRowNumberDigits( sRowH->getDigits() );
        m_table3D->setColNumberDigits( sColH->getDigits() );
        m_table3D->setTableNumberDigits( sTable->getDigits() );
        m_table3D->SetFocusParentSinker(m_focusSinker);
        m_grpMember = m_table3D;
    }
    else
    {
        m_table = new VEMSTableEditor( this, m_gaugeDescMgr, m_prop);
        //, wxDefaultPosition, wxDefaultSize, m_groupManager, m_groupName );

       // m_table->SetWindowStyle( wxNO_BORDER );
        m_table->SetValuePrecision( tableDesc->numberWidth, tableDesc->numberPrec );
        m_table->SetSmallStepValue( tableDesc->smallCellStep );
        m_table->SetLargeStepValue( tableDesc->largeCellStep );
        m_table->SetFocusParentSinker(m_focusSinker);
        m_grpMember = m_table;
    }

    m_rowHeaderViewer = new STViewerTableRowHeader( this ); //TODO set symbol in constructor
//    m_rowHeaderViewer->setSymbol( *sRowH );                 //TODO only symbol need
//    m_rowHeaderViewer->setSymbolName( sRowH->getName() );   //TODO only symbol need
    m_rowHeaderViewer->setSymbol( *sRowH );   //TODO only symbol need
    m_ViewerBaseA.Add( m_rowHeaderViewer );

    //Logger::logf(   LOG_MSG, _T("TableEditor::init() - row header symbol: ") + sRowH->toString() + _T("\n") );
    //Logger::logf(   LOG_MSG, _T("TableEditor::init() - row header viewer symbol: ") +
    //                m_rowHeaderViewer->getSymbol().toString() + _T("\n") );


    m_colHeaderViewer = new STViewerTableColHeader( this );
//    m_colHeaderViewer->setSymbol( *sColH );
//    m_colHeaderViewer->setSymbolName( sColH->getName() );
    m_colHeaderViewer->setSymbol( *sColH );
    m_ViewerBaseA.Add( m_colHeaderViewer );

    m_tableViewer     = new STViewerTable( this );
    m_tableViewer->setSymbol( *sTable );
//    m_tableViewer->setSymbolName( sTable->getName() );
    m_ViewerBaseA.Add( m_tableViewer );

    int row = sRowH->getShape0();
    int col = sColH->getShape0();

    if ( m_is3D ) { m_table3D->setDimension( row, col ); }
    else { m_table->CreateCells( row, col ); }

    m_loadHSymbolName = sLoadH->getName();
    m_loadVSymbolName = sLoadV->getName();
    if ( m_bUseRealtimeTarget ) { m_loadValueSymbolName = sLoadValue->getName(); }

    wxASSERT_MSG( ( m_controller != NULL ), _T("STDataMgrBase* stController should not be NULL in TableEditor::init()") );

    m_controller->RegisterViewer( m_tableViewer );
    m_controller->RegisterTableViewer( m_tableViewer );
    m_controller->RegisterViewer( m_colHeaderViewer );
    m_controller->RegisterViewer( m_rowHeaderViewer );

    // create table names
    wxArrayString tables;

    for ( VemsTableDescH::const_iterator it = tableDescH->begin(); it != tableDescH->end(); ++it )
    {
        wxString title = it->second->title;

        if ( m_is3D && it->second->is3D )
        {
            title += _T(" 3D");
            tables.Add( title );
        }
        else if ( !m_is3D && !it->second->is3D )
        {
            tables.Add( title );
        }
    }

    tables.Sort();

    if ( m_is3D )
    {
        m_table3D->setCellsMinValue( sTable->getLo() );
        m_table3D->setCellsMaxValue( sTable->getHi() );
        m_table3D->SetTables( tables );
        m_table3D->setRowHeaderMinValue( sRowH->getLo() );
        m_table3D->setRowHeaderMaxValue( sRowH->getHi() );
        m_table3D->setColHeaderMinValue( sColH->getLo() );
        m_table3D->setColHeaderMaxValue( sColH->getHi() );
    }
    else
    {
        m_table->SetCellsMinValue( sTable->getLo() );
        m_table->SetCellsMaxValue( sTable->getHi() );
        m_table->SetColHeaderMinValue( sColH->getLo() );
        m_table->SetColHeaderMaxValue( sColH->getHi() );
        m_table->SetRowHeaderMinValue( sRowH->getLo() );
        m_table->SetRowHeaderMaxValue( sRowH->getHi() );
        m_table->SetTables( tables );
    }

    updateData( false );

    wxASSERT_MSG( ( m_pRTController != NULL ), _T("RTDataMgrBase* rtController should not be NULL in TableEditor::init()") );

    m_pRTController->RegisterViewer( this );
    this->SetDataMgr( m_pRTController );


    SetLabel( tableDesc->title );

    if(m_hasTitle){
//        wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
//        wxStaticText* title = new wxStaticText(this, wxID_ANY, tableDesc->title);
//        title->SetForegroundColour(wxColor(255,255,255));
//        title->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_ACTIVECAPTION));
//        wxFont font = title->GetFont();
//        font.SetWeight(wxFONTWEIGHT_BOLD);
//        title->SetFont(font);
//        sizer->Add(title,1,wxEXPAND);
//        sizer->Add(GetTable(),1,wxEXPAND);
//        SetSizer(sizer);
//        sizer->Fit(this);
    }

    //refreshData();
/*
    if ( m_is3D ) {
        if ( GetParent() )
        {
            SetSize( GetParent()->GetClientSize() );
            m_table3D->SetSize( GetClientSize() );
        }

    } else {
        if ( GetParent() )
        {
            SetSize( GetParent()->GetClientSize() );
            m_table->SetSize( GetClientSize() );
        }

        SetInitialSize( m_table->GetClientSize() );
        m_table->SetSize( m_table->GetClientSize() );
        m_table->UpdateSize();

    }*/
}


wxString TableEditor::findTableNameByTitle( const wxString& title )
{
    MTConfigData* configData = m_mtConfigData;
    VemsTableDescH* tableDescH = configData->getTables();

    wxString tmp = title;
    bool is3D = false;
    if ( title.EndsWith( _T(" 3D"), &tmp ) ) { is3D = true; }

    for ( VemsTableDescH::const_iterator it = tableDescH->begin(); it != tableDescH->end(); ++it )
    {
        if ( tmp.IsSameAs( it->second->title ) && ( is3D == it->second->is3D ) )
        return it->first;
    }

    return wxEmptyString;
}

void TableEditor::SetValue( double value )
{
    //not used
}

void TableEditor::refresh( DoubleHash* table )
{

    DoubleHash::const_iterator it = table->find( m_loadHSymbolName );

    if( it != table->end() )
    {
        m_loadHValue = it->second;
    }

    it = table->find( m_loadVSymbolName );

    if( it != table->end() )
    {
        m_loadVValue = it->second;
    }

    if ( m_bUseRealtimeTarget )
    {
        it = table->find( m_loadValueSymbolName );

        if ( it != table->end() ) { m_loadValue = it->second; }
    }

   //SetCurrentLoad( m_loadVValue, m_loadHValue );

    UpdateViewerEvent event;


    event.m_userData[_T("loadVValue")] = m_loadVValue;
    event.m_userData[_T("loadHValue")] = m_loadHValue;
    if ( m_bUseRealtimeTarget ) { event.m_userData[_T("loadValue")] = m_loadValue; }

    this->AddPendingEvent( event );

}

/*
void TableEditor::require( DoubleHash* table)
{
    (*table)[ m_loadHSymbolName ];
    (*table)[ m_loadVSymbolName ];
}
*/

void TableEditor::require()
{
    if( !m_isRTSymbolRegistered )
    {
        m_rtDataMgr->registerSymbol( m_loadHSymbolName );
        m_rtDataMgr->registerSymbol( m_loadVSymbolName );

        if ( m_bUseRealtimeTarget ) { m_rtDataMgr->registerSymbol( m_loadValueSymbolName ); }

        m_isRTSymbolRegistered = true;
    }
}

void TableEditor::unregisterSymbols()
{
    if( m_isRTSymbolRegistered )
    {
        m_rtDataMgr->unregisterSymbol( m_loadHSymbolName );
        m_rtDataMgr->unregisterSymbol( m_loadVSymbolName );

        if ( m_bUseRealtimeTarget ) { m_rtDataMgr->unregisterSymbol( m_loadValueSymbolName ); }

        m_isRTSymbolRegistered = false;
    }
}

void TableEditor::SetValueIdx( int idx )
{
/*
    Logger::logf(LOG_MSG,_T("VemsWidgetBase::SetValueIdx recived index: %i"), idx );

    m_defGaugeId = choices2gaugeDescA.Item(idx);

    Logger::logf(LOG_MSG, _T("VemsWidgetBase m_defGaugeId: %i"), m_defGaugeId );

    m_gd = m_gaugeDescA->Item(m_defGaugeId);
    m_gd->SetGaugeNumbers();

    requireChanged();

    UpdateSize();
	Refresh(false);
*/
}

//void TableEditor::OnSingleCellChanged( int row, int col, double d )
void TableEditor::OnTableEditorEvent( TableEditorEvent& event )
{
    //Logger::logf( LOG_MSG, _T("TableEditor::OnSingleCellChanged() - row : %d, col : %d, d : %f"), event.row, event.col, event.value );

    int id = event.GetId();

    switch( id )
    {
        case TableEditorEvent::CELL_CHANGED :
        {
            //Logger::logf( LOG_MSG, _T("TableEditor::OnTableEditorEvent()") );
            m_tableViewer->updateSingleData( event.row, event.col );
            m_tableViewer->requestWriteSingleData( event.row, event.col  );
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::ROW_HEADER_CHANGED :
        {
            //Logger::logf( LOG_MSG, _T("TableEditor::OnSingleCellChanged()::ROW_HEADER") );
            m_rowHeaderViewer->updateSingleData( event.row );
            m_rowHeaderViewer->requestWriteSingleData( event.row );
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::ROW_HEADER_MOVED :
        {
            m_rowHeaderViewer->updateSingleData( event.row );
            //m_rowHeaderViewer->requestWriteSingleData( event.row );
            m_tableViewer->updateModel();
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::ROW_HEADER_INSERTED :
        {
            m_rowHeaderViewer->updateModel();
            //m_rowHeaderViewer->requestWriteData();
            m_tableViewer->updateModel();
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::COL_HEADER_CHANGED :
        {
            //Logger::logf( LOG_MSG, _T("TableEditor::OnSingleCellChanged()::COL_HEADER") );
            m_colHeaderViewer->updateSingleData( event.col );
            m_colHeaderViewer->requestWriteSingleData( event.col );
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::COL_HEADER_MOVED :
        {
            m_colHeaderViewer->updateSingleData( event.col );
            //m_colHeaderViewer->requestWriteSingleData( event.col );
            m_tableViewer->updateModel();
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::COL_HEADER_INSERTED :
        {
            m_colHeaderViewer->updateModel();
            //m_colHeaderViewer->requestWriteData();
            m_tableViewer->updateModel();
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::TABLE_NAME_CHANGED:
        {
            change( findTableNameByTitle( event.name ) );
        } break;
        case TableEditorEvent::CELL_COPIED:
        {
            m_tableViewer->updateModel();
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::UNDO_ALL_CELLS_MODIFICATION:
        {
            //m_tableViewer->updateData();
            m_controller->updateFromDeviceModel( m_tableViewer->getSymbol() );
            m_controller->updateFromDeviceModel( m_rowHeaderViewer->getSymbol() );
            m_controller->updateFromDeviceModel( m_colHeaderViewer->getSymbol() );
            m_controller->updateRegisteredViewers();
            /// 3DTable undo bug fix
            m_controller->updateRegisteredViewers();
        } break;
        case TableEditorEvent::CELLS_INTERPOLATED:
        case TableEditorEvent::CELLS_CHANGED:
        {
            //Logger::logf( LOG_MSG, _T("TableEditor::OnSingleCellChanged() - CELLS_INTERPOLATED") );
            if ( m_table )
            {
                if ( m_table->AreCellsModified() )
                {
                    //Logger::logf( LOG_MSG, _T("TableEditor::OnSingleCellChanged() - CELLS_INTERPOLATED - cells are modified") );
                    m_tableViewer->updateModel();
                    m_controller->updateRegisteredViewers();
                }
            }
            else if ( m_table3D )
            {
                m_tableViewer->updateModel();
                m_controller->updateRegisteredViewers();
            }
        } break;
        case TableEditorEvent::WRITE_FW_CONFIG:
        {
            writeData();
        } break;

        case TableEditorEvent::BURN_FW_CONFIG:
        {
            burnData();
        } break;

        case TableEditorEvent::REFRESH_FW_CONFIG:
        {
            refreshData();
        } break;

        case TableEditorEvent::IMPORT:
        {
            Logger::logf( LOG_MSG, _T("TableEditorEvent::IMPORT:") );
            wxString msg = _T("Choose a file");
            wxString dir = DirHelper::GetPath( dLOG );
             wxString wildcard = _T("VemsCfg files (*.vemscfg)|*.vemscfg|MSQ files (*.msq)|*.msq|Any files (*.*)|*.*");
            wxFileDialog dialog( this, msg, dir, _T(""), wildcard);

            if ( dialog.ShowModal() == wxID_OK )
            {
                importFromFile( dialog.GetPath() );
            }
        } break;

        case TableEditorEvent::REMAP:
        {
            showRemapDialog();
        } break;

        case TableEditorEvent::CURSOR_MOVED:
        {
            //m_controller->setTablePosition( m_tableViewer->getSymbol().getName(), wxPoint( event.col, event.row ) );
            //m_controller->updateRegisteredViewers();
            m_controller->updateTableViewers( m_tableViewer, m_name, wxPoint( event.col, event.row ) );
        } break;
    }
}

void TableEditor::OnUpdateValue( UpdateViewerEvent& event )
{

    double loadV = event.m_userData[_T("loadVValue")].GetDouble();
    double loadH = event.m_userData[_T("loadHValue")].GetDouble();
    double loadValue = 0.0;

    if ( m_bUseRealtimeTarget ) { loadValue = event.m_userData[_T("loadValue")].GetDouble(); }

    if ( m_is3D )
    {
        m_table3D->setMarkedPosition( loadH, loadV );
        m_table3D->setTargetPosition( loadValue );
        m_table3D->AutoFollowLoad();
        Refresh();
    }
    else {
        m_table->SetCurrentLoad( loadV, loadH );
        m_table->AutoFollowLoad();
    }

}

void TableEditor::OnSize( wxSizeEvent& e )
{
    if ( m_is3D ) { m_table3D->SetSize( GetClientSize() ); }
    if ( m_table ) { m_table->SetSize( GetClientSize() ); }
    Refresh();
}

void TableEditor::SetCellValue( int row, int col, double value )
{
    if ( m_is3D ) { m_table3D->setCellValue( row, col, value ); }
    else { m_table->SetCellValue( row, col, value ); }
}

void TableEditor::SetRowLabelValue( int row, double value )
{
    if ( m_is3D ) { m_table3D->setRowLabelValue( row, value ); }
    else { m_table->SetRowLabelValue( row, value ); }
}

void TableEditor::AcceptModifiedRowLabelValue( int row )
{
    if ( !m_is3D ) { m_table->AcceptModifiedRowLabelValue( row ); }
}

int TableEditor::GetRowLabelValue( int row )
{
    if ( !m_is3D ) { return m_table->GetRowLabelValue( row ); }
    else { return m_table3D->GetRowLabelValue( row ); }
}

void TableEditor::SetColLabelValue( int col, double value )
{
    if ( m_is3D ) { m_table3D->setColLabelValue( col,value ); }
    else { m_table->SetColLabelValue( col,value ); }
}


void TableEditor::AcceptModifiedColLabelValue( int col )
{
    if ( !m_is3D ) { m_table->AcceptModifiedColLabelValue( col ); }
}


int TableEditor::GetColLabelValue( int col )
{
    if ( !m_is3D ) { return m_table->GetColLabelValue( col ); }
    else { return m_table3D->GetColLabelValue( col ); }
}

double TableEditor::GetCellValue( int row, int col )
{
    if ( m_is3D ) { return m_table3D->getCellValue( row, col ); }
    else { return m_table->GetCellValue( row, col ); }
}

int TableEditor::GetNumberRows()
{
    if ( m_is3D ) { return m_table3D->getNumberRows(); }
    else { return m_table->GetNumberRows(); }
}

int TableEditor::GetNumberCols()
{
    if ( m_is3D ) { return m_table3D->getNumberCols(); }
    else { return m_table->GetNumberCols(); }
}

bool TableEditor::WasCellModified( int row, int col )
{
    if ( m_is3D ) { return true; }
    else { return m_table->WasCellModified( row, col ); }
}

void TableEditor::AcceptAllModifiedCells()
{
    if ( m_is3D ) {}
    else { m_table->AcceptAllModifiedCells(); }
}

void TableEditor::AcceptSingleModifiedCell( int row, int col )
{
    if ( m_is3D ) {}
    else { m_table->AcceptSingleModifiedCell( row, col ); }
}

void TableEditor::showRemapDialog()
{
    // create TableMapper descriptor
    TblMapDesc desc;
    desc.m_rowHeaderTbl     = m_rowHeaderViewer->getSymbolName();
    desc.m_rowHeaderTitle   = _T("Y");//m_rowHeaderViewer->getSymbol().getUnit();//_T("MAP Header");
    desc.m_colHeaderTbl     = m_colHeaderViewer->getSymbolName();
    desc.m_colHeaderTitle   = _T("X");//m_colHeaderViewer->getSymbol().getUnit();//_T("RPM Header");
    desc.m_name             = GetLabel() + _T(" mapper");
    desc.m_tables.Add( m_tableViewer->getSymbolName() );

    // create and show TableMapper dialog

    UserDialogFrameBase *userDialogBase = new UserDialogFrameBase(this );
    TableMapper* pMapper = new TableMapper( userDialogBase, desc, m_controller, &m_mtConfigData->getSymbolTable() );
    userDialogBase->GetSizer()->Add( pMapper, 1, wxEXPAND );

    userDialogBase->addControlPanel( pMapper, StaticControlPanel::REMAPPER_BUTTONS );
    userDialogBase->setLayout( desc.m_name );
    pMapper->updateData();

//    UserDialogFrame* frame = new UserDialogFrame( this );
//    frame->createMapper( &desc, m_controller, &m_mtConfigData->getSymbolTable() );
}

void TableEditor::showTable3D()
{
    if ( m_is3D ) { return; }

    MTConfigData* configData = m_mtConfigData;
    VemsTableDescH* tableDescH = configData->getTables();

    VemsTableDesc* tableDesc = (*tableDescH)[ m_name ];
    VemsTableDesc* coupleDesc = (*tableDescH)[ tableDesc->coupleName ];
    //m_pUserDialogManager->showDialog( tableDesc->coupleName );

    // create and show table
    UserDialogFrameBase *frame = new UserDialogFrameBase(this );
    //wxString title  = desc.title;
    StaticViewersController* table3D =
        TableEditor::newInstance(frame, *coupleDesc, m_controller, m_rtDataMgr, m_mtConfigData, m_gaugeDescMgr, getUserDialogBase() );

    int buttons = StaticControlPanel::DEFAULT_BUTTONS;
    if ( !coupleDesc->is3D ) { buttons = StaticControlPanel::DEFAULT_IMPORT_BUTTONS; }
    if ( coupleDesc->useTableMapper ) { buttons = StaticControlPanel::REMAPPER_TABLE_BUTTONS; }
    frame->addControlPanel( table3D, buttons );
    frame->setLayout( coupleDesc->title );


   // UserDialogFrame* frame = new UserDialogFrame( this );
   // frame->createTable( coupleDesc, m_controller, m_rtDataMgr, m_mtConfigData, m_gaugeDescMgr );

    if ( coupleDesc->is3D )
    {
        frame->SetInitialSize( wxSize( 600, 600 ) );
        frame->SetWindowStyle( wxMINIMIZE_BOX | wxCAPTION | wxSYSTEM_MENU | wxCLOSE_BOX  );
        frame->CenterOnScreen();
    }
}

void TableEditor::exportToFile( const wxString& file )
{
    //const uint8_t* buffer = m_tableViewer->getRawBuffer();

}


void TableEditor::importFromFile( const wxString& file )
{
    MTConfigData mtConfigData;
    FirmwareConfigLoader configLoader;
    wxArrayString fileA;
    fileA.Add(file);

    wxObject* obj = VemsContext::getObject(_T("FirmwareInfo"));
    if(obj){
        Logger::logf( LOG_MSG, _T("TableEditor::importFromFile() setDestinationFwInfo") );
        configLoader.setDestinationFwInfo((FirmwareInfo*) obj);
    }

    configLoader.setMTConfigData(&mtConfigData);
    configLoader.parse(fileA);
    configLoader.calculateTables();

    loadTable( configLoader );

    m_controller->updateRegisteredViewers();
}

void TableEditor::loadTable( FirmwareConfigLoader& configLoader )
{
    wxString cfgXAxisName = m_colHeaderViewer->getSymbol().getAlias();
    bool hasCommonAxis = FirmwareConfigLoader::hasCommonAxis(cfgXAxisName);

    if ( hasCommonAxis )
    {
        // TODO: MessageBox ( warning)
        loadTableWithoutHeaders( configLoader );
    }
    else
    {
        loadTableWithHeaders( configLoader );
    }
}

void TableEditor::loadTableWithHeaders( FirmwareConfigLoader& configLoader )
{
    /// load table
    VemsMatrix<double>* dTableUp = configLoader.getUserTable( m_tableViewer->getSymbol().getAlias() );

    if ( dTableUp )
    {
        dTableUp->flip();

        if ( m_table ) { m_table->loadFromVemsMatrix( *dTableUp ); }
        else if ( m_table3D ) { m_table3D->loadFromVemsMatrix( *dTableUp ); }

        VemsMatrixHelper::print( *dTableUp );
        delete dTableUp;
        m_tableViewer->updateModel();

    }


    /// load row header
    VemsMatrix<double>* dRowHeaderUp = configLoader.getUserTable( m_rowHeaderViewer->getSymbol().getAlias() );

    if ( dRowHeaderUp )
    {
        dRowHeaderUp->flip( VemsMatrix<double>::HORIZONTAL );
        if ( m_table ) { m_table->loadRowHeaderFromVemsMatrix( *dRowHeaderUp ); }
        else if ( m_table3D ) { m_table3D->loadRowHeaderFromVemsMatrix( *dRowHeaderUp ); }
        delete dRowHeaderUp;
        m_rowHeaderViewer->updateModel();
    }


    /// load col header
    VemsMatrix<double>* dColHeaderUp = configLoader.getUserTable( m_colHeaderViewer->getSymbol().getAlias() );

    if ( dColHeaderUp )
    {
        if ( m_table ) { m_table->loadColHeaderFromVemsMatrix( *dColHeaderUp ); }
        else if ( m_table3D ) { m_table3D->loadColHeaderFromVemsMatrix( *dColHeaderUp ); }
        delete dColHeaderUp;
        m_colHeaderViewer->updateModel();
    }
}

void TableEditor::loadTableWithoutHeaders( FirmwareConfigLoader& configLoader )
{
    Logger::logf( LOG_MSG, _T("TableEditor::loadTableWithoutHeaders()") );

    VemsMatrix<double>* configTable = configLoader.getUserTable( m_tableViewer->getSymbol().getAlias() );
    VemsMatrix<double>* configRowHeader = configLoader.getUserTable( m_rowHeaderViewer->getSymbol().getAlias() );
    VemsMatrix<double>* configColHeader = configLoader.getUserTable( m_colHeaderViewer->getSymbol().getAlias() );

    VemsMatrix<double>* editorRowHeader = NULL;
    VemsMatrix<double>* editorColHeader = NULL;

    if ( m_table )
    {
        editorRowHeader = m_table->GetRowLabels();
        editorRowHeader->flip(VemsMatrix<double>::HORIZONTAL);
        editorColHeader = m_table->GetColLabels();
    }
    else if ( m_table3D )
    {
        editorRowHeader = new VemsMatrix<double>( *m_table3D->GetRowLabels() );

        editorRowHeader->flip(VemsMatrix<double>::HORIZONTAL);
        editorColHeader = new VemsMatrix<double>( *m_table3D->GetColLabels() );
    }

    if ( configTable && configRowHeader && configColHeader && editorRowHeader && editorColHeader )
    {
        Logger::logf( LOG_MSG, _T("TableEditor::loadTableWithoutHeaders() - all table exist") );
        VemsMatrixHelper::print( *editorRowHeader );
        VemsMatrixHelper::print( *editorColHeader );

        VemsMatrixHelper::resample2D(   *configColHeader, *configRowHeader,
                                        *editorColHeader, *editorRowHeader, configTable );
        configTable->flip();

        if ( m_table ) { m_table->loadFromVemsMatrix( *configTable ); }
        else if ( m_table3D ) { m_table3D->loadFromVemsMatrix( *configTable ); }

        m_tableViewer->updateModel();
    }

    delete configTable;
    delete configRowHeader;
    delete configColHeader;

    delete editorRowHeader;
    delete editorColHeader;
    Logger::logf( LOG_MSG, _T("TableEditor::loadTableWithoutHeaders() - END") );
}


void TableEditor::change( const wxString& name )
{
    if ( m_name.IsSameAs( name ) ) { return; }

    unregisterViewers();
    UnRegisterViewer();

    WX_CLEAR_ARRAY( m_ViewerBaseA );
    m_rowHeaderViewer   = NULL;
    m_colHeaderViewer   = NULL;
    m_tableViewer       = NULL;
    if ( m_table )
    {
        delete m_table;
        m_table = NULL;
    }
    if ( m_table3D )
    {
        delete m_table3D;
        m_table3D = NULL;
    }

    init( name );
    wxString parentname;

    wxWindow * parent = GetParent();

    if( parent ) parentname = GetParent()->GetClassInfo()->GetClassName();

    if ( parentname == _T("wxFrame") ) {
        wxFrame * frame = wxDynamicCast(parent, wxFrame);

        if(frame) {
            frame->SetTitle( GetLabel() );
            frame->SetName( name );
        }
        //if ( m_is3D ) { GetParent()->SetInitialSize( wxSize( 600, 600 ) ); }
        //else { GetParent()->SetInitialSize(); }

    } else if( parentname == _T("TableEditorPanel") ) {
        TableEditorPanel * panel = wxDynamicCast(parent, TableEditorPanel);

        if(panel) {
            panel->SetTitle( GetLabel() );
            panel->SetName( name );
        }
    }

    if ( m_table3D )
        m_table3D->SetInitialSize(GetSize());

    if ( m_table )
        m_table->SetInitialSize(GetSize());


    GetGrpMember()->GetGaugeProperties().symbolName = name;

    m_groupManager->SaveSymbolname( m_grpMember->GetItemID(), name );

}

void TableEditor::OnKeyDown( wxKeyEvent& event )
{
    // int key         = event.GetKeyCode();
    // int modifier    = event.GetModifiers();

    //Logger::logf( LOG_MSG, _T("TableEditor::OnKeyDown() - key: %d, mod: %d"), key, modifier );

//    int keyCode = event.GetKeyCode();

    if ( m_table ) { m_table->ProcessEvent( event ); }
    else if ( m_table3D ) { m_table3D->ProcessEvent( event ); }
}

void TableEditor::OnKeyUp( wxKeyEvent& event )
{
   // int key         = event.GetKeyCode();
   // int modifier    = event.GetModifiers();

    //Logger::logf( LOG_MSG, _T("TableEditor::OnKeyDown() - key: %d, mod: %d"), key, modifier );

//    int keyCode = event.GetKeyCode();

    if ( m_table ) { m_table->ProcessEvent( event ); }
    else if ( m_table3D ) { m_table3D->ProcessEvent( event ); }
}

void TableEditor::OnSetFocus( wxFocusEvent& WXUNUSED(event) )
{
    Logger::logf( LOG_MSG, _T("TableEditor::OnSetFocus()") );
}

void TableEditor::OnKillFocus( wxFocusEvent& WXUNUSED(event) )
{
    Logger::logf( LOG_MSG, _T("TableEditor::OnKillFocus()") );
}

bool TableEditor::onDropFiles( const wxArrayString& filenames )
{
    size_t nFiles = filenames.GetCount();

    if ( nFiles != 1 )
    {
        return false;
    }

    wxString filename = filenames[0];

    if ( !wxFile::Exists( filename ) )
    {
        return false;
    }

    wxFileName fn( filename );

    if( (fn.GetExt() ==  _T("vemscfg") ) || fn.GetExt()  ==  _T("txt") || fn.GetExt()  ==  _T("msq") ){
       importFromFile( filename );
    }

    return true;
}


void TableEditor::moveCursorToPosition( const wxPoint& position )
{
    if ( m_table ) { m_table->moveCursorToPosition( position ); }
    else if ( m_table3D ) { m_table3D->moveCursorToPosition( position ); }
}

wxWindow * TableEditor::GetTable()
{
    if(m_is3D)
        return m_table3D;
    else
        return m_table;
}


void TableEditor::ProcessWidgetCommand(wxVariantHash& cmdHash)
{

    GaugeProperties& gauge_props = GetGrpMember()->GetGaugeProperties();

    if( cmdHash.find(_T("groupname")) == cmdHash.end() || cmdHash[_T("groupname")].GetString() != gauge_props.groupName) return;

    wxWindow* table = GetTable();

    if(cmdHash[_T("target_id")].GetLong() == GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_select")) {
        m_isSelected = true;
        DrawSelectionBitmap(table);
        return;
    }

    if(cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_move") && m_isSelected) {
        wxPointEx delta;
        delta << cmdHash[_T("deltapos")];
        Move( GetPosition() + delta);
        return;
    }

    if(cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_size") && m_isSelected) {
        wxSizeEx delta;
        delta << cmdHash[_T("deltasize")];
        SetSize( GetSize() + delta );
        return;
    }

    if(cmdHash[_T("cmd")].GetString() == _T("gauge_deselect") && m_isSelected) {

        m_isSelected = false;

        table->Refresh(false);

        return;
    }

    if( cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_click") ) {

        m_isSelected = false;

        table->Refresh(false);

        return;
    }


}
