#include "TableMapper.h"

#include <wx/wx.h>

#include "TblMapDesc.h"
#include "STArrayViewerPanel.h"
#include "STMapperViewer.h"
#include "../Symbol.h"
#include "../logger.h"
#include "../stdatamanager/STDataMgrBase.h"

TableMapper::TableMapper( wxWindow* parent,
                          const TblMapDesc& desc,
                          STDataMgrBase* stController,
                          SymbolTable* symbols )
    : wxPanel( parent )
{
    //m_pDesc         = desc;
    m_controller    = stController;
    //m_pSTController = stController;
    m_pSymbolTable  = symbols;

    init(desc);
    buildGUI(desc);
}

TableMapper::~TableMapper() {}

int TableMapper::init(const TblMapDesc& desc)
{
    //wxASSERT_MSG( ( desc ), _T("TableMapper::init() - desc is NULL!!!") );
    wxASSERT_MSG( ( m_pSymbolTable ), _T("TableMapper::init() - m_pSymbolTable is NULL!!!") );
    //wxASSERT_MSG( ( m_pSTController ), _T("TableMapper::init() - m_pSTController is NULL!!!") );



    //verify row header symbol
    SymbolTable::const_iterator IT = m_pSymbolTable->find( desc.m_rowHeaderTbl );

    wxASSERT_MSG( ( IT != m_pSymbolTable->end() ), _T("TableMapper::init() - m_rowHeaderTbl is NULL!!!") );

    m_pRowHeaderTbl = IT->second;

    //verify col header symbol
    IT = m_pSymbolTable->find( desc.m_colHeaderTbl );

    wxASSERT_MSG( ( IT != m_pSymbolTable->end() ), _T("TableMapper::init() - m_colHeaderTbl is NULL!!!") );

    m_pColHeaderTbl = IT->second;

	return 0;

}

void TableMapper::buildGUI(const TblMapDesc& desc)
{
    //main sizer
    SetSizer( new wxBoxSizer( wxVERTICAL ) );

    //header sizer
    wxBoxSizer* headerSizer = new wxBoxSizer( wxHORIZONTAL );

    //row header
    STArrayViewerPanel* pRowHeaderViewer = new STArrayViewerPanel( this, *m_pRowHeaderTbl, desc.m_rowHeaderTitle );
    headerSizer->Add( pRowHeaderViewer, 1, wxEXPAND | wxALL, 5 );

    m_ViewerBaseA.Add( pRowHeaderViewer );
    m_controller->RegisterViewer( pRowHeaderViewer );

    //col header
    STArrayViewerPanel* pColHeaderViewer = new STArrayViewerPanel( this, *m_pColHeaderTbl, desc.m_colHeaderTitle );
    headerSizer->Add( pColHeaderViewer, 1, wxEXPAND | wxALL, 5 );

    m_ViewerBaseA.Add( pColHeaderViewer );
    m_controller->RegisterViewer( pColHeaderViewer );

    GetSizer()->Add( headerSizer, 1, wxEXPAND, 0 );

    // tables for remap
    const wxArrayString& tbls = desc.m_tables;
    wxString strTables;


    for ( size_t i = 0; i < tbls.GetCount(); ++i )
    {
        if ( m_pSymbolTable->find( tbls[i] ) != m_pSymbolTable->end() )
        {
            Logger::logf( LOG_MSG, _T("TableMapper::buildGUI() - ") + tbls[i] +
                          _T("\n row = %d; col = %d"),
                          ( (*m_pSymbolTable)[ tbls[i] ] )->getArrayRowSize(),
                          ( (*m_pSymbolTable)[ tbls[i] ] )->getArrayColSize() );
        }

        STMapperViewer* v = new STMapperViewer( *(*m_pSymbolTable)[ tbls[i] ] );
        m_ViewerBaseA.Add( v );
        m_controller->RegisterViewer( v );
        strTables.Append( tbls[i] );
        if ( i < ( tbls.GetCount() - 1 ) ) { strTables.Append( _T("\n") ); }
    }

    //tables box
    wxStaticBox* pStaticBox =  new wxStaticBox( this, wxID_ANY, _T("Tables") );
    wxFont font( pStaticBox->GetFont() );
    font.SetWeight( wxFONTWEIGHT_BOLD );
    pStaticBox->SetFont( font );
    //pStaticBox->SetForegroundColour( *wxBLACK );
    wxStaticBoxSizer* pTablesSizer = new wxStaticBoxSizer( pStaticBox, wxVERTICAL );
    wxStaticText* pTextTables = new wxStaticText( this, wxID_ANY, strTables );
    pTextTables->SetFont( font );
    pTablesSizer->Add( pTextTables, 0, wxLEFT, 10 );
    GetSizer()->Add( pTablesSizer, 0, wxEXPAND | wxALL, 5 );

    refreshData();
}


void TableMapper::remap()
{
    // verify values
    if ( ! ( (STArrayViewerPanel*)m_ViewerBaseA[0] )->verify() ) { return; }
    if ( ! ( (STArrayViewerPanel*)m_ViewerBaseA[1] )->verify() ) { return; }

    // do remap
    VemsMatrix<double> oldRowHeader;
    VemsMatrix<double> newRowHeader;
    VemsMatrix<double> oldColHeader;
    VemsMatrix<double> newColHeader;

    ( (STArrayViewerPanel*)m_ViewerBaseA[0] )->getOldValues( oldRowHeader );
    ( (STArrayViewerPanel*)m_ViewerBaseA[0] )->getNewValues( newRowHeader );
    ( (STArrayViewerPanel*)m_ViewerBaseA[1] )->getOldValues( oldColHeader );
    ( (STArrayViewerPanel*)m_ViewerBaseA[1] )->getNewValues( newColHeader );

    for ( size_t i = 0; i < m_ViewerBaseA.GetCount(); ++i )
    {
        if ( i > 1 )
        {
            ( (STMapperViewer*)m_ViewerBaseA[i] )->remap( oldColHeader, oldRowHeader, newColHeader, newRowHeader );
        }

        m_ViewerBaseA[i]->updateModel();
    }

    m_controller->updateRegisteredViewers();
}
