#ifndef USERDIALOG_H
#define USERDIALOG_H

#include <wx/wx.h>
#include <wx/string.h>
#include <wx/dynarray.h>
#include <wx/hashmap.h>

#include "../common/typedef.h"

/**********************************************************
*   UserEditField
**********************************************************/
class UserEditField
{
    public:
        UserEditField(const wxString& label, const wxString& symbol, const wxString& expr);
        virtual ~UserEditField();

        wxString getLabel() const { return m_label; }
        wxString getSymbol() const { return m_symbol; }
        wxString getExpr() const { return m_expr; }

        bool isLabel() const { return m_symbol == _T(""); }
    protected:
    private:
        wxString m_label;
        wxString m_symbol;
        wxString m_expr;
};

/**********************************************************
*   UserDialog
**********************************************************/
WX_DECLARE_OBJARRAY(UserEditField, UserEditFieldArray);

class UserDialog : public wxObject
{
    DECLARE_CLASS(UserDialog)

    public:
        UserDialog(const wxString& name, const wxString& title, int offset);
        virtual ~UserDialog();

        void add(const wxString& label, const wxString& symbol, const wxString& expr);
        void addHelpTextLine( const wxString& helpText );
        void setHelpID( const wxString& helpID );

        wxString getHelpID() const { return m_HelpID; }
        wxString getHelpText() const { return m_HelpText; }
        wxString getName() const { return m_name; }
        wxString getTitle() const { return m_title; }
        int getOffset() const { return m_offset; }
        size_t getCount() const { return m_uefs.GetCount(); }
        UserEditField* operator[](uint16_t i) const;
        UserEditFieldArray& getFields() { return m_uefs; }
    protected:
    private:
        wxString    m_name;
        wxString    m_title;
        wxString    m_HelpID;
        wxString    m_HelpText;
        int         m_offset;

        UserEditFieldArray    m_uefs;
};

/**********************************************************
*   UserDialogList
**********************************************************/
WX_DECLARE_STRING_HASH_MAP(UserDialog*, StringToUserDialogHash);

class UserDialogList
{
    public:
        UserDialogList();
        virtual ~UserDialogList();

        void add(const wxString& name,const wxString& title, int offset);
        void add(const wxString& dialog, const wxString& label, const wxString& symbol, const wxString& expr);
        void setHelpID( const wxString& dialog, const wxString& helpID );
        void addHelpTextLine( const wxString& dialog, const wxString& helpText );

        UserDialog* find( const wxString& dialog );
        UserDialog* findByHelpID( const wxString& helpID );

        const StringToUserDialogHash* getUserDialogs() const { return &m_uds; }
        StringToUserDialogHash& getUserDialogs() { return m_uds; }

        void clear();

    private:
        StringToUserDialogHash m_uds;

};

#endif // USERDIALOG_H
