#include "UserDialogFrame.h"
#include <wx/wx.h>

#include "../common/FunctionMeter.h"

#include "TblMapDesc.h"
#include "UserDialog.h"
#include "TableMapper.h"
#include "TableEditor.h"
#include "CurveEditor.h"
#include "SuperCurveEditor.h"
#include "InterpolationWidget.h"
#include "InterpolationWidgetDescriptor.h"
#include "UserDialogTemplate.h"
#include "StaticControlPanel.h"
#include "VemsCurveDialogDesc.h"
#include "CheckBoxMatrixViewer.h"
#include "DialogsLayoutManager.h"
#include "UserDialogBase.h"
#include "../mtconfigdata.h"
#include "../vemsgauge/VemsTableDesc.h"
#include "../stdatamanager/STDataMgrBase.h"
//#include "../gaugedesc/GaugeDescHelper.h"
#include "../gaugedesc/GaugeProperties.h"
#include "../gui/OutputChannels.h"
#include "../gui/IgnitionOutputPanel.h"
#include "../common/FileDropTarget.h"
#include "../aimdisplay/AIMDisplayFrame.h"
#include "../aimdisplay/RoundDemoModeSettings.h"

/*
int ID_STATIC_CONTROL_PANEL = wxNewId();

BEGIN_EVENT_TABLE(UserDialogFrame, wxFrame )
    EVT_CLOSE(UserDialogFrame::OnClose)
    EVT_ACTIVATE(UserDialogFrame::OnActivate)
END_EVENT_TABLE()

UserDialogFrame::UserDialogFrame( wxWindow* parent, UserDialogBase* udManager, DialogsLayoutManager * dlglayoutmgr )
    : wxFrame( parent, wxID_ANY, wxEmptyString,  wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE | wxFRAME_FLOAT_ON_PARENT )
{
    m_stViewersCtrl    = NULL;
    //m_ctrlPanel     = NULL;

    m_udManager = udManager;
    m_dlgLayoutMgr = dlglayoutmgr;
    this->SetSizer( new wxBoxSizer( wxVERTICAL ) );
    m_menuId = 0;
#ifndef __WINCE_STANDARDSDK__
    new FileDropTarget<UserDialogFrame>( this );
#endif
}
*/
UserDialogFrame::UserDialogFrame( wxWindow* parent, ConfigValidatorBase* validator, UserDialogBase* udManager, DialogsLayoutManager * dlglayoutmgr )
: UserDialogFrameBase( parent, validator, dlglayoutmgr )
{

    m_udManager = udManager;

}



UserDialogFrame::~UserDialogFrame()
{

}

void UserDialogFrame::createDialog( const UserDialog& desc, SymbolMgr* symbolMgr, STDataMgrBase* stMgr)
{
    m_stViewersCtrl = UserDialogTemplate::newInstance(this, desc, m_udManager, symbolMgr, stMgr, m_validator, m_symbolsToMark );


/*    ((UserDialogTemplate*)m_stViewersCtrl)->setSymbolsToMark( m_symbolsToMark );

    LOGP( wxString::Format( _T("Pointer=%p"), (UserDialogTemplate*)m_stViewersCtrl ))*/

/*
    UserDialogTemplate* udt = new UserDialogTemplate();
    udt->setController( stMgr );
    udt->Create( this, ud, symbolMgr);
    udt->setUserDialogName( ud->getName() );
    udt->setUserDialogManager( m_udManager );
    udt->updateData();
    m_stViewersCtrl = udt;
    GetSizer()->Add( udt, 1, wxEXPAND );
*/
    wxString title = desc.getTitle();
    addControlPanel( m_stViewersCtrl,  StaticControlPanel::DEFAULT_IMPORT_BUTTONS );
    //setLayout( udt, ud->getTitle() );

    if( !loadLayout( title ) )
    {
        Fit();
        CenterOnScreen();
    }
    prepareAndShow( title );
}

void UserDialogFrame::createTable( const VemsTableDesc& desc, STDataMgrBase* stMgr, RTDataMgrBase* rtMgr, MTConfigData* mtconfigdata, GaugeDescManager* gaugeDescMgr )
{

    wxString title  = desc.title;
    m_stViewersCtrl = TableEditor::newInstance(this, desc, stMgr, rtMgr, mtconfigdata, gaugeDescMgr, m_udManager);

    int buttons = StaticControlPanel::DEFAULT_BUTTONS;

    if ( !desc.is3D ) { buttons = StaticControlPanel::DEFAULT_IMPORT_BUTTONS; }
    if ( desc.useTableMapper ) { buttons = StaticControlPanel::REMAPPER_TABLE_BUTTONS; }
    addControlPanel( m_stViewersCtrl, buttons );
    setLayout( title );
}


void UserDialogFrame::createCurve( VemsCurveDialogDesc* desc, STDataMgrBase* stMgr, RTDataMgrBase* realtimeController, SymbolMgr* symbolmgr)
{
    CurveEditor* curve = new CurveEditor( this, desc, symbolmgr );

    curve->init( stMgr, realtimeController );
    curve->setUserDialogName( desc->name );
    curve->setUserDialogManager( m_udManager );
    m_stViewersCtrl = curve;
    GetSizer()->Add( curve, 1, wxEXPAND );

    addControlPanel(curve, StaticControlPanel::DEFAULT_IMPORT_BUTTONS );
    setLayout( desc->title );
    m_stViewersCtrl->updateData();
}

void UserDialogFrame::createSuperCurve( SuperCurveDescriptor* superCurveDesc,  VemsCurveDialogDescH* curveDescs,
                                        STDataMgrBase* stMgr, RTDataMgrBase* realtimeController,
                                        SymbolMgr* symbolmgr)
{
    SuperCurveEditor* superCurve = new SuperCurveEditor(    this,
                                                            superCurveDesc,
                                                            curveDescs,
                                                            m_udManager,
                                                            stMgr,
                                                            realtimeController,
                                                            symbolmgr );

//    m_stViewersCtrl->updateData();
}

void UserDialogFrame::createInterpolationWidget( InterpolationWidgetDescriptor* desc, STDataMgrBase* stMgr, RTDataMgrBase* realtimeController )
{
    InterpolationWidget* widget = new InterpolationWidget( this, desc, stMgr, realtimeController );

    GetSizer()->Add( widget, 1, wxEXPAND, 0 );

    widget->setUserDialogManager( m_udManager );
//    CurveEditor* curve = new CurveEditor( this, desc );
//
//    curve->init( stMgr, realtimeController );
    widget->setUserDialogName( desc->id );
//    curve->setUserDialogManager( m_udManager );
    m_stViewersCtrl = widget;
//    GetSizer()->Add( curve, 1, wxEXPAND );
//
    //addControlPanel( desc->title, stMgr, widget, StaticControlPanel::DEFAULT_BUTTONS );
    setLayout( desc->title );
//    m_stViewersCtrl->updateData();
}

void UserDialogFrame::createMapper( const TblMapDesc& desc,
                                    STDataMgrBase* stMgr,
                                    SymbolTable* symbols )
{
    TableMapper* pMapper = new TableMapper( this,
                                            desc,
                                            stMgr,
                                            symbols );

    pMapper->setUserDialogManager( m_udManager );
    pMapper->setUserDialogName( desc.m_name );

    m_stViewersCtrl = pMapper;
    GetSizer()->Add( pMapper, 1, wxEXPAND );
    addControlPanel( pMapper, StaticControlPanel::DEFAULT_BUTTONS | StaticControlPanel::REMAP_BUTTON );
    setLayout( desc.m_name );
    m_stViewersCtrl->updateData();
}

void UserDialogFrame::create( SymbolMgr* symbolmgr, CheckBoxMatrixDesc* desc, STDataMgrBase* stMgr, CommandRequestBase* requestMgr )
{
    wxString title  = desc->m_title;

    CheckBoxMatrixViewer* viewer = new CheckBoxMatrixViewer(desc, stMgr, requestMgr, symbolmgr, this);
    viewer->setUserDialogName( desc->m_name );
    viewer->setUserDialogManager( m_udManager );
    m_stViewersCtrl = viewer;
    GetSizer()->Add( viewer, 1, wxEXPAND );
    addControlPanel(viewer, StaticControlPanel::DEFAULT_BUTTONS );
    setLayout( title );
    m_stViewersCtrl->updateData();
}


void UserDialogFrame::createOutputChannelsDialog( const wxString& title, STDataMgrBase* stMgr, CommandRequestBase* requestMgr, SymbolTable* symbols )
{
    OutputChannelsCfg* cfgpanel = new OutputChannelsCfg( this, stMgr, requestMgr, symbols, m_symbolsToMark );
    m_stViewersCtrl = cfgpanel->getController();
    m_stViewersCtrl->setUserDialogName( _T("outputChannels") );
    m_stViewersCtrl->setUserDialogManager( m_udManager );

    GetSizer()->Add( cfgpanel, 1, wxEXPAND );

    GetSizer()->Fit( this );
    GetSizer()->SetSizeHints( this );

    stMgr->updateRegisteredViewers();

    setLayout( title );
}

void UserDialogFrame::createIgnitionOutputsVisual(  const wxString& title,
                                                    STDataMgrBase* stMgr,
                                                    SymbolTable* symbols,
                                                    CommandRequestBase* requestHandler )
{
    IgnitionOutputPanel* panel = new IgnitionOutputPanel( this, stMgr, symbols, requestHandler, m_symbolsToMark );
    m_stViewersCtrl = panel->getController();
    m_stViewersCtrl->setUserDialogName( _T("ignitionOutputsVisual") );
    m_stViewersCtrl->setUserDialogManager( m_udManager );

    GetSizer()->Add( panel, 1, wxEXPAND );

    GetSizer()->Fit( this );
    GetSizer()->SetSizeHints( this );

    stMgr->updateRegisteredViewers();

    setLayout( title );
}

void UserDialogFrame::createRoundDisplaySettingsDialog( const wxString& title, STDataMgrBase* stMgr,
    RTDataMgrBase* rtMgr, SymbolTable* symbols, SymbolMgr* symMgr, FirmwareVersion fwVersion, CommandRequestBase* requestMgr )
{
    AIMDisplayPanel* panel = new AIMDisplayPanel( this, stMgr, rtMgr, symbols, symMgr, fwVersion, requestMgr );

    m_stViewersCtrl = panel->getController();
    m_stViewersCtrl->setUserDialogName( _T("roundMainDisplay") );
    m_stViewersCtrl->setUserDialogManager( m_udManager );

    GetSizer()->Add( panel, 1, wxEXPAND );

    GetSizer()->Fit( this );
    GetSizer()->SetSizeHints( this );

    stMgr->updateRegisteredViewers();

    setLayout( title );

}

void UserDialogFrame::createRoundDisplayDemoDialog(  const wxString& title, STDataMgrBase* stMgr, SymbolTable* symbols, SymbolMgr* symMgr )
{
    RoundDemoModeSettings* demomode = new RoundDemoModeSettings( this, stMgr, symbols, symMgr );

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    m_stViewersCtrl = demomode->getController();
    m_stViewersCtrl->setUserDialogName( _T("ledDisplayDemo") );
    m_stViewersCtrl->setUserDialogManager( m_udManager );

    GetSizer()->Add( demomode, 0 );

//    wxSize size = GetSizer()->CalcMin();
//
//    SetMinSize( size );

    GetSizer()->Fit( this );
    GetSizer()->SetSizeHints( this );

    setLayout( title );

    stMgr->updateRegisteredViewers();
}

/*
void UserDialogFrame::addControlPanel( StaticViewersController* controller, int buttons )
{
    m_ctrlPanel = new StaticControlPanel( this, controller, buttons, ID_STATIC_CONTROL_PANEL );
    ///stMgr->RegisterViewer( m_ctrlPanel );
    m_ctrlPanel->update();
    GetSizer()->Add( m_ctrlPanel, 0, wxEXPAND );
}


bool UserDialogFrame::loadLayout( const wxString& title )
{
    if ( !m_dlgLayoutMgr ) return false;

    wxPoint pos = wxDefaultPosition;
    wxSize size = wxDefaultSize;
    bool ismax  = false;

    if( !m_dlgLayoutMgr->LoadLayout( title, pos, size, ismax ) ) return false;

    Move( pos );
    SetInitialSize(wxSize(200,200));
    SetSize(size);
    Maximize(ismax);

    return true;
}

void UserDialogFrame::prepareAndShow( const wxString& title )
{
    SetIcon(wxIcon(_T("vemsLogoIcon2")));
    SetTitle( title );
    connectKeyDownEvent( this );
    connectKeyUpEvent( this );

    Show();
}

void UserDialogFrame::setLayout( const wxString& title )
{
    if( !loadLayout( title ) )
    {
        SetInitialSize();
        CenterOnScreen();
    }

    prepareAndShow( title );
}

void UserDialogFrame::setLayout( UserDialogTemplate* dialog, const wxString& title )
{
    if( !loadLayout( title ) )
    {
        Fit();
        CenterOnScreen();
    }

    dialog->EnableScrolling(0,1);
    dialog->SetScrollRate(10,5);

    prepareAndShow( title );
}

void UserDialogFrame::connectKeyDownEvent( wxWindow* window )
{
    if( window )
    {
        window->Connect( wxID_ANY,
                         wxEVT_KEY_DOWN,
                         wxKeyEventHandler( UserDialogFrame::OnKeyDown ),
                         (wxObject*) NULL,
                         this );

        wxWindowListNode* node = window->GetChildren().GetFirst();

        while( node )
        {
            wxWindow* child = node->GetData();
            this->connectKeyDownEvent( child );

            node = node->GetNext();
        }
    }
}

void UserDialogFrame::connectKeyUpEvent( wxWindow* window )
{
    if( window )
    {
        window->Connect( wxID_ANY,
                         wxEVT_KEY_UP,
                         wxKeyEventHandler( UserDialogFrame::OnKeyUp ),
                         (wxObject*) NULL,
                         this );

        wxWindowListNode* node = window->GetChildren().GetFirst();

        while( node )
        {
            wxWindow* child = node->GetData();
            this->connectKeyUpEvent( child );

            node = node->GetNext();
        }
    }
}

void UserDialogFrame::setStaticViewersController( StaticViewersController* controller )
{
    m_stViewersCtrl = controller;
}


void UserDialogFrame::OnKeyDown( wxKeyEvent& event )
{
    int keyCode = event.GetKeyCode();

    switch( keyCode )
    {
        case WXK_ESCAPE :
        {
            GetParent()->SetFocus();
        }break;
        case 'R' :
        {
            if( event.AltDown() ) { m_stViewersCtrl->refreshData(); }
            else { event.Skip(); }

        }break;
        case 'W' :
        {
            if( event.AltDown() )
            {
                m_stViewersCtrl->writeData();
            }
            else { event.Skip(); }
        }break;
        case 'B' :
        {
            if( event.AltDown() )
            {
                m_stViewersCtrl->burnData();
            }
            else { event.Skip(); }
        }break;
        case 'C' :
        {
            if( event.AltDown() ) { Close(); }
            else { event.Skip(); }
        }break;
        case WXK_F1 :
        {
            m_stViewersCtrl->showHelp();
            event.Skip();
        }break;
        case 'G':
        {
            m_stViewersCtrl->showTable3D();
        } break;
        case WXK_PAGEDOWN:
        case WXK_PAGEUP:
            break;
        default :
        {
            event.Skip();
        }
    }


}

void UserDialogFrame::OnKeyUp( wxKeyEvent& event )
{
    int keyCode = event.GetKeyCode();

    switch( keyCode )
    {
        case WXK_PAGEUP:
        {
            m_stViewersCtrl->changeToPreviousDialog();
        } break;
        case WXK_PAGEDOWN:
        {
            m_stViewersCtrl->changeToNextDialog();
        } break;
        default :
        {
            event.Skip();
        }
    }
}

void UserDialogFrame::OnClose( wxCloseEvent& event )
{
    if(m_udManager) m_udManager->unActivate(m_menuId);

    event.Skip();
}

bool UserDialogFrame::onDropFiles( const wxArrayString& filenames )
{
    if ( !m_stViewersCtrl )
    {
        Logger::logf( LOG_MSG, _T("UserDialogFrame::OnDropFiles() - m_stViewersCtrl is NULL") );
        return false;
    }

    return m_stViewersCtrl->onDropFiles( filenames );
}

void UserDialogFrame::OnActivate( wxActivateEvent& event )
{
    FunctionMeter fm(_T("OnActivate"));
    Logger::logf( LOG_MSG, _T("UserDialogFrame::OnActivate()") );

    event.Skip();
}
*/
