#include "VemsFlagDescEditor.h"


#include <wx/statline.h>

#include "../gaugedesc/VemsFlagDescManager.h"
#include "../customwidgets/ColorSetupPanel.h"
#include "../wxVariantHelper.h"
#include "../customwidgets/wxTextCtrlNullEx.h"
#include "../events/EditorValueChangedEvent.h"
#include "../logger.h"
#include "../DrawHelper.h"
#include "../wxSizerHelper.h"

BEGIN_EVENT_TABLE(VemsFlagDescEditor, wxFrame)

    EVT_LISTBOX(idFlagListBox,VemsFlagDescEditor::OnFlagListBox )


    EVT_BUTTON(idBtnDelFlag, VemsFlagDescEditor::OnButton)
    EVT_BUTTON(idBtnEditFlag, VemsFlagDescEditor::OnButton)
    EVT_BUTTON(idBtnAppendFlag, VemsFlagDescEditor::OnButton)
    EVT_BUTTON(idBtnCloseEditor, VemsFlagDescEditor::OnButton)
    EVT_BUTTON(idBtnAddFlagState, VemsFlagDescEditor::OnButton)
    EVT_BUTTON(idBtnSaveFlagStates, VemsFlagDescEditor::OnButton)


END_EVENT_TABLE()

VemsFlagDescEditor::VemsFlagDescEditor(wxWindow* parent, VemsFlagDescManager* mgr)
{
    //ctor
    m_stateLinePadding = 2;
    m_lastStateLineRowIdx = 0;
    m_currStateSizer = NULL;

    Create(parent, wxID_ANY, _T("VemsFlagDescEditor"));
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    CreateStateStructDesc();

    m_flagMgr = mgr;

    m_db.Open( m_flagMgr->GetLastDbFile() );

    Show( BulidGUI() );
}

VemsFlagDescEditor::~VemsFlagDescEditor()
{
    //dtor
}

void VemsFlagDescEditor::CreateStateStructDesc()
{
    m_statesGuiDesc.Add( GUIDescStructItem(_T("value"), GDSI_NUMBER, true, false, _T("Value"),_T("")) );
    m_statesGuiDesc.Add( GUIDescStructItem(_T("text"), GDSI_TEXT, true, false, _T("Text"),_T("")) );
    m_statesGuiDesc.Add( GUIDescStructItem(_T("fgcolor"), GDSI_COLOR, true, false, _T("Text color"),_T("")) );
    m_statesGuiDesc.Add( GUIDescStructItem(_T("bgcolor"), GDSI_COLOR, true, false, _T("Background color"),_T("")) );
    //m_statesGuiDesc.Add( GUIDescStructItem(_T("image"), GDSI_TEXT, true, true, _T("Image"),_T("")) );
}

bool VemsFlagDescEditor::BulidGUI()
{

    VemsFlagDescHash& fhash = m_flagMgr->GetAllDesc();

    if(fhash.size() < 1) {
        wxString msg = _T("Not found any descriptor!\nYou should open a log or vemscfg, or connect an ECU.");
        wxMessageBox(msg,_T("VemsFlag descriptor editor warning"), wxOK | wxICON_INFORMATION);
        Close();
        return false;
    }

    m_flagsListBox = new wxListBox(this, idFlagListBox, wxDefaultPosition, wxDefaultSize, 0, NULL, wxLB_EXTENDED | wxLB_NEEDED_SB | wxLB_SORT );

    for(VemsFlagDescHash::iterator it = fhash.begin(); it != fhash.end(); it++ ) {

        wxString name = it->first;

        m_flagsListBox->Append( name );

    }

    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer * editsizer = new wxBoxSizer(wxHORIZONTAL);

        wxStaticBoxSizer * sb1 = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("Flags"));

        sb1->Add( m_flagsListBox, 0, wxEXPAND | wxALL, 5 );

            wxBoxSizer * flageditsizer = new wxBoxSizer(wxVERTICAL);

                wxStaticBoxSizer * sbadd = new wxStaticBoxSizer(wxVERTICAL, this, _T("New flag"));
                m_newFlagCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
                sbadd->Add( m_newFlagCtrl, 1, wxALL, 3 );
                sbadd->Add( new wxButton(this, idBtnAppendFlag, _T("Append")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL , 3 );

            flageditsizer->Add(sbadd, 0, wxALL, 2);

                wxStaticBoxSizer * sbedit = new wxStaticBoxSizer(wxVERTICAL, this, _T("Edit name"));
                m_currentFlagCtrl = new wxTextCtrlNullEx(this, wxID_ANY, wxEmptyString);
                m_currentFlagCtrl->Enable(false); /// !! database problem. (the flag name is set to gauges db
                sbedit->Add( m_currentFlagCtrl, 1, wxALL, 3 );
                m_setFlagNameBtn = new wxButton(this, idBtnEditFlag, _T("Set"));
                m_setFlagNameBtn->Enable(false);
                sbedit->Add( m_setFlagNameBtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL , 3  );

            flageditsizer->Add(sbedit, 0, wxALL, 2);

            flageditsizer->Add( new wxButton(this, idBtnDelFlag, _T("Remove")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL , 3  );

        sb1->Add( flageditsizer, 0, wxALL, 2 );

    editsizer->Add( sb1, 0, wxALL | wxEXPAND, 5 );

        wxStaticBoxSizer * sb2 = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("States of the flags"));

        m_flagStatesPanel = new wxPanel(this, wxID_ANY);

        wxBoxSizer * fssizer = new wxBoxSizer(wxVERTICAL);

        m_flagStatesPanel->SetSizer(fssizer);

        sb2->Add(m_flagStatesPanel, 1, wxEXPAND | wxALL, 5);


    editsizer->Add( sb2, 1, wxALL | wxEXPAND, 5 );

    mainsizer->Add(editsizer, 1, wxEXPAND);

    SetSizer(mainsizer);
    mainsizer->Fit(this);
    mainsizer->SetSizeHints(this);


    m_flagsListBox->Select(0);

    LoadFlagStates( m_flagsListBox->GetString(0) );

    Connect( EditorValueChangedEvent, wxCommandEventHandler(VemsFlagDescEditor::OnChangeCurrentFlagCtrl)  );

    return true;
}

void VemsFlagDescEditor::OnChangeCurrentFlagCtrl(wxCommandEvent& evt)
{
    //m_setFlagNameBtn->Enable(true);
    m_saveStatesBtn->Enable(true);
}

void VemsFlagDescEditor::OnFlagListBox(wxCommandEvent& evt)
{
    LoadFlagStates( evt.GetString() );
}

bool VemsFlagDescEditor::LoadFlagStates(const wxString& name, bool checkBlink)
{
    //Logger::logf(LOG_MSG,_T("name=")+name);

    if( checkBlink && m_currentFlagCtrl->GetValue() == name ) return false;

    m_flagStatesPanel->DestroyChildren();
    m_currentFlagCtrl->ChangeValueEx(name);
    m_currentFlagCtrl->ResetChange();
    m_setFlagNameBtn->Enable(false);

    wxSizer* fmainsizer = m_flagStatesPanel->GetSizer();
    fmainsizer->Clear();

    wxGridBagSizer * gbsizer = new wxGridBagSizer();

    m_currStateSizer = gbsizer;

    int row = 0;

    //m_flagStatesPanel->Show(false);
    m_flagStatesPanel->Freeze();

    int padding = m_stateLinePadding;

    size_t colc = m_statesGuiDesc.GetCount();

    for(size_t a = 0; a < colc; a++ ) {
        GUIDescStructItem& gdi = m_statesGuiDesc[a];

        if( gdi.ShowOnEditor == false ) continue;

        wxStaticText * text = new wxStaticText(m_flagStatesPanel, wxID_ANY, gdi.Title);
        wxFont font = text->GetFont();
        font.SetWeight( wxFONTWEIGHT_BOLD );
        text->SetFont(font);

        gbsizer->Add( text, wxGBPosition(row,a), wxDefaultSpan, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    }

    row++;

    gbsizer->Add( new wxStaticLine(m_flagStatesPanel, wxID_ANY) , wxGBPosition(row,0), wxGBSpan( 1, colc+1 ), wxALL | wxEXPAND, padding );

    row++;

    m_stateGUIHash.clear();
    m_delBtnId2StateId.clear();

    m_currentFlagStateDesc = &(m_flagMgr->GetVemsFlagDescStates(name));

    //if( m_currentFlagStateDesc->size() == 0 ) return false;

    for(VemsFlagStates::iterator it = m_currentFlagStateDesc->begin(); it != m_currentFlagStateDesc->end(); it++) {

        VemsFlagStateDesc& state = it->second;

        AddStateLine(state,row,padding);

        row++;
    }

    fmainsizer->Add(gbsizer,0,wxEXPAND);


    wxButton * ansbtn = new wxButton(m_flagStatesPanel, idBtnAddFlagState, _T("Add new state"));
    fmainsizer->Add( ansbtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    fmainsizer->AddStretchSpacer();

    fmainsizer->Add( new wxStaticLine(m_flagStatesPanel, wxID_ANY), 0, wxALL | wxEXPAND, padding );
    m_saveStatesBtn = new wxButton(m_flagStatesPanel, idBtnSaveFlagStates, _T("Save states properties"));
    fmainsizer->Add( m_saveStatesBtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, padding );
    m_saveStatesBtn->Enable(false);

    fmainsizer->Layout();
    fmainsizer->Fit(m_flagStatesPanel);
    fmainsizer->SetSizeHints(m_flagStatesPanel);

    //m_flagStatesPanel->Show(true);

    GetSizer()->RecalcSizes();
    GetSizer()->Layout();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);

    m_flagStatesPanel->Thaw();

    return true;
}

void VemsFlagDescEditor::AddStateLine(VemsFlagStateDesc& state, int row, int padding)
{

    int state_id = state.Properties[_T("id")].GetLong();

    state.ParseColors();

    wxWindow * win = NULL;

    size_t colc = m_statesGuiDesc.GetCount();

    for(size_t a = 0; a < colc; a++ ) {
        GUIDescStructItem& gdi = m_statesGuiDesc[a];

        if( gdi.ShowOnEditor == false ) continue;

        switch( gdi.Type ) {

            case GDSI_COLOR:{
                win = new ColorSetupPanel(m_flagStatesPanel, wxVariantHelper::GetColor( state.Properties[gdi.Name] ), wxC2S_HTML_SYNTAX );
                }break;

            case GDSI_NUMBER:{
                wxTextCtrlNullEx * ctrl = new wxTextCtrlNullEx(m_flagStatesPanel, wxID_ANY, state.Properties[gdi.Name].GetString() );
                wxTextValidator validator(wxFILTER_NUMERIC);
                ctrl->SetValidator(validator);
                win = ctrl;
                }break;

            case GDSI_TEXT:{
                win = new wxTextCtrlNullEx(m_flagStatesPanel, wxID_ANY, state.Properties[gdi.Name].GetString() );
                }break;
        }

        m_currStateSizer->Add( win, wxGBPosition(row,a), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, padding );

        m_stateGUIHash[state_id].Add( GUIDescGUIItem(gdi.Name, gdi.Type, win) );

    }

    int did = wxNewId();
    wxBitmapButton * btn = new wxBitmapButton(m_flagStatesPanel, did, DrawHelper::GetBitmap(_T("remove-16.png")) );
    m_currStateSizer->Add( btn, wxGBPosition(row,colc), wxDefaultSpan, wxALL, padding );
    btn->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( VemsFlagDescEditor::OnStateDelButton ), NULL, this );
    btn->SetToolTip(_T("Delete state"));

    m_delBtnId2StateId[did] = state_id;

    m_lastStateLineRowIdx = row;
}

wxString VemsFlagDescEditor::GetSelectedFlag()
{
    wxArrayInt arr;
    m_flagsListBox->GetSelections(arr);
    if(arr.GetCount() < 1) return wxEmptyString;

    return m_flagsListBox->GetString( arr[0] );
}

void VemsFlagDescEditor::OnStateDelButton(wxCommandEvent& evt)
{

    int id = evt.GetId();
    if( m_delBtnId2StateId.find(id) == m_delBtnId2StateId.end() ) return;

    int state_id = m_delBtnId2StateId[id];

    if( m_stateGUIHash.find(state_id) == m_stateGUIHash.end() ) return;

    wxString currname = GetSelectedFlag();

    if( currname == wxEmptyString ) return;

    VemsFlagStates& states = m_flagMgr->GetVemsFlagDescStates( currname );

    size_t size = states.size();

    for(VemsFlagStates::iterator it = states.begin(); it != states.end(); it++) {
        VemsFlagStateDesc& state = it->second;
        if( state.Properties[_T("id")].GetLong() != state_id ) continue;

        states.erase(it);
        break;
    }

    if( size == states.size() ) return;

    m_db.ExecuteUpdate( wxString::Format(_T("delete from states where id=%i;"),state_id) );

    GUIDescGUIItemArray& garr = m_stateGUIHash[state_id];

    if(garr.GetCount() < 1 || garr[0].Ctrl == NULL ) return;

    wxWindow * win = garr[0].Ctrl;

    if( m_currStateSizer == NULL ) return;

    wxSizerHelper::DeleteRow(m_currStateSizer, win);

    m_stateGUIHash.erase(state_id);

    m_lastStateLineRowIdx--;

    RefreshStatePanel();
}

void VemsFlagDescEditor::RefreshStatePanel()
{
    m_currStateSizer->RecalcSizes();

    m_currStateSizer->Layout();

    wxSizer* fmainsizer = m_flagStatesPanel->GetSizer();
    fmainsizer->Layout();
    fmainsizer->Fit(m_flagStatesPanel);
    fmainsizer->SetSizeHints(m_flagStatesPanel);

    m_flagStatesPanel->Refresh(false);

    GetSizer()->RecalcSizes();
    GetSizer()->Layout();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
}

void VemsFlagDescEditor::OnButton(wxCommandEvent& evt)
{
    switch( evt.GetId() ) {

        case idBtnDelFlag:
            DelSelectedFlags();
            break;

        case idBtnEditFlag:
            SaveFlagName();
            break;

        case idBtnAppendFlag:
            AddNewFlag();
            break;

        case idBtnCloseEditor:
            Close();
            break;

        case idBtnAddFlagState:
            AddNewState();
            break;

        case idBtnSaveFlagStates:
            SaveFlagStates();
            break;

    }
}

void VemsFlagDescEditor::SaveFlagStates()
{
    m_db.Begin();

    for(GUIDescGUIItemArrayIntHash::iterator it = m_stateGUIHash.begin(); it != m_stateGUIHash.end(); it++) {
        long state_id = it->first;
        GUIDescGUIItemArray& descarr = it->second;

        bool isChangedState = false;

        wxString qs = _T("update states set ");

        VemsFlagStateDesc& sdesc = m_flagMgr->GetVemsFlagStateDescById(state_id);

        for(size_t a=0; a<descarr.GetCount(); a++) {
            GUIDescGUIItem& guidesc = descarr[a];

            if( !guidesc.IsChanged() ) continue;

            wxVariant val = guidesc.GetValue();

            qs += guidesc.Name + _T(" = '");

            qs += wxVariantHelper::MakeString(val) + _T("',");

            sdesc.Properties[guidesc.Name] = val;

            guidesc.ResetChange();

            isChangedState = true;
        }

        qs.RemoveLast();

        qs += wxString::Format(_T(" where id=%i;"),state_id);

        Logger::logf(LOG_MSG,_T("qs=")+qs);

        if(isChangedState)
            m_db.ExecuteUpdate(qs);
    }

    m_db.Commit();

    m_saveStatesBtn->Enable(false);
}

void VemsFlagDescEditor::DelSelectedFlags()
{
    wxArrayInt arr;
    m_flagsListBox->GetSelections(arr);

    if(!arr.GetCount()) return;

//    arr.Sort();

    wxString msg = wxString::Format(_T("Do you really want to delete the %i selected flag(s)?"),arr.GetCount());

    if( wxMessageBox(msg,_T("Confirm"), wxYES_NO | wxICON_QUESTION ) == wxNO ) return;

    VemsFlagDescHash& adesc = m_flagMgr->GetAllDesc();

    m_db.Begin();

    for(int a = arr.GetCount()-1; a >= 0; a-- ) {
        int idx = arr[a];
        wxString selectedflagname = m_flagsListBox->GetString(idx);

        if( !adesc.erase(selectedflagname) ) continue;

        int flag_id = m_db.ExecuteScalar(wxString::Format(_T("select id from flags where name='%s';"),selectedflagname.c_str()));

        if(flag_id > 0) {
            m_db.ExecuteUpdate(wxString::Format(_T("delete from flags where id=%i;"),flag_id));
            m_db.ExecuteUpdate(wxString::Format(_T("delete from states where flag_id=%i;"),flag_id));
        }

        m_flagsListBox->Delete(idx);
    }

    m_db.Commit();

    m_flagsListBox->Select(0);

    LoadFlagStates( GetSelectedFlag() );
}

void VemsFlagDescEditor::AddNewFlag()
{
    wxString newflagname = m_newFlagCtrl->GetValue();

    if( newflagname == wxEmptyString ) return;

    VemsFlagDescHash& adesc = m_flagMgr->GetAllDesc();

    if(adesc.find(newflagname) != adesc.end()) {
        wxMessageBox(_T("This name is engaged.\nPlease choose an other one."),_T("Warning"));
        return;
    }

    m_db.ExecuteUpdate( wxString::Format(_T("insert into flags (name) values ('%s');"),newflagname.c_str()) );

    long flag_id = m_db.GetLastRowId().ToLong();

    VemsFlagDesc fdesc(flag_id,newflagname);

    adesc[newflagname] = fdesc;

    m_flagsListBox->AppendString( newflagname );
}

void VemsFlagDescEditor::SaveFlagName()
{

    wxString currflagname = GetSelectedFlag();

    wxString newflagname = m_currentFlagCtrl->GetValue();

    if( currflagname == newflagname ) return;

    wxArrayInt arr;
    m_flagsListBox->GetSelections(arr);
    if(arr.GetCount() < 1) return;

    m_db.ExecuteUpdate(wxString::Format(_T("update flags set name = '%s' where name = '%s';"),newflagname.c_str(),currflagname.c_str()));

    m_flagsListBox->SetString( arr[0], newflagname );

    m_currentFlagCtrl->ResetChange();
    m_setFlagNameBtn->Enable(false);

}

void VemsFlagDescEditor::AddNewState()
{
    wxString currname = GetSelectedFlag();

    if( currname == wxEmptyString ) return;

    VemsFlagDesc& desc = m_flagMgr->GetVemsFlagDesc(currname);

    long flag_id = desc.Properties[_T("id")].GetLong();

    int maxvalue = m_db.ExecuteScalar(wxString::Format(_T("select max(value) from states where flag_id=%i;"),flag_id));

    maxvalue++;

    m_db.ExecuteUpdate(wxString::Format(_T("insert into states (flag_id,value) values(%i,%i);"),flag_id,maxvalue));

    long state_id = m_db.GetLastRowId().ToLong();

    VemsFlagStateDesc sdesc;

    sdesc.Properties[_T("id")] = state_id;
    sdesc.Properties[_T("flag_id")] = flag_id;

    wxVariantHelper::SetColor(sdesc.Properties[_T("bgcolor")],wxColor(0,0,0,255));
    wxVariantHelper::SetColor(sdesc.Properties[_T("fgcolor")],wxColor(0,0,0,255));
    sdesc.SetValue(maxvalue);

    desc.States[ maxvalue ] = sdesc;

    AddStateLine(sdesc, m_lastStateLineRowIdx+1, m_stateLinePadding);

    RefreshStatePanel();
}

