#include "WindowTipperAgg.h"

#include <wx/dcbuffer.h>

#include "../agg/wxAgg2.h"
#include "../DirHelper.h"
#include "../DrawHelper.h"

#include "../logger.h"

BEGIN_EVENT_TABLE(WindowTipperAgg, wxFrame)

    EVT_TIMER(idTimer,WindowTipperAgg::OnTimer)

    EVT_PAINT(WindowTipperAgg::OnPaint)

    EVT_LEAVE_WINDOW(WindowTipperAgg::OnLeave)
    EVT_ENTER_WINDOW(WindowTipperAgg::OnLeave)

END_EVENT_TABLE()

WindowTipperAgg::WindowTipperAgg(wxWindow * parent, int id, const wxPoint &pos)
    :m_timer(this,idTimer)
{
    //ctor
    m_parent = parent;
    Create(parent, id, _T(""), pos, wxSize(-1,-1), wxFRAME_TOOL_WINDOW | wxFRAME_FLOAT_ON_PARENT | wxBORDER_NONE );


    m_isDelayedShow = false;

    m_faceBmp = NULL;
}

WindowTipperAgg::~WindowTipperAgg()
{
    //dtor
}

void WindowTipperAgg::OnLeave(wxMouseEvent& evt)
{
    HideWin();
}

void WindowTipperAgg::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);

    dc.Clear();

    if(m_faceBmp)
        dc.DrawBitmap(*m_faceBmp,0,0);
}

void WindowTipperAgg::HideWin()
{
    Hide();
    m_timer.Stop();
}

void WindowTipperAgg::SetMessage(const wxString& msg)
{
    wxSize cs = DrawLabel(msg);
    SetSize(cs);
}

wxSize WindowTipperAgg::DrawLabel(const wxString& msg)
{
    if(m_msg == msg) return m_faceBmp ? wxSize(m_faceBmp->GetWidth(),m_faceBmp->GetHeight()) : wxDefaultSize;

    m_msg = msg;

    double font_height = 11;
    double padding = 2;
    double border = 1;

    wxAgg2 dummy(10,10);
    dummy.loadTTFont(DirHelper::GetPath(dFONTS)+_T("arial.ttf"),font_height,0);

    wxRealRect textbox = dummy.getTTFTextBound(msg,0);

    textbox.Inflate(padding+border);

    wxSize canvasSize = textbox.GetOuterBox().GetSize();

    wxAgg2 canvas( canvasSize );
    canvas.loadTTFont(DirHelper::GetPath(dFONTS)+_T("arial.ttf"),font_height,0);

    wxColor bgcolor = wxSystemSettings::GetColour(wxSYS_COLOUR_INFOBK);

    wxAggPen* pen = canvas.getPen();
    wxAggBrush* brush = canvas.getBrush();

    brush->SetColor(bgcolor);
    wxColor endcolor = DrawHelper::ModLuminance(bgcolor,-0.2);
    brush->SetEndColor(endcolor);
    brush->SetStyle( wxAGGBRUSH_GRADIENT );
    brush->SetGradientOrientation(90);

    pen->SetColor(wxColor(0,0,0,255));
    pen->SetWidth(border);

    canvas.drawRectangle(border/2,border/2,(double)canvasSize.x-border,(double)canvasSize.y-border);

    pen->SetColor(bgcolor);
    canvas.drawTTFText( m_msg, padding+border+1, textbox.GetBottom()-border+1 );

    pen->SetColor(wxColor(0,0,0));
    canvas.drawTTFText( m_msg, padding+border, textbox.GetBottom()-border );

    if(m_faceBmp) delete m_faceBmp;
    m_faceBmp = canvas.renderToBitmap();

    return canvasSize;
}

void WindowTipperAgg::ShowWindow(int showlenms, bool moveToParentCorner )
{
    Show(true);
    m_timer.Start(showlenms,wxTIMER_ONE_SHOT);
    if(!m_parent) return;
    if(moveToParentCorner) Move( m_parent->GetScreenPosition() );
    m_parent->SetFocus();
}

void WindowTipperAgg::ShowWindow(const wxPoint& pos, int showlenms, int delayedms)
{
    Move(pos);

    m_timer.Start(delayedms ? delayedms : showlenms,wxTIMER_ONE_SHOT);
    m_isDelayedShow = delayedms;
    Show(!m_isDelayedShow);

    m_showLen = showlenms;
}

void WindowTipperAgg::OnTimer(wxTimerEvent &evt)
{
    if(m_isDelayedShow) {
        m_timer.Start(m_showLen,wxTIMER_ONE_SHOT);
        m_isDelayedShow = false;
        Show();
    } else {
    Hide();
}
}
