#include "ColumnsGauge.h"

#include <wx/dcbuffer.h>

#include "GaugeGroupManager.h"
#include "../events/UpdateViewerEvent.h"
#include "../vemsdialog/MultiGaugeDescEditor.h"
#include "../agg/wxAgg2.h"
#include "../wxHSLColor.h"
#include "../DrawHelper.h"
#include "../DirHelper.h"
#include "../configmanager/ConfigHelper2.h"
#include "../wxRectEx.h"
#include "../wxPointEx.h"
#include "../common/FunctionMeter.h"

BEGIN_EVENT_TABLE(ColumnsGauge, VemsWidgetBase)

    EVT_PAINT(ColumnsGauge::OnPaint)

    EVT_MENU_RANGE(idMenuOpenDescEditor,idMenuEND,ColumnsGauge::OnMenu)

    EVT_TIMER(idTestTimer, ColumnsGauge::OnTestTimer)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(ColumnsGauge, VemsWidgetBase);

enum ColumnsGaugeProperties {
    CG_SYMBOLNAME,
    CG_SYMBOLTEXTFORMAT,
    CG_CHANGED,
    CG_SYMBOLAVG,
    CG_SYMBOLUNIT,
    CG_SYMBOLMAXVALUE,
    CG_SYMBOLMINVALUE,
    CG_SYMBOLTITLE,
    CG_COLPOS,
    CG_COLBMP,
    CG_VALUEBMP
};

ColumnsGauge::ColumnsGauge()
{
    m_acSize = 0;
    m_border = 0;
    m_colPadding = 0;
    m_faceBitmap = 0;
    m_gradient_lum_diff = 0;
    m_isColShowAvg = 0;
    m_isRegistered = 0;
    m_isShowAvg = 0;
    m_isUseAgg = 0;
    m_needRefresh = 0;
    m_profileID = 0;
    m_stickBmp = 0;

}

ColumnsGauge::ColumnsGauge(wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop)
    :m_testTimer(this, idTestTimer)
{
    //ctor
    m_acSize = 16;
    long tmp = 1;
    m_border = 1;
    m_colPadding = 1;
    m_isShowAvg = true;

    prop.descName.ToLong(&tmp);

    m_profileID = tmp;

    m_needRefresh = true;
    m_faceBitmap = NULL;
    m_stickBmp = NULL;
    m_gradient_lum_diff = -0.3;
    m_isRegistered = false;

    m_isUseAgg = true;
    m_isColShowAvg = false;

    ConfigBase* mainCfg = ConfigHelper2::getConfig();
    mainCfg->setPath(_T("/ColumnsGauge"));
    m_isUseAgg = mainCfg->read(_T("AdvancedGraphics"),(long)m_isUseAgg);
    m_acSize = mainCfg->read(_T("AvgHistoryLength"),(long)m_acSize);
    m_isColShowAvg = mainCfg->read(_T("UseColumnsForAvg"),(long)m_isColShowAvg);

    Logger::logf(LOG_MSG,_T("m_isUseAgg=%i"),m_isUseAgg);


    CreateWidget(parent ,id , gdmgr , prop);
    m_hasOwnDescriptorMgr = true;
}

ColumnsGauge::~ColumnsGauge()
{
    ptrdel(m_faceBitmap);
    ptrdel(m_stickBmp);

    clearAll();
}

void ColumnsGauge::OnSize(wxSizeEvent& evt)
{
    UpdateSize();
    evt.Skip();
}

void ColumnsGauge::InitGauge()
{
    RegisterSymbols();

    RedrawWidget();

    Connect( wxEVT_SIZE, wxSizeEventHandler(ColumnsGauge::OnSize), NULL, this );
}

double ColumnsGauge::CalcLimitsSize(int type, double col_width, wxAgg2& aggCanvas)
{
    wxVariantHashArray& symbols = GetDesc().GetSymbols();
    double font_height = aggCanvas.getFontHeight();
    double min_height = 200;
    size_t symbols_count = symbols.GetCount();

    for(size_t a=0; a<symbols_count; a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();

        wxVariantIntHash& cprops = m_customProps[name];

        double limitval = cprops[type].GetDouble();

        wxString slimitval = wxString::Format( cprops[CG_SYMBOLTEXTFORMAT].GetString() , limitval );

        wxRealRect text_rect = aggCanvas.getTTFTextBound( slimitval );

    // adjust font height for the correct width
        double tmph = col_width / text_rect.w * font_height;

        if(min_height > tmph) min_height = tmph;
    }

    return min_height;
}


int ColumnsGauge::CalcLimitsSize(int type, int col_width, wxDC& dc)
{
    wxVariantHashArray& symbols = GetDesc().GetSymbols();
    int font_height = dc.GetFont().GetPointSize();
    int min_height = 200;
    size_t symbols_count = symbols.GetCount();

    for(size_t a=0; a<symbols_count; a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();

        wxVariantIntHash& cprops = m_customProps[name];



        double limitval = cprops[type].GetDouble();

        wxString slimitval = wxString::Format( cprops[CG_SYMBOLTEXTFORMAT].GetString() , limitval );

        wxSize textSize = dc.GetTextExtent( slimitval );

    // adjust font height for the correct width
        int tmph = (int)ceil((double)col_width / (double)textSize.x * (double)font_height);

        if(min_height > tmph) min_height = tmph;

    }

    return min_height;
}

void ColumnsGauge::DrawSymbolColumn(const wxString& name, wxAgg2& aggCanvas, double col_width, int a)
{
//    FUNCTIME


// init value drawing
    wxVariantIntHash& cprops = m_customProps[name];
    wxColor barColor = m_theme->ColorSymbols[_T("normal")];
    MultiSymbolGaugeDesc& desc = GetDesc();
    aggCanvas.clear(wxColor(0,0,0,0));

    wxVariant gmin = desc.GetProp(_T("min_value"));
    wxVariant gmax = desc.GetProp(_T("max_value"));

    wxAggPen* pen = aggCanvas.getPen();
    wxAggBrush* brush = aggCanvas.getBrush();

    double max = gmax == wxNullVariant ? cprops[CG_SYMBOLMAXVALUE].GetDouble() : gmax.GetDouble();
    double min = gmin == wxNullVariant ? cprops[CG_SYMBOLMINVALUE].GetDouble() : gmin.GetDouble();
    wxString stitle = cprops[CG_SYMBOLTITLE].GetString();
    wxString text_format = cprops[CG_SYMBOLTEXTFORMAT].GetString();

// calc value column pos & size
    wxRealRect colRect;
    double col_avalHeight = aggCanvas.getSize().y; //(m_widgetBorderBox.h - m_colPadding*2);
    colRect.w = col_width;
    colRect.x = 0;//m_border + m_colPadding + ( (colRect.w+m_colPadding) * a );
    long col_pos = (int)floor(m_border + m_colPadding + ( (colRect.w+m_colPadding) * a ));
    //colRect.x = 0;
    colRect.h = col_avalHeight;// * (( cval - min ) / ( max - min ));
    colRect.y = 0;//m_widgetBorderBox.y + m_colPadding + ( col_avalHeight - colRect.h );

    cprops[CG_COLPOS] = col_pos;

// value column drawing
    pen->SetStyle( wxAGGPEN_TRANSPARENT );
    brush->SetStyle( wxAGGBRUSH_GRADIENT );
    brush->SetGradientOrientation( 0 );

    //draw colored areas
    ColoredAreaRefArray& caArr = m_coloredAreas[name];

    wxRealRectArray caRectArr;

    for(size_t c = 0; c < caArr.GetCount(); c++) {
        ColoredArea& ca = caArr[c];
        wxColor color = m_theme->ColorSymbols[ca.ColorName];

    // calc colored area column size & pos
        wxRealRect caRect;
        caRect.x = colRect.x;
        caRect.w = col_width;
        caRect.h = col_avalHeight * ((ca.maxValue - ca.minValue) / (max - min));
        //caRect.y = m_widgetBorderBox.y + m_colPadding + col_avalHeight - (col_avalHeight * (( ca.maxValue - min ) / ( max - min )));
        caRect.y = col_avalHeight - (col_avalHeight * (( ca.maxValue - min ) / ( max - min )));

        //Logger::logf(LOG_MSG,_T("ca[%i]")+caRect.GetAsString(2),c);

        brush->SetColor(color);
        brush->SetEndColor( DrawHelper::ModLuminance(color,m_gradient_lum_diff) );// gradient for 3D effect
        aggCanvas.drawRectangle( caRect );
        caRectArr.Add(caRect);
    }

    aggCanvas.clearClipping();

// draw value column(s)
    brush->SetColor( barColor );
    brush->SetEndColor( DrawHelper::ModLuminance(barColor,m_gradient_lum_diff) );// gradient for 3D effect

    wxRealRectArray valueRects = colRect.Subtract( caRectArr ); // remove colored area columns from value column rect

    for(size_t j = 0; j<valueRects.GetCount();j++)
        aggCanvas.drawRectangle( valueRects[j] );

    wxBitmap* obmp = GetBitmap(cprops,CG_COLBMP);
    ptrdel( obmp );
    cprops[CG_COLBMP] = aggCanvas.renderToBitmap();

}

void ColumnsGauge::DrawSymbolStick()
{
// draw stick

    MultiSymbolGaugeDesc& desc = GetDesc();

    wxVariantHashArray& symbols = desc.GetSymbols();

    double col_width = CalcColWidth(symbols.GetCount());
    double height = 4.0;

    double border = 1;

    wxSize canvas_size = wxRealSize(col_width, height).Ceil();

    if( !canvas_size.x || !canvas_size.y ) return;

    wxAgg2 aggCanvas(canvas_size);

    wxRealRect stickRect;
    stickRect.x = border/2;
    stickRect.w = col_width - border;
    stickRect.y = border/2;
    stickRect.h = height - border;

    wxAggPen* pen = aggCanvas.getPen();
    wxAggBrush* brush = aggCanvas.getBrush();

    int alpha = m_isColShowAvg ? 90 : 255;

    pen->SetColor( wxColor(255,255,255,alpha) );

    wxColor stickColor = wxColor( m_theme->ColorSymbols[_T("normal")] );
    DrawHelper::SetColorA(stickColor,alpha);
    brush->SetStyle(wxAGGBRUSH_GRADIENT);
    brush->SetColor(stickColor);
    brush->SetEndColor( DrawHelper::ModLuminance(stickColor,m_gradient_lum_diff) );// gradient for 3D effect

    aggCanvas.drawRectangle(stickRect);

    ptrdel(m_stickBmp);

    m_stickBmp = aggCanvas.renderToBitmap();
}

void ColumnsGauge::DrawWidgetMainFace()
{
    //FUNCTIME

    MultiSymbolGaugeDesc& desc = GetDesc();

    wxVariantHashArray& symbols = desc.GetSymbols();

    size_t symbols_count = symbols.GetCount();

//    if(!symbols_count) return;


    wxVariant gmin = desc.GetProp(_T("min_value"));
    wxVariant gmax = desc.GetProp(_T("max_value"));
// init canvas
    wxAgg2 aggCanvas(this);

    double w = (double)GetClientSize().x;
    double h = (double)GetClientSize().y;

    wxAggPen* pen = aggCanvas.getPen();

    double col_width = CalcColWidth(symbols_count);

    double font_height = col_width * 0.6;
    if(font_height < 1) font_height = 10;
    aggCanvas.loadTTFont( DirHelper::GetPath(dFONTS) + _T("arial.ttf"), font_height , 1.0 );

// calc min/max labels sizes
    double limits_font_size = col_width > 1 ? CalcLimitsSize(CG_SYMBOLMAXVALUE, col_width, aggCanvas) : 15;

    //Logger::logf(LOG_MSG,_T("limits_font_size=%f"),limits_font_size);

// draw border
    m_widgetBorderBox = wxRealRect(m_border/2,m_border/2 + ceil(limits_font_size),w-m_border,h-m_border-2.0*ceil(limits_font_size));
    pen->SetColor( m_theme->BorderColor );
    aggCanvas.drawRectangle( m_widgetBorderBox );

    for(size_t a=0; a<symbols_count; a++ ) {

        wxString name = symbols[a][_T("symbol_name")].GetString();

        wxVariantIntHash& cprops = m_customProps[name];

        wxString stitle = cprops[CG_SYMBOLTITLE].GetString();
        wxString text_format = cprops[CG_SYMBOLTEXTFORMAT].GetString();

        double max = gmax == wxNullVariant ? cprops[CG_SYMBOLMAXVALUE].GetDouble() : gmax.GetDouble();
        double min = gmin == wxNullVariant ? cprops[CG_SYMBOLMINVALUE].GetDouble() : gmin.GetDouble();

        double col_x = m_border + m_colPadding + ( (col_width+m_colPadding) * a );

    // draw symbol title
        wxColor titlecolor = m_theme->DivColor;
        DrawHelper::SetColorA(titlecolor,180);
        pen->SetColor( titlecolor );
        aggCanvas.setFontHeight( font_height );
        wxRealRect titleRect = aggCanvas.getTTFTextBound(stitle, 270 );
        double text_x = col_x + col_width - (col_width-fabs(titleRect.w))/2;
        aggCanvas.drawTTFText(stitle, text_x, m_widgetBorderBox.y + m_colPadding + m_border + 5 + fabs(titleRect.h) , 270 );


    // draw min & max values
        pen->SetColor( m_theme->DivColor );
        aggCanvas.setFontHeight( limits_font_size );
        if(gmax == wxNullVariant) {
            wxString smaxVal =  wxString::Format(text_format,max);
            wxRealRect smaxValRect = aggCanvas.getTTFTextBound(smaxVal);
            aggCanvas.drawTTFText( smaxVal, col_x + (col_width - smaxValRect.w)/2, limits_font_size - 1 );
        }

        if(gmin == wxNullVariant) {
            wxString sminVal =  wxString::Format(text_format,min);
            wxRealRect sminValRect = aggCanvas.getTTFTextBound(sminVal);
            aggCanvas.drawTTFText( sminVal, col_x + (col_width - sminValRect.w)/2, m_widgetBorderBox.GetBottom() + limits_font_size - 1 );
        }

    }

    pen->SetColor( m_theme->DivColor );
    aggCanvas.setFontHeight( limits_font_size );

    if(gmax != wxNullVariant) {
        wxString smaxVal = wxString::Format(_T("%.2f"),gmax.GetDouble());
        wxRealRect smaxValRect = aggCanvas.getTTFTextBound(smaxVal);
        aggCanvas.drawTTFText( smaxVal, m_border + (w - smaxValRect.w)/2, limits_font_size - 1 );
    }

    if(gmin != wxNullVariant) {
        wxString sminVal = wxString::Format(_T("%.2f"),gmin.GetDouble());
        wxRealRect sminValRect = aggCanvas.getTTFTextBound(sminVal);
        aggCanvas.drawTTFText( sminVal, m_border + (w - sminValRect.w)/2, m_widgetBorderBox.GetBottom() + limits_font_size - 1 );
    }

    ptrdel(m_faceBitmap);
    m_faceBitmap = aggCanvas.renderToBitmap();
}

void ColumnsGauge::DrawSymbolColumns()
{
    //FUNCTIME

    MultiSymbolGaugeDesc& desc = GetDesc();

    wxVariantHashArray& symbols = desc.GetSymbols();

    size_t symbols_count = symbols.GetCount();

    if(!symbols_count) return;

// init canvas
    double col_width = CalcColWidth(symbols_count);

    wxSize canvas_size((int)ceil(col_width), (int)ceil( m_widgetBorderBox.h - 2*m_colPadding ));

    if( !canvas_size.x || !canvas_size.y ) return;

    wxAgg2 aggCanvas( canvas_size );

    double font_height = col_width * 0.6;
    aggCanvas.loadTTFont( DirHelper::GetPath(dFONTS) + _T("arial.ttf"), font_height , 1.0 );

    for(size_t a=0; a<symbols_count; a++ ) {

        wxString name = symbols[a][_T("symbol_name")].GetString();

        DrawSymbolColumn(name, aggCanvas, col_width, a);

    }

}

void ColumnsGauge::DrawNumberValuesAgg()
{
    MultiSymbolGaugeDesc& desc = GetDesc();

    wxVariantHashArray& symbols = desc.GetSymbols();

    size_t symbols_count = symbols.GetCount();

    if(!symbols_count) return;

// init canvas
    double col_width = CalcColWidth(symbols_count);

    wxSize canvas_size((int)ceil(col_width), (int)ceil( m_widgetBorderBox.h - 2*m_colPadding ));

    if( !canvas_size.x || !canvas_size.y ) return;

    wxAgg2 aggCanvas(canvas_size);

    double font_height = col_width * 0.6;
    aggCanvas.loadTTFont( DirHelper::GetPath(dFONTS) + _T("arial.ttf"), font_height , 1.0 );

    for(size_t a=0; a<symbols_count; a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();

        wxVariantIntHash& cprops = m_customProps[name];

        if( cprops[CG_CHANGED].GetBool() ) DrawNumberValue(name,aggCanvas,col_width);
    }
}

void ColumnsGauge::DrawNumberValue(const wxString& name, wxAgg2& aggCanvas, double col_width)
{

//LOGP(_T("name=")+name+wxString::Format(_T(", val=%f"),m_hash[name]));

// init text drawing
    wxVariantIntHash& cprops = m_customProps[name];
    aggCanvas.clear(wxColor(0,0,0,0));

    wxAggPen* pen = aggCanvas.getPen();
    wxString text_format = cprops[CG_SYMBOLTEXTFORMAT].GetString();

    double font_height = aggCanvas.getFontHeight();

    pen->SetStyle( wxAGGPEN_SOLID );
    wxString sval = wxString::Format( text_format, m_hash[name] );
    sval += _T(" ") + cprops[CG_SYMBOLUNIT].GetString();

    double text_rotate = 270;
    wxRealRect svalRect = aggCanvas.getTTFTextBound(sval, text_rotate );
    double text_x = col_width - (col_width-fabs(svalRect.w))/2;
    double text_y = aggCanvas.getSize().y - 5.0;

    //aggCanvas.setClippingRect( svalRect.GetOuterBox() );

// draw value with dropped shadow
    int ds_diff = (int)floor(font_height / 10); // drop shadow inner layer shift distance
    chkminmax(ds_diff,1,4);

    pen->SetColor( wxColor(0,0,0,250) );
    aggCanvas.drawTTFText(sval, text_x + ds_diff, text_y + ds_diff, text_rotate );

    pen->SetColor( wxColor(255,255,255,255) );
    aggCanvas.drawTTFText(sval, text_x, text_y, text_rotate );

    wxBitmap* obmp = GetBitmap(cprops,CG_VALUEBMP);
    ptrdel( obmp );
    obmp = aggCanvas.renderToBitmap();
    cprops[CG_VALUEBMP] = obmp;

    //obmp->SaveFile(_T("numval_")+name+_T(".png"),wxBITMAP_TYPE_PNG);
}

void ColumnsGauge::CacheCustomProps(wxVariantHash& symbolHash)
{
    wxString name = symbolHash[_T("symbol_name")].GetString();

    wxVariantIntHash& cprops = m_customProps[name];

    if(cprops.find(CG_SYMBOLMAXVALUE) == cprops.end())
        cprops[CG_SYMBOLMAXVALUE] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName( name, _T("Hi") );

    if(cprops.find(CG_SYMBOLMINVALUE) == cprops.end())
        cprops[CG_SYMBOLMINVALUE] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName( name, _T("Lo") );

    if(cprops.find(CG_SYMBOLTEXTFORMAT) == cprops.end())
        cprops[CG_SYMBOLTEXTFORMAT] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName( name, _T("numformat") );

    if(cprops.find(CG_SYMBOLUNIT) == cprops.end())
        cprops[CG_SYMBOLUNIT] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName( name, _T("unit") );

    if(cprops.find(CG_SYMBOLTITLE) == cprops.end()) {
        wxString stitle = symbolHash[_T("custom_symbol_title")].GetString();
        cprops[CG_SYMBOLTITLE] = stitle.Len() ? stitle : m_gaugeDescMgr->GetGaugeSymbolTitle(name);
    }

    if(m_coloredAreas.find(name) == m_coloredAreas.end())
        m_gaugeDescMgr->GetColoredAreasBySymbolName(name, m_coloredAreas[name]);
}

wxBitmap* ColumnsGauge::GetBitmap(const wxVariantIntHash& hash, int type)
{
    if(hash.find(type) == hash.end()) return NULL;

    const wxVariantIntHash::const_iterator it = hash.find(type);

    wxObject* obj = it->second.GetWxObjectPtr();

    if(!obj) return NULL;

    wxBitmap* bmp = dynamic_cast<wxBitmap*>(obj);

    return bmp;
}


double ColumnsGauge::CalcColWidth(size_t symbols_count)
{
    if(!symbols_count) return 0;

    double w = (double)GetClientSize().x;
    return (w - m_border*2 - ( m_colPadding * (symbols_count+1) )) / symbols_count;
}


wxRect ColumnsGauge::CalcColumn(const wxString& name)
{
    MultiSymbolGaugeDesc& desc = GetDesc();

    wxVariant gmin = desc.GetProp(_T("min_value"));
    wxVariant gmax = desc.GetProp(_T("max_value"));

    wxVariantIntHash& cprops = m_customProps[name];

    wxBitmap* bmp = GetBitmap(cprops,CG_COLBMP);

    if(!bmp) return wxDefaultRect;

    double cval = m_isColShowAvg ? cprops[CG_SYMBOLAVG].GetDouble() : m_hash[name];

    double max = gmax == wxNullVariant ? cprops[CG_SYMBOLMAXVALUE].GetDouble() : gmax.GetDouble();
    double min = gmin == wxNullVariant ? cprops[CG_SYMBOLMINVALUE].GetDouble() : gmin.GetDouble();

    int bmp_height = (int)ceil((double)bmp->GetHeight() * (( cval - min ) / ( max - min )));
    int bmp_y = bmp->GetHeight() - bmp_height;

    //Logger::logf(LOG_MSG,_T("bmp->GetHeight()=%i"),bmp->GetHeight());

    return wxRect(0,bmp_y,bmp->GetWidth(),bmp_height);
}

void ColumnsGauge::PaintNumberValueDC(const wxString& name, wxDC& dc, double col_width)
{
    int font_height = (int)(0.6 * col_width);

    wxFont font = dc.GetFont();
    font.SetPointSize( font_height );
    dc.SetFont(font);

    wxVariantIntHash& cprops = m_customProps[name];

    wxString text_format = cprops[CG_SYMBOLTEXTFORMAT].GetString();

    int text_x = cprops[CG_COLPOS].GetLong();
    int text_y = (int)ceil(m_widgetBorderBox.GetBottom() - 5);

// draw value with dropped shadow
    int ds_diff = (int)floor((double)font_height / 10); // drop shadow inner layer shift distance
    chkminmax(ds_diff,1,4);
    wxString sval = wxString::Format( text_format , m_hash[name] );
    sval += _T(" ") + cprops[CG_SYMBOLUNIT].GetString();
    dc.SetTextForeground( wxColor(0,0,0) );
    dc.DrawRotatedText(sval, text_x + ds_diff, text_y + ds_diff, 90 );
    dc.SetTextForeground( wxColor(255,255,255) );
    dc.DrawRotatedText(sval, text_x,  text_y , 90 );

}

void ColumnsGauge::PaintNumberValueAgg(const wxString& name, wxDC& dc)
{
    wxVariantIntHash& cprops = m_customProps[name];

    wxBitmap* bmp = GetBitmap(cprops,CG_VALUEBMP);

    if(!bmp) return;

    int bmp_x = cprops[CG_COLPOS].GetLong();
    int bmp_y = (int)ceil(m_widgetBorderBox.GetBottom() - bmp->GetHeight());

    dc.DrawBitmap(*bmp,bmp_x,bmp_y);
}

void ColumnsGauge::PaintStick(const wxString& name, wxDC& dc)
{
    MultiSymbolGaugeDesc& desc = GetDesc();

    wxVariantIntHash& cprops = m_customProps[name];

    double stickVal = m_isColShowAvg ? m_hash[name] : cprops[CG_SYMBOLAVG].GetDouble();

    int x_pos = cprops[CG_COLPOS].GetLong();

    wxVariant gmin = desc.GetProp(_T("min_value"));
    wxVariant gmax = desc.GetProp(_T("max_value"));

    double max = gmax == wxNullVariant ? cprops[CG_SYMBOLMAXVALUE].GetDouble() : gmax.GetDouble();
    double min = gmin == wxNullVariant ? cprops[CG_SYMBOLMINVALUE].GetDouble() : gmin.GetDouble();

    double col_avalHeight = (m_widgetBorderBox.h - m_colPadding*2);

    int y_pos = (int)ceil(m_widgetBorderBox.y + m_colPadding + col_avalHeight - (col_avalHeight * (( stickVal - min ) / ( max - min ))));

    //Logger::logf(LOG_MSG,_T("avg=%f, val=%f"),avgVal,m_hash[name]);
    int y_min = (int)m_widgetBorderBox.y;
    int y_max = (int)(m_widgetBorderBox.GetBottom()-m_stickBmp->GetHeight());

    chkminmax(y_pos,y_min,y_max);

    if(!m_stickBmp) DrawSymbolStick();

    dc.DrawBitmap(*m_stickBmp,x_pos,y_pos);
}

void ColumnsGauge::PaintWidget(wxDC& dc)
{
    MultiSymbolGaugeDesc& desc = GetDesc();

    wxVariantHashArray& symbols = desc.GetSymbols();

    size_t symbols_count = symbols.GetCount();

    double col_width = CalcColWidth(symbols_count);

    for(size_t a=0; a<symbols_count; a++ ) {

        wxString name = symbols[a][_T("symbol_name")].GetString();

        wxVariantIntHash& cprops = m_customProps[name];

        wxBitmap* bmp = GetBitmap(cprops,CG_COLBMP);

        if(!bmp) continue;

        wxRect valRect = CalcColumn(name);

        //Logger::logf(LOG_MSG,_T("valRect: h=%i, y=%i"),valRect.height,valRect.y);

        int bmp_y = (int)ceil(valRect.GetTop() + m_widgetBorderBox.GetTop());
        int bmp_x = cprops[CG_COLPOS].GetLong();

        dc.DrawBitmap( bmp->GetSubBitmap(valRect), bmp_x, bmp_y);

        if(m_isUseAgg)
            PaintNumberValueAgg(name,dc);
        else
            PaintNumberValueDC(name,dc,col_width);

        if(m_acSize)
            PaintStick(name,dc);
    }

    dc.DrawBitmap(*m_faceBitmap,0,0);
}

void ColumnsGauge::PaintWidgetAgg(wxDC& dc)
{
    MultiSymbolGaugeDesc& desc = GetDesc();

    wxVariantHashArray& symbols = desc.GetSymbols();

    size_t symbols_count = symbols.GetCount();

    if(!symbols_count) return;

    for(size_t a=0; a<symbols_count; a++ ) {

        wxString name = symbols[a][_T("symbol_name")].GetString();

        wxVariantIntHash& cprops = m_customProps[name];

        wxBitmap* bmp = GetBitmap(cprops,CG_COLBMP);

        if(!bmp) continue;

        wxRect valRect = CalcColumn(name);

        //Logger::logf(LOG_MSG,_T("valRect: h=%i, y=%i"),valRect.height,valRect.y);

        int bmp_y = (int)ceil(valRect.GetTop() + m_widgetBorderBox.GetTop());
        int bmp_x = cprops[CG_COLPOS].GetLong();

        dc.DrawBitmap( bmp->GetSubBitmap(valRect), bmp_x, bmp_y);

        PaintNumberValueAgg(name,dc);

        if(m_acSize)
            PaintStick(name,dc);
    }

    dc.DrawBitmap(*m_faceBitmap,0,0);
}

void ColumnsGauge::CalcAVG(const wxString& name)
{
    double tmp = 0;

    DoubleArray& avgCache = m_avgCache[name];

    for(size_t a=0; a<avgCache.GetCount();a++)
        tmp += avgCache[a];

    m_customProps[name][CG_SYMBOLAVG] = tmp / avgCache.GetCount();
}


void ColumnsGauge::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);

    dc.SetBackground( m_theme->BackgroundColor );

    dc.Clear();

    PaintWidget(dc);

    DrawExtraLayer(dc);
}

wxRegion ColumnsGauge::GetShape()
{
    return wxRegion(0,0,GetSize().x,GetSize().y);
}

wxRegion ColumnsGauge::GetBorderRegion()
{
    return wxRegion(0,0,1,1);
}

void ColumnsGauge::SetNewTheme(WidgetThemeTable *theme)
{
    m_theme = theme;
    GaugeRefresh();
}

void ColumnsGauge::RegisterSymbols()
{
    if( m_isRegistered ) return;

    wxVariantHashArray& symbols = GetDesc().GetSymbols();

    for(size_t a=0; a<symbols.GetCount(); a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();
        m_rtDataMgr->registerSymbol( name );
        m_PrevValueHash[name] = 0.0;
    }

    m_isRegistered = true;
}

void ColumnsGauge::UnRegisterSymbols()
{
    if( !m_isRegistered ) return;

    wxVariantHashArray& symbols = GetDesc().GetSymbols();

    for(size_t a=0; a<symbols.GetCount(); a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();
        m_rtDataMgr->unregisterSymbol( name );
    }

    m_PrevValueHash.clear();

    m_isRegistered = false;
}

void ColumnsGauge::require()
{
    RegisterSymbols();
}

void ColumnsGauge::CacheAVGValue(const wxString& name, double value)
{
    if(!m_acSize) return;

    wxVariantIntHash& cprop = m_customProps[name];

    DoubleArray& avgCache = m_avgCache[name];

    avgCache.Add(value);

    if(avgCache.GetCount() > m_acSize) avgCache.RemoveAt(0);

    double old_val = cprop[CG_SYMBOLAVG].GetDouble();

    CalcAVG(name);

    double new_val = cprop[CG_SYMBOLAVG].GetDouble();

    cprop[CG_CHANGED] = fabs(new_val - old_val) > 0.001 ? true : false;
}

bool ColumnsGauge::IsChangedValue(const wxString& name)
{
    wxString tf = m_customProps[name][CG_SYMBOLTEXTFORMAT].GetString();

    wxString old_val = wxString::Format(tf,m_PrevValueHash[name]);
    wxString new_val = wxString::Format(tf,m_hash[name]);

    return old_val == new_val ? false : true;
}

void ColumnsGauge::refresh(DoubleHash* table)
{
    wxVariantHashArray& symbols = GetDesc().GetSymbols();


    bool isChanged = false;

    for(size_t a=0; a<symbols.GetCount(); a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();

        if(table->find(name) == table->end()) continue;

        double val = (*(table))[name];

        m_hash[name] = val;

        wxVariantIntHash& cprop = m_customProps[name];

        cprop[CG_CHANGED] = IsChangedValue(name);

        m_PrevValueHash[name] = val;

        if(m_acSize)
            CacheAVGValue(name,val);

        if(!isChanged)
            isChanged = cprop[CG_CHANGED].GetBool();
    }

    if(isChanged) {
        if(m_isUseAgg)
            DrawNumberValuesAgg();
    GaugeRefresh();
}
}

void ColumnsGauge::SetValue( double value )
{
}

void ColumnsGauge::GaugeRefresh()
{
    Refresh(false);
}

void ColumnsGauge::RedrawWidget()
{
    clearAll();

    CacheCustomProps();

    DrawWidgetMainFace();

    DrawSymbolColumns();

    if(m_acSize)
        DrawSymbolStick();

    DrawNumberValuesAgg();

    GaugeRefresh();
}

void ColumnsGauge::clearAll()
{
    for(wxVariantIntHashHash::const_iterator it = m_customProps.begin(); it != m_customProps.end(); it++){
        wxBitmap* colbmp = GetBitmap(it->second,CG_COLBMP);
        wxBitmap* valbmp = GetBitmap(it->second,CG_VALUEBMP);

        ptrdel( colbmp );
        ptrdel( valbmp );
    }

    m_customProps.clear();
}

void ColumnsGauge::UpdateSize()
{
    RedrawWidget();
}

// select profile from popup menu
void ColumnsGauge::OnProfileMenu(wxCommandEvent& evt)
{
    int wid = evt.GetId();

    if(m_menuId2ProfileId.find(wid) == m_menuId2ProfileId.end()) return;

    UnRegisterSymbols();

    m_profileID = m_menuId2ProfileId[wid];

    requireChanged();

    wxString descstr = wxString::Format(_T("%i"),m_profileID);

    m_props.descName = descstr;

    //Logger::logf(LOG_MSG,_T("descid=%i, itemid=%i"),m_profileID,GetItemID());

    m_groupManager->SaveDesc( GetItemID(), descstr );

    RedrawWidget();
}

void ColumnsGauge::createWidgetMenu( wxMenu& popupMenu )
{
    CreateProfileChooserMenu(&popupMenu);
}

void ColumnsGauge::OnRightDown(wxMouseEvent& evt)
{
    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

    wxMenu popupMenu;

    CreateProfileChooserMenu(&popupMenu);

    PopupMenu(&popupMenu);
}


void ColumnsGauge::CreateProfileChooserMenu(wxMenu* menu)
{
    wxMenu* profMenu = new wxMenu;

    m_menuId2ProfileId.clear();

    Int2MultiSymbolGaugeDescHash& profiles = GetMgr()->GetProfiles();

    for(Int2MultiSymbolGaugeDescHash::iterator it = profiles.begin(); it != profiles.end(); it++) {

        int wid = wxNewId();
        int prof_id = it->first;

        m_menuId2ProfileId[wid] = prof_id;

        profMenu->AppendRadioItem(wid,it->second.GetName());

        if(prof_id == m_profileID) profMenu->Check(wid,true);

        Connect(wid, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(ColumnsGauge::OnProfileMenu));
    }

    profMenu->AppendSeparator();

    profMenu->Append(idMenuOpenDescEditorProfile,_T("Edit selected profile"));

    profMenu->Append(idMenuOpenDescEditor,_T("Open descriptor editor"));


    menu->AppendSubMenu(profMenu,_T("Profiles..."));

#if defined VEMSDEBUG
    menu->AppendSeparator();
    menu->Append(idTestMenu,_T("Test1"));
#endif

}

void ColumnsGauge::OnMenu(wxCommandEvent& evt)
{
    //Logger::logf(LOG_MSG,_T("m_profileID=%i"),m_profileID);

    switch(evt.GetId()){
        case idMenuOpenDescEditorProfile:
            new MultiGaugeDescEditor(this, _T("Columns gauge descriptor editor"), m_gaugeDescMgr, GetMgr(), false, this, true, m_profileID);
            break;

        case idMenuOpenDescEditor:
            new MultiGaugeDescEditor(this, _T("Columns gauge descriptor editor"), m_gaugeDescMgr, GetMgr(), true, this);
            break;

        case idTestMenu:
            //Benchmark();
            srand( 42 );
            if(m_testTimer.IsRunning())
                m_testTimer.Stop();
            else
                m_testTimer.Start(100);

            break;
    }

}

void ColumnsGauge::OnTestTimer(wxTimerEvent& evt)
{
    DoubleHash hash;
    int random_integer1 = rand();
    int random_integer2 = rand();

    hash[_T("rpm")] = 1000 + random_integer1/7;
    hash[_T("throttle")] = random_integer2/8;

    refresh(&hash);
}

void ColumnsGauge::Benchmark()
{
    Logger::logf(LOG_MSG,_T("ColumnsGauge::Benchmark start"));

    wxStopWatch s;

    s.Start();

    int count = 100;

    for(int a=0; a<count;a++) {

        wxBufferedPaintDC dc(this);

        PaintWidgetAgg(dc);
    }

    double time = s.Time();

    time /= count;

    Logger::logf(LOG_MSG,_T("ColumnsGauge::Benchmark time: %f.3 ms"),time);
}


void ColumnsGauge::CacheCustomProps()
{
    wxVariantHashArray& symbols = GetDesc().GetSymbols();
    size_t symbols_count = symbols.GetCount();

    for(size_t a=0; a<symbols_count; a++ ) {
        //wxString name = symbols[a][_T("symbol_name")].GetString();
        CacheCustomProps(symbols[a]);
    }

}


