#include "DifferentialGauge.h"

#include <wx/dcbuffer.h>

#include "GaugeGroupManager.h"

#include "../DrawHelper.h"
#include "../DirHelper.h"
#include "../logger.h"

#include "../agg/wxAgg2.h"
#include "../events/UpdateViewerEvent.h"
#include "../WidgetThemeTable.h"
#include "../hotkeys/wxHotKeyManager.h"

//#include "../vemsdialog/DiffGaugeDescEditor.h"
#include "../vemsdialog/MultiGaugeDescEditor.h"


const int TIMERID = wxNewId();

BEGIN_EVENT_TABLE( DifferentialGauge,VemsWidgetBase )

    EVT_PAINT( DifferentialGauge::OnPaint )

//    EVT_SIZE ( DifferentialGauge::OnSize )

    EVT_KEY_DOWN ( DifferentialGauge::OnKeyDown )

    EVT_TIMER( TIMERID, DifferentialGauge::OnTimer )

    //EVT_RIGHT_DOWN(DifferentialGauge::OnRightDown)

    EVT_MENU_RANGE(idMenuOpenDescEditor,idMenuOpenDescEditorProfile,DifferentialGauge::OnMenu)


END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS( DifferentialGauge, VemsWidgetBase );

DifferentialGauge::DifferentialGauge( wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop )
    : m_timer(this,TIMERID)
{
    long tmp = 1;

    prop.descName.ToLong(&tmp);

    m_profileID = tmp;

    m_needRefresh = true;

    m_isRegistered = false;
    CreateWidget(parent ,id , gdmgr , prop);
    m_hasOwnDescriptorMgr = true;

    m_isSymbolMenu = false;


    m_showRange = false;
    m_valuesOntheLeft = false;

    m_rmax = m_rmin = 0.0;

    m_timer.Start(40);


    m_hotkeyMgr = new wxHotKeyManager(this);
}

DifferentialGauge::~DifferentialGauge()
{
    delete m_hotkeyMgr;
}

void DifferentialGauge::GaugeRefresh()
{

    m_needRefresh = true;
}

wxRegion DifferentialGauge::GetShape()
{
    return wxRegion(0,0,GetSize().x,GetSize().y);
}

wxRegion DifferentialGauge::GetBorderRegion()
{
    return wxRegion(0,0,1,1);
}

void DifferentialGauge::RedrawWidget()
{
    m_customProps.clear();
    GaugeRefresh();
}

void DifferentialGauge::OnKeyDown(wxKeyEvent &evt)
{

    wxString symbol = m_hotkeyMgr->GetEvent( evt );


    /*if( symbol == _T("DF_SHOWHIDE_TICKS") )
    {
        m_showRange = !m_showRange;
    }*/

    if ( symbol == _T("DF_TOGGLE_LAMBDA_POSITION"))
    {
        m_valuesOntheLeft = !m_valuesOntheLeft;
    }
    GaugeRefresh();

    evt.Skip();
}

void DifferentialGauge::OnProfileMenu(wxCommandEvent& evt)
{
    int wid = evt.GetId();

    if(m_menuId2ProfileId.find(wid) == m_menuId2ProfileId.end()) return;

    UnRegisterSymbols();

    m_profileID = m_menuId2ProfileId[wid];

    requireChanged();

    wxString descstr = wxString::Format(_T("%i"),m_profileID);

    m_props.descName = descstr;

    m_groupManager->SaveDesc( GetItemID(), descstr );

    GaugeRefresh();
}

void DifferentialGauge::CreateProfileChooserMenu(wxMenu* menu)
{
    wxMenu* profMenu = new wxMenu;

    m_menuId2ProfileId.clear();

    Int2MultiSymbolGaugeDescHash& profiles = GetMgr()->GetProfiles();
//    Int2DiffGaugeDescHash& profiles = GetMgr()->GetProfiles();

    for(Int2MultiSymbolGaugeDescHash::iterator it = profiles.begin(); it != profiles.end(); it++) {

        int wid = wxNewId();
        int prof_id = it->first;

        m_menuId2ProfileId[wid] = prof_id;

        profMenu->AppendRadioItem(wid,it->second.GetName());

        if(prof_id == m_profileID) profMenu->Check(wid,true);

        Connect(wid, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(DifferentialGauge::OnProfileMenu));
    }

    profMenu->AppendSeparator();

    profMenu->Append(idMenuOpenDescEditorProfile,_T("Edit selected profile"));

    profMenu->Append(idMenuOpenDescEditor,_T("Open descriptor editor"));


    menu->AppendSubMenu(profMenu,_T("Profiles..."));
}

void DifferentialGauge::OnMenu(wxCommandEvent& evt)
{
    switch(evt.GetId()){
        case idMenuOpenDescEditorProfile:
            //new DiffGaugeDescEditor(this, m_gaugeDescMgr, false, this);
            new MultiGaugeDescEditor(this, _T("Differential gauge descriptor editor"), m_gaugeDescMgr, GetMgr(), false, this, true, m_profileID);
            break;

        case idMenuOpenDescEditor:
            //new DiffGaugeDescEditor(this, m_gaugeDescMgr, true, this);
            new MultiGaugeDescEditor(this, _T("Differential gauge descriptor editor"), m_gaugeDescMgr, GetMgr(), true, this);
            break;
    }

}

void DifferentialGauge::createWidgetMenu( wxMenu& popupMenu )
{
    CreateProfileChooserMenu(&popupMenu);
}

void DifferentialGauge::OnRightDown(wxMouseEvent& evt)
{
    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

    wxMenu popupMenu;

    CreateProfileChooserMenu(&popupMenu);

    PopupMenu(&popupMenu);
}

void DifferentialGauge::OnPaint(wxPaintEvent &evt)
{

    wxBufferedPaintDC dc(this);

    dc.Clear();

    double w = GetSize().x, h = GetSize().y;
    double border_width = 2;

    MultiSymbolGaugeDesc& desc = GetDesc();
    double val_min = desc.GetProp(_T("min_value")).GetDouble();
    double val_max = desc.GetProp(_T("max_value")).GetDouble();

//    DiffGaugeDesc& desc = GetDesc();
//    double val_min = desc.GetMinValue();
//    double val_max = desc.GetMaxValue();

    //double padding = 0;

    wxAgg2 canvas( this );

    wxVariantHashArray& symbols = desc.GetSymbols();

    double maxTextWidth = 0;

    double rect_h = h - 2 * border_width;

    size_t symbols_count = symbols.GetCount();

    double font_height = rect_h / symbols_count * 0.85;

    canvas.loadTTFont( DirHelper::GetPath(dFONTS) + _T("arial.ttf"), font_height , 1.0 );

    for(size_t a=0; a < symbols_count; a++) {

        wxString name = symbols[a][_T("symbol_name")].GetString();

        wxVariantHash& hash = m_customProps[name];

        if(hash.find(_T("symbol_title")) == hash.end()){
            wxString stitle = symbols[a][_T("custom_symbol_title")].GetString();
            hash[_T("symbol_title")] = stitle.Len() ? stitle : m_gaugeDescMgr->GetGaugeSymbolTitle(name);
        }

        if(hash.find(_T("symbol_format")) == hash.end()){
            hash[_T("symbol_format")] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName(name, _T("numformat"));
        }

        if(hash.find(_T("symbol_maxv")) == hash.end()){
            hash[_T("symbol_maxv")] = m_gaugeDescMgr->GetGaugeDescFieldBySymbolName(name, _T("Hi"));
        }

        wxString maxstr = wxString::Format(hash[_T("symbol_format")].GetString(),hash[_T("symbol_maxv")].GetDouble());

        double curr_w = canvas.getTTFTextBound(maxstr,0,0,0).w;

        if(maxTextWidth < curr_w) maxTextWidth = curr_w;

    }

    canvas.clear( m_theme->BackgroundColor );

    wxAggPen* pen = canvas.getPen();

    pen->SetWidth(border_width);
    pen->SetColor( m_theme->ValueColor );

    wxRealRect valueRect(border_width,border_width,w - border_width*2 - maxTextWidth - 2.0, h - border_width*2);

    canvas.drawRectangle(border_width/2,border_width/2, valueRect.w + border_width , valueRect.h + border_width );

    canvas.getBrush()->SetStyle( wxAGGBRUSH_SOLID );

    double padding = 1.0;

    for(size_t a=0; a<symbols.GetCount(); a++) {

        wxVariantHash& symbolProp = symbols[a];

        wxString name = symbolProp[_T("symbol_name")].GetString();

        double val = m_hash[name];

        //Logger::logf(LOG_MSG,name+_T("=%f"),val);

        RealPointArray arr;


        wxRealSize pointer_size(valueRect.w * 0.15,(valueRect.h / symbols_count) - padding);

        double val_center_pos = valueRect.x + valueRect.w * ( (val-val_min) / (val_max - val_min) );

        //wxColor valcolor = symbolProp.find(_T("color")) == symbolProp.end() ? m_theme->ValueColor : DrawHelper::ParseExtColorStr(symbolProp[_T("color")].GetString());
        wxColor valcolor = m_theme->ValueColor;

        if(val_center_pos < valueRect.x + pointer_size.w/2) {
            val_center_pos = valueRect.x + pointer_size.w/2;
            valcolor = m_theme->ColorSymbols[_T("danger")];
        }

        if(val_center_pos > valueRect.GetRight() - pointer_size.w/2) {
            val_center_pos = valueRect.GetRight() - pointer_size.w/2;
            valcolor = m_theme->ColorSymbols[_T("danger")];
        }

        double segment_y = valueRect.y + padding;

        if(a==0) {
            double tick_w = pointer_size.w * 0.15;
            double tick_h = pointer_size.h * 0.5;

            arr.Add( wxRealPoint(val_center_pos - tick_w / 2, segment_y + padding) );
            arr.Add( wxRealPoint(val_center_pos + tick_w / 2, segment_y + padding) );
            arr.Add( wxRealPoint(val_center_pos + tick_w / 2, segment_y + tick_h) );
            arr.Add( wxRealPoint(val_center_pos + pointer_size.w / 2 , segment_y + tick_h) );
            arr.Add( wxRealPoint(val_center_pos + pointer_size.w / 2 , segment_y + pointer_size.h - padding) );
            arr.Add( wxRealPoint(val_center_pos - pointer_size.w / 2 , segment_y + pointer_size.h - padding) );
            arr.Add( wxRealPoint(val_center_pos - pointer_size.w / 2 , segment_y + tick_h) );
            arr.Add( wxRealPoint(val_center_pos - tick_w / 2, segment_y + tick_h) );

        } else {

            arr.Add( wxRealPoint(val_center_pos - pointer_size.w / 2, segment_y + pointer_size.h * a + padding) );
            arr.Add( wxRealPoint(val_center_pos + pointer_size.w / 2, segment_y + pointer_size.h * a + padding) );
            arr.Add( wxRealPoint(val_center_pos + pointer_size.w / 2, segment_y + pointer_size.h * (a+1) - padding ) );
            arr.Add( wxRealPoint(val_center_pos - pointer_size.w / 2, segment_y + pointer_size.h * (a+1) - padding ) );

        }

        pen->SetStyle( wxAGGPEN_TRANSPARENT );

        canvas.getBrush()->SetColor( valcolor );

        canvas.drawPolygon( arr );

        pen->SetStyle( wxAGGPEN_SOLID );
        pen->SetColor( m_theme->ValueColor );

        wxString valstr = wxString::Format(m_customProps[name][_T("symbol_format")].GetString(),val);

        double text_pos_y = pointer_size.h*(a+1) - (pointer_size.h-font_height)/2;

        canvas.setFontHeight(font_height);
        canvas.drawTTFText(valstr, valueRect.GetRight() + border_width + 1.0, text_pos_y );

        wxColor title_color = m_theme->TitleColor;

        DrawHelper::SetColorA(title_color, 128);

        pen->SetColor( title_color );
        canvas.setFontHeight(font_height*0.7);

        canvas.drawTTFText(m_customProps[name][_T("symbol_title")].GetString(), valueRect.x + 10 , text_pos_y );

    }


    canvas.render( dc );

    DrawExtraLayer(dc);

}

void DifferentialGauge::SetNewTheme(WidgetThemeTable *theme)
{
    m_theme = theme;
    GaugeRefresh();
}

void DifferentialGauge::RegisterSymbols()
{
    if( m_isRegistered ) return;

    wxVariantHashArray& symbols = GetDesc().GetSymbols();

    for(size_t a=0; a<symbols.GetCount(); a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();
        m_rtDataMgr->registerSymbol( name );
        m_PrevValueHash[name] = 0.0;
    }

    m_isRegistered = true;
}

void DifferentialGauge::UnRegisterSymbols()
{
    if( !m_isRegistered ) return;

    wxVariantHashArray& symbols = GetDesc().GetSymbols();

    for(size_t a=0; a<symbols.GetCount(); a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();
        m_rtDataMgr->unregisterSymbol( name );
    }

    m_PrevValueHash.clear();

    m_isRegistered = false;
}

void DifferentialGauge::require()
{
    RegisterSymbols();
}

void DifferentialGauge::refresh( DoubleHash* table )
{
    wxVariantHashArray& symbols = GetDesc().GetSymbols();

    bool isChanged = false;

    for(size_t a=0; a<symbols.GetCount(); a++ ) {
        wxString name = symbols[a][_T("symbol_name")].GetString();

        if(table->find(name) == table->end()) continue;

        m_hash[name] = (*(table))[name];

        if( !isChanged && fabs(m_PrevValueHash[name]-m_hash[name]) > 0.0001 ) isChanged = true;
    }
    if(isChanged)
        GaugeRefresh();
}

void DifferentialGauge::OnTimer( wxTimerEvent& event )
{
    if(m_needRefresh)
        Refresh(false);

    m_needRefresh = false;
}
