#include "GPSRouteGauge.h"

#include <wx/dcbuffer.h>
#include <wx/filename.h>
#include <wx/regex.h>

#include "../agg/wxAgg.h"
#include "../agg/wxAgg2.h"

#include "../DrawHelper.h"
#include "../DirHelper.h"
#include "../events/UpdateViewerEvent.h"
#include "../loghistory/LogHistoryBase.h"
#include "../common/FunctionMeter.h"
#include "../configmanager/ConfigHelper2.h"
#include "GaugeGroupManager.h"
#include "../hotkeys/wxHotKeyManager.h"
#include "../wxVariantHelper.h"
#include "../customwidgets/AggToolTip.h"

#include "../bcWidgets/bcSlider.h"
#include "../bcWidgets/bcLabel.h"

BEGIN_EVENT_TABLE(GPSRouteGauge,VemsWidgetBase)

    EVT_PAINT(GPSRouteGauge::OnPaint)

    EVT_SIZE(GPSRouteGauge::OnSize)

    EVT_LEFT_DOWN(GPSRouteGauge::OnLeftDown)
    EVT_MOTION(GPSRouteGauge::OnMouseMove)
    EVT_LEFT_UP(GPSRouteGauge::OnLeftUp)

    EVT_KEY_DOWN(GPSRouteGauge::OnKeyDown)

    EVT_ENTER_WINDOW(GPSRouteGauge::OnEnter)
    EVT_LEAVE_WINDOW(GPSRouteGauge::OnLeave)

    EVT_UPDATE_VIEWER_EVENT( wxID_ANY, GPSRouteGauge::OnUpdateValue )

    EVT_MOUSEWHEEL(GPSRouteGauge::OnWheel)

    EVT_COMMAND_SCROLL_CHANGED(idZoomSlider,GPSRouteGauge::OnSliderZoom)

    EVT_TIMER(idRefreshTimer,GPSRouteGauge::OnRefreshTimer)

    //EVT_RIGHT_DOWN(GPSRouteGauge::OnMouseRightDown)


END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(GPSRouteGauge, VemsWidgetBase);

WX_DECLARE_LIST( double, DoubleListG );
#include <wx/listimpl.cpp>
WX_DEFINE_LIST( DoubleListG );

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY( TileDescArray );


#define GPSLAT _T("latitudeDec")
#define GPSLONG _T("longitudeDec")

#define ATTPROFILE_DEF DefaultAggToolTipProfile
#define ATTPROFILE_CURRVAL _T("currvalue")

#define DEFAULT_MAPSERVER _T("http://tile.openstreetmap.org/cgi-bin/export?bbox=%.8f,%.8f,%.8f,%.8f&scale=%i&format=png")
#define DEFAULT_MAPTILESERVER _T("http://tile.openstreetmap.org")

#define HELP_PANEL_DELTA_MOVE 150

/** NOTES:

    http://tile.openstreetmap.org/cgi-bin/export?bbox=19.0,47.0,19.1,47.1&scale=100000&format=png

    delta longitude: 0.1 deg
    delta latitude: 0.1 deg

    scale : 100000

    result size:
        - longitude:  397 px
        - latitude :  583 px


    scale = ( delta_deg * ref ) / pixels;

    openstreetmap Errors:
     - Invalid bounding box
     - Map too large
     - The load average on the server is too high at the moment. Please wait a few minutes before trying again.


http://tile.openstreetmap.org/cgi-bin/export?bbox=19.11272000,47.44682362,19.15675500,47.48407805&scale=89194&format=png

http://tile.openstreetmap.org/cgi-bin/export?bbox=19.11272000,47.46181667,19.15675500,47.46908500&scale=89194&format=png

    last revision: 6178

*/

/**
    Refactor: use static images instead of cgi-bin/export function

    http://tile.openstreetmap.org/7/63/42.png

    http://tile.openstreetmap.org/level/x/y.png

    calculate level, x, y from coordinates:
    http://wiki.openstreetmap.org/wiki/Slippy_map_tilenames


    256x256 pixels

    /9/281/178.png



*/


GPSRouteGauge::GPSRouteGauge(wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop, LogHistoryBase * lhb)
                :VemsWidgetBase(parent,id, gdmgr, prop)
                ,m_refreshTimer(this, idRefreshTimer)

{
    //ctor
    m_debugFlags = GDF_NO_DEBUG_DRAW;

    CreateBcParent(this);

    m_dataMgr = lhb;
    LOGP(wxString::Format(_T("m_dataMgr=%p"),m_dataMgr));
    m_tileSizePx = 256;
    m_isRegistered = false;
    m_isSymbolMenu = false;
    m_isInited = false;

    m_mouseState = RMS_NONE;

    m_oriGPSBorder = wxDefaultRealRect;
    m_actualMapRect = wxDefaultRealRect;
    m_expandedBorder = wxDefaultRealRect;
    m_actualTileRect = wxDefaultRect;
    m_mapRect = wxDefaultRect;
    m_actualZoom = 0;

    m_lastMapDragPoint = wxDefaultPosition;

    m_textFontPath = DirHelper::GetPath(dFONTS) + _T("arial.ttf");


    m_padding = 10;

    m_bgBmp = NULL;
    m_routeBmp = NULL;
    m_bgBorder = NULL;
    m_toolTipBmp = NULL;
    m_locationPointBmp = NULL;
    m_toolTipRTPosBmp = NULL;

    m_toolTipMgr = new AggToolTip();


    m_location = wxDefaultRealPointEx;

    m_deltaOver = 1.0;


    m_themeDesc[_T("pathlinewidth")] = 2.0;
    m_themeDesc[_T("pathlinehlwidth")] = 6.0;
    m_themeDesc[_T("pointersize")] = 14L;

    m_themeDesc[_T("pathlinecolor")] << wxColor(0,0,255,200);
    m_themeDesc[_T("pathlinehlcolor")] << wxColor(0,255,0,200);
    m_themeDesc[_T("pointercolor")] << wxColor(0,255,0,200);
    m_themeDesc[_T("pointercolor2")] << wxColor(255,0,0,255);
    m_themeDesc[_T("bordercolor")] << wxColor(0,0,0,255);


    wxVariantIntHash props;

    props[ ATT_ARROW_ALIGMENT ] = (long)ATT_AA_BOTTOM;
    props[ ATT_TEXT_ALIGN ] = (long)ATT_TA_LEFT;
    props[ ATT_ROUND_TYPE ] = (long)ATT_RT_ARC;
    props[ ATT_ARROW_REFPOSITION ] = 0.5;
    props[ ATT_BORDER_WIDTH ] = 0.0;
    props[ ATT_TEXT_HEIGHT ] = 8.0;
    props[ ATT_ROUND_RADIUS ] = 3.5;
    props[ ATT_BACKGROUNDCOLOR ] << wxColor(255,255,255,150);
    props[ ATT_BORDER_WIDTH ] = 1.0;
    props[ ATT_BORDER_COLOR ] << wxColor(0,0,0,255);
//    props[ ATT_TEXT_COLOR ] << theme->GetColor(_T("textcolor"));

    m_toolTipMgr->SetProperties( props );

    m_toolTipMgr->SetCurrentProfile(ATTPROFILE_CURRVAL);
    m_toolTipMgr->InitProfile(ATTPROFILE_CURRVAL);
    props[ ATT_BORDER_WIDTH ] = 1.5;
    props[ ATT_BACKGROUNDCOLOR ] << wxColor(255,255,255,255);
    m_toolTipMgr->SetProperties( props );


    m_dragCursor = wxCursor( wxImage( DirHelper::GetPath(dICON)+_T("drag.png"), wxBITMAP_TYPE_PNG ) );
    m_dragDownCursor = wxCursor( wxImage( DirHelper::GetPath(dICON)+_T("drag-down.png"), wxBITMAP_TYPE_PNG ) );


    m_hotKeyMgr = new wxHotKeyManager(this);

    ConfigBase* config = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);

    wxString tileServer = config->getValue(_T("/webservice"),_T("maptileserver"),DEFAULT_MAPTILESERVER);

    wxString cachedir = GetCacheDir();

    m_tileMgr = new OSMTileManager(tileServer,cachedir);
    m_tileMgr->registerObserver(this);


    config->getValue(_T("/webservice"),_T("maps_default_lon"),_T("19.040505")).ToDouble( &(m_defaultGPSCenter.x) );
    config->getValue(_T("/webservice"),_T("maps_default_lat"),_T("47.498358")).ToDouble( &(m_defaultGPSCenter.y) );;
    config->getValue(_T("/webservice"),_T("maps_default_level"),_T("10")).ToLong( &m_defaultLevel );

    m_zoomSlider = new bcSlider(this, idZoomSlider, 1, 18, 1, 0, wxPoint(m_padding+5,m_padding+5), wxSize(35,120));
    m_zoomSlider->SetProperty(BCBS_BACKGROUND_COLOR, wxColor(255,255,255,190));
    m_zoomSlider->SetProperty(bcSlider::BCS_CELL_SPACE, 2L, true);


    m_helpPanel = new bcLabel(this, wxID_ANY, wxEmptyString,wxPoint(20, prop.height + 20), wxSize(10,10), BCLS_AUTO_SIZE );
    m_helpPanel->SetAlpha(0,false);

    m_helpPanel->SetProperty( BCBS_BACKGROUND_COLOR, wxVariantHelper::ToVariant(wxColor(255,255,255,200)) );

    m_infoPanel = new bcLabel(this, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize, BCLS_AUTO_SIZE);

    m_infoPanel->SetAlpha(0,false);

    m_refreshTimeBase = 20;

    m_isRefresh = false;

}

GPSRouteGauge::~GPSRouteGauge()
{
    //dtor
    delete m_toolTipMgr;
    m_refreshTimer.Stop();

    ptrdel(m_bgBmp);
    ptrdel(m_routeBmp);
    ptrdel(m_bgBorder);
    ptrdel(m_toolTipBmp);
    ptrdel(m_toolTipRTPosBmp);
    ptrdel(m_locationPointBmp);

    ptrdel(m_tileMgr);


    WX_CLEAR_ARRAY(m_routeData);
}

wxString GPSRouteGauge::GetCacheDir()
{
    ConfigBase* config = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);

    wxString defcache = DirHelper::GetPath(dWORKINGDIR) + _T("gpscache");

    wxString dircfg = config->getValue(_T("/webservice/"),_T("maptilecache"), wxEmptyString );

    if( dircfg == wxEmptyString ) return defcache;

    wxString cachedir = wxEmptyString;

    wxRegEx r;

    if(r.Compile(_T("%vemsdocuments%\\/(.+)")) && r.Matches(dircfg)) {

        cachedir = DirHelper::GetPath(dWORKINGDIR) + r.GetMatch(dircfg,1);

    } else if(r.Compile(_T("%vemstunedir%\\/(.+)")) && r.Matches(dircfg)) {

        cachedir = DirHelper::GetPath() + r.GetMatch(dircfg,1);

    }

    if( cachedir == wxEmptyString ) cachedir = defcache;

    return cachedir;
}

int GPSRouteGauge::long2tileX(double lon, int z)
{
	return (int)(floor( long2tileXref(lon,z) ));
}

double GPSRouteGauge::long2tileXref(double lon, int z)
{
	return (lon + 180.0) / 360.0 * pow(2.0, z);
}

int GPSRouteGauge::lat2tileY(double lat, int z)
{
	return (int)(floor(lat2tileYref(lat,z)));
}

double GPSRouteGauge::lat2tileYref(double lat, int z)
{
	return (1.0 - log( tan(lat * M_PI/180.0) + 1.0 / cos(lat * M_PI/180.0)) / M_PI) / 2.0 * pow(2.0, z);
}

wxRealPoint GPSRouteGauge::deg2tileRef(const wxRealPoint& deg, int z)
{
    return wxRealPoint( long2tileXref(deg.x, z), lat2tileYref(deg.y, z) );
}

double GPSRouteGauge::tileX2long(int x, int z)
{
	return tileXref2long(x,z);
}

double GPSRouteGauge::tileY2lat(int y, int z)
{
    return tileYref2lat(y,z);
}

double GPSRouteGauge::tileXref2long(double x, int z)
{
    return x / pow(2.0, z) * 360.0 - 180.0;
}

double GPSRouteGauge::tileYref2lat(double y, int z)
{
	double n = M_PI - 2.0 * M_PI * y / pow(2.0, z);
	return 180.0 / M_PI * atan(0.5 * (exp(n) - exp(-n)));
}

wxRealPoint GPSRouteGauge::tileRef2deg(const wxRealPoint& tile, int z)
{
    return wxRealPoint( tileXref2long(tile.x,z), tileYref2lat(tile.y,z) );
}

wxPoint GPSRouteGauge::GetTile(int zoom, const wxRealPoint& deg)
{
    wxPoint p;
    p.x = long2tileX(deg.x,zoom);
    p.y = lat2tileY(deg.y,zoom);
    return p;
}

wxRealRect GPSRouteGauge::tile2DegRect(TileDesc& desc)
{
    return GetDegFromTileRef( wxRealRect(wxRealPoint(desc.x,desc.y),wxRealPoint(desc.x+1,desc.y+1)), desc.zoom );
}

void GPSRouteGauge::InitGauge()
{
    FUNCTIME

    //m_mapServerFormat = ConfigHelper::getValue(_T("/"),_T("mapserverformat"),DEFAULT_MAPSERVER);
    //m_cacheFileFormat = _T("left=%.8f+top=%.8f+right=%.8f+bottom=%.8f+scale=%i.png");

    m_rtDataMgr->getRequiredSymbols();

    //int hc = m_dataMgr->getHistoryCount();

    DoubleHashArray gpsdata;

    //m_dataMgr->getHistory(0,hc-1,gpsdata);

    m_dataMgr->getGPSData( gpsdata );

    Logger::logf(LOG_MSG,_T("gpsdata c=%i"),gpsdata.GetCount());

    BuildGPS(gpsdata);

    InitRoute();

    //DrawRoute(GetThemeColor(_T("pathlinecolor")),true);

    DrawBorders();

    DrawLocationPoint();

    m_refreshTimer.Start(m_refreshTimeBase);

    InitHelpPanel();

    TextDesc desc(_T("Help: press ") + m_hotKeyMgr->GetHotKeyStringBySymbol(_T("HELP")));


    bcLabel* label = new bcLabel(this, wxID_ANY, desc, wxPoint(m_padding+5, GetClientSize().y - m_padding - 25 ), wxDefaultSize, BCLS_AUTO_SIZE, false );
    label->SetProperty( BCBS_BACKGROUND_COLOR, wxColor(255,255,255,180) );
    label->SetProperty( BCBS_BORDER_COLOR, wxColor(0,0,0) );
    label->Refresh();

    m_isInited = true;

}

void GPSRouteGauge::BuildGPS(const DoubleHashArray& gpsdata)
{
    WX_CLEAR_ARRAY(m_routeData);

    for(size_t a=0; a < gpsdata.GetCount();a++) {

        DoubleHash * data = gpsdata[a];

//        double latitude = (*data)[GPSLAT];
//        double longitude = (*data)[GPSLONG];
//
//        if(fabs(latitude) < 0.001 || fabs(longitude) < 0.001) {
//            delete data;
//            continue;
//        }

        m_routeData.Add(new RouteElement(data));
    }

}



void GPSRouteGauge::InitRoute()
{
    wxSize size = GetClientSize();

    size.DecBy( m_padding*2 );

    wxRealRect rect = SearchBorder();

    int tileXCnt = (int)ceil((double)(size.x/m_tileSizePx)) + 1;
    int tileYCnt = (int)ceil((double)(size.y/m_tileSizePx)) + 1;

    wxRealSize size_tile = wxRealSize( (double)size.x/(double)m_tileSizePx, (double)size.y/(double)m_tileSizePx ) ;

    Logger::logf(LOG_MSG,_T("route rect: ")+rect.GetAsStringBorders(6));

    Logger::logf(LOG_MSG,_T("tileXCnt=%i, tileYCnt=%i"),tileXCnt,tileYCnt);

    Logger::logf(LOG_MSG,_T("size_tile w=%f, h=%f"),size_tile.w,size_tile.h);

    if(rect == wxNullRealRect) {

        m_actualZoom = m_defaultLevel;

        wxRealPoint tilecenter = deg2tileRef(m_defaultGPSCenter, m_actualZoom);

        wxRealRect tileref;
        tileref.x = tilecenter.x - size_tile.w/2.0;
        tileref.y = tilecenter.y - size_tile.h/2.0;
        tileref.SetSize(size_tile);

        m_actualMapRect = GetDegFromTileRef( tileref, m_actualZoom );


    } else {

        m_oriGPSBorder = rect;

        int zoom_level = -1;

        int zoomX = -1;
        int zoomY = -1;

    //    Logger::logf(LOG_MSG,_T("tileXCnt=%i,tileYCnt=%i"),tileXCnt,tileYCnt);

        for(size_t a = 1; a <= 18; a++) {

            wxRealRect oriRect = GetTileRealRect( m_oriGPSBorder, a );

            Logger::logf(LOG_MSG,_T("a=%i, w=%f,h=%f"),a,oriRect.w,oriRect.h);

            if( oriRect.w > size_tile.w && zoomX == -1 ) zoomX = a-1;

            if( oriRect.h > size_tile.h && zoomY == -1 ) zoomY = a-1;

            if( zoomX != -1 && zoomY != -1 ) break;
        }

        Logger::logf(LOG_MSG,_T("zoomX=%i,zoomY=%i"),zoomX,zoomY);

        if( zoomX > 0 && (zoomX <= zoomY || zoomY < 0) ) zoom_level = zoomX;

        if( zoomY > 0 && (zoomY <= zoomX || zoomX < 0) ) zoom_level = zoomY;

        if(zoom_level == -1) {
            Logger::logf(LOG_MSG,_T("zoom_level fail"));
            zoom_level = 18;
            //return;
        }

        Logger::logf(LOG_MSG,_T("valid zoomlevel: %i"),zoom_level);

        wxRealRect oriRect = GetTileRealRect( m_oriGPSBorder, zoom_level );

        wxRealPoint center = oriRect.GetCenter();

        wxRealRect actRect( center.x - size_tile.w/2, center.y - size_tile.h/2, size_tile.w, size_tile.h );

        m_actualMapRect = GetDegFromTileRef( actRect, zoom_level );

        m_actualZoom = zoom_level;

    }

    Logger::logf(LOG_MSG,_T("gps: ")+m_oriGPSBorder.GetAsStringBorders(5));
    Logger::logf(LOG_MSG,_T("map: ")+m_actualMapRect.GetAsStringBorders(5));

    wxRect mapTileRect = GetTileRealRect(m_actualMapRect, m_actualZoom).GetOuterBox();

    for( int x = mapTileRect.GetLeft(); x <= mapTileRect.GetRight(); x++ ) {
        for( int y = mapTileRect.GetTop(); y <= mapTileRect.GetBottom(); y++ ) {
            m_tileMgr->Request( TileDesc(x,y,m_actualZoom) );
        }
    }

    m_zoomSlider->ChangeValue( m_actualZoom );

}

wxColor GPSRouteGauge::GetThemeColor(const wxString& name)
{
    if( m_themeDesc.find(name) == m_themeDesc.end() ) return wxNullColour;

    if( m_themeDesc[name].GetType() != _T("wxColour") ) return wxNullColour;

    wxColor c;
    c << m_themeDesc[name];
    return c;
}

double GPSRouteGauge::GetThemeDouble(const wxString& name)
{
    if( m_themeDesc.find(name) == m_themeDesc.end() ) return 0.0;

    return m_themeDesc[name].GetDouble();
}

long GPSRouteGauge::GetThemeLong(const wxString& name)
{
    if( m_themeDesc.find(name) == m_themeDesc.end() ) return 0L;

    return m_themeDesc[name].GetLong();
}

wxRealRect GPSRouteGauge::SearchBorder()
{
    //FUNCTIME
    if( m_routeData.GetCount() < 1 ) return wxNullRealRect;

    double top = -100.0, bottom = 100.0, left = 200.0, right = -200.0;
    double latitude, longitude;
    double prev_latitude, prev_longitude;
    for(size_t a=0; a<m_routeData.GetCount();a++) {

        DoubleHash * data = m_routeData[a]->Data;

        if(data->find(GPSLAT) == data->end() || data->find(GPSLONG) == data->end() ) continue;

        latitude = (*data)[GPSLAT];
        longitude = (*data)[GPSLONG];

        //if( a && (fabs(prev_latitude - latitude) > m_deltaOver || fabs(prev_longitude - longitude) > m_deltaOver) ) continue;

        if(fabs(latitude) < 0.001 || fabs(longitude) < 0.001) continue;

        if( top < latitude ) top = latitude;
        if( bottom > latitude ) bottom = latitude;
        if( left > longitude ) left = longitude;
        if( right < longitude ) right = longitude;

        prev_latitude = latitude;
        prev_longitude = longitude;

    }

    return wxRealRect(wxRealPoint(left,bottom),wxRealPoint(right,top));

}

void GPSRouteGauge::DrawBorders()
{
    double w = GetSize().x;
    double h = GetSize().y;

    double pad = m_padding;

    wxAgg2 aggCanvas((int)w,(int)h);



    double rw = w - 2*pad;
    double rh = h - 2*pad;

    if(m_bgBmp && m_bgBmp->IsOk()) {
        rw = m_bgBmp->GetWidth();
        rh = m_bgBmp->GetHeight();
    }

    double borderw = 2.0;

    wxRect srect( (int)(m_padding + borderw/2.0), (int)(m_padding + borderw/2.0), (int)( rw-borderw ), (int)( rh-borderw ) );

    aggCanvas.drawInnerRectShadow(6,150,srect);

    aggCanvas.getPen()->SetColor( GetThemeColor(_T("bordercolor")) );
    aggCanvas.getPen()->SetWidth(borderw);

    aggCanvas.drawRoundedRectangle(pad,pad, rw, rh, borderw);

    aggCanvas.getBrush()->SetStyle( wxAGGBRUSH_SOLID );

    aggCanvas.setBrush( wxColor(255,255,255,180) );

    double bpad = pad - (borderw / 2.0);

    aggCanvas.getPen()->SetStyle( wxAGGPEN_TRANSPARENT );

    aggCanvas.drawRectangle( 0,0,w,bpad );
    aggCanvas.drawRectangle( 0,h-bpad,w,bpad );
    aggCanvas.drawRectangle( 0,bpad,bpad,h-2*bpad );
    aggCanvas.drawRectangle( w-bpad,bpad,bpad,h-2*bpad );

    ptrdel(m_bgBorder);
    m_bgBorder = aggCanvas.renderToBitmap();
}

void GPSRouteGauge::DrawLocationPoint()
{
    double size = GetThemeDouble(_T("pointersize"));

    int s = (int)ceil(size);

    wxColor c2 = GetThemeColor(_T("pointercolor2"));

    wxAgg aggCanvas(s,s);
    aggCanvas.setBrush( c2 );
    aggCanvas.drawFilledCircle(size/2.0,size/2.0,size/2.0);

    double is = size * 0.7;
    aggCanvas.setBrush( GetThemeColor(_T("pointercolor")) );
    aggCanvas.drawFilledCircle(size/2.0,size/2.0,is/2.0);

    is = size * 0.2;
    aggCanvas.setBrush( c2 );
    aggCanvas.drawFilledCircle(size/2.0,size/2.0,is/2.0);


    ptrdel(m_locationPointBmp);
    m_locationPointBmp = aggCanvas.renderToBitmap();
}

void GPSRouteGauge::DrawNullRoute()
{
    double w = GetSize().x - 2*m_padding;
    double h = GetSize().y - 2*m_padding;

    wxAgg aggCanvas((int)w,(int)h);

    wxString text;// = _T("Log does not contain GPS data");

    double text_height = 12;
    double text_width = aggCanvas.getTextWidth(text,text_height);

    aggCanvas.setPen(wxColor(255,0,0));

    aggCanvas.drawText(text, (w - text_width)/2.0, (h)/2.0, 0, text_height, 2 );

    ptrdel(m_routeBmp);

    m_routeBmp = aggCanvas.renderToBitmap();
}


bool GPSRouteGauge::DrawRouteTile(TileDesc& desc, const wxColor& linecolor, bool calcregion)
{
    if( !m_actualMapRect.Intersects(m_oriGPSBorder) ) return false;

    wxColor color = linecolor;

    if( color == wxNullColour ) color = GetThemeColor(_T("pathlinecolor"));

    int dc = m_routeData.GetCount();

    if(dc < 5) {
        return false;
    }

//    Logger::logf(LOG_MSG,_T("dc=%i"),dc);

    double latitude, longitude,x,y;

    double lw = GetThemeDouble(_T("pathlinewidth"));

    int mod = 0;

    if(dc > 1000) mod = 5;
    if(dc > 5000) mod = 10;
    if(dc > 10000) mod = 20;
    if(dc > 50000) mod = 100;

    RealPointArray path;

    int zoom = desc.zoom;

    wxRealRect gpsTileBorder = tile2DegRect(desc);

    double drawPadding = 20.0;

    wxRealRect tilePixelRect( -drawPadding,-drawPadding, m_tileSizePx+2*drawPadding, m_tileSizePx+2*drawPadding );

    wxRealRect tileInnerPixelRect = tilePixelRect;
    tileInnerPixelRect.Deflate(2*drawPadding);

    bool isFilter = false;

    for(size_t a=0; a<m_routeData.GetCount(); a++ ) {

        if( isFilter && mod && a%mod != 0 ) continue;

        DoubleHash * data = m_routeData[a]->Data;

        if(data->find(GPSLAT) == data->end() || data->find(GPSLONG) == data->end() ) continue;

        latitude = (*data)[GPSLAT];
        longitude = (*data)[GPSLONG];

        //if( !(gpsTileBorder.Contains(longitude,latitude)) ) continue;

        wxRealPoint tilerefpos = deg2tileRef( wxRealPoint( longitude, latitude ), zoom );

        x = (tilerefpos.x-desc.x) * (double)m_tileSizePx;
        y = (tilerefpos.y-desc.y) * (double)m_tileSizePx;

        if( !tilePixelRect.Contains(x,y) ) continue;

        isFilter = tileInnerPixelRect.Contains(x,y) ? true : false;

        path.Add( wxRealPoint(x,y) );

    }

    int pc = path.GetCount();

    if(!pc) return false;

    //Logger::logf(LOG_MSG,_T("path points: %i"),pc);

    wxAgg2 aggCanvas(m_tileSizePx+1,m_tileSizePx+1);
    if(!aggCanvas.isOk()) return false;

    double path_line_highlight_w = GetThemeDouble(_T("pathlinehlwidth"));

    wxAggPen* pen = aggCanvas.getPen();
    pen->SetWidth(path_line_highlight_w);
    pen->SetCap(agg::square_cap);
    pen->SetColor(wxColor(255,255,255,255));

    aggCanvas.drawLine(path);

    pen->SetColor(color);
    pen->SetWidth(lw);
    aggCanvas.drawLine(path);

    wxImage* img = aggCanvas.renderToImage();

    wxBitmap bmp(*img);

    m_routeBitmaps[desc.zoom][desc.x][desc.y] = bmp;

    if(calcregion) {
        wxColor bgcolor = wxColor(0,0,0,0);
        wxRegion reg(bmp, bgcolor, 5);
        //m_pathRegions[m_actualZoom]

    }

    delete img;

    return true;
}

void GPSRouteGauge::DrawRoute(const wxColor& linecolor, bool calcregion)
{
    //FUNCTIME

    wxRealRect tilerefrect = GetTileRealRect( m_oriGPSBorder, m_actualZoom );
    wxRect tileRect( (int)tilerefrect.x, (int)tilerefrect.y, (int)tilerefrect.w, (int)tilerefrect.h );

    for(int x = tileRect.GetLeft(); x <= tileRect.GetRight(); x++ ) {
        for(int y = tileRect.GetTop(); y <= tileRect.GetBottom(); y++ ) {
            TileDesc desc(x,y,m_actualZoom);
            DrawRouteTile( desc, linecolor, calcregion );
        }
    }



//
//    wxColor color = linecolor;
//
//    if( color == wxNullColour ) color = GetThemeColor(_T("pathlinecolor"));
//
//    int dc = m_routeData.GetCount();
//
//    //if( m_expandedBorder == wxDefaultRealRect || m_expandedBorder == wxNullRealRect ) {
//    if( dc < 5 ) {
//        //DrawNullRoute();
//        return;
//    }
//
//    if( m_mapRect == wxDefaultRect ) {
//        m_mapRect = wxRect( wxPoint(0,0), GetClientSize());
//        m_mapRect.Deflate(m_padding);
//    }
//
//
//    double w = m_mapRect.GetWidth();
//    double h = m_mapRect.GetHeight();
//
//
//
//    double latitude, longitude;
////    double prev_latitude, prev_longitude;
//
//
//    double lw = GetThemeDouble(_T("pathlinewidth"));
//
//    int mod = 0;
//
//    if(dc > 1000) mod = 5;
//    if(dc > 5000) mod = 10;
//    if(dc > 10000) mod = 20;
//    if(dc > 50000) mod = 100;
//
//    wxRealRect borders = m_expandedBorder;
//
//    RealPointArray path;
//
//    double x,y;
//    wxRealPoint minp(w,h);
//    wxRealPoint maxp(0,0);
//
//    wxRealRect tilerefrect = GetTileRealRect( m_actualMapRect, m_actualZoom );
//
//    Logger::logf(LOG_MSG,_T("cw=%i,ch=%i"), m_mapRect.width, m_mapRect.height);
//
//    for(size_t a=0; a<m_routeData.GetCount();a++) {
//
//        DoubleHash * data = m_routeData[a]->Data;
//
//        if(data->find(GPSLAT) == data->end() || data->find(GPSLONG) == data->end() ) continue;
//
//        latitude = (*data)[GPSLAT];
//        longitude = (*data)[GPSLONG];
//
//        //if( a && (fabs(prev_latitude - latitude) > m_deltaOver || fabs(prev_longitude - longitude) > m_deltaOver) ) continue;
//
//        if(fabs(latitude) < 0.001 || fabs(longitude) < 0.001) continue;
//
////        x = fabs( longitude - borders.GetLeft() ) / fabs(borders.GetWidth()) * w;
////        y = h - (fabs( latitude - borders.GetTop() ) / fabs(borders.GetHeight()) * h );
//
//        wxRealPoint tileref = deg2tileRef( wxRealPoint( longitude, latitude ), m_actualZoom );
//
//        x = ((tileref.x - tilerefrect.x) / tilerefrect.w * w);
//        y = ((tileref.y - tilerefrect.y) / tilerefrect.h * h);
//
//
//        m_routeData[a]->Point.x = x;
//        m_routeData[a]->Point.y = y;
//
////        prev_latitude = latitude;
////        prev_longitude = longitude;
//
//        if( minp.x > x ) minp.x = x;
//        if( minp.y > y ) minp.y = y;
//
//        if( maxp.x < x ) maxp.x = x;
//        if( maxp.y < y ) maxp.y = y;
//
//        if( mod && a%mod != 0 ) continue;
//
//        path.Add( wxRealPoint(x,y) );
//
//    }
//
//    wxRealRect rr(minp, maxp);
//
//    Logger::logf(LOG_MSG,_T("rr: ")+rr.GetAsStringBorders(3));
//
//    //Logger::logf(LOG_MSG,_T("pc=%i"),path.GetCount());
//
//    double path_line_highlight_w = GetThemeDouble(_T("pathlinehlwidth"));
//
//    minp.x -= path_line_highlight_w;
//    minp.y -= path_line_highlight_w;
//    maxp.x += path_line_highlight_w;
//    maxp.y += path_line_highlight_w;
//
//    wxSize canvassize( (int)(maxp.x-minp.x), (int)(maxp.y-minp.y) );
//
//    minp.x *= -1;
//    minp.y *= -1;
//
//    OffsetPath(path, minp);
//
//    Logger::logf(LOG_MSG,_T("canvassize.x=%i, y=%i"),canvassize.x, canvassize.y);
//
//    wxAgg aggCanvas( canvassize );
//
//    aggCanvas.setPen(wxColor(255,255,255,255));
//    aggCanvas.drawLine(path, path_line_highlight_w );
//
//    aggCanvas.setPen(color);
//    aggCanvas.drawLine(path, lw);
//
//    ptrdel(m_routeBmp);
//    m_routeBmp = aggCanvas.renderToBitmap();
//
//    if(calcregion) {
////        wxColor bgcolor = wxColor(0,0,0,0);
////        m_pathRegion = wxRegion(*m_routeBmp, bgcolor, 5); //GetPathRegion(path);
////        m_pathRegion.Offset(m_padding,m_padding);
//    }
}

wxRealSize GPSRouteGauge::GetTileDegSize(int zoom, const wxPoint& tile)
{
    wxRealSize size;

    size.w = fabs( tileX2long(tile.x,zoom) - tileX2long(tile.x + 1, zoom) );
    size.h = fabs( tileY2lat(tile.y,zoom) - tileY2lat(tile.y + 1, zoom) );

    return size;
}

wxRealSize GPSRouteGauge::GetTileDegSize(int zoom, const wxRect& tilerect)
{
    wxRealSize size(0.0,0.0);

    for(int x = tilerect.GetLeft(); x <= tilerect.GetRight(); x++) {
        size.w += fabs( tileX2long(x,zoom) - tileX2long(x+1, zoom) );
    }

    for(int y = tilerect.GetTop(); y <= tilerect.GetBottom(); y++) {
        size.h += fabs( tileY2lat(y,zoom) - tileY2lat(y+1, zoom) );
    }

    return size;
}

wxRegion GPSRouteGauge::GetPathRegion(const RealPointArray& path)
{
    double w = GetSize().x - 2*m_padding;
    double h = GetSize().y - 2*m_padding;
    wxAgg aggCanvas( (int)w, (int)h );

    wxColor bgcolor = wxColor(255,255,255,255);

    aggCanvas.clear(bgcolor);
    aggCanvas.setPen(wxColor(0,0,0,255));

    aggCanvas.drawLine(path, GetThemeDouble(_T("pathlinehlwidth")));

    wxBitmap * bmp = aggCanvas.renderToBitmap();

    wxRegion reg(*bmp, bgcolor, 5);

    delete bmp;

    return reg;
}


void GPSRouteGauge::PaintTiles(wxDC& dc)
{

    int level = m_actualZoom;

    if(m_tileBitmaps.find(level) == m_tileBitmaps.end()) return;

    wxSize size = GetClientSize();

//    Logger::logf(LOG_MSG,_T("m_actualMapRect: ")+m_actualMapRect.GetAsStringBorders(3));
//    Logger::logf(LOG_MSG,_T("m_actualMapRect: ")+m_actualMapRect.GetAsString(6));

    m_actualTileRect = wxDefaultRect;

    int tile_top=-1,tile_bottom=10000000,tile_left=-1,tile_right=10000000;

    Int2Int2wxBitmapHash& currlevelmaphash = m_tileBitmaps[level];

    wxRealRect tilerefrect = GetTileRealRect( m_actualMapRect, level );

    double longitude_L = 0.0, longitude_R = 0.0, latitude_T = 0.0, latitude_B = 0.0;

    for(Int2Int2wxBitmapHash::iterator itx = currlevelmaphash.begin(); itx != currlevelmaphash.end(); itx++ ) {
        int x = itx->first;

        if( (double)x+1 < tilerefrect.GetLeft() ) continue;
        if( (double)x-1 > tilerefrect.GetRight() ) continue;

        if(m_debugFlags & GDF_PAINT_TILE_INFO) {
            longitude_L = tileX2long(x,level);
            longitude_R = tileX2long(x+1,level);
        }


        int tile_xpos = (int)(((double)x - tilerefrect.GetLeft()) * (double)m_tileSizePx);

        for(Int2wxBitmapHash::iterator ity = itx->second.begin(); ity != itx->second.end(); ity++ ) {

            int y = ity->first;

            if( (double)y+1 < tilerefrect.GetTop() ) continue;
            if( (double)y > tilerefrect.GetBottom() ) continue;

            TileDesc currTile(x,y,level);

            if(m_debugFlags & GDF_PAINT_TILE_INFO) {
                latitude_T = tileY2lat(y,level);
                latitude_B = tileY2lat(y+1,level);
            }


            int tile_ypos = (int)(((double)y - tilerefrect.GetTop()) * (double)m_tileSizePx);

            wxBitmap& bmp = ity->second;

            dc.DrawBitmap(bmp, tile_xpos, tile_ypos);

            if(!TileExistInModel(currTile,m_routeBitmaps)) {
                if(!DrawRouteTile(currTile))
                    m_routeBitmaps[level][x][y] = wxNullBitmap;
            }

            if(TileExistInModel(currTile,m_routeBitmaps)) {
                wxBitmap& bmp = m_routeBitmaps[level][x][y];
                if(bmp.IsOk())
                    dc.DrawBitmap(bmp,tile_xpos,tile_ypos);
            }


            if(m_debugFlags & GDF_PAINT_TILE_INFO) {
                dc.SetBrush(*wxTRANSPARENT_BRUSH);
                dc.SetPen(*wxBLACK_PEN);
                dc.DrawLine( tile_xpos, tile_ypos, tile_xpos+256, tile_ypos);
                dc.DrawLine( tile_xpos, tile_ypos, tile_xpos, tile_ypos+256);

                wxString str = wxString::Format(_T("left = %.5f\nright = %.5f\ntop = %.5f\nbottom = %.5f"),longitude_L, longitude_R, latitude_T, latitude_B);
                str += wxString::Format(_T("\ntileX = %i\ntileY = %i"),x,y);
                wxSize s = dc.GetMultiLineTextExtent(str);
                wxPoint strpos = wxPoint(tile_xpos+ (m_tileSizePx-s.x)/2 ,tile_ypos+(m_tileSizePx-s.y)/2);
                wxSize sp(4,4);
                wxSize canvass = s+sp;
                wxAgg2 tmpcanvas(canvass);
                wxAggBrush * brush = tmpcanvas.getBrush();
                brush->SetColor(wxColor(255,255,255,180));
                brush->SetStyle(wxAGGBRUSH_SOLID);
                tmpcanvas.drawRectangle(0,0,canvass.x,canvass.y);
                wxBitmap* tmpbmp = tmpcanvas.renderToBitmap();
                dc.DrawBitmap(*tmpbmp,strpos.x-sp.x/2,strpos.y-sp.y/2);
                delete tmpbmp;
                DrawHelper::DrawMultiLineText(&dc, str, strpos, wxALIGN_CENTER);
            }

            if( tile_top < y ) tile_top = y;
            if( tile_bottom > y ) tile_bottom = y;

        }

        if( tile_left < x ) tile_left = x;
        if( tile_right > x ) tile_right = x;

    }

    if(tile_top != -1 && tile_left != -1 && tile_bottom != 10000000 && tile_right != 10000000)
        m_actualTileRect = wxRect( wxPoint(tile_left,tile_top), wxPoint(tile_right,tile_bottom) );

}

void GPSRouteGauge::PaintRoute(wxDC& dc)
{
    if(!m_routeBmp) return;

    wxRect pxrect = wxRect( wxPoint(0,0), GetClientSize());
    pxrect.Deflate(m_padding);

    wxRealRect tileOriBorder = GetTileRealRect( m_oriGPSBorder, m_actualZoom );

    wxRealRect actualTileRect = GetTileRealRect( m_actualMapRect, m_actualZoom );

    double path_line_highlight_w = GetThemeDouble(_T("pathlinehlwidth"));

    int x = (int)(((tileOriBorder.x - actualTileRect.x) / actualTileRect.w) * (double)pxrect.width - path_line_highlight_w);
    int y = (int)(((tileOriBorder.y - actualTileRect.y) / actualTileRect.h) * (double)pxrect.height - path_line_highlight_w);

    dc.DrawBitmap( *m_routeBmp,x,y,1);
}

void GPSRouteGauge::PaintLocPos(wxDC& dc)
{
    wxPoint locpos = wxDefaultPosition;

    if(m_locationPointBmp) {

        wxRealPoint tilepos = deg2tileRef(m_location, m_actualZoom);
        wxRealRect tilerefrect = GetTileRealRect(m_actualMapRect,m_actualZoom);

        if(tilerefrect.Contains(tilepos)) {
            int s = (int)(ceil(GetThemeDouble(_T("pointersize"))/2.0));

            wxSize size = GetClientSize();
            size.DecBy(m_padding*2);

            locpos = wxPoint(  (int)((tilepos.x - tilerefrect.x)/tilerefrect.w * (double)size.x),
                                    (int)((tilepos.y - tilerefrect.y)/tilerefrect.h * (double)size.y) );

            wxPoint cpos = locpos - wxPoint( s,s );

            dc.DrawBitmap( *m_locationPointBmp,cpos.x,cpos.y);
        }

    }

    if(m_toolTipRTPosBmp && locpos != wxDefaultPosition) {
        m_toolTipMgr->SetCurrentProfile(ATTPROFILE_CURRVAL);
        wxPoint ap = locpos - m_toolTipMgr->GetArrowPoint();// + GetPaddingPT();
        dc.DrawBitmap( *m_toolTipRTPosBmp,ap.x,ap.y);
    }
}

void GPSRouteGauge::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);

    dc.SetBackground(wxBrush(*wxWHITE_BRUSH));
    dc.Clear();

    PaintTiles(dc);

    if(m_location != wxDefaultRealPointEx) {
        PaintLocPos(dc);
    }

    if(m_toolTipBmp) {

        wxPoint pos = m_ttPos;

        if( pos != wxDefaultPosition ) {
            m_toolTipMgr->SetCurrentProfile(ATTPROFILE_DEF);
            wxPoint ap = pos - m_toolTipMgr->GetArrowPoint() + GetPaddingPT();
            dc.DrawBitmap( *m_toolTipBmp,ap.x,ap.y,1);
        }
    }

    if(m_debugFlags & GDF_USE_DRAW_CURRENT_GPS_POS) {
        PaintDebugPos(dc);
    }

    PaintWidgets(dc);

    if(m_bgBorder)
        dc.DrawBitmap( *m_bgBorder,0,0);

    DrawExtraLayer(dc);
}

void GPSRouteGauge::PaintDebugPos(wxDC& dc)
{

    wxRealRect tilerefrect = GetTileRealRect(m_actualMapRect, m_actualZoom);

    wxSize cs = GetClientSize();
    cs.DecBy( m_padding*2 );

    wxPoint mpos = m_lastMousePos;
    mpos.x -= m_padding;
    mpos.y -= m_padding;

    double xref = (double)mpos.x / (double)cs.x;
    double yref = (double)mpos.y / (double)cs.y;

    wxRealPoint tilep( tilerefrect.x + tilerefrect.w * xref, tilerefrect.y + tilerefrect.h * yref );

    wxRealPoint degp = tileRef2deg( tilep, m_actualZoom );

    wxString ptstr = wxString::Format(_T("x=%.5f\ny=%.5f"),degp.x,degp.y );


    dc.SetPen(*wxRED_PEN);
    int r = 20;
    dc.DrawLine( m_lastMousePos.x-r, m_lastMousePos.y-r, m_lastMousePos.x+r, m_lastMousePos.y+r );
    dc.DrawLine( m_lastMousePos.x+r, m_lastMousePos.y-r, m_lastMousePos.x-r, m_lastMousePos.y+r );

    wxString acstr = m_actualMapRect.GetAsStringBorders(5);
    wxSize size = dc.GetTextExtent(acstr);

    dc.DrawCircle(m_lastMousePos, (int)ceil(0.8 * r));

    dc.SetBrush( *wxWHITE_BRUSH );
    wxRect acRect( (GetClientSize().x-size.x)/2 , cs.y-size.y ,size.x,size.y );
    acRect.Inflate(2);
    dc.DrawRectangle( acRect );
    dc.DrawText(acstr, acRect.x + 2 , cs.y-size.y+1 );


    size = dc.GetMultiLineTextExtent(ptstr);

    int dpad = 15;
    dc.DrawRectangle(m_lastMousePos.x + dpad, m_lastMousePos.y + dpad , size.x+4, size.y+4);
    DrawHelper::DrawMultiLineText(&dc, ptstr, m_lastMousePos + wxPoint(dpad+2,dpad+2), wxALIGN_LEFT );
}

void GPSRouteGauge::OnSize(wxSizeEvent& evt)
{
    UpdateSize();

    evt.Skip();
}

wxRealRect GPSRouteGauge::GetTileRealRect(const wxRealRect& degrect, int zoom)
{
    wxRealRect rect;
    rect.x = long2tileXref( degrect.x, zoom );
    rect.y = lat2tileYref( degrect.y+degrect.h, zoom );

    rect.w = long2tileXref( degrect.GetRight(), zoom ) - rect.x;
    rect.h = lat2tileYref( degrect.GetTop(), zoom ) - rect.y;

    return rect;
}

wxRealRect GPSRouteGauge::GetDegFromTileRef(const wxRealRect& tilerect, int zoom)
{
    wxRealRect rect;

    wxRealPoint tl = tileRef2deg( tilerect.GetTopLeft(), zoom );

    wxRealPoint br = tileRef2deg( tilerect.GetBottomRight(), zoom );

    rect.SetTopLeft( br.y, tl.x );

    rect.SetBottomRight( tl.y, br.x );

    return rect;
}

void GPSRouteGauge::OnWheel(wxMouseEvent& evt)
{
    Zoom((evt.GetWheelRotation() > 0 ? 1 : -1),evt.GetPosition() );

    GaugeRefresh();
}

void GPSRouteGauge::ZoomTo(int newzoom, const wxPoint& zoomcenter)
{
    if( newzoom < 0 ) newzoom = 0;
    if( newzoom > 18 ) newzoom = 18;

    if( m_actualZoom == newzoom ) return;

    wxRealSize size = GetClientSize();

    wxPoint zoomc = zoomcenter == wxDefaultPosition ? wxPoint((int)size.w/2,(int)size.h/2) : zoomcenter;

    if( m_actualTileRect == wxDefaultRect ) return;

    wxRealPoint mpos = wxRealPoint( zoomc.x, zoomc.y );


    double xref = mpos.x / size.w;
    double yref = mpos.y / size.h;


    wxRealRect actualTileRefRect = GetTileRealRect(m_actualMapRect, m_actualZoom);

    wxRealPoint tileRefCenter;
    tileRefCenter.x = actualTileRefRect.x + actualTileRefRect.w * xref;
    tileRefCenter.y = actualTileRefRect.y + actualTileRefRect.h * yref;

    wxRealPoint degRefCenter = tileRef2deg(tileRefCenter, m_actualZoom);

    wxRealPoint newTileRefCenter = deg2tileRef(degRefCenter, newzoom);

    wxRealPoint topleftTileRef( newTileRefCenter.x - (actualTileRefRect.w * xref), newTileRefCenter.y - (actualTileRefRect.h * yref) );

    wxRealRect newrect = GetDegFromTileRef( wxRealRect(topleftTileRef, actualTileRefRect.GetSize()), newzoom );

    size.w -= m_padding*2;
    size.h -= m_padding*2;

//    Logger::logf(LOG_MSG,_T("oldrect: ")+m_actualMapRect.GetAsStringBorders(4));
//    Logger::logf(LOG_MSG,_T("newrect: ")+newrect.GetAsStringBorders(4));

    wxPoint startTile;
    startTile.x = long2tileX( newrect.GetLeft(), newzoom );
    startTile.y = lat2tileY( newrect.GetBottom(), newzoom );

    //Logger::logf(LOG_MSG,_T("oldtiles: x=%i,y=%i,w=%i,h=%i"),m_actualTileRect.x,m_actualTileRect.y,m_actualTileRect.width,m_actualTileRect.height);

    m_actualTileRect.SetTopLeft( startTile );

    m_actualTileRect.width  = (int)ceil(size.w/m_tileSizePx) + 1;
    m_actualTileRect.height = (int)ceil(size.h/m_tileSizePx) + 1;

    m_actualMapRect = newrect;
    m_actualZoom = newzoom;

    //Logger::logf(LOG_MSG,_T("newtiles: x=%i,y=%i,w=%i,h=%i"),m_actualTileRect.x,m_actualTileRect.y,m_actualTileRect.width,m_actualTileRect.height);

    m_tileMgr->ClearHistory();

    CheckMapTiles();

    //DrawRoute();

    m_zoomSlider->ChangeValue( m_actualZoom );


}

void GPSRouteGauge::Zoom(int direction, const wxPoint& zoomcenter)
{
    ZoomTo(m_actualZoom+direction,zoomcenter);
}

void GPSRouteGauge::OnSliderZoom(wxScrollEvent& evt)
{
    ZoomTo(evt.GetPosition());
    GaugeRefresh();
}

void GPSRouteGauge::OnMouseMove(wxMouseEvent& evt)
{
    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }


///  Contains( evt.GetPosition() )    // checking path region
    if(0) {

        if( !(m_mouseState & RMS_OVERPATH) )  {
            //DrawRoute( GetThemeColor(_T("pathlinehlcolor")) );
            SetCursor(wxCursor(wxCURSOR_HAND));
        }

        RouteElement * re = SearchByPoint( evt.GetPosition() - wxPoint(m_padding,m_padding) );
        if(re) {
            m_toolTipMgr->SetCurrentProfile(ATTPROFILE_DEF);
            DrawToolTip(re, &m_toolTipBmp );
            m_ttPos = wxPoint( (int)re->Point.x, (int)re->Point.y );
        } else {
            m_ttPos = wxDefaultPosition;
        }

        m_mouseState |= RMS_OVERPATH;

        GaugeRefresh();

    } else if( m_mouseState & RMS_ONMAPDRAG ) {

        wxRealRect startTileRef = GetTileRealRect(m_startDragRect,m_actualZoom);

        wxPoint cpos = evt.GetPosition();

        double tileSize = m_tileSizePx;

        wxPoint tmp = cpos - m_lastMapDragPoint;

        double xref = (double)tmp.x / tileSize;
        double yref = (double)tmp.y / tileSize;

        startTileRef.Offset( xref * -1 , yref * -1 );

        m_actualMapRect = GetDegFromTileRef(startTileRef,m_actualZoom);

        CheckMapTiles();

        GaugeRefresh();

    } else {

        m_ttPos = wxDefaultPosition;
        if( (m_mouseState & RMS_OVERPATH) )  {
            //DrawRoute();

        }

        m_mouseState &= ~RMS_OVERPATH;
        SetCursor(m_dragCursor);
    }


    m_lastMousePos = evt.GetPosition();


    if(m_debugFlags & GDF_USE_DRAW_CURRENT_GPS_POS) {
        GaugeRefresh();
    }


    evt.Skip();
}

void GPSRouteGauge::GaugeRefresh()
{
    m_isRefresh = true;
}

void GPSRouteGauge::OnRefreshTimer(wxTimerEvent& evt)
{
    if(m_isRefresh)
        Refresh(false);

    m_isRefresh = false;
}

void GPSRouteGauge::CheckMapTiles()
{
    //Logger::logf(LOG_MSG,_T("tile: x=%i,y=%i,w=%i,h=%i"),m_actualTileRect.x,m_actualTileRect.y,m_actualTileRect.width,m_actualTileRect.height);

    if( m_actualTileRect == wxDefaultRect ) return;

    //Logger::logf(LOG_MSG,_T("m_reqTiles.GetCount()=%i"),m_reqTiles.GetCount());

    wxPoint startTile;
    startTile.x = long2tileX( m_actualMapRect.GetLeft(), m_actualZoom );
    startTile.y = lat2tileY( m_actualMapRect.GetBottom(), m_actualZoom );

    wxPoint endTile;
    endTile.x = long2tileX( m_actualMapRect.GetRight(), m_actualZoom );
    endTile.y = lat2tileY( m_actualMapRect.GetTop(), m_actualZoom ) + 1;

    //Logger::logf(LOG_MSG,_T("CheckMapTiles left=%i,top=%i,right=%i,bottom=%i"),startTile.x,startTile.y,endTile.x,endTile.y);

    for( int x = startTile.x; x <= endTile.x; x++ ) {
        for( int y = startTile.y; y <= endTile.y; y++ ) {
            TileDesc desc(x,y,m_actualZoom);

            if(TileExistInModel(desc,m_tileBitmaps)) continue;

            m_tileMgr->Request(desc);
        }
    }

}

bool GPSRouteGauge::TileExistInModel(TileDesc& desc, Int2Int2Int2wxBitmapHash& model)
{
    if( model.find( desc.zoom ) == model.end() ) return false;

    if( model[desc.zoom].find( desc.x ) == model[desc.zoom].end() ) return false;

    if( model[desc.zoom][desc.x].find( desc.y ) == model[desc.zoom][desc.x].end() ) return false;

    return true;
}


size_t GPSRouteGauge::GetModelSize(Int2Int2Int2wxBitmapHash& model)
{
    size_t c = 0;
    size_t bmpsize = sizeof(wxBitmap);

    for(Int2Int2Int2wxBitmapHash::iterator itz = model.begin(); itz != model.end(); itz++ ) {
        for(Int2Int2wxBitmapHash::iterator itx = itz->second.begin(); itx != itz->second.end(); itx++ ) {
            for(Int2wxBitmapHash::iterator ity = itx->second.begin(); ity != itx->second.end(); ity++ ) {
                wxBitmap& bmp = ity->second;
                c += (bmp.GetWidth() * bmp.GetHeight() * bmp.GetDepth() / 8) + bmpsize;
            }
        }
    }
    return c;
}

void GPSRouteGauge::InitHelpPanel()
{
    m_helpPanel->SetProperty(BCBS_BORDER_RADIUS, 3.0);
    m_helpPanel->SetProperty(BCBS_BORDER_WIDTH, 2.0);

    double text_height = 13;

    HotKeyDescArray arr;

    m_hotKeyMgr->GetHotKeyList(arr);

    wxString titleStr = wxEmptyString, hkeyStr = wxEmptyString;

    for(size_t a = 0; a < arr.GetCount(); a++) {
        titleStr += arr[a].Title + _T("\n");
        hkeyStr  += arr[a].HotKey + _T("\n");
    }

    hkeyStr.RemoveLast();
    titleStr.RemoveLast();

    m_helpPanel->ChangeLabel(TextDesc(titleStr,0,wxALIGN_LEFT,text_height));

    wxRect titlesBox = m_helpPanel->GetTextBox(0);

    m_helpPanel->ChangeLabel(TextDesc(hkeyStr,1,wxALIGN_LEFT,text_height, wxRealPoint(titlesBox.width+5,0) ));

    wxRect hkeysBox  = m_helpPanel->GetTextBox(1);

    titlesBox.width += 5 + hkeysBox.width;
    titlesBox.height -= 5;

    wxSize panelsize = titlesBox.GetSize();
    panelsize.IncBy(10);
    m_helpPanel->SetSize( panelsize );

    wxSize csize = GetClientSize();

    m_helpPanel->SetPosition( (csize.x - panelsize.x ) / 2 , (csize.y - panelsize.y ) / 2 + HELP_PANEL_DELTA_MOVE );

}

void GPSRouteGauge::OnLeftUp(wxMouseEvent& evt)
{
    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

    m_mouseState &= ~RMS_ONMAPDRAG;
    m_lastMapDragPoint = wxDefaultPosition;

    SetCursor(m_dragCursor);
}

void GPSRouteGauge::OnLeftDown(wxMouseEvent& evt)
{
    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

    if(m_mouseState & RMS_OVERPATH) {
        RouteElement * re = SearchByPoint( evt.GetPosition() - wxPoint(m_padding,m_padding) );
        if(re) {
            int idx = (int)re->GetVal(strLogIdxPos);
            m_dataMgr->refresh(idx);
        }
    } else {
        m_mouseState |= RMS_ONMAPDRAG;
        m_lastMapDragPoint = evt.GetPosition();
        m_startDragRect = m_actualMapRect;

        SetCursor( wxCursor( m_dragDownCursor ) );
    }

    evt.Skip();
}

void GPSRouteGauge::OnEnter(wxMouseEvent& evt)
{
    m_mouseState = RMS_NONE;
}

void GPSRouteGauge::OnLeave(wxMouseEvent& evt)
{
    m_mouseState = RMS_NONE;
    SetCursor(wxNullCursor);
}

void GPSRouteGauge::notify( wxVariantHash& hash )
{
    if(hash.find(_T("tile")) == hash.end()) {
        LOGP(wxString::Format(_T("no tile, hash size=%i"),hash.size()));
        return;
    }

    if(hash.find(_T("image")) == hash.end()) {
        LOGP(wxString::Format(_T("no image, hash size=%i"),hash.size()));
        return;
    }

    TileDesc desc;
    desc << hash[_T("tile")];

    //Logger::logf(LOG_MSG,_T("GPSRouteGauge::notify hash result=%i, tile=")+desc.GetAsString(), hash[_T("resultType")].GetLong() );

    wxImage img;
    img << hash[_T("image")];

    wxBitmap bmp( img );

    m_tileBitmaps[desc.zoom][desc.x][desc.y] = bmp;

    GaugeRefresh();
}


wxRegion GPSRouteGauge::GetShape()
{
    return wxRegion(0,0,GetSize().x,GetSize().y);
}

wxRegion GPSRouteGauge::GetBorderRegion()
{
    return wxRegion(0,0,1,1);
}

void GPSRouteGauge::UpdateSize()
{
    if(!m_isInited) return;

    InitRoute();

    //DrawRoute(GetThemeColor(_T("pathlinecolor")),true);

    DrawBorders();

    DrawLocationPoint();

    GaugeRefresh();
}

void GPSRouteGauge::setSize(int x, int y)
{

}

void GPSRouteGauge::SetValue(double value)
{

}

void GPSRouteGauge::SetNewTheme(WidgetThemeTable *theme)
{

}

void GPSRouteGauge::require()
{
    if( m_isRegistered ) return;
    //_T("height")
    m_rtDataMgr->registerSymbol( GPSLAT );
    m_rtDataMgr->registerSymbol( GPSLONG );


    m_isRegistered = true;
}

void GPSRouteGauge::refresh(DoubleHash* table)
{
    if( !m_isInited ) return;
    //Logger::logf(LOG_MSG,_T("GPSRouteGauge::refresh"));

    if( table->find( GPSLAT ) == table->end() || table->find( GPSLONG ) == table->end() ){
        LOG
        return;
    }

    UpdateViewerEvent event;

    event.m_userData[GPSLAT] = (*table)[GPSLAT];
    event.m_userData[GPSLONG] = (*table)[GPSLONG];

    AddPendingEvent( event );
}

void GPSRouteGauge::DrawToolTip(RouteElement * re, wxBitmap ** bmp)
{
    if(re == NULL) return;

    double oriref = 0.5;

    m_toolTipMgr->SetProperty( ATT_ARROW_REFPOSITION, oriref );
    m_toolTipMgr->SetProperty( ATT_ARROW_ALIGMENT, ATT_AA_BOTTOM );

    wxString str = wxString::Format(_T("Speed: %.1f km/h\n"),re->GetVal(_T("gpsSpeed")));
    str += _T("Time: ") + FormatTimeSpan( (int)(re->GetVal(_T("time")) * 1000 ));

    m_toolTipMgr->SetProperty( ATT_TEXT_CONTENT, str );

#if 0
    wxSize ttsize = m_toolTipMgr->CalcSize();

    int x = (int)re->Point.x;
    int y = (int)re->Point.y;

    int w = m_mapRect.GetWidth();
    //int h = m_mapRect.GetHeight();

    m_toolTipMgr->DrawBox(wxNullVariant, ATT_SPECIAL_NO_RENDER);
    wxPoint ap = m_toolTipMgr->GetArrowPoint();

    int diff = x - ap.x - 2;

    if(diff < 0) {
        double r = fabs((double)diff) / ttsize.x;
        m_toolTipMgr->SetProperty( ATT_ARROW_REFPOSITION, oriref - r );
    }

    diff = w - (x + ttsize.x - ap.x);

    if(diff < 0) {
        double r = fabs((double)diff) / ttsize.x;
        m_toolTipMgr->SetProperty( ATT_ARROW_REFPOSITION, oriref + r );
    }

    diff = y - ap.y;

    if(diff < 0) {
        m_toolTipMgr->SetProperty( ATT_ARROW_ALIGMENT, ATT_AA_TOP );
    }
#endif
    ptrdel(*bmp);
    *bmp = m_toolTipMgr->DrawBox();
}

void GPSRouteGauge::OnUpdateValue(UpdateViewerEvent& event)
{
    //LOG


    double latitude = event.m_userData[GPSLAT].GetDouble();
    double longitude = event.m_userData[GPSLONG].GetDouble();

    RouteElement * re = SearchByData(longitude,latitude);

    //LOGP(wxString::Format(_T("lat=%0.5f,lon=%0.5f, re=%p"),latitude,longitude,re));

    m_location = wxRealPointEx(longitude,latitude);

    if(!re) {
        m_location = wxDefaultRealPointEx;
        //LOGP(_T("RouteElement not found"));
        return;
    }

    m_toolTipMgr->SetCurrentProfile(ATTPROFILE_CURRVAL);

    //DrawToolTip(re, &m_toolTipRTPosBmp );


    GaugeRefresh();
}


RouteElement* GPSRouteGauge::SearchByPoint(const wxPoint& pos)
{
    size_t rdc = m_routeData.GetCount();

    if( rdc < 2 ) return NULL;

    wxRealPoint p;
    p.x = pos.x;
    p.y = pos.y;

    double dist_sum = 1000000;

    int didx = -1;
    for( size_t a = 0; a < rdc; a++ ) {

        RouteElement * re = m_routeData[a];

        double dx = fabs(re->Point.x - p.x);
        double dy = fabs(re->Point.y - p.y);

        if(dist_sum > dx+dy) {
            dist_sum = dx+dy;
            didx = a;
        }

    }

    if(didx >= 0) return m_routeData[didx];

    return NULL;
}

RouteElement* GPSRouteGauge::SearchByData(double longitude, double latitude)
{
    size_t rdc = m_routeData.GetCount();

    if( rdc < 2 ) return NULL;

    double dist_sum = 542;

    int didx = -1;
    for( size_t a = 0; a < rdc; a++ ) {

        RouteElement * re = m_routeData[a];

        double d_lat = fabs( re->GetVal(GPSLAT) - latitude);
        double d_lon = fabs( re->GetVal(GPSLONG) - longitude);


        if(dist_sum > d_lat+d_lon) {
            dist_sum = d_lat+d_lon;
            didx = a;
        }

    }

//    LOGP(wxString::Format(_T("%i"),didx));

    if(didx >= 0) return m_routeData[didx];

    return NULL;
}

wxString GPSRouteGauge::FormatTimeSpan(int time_millisec, bool isminsize, bool ismillisec)
{
    int minutes = time_millisec / 1000 / 60;
    int seconds = time_millisec / 1000 - minutes * 60;
    int millisecs = time_millisec - minutes*60*1000 - seconds * 1000;

    wxString str = (minutes > 0 || !isminsize) ? wxString::Format(_T("%i:"),minutes) : _T("");
    str += wxString::Format(_T("%02i"),seconds);
    if(ismillisec) str += (millisecs > 0 || !isminsize) ? wxString::Format(_T(".%03i"),millisecs) : _T("");

    return str;
}

void GPSRouteGauge::ToggleHelpPanel()
{
    wxSize size = GetClientSize();

    wxRect helppanelRect = m_helpPanel->GetRect();

    wxPoint panel_pos = helppanelRect.GetTopLeft();

    wxPoint tpos;

    tpos.x = (size.x - helppanelRect.width) / 2;

    bool isShow;

    tpos.y = (size.y - helppanelRect.height) / 2;

    if( m_helpPanel->GetAlpha() ) {

        tpos.y += HELP_PANEL_DELTA_MOVE;

        isShow = false;

    } else {

        isShow = true;
    }

    m_helpPanel->SlidePosition(tpos);
    m_helpPanel->SlideAlpha(isShow);
}

void GPSRouteGauge::ToggleInfoPanel()
{
    if(m_infoPanel->GetAlpha()) {
        m_infoPanel->SlideAlpha(false);
        return;
    }


    wxString str = _T("Cache\n - location: ") + m_tileMgr->GetCacheDir();
    str += _T("\n - size: ") + m_tileMgr->GetCacheSizeStr();

    wxULongLong memsize = GetModelSize(m_tileBitmaps) + GetModelSize(m_routeBitmaps);

    str += _T("\n\nMemory usage: ~") + wxFileName::GetHumanReadableSize(memsize);

    double text_height = 13;

    m_infoPanel->ChangeLabel(TextDesc(str,0,wxALIGN_LEFT,text_height));
    m_infoPanel->DrawWidget();
    m_infoPanel->SetPosition( wxPoint( (GetClientSize().x - (int)(m_infoPanel->GetSize().w))/2, m_padding+10) );

    //m_infoPanel->Show();

    m_infoPanel->SlideAlpha(true);
}

void GPSRouteGauge::CreatePopupMenu(wxMenu& menu)
{

}

void GPSRouteGauge::OnMouseRightDown(wxMouseEvent& evt)
{
    wxMenu popupMenu;

    CreatePopupMenu(popupMenu);

    PopupMenu( &popupMenu );
}

void GPSRouteGauge::MoveMap(int dir)
{
    wxRealRect tileRect = GetTileRealRect(m_actualMapRect,m_actualZoom);

    double diff = 0.1;

    switch(dir){
        case MD_LEFT:
            tileRect.x -= diff;
            break;

        case MD_RIGHT:
            tileRect.x += diff;
            break;

        case MD_UP:
            tileRect.y -= diff;
            break;

        case MD_DOWN:
            tileRect.y += diff;
            break;
    }



    m_actualMapRect = GetDegFromTileRef(tileRect,m_actualZoom);

    CheckMapTiles();
}

void GPSRouteGauge::OnKeyDown(wxKeyEvent &evt)
{
    if( m_groupManager->IsEditMode() ) {
        evt.Skip();
        return;
    }

    wxString symbol = m_hotKeyMgr->GetEvent( evt );


    if( symbol == _T("ZOOM_PLUS") ) {

        Zoom(1);

    } else if( symbol == _T("ZOOM_MINUS") ) {

        Zoom(-1);

    } else if( symbol == _T("DRAW_ALL_DEBUG_INFO") ) {

        if(m_debugFlags == GDF_NO_DEBUG_DRAW)
            m_debugFlags = GDF_ALL;
        else
            m_debugFlags = GDF_NO_DEBUG_DRAW;

    } else if( symbol == _T("HELP") ) {

        ToggleHelpPanel();
        return;

    } else if( symbol == _T("INFO") ) {

        ToggleInfoPanel();
        return;

    } else if( symbol == _T("MOVE_LEFT") ) {

        MoveMap(MD_LEFT);

    } else if( symbol == _T("MOVE_RIGHT") ) {

        MoveMap(MD_RIGHT);

    } else if( symbol == _T("MOVE_UP") ) {

        MoveMap(MD_UP);

    } else if( symbol == _T("MOVE_DOWN") ) {

        MoveMap(MD_DOWN);

    } else {
        evt.Skip();
        return;
    }

    GaugeRefresh();
}



