#include "GaugeCarrierPanel.h"

#include "../logger.h"
#include "TableEditorPanel.h"
#include "VemsWidgetBase.h"

#include <wx/notebook.h>

#include "GaugeCarrierBaseDesc.h"
#include "VemsControlBase.h"

BEGIN_EVENT_TABLE(GaugeCarrierPanel, wxPanel)

 //   EVT_KEY_DOWN(GaugeCarrierPanel::OnChar)

END_EVENT_TABLE()


GaugeCarrierPanel::GaugeCarrierPanel(wxWindow * parent, int id, const wxString &pagetext)
    :GaugeCarrierBase (parent)
{
    Create(parent,id,wxDefaultPosition,wxDefaultSize, wxBORDER_SUNKEN );
    InitWindow(this);
    //ctor
    SetBackgroundColour(wxColour(0,0,0));


    m_desc->m_hasShape = false;

    m_desc->m_graphQual = 2;
    m_desc->m_type = -1;
//    m_orientation = wxVERTICAL;

    m_pageText = pagetext;

}

GaugeCarrierPanel::~GaugeCarrierPanel()
{
    //dtor
}

void GaugeCarrierPanel::SetFocusToItem(int id)
{

}

void GaugeCarrierPanel::ResetMinMax()
{
    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){
        VemsWidgetBase * v = NULL;

        if(v = dynamic_cast<VemsWidgetBase*>(it->second) )
            v->ResetMinMax();
    }

}

void GaugeCarrierPanel::ShowCarrier(bool isShow)
{
    //maybe not neccessary
}

void GaugeCarrierPanel::CloseCarrier()   /// big memory leak,... because the Destroy method is crash
{
    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){
        CloseGauge( it->first );
    }

    wxWindow * win = GetParent();

    if(!win) return;

    if(win->IsKindOf(CLASSINFO(wxBookCtrl))){
        wxBookCtrl * bc =  (wxBookCtrl*)win;

        for(size_t a=0;a<bc->GetPageCount();a++) {
            if( bc->GetPageText( a ) != m_pageText ) continue;

            bc->RemovePage( a );

            Hide();

            break;

        }
    }

   // Destroy();

}

void GaugeCarrierPanel::CloseGauge(int winid, bool eraseHash)
{

    Logger::logf(LOG_MSG,_T("GaugeCarrierPanel::CloseGauge m_desc->m_IDtoControls[%i]=%p"),winid,m_desc->m_IDtoControls[winid]);

    if(winid == -1 || !m_desc->m_IDtoControls[winid] ) return;


    m_desc->m_IDtoControls[winid]->Hide();

    m_desc->m_IDtoControls[winid]->GetParent()->GetParent()->SetFocus();


    m_desc->m_IDtoControls.erase(winid);

    RTDataViewerBase * vwb = 0;
    TableEditorPanel* tep = 0;

    if(vwb = dynamic_cast<RTDataViewerBase*> (m_desc->m_IDtoControls[winid])){
        vwb->UnRegisterViewer();
    } else if(tep = dynamic_cast<TableEditorPanel*> (m_desc->m_IDtoControls[winid])){
        wxWindow* t = tep->GetTable();
        VemsControlBase* vcb = dynamic_cast<VemsControlBase*>(t);
        if(vcb) vcb->b_UnRegisterViewer();
    }

///    m_desc->m_IDtoControls[winid]->Destroy();

}

wxString GaugeCarrierPanel::GetTheme()
{
    return wxEmptyString;
}

void GaugeCarrierPanel::connectKeyDownEvent( wxWindow* window )
{
    //Logger::logf( LOG_MSG, _T("GaugeCarrierPanel::connectKeyDownEvent") );


    if( window )
    {
        window->Connect( wxID_ANY,
                         wxEVT_KEY_DOWN,
                         wxKeyEventHandler( GaugeCarrierPanel::OnChar ),
                         (wxObject*) NULL,
                         this );

        wxWindowListNode* node = window->GetChildren().GetFirst();

        while( node )
        {
            wxWindow* child = node->GetData();
            this->connectKeyDownEvent( child );

            node = node->GetNext();
        }
    }

}


void GaugeCarrierPanel::OnChar( wxKeyEvent& event )
{
    int keyCode = event.GetKeyCode();

    // Logger::logf( LOG_MSG, _T("GaugeCarrierPanel::OnChar() - %c"), keyCode );

    switch( keyCode )
    {
        case WXK_ESCAPE :
        {
            GetParent()->SetFocus();
        }break;
        case 'R' :
        {
            if( event.AltDown() && m_control ) { m_control->b_refreshData(); }
            else { event.Skip(); }//Logger::logf( LOG_MSG, _T("GaugeCarrierFrame::OnChar() - %c"), keyCode ); }

        }break;
        case 'W' :
        {
            if( event.AltDown() && m_control ) { m_control->b_writeData(); }
            else { event.Skip(); }
        }break;
        case 'B' :
        {
            if( event.AltDown() && m_control ) { m_control->b_burnData(); }
            else { event.Skip(); }
        }break;
        case 'G' :
        {
            if ( m_control ) { m_control->b_showTable3D(); }
        } break;
/*
        case 'T' :
        {
            wxString name = ::wxGetTextFromUser( _T(""),
                                                _T("Change table!"),
                                                _T(""),
                                                this,
                                                wxDefaultCoord,
                                                wxDefaultCoord,
                                                true );
            if ( m_table ) { m_table->change( name ); }
        } break;
*/
        default :
        {
            event.Skip();
        }
    }
}

