#include "GaugeFrameBase.h"

#include "../rtdatamanager/RTDataMgrBase.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../vemstoolicon/VemsToolIconBase.h"

#include "ArcBarGauge.h"

#include "VemsGaugeDigital.h"
#include "VemsTextWidget.h"
#include "VemsGraphWidgetAgg.h"
#include "VemsFlagWidget.h"
#include "VemsGaugeAgg.h"
#include "../MTConfigHelper.h"
#include "../WindowReg.h"

#include "../DirHelper.h"

#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"

#include "../VemsStateWidget.h"
#include "../VemsTaskBarIcon.h"

#include "GaugeType.h"
#include "../vemsdialog/TableEditor.h"
#include "../vemsdialog/StaticControlPanel.h"

#include "VemsGaugeDescDialog.h"

#include "../logger.h"

#if defined (__WINCE_STANDARDSDK__)
    //#include <shellsdk.h>		/// for WinAPI support
    #include <aygshell.h>		/// for WinAPI support
#endif

BEGIN_EVENT_TABLE(GaugeFrameBase, wxFrame)
    EVT_LEFT_DCLICK(GaugeFrameBase::OnDoubleClick)
    EVT_LEFT_DOWN(GaugeFrameBase::OnLeftDown)
    EVT_LEFT_UP(GaugeFrameBase::OnLeftDown)
    EVT_CLOSE(GaugeFrameBase::OnFrameClose)
    EVT_ACTIVATE(GaugeFrameBase::OnActivate)
    EVT_KEY_DOWN(GaugeFrameBase::OnChar)
    //EVT_MOVE(GaugeFrameBase::OnMove)
END_EVENT_TABLE()

GaugeFrameBase::GaugeFrameBase(wxWindow* parent, wxWindowID id,
                                //VemsTuneController * ctrl,
                                const wxString& title, const wxPoint& pos, const wxSize& size, long style)
    //:m_vemsTuneController(NULL)
    :m_wreg (NULL)
    ,m_TaskBarIconApp (NULL)
    ,m_CustomControlHash (NULL)
    ,m_rtDataMgr (NULL)
    ,m_stController (NULL)
    ,m_themedObj (NULL)
    ,m_gauge (NULL)
    ,m_table (NULL)
    ,m_rootdir (wxEmptyString)
    ,m_graphQual (0)
    ,m_onlyOneGauge (false)
{
    //ctor
#if defined(__WINCE_STANDARDSDK__)
    Create(parent,id,title,pos,size,style|wxFRAME_SHAPED);
#else
    #if defined(__WXMSW__)
        Create(parent,id,title,pos,size,style|wxFRAME_SHAPED | wxFRAME_TOOL_WINDOW  | wxFRAME_FLOAT_ON_PARENT );
    #else
        Create(parent,id,title,pos,size,style|wxFRAME_SHAPED);
    #endif
#endif

    //SetTransparent(200);
    SetBackgroundColour(wxColour(0,0,0));
/*
    m_vemsTuneController = ctrl;

    m_rtDataMgr = (RTDataMgrBase*)m_vemsTuneController->m_rtDataMgr;
    m_stController = (STDataMgrBase*)m_vemsTuneController->m_stController;
*/
    //SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);

    hasShape = false;

    ConfigBase* cfgbase = ConfigHelper2::getConfig();
    cfgbase->setPath(_T("/"));
    long tmp;
    cfgbase->read(_T("graphics_quality_factor"),&tmp, 0L);
    m_graphQual = tmp;

    m_type = -1;
    m_orientation = wxVERTICAL;

  //  Logger::logf(LOG_MSG,_T("GaugeFrameBase::GaugeFrameBase end"));

}

void GaugeFrameBase::setTaskBarIcon(VemsTaskBarIcon *ptr)
{
    m_TaskBarIconApp = ptr;
}

void GaugeFrameBase::OnActivate(wxActivateEvent& evt)
{
    if( m_table ) { m_table->SetFocus(); }
    if( m_gauge ) { m_gauge->SetFocus(); }

    if(m_TaskBarIconApp == NULL) return;
    if(m_wreg == NULL) return;

   	Logger::logf(LOG_MSG,_T("GaugeFrameBase::OnActivate  evt.GetActive()=%i"),evt.GetActive());

	bool tmp = m_TaskBarIconApp->IsRestored();

	Logger::logf(LOG_MSG,_T("GaugeFrameBase::OnActivate  m_TaskBarIconApp->IsRestored()=%i"),tmp);

    if(evt.GetActive() && m_TaskBarIconApp->IsRestored() ) {

        m_TaskBarIconApp->SetRestored(false);
        m_wreg->Maximize();

		Show(true);
    }


#if defined (__WINCE_STANDARDSDK__)
	SHFullScreen( (HWND)(GetHandle()) , SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON | SHFS_HIDESTARTICON);
#endif
    //evt.Skip();

	Logger::logf(LOG_MSG,_T("GaugeFrameBase::OnActivate END "));

}

void GaugeFrameBase::SetRootDir(wxString dir)
{

    m_rootdir = dir;

}

void GaugeFrameBase::InitInputSources()
{

}



void GaugeFrameBase::OnLeftDown(wxMouseEvent& evt)
{
#if !defined (__WINCE_STANDARDSDK__)

    wxPoint mpoint = evt.GetPosition();

    if( m_IDtoControls.find(evt.GetId()) != m_IDtoControls.end() )
        mpoint += m_IDtoControls[evt.GetId()]->GetPosition();

    Logger::logf(LOG_MSG,_T("GaugeFrameBase::OnLeftDown(x=%i, y=%i)"),mpoint.x,mpoint.y);

    for(wxControlHash::iterator iti = m_IDtoControls.begin(); iti != m_IDtoControls.end(); iti++) {

        if(iti->second->GetName() != _T("VemsToolIconBase")) continue;

        VemsToolIconBase *icon = (VemsToolIconBase*)(iti->second);
        wxRegion iregtmp = icon->GetShape();
        iregtmp.Offset(icon->GetPosition());

        if( iregtmp.Contains(mpoint) == wxInRegion ) {
            Logger::logf(LOG_MSG,_T("match!"));
            icon->ProcessEvent(evt);
            return;
        }

    }
    evt.Skip();

#endif
}

GaugeFrameBase::~GaugeFrameBase()
{
    /*
    if( m_onlyOneGauge )
    {
        Logger::logf(LOG_MSG,_T("GaugeFrameBase::Desctructor() - OnlyOneGauge"));
        if( m_gauge )
        {
            m_gauge->UnRegisterViewer();
        }
    }
    else
    {
        for( uint32_t i = 0; i < m_viewers.GetCount(); ++i )
        {
            Logger::logf( LOG_MSG, _T("GaugeFrameBase::Desctructor() - Gauges") );
            m_viewers.Item( i )->UnRegisterViewer();
        }
    }
    */
}
#if defined (__WINCE_STANDARDSDK__)
/* no such type: HWND  on linux. TODO: what to do ?
HWND GaugeFrameBase::GetHWND()
{
    return (HWND)GetHandle();
}*/
#endif

void GaugeFrameBase::SetDataMgrObj(RTDataMgrBase *rtDataMgr)
{
    m_rtDataMgr = rtDataMgr;
}

void GaugeFrameBase::SetDataMgrObj(STDataMgrBase *stDataMgr)
{
    m_stController = stDataMgr;
}

RTDataMgrBase* GaugeFrameBase::GetDataMgrObj(void)
{
    return m_rtDataMgr;
}

void GaugeFrameBase::ClearFrame(void)
{

  //  Logger::logf(LOG_MSG,_T("GaugeFrameBase::ClearFrame START"));

    for(wxControlHash::iterator it = m_IDtoControls.begin(); it != m_IDtoControls.end(); it++){
        Logger::logf(LOG_MSG,_T("itemName=%s, itemID=%i, itemPtr=%p"),it->second->GetName().c_str(),it->first, it->second);
        it->second->Destroy();
    }

//    Logger::logf(LOG_MSG,_T("GaugeFrameBase::ClearFrame 1"));

    m_IDtoControls.clear();

    //Logger::logf(LOG_MSG,_T("GaugeFrameBase::ClearFrame END"));

    ClearFrameShape();

}


void GaugeFrameBase::ClearFrameShape()
{
    m_region.Clear();
}

void GaugeFrameBase::SetWindowReg(WindowReg *wreg)
{
    m_wreg = wreg;
}
/*
int GaugeFrameBase::AddIcon(const wxString name, wxPoint pos, wxMenu *menu)
{
    int id = wxNewId();

    //Logger::log(LOG_MSG,_T("GaugeFrameBase::AddIcon !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"));

    VemsToolIconBase *icon = NULL;

    if(name == _T("Exit")){
        icon = new ExitIcon(this,id,pos);

    }else if(name == _T("Screen")){
        icon = new ScreenIcon(this,id,pos);
        ((ScreenIcon*)icon)->SetMenu(menu);

    }else if(name == _T("Input")){
        icon = new SelectInputIcon(this,id,pos,m_rtDataMgr);
        ((SelectInputIcon*)icon)->SetVemsTuneController(m_vemsTuneController);

    }else if(name == _T("Gps")){
        icon = new GPSIcon(this,id,pos,m_rtDataMgr);
        ((GPSIcon*)icon)->SetVemsTuneController(m_vemsTuneController);

    }else if(name == _T("Hide")){
        icon = new HideIcon(this,id,pos);
        ((HideIcon*)icon)->SetWindowReg(m_wreg);

    }else if(name == _T("File")){
        icon = new FileIcon(this,id,pos);
        ((FileIcon*)icon)->SetWindowReg(m_wreg);
        ((FileIcon*)icon)->SetVemsCtrl(m_vemsTuneController);

    }else if(name == _T("State.ECU") || name == _T("State.GPS") || name == _T("State.AIM") ){
        icon = new StateIcon(this,id,name,pos);
        ((StateIcon*)icon)->SetVemsTuneController(m_vemsTuneController);

    }else if(name == _T("Devices")){
        icon = new DevicesIcon(this,id,pos);
        ((DevicesIcon*)icon)->SetVemsTuneController(m_vemsTuneController);

    }



    if(icon != NULL) {
        m_IDtoControls[id] = icon;
        if(m_CustomControlHash != NULL) (*m_CustomControlHash)[id] = icon;
        icon->IconInit();
    }

    return 0;

}
*/

void GaugeFrameBase::RegIcon(int id, VemsToolIconBase *icon)
{
    if(icon != NULL) {
        m_IDtoControls[id] = icon;
        if(m_CustomControlHash != NULL) (*m_CustomControlHash)[id] = icon;
        icon->IconInit();
    }
}


GroupMember* GaugeFrameBase::AddVemsTable( GaugeDescManager * gaugemgr, GaugeProperties &prop )
//const wxString& symbolName, wxPoint pos, int width, bool hasControlPanel, GaugeGroupManager* groupManager, const wxString groupName )
{


    if(m_rtDataMgr == NULL) return NULL;
    if(m_stController == NULL) return NULL;


    TableEditor* table = new TableEditor( this , gaugemgr, prop );
    //wxDefaultPosition, wxDefaultSize, gaugemgr->GetGaugeGrpMgr(), prop.groupName );
    table->init( prop.symbolName, m_stController, m_rtDataMgr, MTConfigHelper::getMTConfigData() );


    if( m_onlyOneGauge )
    {

        m_table = table;

        wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );

        sizer->Add( table, 1, wxEXPAND );

        if( prop.extraParams[_T("hasControlPanel")].GetBool() )
        {
            int buttons = StaticControlPanel::DEFAULT_BUTTONS | StaticControlPanel::SHOW_REMAP_BUTTON;
            if ( !table->is3D() ) { buttons = StaticControlPanel::REMAPPER_TABLE_BUTTONS; }

            StaticControlPanel* ctrlPanel = new StaticControlPanel( this, table, buttons );
            m_stController->RegisterViewer( ctrlPanel );
            ctrlPanel->update();
            sizer->Add( ctrlPanel, 0, wxEXPAND );
        }

        SetTitle( table->GetLabel() );
        SetSizer( sizer );

        if ( table->is3D() )
        {
            SetInitialSize( wxSize( 600, 600 ) );
            //SetWindowStyle( wxMINIMIZE_BOX | wxCAPTION | wxSYSTEM_MENU | wxCLOSE_BOX  );
        }
        else { SetInitialSize(); }

        this->connectKeyDownEvent( this );
        m_table->SetFocus();

    }
    else
    {
        table->SetPosition( prop.getPos() );
        table->SetSize( prop.getSize() );
    }

    int id = wxNewId();

    m_IDtoControls[id] = (wxControl*)table;
    if(m_CustomControlHash != NULL) (*m_CustomControlHash)[id] = (wxControl*)table;


    table->GetGrpMember()->SetItemID( prop.itemID );

    return table->GetGrpMember();
}

/*
VemsWidgetBase* GaugeFrameBase::AddGauge( int itemid, const VEMSGaugeDescArray * vemsGaugeDescA, const WidgetThemeTableArray * themes, const wxString type,
                                const wxString& symbolName, wxPoint pos, int width, int height, bool ShowBorder, GaugeGroupManager* groupManager,
                                const wxString groupName, const wxString& currTheme, const wxString& desc, const wxString& ghotkey)
                                */
VemsWidgetBase* GaugeFrameBase::AddGauge( GaugeDescManager * gaugemgr, GaugeProperties &prop )
{
//	Logger::logf( LOG_MSG, _T("GaugeFrameBase::AddGauge() - START vemsGaugeDescA size: %i"), vemsGaugeDescA->GetCount() );
    if(m_rtDataMgr == NULL) return NULL;

    VemsWidgetBase *gauge = NULL;

    int id = wxNewId();

    if( prop.extraParams.find(_T("showBorder")) != prop.extraParams.end() )
        if(!prop.extraParams[_T("showBorder")].GetBool() )
            prop.extraParams[_T("windowStyle")] = (long)wxNO_BORDER;

    m_type = prop.gaugeType;
    switch( prop.gaugeType ) {
        case GaugeType::ROUND:
            //gauge = new VemsGaugeAgg(this,id,pos,wxSize(width,height),style, vemsGaugeDescA, themes, 0, groupManager, groupName,currTheme );
            gauge = new VemsGaugeAgg(this, id, gaugemgr, prop );
            break;
        case GaugeType::ARCBAR:
            //gauge = new ArcBarGauge(this,id,symbolName,pos,width, style, vemsGaugeDescA, themes, currTheme);
            gauge = new ArcBarGauge(this, id, gaugemgr, prop);
            break;

        case GaugeType::DIGITAL:
            //gauge = new VemsGaugeDigital(this,id,symbolName,pos,width, m_rootdir, style, vemsGaugeDescA, themes, groupManager, groupName,currTheme );
            gauge = new VemsGaugeDigital(this, id, gaugemgr, prop );
            break;

        case GaugeType::TEXT:
            //gauge = new VemsTextWidget(this,id,pos,width, style, vemsGaugeDescA, themes,groupManager, groupName,currTheme );
            gauge = new VemsTextWidget(this, id, gaugemgr, prop );
            break;

        case GaugeType::GRAPH:
            //gauge = new VemsGraphWidgetAgg( this, id, pos, wxSize( width, height ), style, vemsGaugeDescA, themes, 0, groupManager, groupName,currTheme );
            gauge = new VemsGraphWidgetAgg( this, id, gaugemgr, prop );
            break;

        default:
            return NULL;
    }


    if(gauge != NULL){
        gauge->SetItemID( prop.itemID );

        //Logger::logf( LOG_MSG, _T("GaugeFrameBase::AddGauge() - set gauge - START") );
        m_IDtoControls[id] = gauge;
        if(m_CustomControlHash != NULL) (*m_CustomControlHash)[id] = gauge;

        //gauge->Hide();
        gauge->SetHotKeyStr( prop.groupHotkey );


        gauge->setDescEditor(new VemsWidgetBaseEditor(gauge));

//! This makes My PNA device freeze the whole device when loading second roundgauge, type "secl" on Gauges 480x272 (last gauge & last screen item)
// ifdef just temporary workaround until it gets fixed
#if defined (__WINCE_STANDARDSDK__)

#else
		if( prop.extraParams.find(_T("showBorder")) != prop.extraParams.end() )
            if(!prop.extraParams[_T("showBorder")].GetBool())
                gauge->SetWindowStyle(wxNO_BORDER);
#endif
//-------------------------------------------------------------

        gauge->SetGaugeBySymbolName( prop.symbolName );

        gauge->SetGaugeDescByName( prop.descName );

        m_rtDataMgr->RegisterViewer(gauge);
        gauge->SetDataMgr(m_rtDataMgr);

        gauge->UpdateSize();

        wxRegion regiontemp = gauge->GetShape();

        gauge->Show(m_onlyOneGauge);

        if(m_onlyOneGauge){

            int x,y,w,h;
            regiontemp.GetBox(x,y,w,h);
            SetSize(w,h);

            wxRegion tmp3 = gauge->GetShape();
            tmp3.Offset(2,2);
            SetShape(tmp3);

            m_gauge = gauge;
            m_gauge->SetShape(true);

        } else {
            regiontemp.Offset( prop.getPos() );
            m_region.Union( regiontemp );
            SetShape(m_region);
        }


    }

    m_themedObj = gauge;


    return gauge;

}

int GaugeFrameBase::AddStateWidget(const wxString name, wxPoint pos, int size, bool isShow)
{
    //Logger::logf(LOG_MSG,_T("GaugeFrameBase::AddStateWidget START"));

    int id = wxNewId();

    VemsStateWidget *statew;

    statew = new VemsStateWidget(this,id, pos,wxSize(size,size/3),0,isShow);
    //statew->Hide();

    m_IDtoControls[id] = statew;
    if(m_CustomControlHash != NULL) (*m_CustomControlHash)[id] = statew;


    return 0;
}


int GaugeFrameBase::AddFlagWidget(const wxString &name, wxPoint pos, int size)
{

//    int id = wxNewId();
//
//    VemsFlagWidget *flag = NULL;//new VemsFlagWidget(this, id, name, pos, size,0,
//#if defined (__WINCE_STANDARDSDK__)
//    false
//#else
//    true
//#endif
//    );
//    //flag->Hide();
//
//    //flag->printThisptr();
//    //Logger::logf(LOG_MSG,_T("GaugeFrameBase::AddFlagWidget 1ptr=%p"),flag);
//
//    m_rtDataMgr->RegisterViewer(flag);
//    flag->SetDataMgr(m_rtDataMgr);
//
//    m_IDtoControls[id] = flag;
//    if(m_CustomControlHash != NULL) (*m_CustomControlHash)[id] = flag;
//
//
//    //bool res =
//    ((RTDataViewerBase*)((wxControl*)flag))->IsRegistered();
//
//    //bool res = ((RTDataViewerBase*)flag)->IsRegistered();
//
//    //Logger::logf(LOG_MSG,_T("GaugeFrameBase::AddFlagWidget ptr1=%p, ptr2=%p, ID=%i, res=%i"),flag,(*m_CustomControlHash)[id], id, res);
//
//    //wxRegion regiontemp = ((VemsFlagWidget*)(*m_CustomControlHash)[id])->GetShape();
//    wxRegion regiontemp = flag->GetShape();
//    regiontemp.Offset(pos);
//    m_region.Union( regiontemp );
//    //int shapres =
//    SetShape(m_region);

    return 0;

}

GroupMember* GaugeFrameBase::AddFlagWidget( const wxString& name, GaugeGroupManager* groupManager, const wxString groupName )
{
    //set sizer
//    wxSizer* sizer = GetSizer();
//
//    if( !sizer )
//    {
//        sizer = new wxBoxSizer( m_orientation );
//        SetSizer( sizer );
//    }
//
//    //add widget
//    int id = wxNewId();
//    int style = wxBORDER_NONE;
//    VemsFlagWidget* flag = NULL;//new VemsFlagWidget( this, id, name, wxDefaultPosition, 80, style, false, groupManager, groupName );
//
//    //register to controller
//    m_rtDataMgr->RegisterViewer( flag );
//    flag->SetDataMgr( m_rtDataMgr );
//
//    //???
//    m_IDtoControls[ id ] = flag;
//    if( m_CustomControlHash != NULL ) (*m_CustomControlHash)[id] = flag;
//    m_viewers.Add( flag );
//    //
//    flag->Show();
//    sizer->Add( flag, 0, 0 );

    return NULL;
}

void GaugeFrameBase::OnFrameClose(wxCloseEvent& event)
{
    Logger::logf(LOG_MSG,_T("GaugeFrameBase::OnFrameClose"));
    Destroy();
}

void GaugeFrameBase::OnDoubleClick(wxMouseEvent& WXUNUSED(evt))
{
    Logger::logf(LOG_MSG,_T("GaugeFrameBase::OnDoubleClick() START"));

    //region.ConvertToBitmap().SaveFile(_T("region.bmp"),wxBITMAP_TYPE_BMP,0);

    if(m_onlyOneGauge ) {

        if( m_gauge )
        {
            if( m_gauge->HasShape() )
            {
                wxSize size = GetClientSize();
                m_gauge->SetSize(size.GetWidth(), size.GetWidth() );
                m_gauge->setSize(size.GetWidth(),size.GetWidth() );
                wxRegion region;
                region.Clear();
                SetShape(region);
                m_gauge->SetShape(false);
                SetWindowStyleFlag(GetWindowStyle()|wxRESIZE_BORDER);
            }
            else
            {
                wxSize size = GetClientSize();
                m_gauge->SetSize(size.GetWidth(), size.GetWidth() );
                m_gauge->setSize(size.GetWidth(),size.GetWidth() );
                wxRegion region = m_gauge->GetShape();
                m_gauge->SetShape(SetShape(region));
                SetWindowStyleFlag(GetWindowStyle()&~wxRESIZE_BORDER);
            }
        }

    }
    else {

        wxRegion tempreg;

        if(hasShape){
            tempreg = m_region;
            hasShape = false;
        } else {
            hasShape = true;
        }

        SetShape(tempreg);

    }



}


wxControlHash* GaugeFrameBase::GetControlHash()
{
    return m_CustomControlHash;
}

void GaugeFrameBase::SetCustomControlHash( wxControlHash *customControlHash)
{
    m_CustomControlHash = customControlHash;
}
/*
void GaugeFrameBase::OnChar(wxKeyEvent& event)
{
   // Logger::logf(LOG_MSG,_T("GaugeFrameBase::OnChar() keyCode: %d"), event.KeyCode() );

    event.Skip();
}
*/
void GaugeFrameBase::unregisterViewer()
{
    //Logger::logf( LOG_MSG, _T("GaugeFrameBase::unregisterViewer()") );

    if( m_onlyOneGauge )
    {
        if( m_gauge )
        {
            m_gauge->UnRegisterViewer();
        }
        if( m_table )
        {
            m_table->UnRegisterViewer();
        }
    }
    else
    {
        for( uint32_t i = 0; i < m_viewers.GetCount(); ++i )
        {
            m_viewers.Item( i )->UnRegisterViewer();
        }
    }

    if ( !m_CustomControlHash ) { return; }

    wxControlHash::iterator it = m_CustomControlHash->begin();

    for( ; it != m_CustomControlHash->end() ; ++it )
    {
        if( it->second->GetName() == _T("VemsWidgetBase") )
        {
            if( ( (VemsWidgetBase*)it->second )->IsRegistered() )
            {
                ( (VemsWidgetBase*)it->second )->UnRegisterViewer();
            }
        }
        else if( it->second->GetName() == _T("VemsFlagWidget") )
        {
            if( ( (VemsFlagWidget*)it->second )->IsRegistered() )
            {
                ( (VemsFlagWidget*)it->second )->UnRegisterViewer();
            }
        }
    }

}

void GaugeFrameBase::connectKeyDownEvent( wxWindow* window )
{
    //Logger::logf( LOG_MSG, _T("GaugeFrameBase::connectKeyDownEvent") );
    if( window )
    {
        window->Connect( wxID_ANY,
                         wxEVT_KEY_DOWN,
                         wxKeyEventHandler( GaugeFrameBase::OnChar ),
                         (wxObject*) NULL,
                         this );

        wxWindowListNode* node = window->GetChildren().GetFirst();

        while( node )
        {
            wxWindow* child = node->GetData();
            this->connectKeyDownEvent( child );

            node = node->GetNext();
        }
    }
}

void GaugeFrameBase::OnChar( wxKeyEvent& event )
{
    int keyCode = event.GetKeyCode();

    // Logger::logf( LOG_MSG, _T("GaugeFrameBase::OnChar() - %c"), keyCode );

    switch( keyCode )
    {
        case WXK_ESCAPE :
        {
            GetParent()->SetFocus();
        }break;
        case 'R' :
        {
            if( event.AltDown() && m_table ) { m_table->refreshData(); }
            else { event.Skip(); }//Logger::logf( LOG_MSG, _T("GaugeFrameBase::OnChar() - %c"), keyCode ); }

        }break;
        case 'W' :
        {
            if( event.AltDown() && m_table ) { m_table->writeData(); }
            else { event.Skip(); }
        }break;
        case 'B' :
        {
            if( event.AltDown() && m_table ) { m_table->burnData(); }
            else { event.Skip(); }
        }break;
        case 'G' :
        {
            if ( m_table ) { m_table->showTable3D(); }
        } break;
/*
        case 'T' :
        {
            wxString name = ::wxGetTextFromUser( _T(""),
                                                _T("Change table!"),
                                                _T(""),
                                                this,
                                                wxDefaultCoord,
                                                wxDefaultCoord,
                                                true );
            if ( m_table ) { m_table->change( name ); }
        } break;
*/
        default :
        {
            event.Skip();
        }
    }
}

void GaugeFrameBase::OnMove( wxMoveEvent& event )
{
    //Logger::logf(LOG_MSG,_T("GaugeFrameBase::OnMove()!") );
}
