#ifndef GAUGEGROUPMANAGER_H
#define GAUGEGROUPMANAGER_H

#include <wx/wx.h>

#if !defined(__WINCE_STANDARDSDK__) && !defined(__POCKETPC__)
#include <wx/aui/tabmdi.h>
#endif

#include "../ThemesHelper.h"
#include "../SymbolNameSaverBase.h"

#include "../common/types/wxVariantHashArray.h"
#include "../common/types/StringHash.h"

#include "../gaugedesc/GaugeGroupDesc.h"

#include "WidgetFactoryBase.h"

class MTConfigData;


//class FrameLayoutHandler;
class SendObjectBase;

class GaugeCarrierFrame;

class GaugeDescManager;
class GroupMemberObject;
class GaugeCarrierBase;
class GaugeProperties;
class LogHistoryBase;

enum GaugeGroupBehav{
    GGB_SINGLE,
    GGB_MULTI
};

enum GaugeGroupFloatBehav{
    GGFB_FLOAT = 0,
    GGFB_ATTACH = 1,
    GGFB_VAR = 2
};

WX_DEFINE_ARRAY( GaugeCarrierFrame*,  FrameArray );
WX_DECLARE_STRING_HASH_MAP( FrameArray*, FrameArrayHash );
WX_DECLARE_HASH_MAP( GaugeCarrierBase*, int, wxPointerHash, wxPointerEqual, CarrierPtrInt );

WX_DEFINE_ARRAY( GaugeCarrierBase*,  GaugeCarrierBaseArray );
WX_DECLARE_STRING_HASH_MAP( GaugeCarrierBaseArray*, GaugeCarrierBaseArrayHash );



class GaugeGroupManager : public wxEvtHandler, public SymbolNameSaverBase
{
    public:
        GaugeGroupManager( wxWindow* window, wxMenu* parent, MTConfigData* mtConfigData, LogHistoryBase * loghist );
        virtual ~GaugeGroupManager();

        /// virtual:
        virtual bool saveAllGroup(const wxString& filename = wxEmptyString);

        virtual bool load(const wxString& gaugesfile, const wxString& groupsfile) = 0;
        virtual bool saveGroup(const wxString& groupName) = 0;

        virtual void showOnStartup(ShowOnStartupFlags flags = SOSF_OFFLINE_LOG) = 0;
        virtual short showGroup(const wxString& groupName, bool wait = true ) = 0;

        virtual void setStickGroup(const wxString &groupname, bool isStick) = 0;
        virtual bool isStickedGroup(const wxString &groupname) = 0;

        virtual void SaveTheme(int gaugeID, const wxString &name) = 0;
        virtual void SaveDesc(int gaugeID, const wxString &name) = 0;
        virtual void SaveSymbolname(int gaugeID, const wxString &name) = 0;
        virtual void SaveMultiSymbolname(int gaugeID, const wxString &name) = 0;

        virtual void reloadHotkeys() = 0;

        virtual bool IsGroupFloating(const wxString &grpname) = 0;
        virtual wxString GetHotKeyStringByGroupname(const wxString &grpname) = 0;
        virtual void SetGroupFirstFocusedItem(const wxString &grpname, int itemid, int winid) = 0;

        virtual wxString& GetGaugesDBFileName() = 0;
        virtual wxString& GetGroupsDBFileName() = 0;

        virtual void StartTransaction() = 0;
        virtual void EndTransaction() = 0;

        virtual bool SaveGauge(GaugeProperties& prop) = 0;

        virtual GaugeGroupDesc GetGroupInfo(int id) = 0;
        virtual GaugeGroupDesc GetGroupInfo(const wxString& field, const wxVariant& value) = 0;

        virtual bool GetGroupDataByField(wxVariantHashArrayHash& data, const wxString& field, const wxVariant& value) = 0;

        ///
        bool showGroupByHotkey(const wxString &hkey);

        void OnSetItemFocusId( wxCommandEvent& event );
        void OnAddGauge( wxCommandEvent& event );
        void OnSave( wxCommandEvent& event );
        void OnFrameClose( wxCloseEvent& event ); /// used in GroupMember
        void OnRemoveGauge( wxCommandEvent& event );

        void hideAndSaveGroups();
        void restoreGroups();
        short hideGroup(const wxString& groupName, bool removeFromSaved = true );

#if !defined(__WINCE_STANDARDSDK__) && !defined(__POCKETPC__)
        void SetMDIParent( wxMDIParentFrame * frame ) { m_MDIparent = frame; };
        void SetAuiMDIParent( wxAuiMDIParentFrame * frame ) { m_AuiMDIparent = frame; };
#endif
        //void SetFrameLayoutHandler( FrameLayoutHandler * flh ) { m_framelh = flh; };
        void SetSendObj( SendObjectBase* obj ) { m_sendObj = obj; };


        wxMenu* GetMenu() { return m_menu; };

        static GaugeGroupBehav getGaugeGroupsBehav();
        static int getGaugeGroupsFloatBehav();

        void ResetGaugesMinMax(const wxString &groupname);

        void ResetAllGaugesMinMax();




        MTConfigData* getMTConfigData(){ return m_mtConfigData; }

        void OnTheme( wxCommandEvent& event );
        void OnGroup( wxCommandEvent& event );

        GaugeDescManager * GetGaugeDescMgr() { return m_gaugeDescMgr; };
        void SetGaugeDescMgr( GaugeDescManager * mgr ) { m_gaugeDescMgr = mgr; };

        bool SlideFocusToNextTable(wxWindow * mainwin = NULL);

        void SetFocus();

        bool IsGroupChanged() { return m_isGrpChanged; };


        virtual void DoAddGauge( const wxString &groupName, wxString gaugeType, const wxPoint& pos = wxPoint(0,0)) = 0;
        virtual void DoAddGauge( GaugeProperties prop ) = 0;

        virtual bool SaveSymbol(int itemid, const wxString& name) = 0;

        LogHistoryBase* GetLogHistory() { return m_logHistory; }

        bool IsEditMode() { return m_isEditMode; }
        void SetEditMode(bool edit);

        virtual void ProcessWidgetCommand(wxVariantHash& cmdHash);

        virtual int AddGaugeToGroup(GaugeProperties& prop, GaugeCarrierBase* maincarrier, bool isFloating = false, GaugeCarrierBase** vgcarrier = NULL, GaugeCarrierBase** tablecarrier = NULL);

        GaugeCarrierBase* GetGroupCarrier(const wxString& groupName);
        wxString GetGroupName(GaugeCarrierBase* carrier);

        void RemoveGauge(GaugeProperties& props) {
            DoRemoveGauge(props);
        }

        void SetFactory(const wxString& type, WidgetFactoryBase* factory);
        void SetFactory(WidgetFactoryBaseHash& factories);
        void SetFactory(WidgetFactoryBase* factory);

        WidgetFactoryBaseHash& GetFactories() { return m_widgetFactories; }

        int GetFirstFocusedWinID() { return m_firstItemFocus; }

    protected:

        void ProcessWidgetCommandGrp(const wxString& grpname, wxVariantHash& cmdHash);

        virtual void SetTheme(int id) = 0;
        virtual void DoRemoveGauge(GaugeCarrierFrame* gaugeFrame) = 0;
        virtual void DoRemoveGauge(GroupMemberObject * gmo) = 0;
        virtual void DoRemoveGauge(GaugeProperties& props) = 0;

        void OnShowGroup(wxCommandEvent &evt);
        void OnTimer(wxTimerEvent &evt);

        GaugeCarrierBase* addGauge( int id,
                                  const wxString& group,
                                  const wxString& type,
                                  const wxString& name,
                                  const wxPoint& position,
                                  const wxSize& size,
                                  const wxString& theme,
                                  const wxString& desc,
                                  int itemid = -1);


        wxMenu* getThemeMenu() { return m_themeMenu; };
        wxString getGroupTheme( const wxString& grpname );

        MTConfigData*    m_mtConfigData; // only is valid is used in child
        //RTDataMgrBase*   m_rtDataMgrBase; // not used!!
        //STDataMgrBase*   m_stDataMgrBase; // not used!!

        //FrameArrayHash      m_frameGroups; // not used!!

        LogHistoryBase*  m_logHistory;  // used in getter and refreshLastPosition() in SQL;

        GaugeCarrierBaseArrayHash   m_carrierGroups;

#if !defined(__WINCE_STANDARDSDK__) && !defined(__POCKETPC__)
        wxMDIParentFrame *  m_MDIparent;
        wxAuiMDIParentFrame *m_AuiMDIparent;
#endif


        wxMenu*             m_menu;

        /// need to move to ThemeManager
        wxMenu*             m_themeMenu;
        wxString            m_mainThemeName;
        ThemeGroupPairHash  m_themeGrpHash;


        //FrameLayoutHandler * m_framelh;
        SendObjectBase*      m_sendObj;
        wxString             m_lastShowedGroup; // only used in SQL
        GaugeCarrierBase    *m_lastCreatedCarrier;
        bool                 m_isGrpChanged;
        int                  m_shwoGrpEvtID;

        double          m_displayRefRatio;
        int             m_firstItemFocus;
        wxArrayString   m_activeGroupNames;
        CarrierPtrInt   m_carrierToItemId;
        StringHash      m_hotkeyHash;
        WidgetFactoryBaseHash   m_widgetFactories;

        //ThemeAbleArrayHash  m_themedObjects;
    private:
        wxWindow*           m_window;
        GaugeCarrierBase * m_lastFocusedCarrier;
        GaugeDescManager * m_gaugeDescMgr;

        bool        m_isEditMode;
        int         m_timerID;
        wxTimer     m_timer;

};

#endif // GAUGEGROUPMANAGER_H
