#include "GaugeType.h"
#include "../wxSizeEx.h"

Int2wxVariantHashStringHash GaugeType::m_defaults;
Int2StringHash GaugeType::m_gaugeTypes;

void GaugeType::Init()
{
    m_gaugeTypes[ROUND] = _T("VemsGauge");
    m_gaugeTypes[DIGITAL] = _T("Digital");
    m_gaugeTypes[TEXT] = _T("Text");
    m_gaugeTypes[GRAPH] = _T("Graph");
    m_gaugeTypes[FLAG] = _T("Flag");
    m_gaugeTypes[ARCBAR] = _T("ArcBar");
    m_gaugeTypes[TABLE] = _T("table");
    m_gaugeTypes[TABLE3D] = _T("table3d");
    m_gaugeTypes[MULTIGRAPH] = _T("MultiGraph");
    m_gaugeTypes[DIFFER] = _T("Differ");
    m_gaugeTypes[EXPRESSION_GRID] = _T("ExpressionGrid");
    m_gaugeTypes[GPSCOMPASS] = _T("Compass");
    m_gaugeTypes[GPSROUTE] = _T("GPS route");
    m_gaugeTypes[COLSGAUGE] = _T("ColumnsGauge");
}


int GaugeType::getType( const wxString &type )
{

    for( Int2StringHash::iterator it = m_gaugeTypes.begin(); it != m_gaugeTypes.end() ; it++ ) {
        if( type == it->second )
            return it->first;
    }

    return -1;

}

wxString GaugeType::toString( int type )
{
    if( m_gaugeTypes.find(type) == m_gaugeTypes.end() ) return wxEmptyString;

    return m_gaugeTypes[type];
}

wxVariant GaugeType::GetDefault( const wxString &type, int gaugetype )
{
    if( m_defaults.find( type ) == m_defaults.end() ){
        return wxNullVariant;
    }

    Int2wxVariantHash& hash = m_defaults[type];

    if( hash.find( gaugetype ) == hash.end() )
        return hash[-1];
    else
        return hash[gaugetype];

}

void GaugeType::InitDefaults()
{
    Int2wxVariantHash ratios;
    ratios[DIFFER] = 0.25;
    ratios[FLAG] = 0.25;
    ratios[-1] = 1.0;

    Int2wxVariantHash symbols;
    symbols[DIFFER] = _T("lambda");
    symbols[-1] = _T("rpm");
    symbols[EXPRESSION_GRID] = _T("BoostTarget");
    symbols[GPSCOMPASS] = _T("course");
    symbols[GPSROUTE] = _T("latitudeDec");
    symbols[FLAG] = _T("isRunning");

    Int2wxVariantHash descs;
    descs[MULTIGRAPH] = _T("1");
    descs[COLSGAUGE] = _T("1");
    descs[DIFFER] = _T("1");
    descs[-1] = _T("defaultdesc");

    Int2wxVariantHash sizes;
    sizes[MULTIGRAPH] << wxSizeEx(700,550);
    sizes[ARCBAR] << wxSizeEx(300,100);
    sizes[COLSGAUGE] << wxSizeEx(50,400);
    sizes[-1] << wxSizeEx(200,200);


    m_defaults[_T("sideratio")] = ratios;
    m_defaults[_T("symbol")] = symbols;
    m_defaults[_T("descriptor")] = descs;
    m_defaults[_T("size")] = sizes;

}
