#include "MVG_ProfilesEditor.h"

#include "../../gaugedesc/MultiGraphDescManager.h"
#include "../../gaugedesc/GaugeDescManager.h"
#include "../GaugeGroupManager.h"
#include "MVG_DescEditor.h"
#include "MVG_ThemeEditor.h"
#include "../../logger.h"

BEGIN_EVENT_TABLE(MVG_ProfilesEditor, wxFrame)

    EVT_COMBOBOX(idCBDescs,MVG_ProfilesEditor::OnCombo)
    EVT_COMBOBOX(idCBThemes,MVG_ProfilesEditor::OnCombo)

    EVT_BUTTON(idBtnDelDesc , MVG_ProfilesEditor::OnButton)
    EVT_BUTTON(idBtnDelTheme , MVG_ProfilesEditor::OnButton)
    EVT_BUTTON(idBtnOpenDescEditor , MVG_ProfilesEditor::OnButton)
    EVT_BUTTON(idBtnOpenThemeEditor , MVG_ProfilesEditor::OnButton)
    EVT_BUTTON(idBtnSetDescTitle , MVG_ProfilesEditor::OnButton)
    EVT_BUTTON(idBtnSetThemeTitle , MVG_ProfilesEditor::OnButton)


END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(MVG_ProfilesEditor,wxFrame);

MVG_ProfilesEditor::MVG_ProfilesEditor(wxWindow* parent, GaugeDescManager * gdmgr)
{
    //ctor
    m_mvg = NULL;
    m_deldescbtn = NULL;
    m_delthemebtn = NULL;

    Create(parent, wxID_ANY, _T("Multi Graph descriptor and theme profiles editor"));
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    m_gdMgr = gdmgr;

    m_mvgDescMgr = m_gdMgr->GetMultiGraphDescMgr();

    InitEditor();
}

MVG_ProfilesEditor::~MVG_ProfilesEditor()
{
    //dtor
}

void MVG_ProfilesEditor::InitEditor()
{
    wxVariantHashArray arr;

    m_descProfiles = new wxComboBox(this, idCBDescs, _T(""), wxDefaultPosition, wxDefaultSize,0,0,wxCB_READONLY);
    m_themeProfiles = new wxComboBox(this, idCBThemes, _T(""), wxDefaultPosition, wxDefaultSize,0,0,wxCB_READONLY);

    m_descTitleCtrl = new wxTextCtrl(this, wxID_ANY, m_descProfiles->GetStringSelection());
    m_themeTitleCtrl = new wxTextCtrl(this, wxID_ANY, m_themeProfiles->GetStringSelection());

    wxButton * setdesctitlebtn = new wxButton(this, idBtnSetDescTitle, _T("Set"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
    wxButton * setthemetitlebtn = new wxButton(this, idBtnSetThemeTitle, _T("Set"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
    m_deldescbtn = new wxButton(this, idBtnDelDesc, _T("Delete descriptor"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
    m_delthemebtn = new wxButton(this, idBtnDelTheme, _T("Delete theme"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
    wxButton * opendescbtn = new wxButton(this, idBtnOpenDescEditor, _T("Open editor"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
    wxButton * openthemebtn = new wxButton(this, idBtnOpenThemeEditor, _T("Open editor"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);

    wxBoxSizer * mainsizer = new wxBoxSizer(wxHORIZONTAL);

        wxStaticBoxSizer * descSizer = new wxStaticBoxSizer(wxVERTICAL, this, _T("Descriptors"));

            wxStaticBoxSizer * desc_sbs_1 = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("Available profiles"));
            desc_sbs_1->Add( m_descProfiles, 1, wxALL | wxALIGN_CENTER_VERTICAL , 5 );
            desc_sbs_1->Add( opendescbtn, 0, wxALL , 5 );
        descSizer->Add(desc_sbs_1, 0, wxALL | wxEXPAND, 5);

            wxStaticBoxSizer * desc_sbs_2 = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("Set profile title"));
            desc_sbs_2->Add( m_descTitleCtrl, 1, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
            desc_sbs_2->Add( setdesctitlebtn, 0, wxALL, 5 );
        descSizer->Add(desc_sbs_2, 0, wxALL | wxEXPAND, 5);

            wxStaticBoxSizer * desc_sbs_3 = new wxStaticBoxSizer(wxVERTICAL, this, _T("Delete profile"));
            desc_sbs_3->Add( m_deldescbtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );
        descSizer->Add(desc_sbs_3, 0, wxALL | wxEXPAND, 5);

    mainsizer->Add( descSizer, 1, wxALL , 5 );

        wxStaticBoxSizer * themeSizer = new wxStaticBoxSizer(wxVERTICAL, this, _T("Themes"));

            wxStaticBoxSizer * theme_sbs_1 = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("Available profiles"));
            theme_sbs_1->Add( m_themeProfiles, 1, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
            theme_sbs_1->Add( openthemebtn, 0, wxALL , 5 );
        themeSizer->Add(theme_sbs_1, 0, wxALL | wxEXPAND, 5);

            wxStaticBoxSizer * theme_sbs_2 = new wxStaticBoxSizer(wxHORIZONTAL, this, _T("Set profile title"));
            theme_sbs_2->Add( m_themeTitleCtrl, 1, wxALL | wxALIGN_CENTER_VERTICAL, 5 );
            theme_sbs_2->Add( setthemetitlebtn, 0, wxALL, 5 );
        themeSizer->Add(theme_sbs_2, 0, wxALL | wxEXPAND, 5);

            wxStaticBoxSizer * theme_sbs_3 = new wxStaticBoxSizer(wxVERTICAL, this, _T("Delete profile"));
            theme_sbs_3->Add( m_delthemebtn, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );
        themeSizer->Add(theme_sbs_3, 0, wxALL | wxEXPAND, 5);

    mainsizer->Add( themeSizer, 1, wxALL , 5 );


    RefreshDescList(0);
    RefreshThemeList(0);


    SetSizer(mainsizer);
    mainsizer->Fit(this);
    mainsizer->SetSizeHints(this);

    Show(true);
}

void MVG_ProfilesEditor::FillList(wxComboBox* cb, wxVariantHashArray& source, String2IntHash& title2idhash)
{
    int selectedID = title2idhash[ cb->GetStringSelection() ];
    cb->Clear();
    title2idhash.clear();

    wxString stitle = wxEmptyString;

    for(size_t a = 0; a < source.GetCount(); a++ ) {
        wxString title = source[a][_T("title")].GetString();
        int id = source[a][_T("id")].GetLong();

        cb->Append( title );
        title2idhash[title] = id;

        if(id == selectedID) stitle = title;
    }

    if( stitle != wxEmptyString ) cb->SetStringSelection( stitle );
}

bool MVG_ProfilesEditor::DelConfirm()
{
    int res = wxMessageBox(_T("Dou you really want to delete the selected profile?"),_T("Confirm"), wxYES_NO | wxICON_QUESTION);

    return res == wxYES ? true : false;
}

void MVG_ProfilesEditor::DelDesc()
{
    if( !DelConfirm() ) return;

    long descid = m_descs[m_descProfiles->GetStringSelection()];
    wxVariantHashArrayHash data;
    m_gdMgr->GetGaugeGrpMgr()->GetGroupDataByField(data,_T("descriptor"),descid);

    if(data.size() > 0) {
        wxString msg = _T("This descriptor profile are using in next groups:\n");
        for(wxVariantHashArrayHash::iterator it = data.begin(); it != data.end(); it++) {
            msg += it->first + wxString::Format(_T(": %i gauge(s)\n"), it->second.GetCount() );
        }
        msg += _T("\n\nRealy want to delete?");
        if(wxMessageBox(msg,_T("Warning"), wxOK | wxCANCEL | wxICON_QUESTION ) == wxCANCEL) return;
    }

    m_mvgDescMgr->DelMultiGraphDescProfile( descid );

    RefreshDescList(0);
}

void MVG_ProfilesEditor::DelTheme()
{
    if( !DelConfirm() ) return;

    long themeid = m_themes[m_themeProfiles->GetStringSelection()];
    wxVariantHashArrayHash data;
    m_gdMgr->GetGaugeGrpMgr()->GetGroupDataByField(data,_T("gauges.theme"),themeid);

    if(data.size() > 0) {
        wxString msg = _T("This theme profile are using in next groups:\n");
        for(wxVariantHashArrayHash::iterator it = data.begin(); it != data.end(); it++) {
            msg += it->first + wxString::Format(_T(": %i gauge(s)\n"), it->second.GetCount() );
        }
        msg += _T("\n\nRealy want to delete?");
        if(wxMessageBox(msg,_T("Warning"), wxOK | wxCANCEL | wxICON_QUESTION ) == wxCANCEL) return;
    }

    m_mvgDescMgr->DelMultiGraphThemeProfile( themeid );

    RefreshThemeList(0);
}

void MVG_ProfilesEditor::OpenDescEditor()
{
    wxString prof = m_descProfiles->GetStringSelection();

    if( m_descs.find(prof) == m_descs.end() ) return;

    MVG_DescEditor* de = new MVG_DescEditor(this, m_gdMgr, m_descs[prof] );
    de->InitEditor();
    if(m_mvg) de->SetMultiGraph(m_mvg);
    de->Show(1);
}

void MVG_ProfilesEditor::OpenThemeEditor()
{
    wxString prof = m_themeProfiles->GetStringSelection();

    if( m_themes.find(prof) == m_themes.end() ) return;

    MVG_ThemeEditor* te = new MVG_ThemeEditor(this, m_gdMgr, m_themes[prof] );
    if(m_mvg) te->SetMultiGraph(m_mvg);
}

void MVG_ProfilesEditor::RefreshDescList(int selectidx)
{
    wxVariantHashArray arr;
    m_mvgDescMgr->GetMultiGraphDescProfiles(arr);
    FillList(m_descProfiles, arr, m_descs);
    if( selectidx != -1 ) m_descProfiles->SetSelection(selectidx);

    m_descTitleCtrl->SetValue( m_descProfiles->GetStringSelection() );

    if(m_deldescbtn) m_deldescbtn->Enable( arr.GetCount() > 1 ? true : false );
}

void MVG_ProfilesEditor::RefreshThemeList(int selectidx)
{
    wxVariantHashArray arr;
    m_mvgDescMgr->GetMultiGraphThemeProfiles(arr);
    FillList(m_themeProfiles, arr, m_themes);
    if( selectidx != -1 ) m_themeProfiles->SetSelection(selectidx);

    m_themeTitleCtrl->SetValue( m_themeProfiles->GetStringSelection() );

    if(m_delthemebtn) m_delthemebtn->Enable( arr.GetCount() > 1 ? true : false );
}

void MVG_ProfilesEditor::OnButton(wxCommandEvent& evt)
{

    switch( evt.GetId() ){
        case idBtnDelDesc:
            DelDesc();
            break;

        case idBtnDelTheme:
            DelTheme();
            break;

        case idBtnOpenDescEditor:
            OpenDescEditor();
            break;

        case idBtnOpenThemeEditor:
            OpenThemeEditor();
            break;

        case idBtnSetDescTitle:
            SetDescTitle();
            break;

        case idBtnSetThemeTitle:
            SetThemeTitle();
            break;
    }
}

void MVG_ProfilesEditor::SetThemeTitle()
{
    wxString otitle = m_themeProfiles->GetStringSelection();
    wxString ntitle = m_themeTitleCtrl->GetValue();
    if(otitle == ntitle) return;

    int id = m_themes[otitle];

    m_mvgDescMgr->SetMultiGraphThemeProfileTitle(id,ntitle);

    RefreshThemeList();
}

void MVG_ProfilesEditor::SetDescTitle()
{
    wxString otitle = m_descProfiles->GetStringSelection();
    wxString ntitle = m_descTitleCtrl->GetValue();

    if(otitle == ntitle) return;

    int id = m_descs[otitle];

    m_mvgDescMgr->SetMultiGraphDescProfileTitle(id,ntitle);

    RefreshDescList();
}

void MVG_ProfilesEditor::OnCombo(wxCommandEvent& evt)
{
    switch(evt.GetId()){
        case idCBDescs:
            m_descTitleCtrl->SetValue(evt.GetString());
            break;

        case idCBThemes:
            m_themeTitleCtrl->SetValue(evt.GetString());
            break;
    }
}
