#include "TableEditorFactory.h"

#include "GaugeCarrierBaseDesc.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../vemsdialog/StaticControlPanel.h"
#include "GroupMember.h"
#include "TableEditorPanel.h"
#include "../vemsdialog/TableEditor.h"
#include "../logger.h"

IMPLEMENT_DYNAMIC_CLASS(TableEditorFactory, WidgetFactoryBase);


TableEditorFactory::TableEditorFactory()
{
    //ctor
}

TableEditorFactory::~TableEditorFactory()
{
    //dtor
}

VemsControlBase* TableEditorFactory::CreateWidget(GaugeProperties &prop, GaugeCarrierBaseDesc* desc)
{

    if(m_rtController == NULL) return NULL;
    if(m_stController == NULL) return NULL;


    Logger::logf(LOG_MSG,_T("prop.width=%i"),prop.width);

    int id = wxNewId();

    prop.winID = id;

    wxWindow * twin = desc->m_window;

    TableEditorPanel * panel = NULL;

    wxSize orisize = prop.getSize();

    if( !desc->m_onlyOneGauge )
    {
        //twin = new wxPanel( desc->m_window, wxID_ANY, prop.getPos(), wxDefaultSize, 0 );
        panel = new TableEditorPanel( desc->m_window, id ,prop.getPos());
        panel->SetGaugeGroupMgr( m_groupMgr );
        twin = panel;
        prop.setPos(0,0);
    }

    prop.setSize(-1,-1);

    TableEditor* table = new TableEditor( twin , m_gdMgr, prop, !desc->m_onlyOneGauge );

    if(panel) table->SetFocusParentSinker(panel);

    table->init( prop.symbolName, m_stController, m_rtController, m_mtconfigdata );

    StaticControlPanel* ctrlPanel = NULL;

    if( prop.extraParams[_T("hasControlPanel")].GetBool() )
    {
        int buttons = StaticControlPanel::DEFAULT_BUTTONS | StaticControlPanel::SHOW_REMAP_BUTTON;
        if ( !table->is3D() ) { buttons = StaticControlPanel::REMAPPER_TABLE_BUTTONS; }

        ctrlPanel = new StaticControlPanel( twin, table, buttons );
        ctrlPanel->update();
    }

    if(desc->m_frame) desc->m_frame->SetTitle( table->GetLabel() );
    if(panel) panel->SetTitle( table->GetLabel() );




    if( desc->m_onlyOneGauge )
    {

        twin->SetInitialSize( wxSize(300,300) );
        twin->SetSize( orisize );


        wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );
        sizer->Add( table, 1, wxEXPAND );
        if(ctrlPanel) sizer->Add( ctrlPanel, 0, wxEXPAND );
        twin->SetSizer( sizer );



        table->SetFocus();


//        Logger::logf(LOG_MSG,_T(" GaugeCarrierBase::AddVemsTable() s=%i,s=%i"),m_table->GetSize().x,m_table->GetSize().y);

    }
    else
    {
        //table->SetPosition(  );

        table->SetSize( prop.getSize() );

        //table->SetSize( wxDefaultSize );
    }

    desc->m_IDtoControls[id] = table;

//    if(desc->m_CustomControlHash != NULL) (*desc->m_CustomControlHash)[id] = table;

    if(panel) {

        desc->m_IDtoControls[id] = panel;

//        Logger::logf(LOG_MSG,_T("panel: desc->m_IDtoControls[%i]=%p"),id,panel);

//        if(desc->m_CustomControlHash != NULL) (*desc->m_CustomControlHash)[id] = panel;

        //panel->SetTableEditor( table );
        panel->SetTable( table );
        panel->SetGroupMember( table->GetGrpMember() );

        if(ctrlPanel) panel->AddElement( ctrlPanel );

        panel->GetMainSizer()->SetMinSize( orisize );
        panel->GetMainSizer()->Layout();
        panel->GetMainSizer()->Fit( panel );

    } else {

        desc->m_IDtoControls[id] = table;
//        if(desc->m_CustomControlHash != NULL) (*desc->m_CustomControlHash)[id] = table;
    }


    table->GetGrpMember()->SetItemID( prop.itemID );

    return table;
}
