#ifndef TABLEEDITORPANEL_H
#define TABLEEDITORPANEL_H

#include "SelectableWidgetBase.h"

#include "FocusParentSinkerBase.h"

class wxStaticTextEx;
class WindowTipper;
class GaugeGroupManager;
class GroupMember;


class TitleBarControl: public wxPanel
{
    public:
        TitleBarControl(wxWindow * parent, int id = wxID_ANY, const wxString& title = wxEmptyString);

        void SetText(const wxString &text);
        wxString GetText();

        void SetActive(bool isactive);

        void SetGaugeGroupMgr(GaugeGroupManager * mgr) { m_ggmgr = mgr; };

    private:

        void OnTextMouse(wxMouseEvent& evt);

		void OnLeftDown(wxMouseEvent& evt);
		void OnMouseMove(wxMouseEvent& evt);
		void OnLeftUp(wxMouseEvent& evt);

        void OutFocus(wxFocusEvent &evt);
        void OnFocus(wxFocusEvent &evt);

        wxStaticText *m_text;

        GaugeGroupManager * m_ggmgr;

        WindowTipper *  m_winTip;

        wxPoint         m_delta;

        DECLARE_EVENT_TABLE();
};

class TableEditorPanel: public wxPanel, public SelectableWidgetBase, public FocusParentSinkerBase
{
    public:
        DECLARE_DYNAMIC_CLASS(TableEditorPanel);
        TableEditorPanel() {};
        TableEditorPanel(wxWindow * parent, int id = wxID_ANY, const wxPoint &pos = wxDefaultPosition);
        virtual ~TableEditorPanel();

        void SetTitle(const wxString &text) { m_titleBar->SetText(text); };
        wxString GetTitle() { return m_titleBar->GetText(); };

        wxSizerItem * AddElement( wxWindow * win );
        wxBoxSizer * GetMainSizer() { return m_mainSizer; };

        wxSizerItem * SetTable( wxWindow* win );
        wxWindow* GetTable() { return m_tableEditor; };

        void SetGroupMember(GroupMember* gm) { m_grpMember = gm; }
        GroupMember* GetGroupMember() { return m_grpMember; }

        void SetGaugeGroupMgr(GaugeGroupManager * mgr) { m_titleBar->SetGaugeGroupMgr(mgr); };

        virtual void ProcessWidgetCommand(wxVariantHash& cmdHash);

        virtual void OnFocusMsg(int type);

    protected:

        void OutFocus(wxFocusEvent &evt);
        void OnFocus(wxFocusEvent &evt);
        //void OnChildFocus(wxChildFocusEvent &evt);

    private:

        //TableEditor     *   m_tableEditor;
        wxWindow*           m_tableEditor;
        GroupMember*        m_grpMember;

        TitleBarControl *   m_titleBar;
        wxBoxSizer      *   m_mainSizer;

        DECLARE_EVENT_TABLE();

};

#endif // TABLEEDITORPANEL_H
