#ifndef VEMSEXPRESSIONGRIDWIDGET_H
#define VEMSEXPRESSIONGRIDWIDGET_H

#include "VemsWidgetBase.h"
#include "../common/types/wxPointArray.h"
#include "../common/types/StringSetArray.h"
#include "../common/types/IntArray.h"
#include "../common/types/Int2StringHash.h"
#include "../common/types/wxBitmapArray.h"
#include "../vemsgauge/ExpressionGridDesc.h"
#include "../stdatamanager/STDataViewerBase.h"

#include "boost/multi_array.hpp"

class VemsExpressionGridWidget;
class WindowTipper;

class GridCell
{
    public:
        wxRect      rect;
        wxString    text;
        wxString    originText;
        int         width;
        int         precision;
        wxAlignment align;
        StringSet   variables;
        ExpressionCellLinkArray 	links;
        ExprCellTextArray           texts;
        ExprCellColorArray          colors;
        wxColor                     bgColor;
        wxColor                     fgColor;
        bool        disabled;
        bool        linkEnabled;
        wxString    activeLink;
        wxString    help;
        StringSet   dlgSymbols;

    public:
        wxString getDefaultText() const;
        bool isLink() const { return !links.IsEmpty(); }
        bool isVariable() const { return !variables.empty(); }
        wxSize getMinSize( const VemsExpressionGridWidget& parent, const wxFont& cellFont ) const;

    private:
       //WindowTipper*           m_winTip;
};

/**
class GridCell
{
    public:
        GridCell( const wxRect& rect )
        {
            m_rect = rect;
        }

        virtual ~GridCell() {}

        virtual void draw( wxDC& dc, WidgetThemeTable* theme ) = 0;
        virtual void onMouseLeft() = 0;

    private:
        wxRect  m_rect;
        bool    m_enabled;
};

class TextCell : public GridCell
{

}

class VariableCell : public GridCell
{
    VariableCell()

    private:
        wxAlignment m_align;
        StringSet   m_variables;
        int         m_precision;

}

class LinkCell : public GridCell
{
    LinkCell()

    private:
        wxString    m_link;
        wxString    m_linkEnableExpr;
        bool        m_linkEnabled;
}

class HelpCell : public GridCell
{

}

*/
/*
DirHelper::GetPath(dICON) + _T("ul-down-xl.png");
wxImage(const wxString& name, long type = wxBITMAP_TYPE_ANY, int index = -1)

class GridCell
{
    public:
        void setRect( wxRect rect ) { m_rect = rect; }

        void setBackgroundColor( wxColor color );
        void setForegroundColor( wxColor color );
        void setDisabledBackgroundColor( wxColor color );
        void setDisabledForegroundColor( wxColor color );



        virtual void draw( wxDC& dc ) const = 0;
        virtual void getMinSize() const = 0;
        virtual void onMouseMove( wxWindow* parent ) {};

    private:
        wxRect      m_rect;
        wxColor     m_backgroundColor;
        wxColor     m_foregroundColor;
        wxColor     m_

};

class TextCell : public GridCell
{
    void setText( const wxString& text );
    wxString void getText( const wxString& text );
};

class VariableCell : public TextCell
{
    public:

};

class ImageCell : public GridCell
{

};
*/

typedef boost::multi_array<GridCell, 2> GridCellMatrix;

class VemsExpressionGridWidget : public VemsWidgetBase, public STDataViewerBase
{
    public:
        VemsExpressionGridWidget(   wxWindow* parent,
                                    wxWindowID id,
                                    GaugeDescManager* gdmgr,
                                    GaugeProperties& prop );
        virtual ~VemsExpressionGridWidget();

        wxRegion GetShape();
        wxRegion GetBorderRegion();
        void UpdateSize() {}
        void setSize(int x , int y) {}

        void SetValue(double value) {}

        void refresh( DoubleHash* table );
        void require();
        void unregisterSymbols();
        void SetNewTheme( WidgetThemeTable *theme );

        wxFont getCellsFont() const { return m_cellFont; }

        /// STDataViewerBase methods
        void update();
        void requestRead() {}
        void updateModel() {}
        void requestWrite() {}

    protected:
        void createWidgetMenu( wxMenu& popupMenu );

    private:
        GridCellMatrix          m_cells;
        size_t                  m_cols;
        size_t                  m_rows;
        int                     m_disableCol;
        bool                    m_isSymbolsRegistered;

        wxBitmap*               m_backbuffer;

        IntArray                m_colsGridPos;
        IntArray                m_rowsGridPos;
        IntArray                m_linesEnabled;

        bool                    m_isResized;
        Int2StringHash          m_menuIDtoGridName;
        size_t                  m_titleHeight;

        IntArray                m_colMaxTextWidth;
        IntArray                m_colMaxTextRow;
        IntArray                m_colAvailableTextWidth;

        IntArray                m_rowHeight;
        IntArray                m_rowAvailableTextHeight;


        wxSize                  m_preferredSize;
        wxSize                  m_preferredTextSize;
        wxSize                  m_decorationsSize;
        wxSize                  m_minTextSize;
        wxSize                  m_minClientSize;
        double                  m_sideRatio;

        int                     m_textHeight;


        ///
        ExpressionGridDesc*     m_gridDescriptor;
        StringSetArray          m_variableSetA;
        wxPointArray            m_variablePointA;
        StringSet               m_rtVariables;

        ///decorations
        wxFont                  m_cellFont;
        wxFont                  m_titleFont;
        size_t                  m_borderSize;
        size_t                  m_fontPointSize;
        size_t                  m_cellBorderSize;
        size_t                  m_gridSize;

        ///icons
        wxBitmapArray           m_helpIcons;
        WindowTipper*           m_winTip;

        void initIcons();
        void createGrid();
        void createDefaultGrid();
        void createGridFromDesc();

        void clear();

        void drawBorder( wxDC& dc );
        void drawTitle( wxDC& dc );
        void drawGrids( wxDC& dc );
        void drawTable( wxDC& dc );
        void drawVariableCellsOnly( wxDC& dc );

        void setCellText( size_t row, size_t col, const wxString& text );

        void calculateFontSize();


        wxString getMaxTextSum();
        wxSize getAvailableTextSize();
        int getColumnMaxTextRowIdx( size_t col );


        void calculatePreferredSize();
        void calculatePreferredGridPosition();
        void calculatePreferredCellRects();

        void calculatePreferredTextSize();
        wxPoint getCellPosition( wxPoint mousePosition );
        void showLink( wxPoint cellPosition );

        void calculateDecorationSize();
        void findColsLongestText();
        void calculateMinSize();
        void calculateSize( bool autoSize = false );
        void caluclateGridsPositions();
        void setDisabledRows();

        wxBitmap* getHelpBmp( const wxRect& rect );
        bool setLinksEnabled();
        void setActiveLink( GridCell& cell );

        bool setActiveTexts();
        void setActiveText( GridCell& cell );

        void setActiveColor( GridCell& cell );
        void getColorFromStr( const wxString& colorStr, wxColor& color );

        void OnPaint( wxPaintEvent &WXUNUSED(event) );
        void OnSize( wxSizeEvent &event );
        void OnPopupMenuChangeGrid( wxCommandEvent& event );
        void OnLeftDown( wxMouseEvent& event );
        void OnMouseMove(wxMouseEvent& evt);
        DECLARE_EVENT_TABLE()
};

#endif // EXPRESSIONGRIDWIDGET_H
