#include "VemsGaugeAgg.h"

#include <math.h>

#include <wx/dcbuffer.h>
#include <wx/listimpl.cpp>

#include "../DrawHelper.h"
#include "../WidgetThemeTable.h"

WX_DEFINE_LIST( PointList );

BEGIN_EVENT_TABLE( VemsGaugeAgg, VemsWidgetBase )
    EVT_PAINT( VemsGaugeAgg::OnPaint )
    EVT_SIZE ( VemsGaugeAgg::OnSize )
END_EVENT_TABLE()


VemsGaugeAgg::VemsGaugeAgg()
{
    m_overResValue = 0.0;
    m_isDataValid = false;
    m_firstSetValue = true;

    initMinMax();
    initTmpGaugePointer();
}


VemsGaugeAgg::VemsGaugeAgg( wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop )
    : VemsWidgetBase( parent, id, gdmgr, prop )
{
	SetBackgroundStyle(wxBG_STYLE_CUSTOM);
    m_gradientAngle         = 135.0;

    m_gaugeOuterRadius      = 0.0;
    m_gaugeInnerRadius      = 0.0;
    m_gaugeMiddleRadius     = 0.0;
    m_gaugeCoverOuterRadius = 0.0;
    m_gaugeCoverInnerRadius = 0.0;
    m_gaugeCoverX           = 0.0;
    m_gaugeCoverY           = 0.0;

    m_tickInnerRadius       = 0.0;
	m_tickOuterRadius       = 0.0;
	m_tickTextRadius        = 0.0;
	m_tickTextHeight        = 0.0;

	m_unitTextHeight        = 0.0;
	m_unitTextY             = 0.0;

    m_valueTextHeight       = 0.0;
    m_valueTextY            = 0.0;

    m_titleTextHeight       = 0.0;
    m_titleTextY            = 0.0;

    m_isResized     = true;
    m_gaugeBMP      = NULL;
    m_gaugeCoverBMP = NULL;

    SetBackgroundStyle( wxBG_STYLE_CUSTOM );

    m_isExtendedResize = false;

    m_overResValue = 0.0;
    m_isDataValid = false;

    m_isGaugeDescMenu = true;
    m_firstSetValue = true;

    initMinMax();
    initTmpGaugePointer();

    m_sideRatioRestriction = 1.0;
}


VemsGaugeAgg::~VemsGaugeAgg()
{
    delete m_gaugeBMP;
    delete m_gaugeCoverBMP;
}

void VemsGaugeAgg::initMinMax()
{
    m_valueMax = 0.0;
    m_valueMin = 0.0;
    m_isValueMaxChanged = true;
    m_isValueMinChanged = true;
}

void VemsGaugeAgg::initTmpGaugePointer()
{
    m_tmpPointerCordArray.Add( wxRealPoint(0.0, 0.0) );
    m_tmpPointerCordArray.Add( wxRealPoint(0.0, 0.0) );
    m_tmpPointerCordArray.Add( wxRealPoint(0.0, 0.0) );
}


void VemsGaugeAgg::drawGaugeFrame( wxAgg& agg )
{
    //draw outer circle
    agg.drawGradientFilledCircle( m_theme->BorderBeginColor , m_theme->BorderEndColor, m_gradientAngle,
                                  m_centerPoint.x, m_centerPoint.y, m_gaugeOuterRadius );

    //draw inner circle
    agg.drawGradientFilledCircle( m_theme->BorderEndColor, m_theme->BorderBeginColor, m_gradientAngle,
                                  m_centerPoint.x, m_centerPoint.y, m_gaugeMiddleRadius );

    //draw cover circle
    agg.setBrush( m_theme->BackgroundColor );
    agg.drawFilledCircle( m_centerPoint.x, m_centerPoint.y, m_gaugeInnerRadius );
}

void VemsGaugeAgg::drawGaugeFrame( wxDC& dc )
{
	dc.SetBrush(wxBrush(m_theme->BorderBeginColor,wxTRANSPARENT));
	dc.SetPen(wxPen(m_theme->BorderBeginColor,(int)(m_gaugeOuterRadius-m_gaugeInnerRadius),wxSOLID));
	dc.DrawCircle(m_centerPoint, (int)m_gaugeMiddleRadius);
	//dc.SetBrush(*wxTRANSPARENT_BRUSH);
	//dc.DrawCircle(m_centerPoint, m_gaugeInnerRadius);
	dc.SetBrush(m_theme->BorderBeginColor);
	dc.DrawCircle(m_centerPoint, (int)m_gaugeCoverOuterRadius);
}

void VemsGaugeAgg::drawColoredArea( wxAgg& agg )
{

    double w = m_tickOuterRadius - m_tickInnerRadius;
    double r = m_tickInnerRadius + w / 2;

    double maxV = m_gd->GetMaxValue();
    double minV = m_gd->GetMinValue();

    double gaugeStartAngle  = (double)m_gd->GetStartAngleRad();
    double gaugeAngle       = (double)m_gd->GetGaugeAngleRad();
    double anglePerRange    = gaugeAngle / ( maxV - minV );
    double offsetAngle      = anglePerRange * minV;

    for(uint16_t i= 0; i <  m_gd->coloredA.Count(); i++)
    {
        ColoredArea * ca = m_gd->coloredA.Item(i);

        if( m_theme->ColorSymbols.find(ca->ColorName) == m_theme->ColorSymbols.end() ) continue;

        if( ca->minValue >  ca->maxValue) continue;
        if( ca->minValue < minV) continue;
        if( ca->maxValue > maxV) continue;
        if( ca->minValue == ca->maxValue) continue;

        double startAngle = gaugeStartAngle + anglePerRange * ca->minValue - offsetAngle;
        double deltaAngle = anglePerRange * ( ca->maxValue - ca->minValue );


        agg.setPen( m_theme->ColorSymbols[ ca->ColorName ] );
        agg.drawArc( m_centerPoint.x, m_centerPoint.y, r, startAngle, deltaAngle, w );
    }
}


void VemsGaugeAgg::drawGaugeCover( wxAgg& agg )
{
    //draw outer circle
    agg.drawGradientFilledCircle( m_theme->BorderBeginColor, m_theme->BorderEndColor, m_gradientAngle,
                                  m_gaugeCoverOuterRadius, m_gaugeCoverOuterRadius, m_gaugeCoverOuterRadius );

    //draw cover circle
    agg.setBrush( wxAgg::gradient(  m_theme->BorderBeginColor, m_theme->BorderEndColor,  0.5 ) );
    agg.drawFilledCircle( m_gaugeCoverOuterRadius, m_gaugeCoverOuterRadius, m_gaugeCoverInnerRadius );
}

void VemsGaugeAgg::drawTickMarks( wxAgg& agg )
{
    int minorDivision       = m_gd->GetMinorDivisions();
    int totalDivision       = m_gd->GetTotalDivisions();
    double tickStartAngle   = m_gd->GetStartAngleRad();
    double gaugeAngle       = m_gd->GetGaugeAngleRad();
    double incrementAngle   = gaugeAngle / totalDivision;

    double x1 = 0.0;
    double y1 = 0.0;
    double x2 = 0.0;
    double y2 = 0.0;
    double angle = 0.0;

    for( uint16_t i = 0; i < totalDivision + 1; ++i )
    {
        angle = tickStartAngle + i * incrementAngle;

        x1 = m_centerPoint.x + ( m_tickOuterRadius ) * cos( angle );
        y1 = m_centerPoint.y + ( m_tickOuterRadius ) * sin( angle );

        x2 = m_centerPoint.x + ( m_tickInnerRadius ) * cos( angle );
        y2 = m_centerPoint.y + ( m_tickInnerRadius ) * sin( angle );

        //agg.setPen( *wxWHITE );
        agg.setPen( m_theme->DivColor );

        if( i % minorDivision )
        {
            agg.drawLine( x1, y1, x2, y2 );
        }
        else
        {
            agg.drawLine( x1, y1, x2, y2, 3.0 );
        }
    }
}

void VemsGaugeAgg::drawTickMarks( wxDC& dc )
{
    int minorDivision       = m_gd->GetMinorDivisions();
    int totalDivision       = m_gd->GetTotalDivisions();
    double tickStartAngle   = m_gd->GetStartAngleRad();
    double gaugeAngle       = m_gd->GetGaugeAngleRad();
    double incrementAngle   = gaugeAngle / totalDivision;

    double x1 = 0.0;
    double y1 = 0.0;
    double x2 = 0.0;
    double y2 = 0.0;
    double angle = 0.0;

    for( uint16_t i = 0; i < totalDivision + 1; ++i )
    {
        angle = tickStartAngle + i * incrementAngle;

        x1 = m_centerPoint.x + ( m_tickOuterRadius ) * cos( angle );
        y1 = m_centerPoint.y + ( m_tickOuterRadius ) * sin( angle );

        x2 = m_centerPoint.x + ( m_tickInnerRadius ) * cos( angle );
        y2 = m_centerPoint.y + ( m_tickInnerRadius ) * sin( angle );

        //agg.setPen( *wxWHITE );
		dc.SetPen( m_theme->DivColor );

        if( i % minorDivision )
        {
            dc.DrawLine( (int)x1, (int)y1, (int)x2, (int)y2 );
        }
        else
        {
            dc.DrawLine( (int)x1, (int)y1, (int)x2, (int)y2 );
        }
    }
}

void VemsGaugeAgg::drawUnitText( wxDC& dc )
{
    if( m_unitTextHeight == 0 ) return;

    wxString text = m_gd->GetUnit();

    if( m_gd->GetUnitsScaleFactor() != 1 )
    {
        text += wxString::Format( _T(" x %i"), m_gd->GetUnitsScaleFactor() );
    }

    wxFont font = dc.GetFont();
    font.SetPointSize( (int)ceil( m_unitTextHeight ) );
    dc.SetFont( font );

    wxSize size = dc.GetTextExtent( text );


    double dangle  = m_gd->GetUnitAngle() * M_PI / 180.0;
    double pos = m_gd->GetUnitPos() / 100.0;

    int x = m_centerPoint.x + (int)((m_gaugeInnerRadius * sin(dangle) * pos) - size.x/2);
    int y = m_centerPoint.y + (int)((m_gaugeInnerRadius * cos(dangle) * pos) - size.y/2);

    dc.SetTextForeground( m_theme->SuffixColor );
    dc.DrawText( text, x, y );
    dc.SetFont( m_font );
}


void VemsGaugeAgg::drawValueText( wxDC& dc )
{
    if( m_valueTextHeight == 0 ) return;

    wxString text = getValueString();

    dc.SetTextForeground( m_tickTextColor );

    wxFont font = dc.GetFont();
    font.SetPointSize( (int)m_valueTextHeight );
    dc.SetFont( font );

    wxSize size = dc.GetTextExtent( text );

    double dangle  = m_gd->GetValAngle() * M_PI / 180.0;
    double pos = m_gd->GetValPos() / 100.0;

    int x = m_centerPoint.x + (int)((m_gaugeInnerRadius * sin(dangle) * pos) - size.x/2);
    int y = m_centerPoint.y + (int)((m_gaugeInnerRadius * cos(dangle) * pos) - size.y/2);


    dc.SetTextForeground( m_theme->ValueColor );
    dc.DrawText( text, x, y );
    dc.SetFont( m_font );
}


void VemsGaugeAgg::drawTitleText( wxDC& dc )
{
    wxString text = GetSymbolTitle();

    dc.SetTextForeground( m_titleTextColor );

    wxFont font = dc.GetFont();
    font.SetPointSize( (int)m_titleTextHeight );
    dc.SetFont( font );

    wxSize size = dc.GetTextExtent( text );

    int y = (int)( m_titleTextY - size.GetHeight() / 2 );
    int x = (int)( m_centerPoint.x - size.GetWidth() / 2.0 );

    dc.SetTextForeground( m_theme->TitleColor );
    dc.DrawText( text, x, y );
    dc.SetFont( m_font );
}


void VemsGaugeAgg::drawMultiLineTitleText( wxDC &dc )
{
    if( m_fontSize == 0 ) return;

    dc.SetTextForeground( m_valueTextColor );

    double textRadius  = m_gaugeInnerRadius * 0.70;

    wxSize ws = dc.GetSize();
    ws.Scale( 0.5, 0.5 );

    wxFont tmpFont = dc.GetFont();

    tmpFont.SetPointSize( (int)ceil( m_fontSize * 1.5 ) );
    dc.SetFont( tmpFont );

    wxString gTitle = GetSymbolTitle();

    int angle  = m_gd->GetTitleAngle();
    double pos = m_gd->GetTitlePos() / 100.0;

    dc.SetPen( m_theme->TitleColor );

    if( m_width > 1300 )
    {
        DrawHelper::DrawArcText( &dc, gTitle, wxPoint( ws.x, ws.y ), textRadius * 1.1, angle, 360 - m_gd->GetGaugeAngle()
                                 , (short)ceil( m_fontSize * 1.7 ) );
    }
    else
    {
        gTitle.Replace(_T(" "), _T("\n"),false);

        wxSize tSize = dc.GetMultiLineTextExtent( gTitle );

        wxPoint titlePos;

        double dangle = angle * M_PI / 180.0;

        titlePos.x = m_centerPoint.x + (int)((m_gaugeInnerRadius * sin(dangle) * pos) - tSize.x/2);
        titlePos.y = m_centerPoint.y + (int)((m_gaugeInnerRadius * cos(dangle) * pos) - tSize.y/2);

		DrawHelper::DrawMultiLineText( &dc, gTitle, titlePos, wxALIGN_CENTER );
    }

    dc.SetFont( m_font );
}

void VemsGaugeAgg::calculatePointerCord(double value, RealPointArray& gaugePointerArray, double pointerRadius)
{
    if(value > m_gd->GetMaxValue() ){
        value = m_gd->GetMaxValue();
    }

    if(value < m_gd->GetMinValue() ){
        value = m_gd->GetMinValue();
    }

    double tickStartAngle   = m_gd->GetStartAngleRad();
    double gaugeAngle       = m_gd->GetGaugeAngleRad();
    double minValue         = m_gd->GetMinValue();
    double incrementAngle   = gaugeAngle /  ( m_gd->GetMaxValue() - minValue );

    double angle = tickStartAngle + ( value - minValue ) * incrementAngle;

    gaugePointerArray.Item(0).x = m_centerPoint.x + ( m_tickOuterRadius ) * cos( angle );
    gaugePointerArray.Item(0).y = m_centerPoint.y + ( m_tickOuterRadius ) * sin( angle );

    gaugePointerArray.Item(1).x = m_centerPoint.x + pointerRadius * cos( angle + M_PI / 2 );
    gaugePointerArray.Item(1).y = m_centerPoint.y + pointerRadius * sin( angle + M_PI / 2 );

    gaugePointerArray.Item(2).x = m_centerPoint.x + pointerRadius * cos( angle - M_PI / 2 );
    gaugePointerArray.Item(2).y = m_centerPoint.y + pointerRadius * sin( angle - M_PI / 2 );
}


void VemsGaugeAgg::drawPointer( wxAgg& agg, double value, wxColor pointerColor, double pointerRadius)
{
    calculatePointerCord(value, m_tmpPointerCordArray, pointerRadius);
    agg.setBrush( pointerColor );
    agg.drawPolygon( m_tmpPointerCordArray );
}


void VemsGaugeAgg::drawPointer( wxDC& dc, double value, wxColor pointerColor, double pointerRadius )
{
    calculatePointerCord(value, m_tmpPointerCordArray, pointerRadius);

    double x1 = m_tmpPointerCordArray.Item(0).x;
    double y1 = m_tmpPointerCordArray.Item(0).y;

    double x2 = m_tmpPointerCordArray.Item(1).x;
    double y2 = m_tmpPointerCordArray.Item(1).y;

    double x3 = m_tmpPointerCordArray.Item(2).x;
    double y3 = m_tmpPointerCordArray.Item(2).y;

    wxPoint points[ 3 ];
    points[ 0 ] = wxPoint( (int)x1, (int)y1 );
    points[ 1 ] = wxPoint( (int)x2, (int)y2 );
    points[ 2 ] = wxPoint( (int)x3, (int)y3 );

    dc.SetBrush( pointerColor );
    dc.SetPen( pointerColor );
    dc.DrawPolygon( 3, points );
}


void VemsGaugeAgg::drawGaugeNumbers( wxDC& dc )
{
   //Logger::logf( LOG_MSG, _T("VemsGaugeAgg::drawGaugeNumbers() dc ") );
    if( !m_gd )
    {
        Logger::logf( LOG_MSG, _T("VemsGaugeAgg::drawGaugeNumbers() - m_gd = NULL") );
        return;
    }

    if( m_tickTextHeight <= 1 ) return;

    int majorDivision       = m_gd->GetMajorDivisions();
    double tickStartAngle   = m_gd->GetStartAngleRad();
    double gaugeAngle       = m_gd->GetGaugeAngleRad();
    double incrementAngle   = gaugeAngle / majorDivision;

    int x = 0;
    int y = 0;
    double angle = 0.0;
    wxString text = wxEmptyString;

    for( uint16_t i = 0; i < m_gd->m_gaugeNumbers.Count(); ++i )
    {
        text = m_gd->m_gaugeNumbers.Item( i );
        //Logger::logf( LOG_MSG, _T("VemsGaugeAgg::drawGaugeNumbers() text= ")+text );
        angle = tickStartAngle + i * incrementAngle;

        x = (int)( m_centerPoint.x + ( m_tickTextRadius ) * cos( angle ) );
        y = (int)( m_centerPoint.y + ( m_tickTextRadius ) * sin( angle ) );

        dc.SetTextForeground( m_theme->DivTextColor );

        wxFont font = dc.GetFont();
        font.SetPointSize( (int)m_tickTextHeight );
        dc.SetFont( font );

        wxSize size = dc.GetTextExtent( text );

        x -= size.GetWidth() / 2;
        y -= size.GetHeight() / 2;

        //dc.SetTextForeground( m_unitTextColor );
        dc.DrawText( text, x, y );
    }
}


void VemsGaugeAgg::calculateSize()
{

    if( !m_gd ) {
       // UserLog::uLog( ULOG_ERROR, _T("The selected descriptor not found!</font><br>The gauge is broken.") );
        return;
    }



    double GAUGE_INNER_RADIUS_RATIO   = 1 - m_theme->BorderWidthRatio;   //0.90;
    double TICK_OUTER_RADIUS_RATIO    = GAUGE_INNER_RADIUS_RATIO * 0.95; //0.85;
    double TICK_INNER_RADIUS_RATIO    = GAUGE_INNER_RADIUS_RATIO * 0.83; //0.75;


    // double TICK_TEXT_RADIUS_RATIO     = 0.65;
    // double TICK_TEXT_HEIGHT_RATIO     = 0.08;
    double COVER_OUTER_RADIUS_RATIO   = 0.15;//0.20;
    double COVER_INNER_RADIUS_RATIO   = 0.13;//0.18;
    // double UNIT_TEXT_HEIGHT_RATIO     = 0.10;
    double UNIT_TEXT_Y_RATIO          = 0.50;
    //double VALUE_TEXT_HEIGHT_RATIO    = 0.10;
    double VALUE_TEXT_Y_RATIO         = 0.30;
    // double TITLE_TEXT_HEIGHT_RATIO    = 0.08;
    double TITLE_TEXT_Y_RATIO         = 0.40;
    // double TEXT_HEIGHT_RATIO          = 0.09;

    GetClientSize( &m_width, &m_height );
    if(m_width>m_height)
    {
        m_width = m_height;
    }
    if(m_height>m_width)
    {
        m_height = m_width;
    }

    m_centerPoint.x = m_width / 2;
    m_centerPoint.y = m_height / 2;

    m_gaugeOuterRadius  = m_width < m_height ? m_width / 2 : m_height / 2;
    m_gaugeOuterRadius  -= 2;
    m_gaugeInnerRadius  = m_gaugeOuterRadius * GAUGE_INNER_RADIUS_RATIO;
    m_gaugeMiddleRadius = m_gaugeInnerRadius + ( m_gaugeOuterRadius - m_gaugeInnerRadius ) / 2;

    m_tickOuterRadius   = m_gaugeOuterRadius * TICK_OUTER_RADIUS_RATIO;
    m_tickInnerRadius   = m_gaugeOuterRadius * TICK_INNER_RADIUS_RATIO;

    m_tickTextRadius    = m_gaugeOuterRadius * m_gd->Properties[_T("TickTextRadius")].GetDouble();
    m_tickTextHeight    = m_gaugeOuterRadius * m_gd->Properties[_T("TickTextHeight")].GetDouble();

    m_gaugeCoverOuterRadius = m_gaugeOuterRadius * COVER_OUTER_RADIUS_RATIO;
    m_gaugeCoverInnerRadius = m_gaugeOuterRadius * COVER_INNER_RADIUS_RATIO;
    m_gaugeCoverX           = m_centerPoint.x - m_gaugeCoverOuterRadius;
    m_gaugeCoverY           = m_centerPoint.y - m_gaugeCoverOuterRadius;

    m_unitTextHeight    = m_gaugeOuterRadius * m_gd->Properties[_T("UnitTextHeight")].GetDouble();
    m_unitTextY         = m_centerPoint.y - m_gaugeOuterRadius * UNIT_TEXT_Y_RATIO;

    m_valueTextHeight   = m_gaugeOuterRadius * m_gd->Properties[_T("ValueTextHeight")].GetDouble();
    m_valueTextY        = m_centerPoint.y + m_gaugeOuterRadius * VALUE_TEXT_Y_RATIO;

    m_titleTextHeight   = m_gaugeOuterRadius * m_gd->Properties[_T("TitleTextHeight")].GetDouble();
    m_titleTextY        = m_centerPoint.y + m_gaugeOuterRadius * TITLE_TEXT_Y_RATIO;

    m_fontSize  = (int)( m_gaugeOuterRadius * m_gd->Properties[_T("TitleTextHeight")].GetDouble() );
    m_font      = wxFont( m_fontSize, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL );

    m_pointerRadius = m_tickInnerRadius / 15;

    //Logger::logf(LOG_MSG,_T("VemsGaugeAgg::calculateSize() EndAngle=%i"),m_gd->GetEndAngle() );

}

void VemsGaugeAgg::setGradient( const wxColor& begin, const wxColor& end, double angle )
{
    m_gradientBeginColor    = begin;
    m_gradientEndColor      = end;
    m_gradientAngle         = angle;
}


wxRegion VemsGaugeAgg::GetBorderRegion()
{

    int width   = GetSize().GetWidth();
    int height  = GetSize().GetHeight();

    int size = width < height ? width : height;

    Logger::logf( LOG_MSG, _T(" VemsGaugeAgg::GetBorderRegion() - size : %i"), size );

    wxBitmap tempbmp = wxBitmap(size,size);

    wxMemoryDC temp_dc;

    temp_dc.SelectObject(tempbmp);
    temp_dc.SetBackground(*wxWHITE);            /// !!!!!!!
    temp_dc.DrawCircle(size/2, size/2, (size/2)-3 );
    temp_dc.SetBrush(*wxBLACK);
    temp_dc.DrawCircle(size/2, size/2, (int)(m_gaugeInnerRadius-3) );

    wxRegion region(tempbmp, m_theme->BackgroundColor);

//    tempbmp.SaveFile(_T("reg1.bmp"),wxBITMAP_TYPE_BMP);

    return region;

}

wxRegion VemsGaugeAgg::GetShape()
{

    int width   = GetSize().GetWidth();
    int height  = GetSize().GetHeight();

    int size = width < height ? width : height;

    //Logger::logf( LOG_MSG, _T(" VemsGaugeAgg::GetShape() - size : %i"), size );

    wxBitmap tempbmp(size,size);

    wxMemoryDC temp_dc;



    temp_dc.SelectObject(tempbmp);
    temp_dc.SetBackground(*wxWHITE);
    temp_dc.DrawCircle(size/2, size/2, (size/2)-1 );

    wxRegion region(tempbmp, *wxBLACK);

    region.Offset(-2,-2); /// makes vs2005 crash

    return region;
}

void VemsGaugeAgg::UpdateSize()
{
//    Logger::logf(LOG_MSG,_T("x=%i,y=%i"),GetSize().x,GetSize().y);

    calculateSize();
    m_isResized = true;
}


void VemsGaugeAgg::setSize(int x , int y)
{
}


void VemsGaugeAgg::SetValue(double value)
{
    if(value == m_overResValue){
        if(m_firstSetValue){
            m_firstSetValue = false;
        }else{
            return;
        }
    }

    if(m_gd == NULL)
      {
//        UserLog::uLog( ULOG_ERROR, _T("The selected symbol not found!</font><br>The gauge is broken.") );
        m_value = 0;
        return;
      }

    //Logger::logf(LOG_MSG,_T("VemsGaugeAgg::SetValue() value:%f"), value);

    m_value = value;
    m_overResValue = m_value;
    m_isDataValid = true;


    if(m_firstSetValue){
        m_valueMin = m_valueMax = value;
    }else{
        if(m_value > m_valueMax){
            m_valueMax = m_value;
            m_isValueMaxChanged = true;
        }
        //else{
        //    m_isValueMaxChanged = false;
        //}

        if(m_value < m_valueMin){
            m_valueMin = m_value;
            m_isValueMinChanged = true;
        }
        //else{
        //    m_isValueMinChanged = false;
        //}
    }


    Refresh(false);
}


void VemsGaugeAgg::OnSize( wxSizeEvent &event )
{

    int width, height;


	wxWindow* parent = 0;//GetParent();

	if( parent )
	{
	    parent->GetSize( &width, &height );
    }
    else
    {
        GetSize( &width, &height );
    }

    //Logger::logf(LOG_MSG,_T("width=%i, height=%i"),width, height);

    if( height > width * m_sideRatioRestriction )
        height = (int)(width * m_sideRatioRestriction);
    if( width * m_sideRatioRestriction > height )
        width = (int)(height / m_sideRatioRestriction);

    SetSize( width, height );

    //Logger::logf(LOG_MSG,_T("width=%i, height=%i"),width, height);

    m_isResized = true;

    Refresh( false );

    event.Skip();
}

void VemsGaugeAgg::drawResizedGauge( wxDC& dc )
{
    calculateSize();

    wxAgg aggCanvas( m_width, m_height );

    delete m_gaugeBMP;
    delete m_gaugeCoverBMP;

    drawGaugeFrame( aggCanvas );
    drawColoredArea( aggCanvas );
    drawTickMarks( aggCanvas );

    m_gaugeBMP = aggCanvas.renderToBitmap();

    //wxMemoryDC mDC( *m_gaugeBMP );

    //drawGaugeNumbers( mDC );
    //drawUnitText( mDC );
    //drawMultiLineTitleText( mDC );

    wxAgg aggCanvasCover( (int)( m_gaugeCoverOuterRadius * 2 ), (int)( m_gaugeCoverOuterRadius * 2 ) );

    drawGaugeCover( aggCanvasCover );

    m_gaugeCoverBMP = aggCanvasCover.renderToBitmap();
}

void VemsGaugeAgg::OnPaint( wxPaintEvent &event )
{
	wxAutoBufferedPaintDC dc( this );
    dc.SetBackground( *wxBLACK );
    dc.Clear();
	if(m_graphQual==2)
	{
		if( m_isResized )
		{
			drawResizedGauge( dc );
		}

		dc.DrawBitmap( *m_gaugeBMP, 0, 0, true );

		try {
			drawValueText( dc );
			drawGaugeNumbers( dc );
			drawUnitText( dc );
			drawMultiLineTitleText( dc );

			if( m_theme->PointerAA )
			{
				wxAgg aggCanvas( m_width, m_height );
				//if(m_isValueMaxChanged){
					drawPointer( aggCanvas, m_valueMax, m_theme->getMinMaxPointerColor(), m_pointerRadius * 0.4);
					m_isValueMaxChanged = false;
				//}
				//if(m_isValueMinChanged){
					drawPointer( aggCanvas, m_valueMin, m_theme->getMinMaxPointerColor(), m_pointerRadius * 0.4);
					m_isValueMaxChanged = false;
				//}
				drawPointer( aggCanvas, m_value, m_theme->PointerColor, m_pointerRadius);
				aggCanvas.render( dc );
			}
			else
			{
				//if(m_isValueMaxChanged){
					drawPointer( dc, m_valueMax, m_theme->getMinMaxPointerColor(), m_pointerRadius * 0.4);
					m_isValueMaxChanged = false;
				//}
				//if(m_isValueMinChanged){
					drawPointer( dc, m_valueMin, m_theme->getMinMaxPointerColor(), m_pointerRadius * 0.4);
					m_isValueMinChanged = false;
				//}
				drawPointer( dc, m_value, m_theme->PointerColor, m_pointerRadius);
			}
		}catch(...){
			Logger::logf( LOG_MSG, _T("VemsGaugeAgg::OnPaint() - Error allocating memory!") );
		}

		dc.DrawBitmap( *m_gaugeCoverBMP, (int)m_gaugeCoverX, (int)m_gaugeCoverY, true );
		m_isResized = false;
	}
	else
	{
		if( m_isResized )
		{
			calculateSize();
		}

		drawTickMarks( dc );
		drawValueText( dc );
		drawGaugeNumbers( dc );
		drawUnitText( dc );
		drawMultiLineTitleText( dc );
		drawPointer( dc, m_valueMax, m_theme->getMinMaxPointerColor(), m_pointerRadius * 0.4);
		m_isValueMaxChanged = false;
		drawPointer( dc, m_valueMin, m_theme->getMinMaxPointerColor(), m_pointerRadius * 0.4);
		m_isValueMinChanged = false;
		drawPointer( dc, m_value, m_theme->PointerColor, m_pointerRadius);
		drawGaugeFrame( dc );
		m_isResized = false;
	}


    DrawExtraLayer(dc);
}


void VemsGaugeAgg::SetNewTheme(WidgetThemeTable *theme)
{
    m_theme = theme;

    m_isResized = true;

    calculateSize();

    Refresh( false );

}


wxString VemsGaugeAgg::getValueString()
{
    wxString text = _T("N/A");

    if ( !GetCurrSymbolName().IsEmpty() && isExist( GetCurrSymbolName() ) && isDataAvailable() )
    {
        text.Printf( m_gd->GetGaugeNumberFormat(), m_value );
    }

    return text;
}
