#ifndef VEMSGAUGEDESCDIALOG_H
#define VEMSGAUGEDESCDIALOG_H

#include <wx/wx.h>
#include <wx/html/htmlwin.h>
#include <wx/gbsizer.h>
#include <wx/notebook.h>

#include "vemsgaugedesc.h"
#include "VemsWidgetBase.h"
#include "VemsWidgetBaseEditorBase.h"


class GaugeVar;

WX_DECLARE_HASH_MAP(int, GaugeVar *, wxIntegerHash, wxIntegerEqual, IntToGaugeVarHash );

WX_DECLARE_HASH_MAP(int, wxSlider *, wxIntegerHash, wxIntegerEqual, IntTowxSliderHash );

class VemsWidgetBaseEditor: public VemsWidgetBaseEditorBase
{
    public:
        VemsWidgetBaseEditor(VemsWidgetBase * pw);
        virtual ~VemsWidgetBaseEditor() {};

        virtual int ShowEditor(VEMSGaugeDesc* gd = NULL);

    private:

        VemsWidgetBase  *           m_parentWidget;

};

class ColoredAreaEdit
{
    public:
        ColoredAreaEdit(wxGridBagSizer * sizer, wxWindow * parent){
            GBSizer = sizer;
            Parent = parent;
            Ca = NULL;
            MinCtrl = NULL;
            MaxCtrl = NULL;
            ColorNameList = NULL;
        };

        ColoredAreaEdit(wxTextCtrl * min, wxTextCtrl * max, wxComboBox * colornms, ColoredArea * ca){
            MinCtrl = min;
            MaxCtrl = max;
            Ca = ca;
            ColorNameList = colornms;
        };

        void Update();
        void RefreshCtrl(const wxString &numformat);

        wxTextCtrl  * MinCtrl;
        wxTextCtrl  * MaxCtrl;
        wxComboBox  * ColorNameList;
        ColoredArea * Ca;
        wxBoxSizer  * Tsizer;
        wxGridBagSizer * GBSizer;
        wxWindow    * Parent;
};

class DescItemDesc
{
    public:
        DescItemDesc(wxSlider * sl, GDDesc * gdd){
            m_slider = sl;
            m_gdd = gdd;
            m_textCtrl = NULL;
            m_cae = NULL;
            m_isNum = false;
        };

        DescItemDesc(wxTextCtrl * ctrl, GDDesc * gdd, bool isnum = true){
            m_textCtrl = ctrl;
            m_gdd = gdd;
            m_slider = NULL;
            m_cae = NULL;
            m_isNum = isnum;
        };

        DescItemDesc(ColoredAreaEdit * cae, GDDesc * gdd){
            m_cae = cae;
            m_textCtrl = NULL;
            m_gdd = gdd;
            m_slider = NULL;
            m_isNum = false;
        };

    wxSlider    * m_slider;
    wxTextCtrl  * m_textCtrl;
    GDDesc      * m_gdd;
    ColoredAreaEdit * m_cae;
    bool        m_isNum;
};

WX_DECLARE_HASH_MAP(int, DescItemDesc *, wxIntegerHash, wxIntegerEqual, IntToDescItemDesc );

class VemsGaugeDescDialog: public wxDialog
{
    public:

        VemsGaugeDescDialog(VemsWidgetBase * pw, VEMSGaugeDesc* gd = NULL);
        virtual ~VemsGaugeDescDialog();

        void RefreshParent();

    protected:

        void CreateDialog();
        void CreateButtons();
        void CreateButton(const wxString &title, int id, wxSizer * sizer);
        void SaveGaugeDesc(const wxString &name, const wxString &type = wxEmptyString);

        VEMSGaugeDesc* GetDesc();

        void OnClose(wxCloseEvent& evt);

    private:

        void AddLines();

        void ResetGauge();

        void OnButton(wxCommandEvent &evt);
        void OnRefreshButton(wxCommandEvent &evt);
        void OnDelCaItem(wxCommandEvent &evt);
        void OnAddCaItem(wxCommandEvent &evt);
        void SetTitleText(const wxString &title);

        void AddSlider( GDDesc * gdd, wxWindow * parent, wxGridBagSizer * sizer );
        void AddNumeric(GDDesc * gdd, wxWindow * parent, wxGridBagSizer * sizer);
        void AddColoredAreas(GDDesc * gdd, wxWindow * parent, wxGridBagSizer * sizer);
        void AddColoredArea(GDDesc * gdd, ColoredArea * ca, wxWindow * parent, wxGridBagSizer * sizer);
        void AddString(GDDesc * gdd, wxWindow * parent, wxGridBagSizer * sizer);

        void UpdateModell(bool refreshParent = true);

        void ClearColoredAreasPanel();

        void OnEnter(wxCommandEvent &evt);

        void RefreshByDesc(DescItemDesc * desc, bool refresparent = false);

        bool IsRefreshButton();

        void CloseEditor(int retcode);

        enum{
            idOkButton = 1000,
            idCancelButton,
            idResetButton,
            idApplyButton,
            idSaveAsButton
        };



        void OnSlider( wxScrollEvent &evt );

        VemsWidgetBase          *   m_parentWidget;

        //IntToGaugeVarHash           m_values;
        IntTowxSliderHash           m_sliders;
        IntToDescItemDesc           m_items;

        IntToDescItemDesc           m_itemsToEnter;

        wxBoxSizer              *   m_mainSizer;

        VEMSGaugeDesc           *   m_defaultDesc;

        wxStaticText            *   m_titleText;

        wxHtmlWindow            *   m_titlePage;

        wxBookCtrl              *   m_panelBook;



        VEMSGaugeDesc*              m_outerGaugeDesc;


};



class GaugeVar
{
    public:
        GaugeVar(wxSlider * sl, long * val){
            m_slider = sl;
            m_val = val;
        };

    wxSlider * m_slider;
    long     * m_val;


};

class NameDialog: public wxDialog
{
    public:
        NameDialog(wxWindow *parent, const wxString &str = wxEmptyString, bool isshowtypelist = false, GaugeDescManager * gdmgr = NULL);

        wxString GetStr();
        void SetStr(const wxString &str) { m_text->SetValue(str); };

        wxString GetType();

    private:
        wxTextCtrl  * m_text;
        wxComboBox  * m_typeList;


};

#endif // VEMSGAUGEDESCDIALOG_H


