#include "VemsGraphWidgetAgg.h"

#include <wx/numdlg.h>
#include <wx/dcbuffer.h>

#include "../hotkeys/wxHotKeyManager.h"
#include "GaugeGroupManager.h"

#include "../agg/wxAgg2.h"
#include "../Math.h"

#include "../hotkeys/HotkeyHelper.h"

const int TIMERID = wxNewId();

BEGIN_EVENT_TABLE( VemsGraphWidgetAgg, VemsWidgetBase )
    EVT_PAINT( VemsGraphWidgetAgg::OnPaint )
    EVT_SIZE ( VemsGraphWidgetAgg::OnSize )
    EVT_KEY_DOWN( VemsGraphWidgetAgg::OnChar )
    EVT_TIMER( TIMERID, VemsGraphWidgetAgg::OnTimer )
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(VemsGraphWidgetAgg, VemsWidgetBase);

VemsGraphWidgetAgg::VemsGraphWidgetAgg( wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop)
                                        /*
                                        wxPoint pos,
                                        wxSize size,
                                        long style,
                                        const VEMSGaugeDescArray* gaugeDescA,
                                        const WidgetThemeTableArray * themes,
                                        long drawingStyle,
                                        GaugeGroupManager* groupManager,
                                        const wxString& groupName,
                                        const wxString& currTheme )*/
    : VemsWidgetBase( parent, id, gdmgr, prop ), m_timer( this, TIMERID )
    //pos, size, style, _T("VemsGraphWidgetAgg"), gaugeDescA, themes, groupManager, groupName, currTheme )
{
//    m_aggCanvas  = NULL;
    m_isResized = true;
    m_showHelpPanel = false;
    m_helpPanel = NULL;

    m_hotkeyMgr = new wxHotKeyManager(this);

    m_frameSize.SetWidth( 0 );
    m_frameSize.SetHeight( 20 );
    m_historyLength = 100;
    m_historyLengthBase = 100;
    m_currentSymbol = wxEmptyString;
    m_maxValue      = 0.0;
    m_minValue      = 0.0;

    m_titleText         = wxEmptyString;
    m_titlePosition     = wxPoint( 0, 0 );
    m_titleFont         = wxFont();
    m_titleColor        = *wxWHITE; //deprecated

    m_valueText         = _T("---");
    m_valuePosition     = wxPoint( 0, 0 );
    m_valueFont         = wxFont();
    m_valueColor        = *wxWHITE;

    m_minValuePosition  = wxPoint( 0, 0 );
    m_maxValuePosition  = wxPoint( 0, 0 );

    m_isGaugeDescMenu   = false;

    m_bShowLines        = true;
    m_bShowLinesValues  = true;
    m_bShowMarker       = false;
    m_dMarkerValue      = m_minValue;
    //Refresh();



    m_backgroundBitmap = NULL;

    LoadSettings();


    m_helpPanel = HotkeyHelper::DrawHotkeyHelp(m_hotkeyMgr);

    if ( m_helpPanel )
    Logger::logf(LOG_MSG,_T("m_helpPanel w=%i h=%i"),m_helpPanel->GetWidth(),m_helpPanel->GetHeight());

    m_timer.Start( 50 );
}


VemsGraphWidgetAgg::~VemsGraphWidgetAgg()
{
    Logger::logf( LOG_MSG, _T("-----VemsGraphWidgetAgg()::Destructor()") );
    SaveSettings();

    if ( m_hotkeyMgr )
    {
        delete m_hotkeyMgr;
        m_hotkeyMgr = NULL;
    }


//    if ( m_aggCanvas )
//    {
        //Logger::logf( LOG_MSG, _T("----------VemsGraphWidgetAgg()::Destructor() - delete m_aggCanvas") );
//        delete m_aggCanvas;
//        m_aggCanvas = NULL;
//    }

    if ( m_backgroundBitmap )
    {
        delete m_backgroundBitmap;
        m_backgroundBitmap = NULL;
    }

    if ( m_helpPanel )
    {
        delete m_helpPanel;
        m_helpPanel = NULL;
}

    WX_CLEAR_ARRAY( m_bmpHistory );

//    Logger::setTextColor( wxColor(255,0,0));
//    Logger::logf( LOG_MSG, _T("-----VemsGraphWidgetAgg()::Destructor()") );

}

void VemsGraphWidgetAgg::drawTitle( wxDC& dc )
{
    dc.SetTextForeground( m_theme->TitleColor );

    setDcFontSize(dc, _T("title"));

    m_titleText         = GetSymbolTitle();
    wxSize titleSize    = dc.GetTextExtent( m_titleText );
    m_titlePosition.x   = GetClientSize().x / 2 - titleSize.GetWidth() / 2;
    m_titlePosition.y   = 0;

    dc.DrawText( m_titleText, m_titlePosition.x, m_titlePosition.y );
}

void VemsGraphWidgetAgg::setDcFontSize(wxDC& dc, const wxString& type)
{
    if( m_fontSizes.find(type) == m_fontSizes.end() ) return;

    int val = m_fontSizes[type][_T("value")].GetLong();

    wxFont font = dc.GetFont();

    font.SetPointSize(val);

    dc.SetFont(font);
}

void VemsGraphWidgetAgg::SetNewTheme(WidgetThemeTable *theme)
{
    m_theme = theme;

    m_isResized = true;

    Refresh( false );
}

void VemsGraphWidgetAgg::drawValue( wxDC& dc )
{
    setDcFontSize(dc, _T("value"));

    wxString selectedSymbol = GetCurrSymbolName();
    wxString valueText      = wxEmptyString;

    if( ( selectedSymbol != wxEmptyString ) && isExist( selectedSymbol ) )
    {
        valueText.Printf( m_gd->GetGaugeNumberFormat(), m_value );
    }
    else
    {
        valueText = _T("---");
    }



    //draw value
    wxSize valueTextSize = dc.GetTextExtent( valueText );
    dc.SetTextForeground( m_theme->ValueColor );
    int x = GetSize().GetWidth() / 2 - valueTextSize.GetWidth() / 2;
    int y = GetSize().GetHeight() - valueTextSize.y; //m_frameSize.GetHeight();
    dc.DrawText( valueText, x, y );

    setDcFontSize(dc, _T("minval") );

    //draw min value
    valueText = wxString::Format( _T("min = ") + m_gd->GetGaugeNumberFormat(), m_minValue );
    valueTextSize = dc.GetTextExtent( valueText );
    //x +=  valueTextSize.GetWidth() + 10;
    x = 0;
    y = GetSize().GetHeight() - valueTextSize.y; //m_frameSize.GetHeight();
    dc.DrawText( valueText, x, y );

    setDcFontSize(dc, _T("maxval") );

    //draw max value
    valueTextSize = dc.GetTextExtent( valueText );
    //x +=  valueTextSize.GetWidth() + 10;
    y = 0;
    valueText = wxString::Format( _T("max = ") + m_gd->GetGaugeNumberFormat(), m_maxValue );
    dc.DrawText( valueText, x, y );

}


void VemsGraphWidgetAgg::drawDivisionsText( wxDC& dc )
{
    double height = GetSize().GetHeight();
    double maxValue         = m_gd->GetMaxValue();
    double minValue         = m_gd->GetMinValue();
    double majorDivisions   = m_gd->GetMajorDivisions();
    double range            = maxValue - minValue;
    double frameSize        = m_frameSize.GetHeight();
    double scaleY           = ( height - frameSize * 2 ) / range;
    double step             = ( maxValue - minValue ) / majorDivisions;

    dc.SetTextForeground( m_theme->DivTextColor );
    wxString text = wxEmptyString;

    int x = 0;
    int y = 0;
    double value = 0.0;

    setDcFontSize(dc, _T("ticks") );

    for( int i = 1; i < majorDivisions + 1; ++i )
    {
        value   = ( maxValue - i * step ) / m_gd->GetUnitsScaleFactor();
        text    = wxString::Format( m_gd->GetGaugeNumberFormat(), value );

        int textHeight = dc.GetTextExtent( text ).GetHeight();
        y = (int)(i * step * scaleY + frameSize - textHeight);

        dc.DrawText( text, x, y );
    }
/*

    //draw max value
    wxString text = wxString::Format( m_gd->GetGaugeNumberFormat(), m_gd->GetMaxValue() );
    x = 10;
    dc.DrawText( text, x, y );

    //draw min value
    text = wxString::Format( m_gd->GetGaugeNumberFormat(), m_gd->GetMinValue() );
    y = GetSize().GetHeight() - m_frameSize.GetHeight();
    dc.DrawText( text, x, y );
*/
}


void VemsGraphWidgetAgg::drawUnitText( wxDC& dc )
{
    setDcFontSize(dc, _T("unit"));

    wxString unitText = m_gd->GetUnit();

    if( m_gd->GetUnitsScaleFactor() != 1 )
    {
        unitText += wxString::Format( _T(" x %i"), m_gd->GetUnitsScaleFactor() );
    }

    wxSize unitTextSize = dc.GetTextExtent( unitText );
    dc.SetTextForeground( m_theme->ValueColor );
    int x = GetSize().GetWidth() / 4 *3 - unitTextSize.GetWidth() / 2;
    int y = GetSize().GetHeight() - unitTextSize.y;//m_frameSize.GetHeight();
    dc.DrawText( unitText,x , y );
}

void VemsGraphWidgetAgg::renderBackground()
{
    if( !m_gd ) return;

    double maxV = m_gd->GetMaxValue();
    double minV = m_gd->GetMinValue();

    wxSize size = GetSize();
    int w = size.GetWidth();
    int h = size.GetHeight();

    //int frameSize = 20;

    //set graph scale
    double width        = w;
    double height       = h;
    double frameSize    = m_frameSize.GetHeight();
    double range    = maxV - minV;
    double scaleY   = ( height - frameSize * 2 ) / range;
    //double offsetY  =  (double)height - (double)frameSize;
    double offsetY = maxV;

    wxAgg2 aggCanvas(w,h);

    aggCanvas.clear( m_theme->BackgroundColor );

    aggCanvas.setPen( m_theme->DivColor );
    aggCanvas.setPenWidth( 1.0 );
    aggCanvas.drawLine( 0, frameSize, width, frameSize );                  //max value
    aggCanvas.drawLine( 0, height - frameSize, width, height - frameSize );//min value

    //draw major division lines
    size_t i;

    aggCanvas.setPenWidth( 0.5 );
    if( m_bShowLines )
    {
        double majorDivisions  = m_gd->GetMajorDivisions();
        double step = ( height - frameSize * 2 ) / majorDivisions;

        for( i = 1; i < majorDivisions; ++i )
        {
            aggCanvas.drawLine( 0, Math::round( frameSize + i * step ) + 0.5, width, Math::round( frameSize + i * step ) + 0.5 );                  //max value
        }
    }

    //draw marker
    if ( m_bShowMarker )
    {
        aggCanvas.setPen( *wxRED );
        aggCanvas.setPenWidth( 2.0 );
        aggCanvas.drawLine(  0,  Math::round( ( offsetY - m_dMarkerValue ) * scaleY + frameSize ) + 0.5,
                                width,  Math::round( ( offsetY - m_dMarkerValue ) * scaleY + frameSize ) + 0.5 );
    }

    if(m_backgroundBitmap)
        delete m_backgroundBitmap;

    wxString str = _T("Toggle help: ");

    if ( m_hotkeyMgr )
    str += m_hotkeyMgr->GetHotKeyStringBySymbol(_T("TOGGLE_HELP"));

    aggCanvas.setPen( m_theme->TitleColor );
    double strw = aggCanvas.getTextWidth(str,8);
    aggCanvas.drawText(str, GetClientSize().x - strw - 5, 12, 0, 8,1);

    m_backgroundBitmap = aggCanvas.renderToBitmap();
}

void VemsGraphWidgetAgg::renderGraph(bool isFullRender)
{
    //Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::drawGraph() - START") );
    if( !m_gd ) return;

    double maxV = m_gd->GetMaxValue();
    double minV = m_gd->GetMinValue();

    wxSize size = GetSize();
    int w = size.GetWidth();
    int h = size.GetHeight();

    //int frameSize = 20;

    //set graph scale
    double width        = w;
    double height       = h;
    double frameSize    = m_frameSize.GetHeight();
    double range    = maxV - minV;
    double scaleY   = ( height - frameSize * 2 ) / range;
    //double offsetY  =  (double)height - (double)frameSize;
    double offsetY = maxV;

    int scaleX = (int)(width / m_historyLength);
    if( !scaleX ) scaleX = 1;

    wxAgg2 aggCanvas(scaleX,h);

    //draw graph

    wxColor color = m_theme->PointerColor;
    if( !m_theme->PointerAA )
        color.Set( color.Red(), color.Green(), color.Blue(), 255 );

    aggCanvas.setPen( color );
    //double      scaleX = width / (double)m_historyLength;

    size_t len = m_valuesA.GetCount();

    //if( len > m_historyLength)
      //  len = m_historyLength;

    double x1 = 0.0;
    double y1 = 0.0;
    double x2 = 0.0;
    double y2 = 0.0;

    int i = len - m_historyLength;

    if( len < 2 ) return;

    if( isFullRender ) {
        WX_CLEAR_ARRAY(m_bmpHistory);

    } else {
        i = len - 2;
    }

    if( i < 0) i=0;

    if( m_bmpHistory.GetCount() == m_historyLength ){
        delete m_bmpHistory[0];
        m_bmpHistory.RemoveAt(0);
    }

   // int f = i;

    aggCanvas.setPenWidth( 2.0 );
    for(; i < (int)(len) - 1 ; ++i )
    {
        aggCanvas.clear( wxColor(0,0,0,0) );

        x1 = 0;
        y1 = ( offsetY - m_valuesA.Item( i ) ) * scaleY + frameSize;
        x2 = scaleX;
        y2 = ( offsetY - m_valuesA.Item( i + 1 ) ) * scaleY + frameSize;


        aggCanvas.drawLine( x1, y1, x2, y2 );

        m_bmpHistory.Add( aggCanvas.renderToBitmap() );
    }

    //Logger::logf(LOG_MSG,_T("scaleX=%i, i=%i, len=%i, bmpc=%i"),scaleX,f,len,m_bmpHistory.GetCount());

}

//VemsWidgetBase virtual functions
wxRegion VemsGraphWidgetAgg::GetShape()
{
    wxRect rect( 0, 0, GetSize().x,GetSize().y );

    rect.Inflate( 1, 1 );

    wxRegion region( rect );

    region.Offset( -1, -2 );

    return region;
}

wxRegion VemsGraphWidgetAgg::GetBorderRegion()
{
    return wxRegion();
}

void VemsGraphWidgetAgg::CalcHistoryLen()
{
    int width   = GetSize().GetWidth();

    int scaleX = width / m_historyLengthBase;
    if( !scaleX ) scaleX = 1;

    m_historyLength = m_historyLengthBase + (width % m_historyLengthBase) / scaleX;
}

void VemsGraphWidgetAgg::UpdateSize()
{
    wxClientDC dc( this );

    //int width   = GetSize().GetWidth();
 //   int height  = GetSize().GetHeight();


    m_isResized = true;

    CalcHistoryLen();

    renderGraph( true );

/*
    m_valueTextPosition     = wxPoint( 0, 0 );
    m_minValueTextPosition  = wxPoint( 0, 0 );
    m_maxValueTextPosition  = wxPoint( 0, 0 );
*/
}

void VemsGraphWidgetAgg::setSize( int x , int y ){}

//RTDataViewerBase virtual functions
void VemsGraphWidgetAgg::SetValue( double value )
{
    //if( value == this->m_value) return;

    if( m_gd == NULL )
    {
        //fprintf( stdout, "m_gd is NULL! \n");
        Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::SetValue() - m_gd is NULL!!!") );
        m_value = 0;
        return;
    }

    //if symbol changed clear values
    if( m_gd->GetName() != m_currentSymbol )
    {
        m_valuesA.Clear();
        m_currentSymbol = m_gd->GetName();
    }

    this->m_value = value;

    if( value > m_gd->GetMaxValue() )
    {
        m_value = m_gd->GetMaxValue();
    }

    if( m_value < m_gd->GetMinValue() )
    {
        m_value = m_gd->GetMinValue();
    }

    if( m_valuesA.GetCount() == m_historyLengthBase*2 )
    {
        m_valuesA.RemoveAt( 0 );
    }

    m_valuesA.Add( m_value );

    //serach max,min value
    m_maxValue = 0.0;
    m_minValue = m_gd->GetMaxValue();
    double v   = 0.0;

    for( size_t i = 0; i < m_valuesA.GetCount(); ++i )
    {
        v = m_valuesA.Item( i );

        if( v < m_minValue )
        {
            m_minValue = v;
        }

        if( v > m_maxValue )
        {
            m_maxValue = v;
        }
    }

    renderGraph();

    // Refresh( false );
}

//event handler functions

void VemsGraphWidgetAgg::OnTimer( wxTimerEvent& event )
{
    Refresh( false );
}

void VemsGraphWidgetAgg::OnSize( wxSizeEvent &event )
{
    UpdateSize();
    Refresh( false );
    event.Skip();
}

void VemsGraphWidgetAgg::OnPaint( wxPaintEvent &WXUNUSED(event) )
{
    wxBufferedPaintDC dc( this );
    dc.SetBackground( m_theme->BackgroundColor );
    dc.Clear();

    if( !m_gd )
    {
        Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::OnPaint() - m_gd is NULL!!!") );
        return;
    }

    //drawGraph();
    if( m_isResized )
	{
        renderBackground();
		m_isResized = false;
	}

    dc.DrawBitmap( *m_backgroundBitmap, 0,0,1);

    int width = GetSize().x;
    int scaleX = width / m_historyLength;
    if( !scaleX ) scaleX = 1;

    int dlen = m_bmpHistory.GetCount();

    int startX = width - (dlen*scaleX) ;

    for( int a = 0; a < dlen;  a++ ) {

        int x = startX + ( scaleX * a );

        dc.DrawBitmap( *(m_bmpHistory[a]), x , 0, 1);
    }


    drawTitle( dc );
    drawValue( dc );
    if ( m_bShowLinesValues ) { drawDivisionsText( dc ); }
    drawUnitText( dc );

    if(m_showHelpPanel && m_helpPanel)
        dc.DrawBitmap(*m_helpPanel, (GetClientSize().x-m_helpPanel->GetWidth())/2, (GetClientSize().y-m_helpPanel->GetHeight())/2 );

    DrawExtraLayer(dc);
}

void VemsGraphWidgetAgg::OnChar( wxKeyEvent& event )
{

    if( m_groupManager->IsEditMode() ) {
        event.Skip();
        return;
    }

    wxString symbol;

    if ( m_hotkeyMgr )
        symbol = m_hotkeyMgr->GetEvent( event );


    if( symbol == _T("SET_MARKER_LINE_POS") ) {

        wxString str = ::wxGetTextFromUser( _T(""),
                                            _T("Set marker line position!"),
                                            _T(""),
                                            this,
                                            wxDefaultCoord,
                                            wxDefaultCoord,
                                            true );

        Logger::logf( LOG_MSG, _T("marker str: ") + str );
        double marker;
        if ( str.ToDouble( &marker ) )
        {
            if ( marker > m_gd->GetMinValue() && marker < m_gd->GetMaxValue() )
            {

                m_dMarkerValue = marker;
                Logger::logf( LOG_MSG, _T("marker: %f"), m_dMarkerValue );

            }
        }

    } else if( symbol == _T("TOGGLE_LINES") ) {

        m_bShowLines = !m_bShowLines;

    } else if( symbol == _T("TOGGLE_LINEVALUES") ) {

        m_bShowLinesValues = !m_bShowLinesValues;

    } else if( symbol == _T("TOGGLE_MARKER") ) {

        m_bShowMarker =! m_bShowMarker;

    } else if( symbol == _T("SET_HISTORY_LENGTH") ) {

        long ret = ::wxGetNumberFromUser(   _T(""),
                                            _T("Value:"),
                                            _T("Set the lenght of the hystory!"),
                                            m_historyLengthBase,
                                            10,
                                            300,
                                            this);


        if( ret > 0 ) m_historyLengthBase = ret;

        UpdateSize();

    } else if( symbol == _T("SET_FONT_SIZES") ) {

        FontSizePanel* f = new FontSizePanel(this, &m_fontSizes);
        f->Show(1);


    } else if( symbol == _T("TOGGLE_HELP") ) {

        m_showHelpPanel = !m_showHelpPanel;

    } else {
        event.Skip();
        return;
    }

//    Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::OnChar") );
//
//    int keyCode = event.GetKeyCode();
//
//    switch( keyCode )
//    {
//        case 'H' :
//        {
//            m_bShowLines = !m_bShowLines;
//        }break;
//        case 'V' :
//        {
//            m_bShowLinesValues = !m_bShowLinesValues;
//        } break;
//        case 'S' :
//        {
//            wxString str = ::wxGetTextFromUser( _T(""),
//                                                _T("Set marker line position!"),
//                                                _T(""),
//                                                this,
//                                                wxDefaultCoord,
//                                                wxDefaultCoord,
//                                                true );
//
//            Logger::logf( LOG_MSG, _T("marker str: ") + str );
//            double marker;
//            if ( str.ToDouble( &marker ) )
//            {
//                if ( marker > m_gd->GetMinValue() && marker < m_gd->GetMaxValue() )
//                {
//
//                    m_dMarkerValue = marker;
//                    Logger::logf( LOG_MSG, _T("marker: %f"), m_dMarkerValue );
//
//                }
//            }
//        } break;
//
//        case 'M':
//        {
//            m_bShowMarker =! m_bShowMarker;
//        } break;
//        default:
//            event.Skip();
//    }
//




    Refresh(false);
}

void VemsGraphWidgetAgg::LoadSettings()
{
    wxVariantHash datas;

    m_fontSizes[_T("unit")][_T("value")] = 10L;
    m_fontSizes[_T("value")][_T("value")] = 10L;
    m_fontSizes[_T("minval")][_T("value")] = 10L;
    m_fontSizes[_T("maxval")][_T("value")] = 10L;
    m_fontSizes[_T("title")][_T("value")] = 10L;
    m_fontSizes[_T("ticks")][_T("value")] = 10L;


    m_fontSizes[_T("unit")][_T("title")] = _T("Unit");
    m_fontSizes[_T("value")][_T("title")] = _T("Value");
    m_fontSizes[_T("minval")][_T("title")] = _T("Minimum value");
    m_fontSizes[_T("maxval")][_T("title")] = _T("Maximum value");
    m_fontSizes[_T("title")][_T("title")] = _T("Title");
    m_fontSizes[_T("ticks")][_T("title")] = _T("Ticks");

    if( LoadSettingsBase( datas ) >= 0) {

        m_bShowLines        = datas[_T("ShowLines")].GetBool();
        m_bShowLinesValues  = datas[_T("ShowLinesValues")].GetBool();
        m_bShowMarker       = datas[_T("ShowMarker")].GetBool();
        m_dMarkerValue      = datas[_T("MakerValue")].GetDouble();
        m_historyLengthBase = datas[_T("HistoryLengthBase")].GetLong();

        for(wxVariantHashHash::iterator it = m_fontSizes.begin(); it != m_fontSizes.end(); it++) {

            long val = datas[it->first];

            if(val > 0) m_fontSizes[it->first][_T("value")] = val;
        }

    } else {

        m_bShowLines        = true;
        m_bShowLinesValues  = true;
        m_bShowMarker       = false;
        m_dMarkerValue      = m_minValue;
        m_historyLengthBase = 100;

    }

    if(m_historyLengthBase < 10) m_historyLengthBase = 100;
}

void VemsGraphWidgetAgg::SaveSettings()
{
    wxVariantHash datas;

    datas[_T("ShowLines")] = m_bShowLines;
    datas[_T("ShowLinesValues")] = m_bShowLinesValues;
    datas[_T("ShowMarker")] = m_bShowMarker;
    datas[_T("MakerValue")] = m_dMarkerValue;
    datas[_T("HistoryLengthBase")] = (long)m_historyLengthBase;

    for(wxVariantHashHash::iterator it = m_fontSizes.begin(); it != m_fontSizes.end(); it++) {

        datas[it->first] = m_fontSizes[it->first][_T("value")];

    }

    SaveSettingsBase( datas );
}

void VemsGraphWidgetAgg::RedrawWidget()
{
    renderBackground();
    renderGraph(true);

    Refresh(false);
}

/**************************************************************************************
                            class FontSizePanel
*/

FontSizePanel::FontSizePanel(VemsWidgetBase* parent, wxVariantHashHash* model, int default_size)
{
    m_parent = parent;

    m_defval = default_size;

    Create(parent, wxID_ANY, _T("Set font sizes"));

    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_model = model;

    wxFlexGridSizer* sizer = new wxFlexGridSizer(2);

    sizer->AddGrowableCol(1);

    for(wxVariantHashHash::iterator it = m_model->begin(); it != m_model->end(); it++ ) {

        wxString name = it->first;

        wxVariantHash& property = it->second;

        sizer->Add(new wxStaticText(this, wxID_ANY, property[_T("title")] ), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );

        int id = wxNewId();

        wxSlider* slider = new wxSlider(this, id, property[_T("value")].GetLong(), 0, 30 , wxDefaultPosition, wxSize(300,-1), wxSL_HORIZONTAL | wxSL_LABELS | wxSL_AUTOTICKS);

        sizer->Add(slider, 0, wxEXPAND | wxALL, 5);

        slider->Connect( wxEVT_SCROLL_TOP, wxCommandEventHandler( FontSizePanel::OnSlider ), NULL, this );
        slider->Connect( wxEVT_SCROLL_BOTTOM, wxCommandEventHandler( FontSizePanel::OnSlider ), NULL, this );
        slider->Connect( wxEVT_SCROLL_LINEUP, wxCommandEventHandler( FontSizePanel::OnSlider ), NULL, this );
        slider->Connect( wxEVT_SCROLL_LINEDOWN, wxCommandEventHandler( FontSizePanel::OnSlider ), NULL, this );
        slider->Connect( wxEVT_SCROLL_PAGEUP, wxCommandEventHandler( FontSizePanel::OnSlider ), NULL, this );
        slider->Connect( wxEVT_SCROLL_PAGEUP, wxCommandEventHandler( FontSizePanel::OnSlider ), NULL, this );
        slider->Connect( wxEVT_SCROLL_THUMBTRACK, wxCommandEventHandler( FontSizePanel::OnSlider ), NULL, this );
        slider->Connect( wxEVT_SCROLL_THUMBRELEASE, wxCommandEventHandler( FontSizePanel::OnSlider ), NULL, this );

        m_sliders[id] = name;
    }

    wxBoxSizer* buttonsizer = new wxBoxSizer(wxHORIZONTAL);

    //buttonsizer->Add(new wxButton(this, idButtonOk, _T("OK") ), 0, wxALL, 5 );
    buttonsizer->Add(new wxButton(this, idButtonReset, _T("Reset to default") ), 0, wxALL, 5 );
    buttonsizer->Add(new wxButton(this, idButtonCancel, _T("Close") ), 0, wxALL, 5 );


    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);


    mainsizer->Add(sizer, 1, wxEXPAND | wxALL, 5);
    mainsizer->Add(buttonsizer, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5);

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    Connect(idButtonCancel, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(FontSizePanel::OnButton), NULL, this );
    Connect(idButtonReset, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(FontSizePanel::OnButton), NULL, this );

    Centre();
}

void FontSizePanel::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()){
        case idButtonCancel:
            Close();
            break;
        case idButtonReset: {

            for(Int2StringHash::iterator it = m_sliders.begin(); it != m_sliders.end(); it++) {
                int id = it->first;
                wxString name = it->second;

                wxWindow* win = FindWindowById(id);

                if(!win) continue;

                wxSlider* slider = wxDynamicCast(win, wxSlider);

                if(!slider) continue;

                slider->SetValue(m_defval);

                if(m_model->find(name) == m_model->end() ) continue;

                (*(m_model))[name][_T("value")] = m_defval;

    }

            m_parent->RedrawWidget();

            }break;
    }
}

void FontSizePanel::OnSlider(wxCommandEvent& evt)
{
    int id = evt.GetId();

    if( m_sliders.find(id) == m_sliders.end() ) return;

    wxString name = m_sliders[id];

    if(m_model->find(name) == m_model->end() ) return;

    (*(m_model))[name][_T("value")] = (long)evt.GetInt();

    m_parent->RedrawWidget();
}


//----------------------------------------------------
/*
void VemsGraphWidgetAgg::drawGraph()
{
    //Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::drawGraph() - START") );
    if( !m_gd ) return;

    double maxV = m_gd->GetMaxValue();
    double minV = m_gd->GetMinValue();

    wxSize size = GetSize();
    int w = size.GetWidth();
    int h = size.GetHeight();

    //int frameSize = 20;

    //set graph scale
    double width        = w;
    double height       = h;
    double frameSize    = m_frameSize.GetHeight();
    double range    = maxV - minV;
    double scaleY   = ( height - frameSize * 2 ) / range;
    //double offsetY  =  (double)height - (double)frameSize;
    double offsetY = maxV;

    m_aggCanvas->clear( m_theme->BackgroundColor );

    //draw min,max line
    //m_aggCanvas->setPen( *wxGREEN );
    m_aggCanvas->setPen( m_theme->DivColor );
    m_aggCanvas->drawLine( 0, frameSize, width, frameSize, 1.0 );                  //max value
    m_aggCanvas->drawLine( 0, height - frameSize, width, height - frameSize, 1.0 );//min value

    //draw major division lines
    size_t i;

    if( m_bShowLines )
    {
        double majorDivisions  = m_gd->GetMajorDivisions();
        double step = ( height - frameSize * 2 ) / majorDivisions;

        for( i = 1; i < majorDivisions; ++i )
        {
            m_aggCanvas->drawLine( 0, Math::round( frameSize + i * step ) + 0.5, width, Math::round( frameSize + i * step ) + 0.5, 0.5 );                  //max value
        }
    }

    //draw marker
    if ( m_bShowMarker )
    {
        m_aggCanvas->setPen( *wxRED );
        m_aggCanvas->drawLine(  0,  Math::round( ( offsetY - m_dMarkerValue ) * scaleY + frameSize ) + 0.5,
                                width,  Math::round( ( offsetY - m_dMarkerValue ) * scaleY + frameSize ) + 0.5, 2.0 );
    }

    //draw graph
    m_aggCanvas->setPen( m_theme->PointerColor );
    double      scaleX = width / (double)m_historyLength;
    size_t    length = m_valuesA.GetCount();

    double x1 = 0.0;
    double y1 = 0.0;
    double x2 = 0.0;
    double y2 = 0.0;

    //Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::drawGraph() - BEGIN FOR") );

    for(i = 0; i < length - 1 ; ++i )
    {
        if( length < 2 ) { break; }

        x1 = width - ( (double)length - i ) * scaleX;
        y1 = ( offsetY - m_valuesA.Item( i ) ) * scaleY + frameSize;
        x2 = width - ( (double)length - ( i + 1 ) ) * scaleX;
        y2 = ( offsetY - m_valuesA.Item( i + 1 ) ) * scaleY + frameSize;

        //Logger::logf( LOG_MSG, _T("\t i = %d"), i );
        //Logger::logf( LOG_MSG, _T("\t width = %f, length = %d, scaleX = %f"), width, length, scaleX );
        //Logger::logf( LOG_MSG, _T("\t x1 = %f, y1 = %f, x2 = %f, y2 = %f"), x1, y1, x2, y2 );

        m_aggCanvas->drawLine( x1, y1, x2, y2, 2.0 );

        //Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::drawGraph() - END DRAW LINE") );
    }

    //Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::drawGraph() - END FOR") );

    //Logger::logf( LOG_MSG, _T("VemsGraphWidgetAgg::drawGraph() - END") );



}
*/
