#include "VemsWidgetBase.h"

#include <wx/menu.h>
#include <wx/msgdlg.h>
#include <wx/dcbuffer.h>

#include "../mtconfigdata.h"
#include "../DirHelper.h"

#include "../configmanager/ConfigHelper2.h"
#include "../ConfigBase.h"

#include "../events/UpdateViewerEvent.h"

#include "GaugeGroupManager.h"
#include "VemsWidgetBaseEditorBase.h"

#include "../gaugevalueselector.h"
#include "../UserLog.h"
#include "../logger.h"
#include "../common/VemsMessageHashHelper.h"
#include "../ThemesHelper.h"
#include "GaugeType.h"
#include "../vemsdialog/WindowTipper.h"

#include "../WindowHelper.h"
#include "../agg/wxAgg.h"
#include "../wxPointEx.h"

#include "../vemsdialog/SizePosDialog.h"
#include "../wxVariantHelper.h"

#include "SelectableWidgetRendererAgg.h"

#include <wx/keybinder.h>

/** \class VemsWidgetBase
  *
  * \~english
  * \brief Base class for the widgets.
  * \details
  *
  *
  * \~hungarian
  * \brief Ős osztálya a főbb widgeteknek.
  * \details
  *
  * \~ \ref vwbover
  */

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(DuplicateMenuItemDescArray);

#define USE_ADVANCED_SELECTION_MODE 1

const double DATA_IS_NOT_AVAILABLE = -1e6;

//wxConfig * VemsWidgetBase::conf;
int GAUGE_QUIT_ID_AS = wxNewId();
int GAUGE_SETLAY_ID = wxNewId();

BEGIN_EVENT_TABLE(VemsWidgetBase, wxControl)
    EVT_MENU(GAUGE_QUIT_ID_AS, VemsWidgetBase::OnQuit)
    EVT_MENU(GAUGE_SETLAY_ID, VemsWidgetBase::OnSetLayout)

    //EVT_MOUSE_EVENTS(VemsWidgetBase::OnAllMouseEvents)

    EVT_LEFT_DCLICK(VemsWidgetBase::OnDoubleClick)
    EVT_MOUSEWHEEL(VemsWidgetBase::OnMouseWheel)
    EVT_LEFT_DOWN(VemsWidgetBase::OnLeftDown)
    EVT_LEFT_UP(VemsWidgetBase::OnLeftUp)
    EVT_MOTION(VemsWidgetBase::OnMouseMove)
    EVT_RIGHT_DOWN(VemsWidgetBase::OnMouseRightClick)
    EVT_KEY_DOWN(VemsWidgetBase::OnChar)
    //EVT_UPDATE_VIEWER_EVENT( wxID_ANY, VemsWidgetBase::OnUpdateValue )

    EVT_SIZE(VemsWidgetBase::OnSizeBase)

END_EVENT_TABLE()

IMPLEMENT_ABSTRACT_CLASS(VemsWidgetBase, wxControl);

VemsWidgetBase::VemsWidgetBase(){};
VemsWidgetBase::VemsWidgetBase(wxWindow* parent, wxWindowID id, GaugeDescManager * gaugemgr, GaugeProperties &prop )
    : m_gd ( NULL )
{
    m_graphQual = 0;

    m_isMouseResize = false;
    m_isSymbolChanged = false;
    m_extraLayerBmp = NULL;
    m_themes = 0;
    m_theme = 0;

//    m_gd = NULL;
    m_parent = NULL;
    m_winTip = NULL;
    m_gaugeDescA = NULL;
    m_gaugeDescEditor = NULL;
    vselector = NULL;

    m_floating = false;


    CreateWidget(parent,id, gaugemgr, prop );

}

void VemsWidgetBase::CreateWidget( wxWindow* parent, wxWindowID id, GaugeDescManager * gaugemgr, GaugeProperties &prop )
{
    m_isSelected = false;
    m_isExtendedResize = false;
    m_isGaugeDescMenu = false;
    m_isSymbolMenu = true;
    m_isThemeMenu = true;
    m_extraLayerBmp = NULL;
    m_delta = wxDefaultPosition;
    m_hasOwnDescriptorMgr = false;

    m_gaugeDescMgr = gaugemgr;

    SelectableWidgetRendererAgg* renderer = new SelectableWidgetRendererAgg();
    SetRenderer(renderer);

    initGroupMember( parent, prop, gaugemgr->GetGaugeGrpMgr(), id);

    m_hasShape = false;
    m_graphQual = 1;

    m_isExtendedResize = false;

    m_gaugeDescEditor = NULL;
    m_isGaugeDescMenu = false;

    m_sideRatioRestriction = DefaultSideRaitoRestriction;

    m_gaugeDescA = gaugemgr->GetVEMSGaugeDescArray();

    Create(parent,id,prop.getPos(),prop.getSize(), prop.extraParams[_T("windowStyle")].GetLong(), wxDefaultValidator, _T("Viewer"));
    //Create(parent,id,prop.getPos(),prop.getSize(), prop.extraParams[_T("windowStyle")].GetLong(), wxDefaultValidator, _T("Viewer"));

    Hide();
    m_parent = parent;

	//vselector = new GaugeValueSelector(this);
    //vselector->SetRTDataViewer(this);

    requireChanged();

    ConfigBase* conf = ConfigHelper2::getConfig();

    wxString grepQualStr = conf->getValue(_T("/"), _T("graphics_quality_factor"), _T("1") );
    m_graphQual = wxAtoi(grepQualStr);
    //conf->SetPath(_T("/"));
    //m_graphQual = conf->Read(_T("graphics_quality_factor"),1);

    InitTheme( prop );


    m_prevSymbolName = wxEmptyString;
    m_isSymbolChanged = true;

    m_name = GaugeType::toString( prop.gaugeType );

    m_winTip = new WindowTipper(this, -1, GetScreenPosition());
    m_isDataAvailable = false;
}

VemsWidgetBase::~VemsWidgetBase()
{
    ptrdel(m_renderer);
}

void VemsWidgetBase::DrawExtraLayer(wxDC& dc)
{
    if( m_extraLayerBmp ) dc.DrawBitmap(*m_extraLayerBmp,0,0,1);

    if( m_selectionBitmap && m_isSelected ) dc.DrawBitmap(*m_selectionBitmap,0,0,1); /// todo: position...
}

void VemsWidgetBase::InitTheme(GaugeProperties &prop)
{
    if(!m_themes) m_themes = m_gaugeDescMgr->GetWidgetThemes();
    if(!m_theme) m_theme = SearchTheme( prop.themeName );
}

void VemsWidgetBase::OnSetLayout(wxCommandEvent& event)
{
    SizePosDialog dlg(this);

    dlg.SetValues(GetTargetWin());

    if(dlg.ShowModal() == wxID_OK)
        dlg.SetToWindow(GetTargetWin());

}

void VemsWidgetBase::OnAllMouseEvents(wxMouseEvent& evt)
{
    evt.Skip();
}

void VemsWidgetBase::OnMouseWheel(wxMouseEvent& event)
{
    //Logger::logf(LOG_MSG,_T("VemsWidgetBase::OnMouseWheel"));

	bool forwards = (event.GetWheelRotation() > 0);



    double inc = m_gd->GetMaxValue() / 50.0;

    //Logger::logf(LOG_MSG,_T("VemsWidgetBase::OnMouseWheel m_value=%i"), m_value);

	if (forwards) {
		SetValue(m_value + inc );
	} else {
		SetValue(m_value - inc );
	}


}

void VemsWidgetBase::OnLeftDown(wxMouseEvent& evt)
{


    if(GetShape().Contains(evt.GetX(),evt.GetY()) != wxInRegion ) {
        wxPoint npos = evt.GetPosition() + GetTargetWin()->GetPosition();
        evt.m_x = npos.x;
        evt.m_y = npos.y;
        m_parent->ProcessEvent(evt);
        return;
    }


    SetFocus();

   // Logger::logf(LOG_MSG,_T("VemsWidgetBase::OnLeftDown valid"));

#if defined (__WINCE_STANDARDSDK__)

    //setMenuItemsState();

    if(choices.GetCount() > 0) {
        //vselector->Clear();
        //vselector->AddItems(choices);
        //vselector->ResizeAndShow(1);
    }
    else {
        wxMessageBox(_T("Please select an input source"),_T("Warning"));
    }

#else

    if( !(m_groupManager->IsEditMode()) ) {
        evt.Skip();
        return;
    }

    CaptureMouse();

    m_beginMousePoint = evt.GetPosition();

    wxPoint pos = ClientToScreen(m_beginMousePoint);

    wxPoint origin = GetTargetWin()->GetPosition();
    int dx =  pos.x - origin.x;
    int dy = pos.y - origin.y;
    m_delta = wxPoint(dx, dy);

    Refresh(false);

#endif

#if USE_ADVANCED_SELECTION_MODE

    if( !m_isSelected ) {

        if( !evt.ShiftDown() ) {

            wxVariantHash hash;
			long id = GetId();
            hash[_T("groupname")] = m_props.groupName;
            hash[_T("cmd")] =  _T("gauge_click");
            hash[_T("sender_id")] = id;
            hash[_T("target_id")] = _T("0");

            m_groupManager->ProcessWidgetCommand(hash);

        }

        m_isSelected = true;
        DrawSelectionBitmap(this);

    } else {

        if( evt.ShiftDown() ) {

            m_isSelected = false;
            Refresh(false);

        }

    }

#endif

}


void VemsWidgetBase::MoveTarget(const wxPoint& delta)
{
    GetTargetWin()->Move( GetTargetWin()->GetPosition() + delta );

    if(m_winTip) m_winTip->ShowMsg(wxString::Format(_T("xpos=%i\nypos=%i"),GetTargetWin()->GetPosition().x,GetTargetWin()->GetPosition().y));
}

void VemsWidgetBase::OnMouseMove(wxMouseEvent& evt)
{

#if !defined (__WINCE_STANDARDSDK__)

    if( !(m_groupManager->IsEditMode()) ) {
        evt.Skip();
        return;
    }

    if(m_delta == wxDefaultPosition) {
        evt.Skip();
        m_parent->AddPendingEvent(evt);
        return;
    }

    if (evt.Dragging())
    {
        wxPoint pt = evt.GetPosition();

        wxPoint pos = ClientToScreen(pt);

        wxPoint cpos = GetTargetWin()->GetPosition();

        GetTargetWin()->Move(wxPoint(pos.x - m_delta.x, pos.y - m_delta.y));

        if(m_winTip) m_winTip->ShowMsg(wxString::Format(_T("xpos=%i\nypos=%i"),GetTargetWin()->GetPosition().x,GetTargetWin()->GetPosition().y));

        wxPointEx rdelta = GetTargetWin()->GetPosition() - cpos;

        wxVariantHash hash;
        hash[_T("groupname")] = m_props.groupName;
        hash[_T("cmd")] =  _T("gauge_move");
        hash[_T("sender_id")] = (long)GetId();
        hash[_T("target_id")] = _T("0");
        hash[_T("deltapos")] << rdelta;

        m_groupManager->ProcessWidgetCommand(hash);

    }


#endif

}

void VemsWidgetBase::OnLeftUp(wxMouseEvent& evt)
{

    if(GetShape().Contains(evt.GetX(),evt.GetY()) != wxInRegion ) {
        m_parent->ProcessEvent(evt);
        return;
    }

    SetFocus();

    if( !(m_groupManager->IsEditMode()) ) {
        evt.Skip();
        return;
    }


#if !defined (__WINCE_STANDARDSDK__)

    if (HasCapture())
    {
        ReleaseMouse();
        //printf("Mouse released\n");
        m_isMouseResize = false;
    }

#endif

    //if(m_winTip) m_winTip->HideWin();

    m_parent->ProcessEvent(evt);

    m_delta = wxDefaultPosition;

    Refresh(false);
}

void VemsWidgetBase::OnDoubleClick(wxMouseEvent& evt)
{

    m_parent->ProcessEvent(evt);

}

void VemsWidgetBase::OnMouseRightClick(wxMouseEvent& event)
{

    if(GetShape().Contains(event.GetPosition()) != wxInRegion ) {
        m_parent->ProcessEvent(event);
        return;
    }


    //Logger::logf(LOG_MSG,_T("VemsWidgetBase::OnMouseRightClick %i"),GetItemID());

	if (FindFocus() != this)
	{
        SetFocus();
	}

	const wxPoint& pos = event.GetPosition();

    wxMenu popupMenu;
    createBasicMenu( popupMenu );
    createWidgetMenu( popupMenu );
    if( m_groupManager && m_groupManager->IsEditMode() ) createEditMenu( popupMenu );

    PopupMenu( &popupMenu , pos.x, pos.y);

	event.Skip();
}


int VemsWidgetBase::compare_function(Symbol **A,Symbol **B)
{
    wxString titleA = (*A)->getTitle();
    wxString titleB  = (*B)->getTitle();

    return titleA.CmpNoCase(titleB);
}

int VemsWidgetBase::MenuListCompFunc( const void *arg1, const void *arg2)
{
    wxString titleA = ((wxMenuItem*)arg1)->GetLabel();
    wxString titleB = ((wxMenuItem*)arg2)->GetLabel();

    return titleA.CmpNoCase(titleB);
}

void VemsWidgetBase::OnThemeSelect(wxCommandEvent& event)
{
    wxMenu * menu = (wxMenu*)event.GetEventObject();
    wxString name = (menu->FindItem(event.GetId()))->GetItemLabel();


    SetNewTheme( SearchTheme(name) );

    m_props.themeName = name;

    m_groupManager->SaveTheme( GetItemID(), name );

    m_isChanged = true;
}

wxMenu* VemsWidgetBase::CreateDuplicateMenu(DuplicateMenuItemDescArray &items, wxEvtHandler * target, wxObjectEventFunction func)
{
    wxMenu * menu = new wxMenu();

    items.Clear();

    items.Add(DuplicateMenuItemDesc(wxNewId(), 1, 0,_T("rects_right.png"),_T("Right")));
    items.Add(DuplicateMenuItemDesc(wxNewId(), 0, 1,_T("rects_bottom.png"),_T("Bottom")));
    items.Add(DuplicateMenuItemDesc(wxNewId(),-1, 0,_T("rects_left.png"),_T("Left")));
    items.Add(DuplicateMenuItemDesc(wxNewId(), 0,-1,_T("rects_top.png"),_T("Top")));
    items.Add(DuplicateMenuItemDesc(wxNewId(),-1,-1,_T("rects_left-top.png"),_T("Left-top")));
    items.Add(DuplicateMenuItemDesc(wxNewId(),-1, 1,_T("rects_left-bottom.png"),_T("Left-bottom")));
    items.Add(DuplicateMenuItemDesc(wxNewId(), 1,-1,_T("rects_right-top.png"),_T("Right-top")));
    items.Add(DuplicateMenuItemDesc(wxNewId(), 1, 1,_T("rects_right-bottom.png"),_T("Right-bottom")));


    for(size_t a=0;a<items.GetCount();a++) {
        menu->Append( items[a].Init(menu) );

        target->Connect(items[a].GetId(), wxEVT_COMMAND_MENU_SELECTED, func );
    }


    return menu;
}

DuplicateMenuItemDesc::DuplicateMenuItemDesc(int id, int xrep, int yrep, const wxString& pngfile, const wxString& text)
{
    Id = id;
    XRep = xrep;
    YRep = yrep;

    m_text = text;
    m_pngPath = DirHelper::GetPath(dICON) + pngfile;
}

wxMenuItem* DuplicateMenuItemDesc::Init(wxMenu * parentmenu)
{

	wxMenuItem * mitem = new wxMenuItem(parentmenu, Id,_T("label"));
#if defined(__WXMSW__) && !defined(__WINCE_STANDARDSDK__)
    mitem->SetBitmaps(wxBitmap(m_pngPath, wxBITMAP_TYPE_PNG ));
#endif
	mitem->SetText(m_text);

    return mitem;
}

wxPoint DuplicateMenuItemDesc::RePos(const wxPoint& oripos, const wxSize &size) const
{
    wxPoint newpos;
    newpos.x = oripos.x + size.x * XRep;
    newpos.y = oripos.y + size.y * YRep;
    return newpos;
}


void VemsWidgetBase::createSymbolMenu( wxMenu& popupMenu )
{
    if ( !m_isSymbolMenu ) return;

    wxMenu* changeGaugeDataMenu = new wxMenu();

    m_symbolDescArr.Clear();
    m_menuId2SymbolName.clear();

    m_gaugeDescMgr->GetSymbolDescs( m_symbolDescArr );

    wxSortedArrayString grparr;

    m_gaugeDescMgr->GetSymbolGroupListM(grparr);

    for(size_t a=0; a < grparr.GetCount(); a++){

        wxString grpname = grparr[a];

        wxMenu * sgmenu = new wxMenu();

        SymbolGaugeDescArray arr;

        m_gaugeDescMgr->GetSymbolDescs(grpname, arr);

        for(size_t i = 0; i < arr.GetCount(); i++ ) {

            SymbolGaugeDesc * desc = arr[i];

            if(i%20 == 0) sgmenu->Break();

            int id = wxNewId();

            sgmenu->AppendCheckItem(id, desc->Title );

            m_menuId2SymbolName[id] = desc->Name;

            Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(VemsWidgetBase::OnPopupMenuSelectSymbol) );

        }

        changeGaugeDataMenu->AppendSubMenu(sgmenu,grpname);
    }


    for ( size_t i = 0; i < m_symbolDescArr.GetCount(); i++ ){
        choices.Add(m_symbolDescArr[i]->Title);
        choices2gaugeDescA.Add(i);
    }

    int cid=0;
    if(GetCurrSymbol()) cid = changeGaugeDataMenu->FindItem(GetSymbolTitle());

	if(cid && cid!=-1)
		changeGaugeDataMenu->Check(cid, true);

    popupMenu.Append(wxID_ANY, _T("Change Gauge Data To"), changeGaugeDataMenu);
}

void VemsWidgetBase::createGaugeDescMenu( wxMenu& popupMenu )
{
    wxMenu* changeGaugeDescMenu = new wxMenu;

    if(m_isGaugeDescMenu) {
        for(size_t i = 0; i< m_gaugeDescA->Count(); i++){
            //addGDMenuItem( i );
            if(!(i%20)) changeGaugeDescMenu->Break();

            VEMSGaugeDesc *gd = m_gaugeDescA->Item(i);

            int id = wxNewId();

            wxMenuItem* menuItem = changeGaugeDescMenu->AppendRadioItem(id, gd->GetName());

            bool isEnabled = gd->GetType() == m_gaugeDescMgr->GetGaugeSymbolType( GetCurrSymbolName() );

            m_menuId2GaugeId[id] = i;

            if ( isEnabled ) Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(VemsWidgetBase::OnPopupMenuSelectGauge) );

            menuItem->Enable( isEnabled );
            if ( i == m_defGaugeId ) menuItem->Check();
        }
    }

    if(m_isGaugeDescMenu) {
        popupMenu.Append(wxID_ANY, _T("Change Gauge Descriptor To"), changeGaugeDescMenu);
        int idgd = wxNewId();
        popupMenu.Append(idgd, _T("Modifiy Descriptor"));
        Connect(idgd , wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(VemsWidgetBase::OnGaugeDescMod) );
    }
}

void VemsWidgetBase::createBasicMenu( wxMenu& popupMenu )
{
    createSymbolMenu( popupMenu );
    createGaugeDescMenu( popupMenu );

    if(m_isGaugeDescMenu || m_isSymbolMenu) {
        popupMenu.AppendSeparator();
    }

    if( m_isThemeMenu ) {

        wxMenu* themesMenu = ThemesHelper::CreateThemeMenu(m_themes, this, wxCommandEventHandler(VemsWidgetBase::OnThemeSelect),m_theme->Name);
        popupMenu.Append(wxID_ANY, _T("Themes"), themesMenu );

    }

    if( GetItemID() != -1 )
    {
        popupMenu.SetTitle( m_props.groupName + _T(" (hotkey: ") + m_hotKeyStr + _T(")") );
    }
    else
    {
        popupMenu.AppendSeparator();
        popupMenu.Append(GAUGE_QUIT_ID_AS, _T("Close window"));
    }
}

void VemsWidgetBase::createEditMenu( wxMenu& popupMenu )
{
    wxMenu * dpmenu = NULL;
    wxMenuItem * focusmenu = NULL;

    if(m_isThemeMenu)
        popupMenu.AppendSeparator();

    if( GetItemID() != -1 )
    {

        popupMenu.AppendSubMenu( createAddSubMenu(), _T("add") );
        popupMenu.Append( createRemoveMenu(&popupMenu) );
        popupMenu.Append( createSaveMenu(&popupMenu) );

        dpmenu = CreateDuplicateMenu(m_dMenuItems,this,wxCommandEventHandler(VemsWidgetBase::OnDuplicateMenu));

        focusmenu = createFocusMenu();
    }
    else
    {
        popupMenu.Append(GAUGE_QUIT_ID_AS, _T("Close window"));
    }

    if(focusmenu){

        popupMenu.AppendSeparator();
        popupMenu.Append( focusmenu );

        if( m_groupManager->GetFirstFocusedWinID() == m_winID ) focusmenu->Check(true);
    }

    popupMenu.AppendSeparator();
    popupMenu.Append(GAUGE_SETLAY_ID, _T("Set Size and Positons"));

    if(dpmenu) {
        popupMenu.AppendSeparator();
        popupMenu.AppendSubMenu(dpmenu,_T("Duplicate to..."));
    }
}

void VemsWidgetBase::OnDuplicateMenu(wxCommandEvent& event)
{
    for(size_t a=0;a<m_dMenuItems.GetCount();a++) {
        if( m_dMenuItems[a].GetId() == event.GetId() )
            DulpicateWidget( m_dMenuItems[a] );
    }
}

void VemsWidgetBase::DulpicateWidget(const DuplicateMenuItemDesc& desc)
{
    wxPoint pos = desc.RePos( GetTargetWin()->GetPosition(), GetTargetWin()->GetSize() );

    GaugeProperties prop;

    prop.groupName = m_props.groupName;
    prop.setPos( pos );
    prop.setSize( GetSize() );

    prop.symbolName     = GetCurrSymbolName();
    prop.gaugeType      = m_props.gaugeType;

    prop.themeName      = m_theme->Name;
    prop.descName       = m_gd->GetName();

    //Logger::logf(LOG_MSG,_T("desc=")+prop.descName + _T(", symbol=")+prop.symbolName);

    m_groupManager->DoAddGauge( prop );
}

void VemsWidgetBase::RedrawWidget()
{
    UpdateSize();
}

void VemsWidgetBase::ShowDescEditor(VEMSGaugeDesc* gd)
{
    if(m_gaugeDescEditor)
        m_gaugeDescEditor->ShowEditor(gd);

}

void VemsWidgetBase::OnGaugeDescMod(wxCommandEvent& event)
{
    ShowDescEditor();
}

void VemsWidgetBase::OnPopupMenuSelectGauge(wxCommandEvent& event)
{

    int id = event.GetId();


    SetDefaultGaugeDesc(m_menuId2GaugeId[id]);

    UpdateSize();

    Refresh(false);

    //Logger::logf(LOG_MSG,_T("VemsWidgetBase::OnPopupMenuSelectGauge"));

    m_props.descName = m_gd->GetName();

    m_groupManager->SaveDesc( GetItemID(), m_gd->GetName() );
}

int VemsWidgetBase::SrcGaugeDescBySymbolName(const wxString &name)
{

    for(uint16_t i = 0; i< m_gaugeDescA->Count(); i++){

        if( m_gaugeDescMgr->GetGaugeSymbolType( name ) == m_gaugeDescA->Item(i)->GetType() ) return i;

    }

    return 0;

}

void VemsWidgetBase::OnPopupMenuSelectSymbol(wxCommandEvent& event)
{
    int id  = event.GetId();

    if(m_menuId2SymbolName.find(id) == m_menuId2SymbolName.end()){
            return;
    }

    SetCurrSymbolName(m_menuId2SymbolName[id]);

    m_defGaugeId = SrcGaugeDescBySymbolName( GetCurrSymbolName() );

    m_gd = m_gaugeDescA->Item(m_defGaugeId);

    if( m_prevSymbolName != GetCurrSymbolName() ) { m_isSymbolChanged = true; }
    m_prevSymbolName = GetCurrSymbolName();


    m_gd->SetGaugeNumbers();
    requireChanged();
	UpdateSize();

	//TEST
	m_parent->SetName( GetCurrSymbolName() );
	m_parent->Refresh(false);

    m_groupManager->SaveSymbolname( m_props.itemID, GetCurrSymbolName() );

    m_props.descName = m_gd->GetName();

    m_groupManager->SaveDesc( GetItemID(), m_gd->GetName() );

    SetValue(0);

}

void VemsWidgetBase::SetGaugeDescA(const VEMSGaugeDescArray* gaugeDescA)
{
    m_gaugeDescA = gaugeDescA;
}

void VemsWidgetBase::SetGaugeDescByName(const wxString& descName)
{
    Symbol * cs = GetCurrSymbol();

    if( !cs ) return;

    if(descName == DEFAULTDESC) {
        SetGaugeBySymbolName(cs->getName());
        return;
    }

    for(size_t i=0; i < m_gaugeDescA->GetCount(); i++){

        if( (m_gaugeDescA->Item(i)->GetName() == descName) && (m_gaugeDescA->Item(i)->GetType() == m_gaugeDescMgr->GetGaugeSymbolType(GetCurrSymbolName()) ) ) {
            SetDefaultGaugeDesc(i);
            return;
        }

    }

    StringHash  params;

    params[_T("descname")] = descName;
    params[_T("gauge_id")] = wxString::Format(_T("%i"),m_props.itemID);

    if(m_hasOwnDescriptorMgr) return;
    UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("gdesc_not_found_2"),&params), ULOGLS_MEDIUM, SPC_NO_DUPLICATE_LINES  );

    SetDefaultGaugeDesc(0);

}

void VemsWidgetBase::SetGaugeBySymbolName(const wxString symbolName)
{


    SetCurrSymbolName(symbolName);

    Symbol *currsymbol = GetCurrSymbol();

    //Logger::logf(LOG_MSG,_T("VemsWidgetBase::SetGaugeBySymbolName s=%p, m_currentSymbolName=")+m_currSymbolName,currsymbol);


    if( !currsymbol ) {

        //Logger::logf(LOG_MSG,_T("m_gaugeDescA=%p"),m_gaugeDescA);

        m_gd = m_gaugeDescA->Item(0);
        return;
    }

    for(size_t i=0; i < m_gaugeDescA->GetCount(); i++){

        if( m_gaugeDescA->Item(i)->GetType() != m_gaugeDescMgr->GetGaugeSymbolType( GetCurrSymbolName() ) ) continue;

        SetDefaultGaugeDesc(i);
        return;
    }

    if(m_hasOwnDescriptorMgr) return;
    UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("gdesc_not_found_1")), ULOGLS_MEDIUM, SPC_NO_DUPLICATE_LINES  );

    SetDefaultGaugeDesc(0);

}

void VemsWidgetBase::SetDefaultGaugeDesc(size_t id)
{
    if( id > m_gaugeDescA->Count()-1 ){ id = 0;}
    m_defGaugeId = id;
    m_gd = m_gaugeDescA->Item(id);
    m_gd->SetGaugeNumbers();
}


bool VemsWidgetBase::isDefaultGaugeDesc()
{

    if(!m_gd) return true;

    wxString gtype = m_gaugeDescMgr->GetGaugeSymbolType( GetCurrSymbolName() );

    for(size_t i=0; i < m_gaugeDescA->GetCount(); i++) {
        if( gtype != m_gaugeDescA->Item(i)->GetType() ) continue;

        if( m_gd->GetName() == m_gaugeDescA->Item(i)->GetName() )
            return true;
        else
            return false;

    }
    return false;
}

wxString VemsWidgetBase::GetSelectedGaugeDesc()
{
    if(!m_gd) return wxEmptyString;

    return m_gd->GetName();

}

void VemsWidgetBase::OnQuit(wxCommandEvent& event)
{
    Logger::logf(LOG_MSG,_T("VemsWidgetBase::OnQuit()"));
    UnRegisterViewer();
    m_parent->Close(true);
}

void VemsWidgetBase::SetValueIdx(int idx)
{

    Logger::logf(LOG_MSG,_T("VemsWidgetBase::SetValueIdx recived index: %i"), idx );

    m_defGaugeId = choices2gaugeDescA.Item(idx);

    Logger::logf(LOG_MSG, _T("VemsWidgetBase m_defGaugeId: %i"), m_defGaugeId );

    //SetGaugeBySymbolName( m_symbolA[idx]->getName() );
    SetGaugeBySymbolName( m_symbolDescArr[idx]->Name );

    /*

    m_gd = m_gaugeDescA->Item(m_defGaugeId);
    m_gd->SetGaugeNumbers();
*/
    requireChanged();

    UpdateSize();
	Refresh(false);

}

void VemsWidgetBase::require()
{
    wxString currentSymbol = GetCurrSymbolName();

    if( m_isSymbolChanged )
    {
        m_rtDataMgr->unregisterSymbol( m_prevSymbolName );
        m_rtDataMgr->registerSymbol( currentSymbol );
        m_isSymbolChanged = false;
    }
}

wxString VemsWidgetBase::GetSymbolTitle()
{
    if(m_gd) {
        if(m_gd->Properties[_T("CustomSymbolTitle")].GetString().Len())
            return m_gd->Properties[_T("CustomSymbolTitle")].GetString();
    }

    return m_gaugeDescMgr->GetGaugeSymbolTitle( GetCurrSymbolName() );
}


void VemsWidgetBase::unregisterSymbols()
{
    //Logger::logf( LOG_MSG, _T("VemsWidgetBase::unregisterSymbols: ") + GetCurrSymbolName() );
    m_rtDataMgr->unregisterSymbol( GetCurrSymbolName() );
}

void VemsWidgetBase::refresh(DoubleHash* table)
{

    //Logger::logf(LOG_MSG,_T("m_currSymbolName=")+GetCurrSymbolName()+_T(", itemid=%i"),GetItemID() );
    if( m_groupManager->IsEditMode() ) return;

    if(table->find(GetCurrSymbolName()) == table->end()){
        return;
    }

    double value = (*table)[GetCurrSymbolName()];

    m_isDataAvailable = value > DATA_IS_NOT_AVAILABLE;

    SetValue( value );

/*
    UpdateViewerEvent event;
    event.m_userData[_T("data")] = ;
    AddPendingEvent( event );
    */
}

wxWindow * VemsWidgetBase::GetTargetWin()
{
    if(m_floating)
        return this;
    else
        return m_parent;
}

void VemsWidgetBase::ProcessWidgetCommand(wxVariantHash& cmdHash)
{
//    LOGP(wxString::Format(_T("cmdhash size=%i"),cmdHash.size()));

    if( cmdHash.find(_T("groupname")) == cmdHash.end() || cmdHash[_T("groupname")].GetString() != m_props.groupName) return;

    if(cmdHash[_T("target_id")].GetLong() == GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_select")) {
        m_isSelected = true;
        DrawSelectionBitmap(this);
        return;
    }

    if(cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_move") && m_isSelected) {
        wxPointEx delta = wxVariantHelper::GetPointEx(cmdHash[_T("deltapos")]);
        MoveTarget( delta );
        return;
    }

    if(cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_size") && m_isSelected) {
        wxSizeEx delta = wxVariantHelper::GetSizeEx(cmdHash[_T("deltasize")]);
        GetTargetWin()->SetSize( GetTargetWin()->GetSize() + delta );
        return;
    }

    if(cmdHash[_T("cmd")].GetString() == _T("gauge_deselect") && m_isSelected) {

        m_isSelected = false;

        Refresh(false);

        return;
    }

    if( cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_click") ) {

        m_isSelected = false;

        Refresh(false);

        return;
    }
}

void VemsWidgetBase::OnSizeBase(wxSizeEvent& evt)
{
    if(m_isSelected)
        DrawSelectionBitmap(this);

    evt.Skip();
}

void VemsWidgetBase::OnChar(wxKeyEvent& event)
{
   //Logger::logf(LOG_MSG,_T("VemsWidgetBase::OnChar() keyCode: %d"), event.GetKeyCode() );

//    Logger::logf(LOG_MSG,_T("event kc=%i,isshift=%i"),event.GetKeyCode(),event.ShiftDown());

    //Logger::logf(LOG_MSG,_T("kss=")+wxKeyBind::GetKeyStrokeString(event));

    int code = event.GetKeyCode();

    if(code == WXK_ESCAPE )
    {
       wxWindow* grandParent = GetGrandParent();
       if( grandParent )
       {
            grandParent->SetFocus();
            //grandParent->GetEventHandler()->ProcessEvent( event );
            //m_parent->Refresh();
       }

    } else if( code == WXK_F11 ){  // F11
        ResetMinMax();

    } else if( code == WXK_F12 ){  // F12
        if(m_groupManager)
            m_groupManager->ResetGaugesMinMax( m_props.groupName );

    } else {

        if( m_groupManager->IsEditMode() ) {

            WindowHelper whe(GetTargetWin(), m_winTip);
            whe.SetSideRatioRestriction( m_sideRatioRestriction );

            int ares = whe.Action(event);

            whe.GroupAction(m_groupManager,ares,m_props.groupName, true);
        }

    }

    event.Skip();
}

void VemsWidgetBase::OnUpdateValue( UpdateViewerEvent& event )
{
    double value = event.m_userData[_T("data")].GetDouble();

    m_isDataAvailable = value > DATA_IS_NOT_AVAILABLE;

    SetValue( value );
}

Symbol * VemsWidgetBase::GetCurrSymbol()
{
    //Symbol * s = m_groupManager->getMTConfigData()->getSymbol( m_currSymbolName );
    Symbol * s = m_gaugeDescMgr->getMTConfigData()->getSymbol( GetCurrSymbolName() );

    StringHash hash;
    hash[_T("symbol")] = GetCurrSymbolName();

    if(!s && GetGaugeType() != GaugeType::EXPRESSION_GRID )
        UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("symbol_not_found_1"),&hash) , ULOGLS_MEDIUM, SPC_NO_DUPLICATE_LINES );

    return s;
}

int VemsWidgetBase::LoadSettingsBase(wxVariantHash &datas)
{
    if(GetItemID() > 0)
        return m_gaugeDescMgr->GetGaugeSettings( m_name, m_props.itemID, datas );
    else {
        ConfigBase* config = ConfigHelper2::getConfig();
        config->setPath(_T("/")+m_name+_T("/"));

        long dummy      = 0;
        wxString entry  = wxEmptyString;
        wxString svalue  = wxEmptyString;
        long lvalue;
        bool bvalue;
        double dvalue;


        bool bCont = config->getFirstEntry( entry, dummy );

        while(bCont) {
            switch( config->getEntryType(entry) ) {
                case ConfigBase::Type_Boolean:
                    config->read(entry,&bvalue);
                    datas[entry] = bvalue;
                    break;

                case ConfigBase::Type_Float:
                    config->read(entry,&dvalue);
                    datas[entry] = dvalue;
                    break;

                case ConfigBase::Type_Integer:
                    config->read(entry,&lvalue);
                    datas[entry] = lvalue;
                    break;

                case ConfigBase::Type_String:
                case ConfigBase::Type_Unknown:
                    config->read(entry,&svalue);
                    datas[entry] = svalue;
                    break;
            }

            bCont = config->getNextEntry( entry, dummy );
        }

        return 0;
    }

}

int VemsWidgetBase::SaveSettingsBase(wxVariantHash &datas)
{
    if(GetItemID() > 0)
        return m_gaugeDescMgr->SaveGaugeSettings( m_name, m_props.itemID, datas );
    else {
        ConfigBase* config = ConfigHelper2::getConfig();
        config->setPath(_T("/")+m_name+_T("/"));

        for(wxVariantHash::iterator it = datas.begin(); it != datas.end() ; it++ ){
            config->write( it->first, it->second.MakeString() );
        }
        config->save();
        return 0;
    }
}


void VemsWidgetBase::ResetMinMax()
{
    m_valueMin = m_value;
    m_valueMax = m_value;
    m_isValueMaxChanged = true;
    m_isValueMinChanged = true;
    Refresh(false);
}


/** \~english
  * \page vwbover VemsWidgetBase overwiev
  * Creating a new VemsWidgetBase based widget: \n
  *
  *
  *
  *
  *
  *
  *
  * \~hungarian
  * \page vwbover VemsWidgetBase körültekintés
  *
  *
  *
  *
  *
  *
  *
  *
  */
