#!/usr/bin/perl -w

# MegasquirtAvr 2003
# speedy firmware-upload 
# Michael Kristensen mik at caffrey.dk and Marcell Gal cell at x-dsl.hu

$stats={
goodpage_verify=>0
};

$m128={
type=>'m128',
str=>'AVREFI1',
erase_cmd=>'e',
pagesizeb=>256,
pagecount=>512,
blspagecount=>8
};	# atmega128

$m168={
type=>'m168',
str=>'AVR2',
erase_cmd=>'S',	# not real erase, just dummy ping
pagesizeb=>'128',
pagecount=>128,
blspagecount=>16
};

$avr_types=[ $m128, $m168 ];

$avr_type = $m128;

$origbaud = 9600;	# old default

$crypto = undef;


# m128, or m168
# AVREFI1 or AVR2
# tries to find a matching device according to string returned by the bootloader,
# or device name
sub set_devicetype
{
	my $whatever = shift;
	my $t;

	foreach $t ( @$avr_types ){
		 my @keys = keys %$t;
		 my $k;
		 foreach $k (@keys){
			if( $whatever eq $t->{$k} ){
				print "Setting AVR Type to $t->{type}, pagesizeb= $t->{pagesizeb}\n";
				$avr_type = $t;
				return;
			}
		}
	}
}

# returns page size in bytes
sub get_pagesizeb()
{
	return $avr_type->{pagesizeb};
}

if (@ARGV < 1) {
    usage();
}

sub usage
{
    print "usage:\n";
    print "\tperl bin/prog.pl vems.hex [flagword] [speedword] [:port]\n";
    print "\t-o : origbaud: baudrate for t command: -o19200\n";
    print "\t-p : -pm128 is atmega128, -pm168 is atmega168 (like in avrdude)\n";
    print "\t-r : -r0x3F80 refuses to upload a page with address 0x3F80 or higher\n";
    print "\n[flagword] (case sensitive)\n";
    print "\tw : write flash (otherwise only verify). This implies 'E'\n";
    print "\tm : write marker word as well (only makes sense together with 'w'\n";
    print "\td : dump config to mcd.txt and tables to mct.txt\n";
    print "\tq : issue mst..msp..msq..  stimout command for testing\n";
    print "\tt : talk to vems (application) first, issuing the menucommand to drive it to bootloader\n";
    print "\tf : when talking to vems (application), forcing bootloader even if RPM>0\n";
    print "\tE : exit bootloader at the end (try to start msavr app.)\n";
    print "\te : bootloader does autoerase (=>we do NOT apply erase)\n";
    print "\ts : save serial configuration file (you can attach it to problem-question if you have problems)\n";
    print "\n[speedword]\n";
    print "\t19200, 38400, 57600, 115200\n\n";
	print "[:port]\n";
	print "\t:COM1, :/dev/ttyS0 (alternatively env.variable set COM_PORT=COM3)\n\n";

    undef $ob if defined $ob;
    exit;
}

use vars qw( $OS_win $ob $port );

$maxerr = 5;
$bootl_speed = 19200;
$timeout = 200;

BEGIN {
$OS_win = ($^O eq "MSWin32") ? 1 : 0;
if ($OS_win) {
  eval "use Win32::SerialPort";
  die "$@\n" if ($@);
} else {
  eval "use Device::SerialPort";
  die "$@\n" if ($@);
}
} # BEGIN


# Install like this: perl -MCPAN -e 'install Device::SerialPort'
# If you are running Windows 95 or later, you want the Win32::SerialPort
# module instead of this one. It has a compatible user interface.


# example ihex:
# http://www.termsys.demon.co.uk/pdf/ihex.pdf
# :02 0000 02 1000 EC
# :10 F800 00 0C9446FC0C9463FC0C9463FC0C9463FC 19
# :10F810000C9463FC0C9463FC0C9463FC0C9463FCEC
# :10F820000C9463FC0C9463FC0C9463FC0C9463FCDC
# ...
# :10FEC000DF80EE80FD800C811B81AA81B981CE0F7D
# :10FED000D11D0FB6F894DEBF0FBECDBFED01089562
# :04FFFC000C9400FC65
# :040000031000F800F1
# :00000001FF

# datarecord: ":llaaaattdddddd...dddddcc"
#             - ll -> length of databytes (ddd...ddd), 2 Hex-digits
#             - aaaa -> startadress of EPROM where databytes of this record will be programmed
#             - tt -> type of record: data is 00
#             - ddd...ddd -> ll-sized sequence of bytes programmed at adress aaaa
#             - cc -> checksum plus llaaaattddd...ddd must be zero.
# endrecord: ":00000001FF"

# Types:
# 00 data
# 01 EOF
# 02 Extended Segment address
# 03 start segment address
# 04 extended linear address (not relevant)
# 05 start linear address (not relevant)

$he = '[a-fA-F0-9]';
$offset = 0;

################      app_init         #########################
	process_param();
# check if the hex file is there
	if( ! hexfile_empty() ){
		open HEXF, "<$hexfile" or die "ERROR, could not open '$hexfile' for reading: $!";
	}
	if( !defined $port ){
		# another chance, to define via environment variable
		if (defined $ENV{COM_PORT} ) {
			$port = $ENV{COM_PORT};
		}
	}

	if($OS_win){
		$port = 'COM1' if !defined $port;
		$ob = Win32::SerialPort->new ($port);
	} else {
		$port = '/dev/ttyS0' if !defined $port;
		$ob = Device::SerialPort->new ($port);
	}


serial_init();


###############################################################

sub serial_init
{
# found on http://members.aol.com/Bbirthisel/alpha.html
# should work on multiple platforms
$confok=0;

$Configuration_File_Name = 'etc/serial.cfg';

# did we succeed?
die "Can't open serial port $port: $^E\n" unless ($ob);

# save the state of the comm port for later
$old_baud = $ob->baudrate;
$old_parity = $ob->parity;
$old_data = $ob->databits;
$old_stop = $ob->stopbits;
$old_hshake = $ob->handshake;

# try using saved configuration
if ($confok){
    print "trying to use config: $Configuration_File_Name\n";
    $ob->restart($Configuration_File_Name) || warn "Can't read $Configuration_File_Name: $!\n";
}

$ob->baudrate($bootl_speed);
$ob->parity('none');
$ob->databits(8);
$ob->stopbits(1);
$ob->handshake('none');
$ob->binary(T);

# without this communication returned too few bytes sometimes
$ob->read_interval(100) if ($OS_win);

$ob->write_settings || undef $ob;

# you need this initially to make your configfile:
$ob->save($Configuration_File_Name) if defined $saveconf;

$tmout = $timeout;
# ok, now port is configured
} # serial_init
##############################################################################
##############################################################################

if(defined $talk2ms){
# say init string to avr (maybe it listens)
# avoid 'm' character, it's destrcutive

    $ob->baudrate($origbaud);
    $ob->read_const_time(500);

	msp();
	if ( dumpmct() ){
		exit 0;
	}

    my $initstr = "Mancb". avrcommspeed() . "cb"; # for 19200 ff would work too
    print "saying $initstr at baud=$origbaud\n";
    comm_send($initstr); # tell msavr to enter bootloader
# tell msavr to enter bootloader
    $b = comm_recv(6);

    if ($b =~ /Hello>/) {
        print "AVR said hello\n";
    } else {
        print "No hello from AVR\n";
    }

    sleep(1); # Let the LCD do the printing before trying to talk with it again
    $ob->lookclear();
    $ob->baudrate($bootl_speed); # bootloader speed
    $ob->read_const_time($timeout);
} # talk2ms

###############################################x

# communicate with bootloader here

bootl_sync();

print "we say S\n";
$ob->lookclear();
comm_send('S');
# atmega128 old returned AVREFI1
# atmega168 returns AVR2
my $t=srl_wait(7);
if($t =~ /(AVR\S+)/){
	set_devicetype( $1 );
} else {
	print "Sorry, device does not reply AVR... to S, instead:$t\n";
	undef $ob if defined $ob;
	exit -2;
};

#$ob->transmit_char(ord('v'));
#syswrite SRL, $sendarr, 1, 0;

print "we say v\n";
comm_send('v');
srl_wait(2);

print "we say V\n";
comm_send('V');
srl_wait(2);

$allerrcnt = 0;

if(defined $write2){

	# WRITE mode !!!
	make_marker_word( undef );
	flash_write();
	flash_verify();	# not strictly needed, it's not a huge problem if marker word stays in
	read_hex();
	make_marker_word( 1 );
	flash_write();

} # end of writemode

	if( !defined $hex_read){
	 	read_hex();	 	
	}

	make_marker_word( 1 );
	flash_verify();

print " ========= STATS: ======= \n";
print_hash( $stats );
print " ======================== \n";

exitbootif();

# ===============
sub hexfile_empty
{
	return $hexfile eq 'FFFF';
}

# ==============================
# returns undef if address is allowed
# and memorizes address in this case
sub page_address_disallowed
{
	my $addrb = shift;


	# the default limit for the device:
	my $limit = bls_start_address();
	if( defined $page_address_limit ){
		# user overrided limit (with -r0x....)
		$limit = $page_address_limit ;
	}

	if( $addrb >= $limit ){
		printf("\nNot verify from address 0x%x  (-r0x%x in effect)\n", $addrb, $limit );
		return 1;
	}
	
	
	# global!!!
	$last_flash_address = $addrb;
	return undef;
}
# ===
sub flash_verify
{
	print "VERIFYING...\n";

	$allerrcnt = 0;
	my @pagelist = get_flash_pagelist();
	foreach $addrb ( @pagelist ){
		if( page_address_disallowed( $addrb )){
			last;	# since addrb is sorted, this is same as next
		}
		
		address_tell($addrb);
		$ob->lookclear();
		comm_send( getCommandDataRead() );
		pad_ff_ckksum($addrb);
		if($e=cksum_err()){
			$allerrcnt++;
			printf("error at page 0x%x (pgidx: %d)\n %s \n", $addrb, $addrb/get_pagesizeb(), $e  );
			if($allerrcnt > $maxerr){
				print "\nTOO many errors, bailing out\n";
				last; # we still want the exitbootif()
			}
		} else { # successful page-verify
			$stats->{'goodpage_verify'}++;
		}
	}
	print "Verified\nTOTAL serious errors at verify: $allerrcnt\n";
	if ($allerrcnt == 0) {
	    print "SUCCESS\n";
	} else {
	    print "FAILURE\n";
	}
} # sub flash_verify

################################################################
sub read_hex
{
	# flag that we have been called (globally available)
	$hex_read = 1;
if( hexfile_empty() ){
	return;
}

$minidx = 0xffffff00;

while($l = <HEXF>) {
    chomp $l;
	
	if (($crypto, $len,$adr,$type,$data,$crypto_checksum) = $l =~ /^(C)?:($he{2})($he{4})($he{2})($he*)$he{2}(:$he{8})?/){
		
# ($l =~ /^:/) 
# (($len,$adr,$type,$data,$chk) = $l =~ /^:($he{2})($he{4})($he{2})($he*)($he{2})/) 
#	    print " $crypto : len = $len : adr = $adr, type = $type, data = $data, chk = $chk\n";
#	    $chk++; # get rid of that warning

	    if (length($data) != 2*hex($len)) {
		print "length-field and actual length of data doesn't match\n";
		print "$l\n";
		undef $ob if defined $ob;
		exit;
	    }

# plenty of possiblities for optimizations

	    # prepare data for calculating the checksum
	    $l =~ /^C?:(.+)$/; # remove ':'
	    $chk_data = $1;
#	    print "$chk_data\n";

	    # calculate hexfile checksum
	    $chk_sum = 0;
	    for ($i=0; $i<(2*hex($len)+10); $i+=2) {
		$v = substr($chk_data, $i, 2);
#		print "$v\n";
		$chk_sum += hex($v);
	    }

	    $chk_sum %= 256;
	    if ($chk_sum != 0) {
	      print "Checksum error\n";
	      print "$l\n";
	      undef $ob if defined $ob;
	      exit;
	    }
	    # checksum correct

	    $tt{$type}++; # count the different numbers of types, debug only
	    if ($type eq "00") { # data
		
		my $baseaddr = hex($adr) + $offset;
		$minidx = $baseaddr if($baseaddr < $minidx);	# update minimum
		# place it in a datastructure; byte oriented, beware!

		set_flash_page_checksum( $baseaddr, $crypto_checksum );
		set_flash_page_crypto( $baseaddr, $crypto );
		for ($i=0; $i<hex($len); $i++) {
		    $v = substr($data, 2*$i, 2);
		    my $a = $baseaddr + $i;
		    set_flashvalue($a, hex($v));
#		    print "$a, $v\n";
		}
	    } elsif ($type eq "01") { # EOF
		# stop reading the hex file
		last;
	    } elsif ($type eq "02") { # Extended Segment address
		$offset = hex($data) * 16;
	    } elsif ($type eq "03") { # Start segment address
		print "type 3 isn't supported\n";
		# is it necessary?
	    } else {
		print "sorry, don't know what to with type ". hex($type) . "\n";
	    }

	} else {
	  print "can't parse this line: '$l'\n";
	}
} # while

foreach $n (keys %tt) {
    print "ihex type $n found $tt{$n} times\n";
}
print "ihex file read and found OK\n";

} # sub read_hex

# ====================================
sub flash_write
{
print "we are writing too, not only verify! Now say e\n";

if (! defined $autoerase ){
	comm_send($avr_type->{erase_cmd});
	srl_wait(1);
}

# $tmout = $timeout; # set it back
$tmout = 4000;

#$ob->transmit_char(ord('e'));
#&send(ord('e'));
#syswrite SRL, $sendarr, 1, 0;
#print SRL 'e';
#srl_wait();


# do we want 'A' command every time ? Does not take much time.
# theoretically hex gaps are handled fine

foreach $addrb ( get_flash_pagelist() ){
	if( page_address_disallowed( $addrb )){
		last;	# since addrb is sorted, this is same as next
	}
	print "\nuploading chunk $addrb:";

	my $maxpageerr = 2;
	my $pageerrcnt = 0;
	do {
		address_tell($addrb);
		pad_ff_ckksum($addrb);
		comm_send( getCommandDataWrite() );
		sendblock( $addrb );
		my $reply = srl_wait(1);
		if( $reply !~ /[\r]/ ){ # oops, some problem (note: \077 that is '?' not accepted)
			$pageerrcnt ++;
			if( $pageerrcnt == $maxpageerr ){
				printf("WARNING, serious error, not going back, received=%s (0x%02x, not 0x0d)\n", $reply, ord($reply));
				$allerrcnt ++;
				if( $allerrcnt > $maxerr){
					print "too many errors, bailing out\n";
					undef $ob if defined $ob;
					exit -3;
				}
				next;
			}
		} else { # successful page-write
			$stats->{'goodpage_written'}++;
			next;
		}
	} while ( $pageerrcnt < $maxpageerr );
} # foreach ... get_flash_pagelist()

print "\nTOTAL serious errors during write: $allerrcnt\n";
} # sub flash_write

#-----
sub print_hash
{
	my $href = shift;
	my $k;
	foreach $k ( keys %$href ){
		print "$k => $href->{$k}\n";
	}
}
##############################################################################
sub upload_bootloader_init
{
	print "we say mcWS\n";
	comm_send('mcWS');
	for ($i=0; $i<$maxerr; $i++) {
	    $r = srl_wait(1);
	    if ($r eq "S") {
		print "AVR in sync\n";
		last;
    		}
	}
	if ($r ne "S") {
	    print "didn't get reply from AVR, out of sync\n";
	    undef $ob if defined $ob;
  		exit -5;
	}

}

# ========================
# send p (many, if necessary) and keep receiving until we get a S
sub bootl_sync
{
	my $tmout_backup = $tmout;

	$tmout = 50;
	# print "we say p\n";
	comm_send('p');
	for ($i=260; $i>0; $i--){
	    my $r = srl_wait(1);
	    if ($r eq "S") {
		# print "AVR in sync\n";
		# restore from backup:
		$tmout = $tmout_backup ;
		return;
	    }
	}
    	print "didn't get reply from AVR, out of sync\n";
    	undef $ob if defined $ob;
    	exit -5;
}
##############################################################################

# sub serial_destroy

# restore state of comm port
$ob->baudrate($old_baud);
$ob->parity($old_parity);
$ob->databits($old_data);
$ob->stopbits($old_stop);
$ob->handshake($old_hshake);

undef $ob if defined $ob;

# ==========
sub set_address_limit
{
	my $num = shift;
	my $limit;

	if( $num =~ /^0x([0-9a-fA-F]+)$/ ){
		$limit = hex( $1 ); # substr( $num, 2, -1 );
	} elsif ( $num =~ /^\d+$/ ){
		$limit = $num;
	} else {
		print "Syntax error, bailing out (while limiting address to \"$num\")\n";
		exit -5;
	}

	$page_address_limit = $limit;

	print "Limiting address to $num (==$limit)\n";
}

#################################
sub process_param
{

#print "number of args:$#ARGV\n";
my $w;
	# number of args was checked earlier
	$hexfile = shift @ARGV;
	
	while($w = shift @ARGV) {
	if( $w =~ /^\d+$/){
		$bootl_speed = $w;
		print "serial bootloader speed set to $w\n";
		next;
	} elsif ($w =~ /^:(\S+)$/){
		$port = $1;
	} elsif ($w =~ /^-o(\d+)$/){
		$origbaud = $1;
	} elsif ($w =~ /^-p(m\S+)$/){
		set_devicetype( $1 );
	} elsif ($w =~ /^-r(\S+)$/){
		set_address_limit( $1 );
	} else { # flagword
		if($w =~ /w/){
			$write2 = 1 ;
			$exitboot = 1; # needed to write marker word
		}
		$talk2ms = 1 if $w =~ /t/;
		$exitboot = 1 if $w =~ /E/;
		$autoerase = 1 if $w =~ /e/;
		$saveconf = 1 if $w =~ /s/;
		$dumpmct = 1 if $w =~ /d/;
		$msp = 1 if $w =~ /q/;
		$marker_word = 0xA55A if $w =~ /m/;

		$bootl_forced = 1 if $w =~ /f/;	# undocumented: even with running trigger

		$w =~ tr/wmtEesfdp//d; # strip the good flags
		if ($w ne "") {
		    print "\nUnknown flag '$w'\n\n";
		    usage();
		}

	}
	} # while
}
##############
sub exitbootif
{
	return if ! defined $exitboot;
	print "we say E\n";
	comm_send('E');
	srl_wait(1);
}
##############
sub avrcommspeed
{
return 'fe' if defined $bootl_forced;
# calculates the number that should be written into the
# register of a 16MHz atmega
return sprintf("%02x", int(1000000/$bootl_speed - 0.5))

}
###############x
# returns 1 if there is an error
sub cksum_err
{
	my $st = srl_wait(4);

	my $pb = get_pagesizeb();

#	print "comparing:" . ord(substr($st,0,1)) ." to ". int($sendarr[$pb + 0]). " \n";
	
	my $h = ord(substr($st,0,1));
	my $l = ord(substr($st,1,1));
	
	my $s1 = $h . ',' . $l;
	
	if( isCrypto() ){
		printf( "Checksum received from bootloader: 0x%02x 0x%02x\n", $h, $l );
	} else {
		$s1 .= ','. ord(substr($st,2,1))
		.','. ord(substr($st,3,1));
	}
	
	my $s2 = int($sendarr[$pb + 0])
	.','.int($sendarr[$pb + 1]);
	
	if( ! isCrypto() ){
		$s2 .= ','.int($sendarr[$pb + 2])
		.','.int($sendarr[$pb + 3]);
	}
	
	return $s1. "\n" . $s2 if($s1 ne $s2); # problem
	return undef; # no problem

}
#######################
sub comm_send
{
    my $str = shift;
    my $offset = 0;
    my $count_out = 0;
    my $left = length($str);

    while ($left != 0) {
    	$count_out = $ob->write(substr($str, $offset, $left));
	if (!$count_out) {
		$count_out = 0;
	}
	$left -= $count_out;
	$offset += $count_out;
    }
    return length($str);
}

#######################
sub comm_recv
{
    my $b = shift;
    ($count_in, $string_in) = $ob->read($b);
    if( (!defined $count_in ) || ($count_in != $b) ){
    	warn "read unsuccessful b=$b count_in=$count_in\n";
    }
    return ($count_in, $string_in);
}

sub flush
{
	my $b = shift;
	$ob->read_const_time($tmout);
	($count_in, $string_in) = $ob->read($b);
	return ($count_in, $string_in);
}

#######################
# tell the bootloader what address we are at
sub address_tell
{
my $address_B = shift;

bootl_sync();

my $chunk = $address_B / 2; # we must tell the word address (not byte)
$chunk &= 0xFFFF;

my $str= 'A' . chr($chunk >> 8) . chr($chunk & 0xFF);
comm_send($str);
$ob->read_const_time($tmout);

# printf("communicated address: %04x\n", $address_B);
return srl_wait(1);
}

########################
sub sendblock
{
	my $addrb= shift;
		my $str = '';

		my $pb4 = 4 + get_pagesizeb();
		 # +4 is the 4 byte checksum
		my $i = 0;
		do{
			$str .= chr($sendarr[$i]);
			$i++;
		} while($i < $pb4 );

		open (COMMLOG, ">>/tmp/commlog.txt");

		my $s = sprintf( "0x%x", $addrb );
		my $padding = substr( '------------------------------', 0, 7 - length( $s ) );
		print COMMLOG $str . "\n---" . $s . "---\n";
		close COMMLOG;
		comm_send($str);
}
#################################################
# what's the reply???
sub srl_wait
{
	my $realcharcnt = shift;
	if (defined($realcharcnt)) {
	    $charcnt = $realcharcnt;
	} else {
	    print "ERROR CALLING srl_wait WITHOUT PARAM\n";
	}
# TODO: if no reply comes within 2 sec, we should start saying 'v' -s or sg.
# finally report the problem (or let the user ctrl-c)
	# $ob->read($in_);
	$ob->read_const_time($tmout);
	my ($cnt, $in_) = comm_recv($charcnt);
	#sysread (SRL, $in_, 1, 0) or die "cannot readback from SRL $!";
	
	if(defined $in_){
#		print "got back $cnt:$in_ :".ord(substr($in_, 0, 1));
#		print ' '.ord(substr($in_, 1, 1)) if(length($in_) > 1);
#		print "\n";
	} else {
		print "nothing\n";
	}
	
	return $in_ ;
}

##############################################
# returns pagesizeb + 4 bytes in global $sendarr[..] array
# from addr_offs (and 4 bytes checksum)
# (fills all, even if $flash ends earlier)

sub pad_ff_ckksum
{
	my $pagestart_addrb = shift; # pagestrat address in bytes

	my $pb = get_pagesizeb();

	my $ck1 = $pagestart_addrb / 2; # addressW in firmware
	my $ck2 = $ck1;
	my $dataw;
	
	my $i = 0;
	do{
		# byte-order matches the bootloader code (little endian for AVR)
		$dataw = pad_ff($pagestart_addrb, $i);
		$i++;
		$dataw += pad_ff($pagestart_addrb, $i) * 256;
		$ck1 += $dataw;
		$ck2 ^= $dataw;
		
		$i++;
	} while ($i < $pb);	# note: i incremented twice (for each word)
	
	# pushing the checksum
	$ck1 &= 0xFFFF;
	$ck2 &= 0xFFFF;
	if( isCrypto() ){
		my $pgidx = int ($pagestart_addrb / $pb);
		$ck1 = $PROG_CHKSUM->[$pgidx];
		if( ! defined $ck1 ){
		 	print STDERR "WARNING: crypto checksum not found: invalid chex file\n";
		} else {
			printf( "applied checksum: 0x%04x ", $ck1);
		}
	}
	$sendarr[$pb + 0] = $ck1 >> 8;
	$sendarr[$pb + 1] = $ck1 & 0xFF;	
	$sendarr[$pb + 2] = $ck2 >> 8;
	$sendarr[$pb + 3] = $ck2 & 0xFF;
}

# helper function to return FF for undefined values
# also sets in $sendarr array
sub pad_ff
{
	my $a = shift; # page start-address
	my $o = shift; # offset within the page
	$a += $o;
	my $fv = get_flashvalue( $a );
	my $v;
	if(defined $fv){
		$v = $fv;
	} else {
		$v = 255;	# padding with 0xFF
	}
	$sendarr[$o] = $v;
	return $v;
}

# ==========================
sub msp
{
	if( !defined $msp) { return undef; };

	# multitooth 64-1 type signal for testing:
	comm_send( "Manmstbemsp02" );

	return 1;
}

# ==================
sub dumpmct
{
	if( !defined $dumpmct) { return undef; };

	# dump_something( "Manmctmdv", "mct.txt" );	# does mdv cause harm ?
	dump_something( "Manmll", "mll0.txt" );	# LCD content
	dump_something( "Manmct", "mct.txt" );
	dump_something( "Manmcd", "mcd.txt" );
	dump_something( "Manmll", "mll.txt" );	# LCD content

	return 1;
}

sub dump_something
{
	my $readblocksize=2048;
	my ( $cmd, $filename) = @_;
	print STDERR "dumping $cmd to $filename\n";

	comm_send( $cmd );
	#	sleep(1);
	my ($cnt, $str);
	open( MCT, ">$filename" );

	$morechances=2;
	do {
		($cnt, $str) = comm_recv( $readblocksize );
		print MCT $str;
		# print STDERR "read: $str\n";
	} while ( $cnt == $readblocksize || ($cnt == 0 && --$morechances>0) );

	close MCT;
}
# ================
# sets given byteaddress to given value
sub set_flashvalue
{
	my ($addr, $value) = @_;
	$PROGMEM->[$addr] = $value;
	$stats->{set_flashvalue_called}++;
	$stats->{set_flashvalue_addrsum}+= $addr;
	$stats->{set_flashvalue_valsum}+= $value;
}

#====
sub set_flash_page_checksum
{
	my ($addr, $crypto_checksum) = @_;
	if( ! defined $crypto_checksum ){
		return;
	}
	
	# extract important information:
	my $real_cksum = hex( substr($crypto_checksum, 1, 4 ));
	printf( "real_cksum=0x%04x\n", $real_cksum );
	my $pb = get_pagesizeb();
	my $pgidx = int( $addr / $pb );
	# store checksum for given pageidx
	$PROG_CHKSUM->[$pgidx] = $real_cksum;	
}

sub set_flash_page_crypto
{
	my ($addr, $cryp) = @_;

	if( !defined $cryp ){
		return;
	}

	my $pb = get_pagesizeb();
	my $pgidx = int( $addr / $pb );
	# store for given pageidx if crypto or not. will be set if ANY line of that oage has the CRYPT
	$PROG_CRYP->[$pgidx] = $cryp;
}

# ==========
	
# first call with undef ( will prepare PROGMEM so it erases marker page)
# than call with 1 (prepare PROGMEM for writing marker word)
sub make_marker_word
{
	my $action = shift;

	if( ! defined $marker_word ){
		print "Beware: marker_word not used => (check m option), \n";
		return;
	}

	my $local_mark;
	if( defined $action ){
		$local_mark = $marker_word;
	} else {
		$local_mark = 0xFFFF;
	}
	my $mark_addr = bls_start_address() - 2;
	# note: little endian
	set_flashvalue( $mark_addr, $local_mark % 256 );
	set_flashvalue( $mark_addr + 1, int( $local_mark / 256) );
}


# returns startaddress of bootloader section
sub bls_start_address
{
	my $p = $avr_type->{pagecount} - $avr_type->{blspagecount};
        my $pb = get_pagesizeb();
	return	$p * $pb;
}

# return flash-value for given byteaddress
sub get_flashvalue
{
	my ($a) = @_;
	return $PROGMEM->[$a];
}

# returns a list, with the first bytes address for every page
sub get_flash_pagelist
{
	my %pagelist;

	my $pb = get_pagesizeb();
	my $cnt = 0;
	my $v;
	foreach $v (@$PROGMEM){
		# print "FPL: $cnt => $v\n";
		if( defined $v ){
			my $pageidx = int( $cnt / $pb );
			# my $offset = $addr % $pb;	# DON't care
			if( defined $pagelist{$pageidx} ){
				$stats->{pagelist_reassure}++;
			} else {
				$stats->{pagelist_newpage}++;
				$pagelist{$pageidx} = 1;
				# print "FPL: new page, cnt=$cnt, pb=$pb, pagestart=$pagestart\n";
			}
        		# print "$cnt : $v \n";
               	}
               	$cnt ++;
        }

	# print and multiply by $pb :

	my $pageidx;
	my @pagestartarr;
	my $s = '';
	my @a = sort { $a <=> $b } (keys %pagelist);
	foreach $pageidx (@a){
		my $pagestart = $pageidx * $pb;
		$s .= "$pageidx ";
		push @pagestartarr, $pagestart;
	}

	print "pagelist: $s\n";
	return @pagestartarr;
}

# ==================
# uses address memorized by page_address_disallowed
sub isCrypto
{
	my $pgidx = int( $last_flash_address / get_pagesizeb() );
	my $crypto_l = $PROG_CRYP->[$pgidx];
	return (defined $crypto_l && $crypto_l eq "C" );
}

sub getCommandDataWrite
{
	if( isCrypto() )
	{
		return "Dcc";	
	}else{
		return "Dkk";
	}
	
}

sub getCommandDataRead
{
	if( isCrypto() )
	{
		print "CRYP ";
		return "DCC";	
	}else{
		return "Dll";
	}


}
