#include "CommunicationClientBase.h"

#include "CommunicationDispatcher.h"
#include "../vemsprotocol/VemsDeviceController.h"
#include "../events/InputEvent.h"
#include "../logger.h"


CommunicationClientBase::CommunicationClientBase(   VemsDeviceController* controller,
                                                    CommunicationDispatcher* dispatcher )
{
    m_bIsIdle       = true;
    m_pController   = controller;
    m_pDispatcher   = dispatcher;
    m_pDispatcher->registerClient( this );
}


CommunicationClientBase::~CommunicationClientBase()
{
    m_pDispatcher->unregisterClient( this );
}


bool CommunicationClientBase::lockController()
{
    if ( m_pDispatcher->isIdle() )
    {
        m_bIsIdle = false;
        connect();
        return true;
    }

    return false;
}


void CommunicationClientBase::unlockController()
{
    disconnect();
    m_bIsIdle = true;
}


bool CommunicationClientBase::start()
{
    if ( m_pDispatcher->isIdle() )
    {
        Logger::logf( LOG_MSG, _T("CommunicationClientBase::start() - STARTED") );
        m_bIsIdle = false;
        connect();
        process();
        return true;
    }
    else
    {
        //message
        Logger::logf( LOG_MSG, _T("CommunicationClientBase::start() - NOT STARTED") );
        return false;
    }
}


void CommunicationClientBase::stop()
{
    disconnect();
    m_bIsIdle = true;
}


void CommunicationClientBase::connect()
{
    Logger::logf( LOG_MSG, _T("CommunicationClientBase::connect()") );

    m_pController->Connect( wxEVT_INPUT,
                            InputEventHandler( CommunicationClientBase::OnSIPR ),
                            NULL,
                            this );

}


void CommunicationClientBase::disconnect()
{
    Logger::logf( LOG_MSG, _T("CommunicationClientBase::disconnect()") );

    m_pController->Disconnect(  wxEVT_INPUT,
                                InputEventHandler( CommunicationClientBase::OnSIPR ),
                                NULL,
                                this );

}

/*
void CommunicationClientBase::OnSIPR( InputEvent& event )
{
    Logger::logf( LOG_MSG, _T("CommunicationClientBase::OnSIPR()") );
}
*/
