#ifndef COMMUNICATIONCONTROLLER_H
#define COMMUNICATIONCONTROLLER_H

#include <wx/event.h>
#include "../common/typedef.h"
#include "../common/ProcessBase.h"

#include "ECUInfo.h"
#include "VemsCommand.h"
#include "CommunicationControllerStateBase.h"
#include "SIPRParser.h"
#include "StateGraph.h"
#include "CommandRequestBase.h"
#include "../FirmwareVersion.h"

extern const wxEventType ECU_IS_PASSWORD_PROTECTED;

class CommunicationBase;
class ProtocolBase;
class ECUStateMgrBase;
class InputEvent;
class FirmwareInfo;
class ComCtrlState;

WX_DECLARE_HASH_MAP(  int, ProtocolBase*, wxIntegerHash, wxIntegerEqual,  IntToProtocolBaseHash );

class CommunicationController : public wxEvtHandler,  public ProcessBase,
     public CommandRequestBase, public CommunicationControllerStateBase
{
    public:
                    CommunicationController(    ECUStateMgrBase* stateMgr,
                                                FirmwareInfo* fwInfo = NULL,
                                                wxEvtHandler* pEventHandler = NULL );
                    virtual ~CommunicationController();

        int         readBytes();
        int         writeBytes();
        void        registerProtocol(ProtocolBase* pb);
        bool        needClear();
//      void        reset(){   m_currProtocol->reset(); }
        int         setState( int state );
        void        setBaudRate(  int baudRate );

        int         type() const { return ECUInfo::MEGATUNE; }
        void        execute(); // handle controller states
        void        setDefaultProtocol( int p ) { m_iDefaultProtocol = p; }

        /// under testing CommunicationControllerStateBase
        int         getDevice();
        int         getState();
        void        save();
        void        restore();

        /// ProcessBase classes
        void        start(int priority = WXTHREAD_DEFAULT_PRIORITY);
        void        stop();
        //bool        isRunning() const;
        void        reset();
        bool        process();
        //virtual bool ProcessEvent(wxEvent& event );
        ///
        /// CommandRequestBase
        void        request( VemsCommand* command );
        ///

        void setDirect(ProtocolBase* pb);
        bool isReadSensors() const { return m_bReadSensors; }

        void setInterWriteDelay(int iwd){
            m_interWriteDelay = iwd;
        }

        void setPreferredBaudrate( int baudrate )
        {
            m_preferredBaudrate = baudrate;
        }

        FirmwareVersion getFirmwareVersion() const;
        void setLeaveBootModeFlag() { m_leaveBootModeFlag = true; }

        /// owned states
        friend class CCS_Sync;
        //friend class CCS_Detect;
        friend class CCS_GetFirmwareInfoSipr;
        //friend class CCS_GetFirmwareInfo;
        friend class CCS_Normal;
        friend class CCS_SetState;

    private:
        IntToProtocolBaseHash   m_ProtocolHash;
        CommunicationBase*      m_commBase;
        ECUStateMgrBase*        m_stateMgr;
        ProtocolBase*           m_pCurrentProtocol;
        wxEvtHandler*           m_pEventHandler;
        uint8_t                 m_rxBuff[16];
        uint8_t                 m_txBuff[255];
        int                     m_state;            // curent state
        int                     m_device;           // current device

        int                     m_iPreviousState;   // for change baudrate

        //VemsCommand*            m_pCurrentCommand;
        VemsCommandList         m_VemsCommandQueue;

        // saved state
        int                     m_iSavedState;

        FirmwareInfo*           m_fwInfo;
        SIPRParser              m_parser;
        ComCtrlState*           m_pCtrlState;
        ComCtrlState*           m_pPreviousCtrlState;

        IntArray                m_detectingOrder;
        size_t                  m_detectingProtocolIdx;

        bool                    m_bReadSensors;         // enable
        int                     m_iDefaultProtocol;
        int                     m_interWriteDelay;
        int                     m_preferredBaudrate;
        bool                    m_deviceIsProtected;
        bool                    m_leaveBootModeFlag;

    private:
        void        finished();

        int         getCurrentProtocol( int m_state );
        bool        isProtocolRegistered( int protocol );
        bool        setProtocolForState( int state );
        void        processNextCommand();

        /// need remove
        void        fetchRoundFirmwareInfo();
        void        fetchV3FirmwareInfo();
        bool        fetchV3FirmwareInfoSipr();
        uint8_t     readSiprRecord( CommunicationBase* cb );
        void        clear();

        void        OnSIPRInput( InputEvent& event );
        //void        OnThread( wxCommandEvent& event );

        void        setCtrlState( ComCtrlState* state );
        void        restorePreviousCtrlState();

        void        sendOfflineEvents();
        void        sendDeviceEvent( int device );
        void        sendECUStateEvent( int state );

        void        sendDeviceEvent();
        void        sendConnectedEvent();
        void        sendNotConnectedEvent();
        void        sendBootModeEvent();
        void        sendPasswordProtectedEvent();

        int         communicate();              // read, write bytes

        bool        m_direct;
        void        debugPrint(const wxString& msg);
        void        setPreferredBaudrate();

        bool        checkPassword();
        void        leaveBootMode();
};

#endif // COMMUNICATIONCONTROLLER_H
