#include "CommunicationControllerGPS.h"

#include "../CommunicationBase.h"
#include "ProtocolBase.h"
#include "GpsProtocol.h"

#include "../events/GPSEvent.h"
#include "../events/GPSCommEvent.h"

#include "../SerialException.h"

#include "../logger.h"
#include "../serialhelper.h"
#include "../serialconfig.h"

/*
CommunicationControllerGPS::CommunicationControllerGPS(
    CommunicationBase* commBase,
    ProtocolBase* pb,
    wxEvtHandler* pEventHandler)
{
    //ctor
    m_commBase  = commBase;
    m_pCurrentProtocol = pb;
    m_pEventHandler = pEventHandler;

    m_state = GPSInfo::NOT_CONNECTED;

    m_noDataCount = 0;
    m_dataReceivedCount = 0;

//    Connect( wxEVT_THREAD, wxCommandEventHandler( CommunicationControllerGPS::OnThread ) );

}
*/

CommunicationControllerGPS::CommunicationControllerGPS( wxEvtHandler* pEventHandler )
{
    m_commBase = SerialHelper::getGPSComm();

    m_isGPSEnabled = ( m_commBase != NULL );
    Logger::logf(LOG_MSG, _T("m_isGPSEnabled=%d"), m_isGPSEnabled);

    m_pEventHandler = pEventHandler;

    if ( m_isGPSEnabled )
    {
        m_pCurrentProtocol = new GpsProtocol( m_pEventHandler );
    }

    m_state = GPSInfo::NOT_CONNECTED;

    m_noDataCount = 0;
    m_dataReceivedCount = 0;
}

void CommunicationControllerGPS::reloadGPSConfig()
{
//    SerialConfig* scfg = ConfigHelper::setGPSSerialConfig();
//    if ( m_commBase ) delete m_commBase;
//    m_commBase = SerialHelper::getGPSComm( scfg->getPortName() );
//    m_isGPSEnabled = ( m_commBase != NULL );
}

CommunicationControllerGPS::~CommunicationControllerGPS()
{
    stop();
//    while ( isRunning() )
//    {
//        stop();
//    }
//    if ( m_commBase != NULL )
//    {
//        delete m_commBase;
//    }
//    if ( m_pCurrentProtocol != NULL )
//    {
//        delete m_pCurrentProtocol;
//    }
}

void CommunicationControllerGPS::start(int priority)
{

    if ( m_isGPSEnabled == false )
    {
        Logger::logf(LOG_MSG, _T("CommunicationControllerGPS::start() - GPS is disabled, enable it first!"));
        return;
    }

    if ( isRunning() )
        return;

    this->ProcessBase::start();

    sendEvent(GPSInfo::ONLINE);
    Logger::logf(LOG_MSG, _T("CommunicationControllerGPS::start()"));
}

void CommunicationControllerGPS::stop()
{
    if ( m_isGPSEnabled == false )
    {
        Logger::logf(LOG_MSG, _T("CommunicationControllerGPS::stop() - GPS is disabled"));
        return;
    }
    m_state = GPSInfo::NOT_CONNECTED;
    this->ProcessBase::stop();
    m_commBase->close();
    Logger::logf(LOG_MSG, _T("CommunicationControllerGPS::stop()"));
}

void CommunicationControllerGPS::finished()
{
    sendEvent(GPSInfo::OFFLINE);
    Logger::logf(LOG_MSG, _T("CommunicationControllerGPS::finished()"));
}

/*
// the callback for our custom event
void CommunicationControllerGPS::OnThread( wxCommandEvent& event )
{
    Logger::logf( LOG_MSG, _T("-----CommunicationControllerGPS::OnThread()") );
    // handle thread events here, i. e. a premature end due to internal error
    m_pThread = NULL; // thread terminated itself (detached,  was freed automatically)

}
*/

void CommunicationControllerGPS::reset()
{
    if ( m_pCurrentProtocol ) m_pCurrentProtocol->reset();

}

bool CommunicationControllerGPS::process()
{
    int result;
    try{
        result = readBytes();

        switch( result )
        {
            case GPSInfo::NO_GPS_DATA :
            {
                //Logger::logf( LOG_MSG, _T("CommunicationControllerGPS::Entry() : NO_GPS_DATA ") );
                m_noDataCount++;
                wxThread::Sleep( 5 );
            }break;
            case GPSInfo::DATA_RECEIVED:
            {
                //Logger::logf( LOG_MSG, _T("CommunicationControllerGPS::Entry() : DATA_RECEIVED ") );
                m_dataReceivedCount++;
                //wxThread::Sleep( 1 );
            }break;
            case GPSInfo::CONNECTED :
            {
                //Logger::logf( LOG_MSG, _T("CommunicationControllerGPS::Entry() : CONNECTED ") );
                m_noDataCount = 0;
                m_dataReceivedCount = 0;
                if( m_state != result ){
                    //m_state = result;
                    sendEvent(GPSInfo::CONNECTED);
                }
            }
        }

        if( (m_noDataCount == 400) /*|| (m_dataReceivedCount == 400)*/  ){
            //m_state = result;
            sendEvent(GPSInfo::NO_GPS_DATA);
            wxThread::Sleep( 1000 );
        }

    }catch(SerialException& se){
        //m_state = GPSInfo::OFFLINE;
        Logger::logf( LOG_MSG, _T("-----CommunicationControllerGPS::process() ") + se.toString() );
        return false;
    }
    return true;
}
/*
bool CommunicationControllerGPS::ProcessEvent(wxEvent& event )
{
    Logger::logf( LOG_MSG, _T("CommunicationControllerGPS::ProcessEvent()"));
    return this->wxEvtHandler::ProcessEvent(event);
    //return ((wxEvtHandler*)this)->ProcessEvent(event);
}
*/

void CommunicationControllerGPS::sendEvent(GPSInfo::CommState state)
{
    GPSCommEvent event;
    event.state = state;
    m_pEventHandler->AddPendingEvent( event );
    //m_state = state;
}

void CommunicationControllerGPS::sendEvent(GPSInfo::State state)
{
    GPSEvent event;
    event.state = state;
    m_pEventHandler->AddPendingEvent( event );
    m_state = state;
}

/// Must be non blocking (invoked from thread)
int CommunicationControllerGPS::readBytes()
{
    //Logger::logf( LOG_MSG, _T("CommunicationController::()") );
    int rxCount = 1;
    m_rxBuff[0] = 0;
    //int receivedBytes = m_commBase->ReadString(m_rxBuff, rxCount);
    m_commBase->ReadString(m_rxBuff, rxCount);
    //if( receivedBytes == 0 ) return GPSInfo::NO_GPS_DATA;
    if(m_rxBuff[0] == 0 ){
        //Logger::logf( LOG_MSG, _T("CommunicationControllerGPS::() NO DATA") );
        return GPSInfo::NO_GPS_DATA;
    }
    int status = m_pCurrentProtocol->rx(m_rxBuff, rxCount);
    //Logger::logf( LOG_MSG, _T("CommunicationControllerGPS::() DATA:%c,%02X, %i"), m_rxBuff[0], m_rxBuff[0], status );
    return status;

}
