#include "ConfigDownloadController.h"
#include "VemsDeviceControllerBase.h"
#include "SIPRTypes.h"
#include "../logger.h"
#include "../communication/clients/ConfigDownloader.h"
#include "../communication/clients/DeviceModeSelector.h"
#include "../events/InputEvent.h"
#include "../events/CommunicationEvent.h"


IMPLEMENT_DYNAMIC_CLASS( ConfigDownloadController, wxEvtHandler )


ConfigDownloadController::ConfigDownloadController()
{
    m_eventHandler              = NULL;
    m_deviceController          = NULL;
}

ConfigDownloadController::ConfigDownloadController( VemsDeviceControllerBase* deviceController, wxEvtHandler* eventHandler )
{
    m_deviceController  = deviceController;
    m_eventHandler      = eventHandler;
}

ConfigDownloadController::~ConfigDownloadController()
{
}

void ConfigDownloadController::setDeviceController( VemsDeviceControllerBase* deviceController )
{
    m_deviceController = deviceController;
}

void ConfigDownloadController::setEventHandler( wxEvtHandler* eventHandler  )
{
    m_eventHandler = eventHandler;
}

void ConfigDownloadController::start( )
{
    if ( m_deviceController->isManMode() ) {
        m_deviceController->registerClient( new ConfigDownloader( this, this, this ) );
    } else {
        m_deviceController->registerClient( new DeviceModeSelector( this, this,
                                                                    m_deviceController->getCommBase(),
                                                                    m_deviceController->getSupportedBaudrates(),
                                                                    m_deviceController->getDeviceMode(),
                                                                    ECUInfo::MAN,
                                                                    &m_deviceMode ) );
    }

    m_deviceController->startCommControllerNow();
}

void ConfigDownloadController::OnDeviceModeSelected( ECUInfo::EcuMode deviceMode )
{
    m_deviceController->registerClient( new ConfigDownloader( this, this, this ) );
    m_deviceController->startCommControllerNow();
}

void ConfigDownloadController::OnConfigDownloaded()
{
    InputEvent event;
    SIPRPacket packet( 0, SIPR_TABLES_END );
    event.setPacket( packet );
    m_eventHandler->AddPendingEvent( event );
}

void ConfigDownloadController::OnTimeout()
{
    CommunicationEvent event( CommunicationEvent::TIMEOUT );
    event.SetEventObject( this );
    m_eventHandler->AddPendingEvent( event );
}

void ConfigDownloadController::receive( const SIPRPacket& packet )
{
    InputEvent event;
    event.setPacket( packet );
    m_eventHandler->AddPendingEvent( event );
}
/**
FirmwareConfigManager::FirmwareConfigManager
ConfigDownloadDirector::ConfigDownloadDirector

void ConfigDownloadDirector::saveDeviceConfig( bool showDialog )
void FirmwareConfigManager::verifyConfig()
void VemsDeviceController::stop( int flag )
*/
