#include "DeviceModeController.h"
#include "SIPRTypes.h"
#include "../logger.h"
#include "../CommunicationBase.h"
#include "../communication/SerialPortHandlerFactory.h"
#include "../communication/CommunicationController2.h"
#include "../communication/CommunicationControllerEvent.h"
#include "../communication/clients/DeviceModeSelector.h"
#include "../events/CommunicationEvent.h"


IMPLEMENT_DYNAMIC_CLASS( DeviceModeController, wxEvtHandler )

DeviceModeController::DeviceModeController()
{
    m_communicationClient       = NULL;
    m_communicationBase         = NULL;
    m_communicationController   = NULL;
//    m_firmwareData              = NULL;
    m_eventHandler              = NULL;

    m_timeout = 0;
    m_interWriteDelay = 0;

    Connect( COMMUNICATION_CONTROLLER_START, wxCommandEventHandler( DeviceModeController::onStartEvent ), NULL, this );
    Connect(    wxID_ANY,
                wxEVT_COMMUNICATION,
                CommunicationEventHandler( DeviceModeController::onCommunicationEvent ),
                NULL, this );
}

DeviceModeController::DeviceModeController( wxEvtHandler* eventHandler, const DeviceInfo& deviceInfo, size_t timeout, size_t interWriteDelay )
{
    m_communicationClient       = NULL;
    m_communicationBase         = NULL;
    m_communicationController   = NULL;
//    m_firmwareData              = NULL;
    m_eventHandler              = eventHandler;

    m_deviceInfo = deviceInfo;
    m_timeout = timeout;
    m_interWriteDelay = interWriteDelay;

    Connect( COMMUNICATION_CONTROLLER_START, wxCommandEventHandler( DeviceModeController::onStartEvent ), NULL, this );

    Connect(    wxID_ANY,
                wxEVT_COMMUNICATION,
                CommunicationEventHandler( DeviceModeController::onCommunicationEvent ),
                NULL, this );
}

DeviceModeController::~DeviceModeController()
{
    Disconnect( COMMUNICATION_CONTROLLER_START, wxCommandEventHandler( DeviceModeController::onStartEvent ), NULL, this );

    Disconnect( wxID_ANY,
                wxEVT_COMMUNICATION,
                CommunicationEventHandler( DeviceModeController::onCommunicationEvent ),
                NULL, this );
//    clearCommunication();
//    delete m_communicationClient;
}

void DeviceModeController::initCommunication()
{
    m_communicationBase         = SerialPortHandlerFactory::create(&m_deviceInfo.serialConfig);
    m_communicationController   = new CommunicationController2( this, m_communicationClient, m_communicationBase );
}

void DeviceModeController::clearCommunication()
{
    if ( m_communicationController ) {
        m_communicationController->stop();
        delete m_communicationController;
        m_communicationController = NULL;
    }

    if ( m_communicationBase ) {
        m_communicationBase->close();
        delete m_communicationBase;
        m_communicationBase = NULL;
    }
}

void DeviceModeController::sendDeviceControllerEvent( VemsDeviceControllerEventType eventType )
{
    VemsDeviceControllerEvent event( eventType );
    event.setDeviceInfo( m_deviceInfo );
    m_eventHandler->AddPendingEvent( event );
}

void DeviceModeController::handleEvent( CommunicationClientEvent::Type eventType )
{
    switch ( eventType )
    {
        case DEVICE_MODE_SELECTED:
        {
//            registerClient( new FirmwareVerifier( this, m_eventHandler, m_deviceInfo.deviceMode, m_firmwareData ) );
//            wxCommandEvent startEvent( COMMUNICATION_CONTROLLER_START, wxID_ANY );
//            m_eventHandler->AddPendingEvent( startEvent );
LOGP( _T("m_selectedMode: ") + ECUInfo::getStateAsString( m_deviceMode ) );
            m_deviceInfo.deviceMode = m_deviceMode;
            LOGP( _T("m_deviceInfo.deviceMode: ") + ECUInfo::getStateAsString( m_deviceInfo.deviceMode ) );
            sendDeviceControllerEvent( VDC_EVENT_DEVICE_MODE_CHANGED );
        } break;

        case TIMEOUT:
        {
            m_deviceInfo.deviceMode = m_deviceMode;
            sendDeviceControllerEvent( VDC_EVENT_DEVICE_DISCONNECTED );
        } break;

        default:
            LOGP( _T("INVALID CommunicationClientEvent::Type: ") + CommunicationClientEvent::toString( eventType ) );
            break;
    }
}

void DeviceModeController::changeMode( ECUInfo::EcuMode mode )
{
    clearCommunication();
    initCommunication();

    registerClient( new DeviceModeSelector( this,
                                            m_deviceInfo.deviceMode,
                                            mode,
                                            &m_deviceMode ) );

    wxCommandEvent startEvent( COMMUNICATION_CONTROLLER_START, wxID_ANY );
    AddPendingEvent( startEvent );
}

//void DeviceModeController::start(  const DeviceInfo& deviceInfo, FirmwareData* firmwareData, size_t preferredBaudRate )
//{
//    m_deviceInfo                = deviceInfo;
//    m_deviceMode                = deviceInfo.deviceMode;
//
//    clearCommunication();
//    initCommunication();
//
//    LOGP( wxString::Format( _T("m_deviceInfo.getBaudrate(): %d, preferredBaudRate: %d"), m_deviceInfo.getBaudrate(), preferredBaudRate ) );
//
//    if ( ( preferredBaudRate == 0 ) || ( preferredBaudRate == m_deviceInfo.getBaudrate() ) ) {
//        registerClient( new FirmwareVerifier( this, m_eventHandler, m_deviceInfo.deviceMode, firmwareData ) );
//    } else {
//        m_firmwareData = firmwareData;
//        registerClient( new BaudRateSelector(   this,
//                                                m_communicationBase,
//                                                m_deviceInfo.deviceMode,
//                                                m_deviceInfo.getBaudrate(),
//                                                preferredBaudRate ) );
//    }
//
//    wxCommandEvent startEvent( COMMUNICATION_CONTROLLER_START, wxID_ANY );
//    AddPendingEvent( startEvent );
//}
//
//void DeviceModeController::stop()
//{
//    clearCommunication();
//}
//
//bool DeviceModeController::isRunning()
//{
//    return m_communicationController->isRunning();
//}


void DeviceModeController::registerClient( CommunicationControllerClient* client )
{
    delete m_communicationClient;
    m_communicationClient = client;

    m_communicationClient->setTimeout( 40 );
    m_communicationClient->setInterWriteDelay( 40 );

    m_communicationController->registerClient( m_communicationClient );
}

void DeviceModeController::onStartEvent( wxCommandEvent& event )
{
    m_communicationController->start();
}

void DeviceModeController::onCommunicationEvent( CommunicationEvent& event )
{
    LOG
    clearCommunication();
    m_eventHandler->AddPendingEvent( event );
}
