#include "EepromDownloadController.h"

#include "ECUStateMgrBase.h"

#include "../communication/clients/EepromReaderClient.h"
#include "../communication/CommunicationController2.h"
#include "../communication/CommunicationControllerEvent.h"

#include "../events/EepromRawDataEvent.h"
#include "../events/CommunicationEvent.h"
#include "../events/GeneralEvent.h"

#include "../CommunicationBase.h"
#include "../VemsTuneController.h"
#include "../SerialPortHelper.h"
#include "../logger.h"

BEGIN_EVENT_TABLE( EepromDownloadController, wxEvtHandler )
    EVT_COMMUNICATION( wxID_ANY, EepromDownloadController::onCommunicationEvent )
    EVT_GENERAL( COMM_START, EepromDownloadController::onStartEvent )
    EVT_GENERAL( wxID_ANY, EepromDownloadController::onGeneralEvent )
    EVT_EEPROM_RAWDATA( EepromDownloadController::onEepromDownloaded )
END_EVENT_TABLE()

EepromDownloadController::EepromDownloadController( VemsTuneController* vtc, CommunicationBase* commbase, bool writeToFile, wxEvtHandler* evtHandler )
{
    if ( evtHandler == NULL )
    {
        m_evtHandler = this;
    }
    else
    {
        m_evtHandler = evtHandler;
    }

    m_writeToFile = writeToFile;

    m_commBase = commbase;

    m_vtc = vtc;

    m_vtc->setECUOnline( false, true );

    //m_mode = (ECUInfo::EcuMode)m_vtc->getEcuStateMgr()->getState();
    m_mode = m_vtc->getDeviceInfo().deviceMode;

    m_communicationClient = new EepromReaderClient( this, this, m_commBase, SerialPortHelper::getSerialBaudratesInt(), m_mode );

    m_controller = new CommunicationController2( this, m_communicationClient,  m_commBase );
}

EepromDownloadController::~EepromDownloadController()
{
    m_controller->stop();

    delete m_controller;

    delete m_communicationClient;
}

void EepromDownloadController::start()
{
    m_controller->stop();

    GeneralEvent startEvent( COMM_START );
    startEvent.SetVoidPtr( this );

    AddPendingEvent( startEvent );
}

void EepromDownloadController::stop()
{
    EepromReaderClient* client = dynamic_cast<EepromReaderClient*>( m_communicationClient );

    if ( client )
    {
        client->stop();
    }
    else
    {
        LOGP(_T("Runtime Error: cannot cast to eepromreaderclient") );
    }

    m_controller->stop();
}

void EepromDownloadController::onStartEvent( GeneralEvent& event )
{
    // check event sent by this, and id = start_id
    if ( event.GetId() == COMM_START && event.GetVoidPtr() == this )
    {
        delete m_communicationClient;

        m_communicationClient = new EepromReaderClient( this, this,  m_commBase, SerialPortHelper::getSerialBaudratesInt(), m_mode );
        m_controller->registerClient( m_communicationClient );
        m_controller->start();
    }
}

void EepromDownloadController::onCommunicationEvent( CommunicationEvent& event )
{
    // nop
}

void EepromDownloadController::onEepromDownloaded( EepromRawDataEvent& event )
{
    wxMemoryBuffer buf = event.GetDataBuffer();
    Logger::logf(LOG_MSG, _T("Eeprom downloaded, size=%d"), buf.GetDataLen() );

    if ( m_evtHandler && m_evtHandler != this )
    {
        m_evtHandler->AddPendingEvent( event );
    }

    // restart thread
    stop();

    m_vtc->setECUOnline( true );
}

void EepromDownloadController::onGeneralEvent( GeneralEvent& event )
{
    if ( event.GetVoidPtr() == m_communicationClient && m_communicationClient != NULL )
    {
        // 'progress' event
        if ( m_evtHandler && m_evtHandler != this )
        {
            m_evtHandler->AddPendingEvent( event );
        }
    }
}
