#include "GpsProtocol.h"

#include "SIPRPacket.h"
#include "SIPRTypes.h"
#include "../Math.h"
#include "../events/InputEvent.h"
#include "../GPSInfo.h"
#include "../logger.h"

GpsProtocol::GpsProtocol( wxEvtHandler* eventHandler )
{
    m_seqence = 1;
    m_eventHandler = eventHandler;

    m_packedGpsData[0] = 0;
    m_packedGpsData[1] = 0;
    m_packedGpsData[2] = SiprSensorType::GPS;
    m_packedGpsData[16] = 255;
}

GpsProtocol::~GpsProtocol() {}

int GpsProtocol::rx( uint8_t* rxBuff, int rxCount )
{
    int status = GPSInfo::DATA_RECEIVED;
    //Logger::logf( LOG_MSG, _T("GpsProtocol::rx() 1111") );
    int parserStatus;
    for( int i = 0; i < rxCount; ++i )
    {
        parserStatus = m_parser.feedGPScharacter( rxBuff[i] );
        //Logger::logf( LOG_MSG, _T("GpsProtocol::rx() %i"),  parserStatus);
        switch( parserStatus )
        {
            case END :
                //Logger::logf( LOG_MSG, _T("RTDataInputGPS::processGPSData() END") );
                //packData();
                //m_gpsData->set( m_packedGpsData );
                notify();
                status = GPSInfo::CONNECTED;
                break;
            //case NMEAParse::GPS_FORMAT_GPRMC:
            //case NMEAParse::GPS_FORMAT_GPGGA:
            default:
                  //m_rawLogger->captureEcuBlock();
                break;
        }
    }

    return status; //TODO - enum NO_ERROR,
}

int GpsProtocol::tx( uint8_t* m_txBuff, int& rxCount )
{
    return 0;
}

void GpsProtocol::packData(uint8_t* packedGpsData)
{
    //uint8_t         m_packedGpsData[13+16];

    GPSDataStruct gpsData = m_parser.gpsdata;

    if( gpsData.errorbyte )
    {
        return;
    }

    int  lat = gpsData.latitude;
    //Logger::logf( LOG_MSG, _T("GpsProtocol::packData() latitude:%i"),lat );

    //N = 0 , S = -1
    if( gpsData.latitudeindicator == 'S' )
    {
        lat *= -1;
    }

    Math::getBytes( lat, &packedGpsData[0] );

    int lon = gpsData.longnitude;

    //E = 0, W = -1
    if( gpsData.longnitudeindicator == 'W' )
    {
        lon *= -1;
    }

    Math::getBytes( lon, &packedGpsData[4] );

    int height = gpsData.antennaheight;

    height += 10000;
    Math::getBytes( height, &packedGpsData[8], 3);

    if( gpsData.dataFormat == NMEAParse::GPS_FORMAT_GPRMC )
    {
        Math::getBytes( gpsData.speed , &packedGpsData[11], 2);
        //0 = Valid
        //1 = Invalid
        if( gpsData.status == 'A' )
        {
            packedGpsData[8] &= 127;  /// 7.bit 0 Valid GPS Data
        }
        else
        {
            packedGpsData[8] |= 128; ///7.bit 1 Invalid GPS Data
        }
    }

}

//send input event
void GpsProtocol::notify()
{
    if ( !m_eventHandler ) { return; }

    packData(&m_packedGpsData[3]);

    //uint32_t lat = Math::getNumber(&m_packedGpsData[3], 4);
    //Logger::logf( LOG_MSG, _T("GpsProtocol::notify() latitude:%i"),lat );
    InputEvent e;
    SIPRPacket gpsPacket( m_seqence++, SIPR_READ_SENSORS, 17, m_packedGpsData );
    e.setPacket( gpsPacket );
    m_eventHandler->AddPendingEvent( e );

    //Logger::logf( LOG_MSG, _T("GpsProtocol::rx()") );

}
