#include "SIPRFileLogger.h"

#include "SIPRTypes.h"
#include "../logger.h"


SIPRFileLogger::SIPRFileLogger(int blockSize)
{
    //ctor
	m_buffer = NULL;
    setBlockSize(blockSize);

    //int m_blockSize;
    m_bufferCount =0;
    //char * m_buffer;
    m_fos = NULL;
    m_isLog = false;
    m_waitFirstPacket = true;
    m_bSaveSDCard = false;
    //m_headerLock = true;
    //StringHash  m_headerHash;


}

SIPRFileLogger::~SIPRFileLogger()
{
    enableLog( false );
    delete[] m_buffer;
    delete m_fos;
}

void SIPRFileLogger::process(const SIPRPacket& p )
{
    if ( m_bSaveSDCard )
    {
        if ( ( p.type() == SIPR_GET_TABLEDATA ) || ( p.type() == SIPR_GET_CONFIG_1 ) || ( p.type() == SIPR_GET_CONFIG_2 ) )
        {
            //Logger::logf( LOG_MSG, _T("SIPRFileLogger::process() - SAVE PAGES") );
            m_SavedPages.push_back( new SIPRPacket( p ) );
            return;
        }
    }

    if(!m_isLog) return;

    //Logger::logf(   LOG_MSG,
    //                _T("SIPRFileLogger::process() - packet type: %d, ")
    //                _T("size: %d"), p.type(), p.packet_size() );
/*
    if ( p.type() == SIPR_SD_LOG_START )
    {
        m_waitFirstPacket = true;
        return;
    }
*/
    if(m_waitFirstPacket){
        writeHeader();
        m_waitFirstPacket = false;

        if ( m_bSaveSDCard )
        {
            for ( SIPRPacketList::iterator it = m_SavedPages.begin(); it != m_SavedPages.end(); ++it )
            {
                //Logger::logf( LOG_MSG, _T("SIPRFileLogger::process() - WRITE SAVED PAGES") );
                SIPRPacket* p = *it;

                size_t packetSize = p->packet_size();
                const uint8_t* data = p->packet_data();
                if( (m_bufferCount + packetSize) > m_blockSize){
                    Flush();
                }
                memcpy( &(m_buffer[m_bufferCount]), data, packetSize);
                m_bufferCount += packetSize;
            }
        }
    }

    size_t packetSize = p.packet_size();
    const uint8_t* data = p.packet_data();
    if( (m_bufferCount + packetSize) > m_blockSize){
        Flush();
    }
    memcpy( &(m_buffer[m_bufferCount]), data, packetSize);
    m_bufferCount += packetSize;
}

void SIPRFileLogger::Flush()
{
    m_fos->Write(m_buffer, m_blockSize);
    fillBuffer();
    m_bufferCount = 0;
}



void SIPRFileLogger::setFile(const wxString& ofileName)
{
    m_fileName = ofileName;
}

void SIPRFileLogger::setBlockSize(int bSize)
{
    m_blockSize = bSize;
    if (m_buffer != NULL) delete[] m_buffer;
    m_buffer = new char[m_blockSize];
    fillBuffer();

}



void SIPRFileLogger::fillBuffer()
{
    size_t i;
    for(i = 0; i< m_blockSize; i++)
    {
        m_buffer[i] = 0;
    }
    m_buffer[m_blockSize-1] = '\n';
}

void SIPRFileLogger::enableLog(bool b)
{
    if(b == true){
        ///startLog();
        m_waitFirstPacket = true;
    }
    if(b == false){
        ///stopLog();
        if ( m_isLog )
        {
            if ( m_bufferCount )
            {
                Flush();
            }
            if(m_fos != NULL){ delete m_fos; m_fos = NULL;}
        }

    }
    m_isLog = b;
}

void SIPRFileLogger::setHeader(const wxString& header)
{
    m_header = header;
}

void SIPRFileLogger::writeHeader()
{
    if(m_fos != NULL){ delete m_fos; m_fos = NULL;}
    m_fos = new wxFileOutputStream(m_fileName);
    m_bufferCount = 0;
    writeHeader(m_header.mb_str(),m_header.Len());
    Flush();
}

/*
void SIPRFileLogger::writeHeader()
{
    if(m_fos != NULL){ delete m_fos; m_fos = NULL;}
    m_fos = new wxFileOutputStream(m_fileName);

    m_bufferCount = 0;
//    m_headerLock = true;
    wxString headerInfo = _T("# Vems Log Header, 512 byte\n");
    writeHeader(headerInfo.mb_str(),headerInfo.Len());

    for(StringHash::iterator it = m_headerHash.begin(); it != m_headerHash.end(); it++) {
        wxString tmp = it->first + _T("=") + it->second + _T("\n");
		writeHeader((const char*)tmp.mb_str(wxConvUTF8),tmp.Len());
    }
    Flush();
    //m_headerLock = false;
}
*/

void SIPRFileLogger::writeHeader(const char * buff, size_t size)
{
    //if(m_fos == NULL) return;
    if((m_bufferCount + size) > m_blockSize){
           Logger::logf(   LOG_MSG,
                    _T("SIPRFileLogger::writeHeader() - header data too long!!") );
        return;
    }

    memcpy( &(m_buffer[m_bufferCount]), buff, size);
    m_bufferCount += size;
}

/*
void SIPRFileLogger::setHeaderItem(const wxString &key, const wxString &value)
{
    m_headerHash[key] = value;
}
*/
void SIPRFileLogger::setSaveSDCard( bool enable )
{
    m_SavedPages.Clear();
    m_bSaveSDCard = enable;
}
