#include "SIPRParser.h"

#include "SIPRPacket.h"
#include "../Math.h"

SIPRParser::SIPRParser()
{
    m_pData = new uint8_t[ SIPRPacket::SIPR_MAX_DATA_SIZE ];
    reset();
}

SIPRParser::~SIPRParser()
{
    delete[] m_pData;
}

void SIPRParser::reset()
{
    m_state     = SIPR_STATE_WAIT_SIGN;
    m_dataIdx   = 0;
    m_verifyCRC = 0;

    m_sequence  = 0;
    m_type      = 0;
    m_size      = 0;
    memset( m_pData, 0x00, SIPRPacket::SIPR_MAX_DATA_SIZE );
    m_crc       = 0;
}

uint8_t SIPRParser::feed( uint8_t c )
{
    if ( ( c == SIPRPacket::SIPR_SIGN ) && ( m_state == SIPR_STATE_WAIT_SIGN ) )
    {
        reset();
        m_state = SIPR_STATE_SEQ;
        return m_state;
    }

    switch( m_state )
    {
        case SIPR_STATE_SEQ:
            m_sequence  = c;
            m_verifyCRC = Math::crc16_sipr( 0xFFFF, c );
            m_state     = SIPR_STATE_TYPE;
            break;
        case SIPR_STATE_TYPE:
            m_type      = c;
            m_verifyCRC = Math::crc16_sipr( m_verifyCRC, c );
            m_state     = SIPR_STATE_SIZE;
            break;
        case SIPR_STATE_SIZE:
            m_size      = c;
            m_verifyCRC = Math::crc16_sipr( m_verifyCRC, c );
            if ( !m_size )
            {
                m_state = SIPR_STATE_CRC1;
            }
            else
            {
                m_state = SIPR_STATE_DATA;
            }
            break;
        case SIPR_STATE_DATA:
            m_pData[ m_dataIdx++ ] = c;
            m_verifyCRC = Math::crc16_sipr( m_verifyCRC, c );
            if ( m_dataIdx == m_size )
            {
                m_state = SIPR_STATE_CRC1;
            }
            break;
        case SIPR_STATE_CRC1:
             m_crc      = ( m_crc & ( 65535-255 ) ) + c; // ADD LOW
             m_state    = SIPR_STATE_CRC2;
             break;
        case SIPR_STATE_CRC2:
            m_crc  = ( m_crc & 255 ) + (((uint16_t)c)<<8); // ADD HIGH
            m_state = SIPR_STATE_WAIT_SIGN;
            if ( m_verifyCRC == m_crc )
            {
                return SIPR_STATE_CRC_OK;
            }
            else
            {
                return SIPR_STATE_CRC_ERROR;
            }
            break;
    }

    return m_state;
}

wxString SIPRParser::toString()
{
    wxString buf;
    for(size_t i = 0; i < m_size; ++i){
        buf += wxString::Format(_T("%02X "), m_pData[ i ]);
    }
    return buf;
}
