#ifndef SIPRPARSER_H
#define SIPRPARSER_H

#include "../common/typedef.h"
#include <wx/wx.h>

class SIPRParser
{
    public:
        // state machine states
        enum SiprState
        {
            SIPR_STATE_WAIT_SIGN    = 0,
            SIPR_STATE_SEQ          = 1,
            SIPR_STATE_TYPE         = 2,
            SIPR_STATE_SIZE         = 3,
            SIPR_STATE_DATA         = 4,
            SIPR_STATE_CRC1         = 5,
            SIPR_STATE_CRC2         = 6,
            SIPR_STATE_CRC_OK       = 7,
            SIPR_STATE_CRC_ERROR    = 8,
            SIPR_STATE_NO_DATA      = 9
        };

    public:
                    SIPRParser();
                    ~SIPRParser();

        // parsing methods
        void            reset();
        uint8_t         feed( uint8_t c );

        // getter methods
        uint8_t         sequence()  const { return m_sequence; }
        uint8_t         type()      const { return m_type; }
        uint8_t         size()      const { return m_size; }
        const uint8_t*  data()      const { return m_pData; }
        uint16_t        crc()       const { return m_crc; }

        wxString toString();
    private:
        uint8_t     m_state;        // state machine state
        uint8_t     m_dataIdx;      // receiving packet data index
        uint16_t    m_verifyCRC;    // crc for verify rx packet

        uint8_t     m_sequence;     // receiving packet datum
        uint8_t     m_type;
        uint8_t     m_size;
        uint8_t*    m_pData;
        uint16_t    m_crc;
};

#endif // SIPRPARSER_H
