#ifndef SIPRTYPES_H
#define SIPRTYPES_H


enum SiprCommand
{
    SIPR_NACK               = 0,
    SIPR_ACK                = 1,
    SIPR_GET_VERSION        = 2,
    SIPR_SET_BAUD           = 3,
    SIPR_GET_TABLE_SIZE     = 4,
    SIPR_GET_TABLEDATA      = 5,
    SIPR_SET_TABLEDATA      = 6,
    SIPR_READ_SENSORS       = 7,
    SIPR_GET_HWSIG          = 8,
    SIPR_ENTER_FLASH        = 9,
    SIPR_SAVE_CONFIG        = 10,
    SIPR_INVALID_DATA       = 11,
    SIPR_ENTER_MEGATUNE     = 12,

    SIPR_GET_CONFIG_1       = 13,
    SIPR_GET_CONFIG_2       = 14,
    SIPR_GET_COMPILE_TIME   = 15,
    SIPR_GET_MT_SIGNATURE   = 16,
//    SIPR_GET_CONFIG_SHARED  = 17,


    SIPR_READ_MEMORY        = 63,
    SIPR_WRITE_MEMORY       = 64,
    SIPR_READ_MEMORY_32     = 65,
    SIPR_WRITE_MEMORY_32    = 66,

    SIPR_READ_STORAGE_32    = 67,
    SIPR_WRITE_STORAGE_32   = 68,

    SIPR_PULSE_TRAIN        = 128,
    /// These commands should be moved to some event for example SIPRControll;
    SIPR_SD_LOG_START       = 129,
    SIPR_SD_LOG_STOP        = 130,
    //SIPR_CONFIG_LINE        = 131,
    SIPR_SD_EMPTY           = 132,
    SIPR_SD_NOT_EMPTY       = 133,
    SIPR_SD_TIMEOUT         = 134,
    SIPR_SD_CRC_ERROR       = 135,
    SIPR_SD_LOG_NEXT_BLOCK  = 136,
    SIPR_SYNC_STARTED       = 137,      // sync of ECU pages started
    SIPR_SYNC_FINISHED      = 138,      // sync of ECU pages finished
    SIPR_SYNC_PAGE          = 139,
    SIPR_CONFIG_LINE        = 140,
    SIPR_CONFIG_ERROR       = 141,
    SIPR_CONFIG_END         = 142,
    SIPR_TABLES_END         = 143,
    SIPR_TIMEOUT            = 144,
    SIPR_AIM                = 145,

    SIPR_LOG_MARKER         = 146,
    SIPR_RT_TIME            = 147,

    ///TESTING
    SIPR_UPDATE_FREQUENCY   = 148,

    // comm model after every n mdx a frame_ended packet is created
    SIPR_MDX_FRAMES_END     = 149,

    SIPR_RT_CRC_ERROR       = 150,

    SIPR_RESERVED       = 255,
};
enum SiprBaud
{
    SIPR_BAUD_9600          = 0,
    SIPR_BAUD_19200         = 1,
    SIPR_BAUD_38400         = 2,
    SIPR_BAUD_57600         = 3,
    SIPR_BAUD_115200        = 4
};

enum SiprTable
{
    SIPR_TABLE_RPM          = 1,
    SIPR_TABLE_KPA          = 2,
    SIPR_TABLE_IGN          = 3,
    SIPR_TABLE_EGOC         = 4,
    SIPR_TABLE_VE           = 5, //
    SIPR_TABLE_LAMBDA       = 6,
    SIPR_TABLE_CONFIG1      = 7, //
    SIPR_TABLE_CONFIG2      = 8,
    SIPR_TABLE_LCD          = 9,

    SIPR_TABLE_H            = 10,
    SIPR_TABLE_BOOST_DC     = 11,
    SIPR_TABLE_BOOST_TPS    = 12,
    SIPR_TABLE_TEMPERATURE  = 13,
    SIPR_TABLE_MAPCORR      = 14,
    SIPR_TABLE_ALT_KNOCK    = 15,
    SIPR_TABLE_A340E        = 16,
    SIPR_TABLE_VVTI         = 17,
    SIPR_TABLE_MAT_TPS_RETARD = 18,

    SIPR_TABLE_CONFIG1_EXT = 19,
    //ROADSPEED_OR_TIME_VS_RPM = 20
    SIPR_TABLE_LAUNCH_RPM = 20,
    SIPR_TABLE_CONFIG_SHARED = 21

};

struct SiprSensorType
{
    enum
    {
        PAGE_0              = 0,        // not implemented
        SIPR_RT_MT_NORMAL   = 1,  // MT A command
        SIPR_RT_MT_EXT      = 2,     // MT D command
        SIPR_RT_ROUND       = 3,
        GPS                 = 32,
        //AIM                 = 33,
        SIPR_READ_MEMORY_32 = 65,
        SIPR_AIM            = 145,
    };
};

enum SiprLogMarkerType
{
    SIPR_LOG_MARKER_START = 0,
    SIPR_LOG_MARKER_STOP  = 1,
    SIPR_REFRESH_START      = 2,
    SIPR_REFRESH_STOP       = 3,
    SIPR_TRIGGERFRAME       = 4,
};


#endif // SIPRTYPES_H
