#include "SerialNumber.h"

#include "../UnicodeHelper.h"
#include "../logger.h"
#include "../Math.h"

/** caller must not delete or change buf after calling
*/
SerialNumber::SerialNumber(char * buf)
{
    serialPacked = (unsigned char * )buf;
}

SerialNumber::SerialNumber(const wxString& buf)
{
    serialPacked = (unsigned char*) UnicodeHelper::U2C(buf);
}

/*
SerialNumber::SerialNumber(wxString wxBuf)
{
    SerialNumber( ( char *) wxBuf.c_str() );

}
*/

SerialNumber::~SerialNumber()
{
    //dtor
    delete serialPacked;
}

// 00 or FF not allowed (because of network addresses)
bool SerialNumber::isBadNetworkByte( unsigned char t)
{
        if( t == 0 || t == 0xFF ){
            return true;
        }

        return false; // OK
}

/** prints warning if serial number is stupid
*/
bool SerialNumber::isOk()
{
#ifdef V3
    return true;
#else
    /* Round!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
    // strncmp( "r0.0\0\0\0\0" ... );
    if( serialPacked[0] != 'r' ) return false;
    if( serialPacked[1] != '0' ) return false;
    if( serialPacked[2] != '.' ) return false;
    if( serialPacked[3] != '0' ) return false;
    if( serialPacked[4] != 0 ) return false;
    if( serialPacked[5] != 0 ) return false;
    if( serialPacked[6] != 0 ) return false;
    if( serialPacked[7] != 0 ) return false;

    if( isBadNetworkByte(serialPacked[8])) return false;
    if( isBadNetworkByte(serialPacked[9])) return false;
    if( isBadNetworkByte(serialPacked[10])) return false;
    if( isBadNetworkByte(serialPacked[11])) return false;

    return true;
#endif
}

/* subtracts 01 01 01 00 from the binary number - historical reasons:
this is printed on the RS232 DSUB9 connector
*/
unsigned int SerialNumber::getserialNumber()
{
    unsigned int tmp32;
    unsigned char t8;
    tmp32 = 0;
    t8 = serialPacked[8]; tmp32 |= t8 << 24;
    t8 = serialPacked[9]; tmp32 |= t8 << 16;
    t8 = serialPacked[10]; tmp32 |= t8 << 8;
    t8 = serialPacked[11]; tmp32 |= t8;
    tmp32 -= 0x01010100;
    Logger::logf(LOG_MSG, _T("SerialNumber::getserialNumber(): %d\n"), tmp32 );
    return tmp32;
}

wxString SerialNumber::getSerialNumberStr()
{
    unsigned int sNumber = getserialNumber();
    return wxString::Format(_T("%i"), sNumber);
}

wxString SerialNumber::getserialNumberHex()
{
#ifdef V3
    str = wxString::FromAscii((char*)serialPacked);
    return str;
#else
/*Round!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
    wxString hex = Math::char2hex(serialPacked, 12);
    return hex;
/*
    int serialLen = 12;
    char * formatedSN = new char[37];
        char * formatedSN1;
        formatedSN1 = formatedSN;
        int i;
         for(i=0; i<serialLen; i++){
             unsigned char tmp = serialPacked[i];
            int r = snprintf(formatedSN1, 4,"%02X-",tmp);
            //printf("r: %d",r);
            formatedSN1 += 3;
         }
         formatedSN[35]=0; //remove "-"
        printf("HEX Serial number: %s\n", formatedSN);
        wxString tmp;
        tmp.Printf(_T("%s"),formatedSN);
        return tmp;
*/
#endif
}
