#include "SerialPortHandlerSFile.h"

#include "../serial/SFile.h"

#include "../serialconfig.h"
#include "../SerialException.h"
#include "../Math.h"

#include "../serial/timer.h"

#include "../UnicodeHelper.h"
#include "../logger.h"

SerialPortHandlerSFile::SerialPortHandlerSFile(SerialConfig* serialConfig)
{
    dev = new SFile();

    m_serialConfig = serialConfig;
    setBaudRate(m_serialConfig->getBaudRate() );
}

SerialPortHandlerSFile::~SerialPortHandlerSFile()
{
    Logger::log(LOG_MSG, _T("SerialPortHandlerSFileSFile DTOR") );
    dev->Close();
    delete dev;
}

bool SerialPortHandlerSFile::IsOpen(){
    return dev->IsOpen();
}

int SerialPortHandlerSFile::open()
{
//fprintf(stdout, "SerialPortHandlerSFile this: %p\n",this);

    //fprintf(stdout,  "SerialPortHandlerSFile:open() START: dev: %p\n", dev);
    //Logger::logf(LOG_WARNING, _T("SerialPortHandlerSFile::open() init!!") );
    if(dev == NULL){
        dev = new SFile(); Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::open() new instance!!") );
    }
//        Logger::log(LOG_MSG, _T("SerialPortHandlerSFile:open()dev NOT NULL!!") );
    if( dev->IsOpen() ){ return 0; }
//    Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::open() init!!") );

    //initSerialPort();

	// try to open the given port appr 500 times (you guess right, windows sucks!)
    int trycnt=4;


	long port=0;
	wxString ports = m_serialConfig->getPortName().AfterLast('M');
	ports.ToLong(&port);
    while( trycnt > 0 && !dev->Open( port,19200 ) ) {

//        Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::open() Cannot open: ") + m_serialConfig->getPortName() );
        //Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::open() Cannot openC: %s"), m_serialConfig->getPortNameC() );
        //wxLogError(_T("Cannot open: %s\n"),devname );
        //printf(" SerialPortHandlerSFile::open() %s", m_serialConfig->getPortNameC() );
        trycnt--;
        // sleep(1); or wxsleep() ?
    }
    if( trycnt == 0 ){
//        Logger::logf(LOG_MSG,  _T("Giving up opening ")+ m_serialConfig->getPortName() );
        // Wrong: Logger::log(_T("Giving up opening 25 %s\n"),UnicodeHelper::U2C( portName.c_str() ) );
        //delete dev;
        //dev = NULL;
        //if(dev == NULL){
        //    fprintf(stdout, "dev is NULL\n");
        //}
        //serialConfig.setStatus("Unable to open!");
        throw SerialException(SP_CANNOT_OPEN);
        //return SP_CANNOT_OPEN;

    }


    dev->SetBaudRate(m_baudrate);



   return 0;
}


void SerialPortHandlerSFile::close()
{
    //Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::close() ") );
    if( dev == NULL) { return; }
    //Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::close() dev not null") );
    if( !dev->IsOpen() ){ return; }
    //Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::close() dev->Close()") );
    dev->Close();
}


int SerialPortHandlerSFile::SendString(const uint8_t * str)
{
     //fprintf(stdout, "LINE: %s\n", str );
     return SendString(str,  strlen((char*)str));
}


int SerialPortHandlerSFile::SendString(const uint8_t* str, size_t len)
{
    open();
    DWORD result;
	dev->Write( (char *) str, len,result);

    if(len != result){
           Logger::log(LOG_ERROR, _T("SerialPortHandlerSFile::SendString() WRITE FAILED!") );
            throw SerialException(SP_WRITE_ERROR);
            //return SP_WRITE_ERROR;
     }
     return result;
}

int SerialPortHandlerSFile::SendString(const wxString& str)
{
    const char * c = UnicodeHelper::U2C(str);
    int res = SendString( (uint8_t*)c );
    delete (char *)c;
    return res;
}

int SerialPortHandlerSFile::Read(uint8_t* str, size_t len)
{
        open();

        DWORD rd=0;
		DWORD bytes = dev->BytesInComm();
		if(bytes)
		{
			if(bytes<len)
				dev->Read((char*)str, bytes, rd);
			else
				dev->Read((char*)str, len, rd);
		}
		else
			rd=0; //just for easy debug

        return rd;
}


int SerialPortHandlerSFile::ReadString(uint8_t* str, size_t len)
{
  //      Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::ReadString()"));
        open();
  //      Logger::logf(LOG_MSG, _T("SerialPortHandlerSFile::ReadString() sp open"));
       // if( open() == SP_CANNOT_OPEN){ return SP_CANNOT_OPEN;}
        //if(dev == NULL){return SP_UNKNOWN;}
        //int timeout = 3 * len; //?????
        int timeout = getTimeOut(len);
        int to = 0;
        //size_t len = strlen(str);
         wxString msg;
        timer t(timeout, &to, NULL);
        t.start();
		int rd=0;
		DWORD bytes = dev->BytesInComm();
		if(bytes)
		{
			if(bytes<len)
				rd = dev->Readv ((char*)str, bytes, &to, true);
			else
				rd = dev->Readv ((char*)str, len, &to, true);
		}
		else
			rd=0; //just for easy debug


        return rd;


}

int SerialPortHandlerSFile::ReadString(uint8_t* data, size_t len, int maxTryCount)
{
    int tryCount =0;
    int result = 0;
    do{
        size_t expLen = len - result;
        int tmp = ReadString(&data[result], expLen);
        if(tmp == 0 ){
             tryCount++;
            wxThread::Sleep(1);
        }
        result += tmp;

    }while( (result < (int)len) && (tryCount < maxTryCount));
    return result;
}

int SerialPortHandlerSFile::getBaudRate()
{
    return m_baudrate;
}

wxString SerialPortHandlerSFile::getPortName() const
{
    return m_serialConfig->getPortName();
}

void SerialPortHandlerSFile::setBaudRate(int bRate)
{
    Logger::logf( LOG_MSG, _T("SerialPortHandlerSFile::setBaudRate: %i"), bRate );
    //open();
    //if( open() == SP_CANNOT_OPEN) return ;
    wxString msg;
	m_baudrate= bRate;


    m_magicTimeoutConstA = 12000.0/m_baudrate;
    m_magicTimeoutConstB = 12.0 * 10000.0/m_baudrate;
    //open();
    if( IsOpen() ) dev->SetBaudRate(m_baudrate);
}


int SerialPortHandlerSFile::clearSerialBuffer(int len)
{
        uint8_t * tmp = new uint8_t[len+32];
        int res = ReadString(tmp, len);
        delete[] tmp;
        return res;
}

int SerialPortHandlerSFile::clearSerialBufferCircle(int len, int count)
{
    int res;
    int c = 0;
    while(count > 0){
        res = clearSerialBuffer(len);
        if(res < len)return (c * len + res);
        count--;
        c++;
    }
   return -1;
   // throw SerialException(SP_CANNOT_CLEAR_BUFF);
   // return SP_CANNOT_CLEAR_BUFF;
}

/*virtual*/
int SerialPortHandlerSFile::clear()
{
	return dev->PurgeComm();
}

int SerialPortHandlerSFile::sendStringBlock(uint8_t* value, size_t len, size_t blockLen)
{
    //char * p = (char *) value;
    int result = 0;
    size_t i = 0;
    size_t currBlock;
    for(i=0; i< len; i += blockLen){
        currBlock = len - i;
        if( currBlock > blockLen ){
            currBlock = blockLen;
        }
        result += SendString( (&(value[i])), currBlock);
    }
    return result;
}

int SerialPortHandlerSFile::getTimeOut(size_t len)
{
    return (int)(len * m_magicTimeoutConstA + m_magicTimeoutConstB);

    //return ((len*10)>>3) + 12; //9600 10000/baudRate

    //return ((len*10000)/m_baudrate) + 12;
}

int SerialPortHandlerSFile::sendNumber(uint32_t number, int length)
{
    uint8_t str[4];
    Math::getBytes(number, str, length);
    return SendString(str, length);
}

int SerialPortHandlerSFile::sendNumber(uint16_t number)
{
    uint8_t str[2];

    str[0] = (number >> 8) &0xFF;
    str[1] = number & 0xFF;

    return SendString( str, 2);
}
