#include "Sipr.h"

#include "SIPRDescriptor.h"
#include "VemsCommand.h"
#include "../Math.h"
#include "../logger.h"
#include "../events/InputEvent.h"

#ifdef __WINCE_STANDARDSDK__
	#include "../wince/src/timer.h"
#elif (_MSC_VER >= 1400) //  1400 = VS2005 1500 = VS2008
	#include "../v3gui/src/timer.h"
#else
	#include <wx/ctb/timer.h>
#endif


/* received bytes
0A 01 00 00 20 00
0A 00 00 00 71 C0
*/

/*
Proposal:
while(){
    int result = m_commBase->ReadString(&c, 1);

    if(result == 1) state = getState(c);
    switch(state){
        case BOOT:
        case NORMAL: (MAN/MEGATUNE)
        case SIPR:
        case SD_CARD: // should move to SIPR
        case AIM:
        case GPS:    //
        case UNKNOWN:
        case NOT_CONNECTED:
    }

    for(registeredProtocols ){
        if(result == 1) protocol->feed(c);
        bool isValid =  protocol->sendRecord(data, len);
        if(isValid) m_commBase->sendString(data, len);
    }

}
*/


Sipr::Sipr( SIPRDescriptor* siprDesc , int sensorType, wxEvtHandler* eventHandler )
{
    m_pSIPRDescriptor   = siprDesc;
    m_eventHandler      = eventHandler;
    m_seqence           = 0;
    m_isReadSensors     = false;
    m_delayReadSensors  = 0;
    m_timerTimedOutFlag = 0;
    m_timerReadSensors  = NULL;
    m_bIsDisabled       = false;
    m_bNeedDisable      = false;
    m_state             = IDLE;
    m_bHasError         = false;
    m_iSensorType       = sensorType;

}

Sipr::~Sipr() {}

void Sipr::setEventHandler( wxEvtHandler* eventHandler )
{
    m_eventHandler = eventHandler;
}




/*
bool Sipr::readSensors(uint8_t dataCount, uint8_t delay, uint8_t pageType, uint8_t* buff)
{
    m_txBuff[0] = SIPR_SIGN;
    m_txBuff[1] = m_seq;
    m_txBuff[2] = READ_SENSORS;
    m_txBuff[3] = 3;
    m_txBuff[4] = dataCount;
    m_txBuff[5] = delay;
    m_txBuff[6] = pageType;

    setCRC16(m_crcStart, 6);

    m_commBase->SendString(m_txBuff, 9);
    uint8_t res = readRecord();
    m_seq++;

    if(res == SIPR_CRC_OK){
        buff[14] = m_rxPacket.data[3]; /// RPM test
        buff[15] = m_rxPacket.data[4];
        buff[8] = m_rxPacket.data[19]; /// TPS test
        return true;
    }

    printDebug(res);
    return false;

}
*/

int Sipr::push( SIPRPacket* packet )
{
    m_txQueue.push_back( packet );

    // send event
    //sendEvent( *packet, SIPR_EVENT_TYPE_TX_REQUEST );

    return 0;
}

void Sipr::ping()
{
    //Logger::logf( LOG_MSG, _T("Sipr::ping()") );

    SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_ACK );
/*
    wxString s;// = wxString::Format( _T("size: %d, data:"), packet->size() );


    for ( size_t i = 0; i < packet->packet_size(); ++i )
    {
        s += wxString::Format( _T("$%02X"), packet->packet_data()[i] );
        //s += wxString::Format( _T("%d"), i );
    }

    Logger::logf( LOG_MSG, _T("Sipr::ping() - ") + s );
*/
    push( packet );
}

void Sipr::get_version()
{
    SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_GET_VERSION );

    push( packet );
}

void Sipr::set_baud( SiprBaud b )
{
    const int PACKET_SIZE = 1;

    uint8_t     data[PACKET_SIZE] = { b };
    SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_READ_SENSORS, PACKET_SIZE, data );

    push( packet );
}

void Sipr::read_sensor( uint8_t count, uint8_t delay, uint8_t page )
{
    //Logger::logf( LOG_MSG, _T("Sipr::read_sensor() - count: %d, delay: %d, page: %d"), count, delay, page );

    const int PACKET_SIZE = 3;

    uint8_t     data[PACKET_SIZE] = { count, delay, page };
    SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_READ_SENSORS, PACKET_SIZE, data );
/*
    wxString s;

    for ( size_t i = 0; i < packet->packet_size(); ++i )
    {
        s += wxString::Format( _T("$%02X"), packet->packet_data()[i] );
        //s += wxString::Format( _T("%d"), i );
    }

    Logger::logf( LOG_MSG, _T("Sipr::read_sensor() - ") + s );
*/
    push( packet );
}


void Sipr::get_table_data( int page )
{
    //Logger::logf( LOG_MSG, _T("Sipr::get_table_data() - page: %d"), page );

    const int PACKET_SIZE = 1;

    uint8_t     data[PACKET_SIZE] = { page };
    SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_GET_TABLEDATA, PACKET_SIZE, data );

    push( packet );
}

void Sipr::set_table_data( uint8_t page, uint8_t offset, uint8_t value )
{
    //Logger::logf( LOG_MSG, _T("Sipr::set_table_data() - page: %u, offset: %u, value: %u"),
    //                page, offset, value );

    const int PACKET_SIZE = 3;

    uint8_t     data[PACKET_SIZE] = { page, offset, value };
    SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_SET_TABLEDATA, PACKET_SIZE, data );

    push( packet );
}

/*
void Sipr::getVersion(uint8_t* major, uint8_t* minor, uint8_t* build)
{
    m_txBuff[0] = SIPR_SIGN;
    m_txBuff[1] = m_seq;
    m_txBuff[2] = GET_VERSION;
    m_txBuff[3] = 0;

    setCRC16(m_crcStart, 3);
    m_commBase->SendString(m_txBuff, 6);

    uint8_t res = readRecord();
    printDebug(res);

    *major = m_rxPacket.data[0];
    *minor = m_rxPacket.data[1];
    *build = m_rxPacket.data[2];

    m_seq++;
}


*/
bool Sipr::checkTimeout()
{
    //Logger::logf( LOG_MSG, _T("Sipr::checkTimeout() - BEGIN") );

    if ( m_waitingQueue.empty() ) { return false; }

    //Logger::logf( LOG_MSG, _T("Sipr::checkTimeout() - WAITING QUEUE IS NOT EMPTY") );

    SIPRPacket* p = m_waitingQueue.front();

    if ( !p->is_timed_out() ) { return false; }

/*
    if ( p->size() > 0 )
    {
        Logger::logf( LOG_MSG, _T("Sipr::checkTimeout() - TIMED_OUT - type: %d, subtype: %d"), p->type(), p->getSubType() );
    }
    else
    {
        Logger::logf( LOG_MSG, _T("Sipr::checkTimeout() - TIMED_OUT - type: %d"), p->type() );
    }
*/

    //sendEvent( *p, SIPR_EVENT_TYPE_TIMED_OUT );
    SIPRPacket timeoutPacket( 0, SIPR_TIMEOUT );
    sendEvent( timeoutPacket, SIPR_EVENT_TYPE_INPUT );
    m_waitingQueue.pop_front();

    delete p;

    m_bHasError = true;

    return true;
}


int Sipr::rx( uint8_t* c, int size )
{
 //   Logger::logf( LOG_MSG, _T("Sipr::rx() - BEGIN - size: %d"), size  );

/*
    if ( size > 0 )
    {
        Logger::logf( LOG_MSG, _T("Sipr::rx() - BEGIN read: %u"), *c );
    }
*/
    checkTimeout();

    for( int i = 0; i < size; ++i )
    {
        int s = m_parser.feed( c[i] );

        //Logger::logf( LOG_MSG, _T("Sipr::rx() - parser result: %d, type: %d, size: %d"), s, m_parser.type(), m_parser.size() );

        if (  s == SIPRParser::SIPR_STATE_CRC_OK )
        {
            Logger::logf( LOG_MSG, _T("Sipr::rx() - CRC OK") );
            /*
            if ( m_parser.size() > 0 )
            {
                Logger::logf(   LOG_MSG, _T("Sipr::rx() - type: %u, subtype: %d, size: %u"),
                                m_parser.type(), m_parser.data()[0], m_parser.size() );
            }
            else
            {
                Logger::logf( LOG_MSG, _T("Sipr::rx() - type: %u, size: %u"), m_parser.type(), m_parser.size() );
            }
            */
            switch( m_parser.type() ){
                case SIPR_PULSE_TRAIN:
                {
                    notify();
                    return 1;
                }
                case SIPR_WRITE_MEMORY_32:
                case SIPR_WRITE_MEMORY:
                    Logger::logf(   LOG_MSG, _T("Sipr::rx() wm- type: %u, subtype: %d, size: %u"),
                                m_parser.type(), m_parser.data()[0], m_parser.size() );
                    Logger::logf(   LOG_MSG, _T("Sipr::rx() hexdata: ")+ m_parser.toString() );

                break;
                case SIPR_READ_MEMORY:
                case SIPR_READ_MEMORY_32:
                {

                    Logger::logf(   LOG_MSG, _T("Sipr::rx() rm- type: %u, subtype: %d, size: %u"),
                                m_parser.type(), m_parser.data()[0], m_parser.size() );
                    Logger::logf(   LOG_MSG, _T("Sipr::rx() hexdata: ")+ m_parser.toString() );
                }
            }

            if ( m_waitingQueue.empty() ) { continue; }

            SIPRPacket* packet = m_waitingQueue.front();

            int state = -1;

            if ( packet->sequence() == m_parser.sequence() )
            {
                if ( m_parser.type() == packet->type() || m_parser.type() == SIPR_ACK )
                {
                    Logger::logf( LOG_MSG, _T("Sipr::rx() - response: OK") );
                    notify();
                    state = 1;// TODO ENUM
                }
                else
                {
                    if ( packet->size() > 0 )
                    {
                        Logger::logf( LOG_MSG, _T("Sipr::rx() - response: BAD TYPE - rx: %d, wait: %d, %d"),
                            m_parser.type(), packet->type(), packet->data()[0] );
                    }
                    else
                    {
                        Logger::logf( LOG_MSG, _T("Sipr::rx() - response: BAD TYPE - rx: %d, wait: %d"),
                            m_parser.type(), packet->type() );
                    }

                    state = 2;// TODO ENUM
                }
            }
            else
            {
                Logger::logf( LOG_MSG, _T("Sipr::rx() - response: BAD SEQUENCE - %d != %d, queue size: %d"),
                                packet->sequence(), m_parser.sequence(), m_waitingQueue.GetCount() );
                state = 2;// TODO ENUM
            }

            m_waitingQueue.pop_front();
            delete packet;
            //Logger::logf( LOG_MSG, _T("Sipr::rx() - waiting queue size: %d"), m_waitingQueue.GetCount() );

            m_state = IDLE;

            return state;
        }
        else if ( s == SIPRParser::SIPR_STATE_CRC_ERROR )
        {
            Logger::logf( LOG_MSG, _T("Sipr::rx() - CRC ERROR") );
            m_state = IDLE;
        }
    }

    return 0; //TODO - enum NO_ERROR,
}

int Sipr::tx( uint8_t* txBuffer, int& size )
{
   //Logger::logf( LOG_MSG, _T("Sipr::tx() - BEGIN") );

    //if ( m_bIsDisabled ) { return; }

    if ( m_state != IDLE )
    {
        return 0;
    }

    if ( m_bNeedDisable && m_txQueue.IsEmpty() )
    {

        m_bIsDisabled   = true;
        m_bNeedDisable  = false;
        return 0;
    }

    if ( !m_bNeedDisable && m_isReadSensors )
    {
        readRealtime( true, 0 );
    }
/*
    if ( m_timerTimedOutFlag && m_timerReadSensors )
    {
        Logger::logf( LOG_MSG, _T("Sipr::tx() - timer start") );
        read_sensor( 1, 0, 1 );
        m_timerTimedOutFlag = 0;
        m_timerReadSensors->start();
    }
*/
    if ( m_txQueue.IsEmpty() )
    {
        return 0;
    }




    //Logger::logf( LOG_MSG, _T("Sipr::tx() - send") );

    // get packet from txQueue
    SIPRPacket* packet = m_txQueue.front();
    Logger::logf( LOG_MSG, _T("Sipr::tx() - send data: ")+ packet->toString() );
    m_txQueue.pop_front();

    // copy packet data ( send data to CommunicationController )
    size = packet->packet_size();
    memcpy( txBuffer, packet->packet_data(), packet->packet_size() );

    // put packet to waiting queue
    packet->set_timeout( 5000 );
    packet->start_timer();
    m_waitingQueue.push_back( packet );
    m_state = WAIT;
    //Logger::logf( LOG_MSG, _T("Sipr::tx() - waiting queue size: %d"), m_waitingQueue.GetCount() );

    // send output event
    //sendEvent( *packet, SIPR_EVENT_TYPE_OUTPUT );

    return 0;
}

//send input event
void Sipr::notify()
{
    //Logger::logf(   LOG_MSG, _T("Sipr::notify() - disabled: %d, need disable: %d"),
    //                m_bIsDisabled, m_bNeedDisable );

    /*
    if ( m_bNeedDisable )
    {
        //m_bNeedDisable = false;
        //m_bIsDisabled = true;

    }
*/
    /*
    if ( m_isReadSensors && !m_bIsDisabled )
    {
        if ( !m_delayReadSensors ){ read_sensor( 1, 0, 1 ); }
    }
*/
    if ( !m_eventHandler ) { return; }

    SIPRPacket packet(  m_parser.sequence(),
                        m_parser.type(),
                        m_parser.size(),
                        m_parser.data(),
                        m_parser.crc() );

    /*
    if ( m_parser.type() == SIPR_GET_TABLEDATA && m_parser.size() > 0 )
    {
        if ( m_parser.data()[0] == SIPR_TABLE_CONFIG1_EXT )
        {
            wxString s = _T("Sipr::notify() - SIPR_TABLE_CONFIG1_EXT dump\n\t");

            for ( int i = 0; i < m_parser.size(); ++i )
            {
                s += wxString::Format( _T("%02X"), m_parser.data()[i] );
            }

            Logger::logf( LOG_MSG, s );
        }
    }
    */

    sendEvent( packet, SIPR_EVENT_TYPE_INPUT );
}


void Sipr::readRealtime( bool read, unsigned int delay )
{
    if ( read ) { read_sensor( 1, 0, m_iSensorType ); }
    m_isReadSensors     = read;
    m_delayReadSensors  = delay;

    if ( delay )
    {
#ifdef CTB15

#else
        m_timerReadSensors = new timer( m_delayReadSensors, &m_timerTimedOutFlag, NULL );
        m_timerReadSensors->start();
#endif
    }
}


void Sipr::sendEvent( const SIPRPacket& packet, int eventType )
{
    //Logger::logf( LOG_MSG, _T("Sipr::sendEvent() - type: %d"), packet.type() );

    InputEvent e;
    e.setPacket( packet );
    e.setType( eventType );
    m_eventHandler->AddPendingEvent( e );
}


void Sipr::readPage( int mtPage )
{
    IntArray siprIDs = m_pSIPRDescriptor->getTables( mtPage );

    if ( siprIDs.IsEmpty() )
    {
        Logger::logf( LOG_MSG,
            _T("Sipr::readPage() - WARNING - no sipr table assigned to megatune page: %d"), mtPage );
    }

    Logger::logf( LOG_MSG, _T("Sipr::readPage() - INFO - mtPage: %d; assigned sipr table count: %d\n"),
        mtPage, siprIDs.GetCount() );

    for ( size_t i = 0; i < siprIDs.GetCount(); ++i )
    {
//        Logger::logf( LOG_MSG, _T("\tSipr::readPage() - INFO - siprTable: %d\n"), siprIDs[i] );
        get_table_data( siprIDs[i] );
    }
}


void Sipr::readAllPage()
{
    int pageCount = m_pSIPRDescriptor->getPageCount();

//    Logger::logf( LOG_MSG, _T("Sipr::readAllPage() - INFO - megatune page count: %d\n"), pageCount );

    for ( int i = 0;  i < pageCount; ++i  )
    {
        int mtPage = m_pSIPRDescriptor->getPageNumber( i );

//        Logger::logf(   LOG_MSG, _T("Sipr::readAllPage() - INFO - idx: %d, megatune page: %d\n"), i, mtPage );

        readPage( mtPage );
    }
}


void Sipr::writePage( int mtPage, int mtOffset, uint8_t value )
{
    IntArray siprIDs = m_pSIPRDescriptor->getTables( mtPage );

    if ( siprIDs.IsEmpty() )
    {
        Logger::logf( LOG_MSG,
            _T("Sipr::writePage() - WARNING - no sipr table assigned to megatune page: %d"), mtPage );
        return;
    }

    int siprTableOffset = 0;
    int siprTableSize   = 0;
    int siprTable       = 0;

    for ( size_t i = 0; i < siprIDs.GetCount(); ++i )
    {
        siprTable       = siprIDs[i];
        siprTableOffset = m_pSIPRDescriptor->getOffset( SIPR_GET_TABLEDATA, siprTable );
        siprTableSize   = m_pSIPRDescriptor->getSize( SIPR_GET_TABLEDATA, siprTable );

        if ( siprTableOffset < 0 || siprTableSize < 0 ) { continue; }

        if ( mtOffset < siprTableOffset || mtOffset > ( siprTableOffset + siprTableSize ) ) { continue; }

        set_table_data( (uint8_t)siprTable, (uint8_t)( mtOffset - siprTableOffset ), value );
    }
}


void Sipr::request( VemsCommand* command )
{
    switch ( command->getType() )
    {
        case VemsCommand::PING:
            ping();
            break;
        case VemsCommand::READ_REALTIME:
            //read_sensor( command->count, command->delay, command->page );
            read_sensor( 1, 0, m_iSensorType );
            break;
        case VemsCommand::READ_PAGE:
            readPage( command->getPageID() );
            break;
        case VemsCommand::READ_ALL_PAGE:
            readAllPage();
            break;
        case VemsCommand::WRITE_PAGE:
            writePage( command->getPageID(), command->getPageOffset(), command->getValue() );
            break;
        case VemsCommand::BURN:
        {
            SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_SAVE_CONFIG );
            push( packet );
        } break;
        case VemsCommand::GET_HWSIG:
        {
            SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_GET_HWSIG );
            push( packet );
        } break;
        case VemsCommand::GET_MT_SIGNATURE:
        {
            SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_GET_MT_SIGNATURE );
            push( packet );
        } break;
        case VemsCommand::ENABLE_READ_REALTIME:
            //m_isReadSensors = command->isReadRealTimeEnabled();
            enableReadRealtime( command->isReadRealTimeEnabled() );
            break;
        case VemsCommand::GET_FIRMWARE_VERSION:
        {
            SIPRPacket* packet  = new SIPRPacket( m_seqence++, SIPR_GET_VERSION );
            push( packet );
        } break;
        case VemsCommand::SET_STATE:
        {
            if ( command->getState() == ECUInfo::MEGATUNE )
            {
                SIPRPacket* packet  = new SIPRPacket( 0, SIPR_ENTER_MEGATUNE );
                push( packet );


                    wxString s;// = wxString::Format( _T("size: %d, data:"), packet->size() );


                    for ( size_t i = 0; i < (size_t)(packet->packet_size()); ++i )
                    {
                        s += wxString::Format( _T("$%02X"), packet->packet_data()[i] );
                        //s += wxString::Format( _T("%d"), i );
                    }

                    Logger::logf( LOG_MSG, _T("Sipr::ping() - ") + s );

            }
        } break;
        case VemsCommand::MEMORY_READ:
        {
            MemoryReadCommand*  mrc = (MemoryReadCommand*)command;
            size_t address = mrc->m_address;
            uint8_t buffer[5];
            uint8_t size;
            SiprCommand siprCommand;
            if(address > 0xFFFF){ /// 32.bit address
                buffer[0] = (mrc->m_address >> 24) & 255;
                buffer[1] = (mrc->m_address >> 16) & 255;
                buffer[2] = (mrc->m_address >> 8) & 255;
                buffer[3] = (mrc->m_address) & 255;
                buffer[4] = mrc->m_length;
                siprCommand = SIPR_READ_MEMORY_32;
                size = 5;
            }else{ /// 16.bit address
                buffer[0] = (mrc->m_address >> 8) & 255;
                buffer[1] = (mrc->m_address) & 255;
                buffer[2] = mrc->m_length;
                siprCommand = SIPR_READ_MEMORY;
                size = 3;
            }
            SIPRPacket* packet  = new SIPRPacket(m_seqence++, siprCommand, size, buffer);
            Logger::logf( LOG_MSG, _T("VemsCommand::MEMORY_READ packet=")+packet->toString() );
            push( packet );
        }break;
        case VemsCommand::MEMORY_WRITE:
        {
            MemoryWriteCommand*  mwc = (MemoryWriteCommand*)command;
            size_t address = mwc->m_address;
            uint8_t buffer[255];

            uint8_t size;
            SiprCommand siprCommand;
            if(address & 0x00200000 ){ //ram section from the memory
                buffer[0] = (mwc->m_address >> 8) & 255;
                buffer[1] = (mwc->m_address) & 255;
                siprCommand = SIPR_WRITE_MEMORY;
                size = 2+mwc->m_length;//todo place the data to buffer
                for (int i=0;i<mwc->m_length;i++)
                    buffer[2+i]=mwc->m_data_ptr[i];

            }else{ /// 16.bit address
                buffer[0] = (mwc->m_address >> 24) & 255;
                buffer[1] = (mwc->m_address >> 16) & 255;
                buffer[2] = (mwc->m_address >> 8) & 255;
                buffer[3] = (mwc->m_address) & 255;
                siprCommand = SIPR_WRITE_MEMORY_32;
                size = 4+mwc->m_length;
                for (int i=0;i<mwc->m_length;i++)
                    buffer[4+i]=mwc->m_data_ptr[i];
            }
            SIPRPacket* packet  = new SIPRPacket(m_seqence++, siprCommand, size, buffer);
            Logger::logf( LOG_MSG, _T("VemsCommand::MEMORY_WRITE packet=")+packet->toString() );

            push( packet );
        }break;
    }
}


void Sipr::enableReadRealtime( bool enable )
{
    if ( m_isReadSensors == enable )
    {
        return;
    }

    readRealtime( enable, 0 );
}


void Sipr::enable()
{
    Logger::logf( LOG_MSG, _T("-----Sipr::enable()") );
    m_bIsDisabled = false;
}

void Sipr::disable()
{
    Logger::logf( LOG_MSG, _T("-----Sipr::disable() - m_bIsDisabled: %d"), m_bIsDisabled );
    if ( !m_bIsDisabled ) { m_bNeedDisable = true; }
}

bool Sipr::isDisabled()
{
    return m_bIsDisabled;
}

void Sipr::reset()
{
    m_seqence           = 0;
    m_isReadSensors     = false;
    m_delayReadSensors  = 0;
    m_timerTimedOutFlag = 0;
    m_timerReadSensors  = NULL;
    m_bIsDisabled       = false;
    m_bNeedDisable      = false;
    m_state             = IDLE;
    m_bHasError         = false;

    m_parser.reset();
    m_txQueue.clear();
    m_waitingQueue.clear();
}
