#ifndef VEMSDEVICECONTROLLER_H
#define VEMSDEVICECONTROLLER_H


#include <wx/wx.h>
#include <wx/timer.h>


#include "VemsDeviceControllerBase.h"
#include "DeviceInfo.h"
#include "SiprPacketReceiver.h"
#include "MegatuneConfig.h"
#include "../common/typedef.h"
#include "../common/types/StringHash.h"
#include "../configmanager/CommunicationConfig.h"
#include "../events/VemsDeviceControllerEvent.h"


class VemsDeviceControllerFSM;
class SIPRPacket;
class CommunicationEvent;
class InputEvent;


extern const wxEventType START_TIMER;


class VemsDeviceController : public VemsDeviceControllerBase, public SiprPacketReceiver
{
    friend class VemsDeviceControllerFSM;

    public:
        VemsDeviceController( const CommunicationConfig& config );
        virtual ~VemsDeviceController();

        void setDetectionSleep( int sleep );

        void start( int flags = VDC_FLAG_DEFAULT );
        void start( const CommunicationConfig& config, int flags = VDC_FLAG_DEFAULT );
        void resume( int flag = VDC_FLAG_DEFAULT );
        void stop( int flag = VDC_FLAG_DEFAULT );
        bool isRunning();

        /// TODO - remove these methods
        void readOffset( bool start, wxArrayString offsets );
        void readOffset( bool start, StringHash vars, wxEvtHandler* evthandler );
        void readOffsetSIPR( wxArrayString addresses, wxEvtHandler* evtHandler );

        void changeBaudRate( size_t baudrate );

        void leaveBootMode();
        void stayBootMode();

        bool isWorking() const;
        bool isConnected() const;
        bool isDetecting() const;

        ///SiprPacketReceiver method
        void receive( const SIPRPacket& packet );

        ///CommandRequestBase
        void request( VemsCommand* command );

        const SIPRPacketList& getDetectedPages() const { return m_megatuneConfig.pages; }

        bool isOnline() const;

        void enableRealtime();

        ///test
        void timeout();

        bool isUseSerialComm() const;

    private:
        enum { MAX_CONNECT_TRY_COUNT = 3 };

        CommunicationConfig             m_config;
        VemsDeviceControllerFSM*        m_fsm;

        DeviceInfo                      m_prevDeviceInfo;

        int                             m_detectionSleep;
        MegatuneConfig                  m_megatuneConfig;
        bool                            m_isFirstDetection;
        bool                            m_isWorking;
        bool                            m_isResuming;
        bool                            m_isSkipCannotOpenPort;
        bool                            m_isSendCannotOpenPortEventEnabled;

        wxTimer                         m_timer;
        size_t                          m_detectWaitTime;
        int                             m_flags;
        bool                            m_isUSBCrashed;
        size_t                          m_connectTryCount;

    private:
        /// FSM actions
        void checkPassword();
        void fetchFirmwareInfo();
        void sync();

        void getAllPages();
        void deviceResumed();
//        void timeout();
        void downloadConfig();
        void stopCommunicationController();
        void baudrateChanged();
        void searchingSDFiles();
        void selectMegatuneMode();
        void resumeBaudrate();
        void changeToPreferredBaudrate();
        void connect();
        bool processPage( InputEvent& event );
        void startDetectingTimer();

        void disconnectFSMSiprInput();

        void sendDeviceControllerEvent( VemsDeviceControllerEventType eventType );

        bool isDeviceResumed() const;
        bool isBaudrateResumed();
        bool isDeviceReconnected() const;

        void refreshDeviceInfoBaudrate();
        void deviceFound();
        void deviceNotFound();

        void save();

        /// FSM guards
        bool isScanningFailed() const;
        bool isSyncingNeeded() const;
        bool hasPreferredBaudrate() const;

        void initCommunication();
        void initMegatuneConfig();
        void initDetectedMegatuneConfig();
        void initUserMegatuneConfig();
        void logConnectionParams();

        void sendPages();
        void registerAndStartCommunicationClient(   CommunicationControllerClient* client,
                                                    size_t clearingReadCount = 0, size_t sleepBeforeClearing = 0 );
        void resetDevice();
        void startDetecting();
        void clearCommBase();

        void OnStartEvent( wxCommandEvent& event );
        void OnStartDetection( wxTimerEvent& event );
        void OnStartTimer( wxCommandEvent& event) ;

        void OnCommunicationEvent( CommunicationEvent& event );

        bool isLeaveBootModeEnabled();
};

#endif // VEMSDEVICECONTROLLER_H

