#ifndef VEMSDEVICECONTROLLERFSM_H
#define VEMSDEVICECONTROLLERFSM_H

#include <wx/wx.h>

#include "ECUInfo.h"
#include "DeviceInfo.h"
#include "../common/typedef.h"
#include "../communication/clients/PortScannerListener.h"
#include "../communication/clients/FirmwareInfoFetcherListener.h"
#include "../communication/clients/MegatuneSyncerListener.h"
#include "../communication/clients/TimeoutListener.h"
#include "../communication/clients/DeviceModeSelectorListener.h"
#include "../communication/clients/BaudRateSelectorListener.h"
#include "../communication/clients/PasswordCheckerListener.h"


class VemsDeviceController;
class SerialPortScannerEvent;
class InputEvent;
class VemsDeviceControllerState;


class VemsDeviceControllerFSM : public wxEvtHandler
    , public PortScannerListener
    , public FirmwareInfoFetcherListener
    , public MegatuneSyncerListener
    , public TimeoutListener
    , public DeviceModeSelectorListener
    , public BaudRateSelectorListener
    , public PasswordCheckerListener
{
    public:
        VemsDeviceControllerFSM( VemsDeviceController* controller );
        virtual ~VemsDeviceControllerFSM();

        /// TODO
        bool isConnected() const;
        bool isDetecting() const;
        bool isReconnecting() const;

        void save();
        void resume( bool getPages );
        void reset();

        void OnSIPRInput( InputEvent& event );

        DeviceInfo getDeviceInfo() const;

        /// listeners
        void OnDeviceFound( PortScannerEvent& event );
        void OnDeviceNotFound();
        void OnFirmwareInfoFetched(  const FirmwareInfo& fwInfo );
        void OnFirmwareDataSynced();
        void OnTimeout();
        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );
        void OnBaudrateChanged( size_t baudrate );
        void OnDeviceProtected();
        void OnDeviceNotProtected();

        void setState( VemsDeviceControllerState* newState );

        void setDevice( ECUInfo::Device device );
        void setDeviceMode( ECUInfo::EcuMode deviceMode );
        void setBaudrate( size_t baudrate );
        void setFirmwareInfo( const FirmwareInfo& fwInfo );
        void setFirmwareVersion( const FirmwareVersion& fwVersion );

        bool isDeviceResumed() const;
        bool isBaudrateResumed() const;
        bool isDeviceReconnected() const;

        void deviceResumed();
        bool hasPreferredBaudrate();
        void changeToPreferredBaudrate();
        void resumeBaudrate();

        void startDetectingTimer();
        void selectMegatuneMode();

        void checkPassword();
        void fetchFirmwareInfo();

        void bootModeFound();

        void sync();
        void connect();
        void getAllPages();
        void baudrateChanged();
        void timeout();

        void sendDeviceNotFoundEvent();
        void sendDeviceReconnectedEvent();
        void sendDeviceResumedEvent();
        void sendDeviceProtectedEvent();

        void deviceFound();
        void deviceNotFound();
        bool isUseSerialComm() const ;
        bool isLeaveBootModeEnabled();

    private:
        VemsDeviceControllerState*  m_state;
        VemsDeviceController*       m_controller;

//        void OnCommunicationEvent( CommunicationEvent& event );
};

#endif // VEMSDEVICECONTROLLERFSM_H
