#ifndef VEMSDEVICECONTROLLERSTATES_H
#define VEMSDEVICECONTROLLERSTATES_H


#include "../communication/clients/PortScannerListener.h"
#include "../communication/clients/FirmwareInfoFetcherListener.h"
#include "../communication/clients/MegatuneSyncerListener.h"
#include "../communication/clients/TimeoutListener.h"
#include "../communication/clients/DeviceModeSelectorListener.h"
#include "../communication/clients/BaudRateSelectorListener.h"
#include "../communication/clients/PasswordCheckerListener.h"

class VemsDeviceControllerFSM;


class VemsDeviceControllerState
    : public PortScannerListener
    , public FirmwareInfoFetcherListener
    , public MegatuneSyncerListener
    , public TimeoutListener
    , public DeviceModeSelectorListener
    , public BaudRateSelectorListener
    , public PasswordCheckerListener
{
    public:
        VemsDeviceControllerState( VemsDeviceControllerFSM* fsm );
        virtual ~VemsDeviceControllerState() {}

        virtual void OnDeviceFound( PortScannerEvent& event ) {};
        virtual void OnDeviceNotFound() {};
        virtual void OnFirmwareInfoFetched(  const FirmwareInfo& fwInfo ) {}
        virtual void OnFirmwareDataSynced() {}
        virtual void OnTimeout() {}
        virtual void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode ) {}
        virtual void OnBaudrateChanged( size_t baudrate ) {}
        virtual void OnDeviceProtected() {}
        virtual void OnDeviceNotProtected() {}
        virtual void OnAllPageReceived() {}

    protected:
        VemsDeviceControllerFSM*    m_fsm;
};


class Detecting : public VemsDeviceControllerState
{
    public:
        Detecting( VemsDeviceControllerFSM* fsm )
            : VemsDeviceControllerState( fsm ) {}

        virtual ~Detecting() {}

        void OnDeviceFound( PortScannerEvent& event );
        void OnDeviceNotFound();
        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );
        void OnBaudrateChanged( size_t baudrate );

    private:
        void OnMegatuneMode();
};

class CheckingPassword : public VemsDeviceControllerState
{
    public:
        CheckingPassword( VemsDeviceControllerFSM* fsm )
            : VemsDeviceControllerState( fsm ) {}

        virtual ~CheckingPassword() {}

        void OnDeviceProtected();
        void OnDeviceNotProtected();
};

class FetchingFirmwareInfo : public VemsDeviceControllerState
{
    public:
        FetchingFirmwareInfo( VemsDeviceControllerFSM* fsm )
            : VemsDeviceControllerState( fsm ) {}

        virtual ~FetchingFirmwareInfo() {}

        void OnFirmwareInfoFetched(  const FirmwareInfo& fwInfo );
        void OnAllPageReceived();
        void OnTimeout();
};

class BootMode : public VemsDeviceControllerState
{
    public:
        BootMode( VemsDeviceControllerFSM* fsm)
            : VemsDeviceControllerState( fsm ) {}

        virtual ~BootMode() {}

        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );
};

class Syncing : public VemsDeviceControllerState
{
    public:
        Syncing( VemsDeviceControllerFSM* fsm )
            : VemsDeviceControllerState( fsm ) {}

        virtual ~Syncing() {}

        void OnFirmwareDataSynced();
        void OnTimeout();
};

class Connected : public VemsDeviceControllerState
{
    public:
        Connected( VemsDeviceControllerFSM* fsm )
            : VemsDeviceControllerState( fsm ) {}

        virtual ~Connected() {}

        void OnTimeout();
        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );
        void OnBaudrateChanged( size_t baudrate );
};

class Resuming : public VemsDeviceControllerState
{
    public:
        Resuming( VemsDeviceControllerFSM* fsm, bool getPages )
            : VemsDeviceControllerState( fsm )
            {
                m_getPages = getPages;
            }

        virtual ~Resuming() {}

        void OnDeviceNotFound();
        void OnDeviceFound( PortScannerEvent& event );
        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );
        void OnBaudrateChanged( size_t baudrate );
        void OnTimeout();
        void OnAllPageReceived();

    private:
        bool m_getPages;
};

class Reconnecting : public VemsDeviceControllerState
{
    public:
        Reconnecting( VemsDeviceControllerFSM* fsm )
            : VemsDeviceControllerState( fsm ) {}

        virtual ~Reconnecting() {}
};


#endif // VEMSDEVICECONTROLLERSTATES_H
