#include "FileIcon.h"

#include "../RawLogHelper.h"
#include "../logger.h"
#include <wx/dialog.h>

#include <wx/menu.h>
#include <wx/msgdlg.h>
#include <wx/app.h>
#include <wx/filedlg.h>
#include <wx/aboutdlg.h>

#include "../FirmwareInfoDialog.h"
#include "../VemsAboutDialog.h"

#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"

#include "../gui/GeneralConfigFrame.h"

#if defined (__WINCE_STANDARDSDK__)// || defined (__WXMSW__)
#include <Windows.h>
#endif

#include "../FileDialog.h"


#include "../VemsTuneController.h"
#include "../stdatamanager/STDataMgrFacade.h"
#include "../WindowReg.h"

int idFIMenuMinimize = wxNewId();
int idFIMenuClose = wxNewId();
int idFIMenuAbout = wxNewId();
int idFIMenuFwInfo = wxNewId();
int idFIMenuOri  = wxNewId();
int idFIMenuProtocolConfig  = wxNewId();



BEGIN_EVENT_TABLE(FileIcon, VemsToolIconBase)

    EVT_LEFT_DOWN(FileIcon::OnLeftDown)
    EVT_LEFT_UP(FileIcon::OnLeftUp)
    EVT_MENU(idFIMenuMinimize,FileIcon::OnMenuMinimize)
    EVT_MENU(idFIMenuClose,FileIcon::OnMenuClose)
    EVT_MENU(idFIMenuAbout,FileIcon::OnMenuAbout)
    EVT_MENU(idFIMenuFwInfo,FileIcon::OnMenuFwInfo)
    EVT_MENU(idFIMenuOri,FileIcon::OnMenuOri)
    EVT_MENU(idFIMenuProtocolConfig, FileIcon::OnProtocolConfig )

END_EVENT_TABLE()


FileIcon::FileIcon(wxWindow* parent, wxWindowID id, const wxString& icondir, const wxPoint &pos)
        :VemsToolIconBase(parent,id,icondir,pos,_T("file.png"))
        ,m_vemsTuneController (NULL)

{
    //ctor
    m_parent = parent;
    m_wreg = NULL;

    m_fileUpBmp   = new wxBitmapLayer(0, icondir +  _T("file.png"));
    m_fileDownBmp = new wxBitmapLayer(0, icondir +  _T("file-down.png"));


    m_menu = new wxMenu();
    m_testmenu = new wxMenu();

    m_menu->Append(idFIMenuFwInfo,_T("Firmware Info"));
    m_menu->Append(idFIMenuAbout,_T("About"));
	m_menu->Append(idFIMenuProtocolConfig,_T("Default protocol"));
    m_menu->AppendSeparator();
    m_menu->Append(idFIMenuMinimize,_T("Minimize"));
    m_menu->AppendSeparator();
    m_menu->Append(idFIMenuClose,_T("Exit"));
    //m_menu->AppendSeparator();   m_menu->Append(idFIMenuOri,_T("Change orientation"));



    m_menu->AppendSubMenu(m_testmenu, _T("Userdialog test"));

}

FileIcon::~FileIcon()
{
    //dtor
}


void FileIcon::SetWindowReg(WindowReg *wreg)
{
    m_wreg = wreg;
};

void FileIcon::IconInit()
{

    IconInitBase(m_fileUpBmp);

}

void FileIcon::OnMenuOri(wxCommandEvent& event)
{

#if defined (__WINCE_STANDARDSDK__) //|| defined (__WXMSW__)
/// incude problem on windows cannot find:DM_DISPLAYORIENTATION etc
    DEVMODE DevMode;
    int CurrentAngle = 0;
    int NewAngle = 0;


    memset (&DevMode, 0, sizeof (DevMode));
    DevMode.dmSize   = sizeof (DevMode);
    DevMode.dmFields = DM_DISPLAYORIENTATION;
    ChangeDisplaySettingsEx(NULL, &DevMode, NULL, CDS_TEST, NULL);


    CurrentAngle = DevMode.dmDisplayOrientation;
    switch(CurrentAngle){
        case DMDO_DEFAULT:  NewAngle = DMDO_90;      break;
        case DMDO_90:       NewAngle = DMDO_180;     break;
        case DMDO_180:      NewAngle = DMDO_270;     break;
        case DMDO_270:      NewAngle = DMDO_DEFAULT; break;
    }



    memset (&DevMode, 0, sizeof (DevMode));
    DevMode.dmSize   = sizeof (DevMode);
    DevMode.dmFields = DM_DISPLAYORIENTATION;
    DevMode.dmDisplayOrientation = NewAngle;


	ChangeDisplaySettingsEx(NULL, &DevMode, NULL, CDS_RESET, NULL);


#endif

}

void FileIcon::OnLeftUp(wxMouseEvent& evt)
{
    m_bmpLayers->AddBitmap(m_fileUpBmp);
    Refresh(false);

    PopupMenu(m_menu);

}

void FileIcon::OnLeftDown(wxMouseEvent& evt)
{
    m_bmpLayers->AddBitmap(m_fileDownBmp);
    Refresh(false);
}

/** @brief OnMenuFwInfo
  *
  * @todo: document this function
  */
void FileIcon::OnMenuFwInfo(wxCommandEvent& event)
{
    FirmwareInfo* fwInfo = m_vemsTuneController->getFirmwareInfo();
    FirmwareInfoDialog(this, fwInfo);
}

/** @brief OnMenuAbout
  *
  * @todo: document this function
  */
void FileIcon::OnMenuAbout(wxCommandEvent& event)
{

    VemsAboutDialog();
    /*UserDialogManager* udm =*/
    /*new UserDialogManager( this, MTConfigHelper::getMTConfigData(),
    m_vemsTuneController->getSTDataMgr(),
    m_vemsTuneController->getRTDataMgr(), m_testmenu );
    */
}

/** @brief OnMenuClose
  *
  * @todo: document this function
  */
void FileIcon::OnMenuClose(wxCommandEvent& event)
{
    m_parent->Close();
    //m_parent->Destroy();

#if defined (__WINCE_STANDARDSDK__)
    wxExit();
#endif

}

/** @brief OnMenuMinimize
  *
  * @todo: document this function
  */
void FileIcon::OnMenuMinimize(wxCommandEvent& event)
{
    if(m_wreg == NULL) return;

	m_wreg->UpdateList();

	m_wreg->Minimize();
}


void FileIcon::SetVemsCtrl(VemsTuneController *ctrl)
{
    m_vemsTuneController = ctrl;

}

void FileIcon::OnProtocolConfig( wxCommandEvent& event )
{
    ConfigBase* config = ConfigHelper2::getConfig();

    GeneralConfigFrame* frame = new GeneralConfigFrame( this, wxID_ANY, _T("Default Protocol"), config );

    frame->addGroup( _T("Default protocol") );
    wxArrayString choices;
    choices.Add(_T("MEGATUNE"));
    choices.Add(_T("SIPR"));
    //frame->addEntrySelectComboBox(_T("protocol"), choices );
    frame->addEntrySelectComboBox(  _T("protocol"), choices, wxEmptyString, _T("/devices") );

    frame->addGroup( _T("Default read realtime command") );
    choices.Clear();
    choices.Add(_T("SIPR_RT_MT_NORMAL"));
    choices.Add(_T("SIPR_RT_MT_EXT"));
    //frame->addEntrySelectComboBox(_T("protocol"), choices );
    frame->addEntrySelectComboBox(  _T("read_realtime"), choices, wxEmptyString, _T("/devices") );

    frame->selectMainGroup();

    //frame->addWarningText();
    frame->addSaveButton();
    frame->CenterOnScreen();
    frame->Show( true );
	frame->Raise();
}
