#ifndef OSMTILEMANAGER_H
#define OSMTILEMANAGER_H

#include <wx/wx.h>
#include "../events/DownloadingStatusEvent.h"
#include "../Observable.h"

class ContentLoaderManager;

class TileDesc: public wxObject
{
    DECLARE_DYNAMIC_CLASS(TileDesc);

    public:
        TileDesc() { x=-1;y=-1;zoom=0; }
        TileDesc(int xx, int yy, int level) { x = xx; y = yy; zoom = level; };
        TileDesc(const wxPoint& tilepoint, int level) { x = tilepoint.x; y = tilepoint.y; zoom = level; };

        bool IsOk() {
            if( x >= 0 && y >= 0 && zoom >= 0 && zoom < 19 )
                return true;
            else
                return false;
        }

        wxString GetAsString() {
            return wxString::Format(_T("%i/%i/%i"),zoom,x,y);
        };

        bool operator==(const TileDesc& tile) const;
        bool operator!=(const TileDesc& tile) const;


    int x;
    int y;
    int zoom;

};

DECLARE_VARIANT_OBJECT(TileDesc);

WX_DECLARE_OBJARRAY(TileDesc, TileDescArray);

class OSMTileManager: public wxEvtHandler, public Observable
{
    public:
        OSMTileManager(const wxString& tileserver, const wxString& cachedir);
        virtual ~OSMTileManager();

        void Request(TileDesc desc, bool checkInHist = true);

        bool CheckReqTilesList(TileDesc& desc);

        void ClearHistory() { m_reqTilesHistory.Clear(); }

        size_t GetCacheSize();
        wxString GetCacheSizeStr();

        wxString GetTileServer() { return m_tileServer; }
        wxString GetCacheDir() { return m_cacheDir; }

    protected:

        void OnLoadedFile(DownloadingStatusEvent& evt);
        void SaveTile(wxVariantHash& hash);

    private:

        TileDescArray   m_reqTilesHistory;

        enum{
            idWebDownloader,
            idCacheLoader
        };

        wxString m_tileServer;
        wxString m_cacheDir;

        ContentLoaderManager*   m_webDownMgr;
        ContentLoaderManager*   m_cacheMgr;

};



#endif // OSMTILEMANAGER_H
