#ifndef WEBINTERFACEPANEL_H
#define WEBINTERFACEPANEL_H

#include <wx/wx.h>
#include <wx/listctrl.h>
#include <wx/statline.h>

#include <wx/regex.h>
#include <wx/dnd.h>
#include "../ProgressBar.h"
#include "WIPAccessories.h"
#include "../common/types/BoolIntHash.h"

#define USE_TEMPLATE_DND 1


class ProcessBase;



class WebInterfacePanel: public wxFrame, public ObserverBase
#if !USE_TEMPLATE_DND
, public wxFileDropTarget
#endif
{
    public:
        WebInterfacePanel(wxWindow *parent = NULL, int id = wxID_ANY, const wxString& title = _T("webize"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);
        virtual ~WebInterfacePanel();

        void Init(WIPDesc& desc);

        virtual void notify( int i );
        virtual void notify(int i, int idx);
        virtual void notify(const wxString& str);
        virtual void Init(const wxString& name, const wxString& msg, const int max) {};

        static wxString GetListCtrlCellText(wxListCtrl * lc, int col);
        static void SetListCtrlCellText(wxListCtrl * lc, int col, const wxString &str);

        static wxString GetListCtrlCellText(wxListCtrl * lc, long item, int col);
        static void SetListCtrlCellText(wxListCtrl * lc, long item, int col, const wxString &str);


#if USE_TEMPLATE_DND
        bool onDropFiles( const wxArrayString& filenames );
#else
        virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString&  filenames);
#endif

        bool process();

    protected:

        void InitLists();

        void FillRemoteList(const wxString& devstring);
        void FillLocalList();

        int AddLine(const wxString& str, wxListCtrl * listctrl);
        void AddLocalFileToList(const wxString& name);

        void MoveUpLine(wxListCtrl * lc, int idx, int dist=1);

        bool DownloadSelectedFiles();
        bool UploadSelectedFiles();

        void DeselectAllItems(wxListCtrl * lc);

        void ReSize();

        bool Install();
        bool Install(long item);
        bool Install(const wxString& filename);

        void ShowLoaderLabel(bool isShow);

        void FillRemoteListByStr(const wxString& content);
        void ParseApacheFileList(const wxString& lines);
        wxString GetSizeFromHumanReadableSize(const wxString& str);

        long SearchCell(wxListCtrl* listctrl, int column, const wxString& text);

    private:

        void OnButton(wxCommandEvent &evt);
        void OnDevList(wxCommandEvent &evt);
        void OnSize(wxSizeEvent &evt);
        void OnListHeaderClick(wxListEvent &evt);
        void OnLocalItemClick(wxListEvent &evt);
        void OnMenu(wxCommandEvent &evt);
        void OnClose(wxCloseEvent& evt);

        void OnTimeOutTimer(wxTimerEvent& evt);

        void OnDownloadList(wxCommandEvent& evt);

        void OnTransferProgress(wxCommandEvent& evt);
        void OnTransferDone(wxCommandEvent& evt);

        void OnAutoInstall(wxCommandEvent& evt);

        enum{
            idRefreshButton = 1000,
            idDownloadButton,
            idUploadButton,
            idDevList,
            idRemoteList,
            idLocalList,
            idInstallMenu,
            idInstallButton,
            idCloseButton,
            itTimeoutTimer,
            idListDownloadEnd,
            idListDownloadStart,
            idCBAutoInstall,
            idTransferEnd,
            idTransferProgress
        };


        wxComboBox  *   m_remPathList;
        wxCheckBox  *   m_autoInstall;

        wxListCtrl  *   m_remoteList;
        wxListCtrl  *   m_localList;

        ProgressBar *   m_progressBar;

        wxBoxSizer  *   m_mainsizer;

        wxButton    *   m_downButton;
        wxButton    *   m_upButton;

        wxButton    *   m_installButton;

        WIPDesc         m_desc;

        DataTransferMgr * m_dataTransMgr;

        wxMenu      *   m_popupRightMenu;


        BoolIntHash          m_localCols;
        BoolIntHash          m_romoteCols;

        ProcessBase*    m_contentLoader;

        wxTimer         m_timeoutTimer;

        DECLARE_EVENT_TABLE();



        wxStaticText    *   m_loadingLabel;

        wxArrayString       m_transferedFiles;

        bool m_tmp;

};


#endif // WEBINTERFACEPANEL_H
