#include "wxWebBrowserBase.h"

#include "../bcWidgets/bcParentPanel.h"
#include "../bcWidgets/bcWrapper.h"
#include "../bcWidgets/bcButton.h"
#include "../bcWidgets/bcTextCtrl.h"

#include "../common/IconSetsHelper.h"

#include "../logger.h"

BEGIN_EVENT_TABLE(wxWebBrowserBase,wxFrame)

    EVT_BUTTON(idToolButtonBack ,wxWebBrowserBase::OnToolButton)
    EVT_BUTTON(idToolButtonForward ,wxWebBrowserBase::OnToolButton)
    EVT_BUTTON(idToolButtonHome ,wxWebBrowserBase::OnToolButton)
    EVT_BUTTON(idToolButtonReload ,wxWebBrowserBase::OnToolButton)

//    EVT_COMBOBOX(idToolURLBar, wxWebBrowserBase::OnURLEntered)
    EVT_TEXT_ENTER(idToolURLBar, wxWebBrowserBase::OnURLEntered)

    EVT_MENU_RANGE(idMenuBack, idMenuEND, wxWebBrowserBase::OnMenu)

END_EVENT_TABLE()

wxWebBrowserBase::wxWebBrowserBase(wxWindow* parent)
{
    //ctor
    Create(parent, wxID_ANY, _T("Browsing"));
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_historyPos = -1;
    m_webContent = NULL;
    m_urlBar = NULL;
    m_urlBarCtrl = NULL;
    m_backButton = NULL;
    m_forwButton = NULL;

    m_home = _T("http://vems.hu/vt");
}

wxWebBrowserBase::~wxWebBrowserBase()
{
    //dtor
}

bool wxWebBrowserBase::CheckHistoryPos()
{
    if(m_urlHistory.GetCount() == 0) return false;

    if(m_historyPos < 0) {
        m_historyPos = 0;
    }else if(m_historyPos >= (int)m_urlHistory.GetCount()) {
        m_historyPos = (int)m_urlHistory.GetCount() - 1;
    }

    if(m_backButton) m_backButton->Enable(m_historyPos != 0);

    if(m_forwButton) m_forwButton->Enable(m_historyPos != (int)m_urlHistory.GetCount()-1);


    return true;
}

wxString wxWebBrowserBase::GetRealURL(const wxString& url)
{
    wxString turl = url;

    if(turl.EndsWith(_T("/"))) {
        Logger::logf(LOG_MSG,_T("EndsWith /"));
        turl.RemoveLast();
    }

    if(turl.Find(_T("://")) == -1 && turl.Find(_T("file:")) == -1 && turl.Find(_T("about:")) == -1)
        return _T("http://") + turl;
    else
        return turl;
}

void wxWebBrowserBase::OnURLEntered(wxCommandEvent& evt)
{
    if(!m_urlBarCtrl) return;

    wxString url = m_urlBarCtrl->GetValue();

    LoadURL(url);
    AddHistory(url);
}

void wxWebBrowserBase::SetHome(const wxString& url)
{
    m_home = url;
}

bool wxWebBrowserBase::DoBack()
{
    Logger::logf(LOG_MSG,_T("m_historyPos=%i"),m_historyPos);
    m_historyPos--;
    Logger::logf(LOG_MSG,_T("m_historyPos=%i"),m_historyPos);
    if(!CheckHistoryPos()) return false;
    LoadURL(m_urlHistory[m_historyPos]);
    return true;
}

bool wxWebBrowserBase::DoForward()
{
    m_historyPos++;
    if(!CheckHistoryPos()) return false;
    LoadURL(m_urlHistory[m_historyPos]);
    return true;
}

bool wxWebBrowserBase::DoHome()
{
    LoadURL(m_home);
    return true;
}

bool wxWebBrowserBase::DoReload()
{
    if(!CheckHistoryPos()) return false;
    LoadURL(m_urlHistory.Last());
    return true;
}

void wxWebBrowserBase::OnMenu(wxCommandEvent& evt)
{
    switch(evt.GetId()){
        case idMenuBack:
            DoBack();
            break;
        case idMenuForward:
            DoForward();
            break;
        case idMenuHome:
            DoHome();
            break;
        case idMenuReload:
            DoReload();
            break;
        case idMenuClose:
            Close();
            break;
    }
}

void wxWebBrowserBase::OnToolButton(wxCommandEvent& evt)
{
    switch(evt.GetId()){
        case idToolButtonBack:
            DoBack();
            break;
        case idToolButtonForward:
            DoForward();
            break;
        case idToolButtonHome:
            DoHome();
            break;
        case idToolButtonReload:
            DoReload();
            break;
    }
}

void wxWebBrowserBase::AddHistory(const wxString& url)
{
    if(!m_urlBarCtrl) return;

    wxString rurl = GetRealURL(url);

    if(m_urlHistory.Index(rurl) == wxNOT_FOUND) {
        m_urlHistory.Add(rurl);
        m_historyPos = m_urlHistory.GetCount()-1;
    } else {
        m_urlHistory.Remove(rurl);
        m_urlHistory.Add(rurl);
    }

    m_urlHistBox->Clear();
    m_urlHistBox->Append(m_urlHistory);

    for(size_t a=0;a<m_urlHistory.GetCount();a++) {
        Logger::logf(LOG_MSG,_T("m_urlHistory[%i]=")+m_urlHistory[a],a);

}

    m_urlBarCtrl->SetValue(rurl);
}

wxWindow* wxWebBrowserBase::BuildToolBar()
{
    bcParentPanel* bar = new bcParentPanel(this);

    wxBoxSizer* toolsizer = new wxBoxSizer(wxHORIZONTAL);

    IconSetSize toolsize = IS_16;

    m_backButton = new bcButton(bar, idToolButtonBack, IconSetsHelper::GetImage(IS_LEFT,toolsize) );
    m_forwButton = new bcButton(bar, idToolButtonForward, IconSetsHelper::GetImage(IS_RIGHT,toolsize) );
    bcButton* homebutton = new bcButton(bar, idToolButtonHome, IconSetsHelper::GetImage(IS_HOME,toolsize) );
    bcButton* reloadbutton = new bcButton(bar, idToolButtonReload, IconSetsHelper::GetImage(IS_REFRESH,toolsize) );

    homebutton->SetImgLabel( IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_HOME,toolsize), bcBUTTON_IMAGE_PRESSED );
    reloadbutton->SetImgLabel( IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_REFRESH,toolsize), bcBUTTON_IMAGE_PRESSED );
    m_backButton->SetImgLabel( IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_LEFT,toolsize), bcBUTTON_IMAGE_PRESSED );
    m_forwButton->SetImgLabel( IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_RIGHT,toolsize), bcBUTTON_IMAGE_PRESSED );


    toolsizer->Add( new bcWrapper(m_backButton), 0, wxALL, 1 );
    toolsizer->Add( new bcWrapper(m_forwButton), 0, wxALL, 1 );
    toolsizer->Add( new bcWrapper(homebutton), 0, wxALL, 1 );
    toolsizer->Add( new bcWrapper(reloadbutton), 0, wxALL, 1 );

//    m_urlBar = new wxComboBox(bar,idToolURLBar,_T(""), wxDefaultPosition, wxDefaultSize, 0,0, wxTE_PROCESS_ENTER );
    m_urlBarCtrl = new wxTextCtrl(bar, idToolURLBar,_T("about:blank"), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);


    toolsizer->Add(m_urlBarCtrl,1,wxALL|wxALIGN_CENTER_VERTICAL,1);

    bar->SetSizer(toolsizer);

    return bar;
}

void wxWebBrowserBase::SetTitlePage(const wxString& pagetitle)
{
    SetTitle(pagetitle);
}

void wxWebBrowserBase::SetLoadingContentURL(const wxString &url)
{
    wxStatusBar* bar = GetStatusBar();

    if(bar) bar->SetStatusText(_T("Loading: ")+url);
}

void wxWebBrowserBase::ContentLoaded(const wxString &url)
{
    wxStatusBar* bar = GetStatusBar();
    if(bar) bar->SetStatusText(_T("Done"));
}

void wxWebBrowserBase::PageLoaded()
{
    wxStatusBar* bar = GetStatusBar();
    if(bar) bar->SetStatusText(_T("Done"));
}

void wxWebBrowserBase::SetLoadingURL(const wxString &url)
{
    if(m_urlBarCtrl) m_urlBarCtrl->SetValue(url);
}

void wxWebBrowserBase::SetFocusToWeb()
{
    if(m_webContent) m_webContent->SetFocus();
}

void wxWebBrowserBase::BuildMainGUI(wxWindow* webwindow)
{
    m_webContent = webwindow;

// build menubar
    wxMenuBar* mbar = new wxMenuBar();

    wxMenu* fileMenu = new wxMenu();
    fileMenu->Append(idMenuClose,_T("Close\tAlt-F4"));
    mbar->Append(fileMenu,_T("File"));


    wxMenu* naviMenu = new wxMenu();
    naviMenu->Append(idMenuBack,_T("Back"));
    naviMenu->Append(idMenuForward,_T("Forward"));
    naviMenu->Append(idMenuHome,_T("Home"));
    naviMenu->Append(idMenuReload,_T("Reload\tF5"));

    mbar->Append(naviMenu,_T("Navigate"));

    SetMenuBar(mbar);

// build toolbar
    wxWindow* toolbar = BuildToolBar();

// sizering
    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    mainsizer->Add(toolbar,0,wxEXPAND);

    m_urlHistBox = new wxListBox(this, wxID_ANY);

    wxBoxSizer* histsizer = new wxBoxSizer(wxHORIZONTAL);

    histsizer->Add(webwindow,1,wxEXPAND);
    histsizer->Add(m_urlHistBox,0,wxEXPAND);

    mainsizer->Add(histsizer,1,wxEXPAND);

    SetSizer(mainsizer);


// build statusbar
    wxStatusBar* statusbar = CreateStatusBar(2);

    statusbar->SetStatusText(_T("Done"));
}
