#include "IACControlPanel.h"

#include "../logger.h"

#include "../vemsprotocol/VemsCommand.h"
#include "../vemsprotocol/CommandRequestBase.h"
#include "IACControl.h"

BEGIN_EVENT_TABLE( IACControlPanel, wxPanel )
    EVT_TEXT_ENTER(wxID_ANY, IACControlPanel::OnEnter)
    EVT_KILL_FOCUS( IACControlPanel::OnKillFocus )
    EVT_KEY_DOWN(IACControlPanel::OnKeyDown)
    EVT_BUTTON( wxID_APPLY, IACControlPanel::OnEnter )
    EVT_BUTTON( wxID_CLOSE, IACControlPanel::OnCloseButton )

    EVT_BUTTON( idButtonStart, IACControlPanel::OnButton )
    EVT_BUTTON( idButtonStop, IACControlPanel::OnButton )

END_EVENT_TABLE()

IACControlPanel::IACControlPanel(wxWindow* parent, CommandRequestBase* requestMgr, IACControl* autoCtrl)
{
    //ctor

    m_requestMgr = requestMgr;

    m_autoCtrl = autoCtrl;

    buildGUI(parent);

}

IACControlPanel::~IACControlPanel()
{
    //dtor
    if(m_autoCtrl) {
        m_autoCtrl->Stop();
        delete m_autoCtrl;
}
    commandRequest(0);
}

void IACControlPanel::buildGUI(wxWindow* parent)
{
    Create(parent, wxID_ANY);

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    wxBoxSizer* m_mainsizer = new wxBoxSizer( wxVERTICAL );
    //m_mainsizer  = new wxBoxSizer( wxVERTICAL );

    wxString msg = _T("Idle Air Controller Lock Dialog\n\n To activate set 1-255\n To disable set 0 or close");
    wxStaticText* text2 = new wxStaticText( this, -1, msg);

    //wxString  warningMsg = _T("\nmax value limited by MaxSteps To Extend");
    wxString  warningMsg = _T("The following settings could disable IAC lock functionality:\n\n");

    warningMsg += _T("Menu: Starting/Idle\n\n");

    warningMsg += _T("1. PWM idle hardware settings:\n    Max/Min PWM duty allowed\n\n");
    warningMsg += _T("2. Stepper idle hardware settings:\n    Max/Min Steps allowed\n");
    wxStaticText* warningST = new wxStaticText( this, -1, warningMsg);
    warningST->SetForegroundColour(wxColour(200,0,0) );
    //text2->SetFont( font );

    wxStaticText* noteST = new wxStaticText( this, -1, _T("NOTE: See help - especially if experiencing oscillation"));

    m_mainsizer->Add( text2, 0, wxALL | wxEXPAND, 5 );

    wxString targetRPMStr, integralStr, diffStr;

    if(m_autoCtrl) {
        targetRPMStr = wxString::Format(_T("%.0f"),m_autoCtrl->GetTargetRPM());
        integralStr = wxString::Format(_T("%.3f"),m_autoCtrl->GetIntegral());
        diffStr = wxString::Format(_T("%.3f"),m_autoCtrl->GetDiffrential());
    }

    wxFlexGridSizer* grid = new wxFlexGridSizer( 3, 1, 5 );
    m_mainsizer->Add( grid, 0, wxEXPAND | wxALL, 0 );

    wxTextValidator validator(wxFILTER_ALPHANUMERIC);

    wxStaticText* textName= new wxStaticText( this, -1, _T("IAC position:"));
    grid->Add( textName, 0, wxALL | wxEXPAND | wxALIGN_CENTER_VERTICAL, 5 );

    m_textctrl = new wxTextCtrl(this, -1, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_BESTWRAP );
    grid->Add( m_textctrl, 0, wxALL | wxEXPAND, 5 );
    m_textctrl->SetValidator(validator);

    wxStaticText* textUnit = new wxStaticText( this, -1, _T(""));
    grid->Add( textUnit, 0, wxALL | wxEXPAND | wxALIGN_CENTER_VERTICAL, 5 );
///////////

    textName = new wxStaticText( this, -1, _T("Target rpm:"));
    grid->Add( textName, 0, wxALL | wxEXPAND | wxALIGN_CENTER_VERTICAL, 5 );

    m_targetRPMctrl = new wxTextCtrl(this, idTargetRPM, targetRPMStr, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_BESTWRAP );
    grid->Add( m_targetRPMctrl, 0, wxALL | wxEXPAND, 5 );
    m_targetRPMctrl->SetValidator(validator);

    textUnit = new wxStaticText( this, -1, _T("1/min"));
    grid->Add( textUnit, 0, wxALL | wxEXPAND | wxALIGN_CENTER_VERTICAL, 5 );
///////////

    grid->Add( new wxStaticText(this, wxID_ANY, _T("I")), 0, wxALL | wxALIGN_CENTER_VERTICAL , 5 );

    m_intergaralVal = new wxTextCtrl(this, idIntegralVal, integralStr, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_BESTWRAP );
    grid->Add(m_intergaralVal, 0, wxALL | wxEXPAND, 5 );
    m_intergaralVal->SetValidator(validator);

    grid->Add( new wxStaticText( this, -1, _T("")), 0, wxALL | wxEXPAND | wxALIGN_CENTER_VERTICAL, 5 );
///////////

    grid->Add( new wxStaticText(this, wxID_ANY, _T("D")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 );

    m_diffVal = new wxTextCtrl(this, idDiffVal, diffStr, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_BESTWRAP );
    grid->Add(m_diffVal, 0, wxALL | wxEXPAND, 5 );
    m_diffVal->SetValidator(validator);

    grid->Add( new wxStaticText( this, -1, _T("")), 0, wxALL | wxEXPAND | wxALIGN_CENTER_VERTICAL, 5 );


///////////
    m_mainsizer->Add( warningST, 0, wxALL | wxEXPAND, 5 );
    m_mainsizer->Add( noteST, 0, wxALL | wxEXPAND, 5 );


    wxButton* sendButton = new wxButton( this, wxID_APPLY , _T("Send") );
    wxButton* closeButton = new wxButton( this, wxID_CLOSE , _T("Close") );

    m_startButton = new wxButton( this, idButtonStart , _T("Start") );
    m_stopButton = new wxButton( this, idButtonStop , _T("Stop") );


    wxBoxSizer* buttonSizer = new wxBoxSizer( wxHORIZONTAL );

    buttonSizer->Add( sendButton, 0, wxALL, 5 );
    buttonSizer->AddStretchSpacer();
    buttonSizer->Add( m_startButton, 0, wxALL, 5 );
    buttonSizer->Add( m_stopButton, 0, wxALL, 5 );
    buttonSizer->AddStretchSpacer();
    buttonSizer->Add( closeButton, 0, wxALL, 5 );

    m_mainsizer->Add( buttonSizer, 1, wxALL | wxEXPAND | wxALIGN_BOTTOM , 5 );

    m_stopButton->Enable(false);
    m_startButton->Enable(true);

    SetSizer( m_mainsizer );
    m_mainsizer->Fit( this );
    m_mainsizer->SetSizeHints( this );
}


void IACControlPanel::OnEnter( wxCommandEvent& event )
{
    Logger::logf( LOG_MSG, _T("IACControlPanel::OnEnter()") );
    switch(event.GetId()) {
        case idTargetRPM: {

            Logger::logf( LOG_MSG, _T("IACControlPanel::OnEnter() target rpm") );

            wxString valueStr = m_targetRPMctrl->GetValue();
            double dValue;
            bool result = valueStr.ToDouble(&dValue);
            if(!result) return;
            if(m_autoCtrl) m_autoCtrl->setTargetRPM((int)dValue);
            return;}

        case idIntegralVal: {

            Logger::logf( LOG_MSG, _T("IACControlPanel::OnEnter() integral") );

            wxString valueStr = m_intergaralVal->GetValue();
            double dValue;
            bool result = valueStr.ToDouble(&dValue);
            if(!result) return;
            if(m_autoCtrl) m_autoCtrl->SetIntegral((int)dValue);
            }return;


        case idDiffVal: {

            Logger::logf( LOG_MSG, _T("IACControlPanel::OnEnter() differential") );

            wxString valueStr = m_diffVal->GetValue();
            double dValue;
            bool result = valueStr.ToDouble(&dValue);
            if(!result) return;
            if(m_autoCtrl) m_autoCtrl->SetDifferential((int)dValue);
            }return;

    }

    SendIACLockVal();
}

void IACControlPanel::SendIACLockVal()
{
    wxString valueStr = m_textctrl->GetValue();
    double dValue;
    bool result = valueStr.ToDouble(&dValue);
    if(!result) return;

    //dValue *= 4;    /// deg * 4;
    int binaryValue = (int)dValue;
    Logger::logf( LOG_MSG, _T("IACControlPanel::OnEnter() value=%d"), binaryValue);
    commandRequest(binaryValue);
}

/// O means desable ing lock
void IACControlPanel::commandRequest(int ignition)
{
    if( (ignition < 0) || (ignition > 255) ) return;
    wxString command = wxString::Format(_T("Manmdi%02Xmbye"), ignition);

    Logger::logf( LOG_MSG, _T("IACControlPanel::OnEnter() iac. lock command: ")+ command);

    VemsCommand* vemsCommand = new StringCommand(command);
    m_requestMgr->request(vemsCommand);

}

void IACControlPanel::OnKillFocus( wxFocusEvent& event )
{
     Logger::logf( LOG_MSG, _T("IACControlPanel::OnKillFocus()") );
}

void IACControlPanel::OnKeyDown( wxKeyEvent& event )
{
    int key = event.GetKeyCode();

    if ( key == WXK_TAB )
    {
        if ( event.ShiftDown() )
        {

            return;
        }

        return;
    }

    event.Skip();
}


void IACControlPanel::OnButton( wxCommandEvent& event )
{
    switch( event.GetId() ) {
        case idButtonStart:

            if(m_autoCtrl) {
                m_autoCtrl->Start();
                m_stopButton->Enable(true);
                m_startButton->Enable(false);
            }

            break;

        case idButtonStop:

            if(m_autoCtrl) {
                m_autoCtrl->Stop();
                commandRequest(0);
                m_stopButton->Enable(false);
                m_startButton->Enable(true);
            }

            break;

    }
}

void IACControlPanel::OnCloseButton( wxCommandEvent& event )
{
    wxWindow* parent = GetParent();

    if ( parent ) {
        parent->Destroy();
    }
}

void IACControlPanel::setAutoControl(IACControl* autoCtrl)
{
    m_autoCtrl = autoCtrl;
}

