#include "wxRealRect.h"

IMPLEMENT_DYNAMIC_CLASS(wxRealRect, wxObject);

IMPLEMENT_VARIANT_OBJECT_SHALLOWCMP(wxRealRect);

wxRealRect wxNullRealRect = wxRealRect(0,0,0,0);
wxRealRect wxDefaultRealRect = wxRealRect();
wxRect wxDefaultRect = wxRect(-1,-1,0,0);

#include "common/types/wxRealRectArray.h"
#include "common/types/DoubleArray.h"

#include "logger.h"

#define DlEq(a,b) ((fabs((a) - (b)) < m_precision) ? true : false)

wxRealRect::wxRealRect(const wxRect& rect)
{
    x = rect.x;
    y = rect.y;
    w = rect.width;
    h = rect.height;
    m_precision = DEFAULTREALPRECISION;
}

wxRealRect::wxRealRect(const wxRealRect& rect)
{
    //ctor
    x = rect.x;
    y = rect.y;
    w = rect.w;
    h = rect.h;
    m_precision = rect.m_precision;
}

wxRealRect::~wxRealRect()
{
    //dtor
}

void wxRealRect::Init(double xx, double yy, double ww, double hh, double precision)
{
    x = xx;
    y = yy;
    w = ww;
    h = hh;

    if( precision < 0 )
        m_precision = DEFAULTREALPRECISION;
    else
        m_precision = precision;
}

/** @brief operator!=
  *
  * @todo: document this function
  */
bool wxRealRect::operator!=(const wxRealRect& rect) const
{
    if( (*this) == rect )
        return false;
    else
        return true;
}

/** @brief operator==
  *
  * @todo: document this function
  */
bool wxRealRect::operator==(const wxRealRect& rect) const
{
    if(  fabs(x - rect.x) < m_precision &&
         fabs(y - rect.y) < m_precision &&
         fabs(w - rect.w) < m_precision &&
         fabs(h - rect.h) < m_precision   )
        return true;
    else
        return false;
}

/** @brief operator=
  *
  * @todo: document this function
  */
void wxRealRect::operator=(const wxRealRect& rect)
{
    x = rect.x;
    y = rect.y;
    w = rect.w;
    h = rect.h;
    m_precision = rect.m_precision;
}

void wxRealRect::operator=(const wxRect& rect)
{
    x = rect.x;
    y = rect.y;
    w = rect.width;
    h = rect.height;
    m_precision = DEFAULTREALPRECISION;
}

wxRealRect& wxRealRect::operator+=(const wxRealRect& rect)
{
    SetLeft( wxMin(x, rect.x) );
    SetTop( wxMin(y, rect.y) );

    SetRight( wxMax( GetRight(), rect.GetRight() ) );
    SetBottom( wxMax( GetBottom(), rect.GetBottom() ) );

    return *this;
}

wxRealRect wxRealRect::operator+(const wxRealRect& rect) const
{
    wxRealRect r;

    r.SetLeft( wxMin(x, rect.x) );
    r.SetTop( wxMin(y, rect.y) );

    r.SetRight( wxMax( GetRight(), rect.GetRight() ) );
    r.SetBottom( wxMax( GetBottom(), rect.GetBottom() ) );

    return r;
}

wxRealRect& wxRealRect::Inflate(double dx, double dy)
{
     if (-2.0 * dx > w)
     {
         // Don't allow deflate to eat more width than we have,
         // a well-defined rectangle cannot have negative width.
         x += w / 2.0;
         w = 0.0;
     }
     else
     {
         // The inflate is valid.
         x -= dx;
         w += 2.0 * dx;
     }

     if (-2.0 * dy > h)
     {
         // Don't allow deflate to eat more height than we have,
         // a well-defined rectangle cannot have negative height.
         y += h / 2.0;
         h = 0.0;
     }
     else
     {
         // The inflate is valid.
         y -= dy;
         h += 2.0 * dy;
     }

    return *this;
}

bool wxRealRect::Contains(double cx, double cy) const
{
    double dh = cy - y;
    double dw = cx - x;
    return ( (cx > x || DlEq(cx,x)) && (cy > y || DlEq(cy,y)) && (dh < h || DlEq(dh,h)) && (dw < w || DlEq(dw,w)) );
}

wxString wxRealRect::GetAsString(int dec)
{
    wxString sf = wxString::Format(_T("x=%%.%if, y=%%.%if, w=%%.%if, h=%%.%if"),dec,dec,dec,dec);
    return wxString::Format(sf,x,y,w,h);
}

wxString wxRealRect::GetAsStringBorders(int dec)
{
    wxString sf = wxString::Format(_T("top=%%.%if, left=%%.%if, bottom=%%.%if, right=%%.%if"),dec,dec,dec,dec);
    return wxString::Format(sf,GetTop(),GetLeft(), GetBottom(), GetRight() );
}

void wxRealRect::OffsetRef(const wxRealSize& offs)
{
    x += x * offs.w;
    y += y * offs.h;
}


bool wxRealRect::Intersects(const wxRealRect& rect) const
{
    wxRealRect r = Intersect(rect);

    // if there is no intersection, both width and height are 0
    return fabs(r.w) < m_precision ? false : true;
}

wxRealRect wxRealRect::Intersect(const wxRealRect& rect) const
{
    wxRealRect r = *this;
    r.Intersect(rect);
    return r;
}

void wxRealRect::Normalize()
{
    if( w < 0 ) { w *= -1.0; x -= w; }

    if( h < 0 ) { h *= -1.0; y -= h; }
}

wxRealRect& wxRealRect::Intersect(const wxRealRect& rect)
{

    double x2 = GetRight(), y2 = GetBottom();

    if ( x < rect.x )
        x = rect.x;
    if ( y < rect.y )
        y = rect.y;
    if ( x2 > rect.GetRight() )
        x2 = rect.GetRight();
    if ( y2 > rect.GetBottom() )
        y2 = rect.GetBottom();

    w = x2 - x;
    h = y2 - y;

    if ( w <= 0 || h <= 0 )
    {
        w = h = 0;
    }

    return *this;
}

wxRealRect& wxRealRect::Union(const wxRealRect& rect)
{
    // ignore empty rectangles: union with an empty rectangle shouldn't extend
    // this one to (0, 0)
    if ( fabs(w) < m_precision || fabs(h) < m_precision  )
    {
        *this = rect;
    }
    else if ( fabs(rect.w) > m_precision || fabs(rect.h) > m_precision )
    {
        double x1 = wxMin(x, rect.x);
        double y1 = wxMin(y, rect.y);
        double y2 = wxMax(y + h, rect.h + rect.y);
        double x2 = wxMax(x + w, rect.w + rect.x);

        x = x1;
        y = y1;
        w = x2 - x1;
        h = y2 - y1;
    }
    //else: we're not empty and rect is empty

    return *this;
}

wxRect wxRealRect::GetOuterBox() const
{
    wxRect rect;
    rect.SetLeft( (int)floor(x) );
    rect.SetTop( (int)floor(y) );
    rect.SetRight( (int)(ceil(GetRight())) );
    rect.SetBottom( (int)(ceil(GetBottom())) );
    return rect;
}

wxRect wxRealRect::GetInnerBox() const
{
    wxRect rect;
    rect.SetLeft( (int)ceil(x) );
    rect.SetTop( (int)ceil(y) );
    rect.SetRight( (int)(floor(GetRight())) );
    rect.SetBottom( (int)(floor(GetBottom())) );
    return rect;
}

wxRealRectArray wxRealRect::Subtract(const wxRealRect& rect)
{
    wxRealRectArray rects;
    rects.Add(rect);
    return Subtract(rects);
}


int double_sort_func(double ** a, double ** b)
{
    return (**a) > (**b) ? 1 : ( (**b) > (**a) ? -1 : 0 );
}

wxRealRectArray wxRealRect::Subtract(const wxRealRectArray& rects, bool removeNullRects)
{
    wxRealRectArray result;

    DoubleArray slice_x;
    DoubleArray slice_y;

    for(size_t a=0; a<rects.GetCount(); a++){

        wxRealRect& crect = rects[a];

        if( !Intersects(crect) ) continue;

        slice_x.Add( crect.GetLeft() );
        slice_y.Add( crect.GetTop() );
        slice_x.Add( crect.GetRight() );
        slice_y.Add( crect.GetBottom() );
    }

    if( !slice_x.GetCount() || !slice_y.GetCount() ) {
        result.Add(*this);
        return result;
    }

    slice_x.Add( GetLeft() );
    slice_y.Add( GetTop() );
    slice_x.Add( GetRight() );
    slice_y.Add( GetBottom() );

    slice_x.Sort(double_sort_func);
    slice_y.Sort(double_sort_func);

    for(size_t sy=0; sy<slice_y.GetCount()-1; sy++){

        double py = slice_y[sy];
        double ny = slice_y[sy+1];

        for(size_t sx=0; sx<slice_x.GetCount()-1; sx++){

            double px = slice_x[sx];
            double nx = slice_x[sx+1];

            wxRealRect resrect( wxRealPoint(px,py), wxRealPoint(nx,ny) );

            if( (resrect.w < m_precision || resrect.h < m_precision) && removeNullRects ) continue;

            bool isFound = false;

            for(size_t a=0; a<rects.GetCount(); a++){
                if( rects[a].Contains( resrect ) ) { isFound = true; break; }
            }

            if(isFound) continue;

            result.Add(resrect);
        }
    }

    return result;
}

void wxRealRect::SubPixelNormalize(double linewidth)
{
    double _x = floor(x) + (((int)linewidth)%2)/2.0;
    double _y = floor(y) + (((int)linewidth)%2)/2.0;
    double _w = ceil(w);
    double _h = ceil(h);

    x = _x;
    y = _y;
    w = _w;
    h = _h;
}

