#ifndef WXREALSIZE_H
#define WXREALSIZE_H

#include <wx/wx.h>

#define DEFAULTREALPRECISION 0.001

class wxRealSize: public wxObject
{
    DECLARE_DYNAMIC_CLASS(wxRealSize);

    public:
        wxRealSize() { Init(-1.0,-1.0); };
        wxRealSize(const wxRealSize& size);
        wxRealSize(double width, double height, double precision = -1.0) { Init(width,height,precision); };
        wxRealSize(const wxSize& size, double precision = -1.0) { Init(size.x,size.y,precision); };
        wxRealSize(const wxRealPoint& size, double precision = -1.0) { Init(size.x, size.y,precision); };
        virtual ~wxRealSize();

        void Init(double width, double height, double precision = -1.0);

        double w;
        double h;

        void IncBy(double dx, double dy) { w += dx; h += dy; }
        void IncBy(const wxSize& sz) { IncBy(sz.x, sz.y); }
        void IncBy(const wxRealSize& sz) { IncBy(sz.w, sz.h); }
        void IncBy(double d) { IncBy(d, d); }

        void DecBy(double dx, double dy) { IncBy(-dx, -dy); }
        void DecBy(const wxSize& sz) { DecBy(sz.x, sz.y); }
        void DecBy(const wxRealSize& sz) { DecBy(sz.w, sz.h); }
        void DecBy(double d) { DecBy(d, d); }

        void SetPrecision(double prec) { m_precision = prec; }
        double GetPrecision() const { return m_precision; };
        double GetWidth() const { return w; };
        double GetHeight() const { return h; };

        void operator=(const wxRealSize& size);
        void operator=(const wxSize& size);
        bool operator==(const wxRealSize& size) const;
        bool operator!=(const wxRealSize& size) const;

        wxRealSize operator+(const wxRealSize& size) const;
        wxRealSize operator-(const wxRealSize& size) const;
        wxRealSize& operator+=(const wxRealSize& size);
        wxRealSize& operator-=(const wxRealSize& size);

        wxRealSize operator*(double factor) const;
        wxRealSize operator/(double factor) const;
        wxRealSize& operator*=(double factor);
        wxRealSize& operator/=(double factor);

        wxSize Ceil() const;
        wxSize Floor() const;

    protected:
    private:

        double m_precision;
};

extern wxRealSize wxDefaultRealSize;

DECLARE_VARIANT_OBJECT(wxRealSize);

#endif // WXREALSIZE_H

