#include "wxVariantDataDoubleHash.h"

/**


    Put data to a wxVariant variable:
    \code
    DoubleHash hash;
    //...put some data to hash
    wxVariantDataDoubleHash* variantHash = new wxVariantDataDoubleHash( hash );
    wxVariant variant( variantHash );
    \endcode

    Get data from a wxVariant variable:
    \code
    if( variant.GetType() == _T("doublehash") )
    {
        wxVariantDataDoubleHash* variantHash = (wxVariantDataDoubleHash*)variant.GetData();
        DoubleHash hash = variantHash->GetValue();
    }
    \endcode
*/


IMPLEMENT_DYNAMIC_CLASS(wxVariantDataDoubleHash, wxVariantData)

wxVariantDataDoubleHash::wxVariantDataDoubleHash( const DoubleHash& value )
{
    m_value = value;
}

wxVariantDataDoubleHash::~wxVariantDataDoubleHash()
{
    //dtor
}

void wxVariantDataDoubleHash::SetValue( const DoubleHash& value )
{
    m_value = value;
}

bool wxVariantDataDoubleHash::Eq( wxVariantData& data ) const
{
    wxASSERT_MSG( (data.GetType() == wxT("doublehash") ), _T("wxVariantDataDoubleHash::Eq: argument mismatch") );

    DoubleHash& inputHash = ( (wxVariantDataDoubleHash&)data ).GetValue();

    if( m_value.size() != inputHash.size() ) { return false; }

    for( DoubleHash::const_iterator it = m_value.begin(); it != m_value.end(); ++it )
    {
        if( inputHash.find( it->first ) == inputHash.end() ) { return false; }
    }

    return true;
}
